# Automatically generated by generate_event_list.py
# Do not edit!

event_names = {
    SDL_FIRSTEVENT : u'NOEVENT',
    SDL_QUIT : 'QUIT',
    SDL_APP_TERMINATING : 'APP_TERMINATING',
    SDL_APP_LOWMEMORY : 'APP_LOWMEMORY',
    SDL_APP_WILLENTERBACKGROUND : 'APP_WILLENTERBACKGROUND',
    SDL_APP_DIDENTERBACKGROUND : 'APP_DIDENTERBACKGROUND',
    SDL_APP_WILLENTERFOREGROUND : 'APP_WILLENTERFOREGROUND',
    SDL_APP_DIDENTERFOREGROUND : 'APP_DIDENTERFOREGROUND',
    SDL_DISPLAYEVENT : 'DISPLAYEVENT',
    SDL_WINDOWEVENT : 'WINDOWEVENT',
    SDL_SYSWMEVENT : 'SYSWMEVENT',
    SDL_KEYDOWN : 'KEYDOWN',
    SDL_KEYUP : 'KEYUP',
    SDL_TEXTEDITING : 'TEXTEDITING',
    SDL_TEXTINPUT : 'TEXTINPUT',
    SDL_KEYMAPCHANGED : 'KEYMAPCHANGED',
    SDL_MOUSEMOTION : 'MOUSEMOTION',
    SDL_MOUSEBUTTONDOWN : 'MOUSEBUTTONDOWN',
    SDL_MOUSEBUTTONUP : 'MOUSEBUTTONUP',
    SDL_MOUSEWHEEL : 'MOUSEWHEEL',
    SDL_JOYAXISMOTION : 'JOYAXISMOTION',
    SDL_JOYBALLMOTION : 'JOYBALLMOTION',
    SDL_JOYHATMOTION : 'JOYHATMOTION',
    SDL_JOYBUTTONDOWN : 'JOYBUTTONDOWN',
    SDL_JOYBUTTONUP : 'JOYBUTTONUP',
    SDL_JOYDEVICEADDED : 'JOYDEVICEADDED',
    SDL_JOYDEVICEREMOVED : 'JOYDEVICEREMOVED',
    SDL_CONTROLLERAXISMOTION : 'CONTROLLERAXISMOTION',
    SDL_CONTROLLERBUTTONDOWN : 'CONTROLLERBUTTONDOWN',
    SDL_CONTROLLERBUTTONUP : 'CONTROLLERBUTTONUP',
    SDL_CONTROLLERDEVICEADDED : 'CONTROLLERDEVICEADDED',
    SDL_CONTROLLERDEVICEREMOVED : 'CONTROLLERDEVICEREMOVED',
    SDL_CONTROLLERDEVICEREMAPPED : 'CONTROLLERDEVICEREMAPPED',
    SDL_FINGERDOWN : 'FINGERDOWN',
    SDL_FINGERUP : 'FINGERUP',
    SDL_FINGERMOTION : 'FINGERMOTION',
    SDL_DOLLARGESTURE : 'DOLLARGESTURE',
    SDL_DOLLARRECORD : 'DOLLARRECORD',
    SDL_MULTIGESTURE : 'MULTIGESTURE',
    SDL_CLIPBOARDUPDATE : 'CLIPBOARDUPDATE',
    SDL_DROPFILE : 'DROPFILE',
    SDL_DROPTEXT : 'DROPTEXT',
    SDL_DROPBEGIN : 'DROPBEGIN',
    SDL_DROPCOMPLETE : 'DROPCOMPLETE',
    SDL_AUDIODEVICEADDED : 'AUDIODEVICEADDED',
    SDL_AUDIODEVICEREMOVED : 'AUDIODEVICEREMOVED',
    SDL_SENSORUPDATE : 'SENSORUPDATE',
    SDL_RENDER_TARGETS_RESET : 'RENDER_TARGETS_RESET',
    SDL_RENDER_DEVICE_RESET : 'RENDER_DEVICE_RESET',
    SDL_USEREVENT : 'USEREVENT',
    SDL_LASTEVENT : 'LASTEVENT',
}
