CONTROLLER_BUTTON_INVALID = SDL_CONTROLLER_BUTTON_INVALID
CONTROLLER_BUTTON_A = SDL_CONTROLLER_BUTTON_A
CONTROLLER_BUTTON_B = SDL_CONTROLLER_BUTTON_B
CONTROLLER_BUTTON_X = SDL_CONTROLLER_BUTTON_X
CONTROLLER_BUTTON_Y = SDL_CONTROLLER_BUTTON_Y
CONTROLLER_BUTTON_BACK = SDL_CONTROLLER_BUTTON_BACK
CONTROLLER_BUTTON_GUIDE = SDL_CONTROLLER_BUTTON_GUIDE
CONTROLLER_BUTTON_START = SDL_CONTROLLER_BUTTON_START
CONTROLLER_BUTTON_LEFTSTICK = SDL_CONTROLLER_BUTTON_LEFTSTICK
CONTROLLER_BUTTON_RIGHTSTICK = SDL_CONTROLLER_BUTTON_RIGHTSTICK
CONTROLLER_BUTTON_LEFTSHOULDER = SDL_CONTROLLER_BUTTON_LEFTSHOULDER
CONTROLLER_BUTTON_RIGHTSHOULDER = SDL_CONTROLLER_BUTTON_RIGHTSHOULDER
CONTROLLER_BUTTON_DPAD_UP = SDL_CONTROLLER_BUTTON_DPAD_UP
CONTROLLER_BUTTON_DPAD_DOWN = SDL_CONTROLLER_BUTTON_DPAD_DOWN
CONTROLLER_BUTTON_DPAD_LEFT = SDL_CONTROLLER_BUTTON_DPAD_LEFT
CONTROLLER_BUTTON_DPAD_RIGHT = SDL_CONTROLLER_BUTTON_DPAD_RIGHT
CONTROLLER_BUTTON_MAX = SDL_CONTROLLER_BUTTON_MAX
CONTROLLER_AXIS_INVALID = SDL_CONTROLLER_AXIS_INVALID
CONTROLLER_AXIS_LEFTX = SDL_CONTROLLER_AXIS_LEFTX
CONTROLLER_AXIS_LEFTY = SDL_CONTROLLER_AXIS_LEFTY
CONTROLLER_AXIS_RIGHTX = SDL_CONTROLLER_AXIS_RIGHTX
CONTROLLER_AXIS_RIGHTY = SDL_CONTROLLER_AXIS_RIGHTY
CONTROLLER_AXIS_TRIGGERLEFT = SDL_CONTROLLER_AXIS_TRIGGERLEFT
CONTROLLER_AXIS_TRIGGERRIGHT = SDL_CONTROLLER_AXIS_TRIGGERRIGHT
CONTROLLER_AXIS_MAX = SDL_CONTROLLER_AXIS_MAX
