/* Generated by Cython 0.29.20 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_20"
#define CYTHON_HEX_VERSION 0x001D14F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pygame_sdl2__color
#define __PYX_HAVE_API__pygame_sdl2__color
/* Early includes */
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <stddef.h>
#include "SDL.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "src/pygame_sdl2/color.pyx",
  "src/pygame_sdl2/color.pxd",
  "include/color_dict.pxi",
};

/*--- Type declarations ---*/
struct __pyx_obj_11pygame_sdl2_5color_Color;
struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma;

/* "pygame_sdl2/color.pxd":26
 * cdef to_sdl_color(color, SDL_Color *out)
 * 
 * cdef class Color:             # <<<<<<<<<<<<<<
 *     # Allow weak references.
 *     cdef object __weakref__
 */
struct __pyx_obj_11pygame_sdl2_5color_Color {
  PyObject_HEAD
  struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *__pyx_vtab;
  PyObject *__weakref__;
  Uint8 r;
  Uint8 g;
  Uint8 b;
  Uint8 a;
  uint8_t length;
};


/* "pygame_sdl2/color.pyx":452
 *         return self.r / 255.0, self.g / 255.0, self.b / 255.0, self.a / 255.0
 * 
 *     def correct_gamma(self, gamma):             # <<<<<<<<<<<<<<
 *         m = map(lambda x: int(round(pow(x / 255.0, gamma) * 255)), tuple(self))
 *         c = type(self)(tuple(m))
 */
struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma {
  PyObject_HEAD
  PyObject *__pyx_v_gamma;
};



/* "pygame_sdl2/color.pyx":63
 *     out.a = color.a
 * 
 * cdef class Color:             # <<<<<<<<<<<<<<
 *     cdef from_rgba(self, Uint8 r, Uint8 g, Uint8 b, Uint8 a):
 *         self.r = r
 */

struct __pyx_vtabstruct_11pygame_sdl2_5color_Color {
  PyObject *(*from_rgba)(struct __pyx_obj_11pygame_sdl2_5color_Color *, Uint8, Uint8, Uint8, Uint8);
  PyObject *(*from_name)(struct __pyx_obj_11pygame_sdl2_5color_Color *, PyObject *);
  PyObject *(*from_hex)(struct __pyx_obj_11pygame_sdl2_5color_Color *, PyObject *);
};
static struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *__pyx_vtabptr_11pygame_sdl2_5color_Color;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractCObj(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_SubtractCObj(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* None.proto */
static CYTHON_INLINE double __Pyx_mod_double(double, double);

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_RemainderObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_RemainderObjC(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceRemainder(op1, op2) : PyNumber_Remainder(op1, op2))
#endif

/* pow2.proto */
#define __Pyx_PyNumber_Power2(a, b) PyNumber_Power(a, b, Py_None)

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_TrueDivideObjC(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* IncludeStringH.proto */
#include <string.h>

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8_t(uint8_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint8_t __Pyx_PyInt_As_uint8_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_11pygame_sdl2_5color_5Color_from_rgba(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, Uint8 __pyx_v_r, Uint8 __pyx_v_g, Uint8 __pyx_v_b, Uint8 __pyx_v_a); /* proto*/
static PyObject *__pyx_f_11pygame_sdl2_5color_5Color_from_hex(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_c); /* proto*/
static PyObject *__pyx_f_11pygame_sdl2_5color_5Color_from_name(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_c); /* proto*/

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'libc.stddef' */

/* Module declarations from 'sdl2' */

/* Module declarations from 'pygame_sdl2.color' */
static PyTypeObject *__pyx_ptype_11pygame_sdl2_5color_Color = 0;
static PyTypeObject *__pyx_ptype_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma = 0;
static PyObject *__pyx_v_11pygame_sdl2_5color_colors = 0;
#define __Pyx_MODULE_NAME "pygame_sdl2.color"
extern int __pyx_module_is_main_pygame_sdl2__color;
int __pyx_module_is_main_pygame_sdl2__color = 0;

/* Implementation of 'pygame_sdl2.color' */
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_map;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_hex;
static PyObject *__pyx_builtin_oct;
static PyObject *__pyx_builtin_IndexError;
static PyObject *__pyx_builtin_round;
static const char __pyx_k_L[] = ">L";
static const char __pyx_k_a[] = "a";
static const char __pyx_k_b[] = "b";
static const char __pyx_k_g[] = "g";
static const char __pyx_k_l[] = "l";
static const char __pyx_k_r[] = "r";
static const char __pyx_k_0x[] = "0x";
static const char __pyx_k__2[] = "";
static const char __pyx_k__3[] = "#";
static const char __pyx_k_08x[] = "%08x";
static const char __pyx_k_BBB[] = "BBB";
static const char __pyx_k_div[] = "div";
static const char __pyx_k_hex[] = "hex";
static const char __pyx_k_map[] = "map";
static const char __pyx_k_mod[] = "mod";
static const char __pyx_k_oct[] = "oct";
static const char __pyx_k_red[] = "red";
static const char __pyx_k_tan[] = "tan";
static const char __pyx_k_BBBB[] = "BBBB";
static const char __pyx_k_blue[] = "blue";
static const char __pyx_k_cyan[] = "cyan";
static const char __pyx_k_gold[] = "gold";
static const char __pyx_k_gray[] = "gray";
static const char __pyx_k_grey[] = "grey";
static const char __pyx_k_hsva[] = "hsva";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_navy[] = "navy";
static const char __pyx_k_pack[] = "pack";
static const char __pyx_k_peru[] = "peru";
static const char __pyx_k_pink[] = "pink";
static const char __pyx_k_plum[] = "plum";
static const char __pyx_k_red1[] = "red1";
static const char __pyx_k_red2[] = "red2";
static const char __pyx_k_red3[] = "red3";
static const char __pyx_k_red4[] = "red4";
static const char __pyx_k_rgba[] = "rgba";
static const char __pyx_k_snow[] = "snow";
static const char __pyx_k_tan1[] = "tan1";
static const char __pyx_k_tan2[] = "tan2";
static const char __pyx_k_tan3[] = "tan3";
static const char __pyx_k_tan4[] = "tan4";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_Color[] = "Color";
static const char __pyx_k_azure[] = "azure";
static const char __pyx_k_beige[] = "beige";
static const char __pyx_k_black[] = "black";
static const char __pyx_k_blue1[] = "blue1";
static const char __pyx_k_blue2[] = "blue2";
static const char __pyx_k_blue3[] = "blue3";
static const char __pyx_k_blue4[] = "blue4";
static const char __pyx_k_brown[] = "brown";
static const char __pyx_k_coral[] = "coral";
static const char __pyx_k_cyan1[] = "cyan1";
static const char __pyx_k_cyan2[] = "cyan2";
static const char __pyx_k_cyan3[] = "cyan3";
static const char __pyx_k_cyan4[] = "cyan4";
static const char __pyx_k_gold1[] = "gold1";
static const char __pyx_k_gold2[] = "gold2";
static const char __pyx_k_gold3[] = "gold3";
static const char __pyx_k_gold4[] = "gold4";
static const char __pyx_k_gray0[] = "gray0";
static const char __pyx_k_gray1[] = "gray1";
static const char __pyx_k_gray2[] = "gray2";
static const char __pyx_k_gray3[] = "gray3";
static const char __pyx_k_gray4[] = "gray4";
static const char __pyx_k_gray5[] = "gray5";
static const char __pyx_k_gray6[] = "gray6";
static const char __pyx_k_gray7[] = "gray7";
static const char __pyx_k_gray8[] = "gray8";
static const char __pyx_k_gray9[] = "gray9";
static const char __pyx_k_green[] = "green";
static const char __pyx_k_grey0[] = "grey0";
static const char __pyx_k_grey1[] = "grey1";
static const char __pyx_k_grey2[] = "grey2";
static const char __pyx_k_grey3[] = "grey3";
static const char __pyx_k_grey4[] = "grey4";
static const char __pyx_k_grey5[] = "grey5";
static const char __pyx_k_grey6[] = "grey6";
static const char __pyx_k_grey7[] = "grey7";
static const char __pyx_k_grey8[] = "grey8";
static const char __pyx_k_grey9[] = "grey9";
static const char __pyx_k_ivory[] = "ivory";
static const char __pyx_k_khaki[] = "khaki";
static const char __pyx_k_linen[] = "linen";
static const char __pyx_k_lower[] = "lower";
static const char __pyx_k_pink1[] = "pink1";
static const char __pyx_k_pink2[] = "pink2";
static const char __pyx_k_pink3[] = "pink3";
static const char __pyx_k_pink4[] = "pink4";
static const char __pyx_k_plum1[] = "plum1";
static const char __pyx_k_plum2[] = "plum2";
static const char __pyx_k_plum3[] = "plum3";
static const char __pyx_k_plum4[] = "plum4";
static const char __pyx_k_round[] = "round";
static const char __pyx_k_snow1[] = "snow1";
static const char __pyx_k_snow2[] = "snow2";
static const char __pyx_k_snow3[] = "snow3";
static const char __pyx_k_snow4[] = "snow4";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_wheat[] = "wheat";
static const char __pyx_k_white[] = "white";
static const char __pyx_k_azure1[] = "azure1";
static const char __pyx_k_azure2[] = "azure2";
static const char __pyx_k_azure3[] = "azure3";
static const char __pyx_k_azure4[] = "azure4";
static const char __pyx_k_bisque[] = "bisque";
static const char __pyx_k_brown1[] = "brown1";
static const char __pyx_k_brown2[] = "brown2";
static const char __pyx_k_brown3[] = "brown3";
static const char __pyx_k_brown4[] = "brown4";
static const char __pyx_k_coral1[] = "coral1";
static const char __pyx_k_coral2[] = "coral2";
static const char __pyx_k_coral3[] = "coral3";
static const char __pyx_k_coral4[] = "coral4";
static const char __pyx_k_gray10[] = "gray10";
static const char __pyx_k_gray11[] = "gray11";
static const char __pyx_k_gray12[] = "gray12";
static const char __pyx_k_gray13[] = "gray13";
static const char __pyx_k_gray14[] = "gray14";
static const char __pyx_k_gray15[] = "gray15";
static const char __pyx_k_gray16[] = "gray16";
static const char __pyx_k_gray17[] = "gray17";
static const char __pyx_k_gray18[] = "gray18";
static const char __pyx_k_gray19[] = "gray19";
static const char __pyx_k_gray20[] = "gray20";
static const char __pyx_k_gray21[] = "gray21";
static const char __pyx_k_gray22[] = "gray22";
static const char __pyx_k_gray23[] = "gray23";
static const char __pyx_k_gray24[] = "gray24";
static const char __pyx_k_gray25[] = "gray25";
static const char __pyx_k_gray26[] = "gray26";
static const char __pyx_k_gray27[] = "gray27";
static const char __pyx_k_gray28[] = "gray28";
static const char __pyx_k_gray29[] = "gray29";
static const char __pyx_k_gray30[] = "gray30";
static const char __pyx_k_gray31[] = "gray31";
static const char __pyx_k_gray32[] = "gray32";
static const char __pyx_k_gray33[] = "gray33";
static const char __pyx_k_gray34[] = "gray34";
static const char __pyx_k_gray35[] = "gray35";
static const char __pyx_k_gray36[] = "gray36";
static const char __pyx_k_gray37[] = "gray37";
static const char __pyx_k_gray38[] = "gray38";
static const char __pyx_k_gray39[] = "gray39";
static const char __pyx_k_gray40[] = "gray40";
static const char __pyx_k_gray41[] = "gray41";
static const char __pyx_k_gray42[] = "gray42";
static const char __pyx_k_gray43[] = "gray43";
static const char __pyx_k_gray44[] = "gray44";
static const char __pyx_k_gray45[] = "gray45";
static const char __pyx_k_gray46[] = "gray46";
static const char __pyx_k_gray47[] = "gray47";
static const char __pyx_k_gray48[] = "gray48";
static const char __pyx_k_gray49[] = "gray49";
static const char __pyx_k_gray50[] = "gray50";
static const char __pyx_k_gray51[] = "gray51";
static const char __pyx_k_gray52[] = "gray52";
static const char __pyx_k_gray53[] = "gray53";
static const char __pyx_k_gray54[] = "gray54";
static const char __pyx_k_gray55[] = "gray55";
static const char __pyx_k_gray56[] = "gray56";
static const char __pyx_k_gray57[] = "gray57";
static const char __pyx_k_gray58[] = "gray58";
static const char __pyx_k_gray59[] = "gray59";
static const char __pyx_k_gray60[] = "gray60";
static const char __pyx_k_gray61[] = "gray61";
static const char __pyx_k_gray62[] = "gray62";
static const char __pyx_k_gray63[] = "gray63";
static const char __pyx_k_gray64[] = "gray64";
static const char __pyx_k_gray65[] = "gray65";
static const char __pyx_k_gray66[] = "gray66";
static const char __pyx_k_gray67[] = "gray67";
static const char __pyx_k_gray68[] = "gray68";
static const char __pyx_k_gray69[] = "gray69";
static const char __pyx_k_gray70[] = "gray70";
static const char __pyx_k_gray71[] = "gray71";
static const char __pyx_k_gray72[] = "gray72";
static const char __pyx_k_gray73[] = "gray73";
static const char __pyx_k_gray74[] = "gray74";
static const char __pyx_k_gray75[] = "gray75";
static const char __pyx_k_gray76[] = "gray76";
static const char __pyx_k_gray77[] = "gray77";
static const char __pyx_k_gray78[] = "gray78";
static const char __pyx_k_gray79[] = "gray79";
static const char __pyx_k_gray80[] = "gray80";
static const char __pyx_k_gray81[] = "gray81";
static const char __pyx_k_gray82[] = "gray82";
static const char __pyx_k_gray83[] = "gray83";
static const char __pyx_k_gray84[] = "gray84";
static const char __pyx_k_gray85[] = "gray85";
static const char __pyx_k_gray86[] = "gray86";
static const char __pyx_k_gray87[] = "gray87";
static const char __pyx_k_gray88[] = "gray88";
static const char __pyx_k_gray89[] = "gray89";
static const char __pyx_k_gray90[] = "gray90";
static const char __pyx_k_gray91[] = "gray91";
static const char __pyx_k_gray92[] = "gray92";
static const char __pyx_k_gray93[] = "gray93";
static const char __pyx_k_gray94[] = "gray94";
static const char __pyx_k_gray95[] = "gray95";
static const char __pyx_k_gray96[] = "gray96";
static const char __pyx_k_gray97[] = "gray97";
static const char __pyx_k_gray98[] = "gray98";
static const char __pyx_k_gray99[] = "gray99";
static const char __pyx_k_green1[] = "green1";
static const char __pyx_k_green2[] = "green2";
static const char __pyx_k_green3[] = "green3";
static const char __pyx_k_green4[] = "green4";
static const char __pyx_k_grey10[] = "grey10";
static const char __pyx_k_grey11[] = "grey11";
static const char __pyx_k_grey12[] = "grey12";
static const char __pyx_k_grey13[] = "grey13";
static const char __pyx_k_grey14[] = "grey14";
static const char __pyx_k_grey15[] = "grey15";
static const char __pyx_k_grey16[] = "grey16";
static const char __pyx_k_grey17[] = "grey17";
static const char __pyx_k_grey18[] = "grey18";
static const char __pyx_k_grey19[] = "grey19";
static const char __pyx_k_grey20[] = "grey20";
static const char __pyx_k_grey21[] = "grey21";
static const char __pyx_k_grey22[] = "grey22";
static const char __pyx_k_grey23[] = "grey23";
static const char __pyx_k_grey24[] = "grey24";
static const char __pyx_k_grey25[] = "grey25";
static const char __pyx_k_grey26[] = "grey26";
static const char __pyx_k_grey27[] = "grey27";
static const char __pyx_k_grey28[] = "grey28";
static const char __pyx_k_grey29[] = "grey29";
static const char __pyx_k_grey30[] = "grey30";
static const char __pyx_k_grey31[] = "grey31";
static const char __pyx_k_grey32[] = "grey32";
static const char __pyx_k_grey33[] = "grey33";
static const char __pyx_k_grey34[] = "grey34";
static const char __pyx_k_grey35[] = "grey35";
static const char __pyx_k_grey36[] = "grey36";
static const char __pyx_k_grey37[] = "grey37";
static const char __pyx_k_grey38[] = "grey38";
static const char __pyx_k_grey39[] = "grey39";
static const char __pyx_k_grey40[] = "grey40";
static const char __pyx_k_grey41[] = "grey41";
static const char __pyx_k_grey42[] = "grey42";
static const char __pyx_k_grey43[] = "grey43";
static const char __pyx_k_grey44[] = "grey44";
static const char __pyx_k_grey45[] = "grey45";
static const char __pyx_k_grey46[] = "grey46";
static const char __pyx_k_grey47[] = "grey47";
static const char __pyx_k_grey48[] = "grey48";
static const char __pyx_k_grey49[] = "grey49";
static const char __pyx_k_grey50[] = "grey50";
static const char __pyx_k_grey51[] = "grey51";
static const char __pyx_k_grey52[] = "grey52";
static const char __pyx_k_grey53[] = "grey53";
static const char __pyx_k_grey54[] = "grey54";
static const char __pyx_k_grey55[] = "grey55";
static const char __pyx_k_grey56[] = "grey56";
static const char __pyx_k_grey57[] = "grey57";
static const char __pyx_k_grey58[] = "grey58";
static const char __pyx_k_grey59[] = "grey59";
static const char __pyx_k_grey60[] = "grey60";
static const char __pyx_k_grey61[] = "grey61";
static const char __pyx_k_grey62[] = "grey62";
static const char __pyx_k_grey63[] = "grey63";
static const char __pyx_k_grey64[] = "grey64";
static const char __pyx_k_grey65[] = "grey65";
static const char __pyx_k_grey66[] = "grey66";
static const char __pyx_k_grey67[] = "grey67";
static const char __pyx_k_grey68[] = "grey68";
static const char __pyx_k_grey69[] = "grey69";
static const char __pyx_k_grey70[] = "grey70";
static const char __pyx_k_grey71[] = "grey71";
static const char __pyx_k_grey72[] = "grey72";
static const char __pyx_k_grey73[] = "grey73";
static const char __pyx_k_grey74[] = "grey74";
static const char __pyx_k_grey75[] = "grey75";
static const char __pyx_k_grey76[] = "grey76";
static const char __pyx_k_grey77[] = "grey77";
static const char __pyx_k_grey78[] = "grey78";
static const char __pyx_k_grey79[] = "grey79";
static const char __pyx_k_grey80[] = "grey80";
static const char __pyx_k_grey81[] = "grey81";
static const char __pyx_k_grey82[] = "grey82";
static const char __pyx_k_grey83[] = "grey83";
static const char __pyx_k_grey84[] = "grey84";
static const char __pyx_k_grey85[] = "grey85";
static const char __pyx_k_grey86[] = "grey86";
static const char __pyx_k_grey87[] = "grey87";
static const char __pyx_k_grey88[] = "grey88";
static const char __pyx_k_grey89[] = "grey89";
static const char __pyx_k_grey90[] = "grey90";
static const char __pyx_k_grey91[] = "grey91";
static const char __pyx_k_grey92[] = "grey92";
static const char __pyx_k_grey93[] = "grey93";
static const char __pyx_k_grey94[] = "grey94";
static const char __pyx_k_grey95[] = "grey95";
static const char __pyx_k_grey96[] = "grey96";
static const char __pyx_k_grey97[] = "grey97";
static const char __pyx_k_grey98[] = "grey98";
static const char __pyx_k_grey99[] = "grey99";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_ivory1[] = "ivory1";
static const char __pyx_k_ivory2[] = "ivory2";
static const char __pyx_k_ivory3[] = "ivory3";
static const char __pyx_k_ivory4[] = "ivory4";
static const char __pyx_k_khaki1[] = "khaki1";
static const char __pyx_k_khaki2[] = "khaki2";
static const char __pyx_k_khaki3[] = "khaki3";
static const char __pyx_k_khaki4[] = "khaki4";
static const char __pyx_k_maroon[] = "maroon";
static const char __pyx_k_orange[] = "orange";
static const char __pyx_k_orchid[] = "orchid";
static const char __pyx_k_purple[] = "purple";
static const char __pyx_k_salmon[] = "salmon";
static const char __pyx_k_sienna[] = "sienna";
static const char __pyx_k_struct[] = "struct";
static const char __pyx_k_tomato[] = "tomato";
static const char __pyx_k_unpack[] = "unpack";
static const char __pyx_k_violet[] = "violet";
static const char __pyx_k_wheat1[] = "wheat1";
static const char __pyx_k_wheat2[] = "wheat2";
static const char __pyx_k_wheat3[] = "wheat3";
static const char __pyx_k_wheat4[] = "wheat4";
static const char __pyx_k_yellow[] = "yellow";
static const char __pyx_k_bisque1[] = "bisque1";
static const char __pyx_k_bisque2[] = "bisque2";
static const char __pyx_k_bisque3[] = "bisque3";
static const char __pyx_k_bisque4[] = "bisque4";
static const char __pyx_k_darkred[] = "darkred";
static const char __pyx_k_dimgray[] = "dimgray";
static const char __pyx_k_dimgrey[] = "dimgrey";
static const char __pyx_k_gray100[] = "gray100";
static const char __pyx_k_grey100[] = "grey100";
static const char __pyx_k_hotpink[] = "hotpink";
static const char __pyx_k_magenta[] = "magenta";
static const char __pyx_k_maroon1[] = "maroon1";
static const char __pyx_k_maroon2[] = "maroon2";
static const char __pyx_k_maroon3[] = "maroon3";
static const char __pyx_k_maroon4[] = "maroon4";
static const char __pyx_k_oldlace[] = "oldlace";
static const char __pyx_k_orange1[] = "orange1";
static const char __pyx_k_orange2[] = "orange2";
static const char __pyx_k_orange3[] = "orange3";
static const char __pyx_k_orange4[] = "orange4";
static const char __pyx_k_orchid1[] = "orchid1";
static const char __pyx_k_orchid2[] = "orchid2";
static const char __pyx_k_orchid3[] = "orchid3";
static const char __pyx_k_orchid4[] = "orchid4";
static const char __pyx_k_purple1[] = "purple1";
static const char __pyx_k_purple2[] = "purple2";
static const char __pyx_k_purple3[] = "purple3";
static const char __pyx_k_purple4[] = "purple4";
static const char __pyx_k_salmon1[] = "salmon1";
static const char __pyx_k_salmon2[] = "salmon2";
static const char __pyx_k_salmon3[] = "salmon3";
static const char __pyx_k_salmon4[] = "salmon4";
static const char __pyx_k_sienna1[] = "sienna1";
static const char __pyx_k_sienna2[] = "sienna2";
static const char __pyx_k_sienna3[] = "sienna3";
static const char __pyx_k_sienna4[] = "sienna4";
static const char __pyx_k_skyblue[] = "skyblue";
static const char __pyx_k_thistle[] = "thistle";
static const char __pyx_k_tomato1[] = "tomato1";
static const char __pyx_k_tomato2[] = "tomato2";
static const char __pyx_k_tomato3[] = "tomato3";
static const char __pyx_k_tomato4[] = "tomato4";
static const char __pyx_k_yellow1[] = "yellow1";
static const char __pyx_k_yellow2[] = "yellow2";
static const char __pyx_k_yellow3[] = "yellow3";
static const char __pyx_k_yellow4[] = "yellow4";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_binascii[] = "binascii";
static const char __pyx_k_cornsilk[] = "cornsilk";
static const char __pyx_k_darkblue[] = "darkblue";
static const char __pyx_k_darkcyan[] = "darkcyan";
static const char __pyx_k_darkgray[] = "darkgray";
static const char __pyx_k_darkgrey[] = "darkgrey";
static const char __pyx_k_deeppink[] = "deeppink";
static const char __pyx_k_honeydew[] = "honeydew";
static const char __pyx_k_hotpink1[] = "hotpink1";
static const char __pyx_k_hotpink2[] = "hotpink2";
static const char __pyx_k_hotpink3[] = "hotpink3";
static const char __pyx_k_hotpink4[] = "hotpink4";
static const char __pyx_k_lavender[] = "lavender";
static const char __pyx_k_magenta1[] = "magenta1";
static const char __pyx_k_magenta2[] = "magenta2";
static const char __pyx_k_magenta3[] = "magenta3";
static const char __pyx_k_magenta4[] = "magenta4";
static const char __pyx_k_moccasin[] = "moccasin";
static const char __pyx_k_navyblue[] = "navyblue";
static const char __pyx_k_seagreen[] = "seagreen";
static const char __pyx_k_seashell[] = "seashell";
static const char __pyx_k_skyblue1[] = "skyblue1";
static const char __pyx_k_skyblue2[] = "skyblue2";
static const char __pyx_k_skyblue3[] = "skyblue3";
static const char __pyx_k_skyblue4[] = "skyblue4";
static const char __pyx_k_thistle1[] = "thistle1";
static const char __pyx_k_thistle2[] = "thistle2";
static const char __pyx_k_thistle3[] = "thistle3";
static const char __pyx_k_thistle4[] = "thistle4";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_aliceblue[] = "aliceblue";
static const char __pyx_k_burlywood[] = "burlywood";
static const char __pyx_k_cadetblue[] = "cadetblue";
static const char __pyx_k_chocolate[] = "chocolate";
static const char __pyx_k_cornsilk1[] = "cornsilk1";
static const char __pyx_k_cornsilk2[] = "cornsilk2";
static const char __pyx_k_cornsilk3[] = "cornsilk3";
static const char __pyx_k_cornsilk4[] = "cornsilk4";
static const char __pyx_k_darkgreen[] = "darkgreen";
static const char __pyx_k_darkkhaki[] = "darkkhaki";
static const char __pyx_k_debianred[] = "debianred";
static const char __pyx_k_deeppink1[] = "deeppink1";
static const char __pyx_k_deeppink2[] = "deeppink2";
static const char __pyx_k_deeppink3[] = "deeppink3";
static const char __pyx_k_deeppink4[] = "deeppink4";
static const char __pyx_k_firebrick[] = "firebrick";
static const char __pyx_k_gainsboro[] = "gainsboro";
static const char __pyx_k_goldenrod[] = "goldenrod";
static const char __pyx_k_honeydew1[] = "honeydew1";
static const char __pyx_k_honeydew2[] = "honeydew2";
static const char __pyx_k_honeydew3[] = "honeydew3";
static const char __pyx_k_honeydew4[] = "honeydew4";
static const char __pyx_k_indianred[] = "indianred";
static const char __pyx_k_lawngreen[] = "lawngreen";
static const char __pyx_k_lightblue[] = "lightblue";
static const char __pyx_k_lightcyan[] = "lightcyan";
static const char __pyx_k_lightgray[] = "lightgray";
static const char __pyx_k_lightgrey[] = "lightgrey";
static const char __pyx_k_lightpink[] = "lightpink";
static const char __pyx_k_limegreen[] = "limegreen";
static const char __pyx_k_mintcream[] = "mintcream";
static const char __pyx_k_mistyrose[] = "mistyrose";
static const char __pyx_k_olivedrab[] = "olivedrab";
static const char __pyx_k_orangered[] = "orangered";
static const char __pyx_k_palegreen[] = "palegreen";
static const char __pyx_k_peachpuff[] = "peachpuff";
static const char __pyx_k_rosybrown[] = "rosybrown";
static const char __pyx_k_royalblue[] = "royalblue";
static const char __pyx_k_seagreen1[] = "seagreen1";
static const char __pyx_k_seagreen2[] = "seagreen2";
static const char __pyx_k_seagreen3[] = "seagreen3";
static const char __pyx_k_seagreen4[] = "seagreen4";
static const char __pyx_k_seashell1[] = "seashell1";
static const char __pyx_k_seashell2[] = "seashell2";
static const char __pyx_k_seashell3[] = "seashell3";
static const char __pyx_k_seashell4[] = "seashell4";
static const char __pyx_k_slateblue[] = "slateblue";
static const char __pyx_k_slategray[] = "slategray";
static const char __pyx_k_slategrey[] = "slategrey";
static const char __pyx_k_steelblue[] = "steelblue";
static const char __pyx_k_turquoise[] = "turquoise";
static const char __pyx_k_unhexlify[] = "unhexlify";
static const char __pyx_k_violetred[] = "violetred";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_aquamarine[] = "aquamarine";
static const char __pyx_k_blueviolet[] = "blueviolet";
static const char __pyx_k_burlywood1[] = "burlywood1";
static const char __pyx_k_burlywood2[] = "burlywood2";
static const char __pyx_k_burlywood3[] = "burlywood3";
static const char __pyx_k_burlywood4[] = "burlywood4";
static const char __pyx_k_cadetblue1[] = "cadetblue1";
static const char __pyx_k_cadetblue2[] = "cadetblue2";
static const char __pyx_k_cadetblue3[] = "cadetblue3";
static const char __pyx_k_cadetblue4[] = "cadetblue4";
static const char __pyx_k_chartreuse[] = "chartreuse";
static const char __pyx_k_chocolate1[] = "chocolate1";
static const char __pyx_k_chocolate2[] = "chocolate2";
static const char __pyx_k_chocolate3[] = "chocolate3";
static const char __pyx_k_chocolate4[] = "chocolate4";
static const char __pyx_k_darkorange[] = "darkorange";
static const char __pyx_k_darkorchid[] = "darkorchid";
static const char __pyx_k_darksalmon[] = "darksalmon";
static const char __pyx_k_darkviolet[] = "darkviolet";
static const char __pyx_k_dodgerblue[] = "dodgerblue";
static const char __pyx_k_firebrick1[] = "firebrick1";
static const char __pyx_k_firebrick2[] = "firebrick2";
static const char __pyx_k_firebrick3[] = "firebrick3";
static const char __pyx_k_firebrick4[] = "firebrick4";
static const char __pyx_k_ghostwhite[] = "ghostwhite";
static const char __pyx_k_goldenrod1[] = "goldenrod1";
static const char __pyx_k_goldenrod2[] = "goldenrod2";
static const char __pyx_k_goldenrod3[] = "goldenrod3";
static const char __pyx_k_goldenrod4[] = "goldenrod4";
static const char __pyx_k_indianred1[] = "indianred1";
static const char __pyx_k_indianred2[] = "indianred2";
static const char __pyx_k_indianred3[] = "indianred3";
static const char __pyx_k_indianred4[] = "indianred4";
static const char __pyx_k_lightblue1[] = "lightblue1";
static const char __pyx_k_lightblue2[] = "lightblue2";
static const char __pyx_k_lightblue3[] = "lightblue3";
static const char __pyx_k_lightblue4[] = "lightblue4";
static const char __pyx_k_lightcoral[] = "lightcoral";
static const char __pyx_k_lightcyan1[] = "lightcyan1";
static const char __pyx_k_lightcyan2[] = "lightcyan2";
static const char __pyx_k_lightcyan3[] = "lightcyan3";
static const char __pyx_k_lightcyan4[] = "lightcyan4";
static const char __pyx_k_lightgreen[] = "lightgreen";
static const char __pyx_k_lightpink1[] = "lightpink1";
static const char __pyx_k_lightpink2[] = "lightpink2";
static const char __pyx_k_lightpink3[] = "lightpink3";
static const char __pyx_k_lightpink4[] = "lightpink4";
static const char __pyx_k_mediumblue[] = "mediumblue";
static const char __pyx_k_mistyrose1[] = "mistyrose1";
static const char __pyx_k_mistyrose2[] = "mistyrose2";
static const char __pyx_k_mistyrose3[] = "mistyrose3";
static const char __pyx_k_mistyrose4[] = "mistyrose4";
static const char __pyx_k_olivedrab1[] = "olivedrab1";
static const char __pyx_k_olivedrab2[] = "olivedrab2";
static const char __pyx_k_olivedrab3[] = "olivedrab3";
static const char __pyx_k_olivedrab4[] = "olivedrab4";
static const char __pyx_k_orangered1[] = "orangered1";
static const char __pyx_k_orangered2[] = "orangered2";
static const char __pyx_k_orangered3[] = "orangered3";
static const char __pyx_k_orangered4[] = "orangered4";
static const char __pyx_k_palegreen1[] = "palegreen1";
static const char __pyx_k_palegreen2[] = "palegreen2";
static const char __pyx_k_palegreen3[] = "palegreen3";
static const char __pyx_k_palegreen4[] = "palegreen4";
static const char __pyx_k_papayawhip[] = "papayawhip";
static const char __pyx_k_peachpuff1[] = "peachpuff1";
static const char __pyx_k_peachpuff2[] = "peachpuff2";
static const char __pyx_k_peachpuff3[] = "peachpuff3";
static const char __pyx_k_peachpuff4[] = "peachpuff4";
static const char __pyx_k_powderblue[] = "powderblue";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_rosybrown1[] = "rosybrown1";
static const char __pyx_k_rosybrown2[] = "rosybrown2";
static const char __pyx_k_rosybrown3[] = "rosybrown3";
static const char __pyx_k_rosybrown4[] = "rosybrown4";
static const char __pyx_k_royalblue1[] = "royalblue1";
static const char __pyx_k_royalblue2[] = "royalblue2";
static const char __pyx_k_royalblue3[] = "royalblue3";
static const char __pyx_k_royalblue4[] = "royalblue4";
static const char __pyx_k_sandybrown[] = "sandybrown";
static const char __pyx_k_slateblue1[] = "slateblue1";
static const char __pyx_k_slateblue2[] = "slateblue2";
static const char __pyx_k_slateblue3[] = "slateblue3";
static const char __pyx_k_slateblue4[] = "slateblue4";
static const char __pyx_k_slategray1[] = "slategray1";
static const char __pyx_k_slategray2[] = "slategray2";
static const char __pyx_k_slategray3[] = "slategray3";
static const char __pyx_k_slategray4[] = "slategray4";
static const char __pyx_k_startswith[] = "startswith";
static const char __pyx_k_steelblue1[] = "steelblue1";
static const char __pyx_k_steelblue2[] = "steelblue2";
static const char __pyx_k_steelblue3[] = "steelblue3";
static const char __pyx_k_steelblue4[] = "steelblue4";
static const char __pyx_k_turquoise1[] = "turquoise1";
static const char __pyx_k_turquoise2[] = "turquoise2";
static const char __pyx_k_turquoise3[] = "turquoise3";
static const char __pyx_k_turquoise4[] = "turquoise4";
static const char __pyx_k_violetred1[] = "violetred1";
static const char __pyx_k_violetred2[] = "violetred2";
static const char __pyx_k_violetred3[] = "violetred3";
static const char __pyx_k_violetred4[] = "violetred4";
static const char __pyx_k_whitesmoke[] = "whitesmoke";
static const char __pyx_k_aquamarine1[] = "aquamarine1";
static const char __pyx_k_aquamarine2[] = "aquamarine2";
static const char __pyx_k_aquamarine3[] = "aquamarine3";
static const char __pyx_k_aquamarine4[] = "aquamarine4";
static const char __pyx_k_chartreuse1[] = "chartreuse1";
static const char __pyx_k_chartreuse2[] = "chartreuse2";
static const char __pyx_k_chartreuse3[] = "chartreuse3";
static const char __pyx_k_chartreuse4[] = "chartreuse4";
static const char __pyx_k_darkmagenta[] = "darkmagenta";
static const char __pyx_k_darkorange1[] = "darkorange1";
static const char __pyx_k_darkorange2[] = "darkorange2";
static const char __pyx_k_darkorange3[] = "darkorange3";
static const char __pyx_k_darkorange4[] = "darkorange4";
static const char __pyx_k_darkorchid1[] = "darkorchid1";
static const char __pyx_k_darkorchid2[] = "darkorchid2";
static const char __pyx_k_darkorchid3[] = "darkorchid3";
static const char __pyx_k_darkorchid4[] = "darkorchid4";
static const char __pyx_k_deepskyblue[] = "deepskyblue";
static const char __pyx_k_dodgerblue1[] = "dodgerblue1";
static const char __pyx_k_dodgerblue2[] = "dodgerblue2";
static const char __pyx_k_dodgerblue3[] = "dodgerblue3";
static const char __pyx_k_dodgerblue4[] = "dodgerblue4";
static const char __pyx_k_floralwhite[] = "floralwhite";
static const char __pyx_k_forestgreen[] = "forestgreen";
static const char __pyx_k_greenyellow[] = "greenyellow";
static const char __pyx_k_lightsalmon[] = "lightsalmon";
static const char __pyx_k_lightyellow[] = "lightyellow";
static const char __pyx_k_navajowhite[] = "navajowhite";
static const char __pyx_k_saddlebrown[] = "saddlebrown";
static const char __pyx_k_springgreen[] = "springgreen";
static const char __pyx_k_yellowgreen[] = "yellowgreen";
static const char __pyx_k_antiquewhite[] = "antiquewhite";
static const char __pyx_k_darkseagreen[] = "darkseagreen";
static const char __pyx_k_deepskyblue1[] = "deepskyblue1";
static const char __pyx_k_deepskyblue2[] = "deepskyblue2";
static const char __pyx_k_deepskyblue3[] = "deepskyblue3";
static const char __pyx_k_deepskyblue4[] = "deepskyblue4";
static const char __pyx_k_lemonchiffon[] = "lemonchiffon";
static const char __pyx_k_lightsalmon1[] = "lightsalmon1";
static const char __pyx_k_lightsalmon2[] = "lightsalmon2";
static const char __pyx_k_lightsalmon3[] = "lightsalmon3";
static const char __pyx_k_lightsalmon4[] = "lightsalmon4";
static const char __pyx_k_lightskyblue[] = "lightskyblue";
static const char __pyx_k_lightyellow1[] = "lightyellow1";
static const char __pyx_k_lightyellow2[] = "lightyellow2";
static const char __pyx_k_lightyellow3[] = "lightyellow3";
static const char __pyx_k_lightyellow4[] = "lightyellow4";
static const char __pyx_k_mediumorchid[] = "mediumorchid";
static const char __pyx_k_mediumpurple[] = "mediumpurple";
static const char __pyx_k_midnightblue[] = "midnightblue";
static const char __pyx_k_navajowhite1[] = "navajowhite1";
static const char __pyx_k_navajowhite2[] = "navajowhite2";
static const char __pyx_k_navajowhite3[] = "navajowhite3";
static const char __pyx_k_navajowhite4[] = "navajowhite4";
static const char __pyx_k_springgreen1[] = "springgreen1";
static const char __pyx_k_springgreen2[] = "springgreen2";
static const char __pyx_k_springgreen3[] = "springgreen3";
static const char __pyx_k_springgreen4[] = "springgreen4";
static const char __pyx_k_antiquewhite1[] = "antiquewhite1";
static const char __pyx_k_antiquewhite2[] = "antiquewhite2";
static const char __pyx_k_antiquewhite3[] = "antiquewhite3";
static const char __pyx_k_antiquewhite4[] = "antiquewhite4";
static const char __pyx_k_darkgoldenrod[] = "darkgoldenrod";
static const char __pyx_k_darkseagreen1[] = "darkseagreen1";
static const char __pyx_k_darkseagreen2[] = "darkseagreen2";
static const char __pyx_k_darkseagreen3[] = "darkseagreen3";
static const char __pyx_k_darkseagreen4[] = "darkseagreen4";
static const char __pyx_k_darkslateblue[] = "darkslateblue";
static const char __pyx_k_darkslategray[] = "darkslategray";
static const char __pyx_k_darkslategrey[] = "darkslategrey";
static const char __pyx_k_darkturquoise[] = "darkturquoise";
static const char __pyx_k_lavenderblush[] = "lavenderblush";
static const char __pyx_k_lemonchiffon1[] = "lemonchiffon1";
static const char __pyx_k_lemonchiffon2[] = "lemonchiffon2";
static const char __pyx_k_lemonchiffon3[] = "lemonchiffon3";
static const char __pyx_k_lemonchiffon4[] = "lemonchiffon4";
static const char __pyx_k_lightseagreen[] = "lightseagreen";
static const char __pyx_k_lightskyblue1[] = "lightskyblue1";
static const char __pyx_k_lightskyblue2[] = "lightskyblue2";
static const char __pyx_k_lightskyblue3[] = "lightskyblue3";
static const char __pyx_k_lightskyblue4[] = "lightskyblue4";
static const char __pyx_k_mediumorchid1[] = "mediumorchid1";
static const char __pyx_k_mediumorchid2[] = "mediumorchid2";
static const char __pyx_k_mediumorchid3[] = "mediumorchid3";
static const char __pyx_k_mediumorchid4[] = "mediumorchid4";
static const char __pyx_k_mediumpurple1[] = "mediumpurple1";
static const char __pyx_k_mediumpurple2[] = "mediumpurple2";
static const char __pyx_k_mediumpurple3[] = "mediumpurple3";
static const char __pyx_k_mediumpurple4[] = "mediumpurple4";
static const char __pyx_k_palegoldenrod[] = "palegoldenrod";
static const char __pyx_k_paleturquoise[] = "paleturquoise";
static const char __pyx_k_palevioletred[] = "palevioletred";
static const char __pyx_k_blanchedalmond[] = "blanchedalmond";
static const char __pyx_k_cornflowerblue[] = "cornflowerblue";
static const char __pyx_k_darkgoldenrod1[] = "darkgoldenrod1";
static const char __pyx_k_darkgoldenrod2[] = "darkgoldenrod2";
static const char __pyx_k_darkgoldenrod3[] = "darkgoldenrod3";
static const char __pyx_k_darkgoldenrod4[] = "darkgoldenrod4";
static const char __pyx_k_darkolivegreen[] = "darkolivegreen";
static const char __pyx_k_darkslategray1[] = "darkslategray1";
static const char __pyx_k_darkslategray2[] = "darkslategray2";
static const char __pyx_k_darkslategray3[] = "darkslategray3";
static const char __pyx_k_darkslategray4[] = "darkslategray4";
static const char __pyx_k_lavenderblush1[] = "lavenderblush1";
static const char __pyx_k_lavenderblush2[] = "lavenderblush2";
static const char __pyx_k_lavenderblush3[] = "lavenderblush3";
static const char __pyx_k_lavenderblush4[] = "lavenderblush4";
static const char __pyx_k_lightgoldenrod[] = "lightgoldenrod";
static const char __pyx_k_lightslateblue[] = "lightslateblue";
static const char __pyx_k_lightslategray[] = "lightslategray";
static const char __pyx_k_lightslategrey[] = "lightslategrey";
static const char __pyx_k_lightsteelblue[] = "lightsteelblue";
static const char __pyx_k_mediumseagreen[] = "mediumseagreen";
static const char __pyx_k_paleturquoise1[] = "paleturquoise1";
static const char __pyx_k_paleturquoise2[] = "paleturquoise2";
static const char __pyx_k_paleturquoise3[] = "paleturquoise3";
static const char __pyx_k_paleturquoise4[] = "paleturquoise4";
static const char __pyx_k_palevioletred1[] = "palevioletred1";
static const char __pyx_k_palevioletred2[] = "palevioletred2";
static const char __pyx_k_palevioletred3[] = "palevioletred3";
static const char __pyx_k_palevioletred4[] = "palevioletred4";
static const char __pyx_k_darkolivegreen1[] = "darkolivegreen1";
static const char __pyx_k_darkolivegreen2[] = "darkolivegreen2";
static const char __pyx_k_darkolivegreen3[] = "darkolivegreen3";
static const char __pyx_k_darkolivegreen4[] = "darkolivegreen4";
static const char __pyx_k_lightgoldenrod1[] = "lightgoldenrod1";
static const char __pyx_k_lightgoldenrod2[] = "lightgoldenrod2";
static const char __pyx_k_lightgoldenrod3[] = "lightgoldenrod3";
static const char __pyx_k_lightgoldenrod4[] = "lightgoldenrod4";
static const char __pyx_k_lightsteelblue1[] = "lightsteelblue1";
static const char __pyx_k_lightsteelblue2[] = "lightsteelblue2";
static const char __pyx_k_lightsteelblue3[] = "lightsteelblue3";
static const char __pyx_k_lightsteelblue4[] = "lightsteelblue4";
static const char __pyx_k_mediumslateblue[] = "mediumslateblue";
static const char __pyx_k_mediumturquoise[] = "mediumturquoise";
static const char __pyx_k_mediumvioletred[] = "mediumvioletred";
static const char __pyx_k_Expected_a_color[] = "Expected a color.";
static const char __pyx_k_mediumaquamarine[] = "mediumaquamarine";
static const char __pyx_k_mod___locals_mod[] = "__mod__.<locals>.mod";
static const char __pyx_k_mediumspringgreen[] = "mediumspringgreen";
static const char __pyx_k_pygame_sdl2_color[] = "pygame_sdl2.color";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_lightgoldenrodyellow[] = "lightgoldenrodyellow";
static const char __pyx_k_floordiv___locals_div[] = "__floordiv__.<locals>.div";
static const char __pyx_k_src_pygame_sdl2_color_pyx[] = "src/pygame_sdl2/color.pyx";
static const char __pyx_k_correct_gamma_locals_lambda[] = "correct_gamma.<locals>.<lambda>";
static const char __pyx_k_Color_from_hex_locals_lambda[] = "Color.from_hex.<locals>.<lambda>";
static PyObject *__pyx_kp_u_08x;
static PyObject *__pyx_kp_u_0x;
static PyObject *__pyx_n_u_BBB;
static PyObject *__pyx_n_u_BBBB;
static PyObject *__pyx_n_s_Color;
static PyObject *__pyx_n_s_Color_from_hex_locals_lambda;
static PyObject *__pyx_kp_u_Expected_a_color;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_kp_u_L;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_u__2;
static PyObject *__pyx_kp_u__3;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_u_aliceblue;
static PyObject *__pyx_n_u_antiquewhite;
static PyObject *__pyx_n_u_antiquewhite1;
static PyObject *__pyx_n_u_antiquewhite2;
static PyObject *__pyx_n_u_antiquewhite3;
static PyObject *__pyx_n_u_antiquewhite4;
static PyObject *__pyx_n_u_aquamarine;
static PyObject *__pyx_n_u_aquamarine1;
static PyObject *__pyx_n_u_aquamarine2;
static PyObject *__pyx_n_u_aquamarine3;
static PyObject *__pyx_n_u_aquamarine4;
static PyObject *__pyx_n_u_azure;
static PyObject *__pyx_n_u_azure1;
static PyObject *__pyx_n_u_azure2;
static PyObject *__pyx_n_u_azure3;
static PyObject *__pyx_n_u_azure4;
static PyObject *__pyx_n_s_b;
static PyObject *__pyx_n_u_beige;
static PyObject *__pyx_n_s_binascii;
static PyObject *__pyx_n_u_bisque;
static PyObject *__pyx_n_u_bisque1;
static PyObject *__pyx_n_u_bisque2;
static PyObject *__pyx_n_u_bisque3;
static PyObject *__pyx_n_u_bisque4;
static PyObject *__pyx_n_u_black;
static PyObject *__pyx_n_u_blanchedalmond;
static PyObject *__pyx_n_u_blue;
static PyObject *__pyx_n_u_blue1;
static PyObject *__pyx_n_u_blue2;
static PyObject *__pyx_n_u_blue3;
static PyObject *__pyx_n_u_blue4;
static PyObject *__pyx_n_u_blueviolet;
static PyObject *__pyx_n_u_brown;
static PyObject *__pyx_n_u_brown1;
static PyObject *__pyx_n_u_brown2;
static PyObject *__pyx_n_u_brown3;
static PyObject *__pyx_n_u_brown4;
static PyObject *__pyx_n_u_burlywood;
static PyObject *__pyx_n_u_burlywood1;
static PyObject *__pyx_n_u_burlywood2;
static PyObject *__pyx_n_u_burlywood3;
static PyObject *__pyx_n_u_burlywood4;
static PyObject *__pyx_n_u_cadetblue;
static PyObject *__pyx_n_u_cadetblue1;
static PyObject *__pyx_n_u_cadetblue2;
static PyObject *__pyx_n_u_cadetblue3;
static PyObject *__pyx_n_u_cadetblue4;
static PyObject *__pyx_n_u_chartreuse;
static PyObject *__pyx_n_u_chartreuse1;
static PyObject *__pyx_n_u_chartreuse2;
static PyObject *__pyx_n_u_chartreuse3;
static PyObject *__pyx_n_u_chartreuse4;
static PyObject *__pyx_n_u_chocolate;
static PyObject *__pyx_n_u_chocolate1;
static PyObject *__pyx_n_u_chocolate2;
static PyObject *__pyx_n_u_chocolate3;
static PyObject *__pyx_n_u_chocolate4;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_u_coral;
static PyObject *__pyx_n_u_coral1;
static PyObject *__pyx_n_u_coral2;
static PyObject *__pyx_n_u_coral3;
static PyObject *__pyx_n_u_coral4;
static PyObject *__pyx_n_u_cornflowerblue;
static PyObject *__pyx_n_u_cornsilk;
static PyObject *__pyx_n_u_cornsilk1;
static PyObject *__pyx_n_u_cornsilk2;
static PyObject *__pyx_n_u_cornsilk3;
static PyObject *__pyx_n_u_cornsilk4;
static PyObject *__pyx_n_s_correct_gamma_locals_lambda;
static PyObject *__pyx_n_u_cyan;
static PyObject *__pyx_n_u_cyan1;
static PyObject *__pyx_n_u_cyan2;
static PyObject *__pyx_n_u_cyan3;
static PyObject *__pyx_n_u_cyan4;
static PyObject *__pyx_n_u_darkblue;
static PyObject *__pyx_n_u_darkcyan;
static PyObject *__pyx_n_u_darkgoldenrod;
static PyObject *__pyx_n_u_darkgoldenrod1;
static PyObject *__pyx_n_u_darkgoldenrod2;
static PyObject *__pyx_n_u_darkgoldenrod3;
static PyObject *__pyx_n_u_darkgoldenrod4;
static PyObject *__pyx_n_u_darkgray;
static PyObject *__pyx_n_u_darkgreen;
static PyObject *__pyx_n_u_darkgrey;
static PyObject *__pyx_n_u_darkkhaki;
static PyObject *__pyx_n_u_darkmagenta;
static PyObject *__pyx_n_u_darkolivegreen;
static PyObject *__pyx_n_u_darkolivegreen1;
static PyObject *__pyx_n_u_darkolivegreen2;
static PyObject *__pyx_n_u_darkolivegreen3;
static PyObject *__pyx_n_u_darkolivegreen4;
static PyObject *__pyx_n_u_darkorange;
static PyObject *__pyx_n_u_darkorange1;
static PyObject *__pyx_n_u_darkorange2;
static PyObject *__pyx_n_u_darkorange3;
static PyObject *__pyx_n_u_darkorange4;
static PyObject *__pyx_n_u_darkorchid;
static PyObject *__pyx_n_u_darkorchid1;
static PyObject *__pyx_n_u_darkorchid2;
static PyObject *__pyx_n_u_darkorchid3;
static PyObject *__pyx_n_u_darkorchid4;
static PyObject *__pyx_n_u_darkred;
static PyObject *__pyx_n_u_darksalmon;
static PyObject *__pyx_n_u_darkseagreen;
static PyObject *__pyx_n_u_darkseagreen1;
static PyObject *__pyx_n_u_darkseagreen2;
static PyObject *__pyx_n_u_darkseagreen3;
static PyObject *__pyx_n_u_darkseagreen4;
static PyObject *__pyx_n_u_darkslateblue;
static PyObject *__pyx_n_u_darkslategray;
static PyObject *__pyx_n_u_darkslategray1;
static PyObject *__pyx_n_u_darkslategray2;
static PyObject *__pyx_n_u_darkslategray3;
static PyObject *__pyx_n_u_darkslategray4;
static PyObject *__pyx_n_u_darkslategrey;
static PyObject *__pyx_n_u_darkturquoise;
static PyObject *__pyx_n_u_darkviolet;
static PyObject *__pyx_n_u_debianred;
static PyObject *__pyx_n_u_deeppink;
static PyObject *__pyx_n_u_deeppink1;
static PyObject *__pyx_n_u_deeppink2;
static PyObject *__pyx_n_u_deeppink3;
static PyObject *__pyx_n_u_deeppink4;
static PyObject *__pyx_n_u_deepskyblue;
static PyObject *__pyx_n_u_deepskyblue1;
static PyObject *__pyx_n_u_deepskyblue2;
static PyObject *__pyx_n_u_deepskyblue3;
static PyObject *__pyx_n_u_deepskyblue4;
static PyObject *__pyx_n_u_dimgray;
static PyObject *__pyx_n_u_dimgrey;
static PyObject *__pyx_n_s_div;
static PyObject *__pyx_n_u_dodgerblue;
static PyObject *__pyx_n_u_dodgerblue1;
static PyObject *__pyx_n_u_dodgerblue2;
static PyObject *__pyx_n_u_dodgerblue3;
static PyObject *__pyx_n_u_dodgerblue4;
static PyObject *__pyx_n_u_firebrick;
static PyObject *__pyx_n_u_firebrick1;
static PyObject *__pyx_n_u_firebrick2;
static PyObject *__pyx_n_u_firebrick3;
static PyObject *__pyx_n_u_firebrick4;
static PyObject *__pyx_n_s_floordiv___locals_div;
static PyObject *__pyx_n_u_floralwhite;
static PyObject *__pyx_n_u_forestgreen;
static PyObject *__pyx_n_s_g;
static PyObject *__pyx_n_u_gainsboro;
static PyObject *__pyx_n_u_ghostwhite;
static PyObject *__pyx_n_u_gold;
static PyObject *__pyx_n_u_gold1;
static PyObject *__pyx_n_u_gold2;
static PyObject *__pyx_n_u_gold3;
static PyObject *__pyx_n_u_gold4;
static PyObject *__pyx_n_u_goldenrod;
static PyObject *__pyx_n_u_goldenrod1;
static PyObject *__pyx_n_u_goldenrod2;
static PyObject *__pyx_n_u_goldenrod3;
static PyObject *__pyx_n_u_goldenrod4;
static PyObject *__pyx_n_u_gray;
static PyObject *__pyx_n_u_gray0;
static PyObject *__pyx_n_u_gray1;
static PyObject *__pyx_n_u_gray10;
static PyObject *__pyx_n_u_gray100;
static PyObject *__pyx_n_u_gray11;
static PyObject *__pyx_n_u_gray12;
static PyObject *__pyx_n_u_gray13;
static PyObject *__pyx_n_u_gray14;
static PyObject *__pyx_n_u_gray15;
static PyObject *__pyx_n_u_gray16;
static PyObject *__pyx_n_u_gray17;
static PyObject *__pyx_n_u_gray18;
static PyObject *__pyx_n_u_gray19;
static PyObject *__pyx_n_u_gray2;
static PyObject *__pyx_n_u_gray20;
static PyObject *__pyx_n_u_gray21;
static PyObject *__pyx_n_u_gray22;
static PyObject *__pyx_n_u_gray23;
static PyObject *__pyx_n_u_gray24;
static PyObject *__pyx_n_u_gray25;
static PyObject *__pyx_n_u_gray26;
static PyObject *__pyx_n_u_gray27;
static PyObject *__pyx_n_u_gray28;
static PyObject *__pyx_n_u_gray29;
static PyObject *__pyx_n_u_gray3;
static PyObject *__pyx_n_u_gray30;
static PyObject *__pyx_n_u_gray31;
static PyObject *__pyx_n_u_gray32;
static PyObject *__pyx_n_u_gray33;
static PyObject *__pyx_n_u_gray34;
static PyObject *__pyx_n_u_gray35;
static PyObject *__pyx_n_u_gray36;
static PyObject *__pyx_n_u_gray37;
static PyObject *__pyx_n_u_gray38;
static PyObject *__pyx_n_u_gray39;
static PyObject *__pyx_n_u_gray4;
static PyObject *__pyx_n_u_gray40;
static PyObject *__pyx_n_u_gray41;
static PyObject *__pyx_n_u_gray42;
static PyObject *__pyx_n_u_gray43;
static PyObject *__pyx_n_u_gray44;
static PyObject *__pyx_n_u_gray45;
static PyObject *__pyx_n_u_gray46;
static PyObject *__pyx_n_u_gray47;
static PyObject *__pyx_n_u_gray48;
static PyObject *__pyx_n_u_gray49;
static PyObject *__pyx_n_u_gray5;
static PyObject *__pyx_n_u_gray50;
static PyObject *__pyx_n_u_gray51;
static PyObject *__pyx_n_u_gray52;
static PyObject *__pyx_n_u_gray53;
static PyObject *__pyx_n_u_gray54;
static PyObject *__pyx_n_u_gray55;
static PyObject *__pyx_n_u_gray56;
static PyObject *__pyx_n_u_gray57;
static PyObject *__pyx_n_u_gray58;
static PyObject *__pyx_n_u_gray59;
static PyObject *__pyx_n_u_gray6;
static PyObject *__pyx_n_u_gray60;
static PyObject *__pyx_n_u_gray61;
static PyObject *__pyx_n_u_gray62;
static PyObject *__pyx_n_u_gray63;
static PyObject *__pyx_n_u_gray64;
static PyObject *__pyx_n_u_gray65;
static PyObject *__pyx_n_u_gray66;
static PyObject *__pyx_n_u_gray67;
static PyObject *__pyx_n_u_gray68;
static PyObject *__pyx_n_u_gray69;
static PyObject *__pyx_n_u_gray7;
static PyObject *__pyx_n_u_gray70;
static PyObject *__pyx_n_u_gray71;
static PyObject *__pyx_n_u_gray72;
static PyObject *__pyx_n_u_gray73;
static PyObject *__pyx_n_u_gray74;
static PyObject *__pyx_n_u_gray75;
static PyObject *__pyx_n_u_gray76;
static PyObject *__pyx_n_u_gray77;
static PyObject *__pyx_n_u_gray78;
static PyObject *__pyx_n_u_gray79;
static PyObject *__pyx_n_u_gray8;
static PyObject *__pyx_n_u_gray80;
static PyObject *__pyx_n_u_gray81;
static PyObject *__pyx_n_u_gray82;
static PyObject *__pyx_n_u_gray83;
static PyObject *__pyx_n_u_gray84;
static PyObject *__pyx_n_u_gray85;
static PyObject *__pyx_n_u_gray86;
static PyObject *__pyx_n_u_gray87;
static PyObject *__pyx_n_u_gray88;
static PyObject *__pyx_n_u_gray89;
static PyObject *__pyx_n_u_gray9;
static PyObject *__pyx_n_u_gray90;
static PyObject *__pyx_n_u_gray91;
static PyObject *__pyx_n_u_gray92;
static PyObject *__pyx_n_u_gray93;
static PyObject *__pyx_n_u_gray94;
static PyObject *__pyx_n_u_gray95;
static PyObject *__pyx_n_u_gray96;
static PyObject *__pyx_n_u_gray97;
static PyObject *__pyx_n_u_gray98;
static PyObject *__pyx_n_u_gray99;
static PyObject *__pyx_n_u_green;
static PyObject *__pyx_n_u_green1;
static PyObject *__pyx_n_u_green2;
static PyObject *__pyx_n_u_green3;
static PyObject *__pyx_n_u_green4;
static PyObject *__pyx_n_u_greenyellow;
static PyObject *__pyx_n_u_grey;
static PyObject *__pyx_n_u_grey0;
static PyObject *__pyx_n_u_grey1;
static PyObject *__pyx_n_u_grey10;
static PyObject *__pyx_n_u_grey100;
static PyObject *__pyx_n_u_grey11;
static PyObject *__pyx_n_u_grey12;
static PyObject *__pyx_n_u_grey13;
static PyObject *__pyx_n_u_grey14;
static PyObject *__pyx_n_u_grey15;
static PyObject *__pyx_n_u_grey16;
static PyObject *__pyx_n_u_grey17;
static PyObject *__pyx_n_u_grey18;
static PyObject *__pyx_n_u_grey19;
static PyObject *__pyx_n_u_grey2;
static PyObject *__pyx_n_u_grey20;
static PyObject *__pyx_n_u_grey21;
static PyObject *__pyx_n_u_grey22;
static PyObject *__pyx_n_u_grey23;
static PyObject *__pyx_n_u_grey24;
static PyObject *__pyx_n_u_grey25;
static PyObject *__pyx_n_u_grey26;
static PyObject *__pyx_n_u_grey27;
static PyObject *__pyx_n_u_grey28;
static PyObject *__pyx_n_u_grey29;
static PyObject *__pyx_n_u_grey3;
static PyObject *__pyx_n_u_grey30;
static PyObject *__pyx_n_u_grey31;
static PyObject *__pyx_n_u_grey32;
static PyObject *__pyx_n_u_grey33;
static PyObject *__pyx_n_u_grey34;
static PyObject *__pyx_n_u_grey35;
static PyObject *__pyx_n_u_grey36;
static PyObject *__pyx_n_u_grey37;
static PyObject *__pyx_n_u_grey38;
static PyObject *__pyx_n_u_grey39;
static PyObject *__pyx_n_u_grey4;
static PyObject *__pyx_n_u_grey40;
static PyObject *__pyx_n_u_grey41;
static PyObject *__pyx_n_u_grey42;
static PyObject *__pyx_n_u_grey43;
static PyObject *__pyx_n_u_grey44;
static PyObject *__pyx_n_u_grey45;
static PyObject *__pyx_n_u_grey46;
static PyObject *__pyx_n_u_grey47;
static PyObject *__pyx_n_u_grey48;
static PyObject *__pyx_n_u_grey49;
static PyObject *__pyx_n_u_grey5;
static PyObject *__pyx_n_u_grey50;
static PyObject *__pyx_n_u_grey51;
static PyObject *__pyx_n_u_grey52;
static PyObject *__pyx_n_u_grey53;
static PyObject *__pyx_n_u_grey54;
static PyObject *__pyx_n_u_grey55;
static PyObject *__pyx_n_u_grey56;
static PyObject *__pyx_n_u_grey57;
static PyObject *__pyx_n_u_grey58;
static PyObject *__pyx_n_u_grey59;
static PyObject *__pyx_n_u_grey6;
static PyObject *__pyx_n_u_grey60;
static PyObject *__pyx_n_u_grey61;
static PyObject *__pyx_n_u_grey62;
static PyObject *__pyx_n_u_grey63;
static PyObject *__pyx_n_u_grey64;
static PyObject *__pyx_n_u_grey65;
static PyObject *__pyx_n_u_grey66;
static PyObject *__pyx_n_u_grey67;
static PyObject *__pyx_n_u_grey68;
static PyObject *__pyx_n_u_grey69;
static PyObject *__pyx_n_u_grey7;
static PyObject *__pyx_n_u_grey70;
static PyObject *__pyx_n_u_grey71;
static PyObject *__pyx_n_u_grey72;
static PyObject *__pyx_n_u_grey73;
static PyObject *__pyx_n_u_grey74;
static PyObject *__pyx_n_u_grey75;
static PyObject *__pyx_n_u_grey76;
static PyObject *__pyx_n_u_grey77;
static PyObject *__pyx_n_u_grey78;
static PyObject *__pyx_n_u_grey79;
static PyObject *__pyx_n_u_grey8;
static PyObject *__pyx_n_u_grey80;
static PyObject *__pyx_n_u_grey81;
static PyObject *__pyx_n_u_grey82;
static PyObject *__pyx_n_u_grey83;
static PyObject *__pyx_n_u_grey84;
static PyObject *__pyx_n_u_grey85;
static PyObject *__pyx_n_u_grey86;
static PyObject *__pyx_n_u_grey87;
static PyObject *__pyx_n_u_grey88;
static PyObject *__pyx_n_u_grey89;
static PyObject *__pyx_n_u_grey9;
static PyObject *__pyx_n_u_grey90;
static PyObject *__pyx_n_u_grey91;
static PyObject *__pyx_n_u_grey92;
static PyObject *__pyx_n_u_grey93;
static PyObject *__pyx_n_u_grey94;
static PyObject *__pyx_n_u_grey95;
static PyObject *__pyx_n_u_grey96;
static PyObject *__pyx_n_u_grey97;
static PyObject *__pyx_n_u_grey98;
static PyObject *__pyx_n_u_grey99;
static PyObject *__pyx_n_s_hex;
static PyObject *__pyx_n_u_honeydew;
static PyObject *__pyx_n_u_honeydew1;
static PyObject *__pyx_n_u_honeydew2;
static PyObject *__pyx_n_u_honeydew3;
static PyObject *__pyx_n_u_honeydew4;
static PyObject *__pyx_n_u_hotpink;
static PyObject *__pyx_n_u_hotpink1;
static PyObject *__pyx_n_u_hotpink2;
static PyObject *__pyx_n_u_hotpink3;
static PyObject *__pyx_n_u_hotpink4;
static PyObject *__pyx_n_s_hsva;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_u_indianred;
static PyObject *__pyx_n_u_indianred1;
static PyObject *__pyx_n_u_indianred2;
static PyObject *__pyx_n_u_indianred3;
static PyObject *__pyx_n_u_indianred4;
static PyObject *__pyx_n_u_ivory;
static PyObject *__pyx_n_u_ivory1;
static PyObject *__pyx_n_u_ivory2;
static PyObject *__pyx_n_u_ivory3;
static PyObject *__pyx_n_u_ivory4;
static PyObject *__pyx_n_u_khaki;
static PyObject *__pyx_n_u_khaki1;
static PyObject *__pyx_n_u_khaki2;
static PyObject *__pyx_n_u_khaki3;
static PyObject *__pyx_n_u_khaki4;
static PyObject *__pyx_n_s_l;
static PyObject *__pyx_n_u_lavender;
static PyObject *__pyx_n_u_lavenderblush;
static PyObject *__pyx_n_u_lavenderblush1;
static PyObject *__pyx_n_u_lavenderblush2;
static PyObject *__pyx_n_u_lavenderblush3;
static PyObject *__pyx_n_u_lavenderblush4;
static PyObject *__pyx_n_u_lawngreen;
static PyObject *__pyx_n_u_lemonchiffon;
static PyObject *__pyx_n_u_lemonchiffon1;
static PyObject *__pyx_n_u_lemonchiffon2;
static PyObject *__pyx_n_u_lemonchiffon3;
static PyObject *__pyx_n_u_lemonchiffon4;
static PyObject *__pyx_n_u_lightblue;
static PyObject *__pyx_n_u_lightblue1;
static PyObject *__pyx_n_u_lightblue2;
static PyObject *__pyx_n_u_lightblue3;
static PyObject *__pyx_n_u_lightblue4;
static PyObject *__pyx_n_u_lightcoral;
static PyObject *__pyx_n_u_lightcyan;
static PyObject *__pyx_n_u_lightcyan1;
static PyObject *__pyx_n_u_lightcyan2;
static PyObject *__pyx_n_u_lightcyan3;
static PyObject *__pyx_n_u_lightcyan4;
static PyObject *__pyx_n_u_lightgoldenrod;
static PyObject *__pyx_n_u_lightgoldenrod1;
static PyObject *__pyx_n_u_lightgoldenrod2;
static PyObject *__pyx_n_u_lightgoldenrod3;
static PyObject *__pyx_n_u_lightgoldenrod4;
static PyObject *__pyx_n_u_lightgoldenrodyellow;
static PyObject *__pyx_n_u_lightgray;
static PyObject *__pyx_n_u_lightgreen;
static PyObject *__pyx_n_u_lightgrey;
static PyObject *__pyx_n_u_lightpink;
static PyObject *__pyx_n_u_lightpink1;
static PyObject *__pyx_n_u_lightpink2;
static PyObject *__pyx_n_u_lightpink3;
static PyObject *__pyx_n_u_lightpink4;
static PyObject *__pyx_n_u_lightsalmon;
static PyObject *__pyx_n_u_lightsalmon1;
static PyObject *__pyx_n_u_lightsalmon2;
static PyObject *__pyx_n_u_lightsalmon3;
static PyObject *__pyx_n_u_lightsalmon4;
static PyObject *__pyx_n_u_lightseagreen;
static PyObject *__pyx_n_u_lightskyblue;
static PyObject *__pyx_n_u_lightskyblue1;
static PyObject *__pyx_n_u_lightskyblue2;
static PyObject *__pyx_n_u_lightskyblue3;
static PyObject *__pyx_n_u_lightskyblue4;
static PyObject *__pyx_n_u_lightslateblue;
static PyObject *__pyx_n_u_lightslategray;
static PyObject *__pyx_n_u_lightslategrey;
static PyObject *__pyx_n_u_lightsteelblue;
static PyObject *__pyx_n_u_lightsteelblue1;
static PyObject *__pyx_n_u_lightsteelblue2;
static PyObject *__pyx_n_u_lightsteelblue3;
static PyObject *__pyx_n_u_lightsteelblue4;
static PyObject *__pyx_n_u_lightyellow;
static PyObject *__pyx_n_u_lightyellow1;
static PyObject *__pyx_n_u_lightyellow2;
static PyObject *__pyx_n_u_lightyellow3;
static PyObject *__pyx_n_u_lightyellow4;
static PyObject *__pyx_n_u_limegreen;
static PyObject *__pyx_n_u_linen;
static PyObject *__pyx_n_s_lower;
static PyObject *__pyx_n_u_magenta;
static PyObject *__pyx_n_u_magenta1;
static PyObject *__pyx_n_u_magenta2;
static PyObject *__pyx_n_u_magenta3;
static PyObject *__pyx_n_u_magenta4;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_map;
static PyObject *__pyx_n_u_maroon;
static PyObject *__pyx_n_u_maroon1;
static PyObject *__pyx_n_u_maroon2;
static PyObject *__pyx_n_u_maroon3;
static PyObject *__pyx_n_u_maroon4;
static PyObject *__pyx_n_u_mediumaquamarine;
static PyObject *__pyx_n_u_mediumblue;
static PyObject *__pyx_n_u_mediumorchid;
static PyObject *__pyx_n_u_mediumorchid1;
static PyObject *__pyx_n_u_mediumorchid2;
static PyObject *__pyx_n_u_mediumorchid3;
static PyObject *__pyx_n_u_mediumorchid4;
static PyObject *__pyx_n_u_mediumpurple;
static PyObject *__pyx_n_u_mediumpurple1;
static PyObject *__pyx_n_u_mediumpurple2;
static PyObject *__pyx_n_u_mediumpurple3;
static PyObject *__pyx_n_u_mediumpurple4;
static PyObject *__pyx_n_u_mediumseagreen;
static PyObject *__pyx_n_u_mediumslateblue;
static PyObject *__pyx_n_u_mediumspringgreen;
static PyObject *__pyx_n_u_mediumturquoise;
static PyObject *__pyx_n_u_mediumvioletred;
static PyObject *__pyx_n_u_midnightblue;
static PyObject *__pyx_n_u_mintcream;
static PyObject *__pyx_n_u_mistyrose;
static PyObject *__pyx_n_u_mistyrose1;
static PyObject *__pyx_n_u_mistyrose2;
static PyObject *__pyx_n_u_mistyrose3;
static PyObject *__pyx_n_u_mistyrose4;
static PyObject *__pyx_n_u_moccasin;
static PyObject *__pyx_n_s_mod;
static PyObject *__pyx_n_s_mod___locals_mod;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_u_navajowhite;
static PyObject *__pyx_n_u_navajowhite1;
static PyObject *__pyx_n_u_navajowhite2;
static PyObject *__pyx_n_u_navajowhite3;
static PyObject *__pyx_n_u_navajowhite4;
static PyObject *__pyx_n_u_navy;
static PyObject *__pyx_n_u_navyblue;
static PyObject *__pyx_n_s_oct;
static PyObject *__pyx_n_u_oldlace;
static PyObject *__pyx_n_u_olivedrab;
static PyObject *__pyx_n_u_olivedrab1;
static PyObject *__pyx_n_u_olivedrab2;
static PyObject *__pyx_n_u_olivedrab3;
static PyObject *__pyx_n_u_olivedrab4;
static PyObject *__pyx_n_u_orange;
static PyObject *__pyx_n_u_orange1;
static PyObject *__pyx_n_u_orange2;
static PyObject *__pyx_n_u_orange3;
static PyObject *__pyx_n_u_orange4;
static PyObject *__pyx_n_u_orangered;
static PyObject *__pyx_n_u_orangered1;
static PyObject *__pyx_n_u_orangered2;
static PyObject *__pyx_n_u_orangered3;
static PyObject *__pyx_n_u_orangered4;
static PyObject *__pyx_n_u_orchid;
static PyObject *__pyx_n_u_orchid1;
static PyObject *__pyx_n_u_orchid2;
static PyObject *__pyx_n_u_orchid3;
static PyObject *__pyx_n_u_orchid4;
static PyObject *__pyx_n_s_pack;
static PyObject *__pyx_n_u_palegoldenrod;
static PyObject *__pyx_n_u_palegreen;
static PyObject *__pyx_n_u_palegreen1;
static PyObject *__pyx_n_u_palegreen2;
static PyObject *__pyx_n_u_palegreen3;
static PyObject *__pyx_n_u_palegreen4;
static PyObject *__pyx_n_u_paleturquoise;
static PyObject *__pyx_n_u_paleturquoise1;
static PyObject *__pyx_n_u_paleturquoise2;
static PyObject *__pyx_n_u_paleturquoise3;
static PyObject *__pyx_n_u_paleturquoise4;
static PyObject *__pyx_n_u_palevioletred;
static PyObject *__pyx_n_u_palevioletred1;
static PyObject *__pyx_n_u_palevioletred2;
static PyObject *__pyx_n_u_palevioletred3;
static PyObject *__pyx_n_u_palevioletred4;
static PyObject *__pyx_n_u_papayawhip;
static PyObject *__pyx_n_u_peachpuff;
static PyObject *__pyx_n_u_peachpuff1;
static PyObject *__pyx_n_u_peachpuff2;
static PyObject *__pyx_n_u_peachpuff3;
static PyObject *__pyx_n_u_peachpuff4;
static PyObject *__pyx_n_u_peru;
static PyObject *__pyx_n_u_pink;
static PyObject *__pyx_n_u_pink1;
static PyObject *__pyx_n_u_pink2;
static PyObject *__pyx_n_u_pink3;
static PyObject *__pyx_n_u_pink4;
static PyObject *__pyx_n_u_plum;
static PyObject *__pyx_n_u_plum1;
static PyObject *__pyx_n_u_plum2;
static PyObject *__pyx_n_u_plum3;
static PyObject *__pyx_n_u_plum4;
static PyObject *__pyx_n_u_powderblue;
static PyObject *__pyx_n_u_purple;
static PyObject *__pyx_n_u_purple1;
static PyObject *__pyx_n_u_purple2;
static PyObject *__pyx_n_u_purple3;
static PyObject *__pyx_n_u_purple4;
static PyObject *__pyx_n_s_pygame_sdl2_color;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_r;
static PyObject *__pyx_n_u_red;
static PyObject *__pyx_n_u_red1;
static PyObject *__pyx_n_u_red2;
static PyObject *__pyx_n_u_red3;
static PyObject *__pyx_n_u_red4;
static PyObject *__pyx_n_u_rgba;
static PyObject *__pyx_n_u_rosybrown;
static PyObject *__pyx_n_u_rosybrown1;
static PyObject *__pyx_n_u_rosybrown2;
static PyObject *__pyx_n_u_rosybrown3;
static PyObject *__pyx_n_u_rosybrown4;
static PyObject *__pyx_n_s_round;
static PyObject *__pyx_n_u_royalblue;
static PyObject *__pyx_n_u_royalblue1;
static PyObject *__pyx_n_u_royalblue2;
static PyObject *__pyx_n_u_royalblue3;
static PyObject *__pyx_n_u_royalblue4;
static PyObject *__pyx_n_u_saddlebrown;
static PyObject *__pyx_n_u_salmon;
static PyObject *__pyx_n_u_salmon1;
static PyObject *__pyx_n_u_salmon2;
static PyObject *__pyx_n_u_salmon3;
static PyObject *__pyx_n_u_salmon4;
static PyObject *__pyx_n_u_sandybrown;
static PyObject *__pyx_n_u_seagreen;
static PyObject *__pyx_n_u_seagreen1;
static PyObject *__pyx_n_u_seagreen2;
static PyObject *__pyx_n_u_seagreen3;
static PyObject *__pyx_n_u_seagreen4;
static PyObject *__pyx_n_u_seashell;
static PyObject *__pyx_n_u_seashell1;
static PyObject *__pyx_n_u_seashell2;
static PyObject *__pyx_n_u_seashell3;
static PyObject *__pyx_n_u_seashell4;
static PyObject *__pyx_n_u_sienna;
static PyObject *__pyx_n_u_sienna1;
static PyObject *__pyx_n_u_sienna2;
static PyObject *__pyx_n_u_sienna3;
static PyObject *__pyx_n_u_sienna4;
static PyObject *__pyx_n_u_skyblue;
static PyObject *__pyx_n_u_skyblue1;
static PyObject *__pyx_n_u_skyblue2;
static PyObject *__pyx_n_u_skyblue3;
static PyObject *__pyx_n_u_skyblue4;
static PyObject *__pyx_n_u_slateblue;
static PyObject *__pyx_n_u_slateblue1;
static PyObject *__pyx_n_u_slateblue2;
static PyObject *__pyx_n_u_slateblue3;
static PyObject *__pyx_n_u_slateblue4;
static PyObject *__pyx_n_u_slategray;
static PyObject *__pyx_n_u_slategray1;
static PyObject *__pyx_n_u_slategray2;
static PyObject *__pyx_n_u_slategray3;
static PyObject *__pyx_n_u_slategray4;
static PyObject *__pyx_n_u_slategrey;
static PyObject *__pyx_n_u_snow;
static PyObject *__pyx_n_u_snow1;
static PyObject *__pyx_n_u_snow2;
static PyObject *__pyx_n_u_snow3;
static PyObject *__pyx_n_u_snow4;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_u_springgreen;
static PyObject *__pyx_n_u_springgreen1;
static PyObject *__pyx_n_u_springgreen2;
static PyObject *__pyx_n_u_springgreen3;
static PyObject *__pyx_n_u_springgreen4;
static PyObject *__pyx_kp_s_src_pygame_sdl2_color_pyx;
static PyObject *__pyx_n_s_startswith;
static PyObject *__pyx_n_u_steelblue;
static PyObject *__pyx_n_u_steelblue1;
static PyObject *__pyx_n_u_steelblue2;
static PyObject *__pyx_n_u_steelblue3;
static PyObject *__pyx_n_u_steelblue4;
static PyObject *__pyx_n_s_struct;
static PyObject *__pyx_n_u_tan;
static PyObject *__pyx_n_u_tan1;
static PyObject *__pyx_n_u_tan2;
static PyObject *__pyx_n_u_tan3;
static PyObject *__pyx_n_u_tan4;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_u_thistle;
static PyObject *__pyx_n_u_thistle1;
static PyObject *__pyx_n_u_thistle2;
static PyObject *__pyx_n_u_thistle3;
static PyObject *__pyx_n_u_thistle4;
static PyObject *__pyx_n_u_tomato;
static PyObject *__pyx_n_u_tomato1;
static PyObject *__pyx_n_u_tomato2;
static PyObject *__pyx_n_u_tomato3;
static PyObject *__pyx_n_u_tomato4;
static PyObject *__pyx_n_u_turquoise;
static PyObject *__pyx_n_u_turquoise1;
static PyObject *__pyx_n_u_turquoise2;
static PyObject *__pyx_n_u_turquoise3;
static PyObject *__pyx_n_u_turquoise4;
static PyObject *__pyx_n_s_unhexlify;
static PyObject *__pyx_n_s_unpack;
static PyObject *__pyx_n_u_violet;
static PyObject *__pyx_n_u_violetred;
static PyObject *__pyx_n_u_violetred1;
static PyObject *__pyx_n_u_violetred2;
static PyObject *__pyx_n_u_violetred3;
static PyObject *__pyx_n_u_violetred4;
static PyObject *__pyx_n_u_wheat;
static PyObject *__pyx_n_u_wheat1;
static PyObject *__pyx_n_u_wheat2;
static PyObject *__pyx_n_u_wheat3;
static PyObject *__pyx_n_u_wheat4;
static PyObject *__pyx_n_u_white;
static PyObject *__pyx_n_u_whitesmoke;
static PyObject *__pyx_n_u_yellow;
static PyObject *__pyx_n_u_yellow1;
static PyObject *__pyx_n_u_yellow2;
static PyObject *__pyx_n_u_yellow3;
static PyObject *__pyx_n_u_yellow4;
static PyObject *__pyx_n_u_yellowgreen;
static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color___richcmp__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_x, PyObject *__pyx_v_y, int __pyx_v_op); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_2__cinit__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_4__init__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_6__repr__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_8__int__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
#if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_10__hex__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
#endif
#if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_12__oct__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
#endif
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_14__float__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_16__reduce__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_18__setstate__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_d); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_20__setitem__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_val); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_22__getitem__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static Py_ssize_t __pyx_pf_11pygame_sdl2_5color_5Color_24__len__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_26__mul__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_28__add__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_30__sub__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_7__mod___mod(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_l, PyObject *__pyx_v_r); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_32__mod__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_34__div__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_12__floordiv___div(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_b); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_36__floordiv__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_3cmy___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_3cmy_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_val); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_4hsva___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_4hsva_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_val); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_4hsla___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_4hsla_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_val); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_6i1i2i3___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_6i1i2i3_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_val); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_38normalize(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda1(PyObject *__pyx_self, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_40correct_gamma(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_gamma); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_42set_length(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_n); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_1r___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_1r_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_1g___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_1g_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_1b___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_1b_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_1a___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self); /* proto */
static int __pyx_pf_11pygame_sdl2_5color_5Color_1a_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_tp_new_11pygame_sdl2_5color_Color(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_255_0;
static PyObject *__pyx_float_360_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_5;
static PyObject *__pyx_int_7;
static PyObject *__pyx_int_8;
static PyObject *__pyx_int_10;
static PyObject *__pyx_int_11;
static PyObject *__pyx_int_12;
static PyObject *__pyx_int_13;
static PyObject *__pyx_int_14;
static PyObject *__pyx_int_15;
static PyObject *__pyx_int_16;
static PyObject *__pyx_int_18;
static PyObject *__pyx_int_19;
static PyObject *__pyx_int_20;
static PyObject *__pyx_int_21;
static PyObject *__pyx_int_23;
static PyObject *__pyx_int_24;
static PyObject *__pyx_int_25;
static PyObject *__pyx_int_26;
static PyObject *__pyx_int_28;
static PyObject *__pyx_int_29;
static PyObject *__pyx_int_30;
static PyObject *__pyx_int_31;
static PyObject *__pyx_int_32;
static PyObject *__pyx_int_33;
static PyObject *__pyx_int_34;
static PyObject *__pyx_int_35;
static PyObject *__pyx_int_36;
static PyObject *__pyx_int_37;
static PyObject *__pyx_int_38;
static PyObject *__pyx_int_39;
static PyObject *__pyx_int_41;
static PyObject *__pyx_int_42;
static PyObject *__pyx_int_43;
static PyObject *__pyx_int_44;
static PyObject *__pyx_int_45;
static PyObject *__pyx_int_46;
static PyObject *__pyx_int_47;
static PyObject *__pyx_int_48;
static PyObject *__pyx_int_50;
static PyObject *__pyx_int_51;
static PyObject *__pyx_int_52;
static PyObject *__pyx_int_54;
static PyObject *__pyx_int_55;
static PyObject *__pyx_int_56;
static PyObject *__pyx_int_57;
static PyObject *__pyx_int_58;
static PyObject *__pyx_int_59;
static PyObject *__pyx_int_60;
static PyObject *__pyx_int_61;
static PyObject *__pyx_int_62;
static PyObject *__pyx_int_63;
static PyObject *__pyx_int_64;
static PyObject *__pyx_int_65;
static PyObject *__pyx_int_66;
static PyObject *__pyx_int_67;
static PyObject *__pyx_int_69;
static PyObject *__pyx_int_70;
static PyObject *__pyx_int_71;
static PyObject *__pyx_int_72;
static PyObject *__pyx_int_73;
static PyObject *__pyx_int_74;
static PyObject *__pyx_int_76;
static PyObject *__pyx_int_77;
static PyObject *__pyx_int_78;
static PyObject *__pyx_int_79;
static PyObject *__pyx_int_80;
static PyObject *__pyx_int_81;
static PyObject *__pyx_int_82;
static PyObject *__pyx_int_83;
static PyObject *__pyx_int_84;
static PyObject *__pyx_int_85;
static PyObject *__pyx_int_86;
static PyObject *__pyx_int_87;
static PyObject *__pyx_int_89;
static PyObject *__pyx_int_90;
static PyObject *__pyx_int_91;
static PyObject *__pyx_int_92;
static PyObject *__pyx_int_93;
static PyObject *__pyx_int_94;
static PyObject *__pyx_int_95;
static PyObject *__pyx_int_96;
static PyObject *__pyx_int_97;
static PyObject *__pyx_int_98;
static PyObject *__pyx_int_99;
static PyObject *__pyx_int_100;
static PyObject *__pyx_int_101;
static PyObject *__pyx_int_102;
static PyObject *__pyx_int_103;
static PyObject *__pyx_int_104;
static PyObject *__pyx_int_105;
static PyObject *__pyx_int_106;
static PyObject *__pyx_int_107;
static PyObject *__pyx_int_108;
static PyObject *__pyx_int_110;
static PyObject *__pyx_int_111;
static PyObject *__pyx_int_112;
static PyObject *__pyx_int_113;
static PyObject *__pyx_int_114;
static PyObject *__pyx_int_115;
static PyObject *__pyx_int_116;
static PyObject *__pyx_int_117;
static PyObject *__pyx_int_118;
static PyObject *__pyx_int_119;
static PyObject *__pyx_int_120;
static PyObject *__pyx_int_121;
static PyObject *__pyx_int_122;
static PyObject *__pyx_int_123;
static PyObject *__pyx_int_124;
static PyObject *__pyx_int_125;
static PyObject *__pyx_int_126;
static PyObject *__pyx_int_127;
static PyObject *__pyx_int_128;
static PyObject *__pyx_int_129;
static PyObject *__pyx_int_130;
static PyObject *__pyx_int_131;
static PyObject *__pyx_int_132;
static PyObject *__pyx_int_133;
static PyObject *__pyx_int_134;
static PyObject *__pyx_int_135;
static PyObject *__pyx_int_136;
static PyObject *__pyx_int_137;
static PyObject *__pyx_int_138;
static PyObject *__pyx_int_139;
static PyObject *__pyx_int_140;
static PyObject *__pyx_int_141;
static PyObject *__pyx_int_142;
static PyObject *__pyx_int_143;
static PyObject *__pyx_int_144;
static PyObject *__pyx_int_145;
static PyObject *__pyx_int_147;
static PyObject *__pyx_int_148;
static PyObject *__pyx_int_149;
static PyObject *__pyx_int_150;
static PyObject *__pyx_int_151;
static PyObject *__pyx_int_152;
static PyObject *__pyx_int_153;
static PyObject *__pyx_int_154;
static PyObject *__pyx_int_155;
static PyObject *__pyx_int_156;
static PyObject *__pyx_int_158;
static PyObject *__pyx_int_159;
static PyObject *__pyx_int_160;
static PyObject *__pyx_int_161;
static PyObject *__pyx_int_162;
static PyObject *__pyx_int_163;
static PyObject *__pyx_int_164;
static PyObject *__pyx_int_165;
static PyObject *__pyx_int_166;
static PyObject *__pyx_int_167;
static PyObject *__pyx_int_168;
static PyObject *__pyx_int_169;
static PyObject *__pyx_int_170;
static PyObject *__pyx_int_171;
static PyObject *__pyx_int_172;
static PyObject *__pyx_int_173;
static PyObject *__pyx_int_174;
static PyObject *__pyx_int_175;
static PyObject *__pyx_int_176;
static PyObject *__pyx_int_177;
static PyObject *__pyx_int_178;
static PyObject *__pyx_int_179;
static PyObject *__pyx_int_180;
static PyObject *__pyx_int_181;
static PyObject *__pyx_int_182;
static PyObject *__pyx_int_183;
static PyObject *__pyx_int_184;
static PyObject *__pyx_int_185;
static PyObject *__pyx_int_186;
static PyObject *__pyx_int_187;
static PyObject *__pyx_int_188;
static PyObject *__pyx_int_189;
static PyObject *__pyx_int_190;
static PyObject *__pyx_int_191;
static PyObject *__pyx_int_192;
static PyObject *__pyx_int_193;
static PyObject *__pyx_int_194;
static PyObject *__pyx_int_196;
static PyObject *__pyx_int_197;
static PyObject *__pyx_int_198;
static PyObject *__pyx_int_199;
static PyObject *__pyx_int_200;
static PyObject *__pyx_int_201;
static PyObject *__pyx_int_202;
static PyObject *__pyx_int_203;
static PyObject *__pyx_int_204;
static PyObject *__pyx_int_205;
static PyObject *__pyx_int_206;
static PyObject *__pyx_int_207;
static PyObject *__pyx_int_208;
static PyObject *__pyx_int_209;
static PyObject *__pyx_int_210;
static PyObject *__pyx_int_211;
static PyObject *__pyx_int_212;
static PyObject *__pyx_int_213;
static PyObject *__pyx_int_214;
static PyObject *__pyx_int_215;
static PyObject *__pyx_int_216;
static PyObject *__pyx_int_217;
static PyObject *__pyx_int_218;
static PyObject *__pyx_int_219;
static PyObject *__pyx_int_220;
static PyObject *__pyx_int_221;
static PyObject *__pyx_int_222;
static PyObject *__pyx_int_223;
static PyObject *__pyx_int_224;
static PyObject *__pyx_int_225;
static PyObject *__pyx_int_226;
static PyObject *__pyx_int_227;
static PyObject *__pyx_int_228;
static PyObject *__pyx_int_229;
static PyObject *__pyx_int_230;
static PyObject *__pyx_int_231;
static PyObject *__pyx_int_232;
static PyObject *__pyx_int_233;
static PyObject *__pyx_int_235;
static PyObject *__pyx_int_236;
static PyObject *__pyx_int_237;
static PyObject *__pyx_int_238;
static PyObject *__pyx_int_239;
static PyObject *__pyx_int_240;
static PyObject *__pyx_int_242;
static PyObject *__pyx_int_244;
static PyObject *__pyx_int_245;
static PyObject *__pyx_int_246;
static PyObject *__pyx_int_247;
static PyObject *__pyx_int_248;
static PyObject *__pyx_int_250;
static PyObject *__pyx_int_251;
static PyObject *__pyx_int_252;
static PyObject *__pyx_int_253;
static PyObject *__pyx_int_255;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_slice__4;
static PyObject *__pyx_slice__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__53;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__58;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__61;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__63;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__65;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__67;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__69;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_tuple__71;
static PyObject *__pyx_tuple__72;
static PyObject *__pyx_tuple__73;
static PyObject *__pyx_tuple__74;
static PyObject *__pyx_tuple__75;
static PyObject *__pyx_tuple__76;
static PyObject *__pyx_tuple__77;
static PyObject *__pyx_tuple__78;
static PyObject *__pyx_tuple__79;
static PyObject *__pyx_tuple__80;
static PyObject *__pyx_tuple__81;
static PyObject *__pyx_tuple__82;
static PyObject *__pyx_tuple__83;
static PyObject *__pyx_tuple__84;
static PyObject *__pyx_tuple__85;
static PyObject *__pyx_tuple__86;
static PyObject *__pyx_tuple__87;
static PyObject *__pyx_tuple__88;
static PyObject *__pyx_tuple__89;
static PyObject *__pyx_tuple__90;
static PyObject *__pyx_tuple__91;
static PyObject *__pyx_tuple__92;
static PyObject *__pyx_tuple__93;
static PyObject *__pyx_tuple__94;
static PyObject *__pyx_tuple__95;
static PyObject *__pyx_tuple__96;
static PyObject *__pyx_tuple__97;
static PyObject *__pyx_tuple__98;
static PyObject *__pyx_tuple__99;
static PyObject *__pyx_codeobj__7;
static PyObject *__pyx_codeobj__9;
static PyObject *__pyx_tuple__100;
static PyObject *__pyx_tuple__101;
static PyObject *__pyx_tuple__102;
static PyObject *__pyx_tuple__103;
static PyObject *__pyx_tuple__104;
static PyObject *__pyx_tuple__105;
static PyObject *__pyx_tuple__106;
static PyObject *__pyx_tuple__107;
static PyObject *__pyx_tuple__108;
static PyObject *__pyx_tuple__109;
static PyObject *__pyx_tuple__110;
static PyObject *__pyx_tuple__111;
static PyObject *__pyx_tuple__112;
static PyObject *__pyx_tuple__113;
static PyObject *__pyx_tuple__114;
static PyObject *__pyx_tuple__115;
static PyObject *__pyx_tuple__116;
static PyObject *__pyx_tuple__117;
static PyObject *__pyx_tuple__118;
static PyObject *__pyx_tuple__119;
static PyObject *__pyx_tuple__120;
static PyObject *__pyx_tuple__121;
static PyObject *__pyx_tuple__122;
static PyObject *__pyx_tuple__123;
static PyObject *__pyx_tuple__124;
static PyObject *__pyx_tuple__125;
static PyObject *__pyx_tuple__126;
static PyObject *__pyx_tuple__127;
static PyObject *__pyx_tuple__128;
static PyObject *__pyx_tuple__129;
static PyObject *__pyx_tuple__130;
static PyObject *__pyx_tuple__131;
static PyObject *__pyx_tuple__132;
static PyObject *__pyx_tuple__133;
static PyObject *__pyx_tuple__134;
static PyObject *__pyx_tuple__135;
static PyObject *__pyx_tuple__136;
static PyObject *__pyx_tuple__137;
static PyObject *__pyx_tuple__138;
static PyObject *__pyx_tuple__139;
static PyObject *__pyx_tuple__140;
static PyObject *__pyx_tuple__141;
static PyObject *__pyx_tuple__142;
static PyObject *__pyx_tuple__143;
static PyObject *__pyx_tuple__144;
static PyObject *__pyx_tuple__145;
static PyObject *__pyx_tuple__146;
static PyObject *__pyx_tuple__147;
static PyObject *__pyx_tuple__148;
static PyObject *__pyx_tuple__149;
static PyObject *__pyx_tuple__150;
static PyObject *__pyx_tuple__151;
static PyObject *__pyx_tuple__152;
static PyObject *__pyx_tuple__153;
static PyObject *__pyx_tuple__154;
static PyObject *__pyx_tuple__155;
static PyObject *__pyx_tuple__156;
static PyObject *__pyx_tuple__157;
static PyObject *__pyx_tuple__158;
static PyObject *__pyx_tuple__159;
static PyObject *__pyx_tuple__160;
static PyObject *__pyx_tuple__161;
static PyObject *__pyx_tuple__162;
static PyObject *__pyx_tuple__163;
static PyObject *__pyx_tuple__164;
static PyObject *__pyx_tuple__165;
static PyObject *__pyx_tuple__166;
static PyObject *__pyx_tuple__167;
static PyObject *__pyx_tuple__168;
static PyObject *__pyx_tuple__169;
static PyObject *__pyx_tuple__170;
static PyObject *__pyx_tuple__171;
static PyObject *__pyx_tuple__172;
static PyObject *__pyx_tuple__173;
static PyObject *__pyx_tuple__174;
static PyObject *__pyx_tuple__175;
static PyObject *__pyx_tuple__176;
static PyObject *__pyx_tuple__177;
static PyObject *__pyx_tuple__178;
static PyObject *__pyx_tuple__179;
static PyObject *__pyx_tuple__180;
static PyObject *__pyx_tuple__181;
static PyObject *__pyx_tuple__182;
static PyObject *__pyx_tuple__183;
static PyObject *__pyx_tuple__184;
static PyObject *__pyx_tuple__185;
static PyObject *__pyx_tuple__186;
static PyObject *__pyx_tuple__187;
static PyObject *__pyx_tuple__188;
static PyObject *__pyx_tuple__189;
static PyObject *__pyx_tuple__190;
static PyObject *__pyx_tuple__191;
static PyObject *__pyx_tuple__192;
static PyObject *__pyx_tuple__193;
static PyObject *__pyx_tuple__194;
static PyObject *__pyx_tuple__195;
static PyObject *__pyx_tuple__196;
static PyObject *__pyx_tuple__197;
static PyObject *__pyx_tuple__198;
static PyObject *__pyx_tuple__199;
static PyObject *__pyx_tuple__200;
static PyObject *__pyx_tuple__201;
static PyObject *__pyx_tuple__202;
static PyObject *__pyx_tuple__203;
static PyObject *__pyx_tuple__204;
static PyObject *__pyx_tuple__205;
static PyObject *__pyx_tuple__206;
static PyObject *__pyx_tuple__207;
static PyObject *__pyx_tuple__208;
static PyObject *__pyx_tuple__209;
static PyObject *__pyx_tuple__210;
static PyObject *__pyx_tuple__211;
static PyObject *__pyx_tuple__212;
static PyObject *__pyx_tuple__213;
static PyObject *__pyx_tuple__214;
static PyObject *__pyx_tuple__215;
static PyObject *__pyx_tuple__216;
static PyObject *__pyx_tuple__217;
static PyObject *__pyx_tuple__218;
static PyObject *__pyx_tuple__219;
static PyObject *__pyx_tuple__220;
static PyObject *__pyx_tuple__221;
static PyObject *__pyx_tuple__222;
static PyObject *__pyx_tuple__223;
static PyObject *__pyx_tuple__224;
static PyObject *__pyx_tuple__225;
static PyObject *__pyx_tuple__226;
static PyObject *__pyx_tuple__227;
static PyObject *__pyx_tuple__228;
static PyObject *__pyx_tuple__229;
static PyObject *__pyx_tuple__230;
static PyObject *__pyx_tuple__231;
static PyObject *__pyx_tuple__232;
static PyObject *__pyx_tuple__233;
static PyObject *__pyx_tuple__234;
static PyObject *__pyx_tuple__235;
static PyObject *__pyx_tuple__236;
static PyObject *__pyx_tuple__237;
static PyObject *__pyx_tuple__238;
static PyObject *__pyx_tuple__239;
static PyObject *__pyx_tuple__240;
static PyObject *__pyx_tuple__241;
static PyObject *__pyx_tuple__242;
static PyObject *__pyx_tuple__243;
static PyObject *__pyx_tuple__244;
static PyObject *__pyx_tuple__245;
static PyObject *__pyx_tuple__246;
static PyObject *__pyx_tuple__247;
static PyObject *__pyx_tuple__248;
static PyObject *__pyx_tuple__249;
static PyObject *__pyx_tuple__250;
static PyObject *__pyx_tuple__251;
static PyObject *__pyx_tuple__252;
static PyObject *__pyx_tuple__253;
static PyObject *__pyx_tuple__254;
static PyObject *__pyx_tuple__255;
static PyObject *__pyx_tuple__256;
static PyObject *__pyx_tuple__257;
static PyObject *__pyx_tuple__258;
static PyObject *__pyx_tuple__259;
static PyObject *__pyx_tuple__260;
static PyObject *__pyx_tuple__261;
static PyObject *__pyx_tuple__262;
static PyObject *__pyx_tuple__263;
static PyObject *__pyx_tuple__264;
static PyObject *__pyx_tuple__265;
static PyObject *__pyx_tuple__266;
static PyObject *__pyx_tuple__267;
static PyObject *__pyx_tuple__268;
static PyObject *__pyx_tuple__269;
static PyObject *__pyx_tuple__270;
static PyObject *__pyx_tuple__271;
static PyObject *__pyx_tuple__272;
static PyObject *__pyx_tuple__273;
static PyObject *__pyx_tuple__274;
static PyObject *__pyx_tuple__275;
static PyObject *__pyx_tuple__276;
static PyObject *__pyx_tuple__277;
static PyObject *__pyx_tuple__278;
static PyObject *__pyx_tuple__279;
static PyObject *__pyx_tuple__280;
static PyObject *__pyx_tuple__281;
static PyObject *__pyx_tuple__282;
static PyObject *__pyx_tuple__283;
static PyObject *__pyx_tuple__284;
static PyObject *__pyx_tuple__285;
static PyObject *__pyx_tuple__286;
static PyObject *__pyx_tuple__287;
static PyObject *__pyx_tuple__288;
static PyObject *__pyx_tuple__289;
static PyObject *__pyx_tuple__290;
static PyObject *__pyx_tuple__291;
static PyObject *__pyx_tuple__292;
static PyObject *__pyx_tuple__293;
static PyObject *__pyx_tuple__294;
static PyObject *__pyx_tuple__295;
static PyObject *__pyx_tuple__296;
static PyObject *__pyx_tuple__297;
static PyObject *__pyx_tuple__298;
static PyObject *__pyx_tuple__299;
static PyObject *__pyx_tuple__300;
static PyObject *__pyx_tuple__301;
static PyObject *__pyx_tuple__302;
static PyObject *__pyx_tuple__303;
static PyObject *__pyx_tuple__304;
static PyObject *__pyx_tuple__305;
static PyObject *__pyx_tuple__306;
static PyObject *__pyx_tuple__307;
static PyObject *__pyx_tuple__308;
static PyObject *__pyx_tuple__309;
static PyObject *__pyx_tuple__310;
static PyObject *__pyx_tuple__311;
static PyObject *__pyx_tuple__312;
static PyObject *__pyx_tuple__313;
static PyObject *__pyx_tuple__314;
static PyObject *__pyx_tuple__315;
static PyObject *__pyx_tuple__316;
static PyObject *__pyx_tuple__317;
static PyObject *__pyx_tuple__318;
static PyObject *__pyx_tuple__319;
static PyObject *__pyx_tuple__320;
static PyObject *__pyx_tuple__321;
static PyObject *__pyx_tuple__322;
static PyObject *__pyx_tuple__323;
static PyObject *__pyx_tuple__324;
static PyObject *__pyx_tuple__325;
static PyObject *__pyx_tuple__326;
static PyObject *__pyx_tuple__327;
static PyObject *__pyx_tuple__328;
static PyObject *__pyx_tuple__329;
static PyObject *__pyx_tuple__330;
static PyObject *__pyx_tuple__331;
static PyObject *__pyx_tuple__332;
static PyObject *__pyx_tuple__333;
static PyObject *__pyx_tuple__334;
static PyObject *__pyx_tuple__335;
static PyObject *__pyx_tuple__336;
static PyObject *__pyx_tuple__337;
static PyObject *__pyx_tuple__338;
static PyObject *__pyx_tuple__339;
static PyObject *__pyx_tuple__340;
static PyObject *__pyx_tuple__341;
static PyObject *__pyx_tuple__342;
static PyObject *__pyx_tuple__343;
static PyObject *__pyx_tuple__344;
static PyObject *__pyx_tuple__345;
static PyObject *__pyx_tuple__346;
static PyObject *__pyx_tuple__347;
static PyObject *__pyx_tuple__348;
static PyObject *__pyx_tuple__349;
static PyObject *__pyx_tuple__350;
static PyObject *__pyx_tuple__351;
static PyObject *__pyx_tuple__352;
static PyObject *__pyx_tuple__353;
static PyObject *__pyx_tuple__354;
static PyObject *__pyx_tuple__355;
static PyObject *__pyx_tuple__356;
static PyObject *__pyx_tuple__357;
static PyObject *__pyx_tuple__358;
static PyObject *__pyx_tuple__359;
static PyObject *__pyx_tuple__360;
static PyObject *__pyx_tuple__361;
static PyObject *__pyx_tuple__362;
static PyObject *__pyx_tuple__363;
static PyObject *__pyx_tuple__364;
static PyObject *__pyx_tuple__365;
static PyObject *__pyx_tuple__366;
static PyObject *__pyx_tuple__367;
static PyObject *__pyx_tuple__368;
static PyObject *__pyx_tuple__369;
static PyObject *__pyx_tuple__370;
static PyObject *__pyx_tuple__371;
static PyObject *__pyx_tuple__372;
static PyObject *__pyx_tuple__373;
static PyObject *__pyx_tuple__374;
static PyObject *__pyx_tuple__375;
static PyObject *__pyx_tuple__376;
static PyObject *__pyx_tuple__377;
static PyObject *__pyx_tuple__378;
static PyObject *__pyx_tuple__379;
static PyObject *__pyx_tuple__380;
static PyObject *__pyx_tuple__381;
static PyObject *__pyx_tuple__382;
static PyObject *__pyx_tuple__383;
static PyObject *__pyx_tuple__384;
static PyObject *__pyx_tuple__385;
static PyObject *__pyx_tuple__386;
static PyObject *__pyx_tuple__387;
static PyObject *__pyx_tuple__388;
static PyObject *__pyx_tuple__389;
static PyObject *__pyx_tuple__390;
static PyObject *__pyx_tuple__391;
static PyObject *__pyx_tuple__392;
static PyObject *__pyx_tuple__393;
static PyObject *__pyx_tuple__394;
static PyObject *__pyx_tuple__395;
static PyObject *__pyx_tuple__396;
static PyObject *__pyx_tuple__397;
static PyObject *__pyx_tuple__398;
static PyObject *__pyx_tuple__399;
static PyObject *__pyx_tuple__400;
static PyObject *__pyx_tuple__401;
static PyObject *__pyx_tuple__402;
static PyObject *__pyx_tuple__403;
static PyObject *__pyx_tuple__404;
static PyObject *__pyx_tuple__405;
static PyObject *__pyx_tuple__406;
static PyObject *__pyx_tuple__407;
static PyObject *__pyx_tuple__408;
static PyObject *__pyx_tuple__409;
static PyObject *__pyx_tuple__410;
static PyObject *__pyx_tuple__411;
static PyObject *__pyx_tuple__412;
static PyObject *__pyx_tuple__413;
static PyObject *__pyx_tuple__414;
static PyObject *__pyx_tuple__415;
static PyObject *__pyx_tuple__416;
static PyObject *__pyx_tuple__417;
static PyObject *__pyx_tuple__418;
static PyObject *__pyx_tuple__419;
static PyObject *__pyx_tuple__420;
static PyObject *__pyx_tuple__421;
static PyObject *__pyx_tuple__422;
static PyObject *__pyx_tuple__423;
static PyObject *__pyx_tuple__424;
static PyObject *__pyx_tuple__425;
static PyObject *__pyx_tuple__426;
static PyObject *__pyx_tuple__427;
static PyObject *__pyx_tuple__428;
static PyObject *__pyx_tuple__429;
static PyObject *__pyx_tuple__430;
static PyObject *__pyx_tuple__431;
static PyObject *__pyx_tuple__432;
static PyObject *__pyx_tuple__433;
static PyObject *__pyx_tuple__434;
static PyObject *__pyx_tuple__435;
static PyObject *__pyx_tuple__436;
static PyObject *__pyx_tuple__437;
static PyObject *__pyx_tuple__438;
static PyObject *__pyx_tuple__439;
static PyObject *__pyx_tuple__440;
static PyObject *__pyx_tuple__441;
static PyObject *__pyx_tuple__442;
static PyObject *__pyx_tuple__443;
static PyObject *__pyx_tuple__444;
static PyObject *__pyx_tuple__445;
static PyObject *__pyx_tuple__446;
static PyObject *__pyx_tuple__447;
static PyObject *__pyx_tuple__448;
static PyObject *__pyx_tuple__449;
static PyObject *__pyx_tuple__450;
static PyObject *__pyx_tuple__451;
static PyObject *__pyx_tuple__452;
static PyObject *__pyx_tuple__453;
static PyObject *__pyx_tuple__454;
static PyObject *__pyx_tuple__455;
static PyObject *__pyx_tuple__456;
static PyObject *__pyx_tuple__457;
static PyObject *__pyx_tuple__458;
static PyObject *__pyx_tuple__459;
static PyObject *__pyx_tuple__460;
static PyObject *__pyx_tuple__461;
static PyObject *__pyx_tuple__462;
static PyObject *__pyx_tuple__463;
static PyObject *__pyx_tuple__464;
static PyObject *__pyx_tuple__465;
static PyObject *__pyx_tuple__466;
static PyObject *__pyx_tuple__467;
static PyObject *__pyx_tuple__468;
static PyObject *__pyx_tuple__469;
static PyObject *__pyx_tuple__470;
static PyObject *__pyx_tuple__471;
static PyObject *__pyx_tuple__472;
static PyObject *__pyx_tuple__473;
static PyObject *__pyx_tuple__474;
static PyObject *__pyx_tuple__475;
static PyObject *__pyx_tuple__476;
static PyObject *__pyx_tuple__477;
static PyObject *__pyx_tuple__478;
static PyObject *__pyx_tuple__479;
static PyObject *__pyx_tuple__480;
static PyObject *__pyx_tuple__481;
static PyObject *__pyx_tuple__482;
static PyObject *__pyx_tuple__483;
static PyObject *__pyx_tuple__484;
static PyObject *__pyx_tuple__485;
static PyObject *__pyx_tuple__486;
static PyObject *__pyx_tuple__487;
static PyObject *__pyx_tuple__488;
static PyObject *__pyx_tuple__489;
static PyObject *__pyx_tuple__490;
static PyObject *__pyx_tuple__491;
static PyObject *__pyx_tuple__492;
static PyObject *__pyx_tuple__493;
static PyObject *__pyx_tuple__494;
static PyObject *__pyx_tuple__495;
static PyObject *__pyx_tuple__496;
static PyObject *__pyx_tuple__497;
static PyObject *__pyx_tuple__498;
static PyObject *__pyx_tuple__499;
static PyObject *__pyx_tuple__500;
static PyObject *__pyx_tuple__501;
static PyObject *__pyx_tuple__502;
static PyObject *__pyx_tuple__503;
static PyObject *__pyx_tuple__504;
static PyObject *__pyx_tuple__505;
static PyObject *__pyx_tuple__506;
static PyObject *__pyx_tuple__507;
static PyObject *__pyx_tuple__508;
static PyObject *__pyx_tuple__509;
static PyObject *__pyx_tuple__510;
static PyObject *__pyx_tuple__511;
static PyObject *__pyx_tuple__512;
/* Late includes */

/* "pygame_sdl2/color.pyx":26
 * include "color_dict.pxi"
 * 
 * cdef Uint32 map_color(SDL_Surface *surface, color) except? 0xaabbccdd:             # <<<<<<<<<<<<<<
 *     """
 *     Maps `color` into an RGBA color value that can be used with `surface`.
 */

static Uint32 __pyx_f_11pygame_sdl2_5color_map_color(SDL_Surface *__pyx_v_surface, PyObject *__pyx_v_color) {
  Uint8 __pyx_v_r;
  Uint8 __pyx_v_g;
  Uint8 __pyx_v_b;
  Uint8 __pyx_v_a;
  Uint32 __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *(*__pyx_t_11)(PyObject *);
  Uint8 __pyx_t_12;
  Uint8 __pyx_t_13;
  Uint8 __pyx_t_14;
  Uint8 __pyx_t_15;
  Uint32 __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("map_color", 0);

  /* "pygame_sdl2/color.pyx":33
 *     cdef Uint8 r, g, b, a
 * 
 *     if isinstance(color, (tuple, list, Color)) and len(color) == 4:             # <<<<<<<<<<<<<<
 *         r, g, b, a = color
 *     elif isinstance(color, (tuple, list, Color)) and len(color) == 3:
 */
  __pyx_t_3 = PyTuple_Check(__pyx_v_color); 
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (!__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_4 = PyList_Check(__pyx_v_color); 
  __pyx_t_3 = (__pyx_t_4 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_color, __pyx_ptype_11pygame_sdl2_5color_Color); 
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L6_bool_binop_done:;
  __pyx_t_4 = (__pyx_t_2 != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = PyObject_Length(__pyx_v_color); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 33, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_5 == 4) != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pygame_sdl2/color.pyx":34
 * 
 *     if isinstance(color, (tuple, list, Color)) and len(color) == 4:
 *         r, g, b, a = color             # <<<<<<<<<<<<<<
 *     elif isinstance(color, (tuple, list, Color)) and len(color) == 3:
 *         r, g, b = color
 */
    if ((likely(PyTuple_CheckExact(__pyx_v_color))) || (PyList_CheckExact(__pyx_v_color))) {
      PyObject* sequence = __pyx_v_color;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 4)) {
        if (size > 4) __Pyx_RaiseTooManyValuesError(4);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 34, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 2); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 3); 
      } else {
        __pyx_t_6 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_8 = PyList_GET_ITEM(sequence, 2); 
        __pyx_t_9 = PyList_GET_ITEM(sequence, 3); 
      }
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[4] = {&__pyx_t_6,&__pyx_t_7,&__pyx_t_8,&__pyx_t_9};
        for (i=0; i < 4; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 34, __pyx_L1_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
    } else {
      Py_ssize_t index = -1;
      PyObject** temps[4] = {&__pyx_t_6,&__pyx_t_7,&__pyx_t_8,&__pyx_t_9};
      __pyx_t_10 = PyObject_GetIter(__pyx_v_color); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 34, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_11 = Py_TYPE(__pyx_t_10)->tp_iternext;
      for (index=0; index < 4; index++) {
        PyObject* item = __pyx_t_11(__pyx_t_10); if (unlikely(!item)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(item);
        *(temps[index]) = item;
      }
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_10), 4) < 0) __PYX_ERR(0, 34, __pyx_L1_error)
      __pyx_t_11 = NULL;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L10_unpacking_done;
      __pyx_L9_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_11 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 34, __pyx_L1_error)
      __pyx_L10_unpacking_done:;
    }
    __pyx_t_12 = __Pyx_PyInt_As_uint8_t(__pyx_t_6); if (unlikely((__pyx_t_12 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_13 = __Pyx_PyInt_As_uint8_t(__pyx_t_7); if (unlikely((__pyx_t_13 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_14 = __Pyx_PyInt_As_uint8_t(__pyx_t_8); if (unlikely((__pyx_t_14 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_15 = __Pyx_PyInt_As_uint8_t(__pyx_t_9); if (unlikely((__pyx_t_15 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_v_r = __pyx_t_12;
    __pyx_v_g = __pyx_t_13;
    __pyx_v_b = __pyx_t_14;
    __pyx_v_a = __pyx_t_15;

    /* "pygame_sdl2/color.pyx":33
 *     cdef Uint8 r, g, b, a
 * 
 *     if isinstance(color, (tuple, list, Color)) and len(color) == 4:             # <<<<<<<<<<<<<<
 *         r, g, b, a = color
 *     elif isinstance(color, (tuple, list, Color)) and len(color) == 3:
 */
    goto __pyx_L3;
  }

  /* "pygame_sdl2/color.pyx":35
 *     if isinstance(color, (tuple, list, Color)) and len(color) == 4:
 *         r, g, b, a = color
 *     elif isinstance(color, (tuple, list, Color)) and len(color) == 3:             # <<<<<<<<<<<<<<
 *         r, g, b = color
 *         a = 255
 */
  __pyx_t_2 = PyTuple_Check(__pyx_v_color); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_4 = __pyx_t_3;
    goto __pyx_L13_bool_binop_done;
  }
  __pyx_t_3 = PyList_Check(__pyx_v_color); 
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_4 = __pyx_t_2;
    goto __pyx_L13_bool_binop_done;
  }
  __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_color, __pyx_ptype_11pygame_sdl2_5color_Color); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_4 = __pyx_t_3;
  __pyx_L13_bool_binop_done:;
  __pyx_t_3 = (__pyx_t_4 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L11_bool_binop_done;
  }
  __pyx_t_5 = PyObject_Length(__pyx_v_color); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 35, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_5 == 3) != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L11_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pygame_sdl2/color.pyx":36
 *         r, g, b, a = color
 *     elif isinstance(color, (tuple, list, Color)) and len(color) == 3:
 *         r, g, b = color             # <<<<<<<<<<<<<<
 *         a = 255
 *     elif isinstance(color, int):
 */
    if ((likely(PyTuple_CheckExact(__pyx_v_color))) || (PyList_CheckExact(__pyx_v_color))) {
      PyObject* sequence = __pyx_v_color;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 36, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_8 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_7 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_7);
      #else
      __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 36, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_8 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 36, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 36, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      #endif
    } else {
      Py_ssize_t index = -1;
      __pyx_t_6 = PyObject_GetIter(__pyx_v_color); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 36, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_11 = Py_TYPE(__pyx_t_6)->tp_iternext;
      index = 0; __pyx_t_9 = __pyx_t_11(__pyx_t_6); if (unlikely(!__pyx_t_9)) goto __pyx_L16_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      index = 1; __pyx_t_8 = __pyx_t_11(__pyx_t_6); if (unlikely(!__pyx_t_8)) goto __pyx_L16_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_8);
      index = 2; __pyx_t_7 = __pyx_t_11(__pyx_t_6); if (unlikely(!__pyx_t_7)) goto __pyx_L16_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_7);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_6), 3) < 0) __PYX_ERR(0, 36, __pyx_L1_error)
      __pyx_t_11 = NULL;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L17_unpacking_done;
      __pyx_L16_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_11 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 36, __pyx_L1_error)
      __pyx_L17_unpacking_done:;
    }
    __pyx_t_15 = __Pyx_PyInt_As_uint8_t(__pyx_t_9); if (unlikely((__pyx_t_15 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 36, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_14 = __Pyx_PyInt_As_uint8_t(__pyx_t_8); if (unlikely((__pyx_t_14 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 36, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_13 = __Pyx_PyInt_As_uint8_t(__pyx_t_7); if (unlikely((__pyx_t_13 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 36, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_r = __pyx_t_15;
    __pyx_v_g = __pyx_t_14;
    __pyx_v_b = __pyx_t_13;

    /* "pygame_sdl2/color.pyx":37
 *     elif isinstance(color, (tuple, list, Color)) and len(color) == 3:
 *         r, g, b = color
 *         a = 255             # <<<<<<<<<<<<<<
 *     elif isinstance(color, int):
 *         return color
 */
    __pyx_v_a = 0xFF;

    /* "pygame_sdl2/color.pyx":35
 *     if isinstance(color, (tuple, list, Color)) and len(color) == 4:
 *         r, g, b, a = color
 *     elif isinstance(color, (tuple, list, Color)) and len(color) == 3:             # <<<<<<<<<<<<<<
 *         r, g, b = color
 *         a = 255
 */
    goto __pyx_L3;
  }

  /* "pygame_sdl2/color.pyx":38
 *         r, g, b = color
 *         a = 255
 *     elif isinstance(color, int):             # <<<<<<<<<<<<<<
 *         return color
 *     else:
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_color); 
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (likely(__pyx_t_3)) {

    /* "pygame_sdl2/color.pyx":39
 *         a = 255
 *     elif isinstance(color, int):
 *         return color             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError("Expected a color.")
 */
    __pyx_t_16 = __Pyx_PyInt_As_uint32_t(__pyx_v_color); if (unlikely((__pyx_t_16 == ((Uint32)-1)) && PyErr_Occurred())) __PYX_ERR(0, 39, __pyx_L1_error)
    __pyx_r = __pyx_t_16;
    goto __pyx_L0;

    /* "pygame_sdl2/color.pyx":38
 *         r, g, b = color
 *         a = 255
 *     elif isinstance(color, int):             # <<<<<<<<<<<<<<
 *         return color
 *     else:
 */
  }

  /* "pygame_sdl2/color.pyx":41
 *         return color
 *     else:
 *         raise TypeError("Expected a color.")             # <<<<<<<<<<<<<<
 * 
 *     return SDL_MapRGBA(surface.format, r, g, b, a)
 */
  /*else*/ {
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 41, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __PYX_ERR(0, 41, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "pygame_sdl2/color.pyx":43
 *         raise TypeError("Expected a color.")
 * 
 *     return SDL_MapRGBA(surface.format, r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 * cdef object get_color(Uint32 pixel, SDL_Surface *surface):
 */
  __pyx_r = SDL_MapRGBA(__pyx_v_surface->format, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a);
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":26
 * include "color_dict.pxi"
 * 
 * cdef Uint32 map_color(SDL_Surface *surface, color) except? 0xaabbccdd:             # <<<<<<<<<<<<<<
 *     """
 *     Maps `color` into an RGBA color value that can be used with `surface`.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pygame_sdl2.color.map_color", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0xaabbccdd;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":45
 *     return SDL_MapRGBA(surface.format, r, g, b, a)
 * 
 * cdef object get_color(Uint32 pixel, SDL_Surface *surface):             # <<<<<<<<<<<<<<
 *     cdef Uint8 r
 *     cdef Uint8 g
 */

static PyObject *__pyx_f_11pygame_sdl2_5color_get_color(Uint32 __pyx_v_pixel, SDL_Surface *__pyx_v_surface) {
  Uint8 __pyx_v_r;
  Uint8 __pyx_v_g;
  Uint8 __pyx_v_b;
  Uint8 __pyx_v_a;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_color", 0);

  /* "pygame_sdl2/color.pyx":51
 *     cdef Uint8 a
 * 
 *     SDL_GetRGBA(pixel, surface.format, &r, &g, &b, &a)             # <<<<<<<<<<<<<<
 * 
 *     return Color(r, g, b, a)
 */
  SDL_GetRGBA(__pyx_v_pixel, __pyx_v_surface->format, (&__pyx_v_r), (&__pyx_v_g), (&__pyx_v_b), (&__pyx_v_a));

  /* "pygame_sdl2/color.pyx":53
 *     SDL_GetRGBA(pixel, surface.format, &r, &g, &b, &a)
 * 
 *     return Color(r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 * cdef to_sdl_color(color, SDL_Color *out):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint8_t(__pyx_v_r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_g); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_uint8_t(__pyx_v_b); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_uint8_t(__pyx_v_a); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_11pygame_sdl2_5color_Color), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":45
 *     return SDL_MapRGBA(surface.format, r, g, b, a)
 * 
 * cdef object get_color(Uint32 pixel, SDL_Surface *surface):             # <<<<<<<<<<<<<<
 *     cdef Uint8 r
 *     cdef Uint8 g
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pygame_sdl2.color.get_color", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":55
 *     return Color(r, g, b, a)
 * 
 * cdef to_sdl_color(color, SDL_Color *out):             # <<<<<<<<<<<<<<
 *     if not isinstance(color, Color):
 *         color = Color(color)
 */

static PyObject *__pyx_f_11pygame_sdl2_5color_to_sdl_color(PyObject *__pyx_v_color, SDL_Color *__pyx_v_out) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Uint8 __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("to_sdl_color", 0);
  __Pyx_INCREF(__pyx_v_color);

  /* "pygame_sdl2/color.pyx":56
 * 
 * cdef to_sdl_color(color, SDL_Color *out):
 *     if not isinstance(color, Color):             # <<<<<<<<<<<<<<
 *         color = Color(color)
 *     out.r = color.r
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_color, __pyx_ptype_11pygame_sdl2_5color_Color); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":57
 * cdef to_sdl_color(color, SDL_Color *out):
 *     if not isinstance(color, Color):
 *         color = Color(color)             # <<<<<<<<<<<<<<
 *     out.r = color.r
 *     out.g = color.g
 */
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_11pygame_sdl2_5color_Color), __pyx_v_color); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_color, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pygame_sdl2/color.pyx":56
 * 
 * cdef to_sdl_color(color, SDL_Color *out):
 *     if not isinstance(color, Color):             # <<<<<<<<<<<<<<
 *         color = Color(color)
 *     out.r = color.r
 */
  }

  /* "pygame_sdl2/color.pyx":58
 *     if not isinstance(color, Color):
 *         color = Color(color)
 *     out.r = color.r             # <<<<<<<<<<<<<<
 *     out.g = color.g
 *     out.b = color.b
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_color, __pyx_n_s_r); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_4 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_out->r = __pyx_t_4;

  /* "pygame_sdl2/color.pyx":59
 *         color = Color(color)
 *     out.r = color.r
 *     out.g = color.g             # <<<<<<<<<<<<<<
 *     out.b = color.b
 *     out.a = color.a
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_color, __pyx_n_s_g); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_4 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_out->g = __pyx_t_4;

  /* "pygame_sdl2/color.pyx":60
 *     out.r = color.r
 *     out.g = color.g
 *     out.b = color.b             # <<<<<<<<<<<<<<
 *     out.a = color.a
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_color, __pyx_n_s_b); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_4 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_out->b = __pyx_t_4;

  /* "pygame_sdl2/color.pyx":61
 *     out.g = color.g
 *     out.b = color.b
 *     out.a = color.a             # <<<<<<<<<<<<<<
 * 
 * cdef class Color:
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_color, __pyx_n_s_a); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_4 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_out->a = __pyx_t_4;

  /* "pygame_sdl2/color.pyx":55
 *     return Color(r, g, b, a)
 * 
 * cdef to_sdl_color(color, SDL_Color *out):             # <<<<<<<<<<<<<<
 *     if not isinstance(color, Color):
 *         color = Color(color)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pygame_sdl2.color.to_sdl_color", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_color);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":64
 * 
 * cdef class Color:
 *     cdef from_rgba(self, Uint8 r, Uint8 g, Uint8 b, Uint8 a):             # <<<<<<<<<<<<<<
 *         self.r = r
 *         self.g = g
 */

static PyObject *__pyx_f_11pygame_sdl2_5color_5Color_from_rgba(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, Uint8 __pyx_v_r, Uint8 __pyx_v_g, Uint8 __pyx_v_b, Uint8 __pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("from_rgba", 0);

  /* "pygame_sdl2/color.pyx":65
 * cdef class Color:
 *     cdef from_rgba(self, Uint8 r, Uint8 g, Uint8 b, Uint8 a):
 *         self.r = r             # <<<<<<<<<<<<<<
 *         self.g = g
 *         self.b = b
 */
  __pyx_v_self->r = __pyx_v_r;

  /* "pygame_sdl2/color.pyx":66
 *     cdef from_rgba(self, Uint8 r, Uint8 g, Uint8 b, Uint8 a):
 *         self.r = r
 *         self.g = g             # <<<<<<<<<<<<<<
 *         self.b = b
 *         self.a = a
 */
  __pyx_v_self->g = __pyx_v_g;

  /* "pygame_sdl2/color.pyx":67
 *         self.r = r
 *         self.g = g
 *         self.b = b             # <<<<<<<<<<<<<<
 *         self.a = a
 * 
 */
  __pyx_v_self->b = __pyx_v_b;

  /* "pygame_sdl2/color.pyx":68
 *         self.g = g
 *         self.b = b
 *         self.a = a             # <<<<<<<<<<<<<<
 * 
 *     cdef from_hex(self, c):
 */
  __pyx_v_self->a = __pyx_v_a;

  /* "pygame_sdl2/color.pyx":64
 * 
 * cdef class Color:
 *     cdef from_rgba(self, Uint8 r, Uint8 g, Uint8 b, Uint8 a):             # <<<<<<<<<<<<<<
 *         self.r = r
 *         self.g = g
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":73
 *         # Handle short hex strings.
 *         if len(c) == 3 or len(c) == 4:
 *             c = "".join(map(lambda x: x*2, c))             # <<<<<<<<<<<<<<
 * 
 *         try:
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_8from_hex_lambda(PyObject *__pyx_self, PyObject *__pyx_v_x); /*proto*/
static PyMethodDef __pyx_mdef_11pygame_sdl2_5color_5Color_8from_hex_lambda = {"lambda", (PyCFunction)__pyx_pw_11pygame_sdl2_5color_5Color_8from_hex_lambda, METH_O, 0};
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_8from_hex_lambda(PyObject *__pyx_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda (wrapper)", 0);
  __pyx_r = __pyx_lambda_funcdef_lambda(__pyx_self, ((PyObject *)__pyx_v_x));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyNumber_Multiply(__pyx_v_x, __pyx_int_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.from_hex.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":70
 *         self.a = a
 * 
 *     cdef from_hex(self, c):             # <<<<<<<<<<<<<<
 *         # Handle short hex strings.
 *         if len(c) == 3 or len(c) == 4:
 */

static PyObject *__pyx_f_11pygame_sdl2_5color_5Color_from_hex(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_c) {
  PyObject *__pyx_v_r = NULL;
  PyObject *__pyx_v_g = NULL;
  PyObject *__pyx_v_b = NULL;
  PyObject *__pyx_v_a = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *(*__pyx_t_13)(PyObject *);
  PyObject *__pyx_t_14 = NULL;
  int __pyx_t_15;
  char const *__pyx_t_16;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  Uint8 __pyx_t_23;
  Uint8 __pyx_t_24;
  Uint8 __pyx_t_25;
  Uint8 __pyx_t_26;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_hex", 0);
  __Pyx_INCREF(__pyx_v_c);

  /* "pygame_sdl2/color.pyx":72
 *     cdef from_hex(self, c):
 *         # Handle short hex strings.
 *         if len(c) == 3 or len(c) == 4:             # <<<<<<<<<<<<<<
 *             c = "".join(map(lambda x: x*2, c))
 * 
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_c); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 72, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_2 == 3) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyObject_Length(__pyx_v_c); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 72, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_2 == 4) != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pygame_sdl2/color.pyx":73
 *         # Handle short hex strings.
 *         if len(c) == 3 or len(c) == 4:
 *             c = "".join(map(lambda x: x*2, c))             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
    __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_11pygame_sdl2_5color_5Color_8from_hex_lambda, 0, __pyx_n_s_Color_from_hex_locals_lambda, NULL, __pyx_n_s_pygame_sdl2_color, __pyx_d, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_c);
    __Pyx_GIVEREF(__pyx_v_c);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_c);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_map, __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyUnicode_Join(__pyx_kp_u__2, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 73, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_c, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "pygame_sdl2/color.pyx":72
 *     cdef from_hex(self, c):
 *         # Handle short hex strings.
 *         if len(c) == 3 or len(c) == 4:             # <<<<<<<<<<<<<<
 *             c = "".join(map(lambda x: x*2, c))
 * 
 */
  }

  /* "pygame_sdl2/color.pyx":75
 *             c = "".join(map(lambda x: x*2, c))
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if len(c) == 6:
 *                 r, g, b = struct.unpack('BBB', binascii.unhexlify(c))
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
    __Pyx_XGOTREF(__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_8);
    /*try:*/ {

      /* "pygame_sdl2/color.pyx":76
 * 
 *         try:
 *             if len(c) == 6:             # <<<<<<<<<<<<<<
 *                 r, g, b = struct.unpack('BBB', binascii.unhexlify(c))
 *                 a = 255
 */
      __pyx_t_2 = PyObject_Length(__pyx_v_c); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 76, __pyx_L6_error)
      __pyx_t_1 = ((__pyx_t_2 == 6) != 0);
      if (__pyx_t_1) {

        /* "pygame_sdl2/color.pyx":77
 *         try:
 *             if len(c) == 6:
 *                 r, g, b = struct.unpack('BBB', binascii.unhexlify(c))             # <<<<<<<<<<<<<<
 *                 a = 255
 *             elif len(c) == 8:
 */
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_struct); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 77, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_unpack); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 77, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_binascii); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 77, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_unhexlify); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 77, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_11);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_11, function);
          }
        }
        __pyx_t_4 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_10, __pyx_v_c) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_v_c);
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 77, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_12 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_9);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_9, function);
            __pyx_t_12 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_9)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_n_u_BBB, __pyx_t_4};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 77, __pyx_L6_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_n_u_BBB, __pyx_t_4};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 77, __pyx_L6_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 77, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_10);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_n_u_BBB);
          __Pyx_GIVEREF(__pyx_n_u_BBB);
          PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_12, __pyx_n_u_BBB);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_12, __pyx_t_4);
          __pyx_t_4 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_10, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 77, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
          PyObject* sequence = __pyx_t_5;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 3)) {
            if (size > 3) __Pyx_RaiseTooManyValuesError(3);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 77, __pyx_L6_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
            __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
          } else {
            __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
            __pyx_t_4 = PyList_GET_ITEM(sequence, 2); 
          }
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_4);
          #else
          __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 77, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 77, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_4 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 77, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_11 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 77, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_13 = Py_TYPE(__pyx_t_11)->tp_iternext;
          index = 0; __pyx_t_9 = __pyx_t_13(__pyx_t_11); if (unlikely(!__pyx_t_9)) goto __pyx_L13_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_9);
          index = 1; __pyx_t_10 = __pyx_t_13(__pyx_t_11); if (unlikely(!__pyx_t_10)) goto __pyx_L13_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_10);
          index = 2; __pyx_t_4 = __pyx_t_13(__pyx_t_11); if (unlikely(!__pyx_t_4)) goto __pyx_L13_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_4);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_11), 3) < 0) __PYX_ERR(0, 77, __pyx_L6_error)
          __pyx_t_13 = NULL;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          goto __pyx_L14_unpacking_done;
          __pyx_L13_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __pyx_t_13 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 77, __pyx_L6_error)
          __pyx_L14_unpacking_done:;
        }
        __pyx_v_r = __pyx_t_9;
        __pyx_t_9 = 0;
        __pyx_v_g = __pyx_t_10;
        __pyx_t_10 = 0;
        __pyx_v_b = __pyx_t_4;
        __pyx_t_4 = 0;

        /* "pygame_sdl2/color.pyx":78
 *             if len(c) == 6:
 *                 r, g, b = struct.unpack('BBB', binascii.unhexlify(c))
 *                 a = 255             # <<<<<<<<<<<<<<
 *             elif len(c) == 8:
 *                 r, g, b, a = struct.unpack('BBBB', binascii.unhexlify(c))
 */
        __Pyx_INCREF(__pyx_int_255);
        __pyx_v_a = __pyx_int_255;

        /* "pygame_sdl2/color.pyx":76
 * 
 *         try:
 *             if len(c) == 6:             # <<<<<<<<<<<<<<
 *                 r, g, b = struct.unpack('BBB', binascii.unhexlify(c))
 *                 a = 255
 */
        goto __pyx_L12;
      }

      /* "pygame_sdl2/color.pyx":79
 *                 r, g, b = struct.unpack('BBB', binascii.unhexlify(c))
 *                 a = 255
 *             elif len(c) == 8:             # <<<<<<<<<<<<<<
 *                 r, g, b, a = struct.unpack('BBBB', binascii.unhexlify(c))
 *             else:
 */
      __pyx_t_2 = PyObject_Length(__pyx_v_c); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 79, __pyx_L6_error)
      __pyx_t_1 = ((__pyx_t_2 == 8) != 0);
      if (likely(__pyx_t_1)) {

        /* "pygame_sdl2/color.pyx":80
 *                 a = 255
 *             elif len(c) == 8:
 *                 r, g, b, a = struct.unpack('BBBB', binascii.unhexlify(c))             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(c)
 */
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_struct); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 80, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_unpack); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 80, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_binascii); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 80, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_unhexlify); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 80, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_9 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_11);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_11, function);
          }
        }
        __pyx_t_4 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_9, __pyx_v_c) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_v_c);
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 80, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_12 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
            __pyx_t_12 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_n_u_BBBB, __pyx_t_4};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 80, __pyx_L6_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_n_u_BBBB, __pyx_t_4};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_12, 2+__pyx_t_12); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 80, __pyx_L6_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_9 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 80, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_9);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_n_u_BBBB);
          __Pyx_GIVEREF(__pyx_n_u_BBBB);
          PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_12, __pyx_n_u_BBBB);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_12, __pyx_t_4);
          __pyx_t_4 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 80, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
          PyObject* sequence = __pyx_t_5;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 4)) {
            if (size > 4) __Pyx_RaiseTooManyValuesError(4);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 80, __pyx_L6_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
            __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
            __pyx_t_11 = PyTuple_GET_ITEM(sequence, 3); 
          } else {
            __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
            __pyx_t_4 = PyList_GET_ITEM(sequence, 2); 
            __pyx_t_11 = PyList_GET_ITEM(sequence, 3); 
          }
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_11);
          #else
          {
            Py_ssize_t i;
            PyObject** temps[4] = {&__pyx_t_10,&__pyx_t_9,&__pyx_t_4,&__pyx_t_11};
            for (i=0; i < 4; i++) {
              PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 80, __pyx_L6_error)
              __Pyx_GOTREF(item);
              *(temps[i]) = item;
            }
          }
          #endif
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else {
          Py_ssize_t index = -1;
          PyObject** temps[4] = {&__pyx_t_10,&__pyx_t_9,&__pyx_t_4,&__pyx_t_11};
          __pyx_t_14 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 80, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_13 = Py_TYPE(__pyx_t_14)->tp_iternext;
          for (index=0; index < 4; index++) {
            PyObject* item = __pyx_t_13(__pyx_t_14); if (unlikely(!item)) goto __pyx_L15_unpacking_failed;
            __Pyx_GOTREF(item);
            *(temps[index]) = item;
          }
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_14), 4) < 0) __PYX_ERR(0, 80, __pyx_L6_error)
          __pyx_t_13 = NULL;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          goto __pyx_L16_unpacking_done;
          __pyx_L15_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __pyx_t_13 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 80, __pyx_L6_error)
          __pyx_L16_unpacking_done:;
        }
        __pyx_v_r = __pyx_t_10;
        __pyx_t_10 = 0;
        __pyx_v_g = __pyx_t_9;
        __pyx_t_9 = 0;
        __pyx_v_b = __pyx_t_4;
        __pyx_t_4 = 0;
        __pyx_v_a = __pyx_t_11;
        __pyx_t_11 = 0;

        /* "pygame_sdl2/color.pyx":79
 *                 r, g, b = struct.unpack('BBB', binascii.unhexlify(c))
 *                 a = 255
 *             elif len(c) == 8:             # <<<<<<<<<<<<<<
 *                 r, g, b, a = struct.unpack('BBBB', binascii.unhexlify(c))
 *             else:
 */
        goto __pyx_L12;
      }

      /* "pygame_sdl2/color.pyx":82
 *                 r, g, b, a = struct.unpack('BBBB', binascii.unhexlify(c))
 *             else:
 *                 raise ValueError(c)             # <<<<<<<<<<<<<<
 *         except TypeError as e:
 *             raise ValueError(c)
 */
      /*else*/ {
        __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_c); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 82, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_Raise(__pyx_t_5, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __PYX_ERR(0, 82, __pyx_L6_error)
      }
      __pyx_L12:;

      /* "pygame_sdl2/color.pyx":75
 *             c = "".join(map(lambda x: x*2, c))
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if len(c) == 6:
 *                 r, g, b = struct.unpack('BBB', binascii.unhexlify(c))
 */
    }
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    goto __pyx_L11_try_end;
    __pyx_L6_error:;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "pygame_sdl2/color.pyx":83
 *             else:
 *                 raise ValueError(c)
 *         except TypeError as e:             # <<<<<<<<<<<<<<
 *             raise ValueError(c)
 * 
 */
    __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_12) {
      __Pyx_AddTraceback("pygame_sdl2.color.Color.from_hex", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_11, &__pyx_t_4) < 0) __PYX_ERR(0, 83, __pyx_L8_except_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_11);
      __pyx_v_e = __pyx_t_11;
      /*try:*/ {

        /* "pygame_sdl2/color.pyx":84
 *                 raise ValueError(c)
 *         except TypeError as e:
 *             raise ValueError(c)             # <<<<<<<<<<<<<<
 * 
 *         self.from_rgba(r, g, b, a)
 */
        __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_c); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 84, __pyx_L22_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_Raise(__pyx_t_9, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __PYX_ERR(0, 84, __pyx_L22_error)
      }

      /* "pygame_sdl2/color.pyx":83
 *             else:
 *                 raise ValueError(c)
 *         except TypeError as e:             # <<<<<<<<<<<<<<
 *             raise ValueError(c)
 * 
 */
      /*finally:*/ {
        __pyx_L22_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
          if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19) < 0)) __Pyx_ErrFetch(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_22);
          __pyx_t_12 = __pyx_lineno; __pyx_t_15 = __pyx_clineno; __pyx_t_16 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_e);
            __pyx_v_e = NULL;
          }
          if (PY_MAJOR_VERSION >= 3) {
            __Pyx_XGIVEREF(__pyx_t_20);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_22);
            __Pyx_ExceptionReset(__pyx_t_20, __pyx_t_21, __pyx_t_22);
          }
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_ErrRestore(__pyx_t_17, __pyx_t_18, __pyx_t_19);
          __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0;
          __pyx_lineno = __pyx_t_12; __pyx_clineno = __pyx_t_15; __pyx_filename = __pyx_t_16;
          goto __pyx_L8_except_error;
        }
      }
    }
    goto __pyx_L8_except_error;
    __pyx_L8_except_error:;

    /* "pygame_sdl2/color.pyx":75
 *             c = "".join(map(lambda x: x*2, c))
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if len(c) == 6:
 *                 r, g, b = struct.unpack('BBB', binascii.unhexlify(c))
 */
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
    goto __pyx_L1_error;
    __pyx_L11_try_end:;
  }

  /* "pygame_sdl2/color.pyx":86
 *             raise ValueError(c)
 * 
 *         self.from_rgba(r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 *     cdef from_name(self, c):
 */
  __pyx_t_23 = __Pyx_PyInt_As_uint8_t(__pyx_v_r); if (unlikely((__pyx_t_23 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 86, __pyx_L1_error)
  __pyx_t_24 = __Pyx_PyInt_As_uint8_t(__pyx_v_g); if (unlikely((__pyx_t_24 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 86, __pyx_L1_error)
  __pyx_t_25 = __Pyx_PyInt_As_uint8_t(__pyx_v_b); if (unlikely((__pyx_t_25 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 86, __pyx_L1_error)
  __pyx_t_26 = __Pyx_PyInt_As_uint8_t(__pyx_v_a); if (unlikely((__pyx_t_26 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 86, __pyx_L1_error)
  __pyx_t_4 = ((struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *)__pyx_v_self->__pyx_vtab)->from_rgba(__pyx_v_self, __pyx_t_23, __pyx_t_24, __pyx_t_25, __pyx_t_26); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pygame_sdl2/color.pyx":70
 *         self.a = a
 * 
 *     cdef from_hex(self, c):             # <<<<<<<<<<<<<<
 *         # Handle short hex strings.
 *         if len(c) == 3 or len(c) == 4:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.from_hex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_c);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":88
 *         self.from_rgba(r, g, b, a)
 * 
 *     cdef from_name(self, c):             # <<<<<<<<<<<<<<
 *         # Remove all whitespace.
 *         c = "".join(c.split()).lower()
 */

static PyObject *__pyx_f_11pygame_sdl2_5color_5Color_from_name(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_c) {
  PyObject *__pyx_v_r = NULL;
  PyObject *__pyx_v_g = NULL;
  PyObject *__pyx_v_b = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  int __pyx_t_10;
  int __pyx_t_11;
  char const *__pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  Uint8 __pyx_t_19;
  Uint8 __pyx_t_20;
  Uint8 __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_name", 0);
  __Pyx_INCREF(__pyx_v_c);

  /* "pygame_sdl2/color.pyx":90
 *     cdef from_name(self, c):
 *         # Remove all whitespace.
 *         c = "".join(c.split()).lower()             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_c, __pyx_n_s_split); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyUnicode_Join(__pyx_kp_u__2, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_c, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":92
 *         c = "".join(c.split()).lower()
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             r, g, b = colors[c]
 *         except KeyError as e:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_7);
    /*try:*/ {

      /* "pygame_sdl2/color.pyx":93
 * 
 *         try:
 *             r, g, b = colors[c]             # <<<<<<<<<<<<<<
 *         except KeyError as e:
 *             raise ValueError(c)
 */
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_11pygame_sdl2_5color_colors, __pyx_v_c); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 93, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_1);
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 93, __pyx_L3_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_4 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        #else
        __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 93, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 93, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 93, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 93, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
        index = 0; __pyx_t_2 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_2)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_2);
        index = 1; __pyx_t_3 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_3)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_3);
        index = 2; __pyx_t_4 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_4)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_4);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 3) < 0) __PYX_ERR(0, 93, __pyx_L3_error)
        __pyx_t_9 = NULL;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L10_unpacking_done;
        __pyx_L9_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_9 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 93, __pyx_L3_error)
        __pyx_L10_unpacking_done:;
      }
      __pyx_v_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __pyx_v_g = __pyx_t_3;
      __pyx_t_3 = 0;
      __pyx_v_b = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "pygame_sdl2/color.pyx":92
 *         c = "".join(c.split()).lower()
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             r, g, b = colors[c]
 *         except KeyError as e:
 */
    }
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "pygame_sdl2/color.pyx":94
 *         try:
 *             r, g, b = colors[c]
 *         except KeyError as e:             # <<<<<<<<<<<<<<
 *             raise ValueError(c)
 *         self.from_rgba(r, g, b, 255)
 */
    __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_10) {
      __Pyx_AddTraceback("pygame_sdl2.color.Color.from_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 94, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_v_e = __pyx_t_4;
      /*try:*/ {

        /* "pygame_sdl2/color.pyx":95
 *             r, g, b = colors[c]
 *         except KeyError as e:
 *             raise ValueError(c)             # <<<<<<<<<<<<<<
 *         self.from_rgba(r, g, b, 255)
 * 
 */
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_c); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L16_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_Raise(__pyx_t_2, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 95, __pyx_L16_error)
      }

      /* "pygame_sdl2/color.pyx":94
 *         try:
 *             r, g, b = colors[c]
 *         except KeyError as e:             # <<<<<<<<<<<<<<
 *             raise ValueError(c)
 *         self.from_rgba(r, g, b, 255)
 */
      /*finally:*/ {
        __pyx_L16_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
          if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15) < 0)) __Pyx_ErrFetch(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_13);
          __Pyx_XGOTREF(__pyx_t_14);
          __Pyx_XGOTREF(__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_18);
          __pyx_t_10 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_12 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_e);
            __pyx_v_e = NULL;
          }
          if (PY_MAJOR_VERSION >= 3) {
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
          }
          __Pyx_XGIVEREF(__pyx_t_13);
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_ErrRestore(__pyx_t_13, __pyx_t_14, __pyx_t_15);
          __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
          __pyx_lineno = __pyx_t_10; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_12;
          goto __pyx_L5_except_error;
        }
      }
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "pygame_sdl2/color.pyx":92
 *         c = "".join(c.split()).lower()
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             r, g, b = colors[c]
 *         except KeyError as e:
 */
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "pygame_sdl2/color.pyx":96
 *         except KeyError as e:
 *             raise ValueError(c)
 *         self.from_rgba(r, g, b, 255)             # <<<<<<<<<<<<<<
 * 
 *     def __richcmp__(Color x, y, int op):
 */
  __pyx_t_19 = __Pyx_PyInt_As_uint8_t(__pyx_v_r); if (unlikely((__pyx_t_19 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 96, __pyx_L1_error)
  __pyx_t_20 = __Pyx_PyInt_As_uint8_t(__pyx_v_g); if (unlikely((__pyx_t_20 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 96, __pyx_L1_error)
  __pyx_t_21 = __Pyx_PyInt_As_uint8_t(__pyx_v_b); if (unlikely((__pyx_t_21 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 96, __pyx_L1_error)
  __pyx_t_3 = ((struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *)__pyx_v_self->__pyx_vtab)->from_rgba(__pyx_v_self, __pyx_t_19, __pyx_t_20, __pyx_t_21, 0xFF); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":88
 *         self.from_rgba(r, g, b, a)
 * 
 *     cdef from_name(self, c):             # <<<<<<<<<<<<<<
 *         # Remove all whitespace.
 *         c = "".join(c.split()).lower()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.from_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_c);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":98
 *         self.from_rgba(r, g, b, 255)
 * 
 *     def __richcmp__(Color x, y, int op):             # <<<<<<<<<<<<<<
 *         if op == 3:
 *             return not (x == y)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_1__richcmp__(PyObject *__pyx_v_x, PyObject *__pyx_v_y, int __pyx_v_op); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_1__richcmp__(PyObject *__pyx_v_x, PyObject *__pyx_v_y, int __pyx_v_op) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color___richcmp__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_x), ((PyObject *)__pyx_v_y), ((int)__pyx_v_op));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color___richcmp__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_x, PyObject *__pyx_v_y, int __pyx_v_op) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__richcmp__", 0);
  __Pyx_INCREF(__pyx_v_y);

  /* "pygame_sdl2/color.pyx":99
 * 
 *     def __richcmp__(Color x, y, int op):
 *         if op == 3:             # <<<<<<<<<<<<<<
 *             return not (x == y)
 * 
 */
  __pyx_t_1 = ((__pyx_v_op == 3) != 0);
  if (__pyx_t_1) {

    /* "pygame_sdl2/color.pyx":100
 *     def __richcmp__(Color x, y, int op):
 *         if op == 3:
 *             return not (x == y)             # <<<<<<<<<<<<<<
 * 
 *         if isinstance(y, tuple):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyObject_RichCompare(((PyObject *)__pyx_v_x), __pyx_v_y, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 100, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 100, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyBool_FromLong((!__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 100, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "pygame_sdl2/color.pyx":99
 * 
 *     def __richcmp__(Color x, y, int op):
 *         if op == 3:             # <<<<<<<<<<<<<<
 *             return not (x == y)
 * 
 */
  }

  /* "pygame_sdl2/color.pyx":102
 *             return not (x == y)
 * 
 *         if isinstance(y, tuple):             # <<<<<<<<<<<<<<
 *             y = Color(y)
 *         if not isinstance(y, Color):
 */
  __pyx_t_1 = PyTuple_Check(__pyx_v_y); 
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "pygame_sdl2/color.pyx":103
 * 
 *         if isinstance(y, tuple):
 *             y = Color(y)             # <<<<<<<<<<<<<<
 *         if not isinstance(y, Color):
 *             return False
 */
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_11pygame_sdl2_5color_Color), __pyx_v_y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_y, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pygame_sdl2/color.pyx":102
 *             return not (x == y)
 * 
 *         if isinstance(y, tuple):             # <<<<<<<<<<<<<<
 *             y = Color(y)
 *         if not isinstance(y, Color):
 */
  }

  /* "pygame_sdl2/color.pyx":104
 *         if isinstance(y, tuple):
 *             y = Color(y)
 *         if not isinstance(y, Color):             # <<<<<<<<<<<<<<
 *             return False
 *         if op == 2:
 */
  __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_y, __pyx_ptype_11pygame_sdl2_5color_Color); 
  __pyx_t_1 = ((!(__pyx_t_3 != 0)) != 0);
  if (__pyx_t_1) {

    /* "pygame_sdl2/color.pyx":105
 *             y = Color(y)
 *         if not isinstance(y, Color):
 *             return False             # <<<<<<<<<<<<<<
 *         if op == 2:
 *             return x.r == y.r and x.g == y.g and x.b == y.b and x.a == y.a
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "pygame_sdl2/color.pyx":104
 *         if isinstance(y, tuple):
 *             y = Color(y)
 *         if not isinstance(y, Color):             # <<<<<<<<<<<<<<
 *             return False
 *         if op == 2:
 */
  }

  /* "pygame_sdl2/color.pyx":106
 *         if not isinstance(y, Color):
 *             return False
 *         if op == 2:             # <<<<<<<<<<<<<<
 *             return x.r == y.r and x.g == y.g and x.b == y.b and x.a == y.a
 * 
 */
  __pyx_t_1 = ((__pyx_v_op == 2) != 0);
  if (__pyx_t_1) {

    /* "pygame_sdl2/color.pyx":107
 *             return False
 *         if op == 2:
 *             return x.r == y.r and x.g == y.g and x.b == y.b and x.a == y.a             # <<<<<<<<<<<<<<
 * 
 *     def __cinit__(self):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyInt_From_uint8_t(__pyx_v_x->r); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_y, __pyx_n_s_r); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyObject_RichCompare(__pyx_t_4, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 107, __pyx_L1_error)
    if (__pyx_t_1) {
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_2 = __pyx_t_6;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_6 = __Pyx_PyInt_From_uint8_t(__pyx_v_x->g); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_y, __pyx_n_s_g); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_6, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 107, __pyx_L1_error)
    if (__pyx_t_1) {
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_2 = __pyx_t_4;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_4 = __Pyx_PyInt_From_uint8_t(__pyx_v_x->b); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_y, __pyx_n_s_b); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyObject_RichCompare(__pyx_t_4, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 107, __pyx_L1_error)
    if (__pyx_t_1) {
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_2 = __pyx_t_6;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_6 = __Pyx_PyInt_From_uint8_t(__pyx_v_x->a); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_y, __pyx_n_s_a); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_6, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_2 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_L7_bool_binop_done:;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "pygame_sdl2/color.pyx":106
 *         if not isinstance(y, Color):
 *             return False
 *         if op == 2:             # <<<<<<<<<<<<<<
 *             return x.r == y.r and x.g == y.g and x.b == y.b and x.a == y.a
 * 
 */
  }

  /* "pygame_sdl2/color.pyx":98
 *         self.from_rgba(r, g, b, 255)
 * 
 *     def __richcmp__(Color x, y, int op):             # <<<<<<<<<<<<<<
 *         if op == 3:
 *             return not (x == y)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_y);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":109
 *             return x.r == y.r and x.g == y.g and x.b == y.b and x.a == y.a
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.length = 4
 *         self.r = 0
 */

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_3__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_3__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_2__cinit__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_2__cinit__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "pygame_sdl2/color.pyx":110
 * 
 *     def __cinit__(self):
 *         self.length = 4             # <<<<<<<<<<<<<<
 *         self.r = 0
 *         self.g = 0
 */
  __pyx_v_self->length = 4;

  /* "pygame_sdl2/color.pyx":111
 *     def __cinit__(self):
 *         self.length = 4
 *         self.r = 0             # <<<<<<<<<<<<<<
 *         self.g = 0
 *         self.b = 0
 */
  __pyx_v_self->r = 0;

  /* "pygame_sdl2/color.pyx":112
 *         self.length = 4
 *         self.r = 0
 *         self.g = 0             # <<<<<<<<<<<<<<
 *         self.b = 0
 *         self.a = 255
 */
  __pyx_v_self->g = 0;

  /* "pygame_sdl2/color.pyx":113
 *         self.r = 0
 *         self.g = 0
 *         self.b = 0             # <<<<<<<<<<<<<<
 *         self.a = 255
 * 
 */
  __pyx_v_self->b = 0;

  /* "pygame_sdl2/color.pyx":114
 *         self.g = 0
 *         self.b = 0
 *         self.a = 255             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, *args):
 */
  __pyx_v_self->a = 0xFF;

  /* "pygame_sdl2/color.pyx":109
 *             return x.r == y.r and x.g == y.g and x.b == y.b and x.a == y.a
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         self.length = 4
 *         self.r = 0
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":116
 *         self.a = 255
 * 
 *     def __init__(self, *args):             # <<<<<<<<<<<<<<
 *         self.length = 4
 * 
 */

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_5__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_5__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_4__init__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), __pyx_v_args);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_4__init__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_args) {
  PyObject *__pyx_v_c = NULL;
  PyObject *__pyx_v_r = NULL;
  PyObject *__pyx_v_g = NULL;
  PyObject *__pyx_v_b = NULL;
  PyObject *__pyx_v_a = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Uint8 __pyx_t_8;
  Uint8 __pyx_t_9;
  Uint8 __pyx_t_10;
  Uint8 __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pygame_sdl2/color.pyx":117
 * 
 *     def __init__(self, *args):
 *         self.length = 4             # <<<<<<<<<<<<<<
 * 
 *         if len(args) == 1:
 */
  __pyx_v_self->length = 4;

  /* "pygame_sdl2/color.pyx":119
 *         self.length = 4
 * 
 *         if len(args) == 1:             # <<<<<<<<<<<<<<
 *             c = args[0]
 *             if isinstance(c, basestring):
 */
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 119, __pyx_L1_error)
  __pyx_t_2 = ((__pyx_t_1 == 1) != 0);
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":120
 * 
 *         if len(args) == 1:
 *             c = args[0]             # <<<<<<<<<<<<<<
 *             if isinstance(c, basestring):
 *                 if c.startswith('#'):
 */
    __pyx_t_3 = __Pyx_GetItemInt_Tuple(__pyx_v_args, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 120, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_c = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pygame_sdl2/color.pyx":121
 *         if len(args) == 1:
 *             c = args[0]
 *             if isinstance(c, basestring):             # <<<<<<<<<<<<<<
 *                 if c.startswith('#'):
 *                     self.from_hex(c[1:])
 */
    __pyx_t_2 = __Pyx_PyBaseString_Check(__pyx_v_c); 
    __pyx_t_4 = (__pyx_t_2 != 0);
    if (__pyx_t_4) {

      /* "pygame_sdl2/color.pyx":122
 *             c = args[0]
 *             if isinstance(c, basestring):
 *                 if c.startswith('#'):             # <<<<<<<<<<<<<<
 *                     self.from_hex(c[1:])
 *                 elif c.startswith('0x'):
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_c, __pyx_n_s_startswith); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 122, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_kp_u__3) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_kp_u__3);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 122, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 122, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_4) {

        /* "pygame_sdl2/color.pyx":123
 *             if isinstance(c, basestring):
 *                 if c.startswith('#'):
 *                     self.from_hex(c[1:])             # <<<<<<<<<<<<<<
 *                 elif c.startswith('0x'):
 *                     self.from_hex(c[2:])
 */
        __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_c, 1, 0, NULL, NULL, &__pyx_slice__4, 1, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 123, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = ((struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *)__pyx_v_self->__pyx_vtab)->from_hex(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 123, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "pygame_sdl2/color.pyx":122
 *             c = args[0]
 *             if isinstance(c, basestring):
 *                 if c.startswith('#'):             # <<<<<<<<<<<<<<
 *                     self.from_hex(c[1:])
 *                 elif c.startswith('0x'):
 */
        goto __pyx_L5;
      }

      /* "pygame_sdl2/color.pyx":124
 *                 if c.startswith('#'):
 *                     self.from_hex(c[1:])
 *                 elif c.startswith('0x'):             # <<<<<<<<<<<<<<
 *                     self.from_hex(c[2:])
 *                 else:
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_c, __pyx_n_s_startswith); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 124, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_5 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_kp_u_0x) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_0x);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 124, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 124, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (__pyx_t_4) {

        /* "pygame_sdl2/color.pyx":125
 *                     self.from_hex(c[1:])
 *                 elif c.startswith('0x'):
 *                     self.from_hex(c[2:])             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.from_name(c)
 */
        __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_v_c, 2, 0, NULL, NULL, &__pyx_slice__5, 1, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 125, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_3 = ((struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *)__pyx_v_self->__pyx_vtab)->from_hex(__pyx_v_self, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 125, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pygame_sdl2/color.pyx":124
 *                 if c.startswith('#'):
 *                     self.from_hex(c[1:])
 *                 elif c.startswith('0x'):             # <<<<<<<<<<<<<<
 *                     self.from_hex(c[2:])
 *                 else:
 */
        goto __pyx_L5;
      }

      /* "pygame_sdl2/color.pyx":127
 *                     self.from_hex(c[2:])
 *                 else:
 *                     self.from_name(c)             # <<<<<<<<<<<<<<
 *             elif isinstance(c, (tuple, list, Color)):
 *                 if len(c) == 4:
 */
      /*else*/ {
        __pyx_t_3 = ((struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *)__pyx_v_self->__pyx_vtab)->from_name(__pyx_v_self, __pyx_v_c); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 127, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __pyx_L5:;

      /* "pygame_sdl2/color.pyx":121
 *         if len(args) == 1:
 *             c = args[0]
 *             if isinstance(c, basestring):             # <<<<<<<<<<<<<<
 *                 if c.startswith('#'):
 *                     self.from_hex(c[1:])
 */
      goto __pyx_L4;
    }

    /* "pygame_sdl2/color.pyx":128
 *                 else:
 *                     self.from_name(c)
 *             elif isinstance(c, (tuple, list, Color)):             # <<<<<<<<<<<<<<
 *                 if len(c) == 4:
 *                     self.from_rgba(c[0], c[1], c[2], c[3])
 */
    __pyx_t_2 = PyTuple_Check(__pyx_v_c); 
    __pyx_t_7 = (__pyx_t_2 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_4 = __pyx_t_7;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_7 = PyList_Check(__pyx_v_c); 
    __pyx_t_2 = (__pyx_t_7 != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_4 = __pyx_t_2;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_c, __pyx_ptype_11pygame_sdl2_5color_Color); 
    __pyx_t_7 = (__pyx_t_2 != 0);
    __pyx_t_4 = __pyx_t_7;
    __pyx_L6_bool_binop_done:;
    __pyx_t_7 = (__pyx_t_4 != 0);
    if (__pyx_t_7) {

      /* "pygame_sdl2/color.pyx":129
 *                     self.from_name(c)
 *             elif isinstance(c, (tuple, list, Color)):
 *                 if len(c) == 4:             # <<<<<<<<<<<<<<
 *                     self.from_rgba(c[0], c[1], c[2], c[3])
 *                 elif len(c) == 3:
 */
      __pyx_t_1 = PyObject_Length(__pyx_v_c); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 129, __pyx_L1_error)
      __pyx_t_7 = ((__pyx_t_1 == 4) != 0);
      if (__pyx_t_7) {

        /* "pygame_sdl2/color.pyx":130
 *             elif isinstance(c, (tuple, list, Color)):
 *                 if len(c) == 4:
 *                     self.from_rgba(c[0], c[1], c[2], c[3])             # <<<<<<<<<<<<<<
 *                 elif len(c) == 3:
 *                     self.from_rgba(c[0], c[1], c[2], 255)
 */
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_c, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_8 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_8 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_c, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_9 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_9 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_c, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_10 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_c, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_11 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_11 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = ((struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *)__pyx_v_self->__pyx_vtab)->from_rgba(__pyx_v_self, __pyx_t_8, __pyx_t_9, __pyx_t_10, __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pygame_sdl2/color.pyx":129
 *                     self.from_name(c)
 *             elif isinstance(c, (tuple, list, Color)):
 *                 if len(c) == 4:             # <<<<<<<<<<<<<<
 *                     self.from_rgba(c[0], c[1], c[2], c[3])
 *                 elif len(c) == 3:
 */
        goto __pyx_L9;
      }

      /* "pygame_sdl2/color.pyx":131
 *                 if len(c) == 4:
 *                     self.from_rgba(c[0], c[1], c[2], c[3])
 *                 elif len(c) == 3:             # <<<<<<<<<<<<<<
 *                     self.from_rgba(c[0], c[1], c[2], 255)
 *                 else:
 */
      __pyx_t_1 = PyObject_Length(__pyx_v_c); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 131, __pyx_L1_error)
      __pyx_t_7 = ((__pyx_t_1 == 3) != 0);
      if (likely(__pyx_t_7)) {

        /* "pygame_sdl2/color.pyx":132
 *                     self.from_rgba(c[0], c[1], c[2], c[3])
 *                 elif len(c) == 3:
 *                     self.from_rgba(c[0], c[1], c[2], 255)             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(c)
 */
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_c, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 132, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_11 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_11 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 132, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_c, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 132, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_10 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 132, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_c, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 132, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_9 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_9 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 132, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = ((struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *)__pyx_v_self->__pyx_vtab)->from_rgba(__pyx_v_self, __pyx_t_11, __pyx_t_10, __pyx_t_9, 0xFF); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 132, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pygame_sdl2/color.pyx":131
 *                 if len(c) == 4:
 *                     self.from_rgba(c[0], c[1], c[2], c[3])
 *                 elif len(c) == 3:             # <<<<<<<<<<<<<<
 *                     self.from_rgba(c[0], c[1], c[2], 255)
 *                 else:
 */
        goto __pyx_L9;
      }

      /* "pygame_sdl2/color.pyx":134
 *                     self.from_rgba(c[0], c[1], c[2], 255)
 *                 else:
 *                     raise ValueError(c)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.from_hex("%08x" % c)
 */
      /*else*/ {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_c); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 134, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 134, __pyx_L1_error)
      }
      __pyx_L9:;

      /* "pygame_sdl2/color.pyx":128
 *                 else:
 *                     self.from_name(c)
 *             elif isinstance(c, (tuple, list, Color)):             # <<<<<<<<<<<<<<
 *                 if len(c) == 4:
 *                     self.from_rgba(c[0], c[1], c[2], c[3])
 */
      goto __pyx_L4;
    }

    /* "pygame_sdl2/color.pyx":136
 *                     raise ValueError(c)
 *             else:
 *                 self.from_hex("%08x" % c)             # <<<<<<<<<<<<<<
 * 
 *         elif len(args) == 3:
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_08x, __pyx_v_c); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 136, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = ((struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *)__pyx_v_self->__pyx_vtab)->from_hex(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 136, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_L4:;

    /* "pygame_sdl2/color.pyx":119
 *         self.length = 4
 * 
 *         if len(args) == 1:             # <<<<<<<<<<<<<<
 *             c = args[0]
 *             if isinstance(c, basestring):
 */
    goto __pyx_L3;
  }

  /* "pygame_sdl2/color.pyx":138
 *                 self.from_hex("%08x" % c)
 * 
 *         elif len(args) == 3:             # <<<<<<<<<<<<<<
 *             r, g, b = args
 *             self.from_rgba(r, g, b, 255)
 */
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 138, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_1 == 3) != 0);
  if (__pyx_t_7) {

    /* "pygame_sdl2/color.pyx":139
 * 
 *         elif len(args) == 3:
 *             r, g, b = args             # <<<<<<<<<<<<<<
 *             self.from_rgba(r, g, b, 255)
 *         elif len(args) == 4:
 */
    if (1) {
      PyObject* sequence = __pyx_v_args;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 139, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 2); 
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 139, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 139, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 139, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
    }
    __pyx_v_r = __pyx_t_5;
    __pyx_t_5 = 0;
    __pyx_v_g = __pyx_t_3;
    __pyx_t_3 = 0;
    __pyx_v_b = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "pygame_sdl2/color.pyx":140
 *         elif len(args) == 3:
 *             r, g, b = args
 *             self.from_rgba(r, g, b, 255)             # <<<<<<<<<<<<<<
 *         elif len(args) == 4:
 *             r, g, b, a = args
 */
    __pyx_t_9 = __Pyx_PyInt_As_uint8_t(__pyx_v_r); if (unlikely((__pyx_t_9 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 140, __pyx_L1_error)
    __pyx_t_10 = __Pyx_PyInt_As_uint8_t(__pyx_v_g); if (unlikely((__pyx_t_10 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 140, __pyx_L1_error)
    __pyx_t_11 = __Pyx_PyInt_As_uint8_t(__pyx_v_b); if (unlikely((__pyx_t_11 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 140, __pyx_L1_error)
    __pyx_t_6 = ((struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *)__pyx_v_self->__pyx_vtab)->from_rgba(__pyx_v_self, __pyx_t_9, __pyx_t_10, __pyx_t_11, 0xFF); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "pygame_sdl2/color.pyx":138
 *                 self.from_hex("%08x" % c)
 * 
 *         elif len(args) == 3:             # <<<<<<<<<<<<<<
 *             r, g, b = args
 *             self.from_rgba(r, g, b, 255)
 */
    goto __pyx_L3;
  }

  /* "pygame_sdl2/color.pyx":141
 *             r, g, b = args
 *             self.from_rgba(r, g, b, 255)
 *         elif len(args) == 4:             # <<<<<<<<<<<<<<
 *             r, g, b, a = args
 *             self.from_rgba(r, g, b, a)
 */
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 141, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_1 == 4) != 0);
  if (__pyx_t_7) {

    /* "pygame_sdl2/color.pyx":142
 *             self.from_rgba(r, g, b, 255)
 *         elif len(args) == 4:
 *             r, g, b, a = args             # <<<<<<<<<<<<<<
 *             self.from_rgba(r, g, b, a)
 * 
 */
    if (1) {
      PyObject* sequence = __pyx_v_args;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 4)) {
        if (size > 4) __Pyx_RaiseTooManyValuesError(4);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 142, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_12 = PyTuple_GET_ITEM(sequence, 3); 
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_12);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[4] = {&__pyx_t_6,&__pyx_t_3,&__pyx_t_5,&__pyx_t_12};
        for (i=0; i < 4; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 142, __pyx_L1_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
    }
    __pyx_v_r = __pyx_t_6;
    __pyx_t_6 = 0;
    __pyx_v_g = __pyx_t_3;
    __pyx_t_3 = 0;
    __pyx_v_b = __pyx_t_5;
    __pyx_t_5 = 0;
    __pyx_v_a = __pyx_t_12;
    __pyx_t_12 = 0;

    /* "pygame_sdl2/color.pyx":143
 *         elif len(args) == 4:
 *             r, g, b, a = args
 *             self.from_rgba(r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
    __pyx_t_11 = __Pyx_PyInt_As_uint8_t(__pyx_v_r); if (unlikely((__pyx_t_11 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 143, __pyx_L1_error)
    __pyx_t_10 = __Pyx_PyInt_As_uint8_t(__pyx_v_g); if (unlikely((__pyx_t_10 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 143, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyInt_As_uint8_t(__pyx_v_b); if (unlikely((__pyx_t_9 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 143, __pyx_L1_error)
    __pyx_t_8 = __Pyx_PyInt_As_uint8_t(__pyx_v_a); if (unlikely((__pyx_t_8 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 143, __pyx_L1_error)
    __pyx_t_12 = ((struct __pyx_vtabstruct_11pygame_sdl2_5color_Color *)__pyx_v_self->__pyx_vtab)->from_rgba(__pyx_v_self, __pyx_t_11, __pyx_t_10, __pyx_t_9, __pyx_t_8); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "pygame_sdl2/color.pyx":141
 *             r, g, b = args
 *             self.from_rgba(r, g, b, 255)
 *         elif len(args) == 4:             # <<<<<<<<<<<<<<
 *             r, g, b, a = args
 *             self.from_rgba(r, g, b, a)
 */
  }
  __pyx_L3:;

  /* "pygame_sdl2/color.pyx":116
 *         self.a = 255
 * 
 *     def __init__(self, *args):             # <<<<<<<<<<<<<<
 *         self.length = 4
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_c);
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":145
 *             self.from_rgba(r, g, b, a)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return str((self.r, self.g, self.b, self.a))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_7__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_7__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_6__repr__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_6__repr__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "pygame_sdl2/color.pyx":146
 * 
 *     def __repr__(self):
 *         return str((self.r, self.g, self.b, self.a))             # <<<<<<<<<<<<<<
 * 
 *     def __int__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->b); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->a); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":145
 *             self.from_rgba(r, g, b, a)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return str((self.r, self.g, self.b, self.a))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":148
 *         return str((self.r, self.g, self.b, self.a))
 * 
 *     def __int__(self):             # <<<<<<<<<<<<<<
 *         packed = struct.pack('BBBB', self.r, self.g, self.b, self.a)
 *         return struct.unpack('>L', packed)[0]
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_9__int__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_9__int__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__int__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_8__int__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_8__int__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_v_packed = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__int__", 0);

  /* "pygame_sdl2/color.pyx":149
 * 
 *     def __int__(self):
 *         packed = struct.pack('BBBB', self.r, self.g, self.b, self.a)             # <<<<<<<<<<<<<<
 *         return struct.unpack('>L', packed)[0]
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_struct); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_pack); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->r); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->b); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->a); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[6] = {__pyx_t_7, __pyx_n_u_BBBB, __pyx_t_2, __pyx_t_4, __pyx_t_5, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 5+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[6] = {__pyx_t_7, __pyx_n_u_BBBB, __pyx_t_2, __pyx_t_4, __pyx_t_5, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 5+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_9 = PyTuple_New(5+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_n_u_BBBB);
    __Pyx_GIVEREF(__pyx_n_u_BBBB);
    PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_n_u_BBBB);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_9, 3+__pyx_t_8, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 4+__pyx_t_8, __pyx_t_6);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_packed = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":150
 *     def __int__(self):
 *         packed = struct.pack('BBBB', self.r, self.g, self.b, self.a)
 *         return struct.unpack('>L', packed)[0]             # <<<<<<<<<<<<<<
 * 
 *     def __hex__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_struct); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_unpack); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_9)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_kp_u_L, __pyx_v_packed};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_kp_u_L, __pyx_v_packed};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_kp_u_L);
    __Pyx_GIVEREF(__pyx_kp_u_L);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_8, __pyx_kp_u_L);
    __Pyx_INCREF(__pyx_v_packed);
    __Pyx_GIVEREF(__pyx_v_packed);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_8, __pyx_v_packed);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":148
 *         return str((self.r, self.g, self.b, self.a))
 * 
 *     def __int__(self):             # <<<<<<<<<<<<<<
 *         packed = struct.pack('BBBB', self.r, self.g, self.b, self.a)
 *         return struct.unpack('>L', packed)[0]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__int__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_packed);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":152
 *         return struct.unpack('>L', packed)[0]
 * 
 *     def __hex__(self):             # <<<<<<<<<<<<<<
 *         return hex(int(self))
 * 
 */

/* Python wrapper */
#if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_11__hex__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_11__hex__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hex__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_10__hex__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
#endif /*!(#if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000))*/

#if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_10__hex__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__hex__", 0);

  /* "pygame_sdl2/color.pyx":153
 * 
 *     def __hex__(self):
 *         return hex(int(self))             # <<<<<<<<<<<<<<
 * 
 *     def __oct__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyNumber_Int(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_hex, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":152
 *         return struct.unpack('>L', packed)[0]
 * 
 *     def __hex__(self):             # <<<<<<<<<<<<<<
 *         return hex(int(self))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__hex__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
#endif /*!(#if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000))*/

/* "pygame_sdl2/color.pyx":155
 *         return hex(int(self))
 * 
 *     def __oct__(self):             # <<<<<<<<<<<<<<
 *         return oct(int(self))
 * 
 */

/* Python wrapper */
#if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_13__oct__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_13__oct__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__oct__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_12__oct__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
#endif /*!(#if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000))*/

#if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_12__oct__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__oct__", 0);

  /* "pygame_sdl2/color.pyx":156
 * 
 *     def __oct__(self):
 *         return oct(int(self))             # <<<<<<<<<<<<<<
 * 
 *     def __float__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyNumber_Int(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_oct, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":155
 *         return hex(int(self))
 * 
 *     def __oct__(self):             # <<<<<<<<<<<<<<
 *         return oct(int(self))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__oct__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
#endif /*!(#if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000))*/

/* "pygame_sdl2/color.pyx":158
 *         return oct(int(self))
 * 
 *     def __float__(self):             # <<<<<<<<<<<<<<
 *         return float(int(self))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_15__float__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_15__float__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__float__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_14__float__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_14__float__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__float__", 0);

  /* "pygame_sdl2/color.pyx":159
 * 
 *     def __float__(self):
 *         return float(int(self))             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyNumber_Int(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyNumber_Float(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":158
 *         return oct(int(self))
 * 
 *     def __float__(self):             # <<<<<<<<<<<<<<
 *         return float(int(self))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__float__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":161
 *         return float(int(self))
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         d = {}
 *         d['rgba'] = (self.r, self.g, self.b, self.a)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_17__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_17__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_16__reduce__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_16__reduce__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_v_d = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "pygame_sdl2/color.pyx":162
 * 
 *     def __reduce__(self):
 *         d = {}             # <<<<<<<<<<<<<<
 *         d['rgba'] = (self.r, self.g, self.b, self.a)
 *         return (Color, (), d)
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_d = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":163
 *     def __reduce__(self):
 *         d = {}
 *         d['rgba'] = (self.r, self.g, self.b, self.a)             # <<<<<<<<<<<<<<
 *         return (Color, (), d)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->b); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->a); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (unlikely(PyDict_SetItem(__pyx_v_d, __pyx_n_u_rgba, __pyx_t_5) < 0)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pygame_sdl2/color.pyx":164
 *         d = {}
 *         d['rgba'] = (self.r, self.g, self.b, self.a)
 *         return (Color, (), d)             # <<<<<<<<<<<<<<
 * 
 *     def __setstate__(self, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_11pygame_sdl2_5color_Color));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_11pygame_sdl2_5color_Color));
  PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_ptype_11pygame_sdl2_5color_Color));
  __Pyx_INCREF(__pyx_empty_tuple);
  __Pyx_GIVEREF(__pyx_empty_tuple);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_empty_tuple);
  __Pyx_INCREF(__pyx_v_d);
  __Pyx_GIVEREF(__pyx_v_d);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_d);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":161
 *         return float(int(self))
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         d = {}
 *         d['rgba'] = (self.r, self.g, self.b, self.a)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":166
 *         return (Color, (), d)
 * 
 *     def __setstate__(self, d):             # <<<<<<<<<<<<<<
 *         self.r, self.g, self.b, self.a = d['rgba']
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_19__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_d); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_19__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_18__setstate__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_d));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_18__setstate__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *(*__pyx_t_7)(PyObject *);
  Uint8 __pyx_t_8;
  Uint8 __pyx_t_9;
  Uint8 __pyx_t_10;
  Uint8 __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate__", 0);

  /* "pygame_sdl2/color.pyx":167
 * 
 *     def __setstate__(self, d):
 *         self.r, self.g, self.b, self.a = d['rgba']             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, key, val):
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_d, __pyx_n_u_rgba); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 4)) {
      if (size > 4) __Pyx_RaiseTooManyValuesError(4);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 167, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 3); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_4 = PyList_GET_ITEM(sequence, 2); 
      __pyx_t_5 = PyList_GET_ITEM(sequence, 3); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[4] = {&__pyx_t_2,&__pyx_t_3,&__pyx_t_4,&__pyx_t_5};
      for (i=0; i < 4; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 167, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    PyObject** temps[4] = {&__pyx_t_2,&__pyx_t_3,&__pyx_t_4,&__pyx_t_5};
    __pyx_t_6 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = Py_TYPE(__pyx_t_6)->tp_iternext;
    for (index=0; index < 4; index++) {
      PyObject* item = __pyx_t_7(__pyx_t_6); if (unlikely(!item)) goto __pyx_L3_unpacking_failed;
      __Pyx_GOTREF(item);
      *(temps[index]) = item;
    }
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_6), 4) < 0) __PYX_ERR(0, 167, __pyx_L1_error)
    __pyx_t_7 = NULL;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 167, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_t_8 = __Pyx_PyInt_As_uint8_t(__pyx_t_2); if (unlikely((__pyx_t_8 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_9 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = __Pyx_PyInt_As_uint8_t(__pyx_t_4); if (unlikely((__pyx_t_10 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_11 = __Pyx_PyInt_As_uint8_t(__pyx_t_5); if (unlikely((__pyx_t_11 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_self->r = __pyx_t_8;
  __pyx_v_self->g = __pyx_t_9;
  __pyx_v_self->b = __pyx_t_10;
  __pyx_v_self->a = __pyx_t_11;

  /* "pygame_sdl2/color.pyx":166
 *         return (Color, (), d)
 * 
 *     def __setstate__(self, d):             # <<<<<<<<<<<<<<
 *         self.r, self.g, self.b, self.a = d['rgba']
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__setstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":169
 *         self.r, self.g, self.b, self.a = d['rgba']
 * 
 *     def __setitem__(self, key, val):             # <<<<<<<<<<<<<<
 *         if not isinstance(val, int):
 *             raise ValueError(val)
 */

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_21__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_val); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_21__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_val) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_20__setitem__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_key), ((PyObject *)__pyx_v_val));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_20__setitem__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_val) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Uint8 __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "pygame_sdl2/color.pyx":170
 * 
 *     def __setitem__(self, key, val):
 *         if not isinstance(val, int):             # <<<<<<<<<<<<<<
 *             raise ValueError(val)
 *         if key >= self.length:
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_val); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "pygame_sdl2/color.pyx":171
 *     def __setitem__(self, key, val):
 *         if not isinstance(val, int):
 *             raise ValueError(val)             # <<<<<<<<<<<<<<
 *         if key >= self.length:
 *             raise IndexError(key)
 */
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_val); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 171, __pyx_L1_error)

    /* "pygame_sdl2/color.pyx":170
 * 
 *     def __setitem__(self, key, val):
 *         if not isinstance(val, int):             # <<<<<<<<<<<<<<
 *             raise ValueError(val)
 *         if key >= self.length:
 */
  }

  /* "pygame_sdl2/color.pyx":172
 *         if not isinstance(val, int):
 *             raise ValueError(val)
 *         if key >= self.length:             # <<<<<<<<<<<<<<
 *             raise IndexError(key)
 *         if val < 0 or val > 255:
 */
  __pyx_t_3 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->length); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_key, __pyx_t_3, Py_GE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(__pyx_t_2)) {

    /* "pygame_sdl2/color.pyx":173
 *             raise ValueError(val)
 *         if key >= self.length:
 *             raise IndexError(key)             # <<<<<<<<<<<<<<
 *         if val < 0 or val > 255:
 *             raise ValueError(val)
 */
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_v_key); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 173, __pyx_L1_error)

    /* "pygame_sdl2/color.pyx":172
 *         if not isinstance(val, int):
 *             raise ValueError(val)
 *         if key >= self.length:             # <<<<<<<<<<<<<<
 *             raise IndexError(key)
 *         if val < 0 or val > 255:
 */
  }

  /* "pygame_sdl2/color.pyx":174
 *         if key >= self.length:
 *             raise IndexError(key)
 *         if val < 0 or val > 255:             # <<<<<<<<<<<<<<
 *             raise ValueError(val)
 * 
 */
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_val, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 174, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_val, __pyx_int_255, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 174, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_2 = __pyx_t_1;
  __pyx_L6_bool_binop_done:;
  if (unlikely(__pyx_t_2)) {

    /* "pygame_sdl2/color.pyx":175
 *             raise IndexError(key)
 *         if val < 0 or val > 255:
 *             raise ValueError(val)             # <<<<<<<<<<<<<<
 * 
 *         if key == 0: self.r = val
 */
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_val); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 175, __pyx_L1_error)

    /* "pygame_sdl2/color.pyx":174
 *         if key >= self.length:
 *             raise IndexError(key)
 *         if val < 0 or val > 255:             # <<<<<<<<<<<<<<
 *             raise ValueError(val)
 * 
 */
  }

  /* "pygame_sdl2/color.pyx":177
 *             raise ValueError(val)
 * 
 *         if key == 0: self.r = val             # <<<<<<<<<<<<<<
 *         elif key == 1: self.g = val
 *         elif key == 2: self.b = val
 */
  __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_key, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_2) {
    __pyx_t_5 = __Pyx_PyInt_As_uint8_t(__pyx_v_val); if (unlikely((__pyx_t_5 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 177, __pyx_L1_error)
    __pyx_v_self->r = __pyx_t_5;
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":178
 * 
 *         if key == 0: self.r = val
 *         elif key == 1: self.g = val             # <<<<<<<<<<<<<<
 *         elif key == 2: self.b = val
 *         elif key == 3: self.a = val
 */
  __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_key, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_2) {
    __pyx_t_5 = __Pyx_PyInt_As_uint8_t(__pyx_v_val); if (unlikely((__pyx_t_5 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 178, __pyx_L1_error)
    __pyx_v_self->g = __pyx_t_5;
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":179
 *         if key == 0: self.r = val
 *         elif key == 1: self.g = val
 *         elif key == 2: self.b = val             # <<<<<<<<<<<<<<
 *         elif key == 3: self.a = val
 *         else:
 */
  __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_key, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_2) {
    __pyx_t_5 = __Pyx_PyInt_As_uint8_t(__pyx_v_val); if (unlikely((__pyx_t_5 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 179, __pyx_L1_error)
    __pyx_v_self->b = __pyx_t_5;
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":180
 *         elif key == 1: self.g = val
 *         elif key == 2: self.b = val
 *         elif key == 3: self.a = val             # <<<<<<<<<<<<<<
 *         else:
 *             raise IndexError(key)
 */
  __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_key, __pyx_int_3, 3, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(__pyx_t_2)) {
    __pyx_t_5 = __Pyx_PyInt_As_uint8_t(__pyx_v_val); if (unlikely((__pyx_t_5 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 180, __pyx_L1_error)
    __pyx_v_self->a = __pyx_t_5;
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":182
 *         elif key == 3: self.a = val
 *         else:
 *             raise IndexError(key)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, key):
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_v_key); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 182, __pyx_L1_error)
  }
  __pyx_L8:;

  /* "pygame_sdl2/color.pyx":169
 *         self.r, self.g, self.b, self.a = d['rgba']
 * 
 *     def __setitem__(self, key, val):             # <<<<<<<<<<<<<<
 *         if not isinstance(val, int):
 *             raise ValueError(val)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":184
 *             raise IndexError(key)
 * 
 *     def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *         if isinstance(key, slice):
 *             return tuple(self)[key]
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_23__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_23__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_22__getitem__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_22__getitem__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "pygame_sdl2/color.pyx":185
 * 
 *     def __getitem__(self, key):
 *         if isinstance(key, slice):             # <<<<<<<<<<<<<<
 *             return tuple(self)[key]
 *         if key >= self.length:
 */
  __pyx_t_1 = PySlice_Check(__pyx_v_key); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":186
 *     def __getitem__(self, key):
 *         if isinstance(key, slice):
 *             return tuple(self)[key]             # <<<<<<<<<<<<<<
 *         if key >= self.length:
 *             raise IndexError(key)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = PySequence_Tuple(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_3, __pyx_v_key); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "pygame_sdl2/color.pyx":185
 * 
 *     def __getitem__(self, key):
 *         if isinstance(key, slice):             # <<<<<<<<<<<<<<
 *             return tuple(self)[key]
 *         if key >= self.length:
 */
  }

  /* "pygame_sdl2/color.pyx":187
 *         if isinstance(key, slice):
 *             return tuple(self)[key]
 *         if key >= self.length:             # <<<<<<<<<<<<<<
 *             raise IndexError(key)
 *         if key == 0: return self.r
 */
  __pyx_t_4 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->length); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_key, __pyx_t_4, Py_GE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_t_2)) {

    /* "pygame_sdl2/color.pyx":188
 *             return tuple(self)[key]
 *         if key >= self.length:
 *             raise IndexError(key)             # <<<<<<<<<<<<<<
 *         if key == 0: return self.r
 *         elif key == 1: return self.g
 */
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_v_key); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 188, __pyx_L1_error)

    /* "pygame_sdl2/color.pyx":187
 *         if isinstance(key, slice):
 *             return tuple(self)[key]
 *         if key >= self.length:             # <<<<<<<<<<<<<<
 *             raise IndexError(key)
 *         if key == 0: return self.r
 */
  }

  /* "pygame_sdl2/color.pyx":189
 *         if key >= self.length:
 *             raise IndexError(key)
 *         if key == 0: return self.r             # <<<<<<<<<<<<<<
 *         elif key == 1: return self.g
 *         elif key == 2: return self.b
 */
  __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_key, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->r); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pygame_sdl2/color.pyx":190
 *             raise IndexError(key)
 *         if key == 0: return self.r
 *         elif key == 1: return self.g             # <<<<<<<<<<<<<<
 *         elif key == 2: return self.b
 *         elif key == 3: return self.a
 */
  __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_key, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pygame_sdl2/color.pyx":191
 *         if key == 0: return self.r
 *         elif key == 1: return self.g
 *         elif key == 2: return self.b             # <<<<<<<<<<<<<<
 *         elif key == 3: return self.a
 *         else:
 */
  __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_key, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->b); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pygame_sdl2/color.pyx":192
 *         elif key == 1: return self.g
 *         elif key == 2: return self.b
 *         elif key == 3: return self.a             # <<<<<<<<<<<<<<
 *         else:
 *             raise IndexError(key)
 */
  __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_v_key, __pyx_int_3, 3, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(__pyx_t_2)) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->a); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pygame_sdl2/color.pyx":194
 *         elif key == 3: return self.a
 *         else:
 *             raise IndexError(key)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_v_key); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 194, __pyx_L1_error)
  }

  /* "pygame_sdl2/color.pyx":184
 *             raise IndexError(key)
 * 
 *     def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *         if isinstance(key, slice):
 *             return tuple(self)[key]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":196
 *             raise IndexError(key)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self.length
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_11pygame_sdl2_5color_5Color_25__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_11pygame_sdl2_5color_5Color_25__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_24__len__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_11pygame_sdl2_5color_5Color_24__len__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "pygame_sdl2/color.pyx":197
 * 
 *     def __len__(self):
 *         return self.length             # <<<<<<<<<<<<<<
 * 
 *     def __mul__(self not None, Color rhs not None):
 */
  __pyx_r = __pyx_v_self->length;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":196
 *             raise IndexError(key)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self.length
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":199
 *         return self.length
 * 
 *     def __mul__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 *         # Multiplying this way doesn't make much sense,
 *         # but it's how pygame does it.
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_27__mul__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_27__mul__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__mul__ (wrapper)", 0);
  if (unlikely(((PyObject *)__pyx_v_self) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "self"); __PYX_ERR(0, 199, __pyx_L1_error)
  }
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rhs), __pyx_ptype_11pygame_sdl2_5color_Color, 0, "rhs", 0))) __PYX_ERR(0, 199, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_26__mul__(((PyObject *)__pyx_v_self), ((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_rhs));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_26__mul__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs) {
  PyObject *__pyx_v_r = NULL;
  PyObject *__pyx_v_g = NULL;
  PyObject *__pyx_v_b = NULL;
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  long __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__mul__", 0);

  /* "pygame_sdl2/color.pyx":203
 *         # but it's how pygame does it.
 * 
 *         r = min(255, self.r * rhs.r)             # <<<<<<<<<<<<<<
 *         g = min(255, self.g * rhs.g)
 *         b = min(255, self.b * rhs.b)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->r); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_r = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":204
 * 
 *         r = min(255, self.r * rhs.r)
 *         g = min(255, self.g * rhs.g)             # <<<<<<<<<<<<<<
 *         b = min(255, self.b * rhs.b)
 *         a = min(255, self.a * rhs.a)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_g); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->g); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyNumber_Multiply(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_5, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
  } else {
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_g = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pygame_sdl2/color.pyx":205
 *         r = min(255, self.r * rhs.r)
 *         g = min(255, self.g * rhs.g)
 *         b = min(255, self.b * rhs.b)             # <<<<<<<<<<<<<<
 *         a = min(255, self.a * rhs.a)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_b); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->b); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Multiply(__pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_5, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
  } else {
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_b = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":206
 *         g = min(255, self.g * rhs.g)
 *         b = min(255, self.b * rhs.b)
 *         a = min(255, self.a * rhs.a)             # <<<<<<<<<<<<<<
 * 
 *         return type(self)(r, g, b, a)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_a); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 206, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_a = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":208
 *         a = min(255, self.a * rhs.a)
 * 
 *         return type(self)(r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 *     def __add__(self not None, Color rhs not None):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_self)));
  __pyx_t_2 = ((PyObject *)Py_TYPE(__pyx_v_self)); __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(4+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_INCREF(__pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_7, __pyx_v_r);
    __Pyx_INCREF(__pyx_v_g);
    __Pyx_GIVEREF(__pyx_v_g);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_7, __pyx_v_g);
    __Pyx_INCREF(__pyx_v_b);
    __Pyx_GIVEREF(__pyx_v_b);
    PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_7, __pyx_v_b);
    __Pyx_INCREF(__pyx_v_a);
    __Pyx_GIVEREF(__pyx_v_a);
    PyTuple_SET_ITEM(__pyx_t_1, 3+__pyx_t_7, __pyx_v_a);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":199
 *         return self.length
 * 
 *     def __mul__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 *         # Multiplying this way doesn't make much sense,
 *         # but it's how pygame does it.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__mul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":210
 *         return type(self)(r, g, b, a)
 * 
 *     def __add__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 *         r = min(255, self.r + rhs.r)
 *         g = min(255, self.g + rhs.g)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_29__add__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_29__add__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__add__ (wrapper)", 0);
  if (unlikely(((PyObject *)__pyx_v_self) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "self"); __PYX_ERR(0, 210, __pyx_L1_error)
  }
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rhs), __pyx_ptype_11pygame_sdl2_5color_Color, 0, "rhs", 0))) __PYX_ERR(0, 210, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_28__add__(((PyObject *)__pyx_v_self), ((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_rhs));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_28__add__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs) {
  PyObject *__pyx_v_r = NULL;
  PyObject *__pyx_v_g = NULL;
  PyObject *__pyx_v_b = NULL;
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  long __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__add__", 0);

  /* "pygame_sdl2/color.pyx":211
 * 
 *     def __add__(self not None, Color rhs not None):
 *         r = min(255, self.r + rhs.r)             # <<<<<<<<<<<<<<
 *         g = min(255, self.g + rhs.g)
 *         b = min(255, self.b + rhs.b)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->r); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_r = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":212
 *     def __add__(self not None, Color rhs not None):
 *         r = min(255, self.r + rhs.r)
 *         g = min(255, self.g + rhs.g)             # <<<<<<<<<<<<<<
 *         b = min(255, self.b + rhs.b)
 *         a = min(255, self.a + rhs.a)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_g); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->g); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyNumber_Add(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_5, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
  } else {
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_g = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pygame_sdl2/color.pyx":213
 *         r = min(255, self.r + rhs.r)
 *         g = min(255, self.g + rhs.g)
 *         b = min(255, self.b + rhs.b)             # <<<<<<<<<<<<<<
 *         a = min(255, self.a + rhs.a)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_b); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->b); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Add(__pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_5, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
  } else {
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_b = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":214
 *         g = min(255, self.g + rhs.g)
 *         b = min(255, self.b + rhs.b)
 *         a = min(255, self.a + rhs.a)             # <<<<<<<<<<<<<<
 * 
 *         return type(self)(r, g, b, a)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_a); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_a = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":216
 *         a = min(255, self.a + rhs.a)
 * 
 *         return type(self)(r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 *     def __sub__(self not None, Color rhs not None):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_self)));
  __pyx_t_2 = ((PyObject *)Py_TYPE(__pyx_v_self)); __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(4+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_INCREF(__pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_7, __pyx_v_r);
    __Pyx_INCREF(__pyx_v_g);
    __Pyx_GIVEREF(__pyx_v_g);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_7, __pyx_v_g);
    __Pyx_INCREF(__pyx_v_b);
    __Pyx_GIVEREF(__pyx_v_b);
    PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_7, __pyx_v_b);
    __Pyx_INCREF(__pyx_v_a);
    __Pyx_GIVEREF(__pyx_v_a);
    PyTuple_SET_ITEM(__pyx_t_1, 3+__pyx_t_7, __pyx_v_a);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 216, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":210
 *         return type(self)(r, g, b, a)
 * 
 *     def __add__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 *         r = min(255, self.r + rhs.r)
 *         g = min(255, self.g + rhs.g)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__add__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":218
 *         return type(self)(r, g, b, a)
 * 
 *     def __sub__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 *         r = max(0, self.r - rhs.r)
 *         g = max(0, self.g - rhs.g)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_31__sub__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_31__sub__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__sub__ (wrapper)", 0);
  if (unlikely(((PyObject *)__pyx_v_self) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "self"); __PYX_ERR(0, 218, __pyx_L1_error)
  }
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rhs), __pyx_ptype_11pygame_sdl2_5color_Color, 0, "rhs", 0))) __PYX_ERR(0, 218, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_30__sub__(((PyObject *)__pyx_v_self), ((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_rhs));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_30__sub__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs) {
  PyObject *__pyx_v_r = NULL;
  PyObject *__pyx_v_g = NULL;
  PyObject *__pyx_v_b = NULL;
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  long __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__sub__", 0);

  /* "pygame_sdl2/color.pyx":219
 * 
 *     def __sub__(self not None, Color rhs not None):
 *         r = max(0, self.r - rhs.r)             # <<<<<<<<<<<<<<
 *         g = max(0, self.g - rhs.g)
 *         b = max(0, self.b - rhs.b)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->r); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_GT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 219, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_r = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":220
 *     def __sub__(self not None, Color rhs not None):
 *         r = max(0, self.r - rhs.r)
 *         g = max(0, self.g - rhs.g)             # <<<<<<<<<<<<<<
 *         b = max(0, self.b - rhs.b)
 *         a = max(0, self.a - rhs.a)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_g); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->g); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyNumber_Subtract(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_5, __pyx_t_3, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
  } else {
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_g = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pygame_sdl2/color.pyx":221
 *         r = max(0, self.r - rhs.r)
 *         g = max(0, self.g - rhs.g)
 *         b = max(0, self.b - rhs.b)             # <<<<<<<<<<<<<<
 *         a = max(0, self.a - rhs.a)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_b); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->b); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Subtract(__pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_5, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
  } else {
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_b = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":222
 *         g = max(0, self.g - rhs.g)
 *         b = max(0, self.b - rhs.b)
 *         a = max(0, self.a - rhs.a)             # <<<<<<<<<<<<<<
 * 
 *         return type(self)(r, g, b, a)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_a); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_GT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_a = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":224
 *         a = max(0, self.a - rhs.a)
 * 
 *         return type(self)(r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 *     def __mod__(self not None, Color rhs not None):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_self)));
  __pyx_t_2 = ((PyObject *)Py_TYPE(__pyx_v_self)); __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(4+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_INCREF(__pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_7, __pyx_v_r);
    __Pyx_INCREF(__pyx_v_g);
    __Pyx_GIVEREF(__pyx_v_g);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_7, __pyx_v_g);
    __Pyx_INCREF(__pyx_v_b);
    __Pyx_GIVEREF(__pyx_v_b);
    PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_7, __pyx_v_b);
    __Pyx_INCREF(__pyx_v_a);
    __Pyx_GIVEREF(__pyx_v_a);
    PyTuple_SET_ITEM(__pyx_t_1, 3+__pyx_t_7, __pyx_v_a);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":218
 *         return type(self)(r, g, b, a)
 * 
 *     def __sub__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 *         r = max(0, self.r - rhs.r)
 *         g = max(0, self.g - rhs.g)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__sub__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":226
 *         return type(self)(r, g, b, a)
 * 
 *     def __mod__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 * 
 *         def mod(l, r):
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_33__mod__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_33__mod__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__mod__ (wrapper)", 0);
  if (unlikely(((PyObject *)__pyx_v_self) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "self"); __PYX_ERR(0, 226, __pyx_L1_error)
  }
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rhs), __pyx_ptype_11pygame_sdl2_5color_Color, 0, "rhs", 0))) __PYX_ERR(0, 226, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_32__mod__(((PyObject *)__pyx_v_self), ((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_rhs));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":228
 *     def __mod__(self not None, Color rhs not None):
 * 
 *         def mod(l, r):             # <<<<<<<<<<<<<<
 *             if r == 0:
 *                 return 0
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_7__mod___1mod(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_11pygame_sdl2_5color_5Color_7__mod___1mod = {"mod", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_11pygame_sdl2_5color_5Color_7__mod___1mod, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_7__mod___1mod(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_l = 0;
  PyObject *__pyx_v_r = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("mod (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_l,&__pyx_n_s_r,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_l)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_r)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("mod", 1, 2, 2, 1); __PYX_ERR(0, 228, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "mod") < 0)) __PYX_ERR(0, 228, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_l = values[0];
    __pyx_v_r = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("mod", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 228, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__mod__.mod", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_7__mod___mod(__pyx_self, __pyx_v_l, __pyx_v_r);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_7__mod___mod(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_l, PyObject *__pyx_v_r) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("mod", 0);

  /* "pygame_sdl2/color.pyx":229
 * 
 *         def mod(l, r):
 *             if r == 0:             # <<<<<<<<<<<<<<
 *                 return 0
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_r, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 229, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":230
 *         def mod(l, r):
 *             if r == 0:
 *                 return 0             # <<<<<<<<<<<<<<
 * 
 *             return l % r
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_int_0);
    __pyx_r = __pyx_int_0;
    goto __pyx_L0;

    /* "pygame_sdl2/color.pyx":229
 * 
 *         def mod(l, r):
 *             if r == 0:             # <<<<<<<<<<<<<<
 *                 return 0
 * 
 */
  }

  /* "pygame_sdl2/color.pyx":232
 *                 return 0
 * 
 *             return l % r             # <<<<<<<<<<<<<<
 * 
 *         r = mod(self.r, rhs.r)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyNumber_Remainder(__pyx_v_l, __pyx_v_r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":228
 *     def __mod__(self not None, Color rhs not None):
 * 
 *         def mod(l, r):             # <<<<<<<<<<<<<<
 *             if r == 0:
 *                 return 0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__mod__.mod", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":226
 *         return type(self)(r, g, b, a)
 * 
 *     def __mod__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 * 
 *         def mod(l, r):
 */

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_32__mod__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs) {
  PyObject *__pyx_v_mod = 0;
  PyObject *__pyx_v_r = NULL;
  PyObject *__pyx_v_g = NULL;
  PyObject *__pyx_v_b = NULL;
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__mod__", 0);

  /* "pygame_sdl2/color.pyx":228
 *     def __mod__(self not None, Color rhs not None):
 * 
 *         def mod(l, r):             # <<<<<<<<<<<<<<
 *             if r == 0:
 *                 return 0
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_11pygame_sdl2_5color_5Color_7__mod___1mod, 0, __pyx_n_s_mod___locals_mod, NULL, __pyx_n_s_pygame_sdl2_color, __pyx_d, ((PyObject *)__pyx_codeobj__7)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_mod = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":234
 *             return l % r
 * 
 *         r = mod(self.r, rhs.r)             # <<<<<<<<<<<<<<
 *         g = mod(self.g, rhs.g)
 *         b = mod(self.b, rhs.b)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->r); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_pf_11pygame_sdl2_5color_5Color_7__mod___mod(__pyx_v_mod, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_r = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":235
 * 
 *         r = mod(self.r, rhs.r)
 *         g = mod(self.g, rhs.g)             # <<<<<<<<<<<<<<
 *         b = mod(self.b, rhs.b)
 *         a = mod(self.a, rhs.a)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_g); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->g); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __pyx_pf_11pygame_sdl2_5color_5Color_7__mod___mod(__pyx_v_mod, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_g = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":236
 *         r = mod(self.r, rhs.r)
 *         g = mod(self.g, rhs.g)
 *         b = mod(self.b, rhs.b)             # <<<<<<<<<<<<<<
 *         a = mod(self.a, rhs.a)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_b); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->b); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_pf_11pygame_sdl2_5color_5Color_7__mod___mod(__pyx_v_mod, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_b = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":237
 *         g = mod(self.g, rhs.g)
 *         b = mod(self.b, rhs.b)
 *         a = mod(self.a, rhs.a)             # <<<<<<<<<<<<<<
 * 
 *         return type(self)(r, g, b, a)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_a); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __pyx_pf_11pygame_sdl2_5color_5Color_7__mod___mod(__pyx_v_mod, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_a = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":239
 *         a = mod(self.a, rhs.a)
 * 
 *         return type(self)(r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 *     def __div__(self not None, Color rhs not None):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_self)));
  __pyx_t_2 = ((PyObject *)Py_TYPE(__pyx_v_self)); __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_3, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 4+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_3, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 4+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(4+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 239, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_r);
    __Pyx_INCREF(__pyx_v_g);
    __Pyx_GIVEREF(__pyx_v_g);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_g);
    __Pyx_INCREF(__pyx_v_b);
    __Pyx_GIVEREF(__pyx_v_b);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_b);
    __Pyx_INCREF(__pyx_v_a);
    __Pyx_GIVEREF(__pyx_v_a);
    PyTuple_SET_ITEM(__pyx_t_5, 3+__pyx_t_4, __pyx_v_a);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":226
 *         return type(self)(r, g, b, a)
 * 
 *     def __mod__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 * 
 *         def mod(l, r):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__mod__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_mod);
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":241
 *         return type(self)(r, g, b, a)
 * 
 *     def __div__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 *         r = min(255, self.r / rhs.r)
 *         g = min(255, self.g / rhs.g)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_35__div__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_35__div__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__div__ (wrapper)", 0);
  if (unlikely(((PyObject *)__pyx_v_self) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "self"); __PYX_ERR(0, 241, __pyx_L1_error)
  }
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rhs), __pyx_ptype_11pygame_sdl2_5color_Color, 0, "rhs", 0))) __PYX_ERR(0, 241, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_34__div__(((PyObject *)__pyx_v_self), ((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_rhs));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_34__div__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs) {
  PyObject *__pyx_v_r = NULL;
  PyObject *__pyx_v_g = NULL;
  PyObject *__pyx_v_b = NULL;
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  long __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__div__", 0);

  /* "pygame_sdl2/color.pyx":242
 * 
 *     def __div__(self not None, Color rhs not None):
 *         r = min(255, self.r / rhs.r)             # <<<<<<<<<<<<<<
 *         g = min(255, self.g / rhs.g)
 *         b = min(255, self.b / rhs.b)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->r); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 242, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_r = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":243
 *     def __div__(self not None, Color rhs not None):
 *         r = min(255, self.r / rhs.r)
 *         g = min(255, self.g / rhs.g)             # <<<<<<<<<<<<<<
 *         b = min(255, self.b / rhs.b)
 *         a = min(255, self.a / rhs.a)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_g); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->g); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_5, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
  } else {
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 243, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_g = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pygame_sdl2/color.pyx":244
 *         r = min(255, self.r / rhs.r)
 *         g = min(255, self.g / rhs.g)
 *         b = min(255, self.b / rhs.b)             # <<<<<<<<<<<<<<
 *         a = min(255, self.a / rhs.a)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_b); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->b); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_5, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
  } else {
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 244, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_b = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":245
 *         g = min(255, self.g / rhs.g)
 *         b = min(255, self.b / rhs.b)
 *         a = min(255, self.a / rhs.a)             # <<<<<<<<<<<<<<
 * 
 *         return type(self)(r, g, b, a)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_a); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = 0xFF;
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
  } else {
    __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 245, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_a = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":247
 *         a = min(255, self.a / rhs.a)
 * 
 *         return type(self)(r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 *     def __floordiv__(self not None, Color rhs not None):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_self)));
  __pyx_t_2 = ((PyObject *)Py_TYPE(__pyx_v_self)); __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(4+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_INCREF(__pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_7, __pyx_v_r);
    __Pyx_INCREF(__pyx_v_g);
    __Pyx_GIVEREF(__pyx_v_g);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_7, __pyx_v_g);
    __Pyx_INCREF(__pyx_v_b);
    __Pyx_GIVEREF(__pyx_v_b);
    PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_7, __pyx_v_b);
    __Pyx_INCREF(__pyx_v_a);
    __Pyx_GIVEREF(__pyx_v_a);
    PyTuple_SET_ITEM(__pyx_t_1, 3+__pyx_t_7, __pyx_v_a);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":241
 *         return type(self)(r, g, b, a)
 * 
 *     def __div__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 *         r = min(255, self.r / rhs.r)
 *         g = min(255, self.g / rhs.g)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__div__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":249
 *         return type(self)(r, g, b, a)
 * 
 *     def __floordiv__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 * 
 *         def div(a, b):
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_37__floordiv__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_37__floordiv__(PyObject *__pyx_v_self, PyObject *__pyx_v_rhs) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__floordiv__ (wrapper)", 0);
  if (unlikely(((PyObject *)__pyx_v_self) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "self"); __PYX_ERR(0, 249, __pyx_L1_error)
  }
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_rhs), __pyx_ptype_11pygame_sdl2_5color_Color, 0, "rhs", 0))) __PYX_ERR(0, 249, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_36__floordiv__(((PyObject *)__pyx_v_self), ((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_rhs));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":251
 *     def __floordiv__(self not None, Color rhs not None):
 * 
 *         def div(a, b):             # <<<<<<<<<<<<<<
 *             if b == 0:
 *                 return b
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_12__floordiv___1div(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_11pygame_sdl2_5color_5Color_12__floordiv___1div = {"div", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_11pygame_sdl2_5color_5Color_12__floordiv___1div, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_12__floordiv___1div(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_a = 0;
  PyObject *__pyx_v_b = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("div (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("div", 1, 2, 2, 1); __PYX_ERR(0, 251, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "div") < 0)) __PYX_ERR(0, 251, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_a = values[0];
    __pyx_v_b = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("div", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 251, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__floordiv__.div", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_12__floordiv___div(__pyx_self, __pyx_v_a, __pyx_v_b);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_12__floordiv___div(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  long __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("div", 0);

  /* "pygame_sdl2/color.pyx":252
 * 
 *         def div(a, b):
 *             if b == 0:             # <<<<<<<<<<<<<<
 *                 return b
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_b, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":253
 *         def div(a, b):
 *             if b == 0:
 *                 return b             # <<<<<<<<<<<<<<
 * 
 *             return min(255, a // b)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_b);
    __pyx_r = __pyx_v_b;
    goto __pyx_L0;

    /* "pygame_sdl2/color.pyx":252
 * 
 *         def div(a, b):
 *             if b == 0:             # <<<<<<<<<<<<<<
 *                 return b
 * 
 */
  }

  /* "pygame_sdl2/color.pyx":255
 *                 return b
 * 
 *             return min(255, a // b)             # <<<<<<<<<<<<<<
 * 
 *         r = div(self.r, rhs.r)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyNumber_FloorDivide(__pyx_v_a, __pyx_v_b); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = 0xFF;
  __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyObject_RichCompare(__pyx_t_1, __pyx_t_5, Py_LT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (__pyx_t_2) {
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_4 = __pyx_t_1;
  } else {
    __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 255, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = __pyx_t_6;
    __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":251
 *     def __floordiv__(self not None, Color rhs not None):
 * 
 *         def div(a, b):             # <<<<<<<<<<<<<<
 *             if b == 0:
 *                 return b
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__floordiv__.div", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":249
 *         return type(self)(r, g, b, a)
 * 
 *     def __floordiv__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 * 
 *         def div(a, b):
 */

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_36__floordiv__(PyObject *__pyx_v_self, struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_rhs) {
  PyObject *__pyx_v_div = 0;
  PyObject *__pyx_v_r = NULL;
  PyObject *__pyx_v_g = NULL;
  PyObject *__pyx_v_b = NULL;
  PyObject *__pyx_v_a = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__floordiv__", 0);

  /* "pygame_sdl2/color.pyx":251
 *     def __floordiv__(self not None, Color rhs not None):
 * 
 *         def div(a, b):             # <<<<<<<<<<<<<<
 *             if b == 0:
 *                 return b
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_11pygame_sdl2_5color_5Color_12__floordiv___1div, 0, __pyx_n_s_floordiv___locals_div, NULL, __pyx_n_s_pygame_sdl2_color, __pyx_d, ((PyObject *)__pyx_codeobj__9)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_div = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":257
 *             return min(255, a // b)
 * 
 *         r = div(self.r, rhs.r)             # <<<<<<<<<<<<<<
 *         g = div(self.g, rhs.g)
 *         b = div(self.b, rhs.b)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_r); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->r); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_pf_11pygame_sdl2_5color_5Color_12__floordiv___div(__pyx_v_div, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_r = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":258
 * 
 *         r = div(self.r, rhs.r)
 *         g = div(self.g, rhs.g)             # <<<<<<<<<<<<<<
 *         b = div(self.b, rhs.b)
 *         a = div(self.a, rhs.a)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_g); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->g); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __pyx_pf_11pygame_sdl2_5color_5Color_12__floordiv___div(__pyx_v_div, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_g = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":259
 *         r = div(self.r, rhs.r)
 *         g = div(self.g, rhs.g)
 *         b = div(self.b, rhs.b)             # <<<<<<<<<<<<<<
 *         a = div(self.a, rhs.a)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_b); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->b); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_pf_11pygame_sdl2_5color_5Color_12__floordiv___div(__pyx_v_div, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_b = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":260
 *         g = div(self.g, rhs.g)
 *         b = div(self.b, rhs.b)
 *         a = div(self.a, rhs.a)             # <<<<<<<<<<<<<<
 * 
 *         return type(self)(r, g, b, a)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_a); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_uint8_t(__pyx_v_rhs->a); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __pyx_pf_11pygame_sdl2_5color_5Color_12__floordiv___div(__pyx_v_div, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_a = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":262
 *         a = div(self.a, rhs.a)
 * 
 *         return type(self)(r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 *     property cmy:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_self)));
  __pyx_t_2 = ((PyObject *)Py_TYPE(__pyx_v_self)); __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_3, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 4+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_3, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 4+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(4+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_r);
    __Pyx_GIVEREF(__pyx_v_r);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_r);
    __Pyx_INCREF(__pyx_v_g);
    __Pyx_GIVEREF(__pyx_v_g);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_g);
    __Pyx_INCREF(__pyx_v_b);
    __Pyx_GIVEREF(__pyx_v_b);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_b);
    __Pyx_INCREF(__pyx_v_a);
    __Pyx_GIVEREF(__pyx_v_a);
    PyTuple_SET_ITEM(__pyx_t_5, 3+__pyx_t_4, __pyx_v_a);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":249
 *         return type(self)(r, g, b, a)
 * 
 *     def __floordiv__(self not None, Color rhs not None):             # <<<<<<<<<<<<<<
 * 
 *         def div(a, b):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.__floordiv__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_div);
  __Pyx_XDECREF(__pyx_v_r);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XDECREF(__pyx_v_a);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":265
 * 
 *     property cmy:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return 1 - (self.r / 255.0), 1 - (self.g / 255.0), 1 - (self.b / 255.0)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_3cmy_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_3cmy_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_3cmy___get__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_3cmy___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pygame_sdl2/color.pyx":266
 *     property cmy:
 *         def __get__(self):
 *             return 1 - (self.r / 255.0), 1 - (self.g / 255.0), 1 - (self.b / 255.0)             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, val):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble((1.0 - (((double)__pyx_v_self->r) / 255.0))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble((1.0 - (((double)__pyx_v_self->g) / 255.0))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble((1.0 - (((double)__pyx_v_self->b) / 255.0))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":265
 * 
 *     property cmy:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return 1 - (self.r / 255.0), 1 - (self.g / 255.0), 1 - (self.b / 255.0)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.cmy.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":268
 *             return 1 - (self.r / 255.0), 1 - (self.g / 255.0), 1 - (self.b / 255.0)
 * 
 *         def __set__(self, val):             # <<<<<<<<<<<<<<
 *             c, m, y = val
 *             self.r = (1 - c) * 255
 */

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_3cmy_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_val); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_3cmy_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_val) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_3cmy_2__set__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_val));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_3cmy_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_val) {
  PyObject *__pyx_v_c = NULL;
  PyObject *__pyx_v_m = NULL;
  PyObject *__pyx_v_y = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  Uint8 __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pygame_sdl2/color.pyx":269
 * 
 *         def __set__(self, val):
 *             c, m, y = val             # <<<<<<<<<<<<<<
 *             self.r = (1 - c) * 255
 *             self.g = (1 - m) * 255
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_val))) || (PyList_CheckExact(__pyx_v_val))) {
    PyObject* sequence = __pyx_v_val;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 3)) {
      if (size > 3) __Pyx_RaiseTooManyValuesError(3);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 269, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_v_val); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_1)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_2 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 2; __pyx_t_3 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_4), 3) < 0) __PYX_ERR(0, 269, __pyx_L1_error)
    __pyx_t_5 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 269, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_c = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_m = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_y = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pygame_sdl2/color.pyx":270
 *         def __set__(self, val):
 *             c, m, y = val
 *             self.r = (1 - c) * 255             # <<<<<<<<<<<<<<
 *             self.g = (1 - m) * 255
 *             self.b = (1 - y) * 255
 */
  __pyx_t_3 = __Pyx_PyInt_SubtractCObj(__pyx_int_1, __pyx_v_c, 1, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_3, __pyx_int_255); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_uint8_t(__pyx_t_2); if (unlikely((__pyx_t_6 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->r = __pyx_t_6;

  /* "pygame_sdl2/color.pyx":271
 *             c, m, y = val
 *             self.r = (1 - c) * 255
 *             self.g = (1 - m) * 255             # <<<<<<<<<<<<<<
 *             self.b = (1 - y) * 255
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_SubtractCObj(__pyx_int_1, __pyx_v_m, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_2, __pyx_int_255); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_uint8_t(__pyx_t_3); if (unlikely((__pyx_t_6 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_self->g = __pyx_t_6;

  /* "pygame_sdl2/color.pyx":272
 *             self.r = (1 - c) * 255
 *             self.g = (1 - m) * 255
 *             self.b = (1 - y) * 255             # <<<<<<<<<<<<<<
 * 
 *     property hsva:
 */
  __pyx_t_3 = __Pyx_PyInt_SubtractCObj(__pyx_int_1, __pyx_v_y, 1, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_3, __pyx_int_255); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_uint8_t(__pyx_t_2); if (unlikely((__pyx_t_6 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->b = __pyx_t_6;

  /* "pygame_sdl2/color.pyx":268
 *             return 1 - (self.r / 255.0), 1 - (self.g / 255.0), 1 - (self.b / 255.0)
 * 
 *         def __set__(self, val):             # <<<<<<<<<<<<<<
 *             c, m, y = val
 *             self.r = (1 - c) * 255
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.cmy.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_c);
  __Pyx_XDECREF(__pyx_v_m);
  __Pyx_XDECREF(__pyx_v_y);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":275
 * 
 *     property hsva:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef double r = self.r / 255.0
 *             cdef double g = self.g / 255.0
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_4hsva_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_4hsva_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_4hsva___get__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_4hsva___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  double __pyx_v_r;
  double __pyx_v_g;
  double __pyx_v_b;
  double __pyx_v_cmax;
  double __pyx_v_cmin;
  double __pyx_v_delta;
  double __pyx_v_h;
  double __pyx_v_s;
  double __pyx_v_v;
  double __pyx_v_a;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  double __pyx_t_2;
  double __pyx_t_3;
  double __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pygame_sdl2/color.pyx":276
 *     property hsva:
 *         def __get__(self):
 *             cdef double r = self.r / 255.0             # <<<<<<<<<<<<<<
 *             cdef double g = self.g / 255.0
 *             cdef double b = self.b / 255.0
 */
  __pyx_v_r = (((double)__pyx_v_self->r) / 255.0);

  /* "pygame_sdl2/color.pyx":277
 *         def __get__(self):
 *             cdef double r = self.r / 255.0
 *             cdef double g = self.g / 255.0             # <<<<<<<<<<<<<<
 *             cdef double b = self.b / 255.0
 * 
 */
  __pyx_v_g = (((double)__pyx_v_self->g) / 255.0);

  /* "pygame_sdl2/color.pyx":278
 *             cdef double r = self.r / 255.0
 *             cdef double g = self.g / 255.0
 *             cdef double b = self.b / 255.0             # <<<<<<<<<<<<<<
 * 
 *             cdef double cmax = max(r, g, b)
 */
  __pyx_v_b = (((double)__pyx_v_self->b) / 255.0);

  /* "pygame_sdl2/color.pyx":280
 *             cdef double b = self.b / 255.0
 * 
 *             cdef double cmax = max(r, g, b)             # <<<<<<<<<<<<<<
 *             cdef double cmin = min(r, g, b)
 *             cdef double delta = cmax - cmin
 */
  __pyx_t_1 = __pyx_v_g;
  __pyx_t_2 = __pyx_v_b;
  __pyx_t_3 = __pyx_v_r;
  if (((__pyx_t_1 > __pyx_t_3) != 0)) {
    __pyx_t_4 = __pyx_t_1;
  } else {
    __pyx_t_4 = __pyx_t_3;
  }
  __pyx_t_3 = __pyx_t_4;
  if (((__pyx_t_2 > __pyx_t_3) != 0)) {
    __pyx_t_4 = __pyx_t_2;
  } else {
    __pyx_t_4 = __pyx_t_3;
  }
  __pyx_v_cmax = __pyx_t_4;

  /* "pygame_sdl2/color.pyx":281
 * 
 *             cdef double cmax = max(r, g, b)
 *             cdef double cmin = min(r, g, b)             # <<<<<<<<<<<<<<
 *             cdef double delta = cmax - cmin
 * 
 */
  __pyx_t_4 = __pyx_v_g;
  __pyx_t_1 = __pyx_v_b;
  __pyx_t_2 = __pyx_v_r;
  if (((__pyx_t_4 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_4;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_t_2 = __pyx_t_3;
  if (((__pyx_t_1 < __pyx_t_2) != 0)) {
    __pyx_t_3 = __pyx_t_1;
  } else {
    __pyx_t_3 = __pyx_t_2;
  }
  __pyx_v_cmin = __pyx_t_3;

  /* "pygame_sdl2/color.pyx":282
 *             cdef double cmax = max(r, g, b)
 *             cdef double cmin = min(r, g, b)
 *             cdef double delta = cmax - cmin             # <<<<<<<<<<<<<<
 * 
 *             cdef double h, s, v, a
 */
  __pyx_v_delta = (__pyx_v_cmax - __pyx_v_cmin);

  /* "pygame_sdl2/color.pyx":286
 *             cdef double h, s, v, a
 * 
 *             if r == g == b:             # <<<<<<<<<<<<<<
 *                 h = 0.0
 *                 s = 0.0
 */
  __pyx_t_5 = (__pyx_v_r == __pyx_v_g);
  if (__pyx_t_5) {
    __pyx_t_5 = (__pyx_v_g == __pyx_v_b);
  }
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "pygame_sdl2/color.pyx":287
 * 
 *             if r == g == b:
 *                 h = 0.0             # <<<<<<<<<<<<<<
 *                 s = 0.0
 *             else:
 */
    __pyx_v_h = 0.0;

    /* "pygame_sdl2/color.pyx":288
 *             if r == g == b:
 *                 h = 0.0
 *                 s = 0.0             # <<<<<<<<<<<<<<
 *             else:
 *                 if cmax == r:
 */
    __pyx_v_s = 0.0;

    /* "pygame_sdl2/color.pyx":286
 *             cdef double h, s, v, a
 * 
 *             if r == g == b:             # <<<<<<<<<<<<<<
 *                 h = 0.0
 *                 s = 0.0
 */
    goto __pyx_L3;
  }

  /* "pygame_sdl2/color.pyx":290
 *                 s = 0.0
 *             else:
 *                 if cmax == r:             # <<<<<<<<<<<<<<
 *                     h = 60.0 * ((g - b) / delta % 6)
 *                 elif cmax == g:
 */
  /*else*/ {
    __pyx_t_6 = ((__pyx_v_cmax == __pyx_v_r) != 0);
    if (__pyx_t_6) {

      /* "pygame_sdl2/color.pyx":291
 *             else:
 *                 if cmax == r:
 *                     h = 60.0 * ((g - b) / delta % 6)             # <<<<<<<<<<<<<<
 *                 elif cmax == g:
 *                     h = 60.0 * ((b - r) / delta + 2)
 */
      __pyx_t_3 = (__pyx_v_g - __pyx_v_b);
      if (unlikely(__pyx_v_delta == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(0, 291, __pyx_L1_error)
      }
      __pyx_v_h = (60.0 * __Pyx_mod_double((__pyx_t_3 / __pyx_v_delta), 6.0));

      /* "pygame_sdl2/color.pyx":290
 *                 s = 0.0
 *             else:
 *                 if cmax == r:             # <<<<<<<<<<<<<<
 *                     h = 60.0 * ((g - b) / delta % 6)
 *                 elif cmax == g:
 */
      goto __pyx_L4;
    }

    /* "pygame_sdl2/color.pyx":292
 *                 if cmax == r:
 *                     h = 60.0 * ((g - b) / delta % 6)
 *                 elif cmax == g:             # <<<<<<<<<<<<<<
 *                     h = 60.0 * ((b - r) / delta + 2)
 *                 else:
 */
    __pyx_t_6 = ((__pyx_v_cmax == __pyx_v_g) != 0);
    if (__pyx_t_6) {

      /* "pygame_sdl2/color.pyx":293
 *                     h = 60.0 * ((g - b) / delta % 6)
 *                 elif cmax == g:
 *                     h = 60.0 * ((b - r) / delta + 2)             # <<<<<<<<<<<<<<
 *                 else:
 *                     h = 60.0 * ((r - g) / delta + 4)
 */
      __pyx_t_3 = (__pyx_v_b - __pyx_v_r);
      if (unlikely(__pyx_v_delta == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(0, 293, __pyx_L1_error)
      }
      __pyx_v_h = (60.0 * ((__pyx_t_3 / __pyx_v_delta) + 2.0));

      /* "pygame_sdl2/color.pyx":292
 *                 if cmax == r:
 *                     h = 60.0 * ((g - b) / delta % 6)
 *                 elif cmax == g:             # <<<<<<<<<<<<<<
 *                     h = 60.0 * ((b - r) / delta + 2)
 *                 else:
 */
      goto __pyx_L4;
    }

    /* "pygame_sdl2/color.pyx":295
 *                     h = 60.0 * ((b - r) / delta + 2)
 *                 else:
 *                     h = 60.0 * ((r - g) / delta + 4)             # <<<<<<<<<<<<<<
 * 
 *                 if cmax == 0.0:
 */
    /*else*/ {
      __pyx_t_3 = (__pyx_v_r - __pyx_v_g);
      if (unlikely(__pyx_v_delta == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(0, 295, __pyx_L1_error)
      }
      __pyx_v_h = (60.0 * ((__pyx_t_3 / __pyx_v_delta) + 4.0));
    }
    __pyx_L4:;

    /* "pygame_sdl2/color.pyx":297
 *                     h = 60.0 * ((r - g) / delta + 4)
 * 
 *                 if cmax == 0.0:             # <<<<<<<<<<<<<<
 *                     s = 0.0
 *                 else:
 */
    __pyx_t_6 = ((__pyx_v_cmax == 0.0) != 0);
    if (__pyx_t_6) {

      /* "pygame_sdl2/color.pyx":298
 * 
 *                 if cmax == 0.0:
 *                     s = 0.0             # <<<<<<<<<<<<<<
 *                 else:
 *                     s = delta / cmax * 100
 */
      __pyx_v_s = 0.0;

      /* "pygame_sdl2/color.pyx":297
 *                     h = 60.0 * ((r - g) / delta + 4)
 * 
 *                 if cmax == 0.0:             # <<<<<<<<<<<<<<
 *                     s = 0.0
 *                 else:
 */
      goto __pyx_L5;
    }

    /* "pygame_sdl2/color.pyx":300
 *                     s = 0.0
 *                 else:
 *                     s = delta / cmax * 100             # <<<<<<<<<<<<<<
 * 
 *             v = cmax * 100
 */
    /*else*/ {
      if (unlikely(__pyx_v_cmax == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(0, 300, __pyx_L1_error)
      }
      __pyx_v_s = ((__pyx_v_delta / __pyx_v_cmax) * 100.0);
    }
    __pyx_L5:;
  }
  __pyx_L3:;

  /* "pygame_sdl2/color.pyx":302
 *                     s = delta / cmax * 100
 * 
 *             v = cmax * 100             # <<<<<<<<<<<<<<
 *             a = self.a / 255.0 * 100
 *             return h, s, v, a
 */
  __pyx_v_v = (__pyx_v_cmax * 100.0);

  /* "pygame_sdl2/color.pyx":303
 * 
 *             v = cmax * 100
 *             a = self.a / 255.0 * 100             # <<<<<<<<<<<<<<
 *             return h, s, v, a
 * 
 */
  __pyx_v_a = ((((double)__pyx_v_self->a) / 255.0) * 100.0);

  /* "pygame_sdl2/color.pyx":304
 *             v = cmax * 100
 *             a = self.a / 255.0 * 100
 *             return h, s, v, a             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, val):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_h); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_s); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_v); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_a); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_New(4); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_11, 3, __pyx_t_10);
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_r = __pyx_t_11;
  __pyx_t_11 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":275
 * 
 *     property hsva:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef double r = self.r / 255.0
 *             cdef double g = self.g / 255.0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.hsva.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":306
 *             return h, s, v, a
 * 
 *         def __set__(self, val):             # <<<<<<<<<<<<<<
 *             cdef double h, s, v, a
 *             if len(val) == 3:
 */

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_4hsva_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_val); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_4hsva_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_val) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_4hsva_2__set__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_val));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_4hsva_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_val) {
  double __pyx_v_h;
  double __pyx_v_s;
  double __pyx_v_v;
  double __pyx_v_a;
  double __pyx_v_c;
  double __pyx_v_x;
  double __pyx_v_m;
  double __pyx_v_r;
  double __pyx_v_g;
  double __pyx_v_b;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *(*__pyx_t_7)(PyObject *);
  double __pyx_t_8;
  double __pyx_t_9;
  double __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  double __pyx_t_12;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pygame_sdl2/color.pyx":308
 *         def __set__(self, val):
 *             cdef double h, s, v, a
 *             if len(val) == 3:             # <<<<<<<<<<<<<<
 *                 h, s, v = val
 *                 a = 0.0
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_val); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 308, __pyx_L1_error)
  __pyx_t_2 = ((__pyx_t_1 == 3) != 0);
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":309
 *             cdef double h, s, v, a
 *             if len(val) == 3:
 *                 h, s, v = val             # <<<<<<<<<<<<<<
 *                 a = 0.0
 *             else:
 */
    if ((likely(PyTuple_CheckExact(__pyx_v_val))) || (PyList_CheckExact(__pyx_v_val))) {
      PyObject* sequence = __pyx_v_val;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 309, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_5 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      #endif
    } else {
      Py_ssize_t index = -1;
      __pyx_t_6 = PyObject_GetIter(__pyx_v_val); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = Py_TYPE(__pyx_t_6)->tp_iternext;
      index = 0; __pyx_t_3 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_3)) goto __pyx_L4_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 1; __pyx_t_4 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_4)) goto __pyx_L4_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      index = 2; __pyx_t_5 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_5)) goto __pyx_L4_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_6), 3) < 0) __PYX_ERR(0, 309, __pyx_L1_error)
      __pyx_t_7 = NULL;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L5_unpacking_done;
      __pyx_L4_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 309, __pyx_L1_error)
      __pyx_L5_unpacking_done:;
    }
    __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_9 = __pyx_PyFloat_AsDouble(__pyx_t_4); if (unlikely((__pyx_t_9 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_10 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_10 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_h = __pyx_t_8;
    __pyx_v_s = __pyx_t_9;
    __pyx_v_v = __pyx_t_10;

    /* "pygame_sdl2/color.pyx":310
 *             if len(val) == 3:
 *                 h, s, v = val
 *                 a = 0.0             # <<<<<<<<<<<<<<
 *             else:
 *                 h, s, v, a = val
 */
    __pyx_v_a = 0.0;

    /* "pygame_sdl2/color.pyx":308
 *         def __set__(self, val):
 *             cdef double h, s, v, a
 *             if len(val) == 3:             # <<<<<<<<<<<<<<
 *                 h, s, v = val
 *                 a = 0.0
 */
    goto __pyx_L3;
  }

  /* "pygame_sdl2/color.pyx":312
 *                 a = 0.0
 *             else:
 *                 h, s, v, a = val             # <<<<<<<<<<<<<<
 * 
 *             h = h % 360.0
 */
  /*else*/ {
    if ((likely(PyTuple_CheckExact(__pyx_v_val))) || (PyList_CheckExact(__pyx_v_val))) {
      PyObject* sequence = __pyx_v_val;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 4)) {
        if (size > 4) __Pyx_RaiseTooManyValuesError(4);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 312, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 3); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 3); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[4] = {&__pyx_t_5,&__pyx_t_4,&__pyx_t_3,&__pyx_t_6};
        for (i=0; i < 4; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 312, __pyx_L1_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
    } else {
      Py_ssize_t index = -1;
      PyObject** temps[4] = {&__pyx_t_5,&__pyx_t_4,&__pyx_t_3,&__pyx_t_6};
      __pyx_t_11 = PyObject_GetIter(__pyx_v_val); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 312, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_7 = Py_TYPE(__pyx_t_11)->tp_iternext;
      for (index=0; index < 4; index++) {
        PyObject* item = __pyx_t_7(__pyx_t_11); if (unlikely(!item)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(item);
        *(temps[index]) = item;
      }
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_11), 4) < 0) __PYX_ERR(0, 312, __pyx_L1_error)
      __pyx_t_7 = NULL;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L7_unpacking_done;
      __pyx_L6_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_7 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 312, __pyx_L1_error)
      __pyx_L7_unpacking_done:;
    }
    __pyx_t_10 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_10 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_9 = __pyx_PyFloat_AsDouble(__pyx_t_4); if (unlikely((__pyx_t_9 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_12 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_12 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_h = __pyx_t_10;
    __pyx_v_s = __pyx_t_9;
    __pyx_v_v = __pyx_t_8;
    __pyx_v_a = __pyx_t_12;
  }
  __pyx_L3:;

  /* "pygame_sdl2/color.pyx":314
 *                 h, s, v, a = val
 * 
 *             h = h % 360.0             # <<<<<<<<<<<<<<
 * 
 *             # These should be in a range of [0.0, 1.0]
 */
  __pyx_v_h = __Pyx_mod_double(__pyx_v_h, 360.0);

  /* "pygame_sdl2/color.pyx":317
 * 
 *             # These should be in a range of [0.0, 1.0]
 *             s /= 100.0             # <<<<<<<<<<<<<<
 *             v /= 100.0
 *             a /= 100.0
 */
  __pyx_v_s = (__pyx_v_s / 100.0);

  /* "pygame_sdl2/color.pyx":318
 *             # These should be in a range of [0.0, 1.0]
 *             s /= 100.0
 *             v /= 100.0             # <<<<<<<<<<<<<<
 *             a /= 100.0
 * 
 */
  __pyx_v_v = (__pyx_v_v / 100.0);

  /* "pygame_sdl2/color.pyx":319
 *             s /= 100.0
 *             v /= 100.0
 *             a /= 100.0             # <<<<<<<<<<<<<<
 * 
 *             cdef double c = v * s
 */
  __pyx_v_a = (__pyx_v_a / 100.0);

  /* "pygame_sdl2/color.pyx":321
 *             a /= 100.0
 * 
 *             cdef double c = v * s             # <<<<<<<<<<<<<<
 *             cdef double x = c * (1 - abs((h / 60.0) % 2 - 1))
 *             cdef double m = v - c
 */
  __pyx_v_c = (__pyx_v_v * __pyx_v_s);

  /* "pygame_sdl2/color.pyx":322
 * 
 *             cdef double c = v * s
 *             cdef double x = c * (1 - abs((h / 60.0) % 2 - 1))             # <<<<<<<<<<<<<<
 *             cdef double m = v - c
 * 
 */
  __pyx_v_x = (__pyx_v_c * (1.0 - fabs((__Pyx_mod_double((__pyx_v_h / 60.0), 2.0) - 1.0))));

  /* "pygame_sdl2/color.pyx":323
 *             cdef double c = v * s
 *             cdef double x = c * (1 - abs((h / 60.0) % 2 - 1))
 *             cdef double m = v - c             # <<<<<<<<<<<<<<
 * 
 *             cdef double r, g, b
 */
  __pyx_v_m = (__pyx_v_v - __pyx_v_c);

  /* "pygame_sdl2/color.pyx":327
 *             cdef double r, g, b
 * 
 *             if 0 <= h < 60:             # <<<<<<<<<<<<<<
 *                 r, g, b = c, x, 0
 *             elif 60 <= h < 120:
 */
  __pyx_t_2 = (0.0 <= __pyx_v_h);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_h < 60.0);
  }
  __pyx_t_13 = (__pyx_t_2 != 0);
  if (__pyx_t_13) {

    /* "pygame_sdl2/color.pyx":328
 * 
 *             if 0 <= h < 60:
 *                 r, g, b = c, x, 0             # <<<<<<<<<<<<<<
 *             elif 60 <= h < 120:
 *                 r, g, b = x, c, 0
 */
    __pyx_t_12 = __pyx_v_c;
    __pyx_t_8 = __pyx_v_x;
    __pyx_t_9 = 0.0;
    __pyx_v_r = __pyx_t_12;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_9;

    /* "pygame_sdl2/color.pyx":327
 *             cdef double r, g, b
 * 
 *             if 0 <= h < 60:             # <<<<<<<<<<<<<<
 *                 r, g, b = c, x, 0
 *             elif 60 <= h < 120:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":329
 *             if 0 <= h < 60:
 *                 r, g, b = c, x, 0
 *             elif 60 <= h < 120:             # <<<<<<<<<<<<<<
 *                 r, g, b = x, c, 0
 *             elif 120 <= h < 180:
 */
  __pyx_t_13 = (60.0 <= __pyx_v_h);
  if (__pyx_t_13) {
    __pyx_t_13 = (__pyx_v_h < 120.0);
  }
  __pyx_t_2 = (__pyx_t_13 != 0);
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":330
 *                 r, g, b = c, x, 0
 *             elif 60 <= h < 120:
 *                 r, g, b = x, c, 0             # <<<<<<<<<<<<<<
 *             elif 120 <= h < 180:
 *                 r, g, b = 0, c, x
 */
    __pyx_t_9 = __pyx_v_x;
    __pyx_t_8 = __pyx_v_c;
    __pyx_t_12 = 0.0;
    __pyx_v_r = __pyx_t_9;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_12;

    /* "pygame_sdl2/color.pyx":329
 *             if 0 <= h < 60:
 *                 r, g, b = c, x, 0
 *             elif 60 <= h < 120:             # <<<<<<<<<<<<<<
 *                 r, g, b = x, c, 0
 *             elif 120 <= h < 180:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":331
 *             elif 60 <= h < 120:
 *                 r, g, b = x, c, 0
 *             elif 120 <= h < 180:             # <<<<<<<<<<<<<<
 *                 r, g, b = 0, c, x
 *             elif 180 <= h < 240:
 */
  __pyx_t_2 = (120.0 <= __pyx_v_h);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_h < 180.0);
  }
  __pyx_t_13 = (__pyx_t_2 != 0);
  if (__pyx_t_13) {

    /* "pygame_sdl2/color.pyx":332
 *                 r, g, b = x, c, 0
 *             elif 120 <= h < 180:
 *                 r, g, b = 0, c, x             # <<<<<<<<<<<<<<
 *             elif 180 <= h < 240:
 *                 r, g, b = 0, x, c
 */
    __pyx_t_12 = 0.0;
    __pyx_t_8 = __pyx_v_c;
    __pyx_t_9 = __pyx_v_x;
    __pyx_v_r = __pyx_t_12;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_9;

    /* "pygame_sdl2/color.pyx":331
 *             elif 60 <= h < 120:
 *                 r, g, b = x, c, 0
 *             elif 120 <= h < 180:             # <<<<<<<<<<<<<<
 *                 r, g, b = 0, c, x
 *             elif 180 <= h < 240:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":333
 *             elif 120 <= h < 180:
 *                 r, g, b = 0, c, x
 *             elif 180 <= h < 240:             # <<<<<<<<<<<<<<
 *                 r, g, b = 0, x, c
 *             elif 240 <= h < 300:
 */
  __pyx_t_13 = (180.0 <= __pyx_v_h);
  if (__pyx_t_13) {
    __pyx_t_13 = (__pyx_v_h < 240.0);
  }
  __pyx_t_2 = (__pyx_t_13 != 0);
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":334
 *                 r, g, b = 0, c, x
 *             elif 180 <= h < 240:
 *                 r, g, b = 0, x, c             # <<<<<<<<<<<<<<
 *             elif 240 <= h < 300:
 *                 r, g, b = x, 0, c
 */
    __pyx_t_9 = 0.0;
    __pyx_t_8 = __pyx_v_x;
    __pyx_t_12 = __pyx_v_c;
    __pyx_v_r = __pyx_t_9;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_12;

    /* "pygame_sdl2/color.pyx":333
 *             elif 120 <= h < 180:
 *                 r, g, b = 0, c, x
 *             elif 180 <= h < 240:             # <<<<<<<<<<<<<<
 *                 r, g, b = 0, x, c
 *             elif 240 <= h < 300:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":335
 *             elif 180 <= h < 240:
 *                 r, g, b = 0, x, c
 *             elif 240 <= h < 300:             # <<<<<<<<<<<<<<
 *                 r, g, b = x, 0, c
 *             elif 300 <= h < 360:
 */
  __pyx_t_2 = (240.0 <= __pyx_v_h);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_h < 300.0);
  }
  __pyx_t_13 = (__pyx_t_2 != 0);
  if (__pyx_t_13) {

    /* "pygame_sdl2/color.pyx":336
 *                 r, g, b = 0, x, c
 *             elif 240 <= h < 300:
 *                 r, g, b = x, 0, c             # <<<<<<<<<<<<<<
 *             elif 300 <= h < 360:
 *                 r, g, b = c, 0, x
 */
    __pyx_t_12 = __pyx_v_x;
    __pyx_t_8 = 0.0;
    __pyx_t_9 = __pyx_v_c;
    __pyx_v_r = __pyx_t_12;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_9;

    /* "pygame_sdl2/color.pyx":335
 *             elif 180 <= h < 240:
 *                 r, g, b = 0, x, c
 *             elif 240 <= h < 300:             # <<<<<<<<<<<<<<
 *                 r, g, b = x, 0, c
 *             elif 300 <= h < 360:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":337
 *             elif 240 <= h < 300:
 *                 r, g, b = x, 0, c
 *             elif 300 <= h < 360:             # <<<<<<<<<<<<<<
 *                 r, g, b = c, 0, x
 *             else:
 */
  __pyx_t_13 = (300.0 <= __pyx_v_h);
  if (__pyx_t_13) {
    __pyx_t_13 = (__pyx_v_h < 360.0);
  }
  __pyx_t_2 = (__pyx_t_13 != 0);
  if (likely(__pyx_t_2)) {

    /* "pygame_sdl2/color.pyx":338
 *                 r, g, b = x, 0, c
 *             elif 300 <= h < 360:
 *                 r, g, b = c, 0, x             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError()
 */
    __pyx_t_9 = __pyx_v_c;
    __pyx_t_8 = 0.0;
    __pyx_t_12 = __pyx_v_x;
    __pyx_v_r = __pyx_t_9;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_12;

    /* "pygame_sdl2/color.pyx":337
 *             elif 240 <= h < 300:
 *                 r, g, b = x, 0, c
 *             elif 300 <= h < 360:             # <<<<<<<<<<<<<<
 *                 r, g, b = c, 0, x
 *             else:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":340
 *                 r, g, b = c, 0, x
 *             else:
 *                 raise ValueError()             # <<<<<<<<<<<<<<
 * 
 *             self.r = int(255 * (r + m))
 */
  /*else*/ {
    __pyx_t_6 = __Pyx_PyObject_CallNoArg(__pyx_builtin_ValueError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 340, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 340, __pyx_L1_error)
  }
  __pyx_L8:;

  /* "pygame_sdl2/color.pyx":342
 *                 raise ValueError()
 * 
 *             self.r = int(255 * (r + m))             # <<<<<<<<<<<<<<
 *             self.g = int(255 * (g + m))
 *             self.b = int(255 * (b + m))
 */
  __pyx_v_self->r = ((Uint8)(255.0 * (__pyx_v_r + __pyx_v_m)));

  /* "pygame_sdl2/color.pyx":343
 * 
 *             self.r = int(255 * (r + m))
 *             self.g = int(255 * (g + m))             # <<<<<<<<<<<<<<
 *             self.b = int(255 * (b + m))
 *             self.a = int(255 * a)
 */
  __pyx_v_self->g = ((Uint8)(255.0 * (__pyx_v_g + __pyx_v_m)));

  /* "pygame_sdl2/color.pyx":344
 *             self.r = int(255 * (r + m))
 *             self.g = int(255 * (g + m))
 *             self.b = int(255 * (b + m))             # <<<<<<<<<<<<<<
 *             self.a = int(255 * a)
 * 
 */
  __pyx_v_self->b = ((Uint8)(255.0 * (__pyx_v_b + __pyx_v_m)));

  /* "pygame_sdl2/color.pyx":345
 *             self.g = int(255 * (g + m))
 *             self.b = int(255 * (b + m))
 *             self.a = int(255 * a)             # <<<<<<<<<<<<<<
 * 
 *     property hsla:
 */
  __pyx_v_self->a = ((Uint8)(255.0 * __pyx_v_a));

  /* "pygame_sdl2/color.pyx":306
 *             return h, s, v, a
 * 
 *         def __set__(self, val):             # <<<<<<<<<<<<<<
 *             cdef double h, s, v, a
 *             if len(val) == 3:
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.hsva.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":348
 * 
 *     property hsla:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef double h, s, l, a
 *             cdef double r, g, b
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_4hsla_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_4hsla_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_4hsla___get__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_4hsla___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  double __pyx_v_h;
  double __pyx_v_s;
  double __pyx_v_l;
  double __pyx_v_a;
  double __pyx_v_r;
  double __pyx_v_g;
  double __pyx_v_b;
  double __pyx_v_cmin;
  double __pyx_v_cmax;
  double __pyx_v_delta;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  double __pyx_t_3;
  double __pyx_t_4;
  double __pyx_t_5;
  double __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pygame_sdl2/color.pyx":353
 *             cdef double cmin, cmax, delta
 * 
 *             h = self.hsva[0] % 360.0             # <<<<<<<<<<<<<<
 * 
 *             r = self.r / 255.0
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_hsva); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyFloat_RemainderObjC(__pyx_t_2, __pyx_float_360_0, 360.0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 353, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_h = __pyx_t_3;

  /* "pygame_sdl2/color.pyx":355
 *             h = self.hsva[0] % 360.0
 * 
 *             r = self.r / 255.0             # <<<<<<<<<<<<<<
 *             g = self.g / 255.0
 *             b = self.b / 255.0
 */
  __pyx_v_r = (((double)__pyx_v_self->r) / 255.0);

  /* "pygame_sdl2/color.pyx":356
 * 
 *             r = self.r / 255.0
 *             g = self.g / 255.0             # <<<<<<<<<<<<<<
 *             b = self.b / 255.0
 * 
 */
  __pyx_v_g = (((double)__pyx_v_self->g) / 255.0);

  /* "pygame_sdl2/color.pyx":357
 *             r = self.r / 255.0
 *             g = self.g / 255.0
 *             b = self.b / 255.0             # <<<<<<<<<<<<<<
 * 
 *             cmin = min(r, g, b)
 */
  __pyx_v_b = (((double)__pyx_v_self->b) / 255.0);

  /* "pygame_sdl2/color.pyx":359
 *             b = self.b / 255.0
 * 
 *             cmin = min(r, g, b)             # <<<<<<<<<<<<<<
 *             cmax = max(r, g, b)
 *             delta = cmax - cmin
 */
  __pyx_t_3 = __pyx_v_g;
  __pyx_t_4 = __pyx_v_b;
  __pyx_t_5 = __pyx_v_r;
  if (((__pyx_t_3 < __pyx_t_5) != 0)) {
    __pyx_t_6 = __pyx_t_3;
  } else {
    __pyx_t_6 = __pyx_t_5;
  }
  __pyx_t_5 = __pyx_t_6;
  if (((__pyx_t_4 < __pyx_t_5) != 0)) {
    __pyx_t_6 = __pyx_t_4;
  } else {
    __pyx_t_6 = __pyx_t_5;
  }
  __pyx_v_cmin = __pyx_t_6;

  /* "pygame_sdl2/color.pyx":360
 * 
 *             cmin = min(r, g, b)
 *             cmax = max(r, g, b)             # <<<<<<<<<<<<<<
 *             delta = cmax - cmin
 * 
 */
  __pyx_t_6 = __pyx_v_g;
  __pyx_t_3 = __pyx_v_b;
  __pyx_t_4 = __pyx_v_r;
  if (((__pyx_t_6 > __pyx_t_4) != 0)) {
    __pyx_t_5 = __pyx_t_6;
  } else {
    __pyx_t_5 = __pyx_t_4;
  }
  __pyx_t_4 = __pyx_t_5;
  if (((__pyx_t_3 > __pyx_t_4) != 0)) {
    __pyx_t_5 = __pyx_t_3;
  } else {
    __pyx_t_5 = __pyx_t_4;
  }
  __pyx_v_cmax = __pyx_t_5;

  /* "pygame_sdl2/color.pyx":361
 *             cmin = min(r, g, b)
 *             cmax = max(r, g, b)
 *             delta = cmax - cmin             # <<<<<<<<<<<<<<
 * 
 *             l = (cmax + cmin) / 2.0
 */
  __pyx_v_delta = (__pyx_v_cmax - __pyx_v_cmin);

  /* "pygame_sdl2/color.pyx":363
 *             delta = cmax - cmin
 * 
 *             l = (cmax + cmin) / 2.0             # <<<<<<<<<<<<<<
 * 
 *             if delta == 0:
 */
  __pyx_v_l = ((__pyx_v_cmax + __pyx_v_cmin) / 2.0);

  /* "pygame_sdl2/color.pyx":365
 *             l = (cmax + cmin) / 2.0
 * 
 *             if delta == 0:             # <<<<<<<<<<<<<<
 *                 s = 0.0
 *             else:
 */
  __pyx_t_7 = ((__pyx_v_delta == 0.0) != 0);
  if (__pyx_t_7) {

    /* "pygame_sdl2/color.pyx":366
 * 
 *             if delta == 0:
 *                 s = 0.0             # <<<<<<<<<<<<<<
 *             else:
 *                 s = delta / (1 - abs(2 * l - 1))
 */
    __pyx_v_s = 0.0;

    /* "pygame_sdl2/color.pyx":365
 *             l = (cmax + cmin) / 2.0
 * 
 *             if delta == 0:             # <<<<<<<<<<<<<<
 *                 s = 0.0
 *             else:
 */
    goto __pyx_L3;
  }

  /* "pygame_sdl2/color.pyx":368
 *                 s = 0.0
 *             else:
 *                 s = delta / (1 - abs(2 * l - 1))             # <<<<<<<<<<<<<<
 * 
 *             a = self.a / 255.0 * 100
 */
  /*else*/ {
    __pyx_t_5 = (1.0 - fabs(((2.0 * __pyx_v_l) - 1.0)));
    if (unlikely(__pyx_t_5 == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 368, __pyx_L1_error)
    }
    __pyx_v_s = (__pyx_v_delta / __pyx_t_5);
  }
  __pyx_L3:;

  /* "pygame_sdl2/color.pyx":370
 *                 s = delta / (1 - abs(2 * l - 1))
 * 
 *             a = self.a / 255.0 * 100             # <<<<<<<<<<<<<<
 * 
 *             s = min(100.0, s * 100)
 */
  __pyx_v_a = ((((double)__pyx_v_self->a) / 255.0) * 100.0);

  /* "pygame_sdl2/color.pyx":372
 *             a = self.a / 255.0 * 100
 * 
 *             s = min(100.0, s * 100)             # <<<<<<<<<<<<<<
 *             l = min(100.0, l * 100)
 * 
 */
  __pyx_t_5 = (__pyx_v_s * 100.0);
  __pyx_t_6 = 100.0;
  if (((__pyx_t_5 < __pyx_t_6) != 0)) {
    __pyx_t_3 = __pyx_t_5;
  } else {
    __pyx_t_3 = __pyx_t_6;
  }
  __pyx_v_s = __pyx_t_3;

  /* "pygame_sdl2/color.pyx":373
 * 
 *             s = min(100.0, s * 100)
 *             l = min(100.0, l * 100)             # <<<<<<<<<<<<<<
 * 
 *             return h, s, l, a
 */
  __pyx_t_3 = (__pyx_v_l * 100.0);
  __pyx_t_5 = 100.0;
  if (((__pyx_t_3 < __pyx_t_5) != 0)) {
    __pyx_t_6 = __pyx_t_3;
  } else {
    __pyx_t_6 = __pyx_t_5;
  }
  __pyx_v_l = __pyx_t_6;

  /* "pygame_sdl2/color.pyx":375
 *             l = min(100.0, l * 100)
 * 
 *             return h, s, l, a             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, val):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_h); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_s); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = PyFloat_FromDouble(__pyx_v_l); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyFloat_FromDouble(__pyx_v_a); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = PyTuple_New(4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_10, 3, __pyx_t_9);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_r = __pyx_t_10;
  __pyx_t_10 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":348
 * 
 *     property hsla:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef double h, s, l, a
 *             cdef double r, g, b
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.hsla.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":377
 *             return h, s, l, a
 * 
 *         def __set__(self, val):             # <<<<<<<<<<<<<<
 *             cdef double h, s, l, a
 *             if len(val) == 3:
 */

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_4hsla_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_val); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_4hsla_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_val) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_4hsla_2__set__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_val));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_4hsla_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_val) {
  double __pyx_v_h;
  double __pyx_v_s;
  double __pyx_v_l;
  double __pyx_v_a;
  double __pyx_v_c;
  double __pyx_v_x;
  double __pyx_v_m;
  double __pyx_v_r;
  double __pyx_v_g;
  double __pyx_v_b;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *(*__pyx_t_7)(PyObject *);
  double __pyx_t_8;
  double __pyx_t_9;
  double __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  double __pyx_t_12;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pygame_sdl2/color.pyx":379
 *         def __set__(self, val):
 *             cdef double h, s, l, a
 *             if len(val) == 3:             # <<<<<<<<<<<<<<
 *                 h, s, l = val
 *                 a = 0.0
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_val); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 379, __pyx_L1_error)
  __pyx_t_2 = ((__pyx_t_1 == 3) != 0);
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":380
 *             cdef double h, s, l, a
 *             if len(val) == 3:
 *                 h, s, l = val             # <<<<<<<<<<<<<<
 *                 a = 0.0
 *             else:
 */
    if ((likely(PyTuple_CheckExact(__pyx_v_val))) || (PyList_CheckExact(__pyx_v_val))) {
      PyObject* sequence = __pyx_v_val;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 380, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_5 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      #endif
    } else {
      Py_ssize_t index = -1;
      __pyx_t_6 = PyObject_GetIter(__pyx_v_val); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = Py_TYPE(__pyx_t_6)->tp_iternext;
      index = 0; __pyx_t_3 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_3)) goto __pyx_L4_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 1; __pyx_t_4 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_4)) goto __pyx_L4_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      index = 2; __pyx_t_5 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_5)) goto __pyx_L4_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_6), 3) < 0) __PYX_ERR(0, 380, __pyx_L1_error)
      __pyx_t_7 = NULL;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L5_unpacking_done;
      __pyx_L4_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 380, __pyx_L1_error)
      __pyx_L5_unpacking_done:;
    }
    __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_9 = __pyx_PyFloat_AsDouble(__pyx_t_4); if (unlikely((__pyx_t_9 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_10 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_10 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_h = __pyx_t_8;
    __pyx_v_s = __pyx_t_9;
    __pyx_v_l = __pyx_t_10;

    /* "pygame_sdl2/color.pyx":381
 *             if len(val) == 3:
 *                 h, s, l = val
 *                 a = 0.0             # <<<<<<<<<<<<<<
 *             else:
 *                 h, s, l, a = val
 */
    __pyx_v_a = 0.0;

    /* "pygame_sdl2/color.pyx":379
 *         def __set__(self, val):
 *             cdef double h, s, l, a
 *             if len(val) == 3:             # <<<<<<<<<<<<<<
 *                 h, s, l = val
 *                 a = 0.0
 */
    goto __pyx_L3;
  }

  /* "pygame_sdl2/color.pyx":383
 *                 a = 0.0
 *             else:
 *                 h, s, l, a = val             # <<<<<<<<<<<<<<
 * 
 *             s /= 100.0
 */
  /*else*/ {
    if ((likely(PyTuple_CheckExact(__pyx_v_val))) || (PyList_CheckExact(__pyx_v_val))) {
      PyObject* sequence = __pyx_v_val;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 4)) {
        if (size > 4) __Pyx_RaiseTooManyValuesError(4);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 383, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 3); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 3); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[4] = {&__pyx_t_5,&__pyx_t_4,&__pyx_t_3,&__pyx_t_6};
        for (i=0; i < 4; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 383, __pyx_L1_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
    } else {
      Py_ssize_t index = -1;
      PyObject** temps[4] = {&__pyx_t_5,&__pyx_t_4,&__pyx_t_3,&__pyx_t_6};
      __pyx_t_11 = PyObject_GetIter(__pyx_v_val); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_7 = Py_TYPE(__pyx_t_11)->tp_iternext;
      for (index=0; index < 4; index++) {
        PyObject* item = __pyx_t_7(__pyx_t_11); if (unlikely(!item)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(item);
        *(temps[index]) = item;
      }
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_11), 4) < 0) __PYX_ERR(0, 383, __pyx_L1_error)
      __pyx_t_7 = NULL;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L7_unpacking_done;
      __pyx_L6_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_7 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 383, __pyx_L1_error)
      __pyx_L7_unpacking_done:;
    }
    __pyx_t_10 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_10 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_9 = __pyx_PyFloat_AsDouble(__pyx_t_4); if (unlikely((__pyx_t_9 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_12 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_12 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_h = __pyx_t_10;
    __pyx_v_s = __pyx_t_9;
    __pyx_v_l = __pyx_t_8;
    __pyx_v_a = __pyx_t_12;
  }
  __pyx_L3:;

  /* "pygame_sdl2/color.pyx":385
 *                 h, s, l, a = val
 * 
 *             s /= 100.0             # <<<<<<<<<<<<<<
 *             l /= 100.0
 *             a /= 100.0
 */
  __pyx_v_s = (__pyx_v_s / 100.0);

  /* "pygame_sdl2/color.pyx":386
 * 
 *             s /= 100.0
 *             l /= 100.0             # <<<<<<<<<<<<<<
 *             a /= 100.0
 * 
 */
  __pyx_v_l = (__pyx_v_l / 100.0);

  /* "pygame_sdl2/color.pyx":387
 *             s /= 100.0
 *             l /= 100.0
 *             a /= 100.0             # <<<<<<<<<<<<<<
 * 
 *             cdef double c = (1 - abs(2*l - 1)) * s
 */
  __pyx_v_a = (__pyx_v_a / 100.0);

  /* "pygame_sdl2/color.pyx":389
 *             a /= 100.0
 * 
 *             cdef double c = (1 - abs(2*l - 1)) * s             # <<<<<<<<<<<<<<
 *             cdef double x = c * (1 - abs((h / 60.0) % 2 - 1))
 *             cdef double m = l - c / 2.0
 */
  __pyx_v_c = ((1.0 - fabs(((2.0 * __pyx_v_l) - 1.0))) * __pyx_v_s);

  /* "pygame_sdl2/color.pyx":390
 * 
 *             cdef double c = (1 - abs(2*l - 1)) * s
 *             cdef double x = c * (1 - abs((h / 60.0) % 2 - 1))             # <<<<<<<<<<<<<<
 *             cdef double m = l - c / 2.0
 * 
 */
  __pyx_v_x = (__pyx_v_c * (1.0 - fabs((__Pyx_mod_double((__pyx_v_h / 60.0), 2.0) - 1.0))));

  /* "pygame_sdl2/color.pyx":391
 *             cdef double c = (1 - abs(2*l - 1)) * s
 *             cdef double x = c * (1 - abs((h / 60.0) % 2 - 1))
 *             cdef double m = l - c / 2.0             # <<<<<<<<<<<<<<
 * 
 *             cdef double r, g, b
 */
  __pyx_v_m = (__pyx_v_l - (__pyx_v_c / 2.0));

  /* "pygame_sdl2/color.pyx":394
 * 
 *             cdef double r, g, b
 *             if 0 <= h < 60:             # <<<<<<<<<<<<<<
 *                 r, g, b = c, x, 0
 *             elif 60 <= h < 120:
 */
  __pyx_t_2 = (0.0 <= __pyx_v_h);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_h < 60.0);
  }
  __pyx_t_13 = (__pyx_t_2 != 0);
  if (__pyx_t_13) {

    /* "pygame_sdl2/color.pyx":395
 *             cdef double r, g, b
 *             if 0 <= h < 60:
 *                 r, g, b = c, x, 0             # <<<<<<<<<<<<<<
 *             elif 60 <= h < 120:
 *                 r, g, b = x, c, 0
 */
    __pyx_t_12 = __pyx_v_c;
    __pyx_t_8 = __pyx_v_x;
    __pyx_t_9 = 0.0;
    __pyx_v_r = __pyx_t_12;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_9;

    /* "pygame_sdl2/color.pyx":394
 * 
 *             cdef double r, g, b
 *             if 0 <= h < 60:             # <<<<<<<<<<<<<<
 *                 r, g, b = c, x, 0
 *             elif 60 <= h < 120:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":396
 *             if 0 <= h < 60:
 *                 r, g, b = c, x, 0
 *             elif 60 <= h < 120:             # <<<<<<<<<<<<<<
 *                 r, g, b = x, c, 0
 *             elif 120 <= h < 180:
 */
  __pyx_t_13 = (60.0 <= __pyx_v_h);
  if (__pyx_t_13) {
    __pyx_t_13 = (__pyx_v_h < 120.0);
  }
  __pyx_t_2 = (__pyx_t_13 != 0);
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":397
 *                 r, g, b = c, x, 0
 *             elif 60 <= h < 120:
 *                 r, g, b = x, c, 0             # <<<<<<<<<<<<<<
 *             elif 120 <= h < 180:
 *                 r, g, b = 0, c, x
 */
    __pyx_t_9 = __pyx_v_x;
    __pyx_t_8 = __pyx_v_c;
    __pyx_t_12 = 0.0;
    __pyx_v_r = __pyx_t_9;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_12;

    /* "pygame_sdl2/color.pyx":396
 *             if 0 <= h < 60:
 *                 r, g, b = c, x, 0
 *             elif 60 <= h < 120:             # <<<<<<<<<<<<<<
 *                 r, g, b = x, c, 0
 *             elif 120 <= h < 180:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":398
 *             elif 60 <= h < 120:
 *                 r, g, b = x, c, 0
 *             elif 120 <= h < 180:             # <<<<<<<<<<<<<<
 *                 r, g, b = 0, c, x
 *             elif 180 <= h < 240:
 */
  __pyx_t_2 = (120.0 <= __pyx_v_h);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_h < 180.0);
  }
  __pyx_t_13 = (__pyx_t_2 != 0);
  if (__pyx_t_13) {

    /* "pygame_sdl2/color.pyx":399
 *                 r, g, b = x, c, 0
 *             elif 120 <= h < 180:
 *                 r, g, b = 0, c, x             # <<<<<<<<<<<<<<
 *             elif 180 <= h < 240:
 *                 r, g, b = 0, x, c
 */
    __pyx_t_12 = 0.0;
    __pyx_t_8 = __pyx_v_c;
    __pyx_t_9 = __pyx_v_x;
    __pyx_v_r = __pyx_t_12;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_9;

    /* "pygame_sdl2/color.pyx":398
 *             elif 60 <= h < 120:
 *                 r, g, b = x, c, 0
 *             elif 120 <= h < 180:             # <<<<<<<<<<<<<<
 *                 r, g, b = 0, c, x
 *             elif 180 <= h < 240:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":400
 *             elif 120 <= h < 180:
 *                 r, g, b = 0, c, x
 *             elif 180 <= h < 240:             # <<<<<<<<<<<<<<
 *                 r, g, b = 0, x, c
 *             elif 240 <= h < 300:
 */
  __pyx_t_13 = (180.0 <= __pyx_v_h);
  if (__pyx_t_13) {
    __pyx_t_13 = (__pyx_v_h < 240.0);
  }
  __pyx_t_2 = (__pyx_t_13 != 0);
  if (__pyx_t_2) {

    /* "pygame_sdl2/color.pyx":401
 *                 r, g, b = 0, c, x
 *             elif 180 <= h < 240:
 *                 r, g, b = 0, x, c             # <<<<<<<<<<<<<<
 *             elif 240 <= h < 300:
 *                 r, g, b = x, 0, c
 */
    __pyx_t_9 = 0.0;
    __pyx_t_8 = __pyx_v_x;
    __pyx_t_12 = __pyx_v_c;
    __pyx_v_r = __pyx_t_9;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_12;

    /* "pygame_sdl2/color.pyx":400
 *             elif 120 <= h < 180:
 *                 r, g, b = 0, c, x
 *             elif 180 <= h < 240:             # <<<<<<<<<<<<<<
 *                 r, g, b = 0, x, c
 *             elif 240 <= h < 300:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":402
 *             elif 180 <= h < 240:
 *                 r, g, b = 0, x, c
 *             elif 240 <= h < 300:             # <<<<<<<<<<<<<<
 *                 r, g, b = x, 0, c
 *             elif 300 <= h < 360:
 */
  __pyx_t_2 = (240.0 <= __pyx_v_h);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_h < 300.0);
  }
  __pyx_t_13 = (__pyx_t_2 != 0);
  if (__pyx_t_13) {

    /* "pygame_sdl2/color.pyx":403
 *                 r, g, b = 0, x, c
 *             elif 240 <= h < 300:
 *                 r, g, b = x, 0, c             # <<<<<<<<<<<<<<
 *             elif 300 <= h < 360:
 *                 r, g, b = c, 0, x
 */
    __pyx_t_12 = __pyx_v_x;
    __pyx_t_8 = 0.0;
    __pyx_t_9 = __pyx_v_c;
    __pyx_v_r = __pyx_t_12;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_9;

    /* "pygame_sdl2/color.pyx":402
 *             elif 180 <= h < 240:
 *                 r, g, b = 0, x, c
 *             elif 240 <= h < 300:             # <<<<<<<<<<<<<<
 *                 r, g, b = x, 0, c
 *             elif 300 <= h < 360:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":404
 *             elif 240 <= h < 300:
 *                 r, g, b = x, 0, c
 *             elif 300 <= h < 360:             # <<<<<<<<<<<<<<
 *                 r, g, b = c, 0, x
 *             else:
 */
  __pyx_t_13 = (300.0 <= __pyx_v_h);
  if (__pyx_t_13) {
    __pyx_t_13 = (__pyx_v_h < 360.0);
  }
  __pyx_t_2 = (__pyx_t_13 != 0);
  if (likely(__pyx_t_2)) {

    /* "pygame_sdl2/color.pyx":405
 *                 r, g, b = x, 0, c
 *             elif 300 <= h < 360:
 *                 r, g, b = c, 0, x             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError()
 */
    __pyx_t_9 = __pyx_v_c;
    __pyx_t_8 = 0.0;
    __pyx_t_12 = __pyx_v_x;
    __pyx_v_r = __pyx_t_9;
    __pyx_v_g = __pyx_t_8;
    __pyx_v_b = __pyx_t_12;

    /* "pygame_sdl2/color.pyx":404
 *             elif 240 <= h < 300:
 *                 r, g, b = x, 0, c
 *             elif 300 <= h < 360:             # <<<<<<<<<<<<<<
 *                 r, g, b = c, 0, x
 *             else:
 */
    goto __pyx_L8;
  }

  /* "pygame_sdl2/color.pyx":407
 *                 r, g, b = c, 0, x
 *             else:
 *                 raise ValueError()             # <<<<<<<<<<<<<<
 * 
 *             self.r = int(255 * (r + m))
 */
  /*else*/ {
    __pyx_t_6 = __Pyx_PyObject_CallNoArg(__pyx_builtin_ValueError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 407, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 407, __pyx_L1_error)
  }
  __pyx_L8:;

  /* "pygame_sdl2/color.pyx":409
 *                 raise ValueError()
 * 
 *             self.r = int(255 * (r + m))             # <<<<<<<<<<<<<<
 *             self.g = int(255 * (g + m))
 *             self.b = int(255 * (b + m))
 */
  __pyx_v_self->r = ((Uint8)(255.0 * (__pyx_v_r + __pyx_v_m)));

  /* "pygame_sdl2/color.pyx":410
 * 
 *             self.r = int(255 * (r + m))
 *             self.g = int(255 * (g + m))             # <<<<<<<<<<<<<<
 *             self.b = int(255 * (b + m))
 *             self.a = int(255 * a)
 */
  __pyx_v_self->g = ((Uint8)(255.0 * (__pyx_v_g + __pyx_v_m)));

  /* "pygame_sdl2/color.pyx":411
 *             self.r = int(255 * (r + m))
 *             self.g = int(255 * (g + m))
 *             self.b = int(255 * (b + m))             # <<<<<<<<<<<<<<
 *             self.a = int(255 * a)
 * 
 */
  __pyx_v_self->b = ((Uint8)(255.0 * (__pyx_v_b + __pyx_v_m)));

  /* "pygame_sdl2/color.pyx":412
 *             self.g = int(255 * (g + m))
 *             self.b = int(255 * (b + m))
 *             self.a = int(255 * a)             # <<<<<<<<<<<<<<
 * 
 *     property i1i2i3:
 */
  __pyx_v_self->a = ((Uint8)(255.0 * __pyx_v_a));

  /* "pygame_sdl2/color.pyx":377
 *             return h, s, l, a
 * 
 *         def __set__(self, val):             # <<<<<<<<<<<<<<
 *             cdef double h, s, l, a
 *             if len(val) == 3:
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.hsla.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":415
 * 
 *     property i1i2i3:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             # Take the dot product as described here:
 *             # http://de.wikipedia.org/wiki/I1I2I3-Farbraum
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_6i1i2i3_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_6i1i2i3_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_6i1i2i3___get__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_6i1i2i3___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  double __pyx_v_i1;
  double __pyx_v_i2;
  double __pyx_v_i3;
  double __pyx_v_r;
  double __pyx_v_g;
  double __pyx_v_b;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pygame_sdl2/color.pyx":422
 *             cdef double r, g, b
 * 
 *             r = self.r / 255.0             # <<<<<<<<<<<<<<
 *             g = self.g / 255.0
 *             b = self.b / 255.0
 */
  __pyx_v_r = (((double)__pyx_v_self->r) / 255.0);

  /* "pygame_sdl2/color.pyx":423
 * 
 *             r = self.r / 255.0
 *             g = self.g / 255.0             # <<<<<<<<<<<<<<
 *             b = self.b / 255.0
 * 
 */
  __pyx_v_g = (((double)__pyx_v_self->g) / 255.0);

  /* "pygame_sdl2/color.pyx":424
 *             r = self.r / 255.0
 *             g = self.g / 255.0
 *             b = self.b / 255.0             # <<<<<<<<<<<<<<
 * 
 *             i1 = (r + g + b) / 3.0
 */
  __pyx_v_b = (((double)__pyx_v_self->b) / 255.0);

  /* "pygame_sdl2/color.pyx":426
 *             b = self.b / 255.0
 * 
 *             i1 = (r + g + b) / 3.0             # <<<<<<<<<<<<<<
 *             i2 = (r - b) / 2.0
 *             i3 = (2*g - r - b) / 4.0
 */
  __pyx_v_i1 = (((__pyx_v_r + __pyx_v_g) + __pyx_v_b) / 3.0);

  /* "pygame_sdl2/color.pyx":427
 * 
 *             i1 = (r + g + b) / 3.0
 *             i2 = (r - b) / 2.0             # <<<<<<<<<<<<<<
 *             i3 = (2*g - r - b) / 4.0
 * 
 */
  __pyx_v_i2 = ((__pyx_v_r - __pyx_v_b) / 2.0);

  /* "pygame_sdl2/color.pyx":428
 *             i1 = (r + g + b) / 3.0
 *             i2 = (r - b) / 2.0
 *             i3 = (2*g - r - b) / 4.0             # <<<<<<<<<<<<<<
 * 
 *             return i1, i2, i3
 */
  __pyx_v_i3 = ((((2.0 * __pyx_v_g) - __pyx_v_r) - __pyx_v_b) / 4.0);

  /* "pygame_sdl2/color.pyx":430
 *             i3 = (2*g - r - b) / 4.0
 * 
 *             return i1, i2, i3             # <<<<<<<<<<<<<<
 * 
 *         def __set__(self, val):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_i1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_i2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_i3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":415
 * 
 *     property i1i2i3:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             # Take the dot product as described here:
 *             # http://de.wikipedia.org/wiki/I1I2I3-Farbraum
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.i1i2i3.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":432
 *             return i1, i2, i3
 * 
 *         def __set__(self, val):             # <<<<<<<<<<<<<<
 *             # Dot product with the inverted matrix.
 * 
 */

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_6i1i2i3_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_val); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_6i1i2i3_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_val) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_6i1i2i3_2__set__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_val));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_6i1i2i3_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_val) {
  double __pyx_v_i1;
  double __pyx_v_i2;
  double __pyx_v_i3;
  double __pyx_v_r;
  double __pyx_v_g;
  double __pyx_v_b;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  double __pyx_t_6;
  double __pyx_t_7;
  double __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "pygame_sdl2/color.pyx":438
 *             cdef double r, g, b
 * 
 *             i1, i2, i3 = val             # <<<<<<<<<<<<<<
 * 
 *             r = i1 + i2 - (2.0/3.0 * i3)
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_val))) || (PyList_CheckExact(__pyx_v_val))) {
    PyObject* sequence = __pyx_v_val;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 3)) {
      if (size > 3) __Pyx_RaiseTooManyValuesError(3);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 438, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_v_val); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_1)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_2 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 2; __pyx_t_3 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_4), 3) < 0) __PYX_ERR(0, 438, __pyx_L1_error)
    __pyx_t_5 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 438, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_t_6 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_6 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_7 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_i1 = __pyx_t_6;
  __pyx_v_i2 = __pyx_t_7;
  __pyx_v_i3 = __pyx_t_8;

  /* "pygame_sdl2/color.pyx":440
 *             i1, i2, i3 = val
 * 
 *             r = i1 + i2 - (2.0/3.0 * i3)             # <<<<<<<<<<<<<<
 *             g = i1 + (4.0/3.0 * i3)
 *             b = i1 - i2  - (2.0/3.0 * i3)
 */
  __pyx_v_r = ((__pyx_v_i1 + __pyx_v_i2) - ((2.0 / 3.0) * __pyx_v_i3));

  /* "pygame_sdl2/color.pyx":441
 * 
 *             r = i1 + i2 - (2.0/3.0 * i3)
 *             g = i1 + (4.0/3.0 * i3)             # <<<<<<<<<<<<<<
 *             b = i1 - i2  - (2.0/3.0 * i3)
 * 
 */
  __pyx_v_g = (__pyx_v_i1 + ((4.0 / 3.0) * __pyx_v_i3));

  /* "pygame_sdl2/color.pyx":442
 *             r = i1 + i2 - (2.0/3.0 * i3)
 *             g = i1 + (4.0/3.0 * i3)
 *             b = i1 - i2  - (2.0/3.0 * i3)             # <<<<<<<<<<<<<<
 * 
 *             # Don't change alpha.
 */
  __pyx_v_b = ((__pyx_v_i1 - __pyx_v_i2) - ((2.0 / 3.0) * __pyx_v_i3));

  /* "pygame_sdl2/color.pyx":445
 * 
 *             # Don't change alpha.
 *             self.r = int(r * 255)             # <<<<<<<<<<<<<<
 *             self.g = int(g * 255)
 *             self.b = int(b * 255)
 */
  __pyx_v_self->r = ((Uint8)(__pyx_v_r * 255.0));

  /* "pygame_sdl2/color.pyx":446
 *             # Don't change alpha.
 *             self.r = int(r * 255)
 *             self.g = int(g * 255)             # <<<<<<<<<<<<<<
 *             self.b = int(b * 255)
 * 
 */
  __pyx_v_self->g = ((Uint8)(__pyx_v_g * 255.0));

  /* "pygame_sdl2/color.pyx":447
 *             self.r = int(r * 255)
 *             self.g = int(g * 255)
 *             self.b = int(b * 255)             # <<<<<<<<<<<<<<
 * 
 *     def normalize(self):
 */
  __pyx_v_self->b = ((Uint8)(__pyx_v_b * 255.0));

  /* "pygame_sdl2/color.pyx":432
 *             return i1, i2, i3
 * 
 *         def __set__(self, val):             # <<<<<<<<<<<<<<
 *             # Dot product with the inverted matrix.
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.i1i2i3.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":449
 *             self.b = int(b * 255)
 * 
 *     def normalize(self):             # <<<<<<<<<<<<<<
 *         return self.r / 255.0, self.g / 255.0, self.b / 255.0, self.a / 255.0
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_39normalize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_39normalize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("normalize (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_38normalize(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_38normalize(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("normalize", 0);

  /* "pygame_sdl2/color.pyx":450
 * 
 *     def normalize(self):
 *         return self.r / 255.0, self.g / 255.0, self.b / 255.0, self.a / 255.0             # <<<<<<<<<<<<<<
 * 
 *     def correct_gamma(self, gamma):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble((((double)__pyx_v_self->r) / 255.0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble((((double)__pyx_v_self->g) / 255.0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble((((double)__pyx_v_self->b) / 255.0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble((((double)__pyx_v_self->a) / 255.0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":449
 *             self.b = int(b * 255)
 * 
 *     def normalize(self):             # <<<<<<<<<<<<<<
 *         return self.r / 255.0, self.g / 255.0, self.b / 255.0, self.a / 255.0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.normalize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":452
 *         return self.r / 255.0, self.g / 255.0, self.b / 255.0, self.a / 255.0
 * 
 *     def correct_gamma(self, gamma):             # <<<<<<<<<<<<<<
 *         m = map(lambda x: int(round(pow(x / 255.0, gamma) * 255)), tuple(self))
 *         c = type(self)(tuple(m))
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_41correct_gamma(PyObject *__pyx_v_self, PyObject *__pyx_v_gamma); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_41correct_gamma(PyObject *__pyx_v_self, PyObject *__pyx_v_gamma) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("correct_gamma (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_40correct_gamma(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_gamma));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":453
 * 
 *     def correct_gamma(self, gamma):
 *         m = map(lambda x: int(round(pow(x / 255.0, gamma) * 255)), tuple(self))             # <<<<<<<<<<<<<<
 *         c = type(self)(tuple(m))
 *         return c
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_13correct_gamma_lambda1(PyObject *__pyx_self, PyObject *__pyx_v_x); /*proto*/
static PyMethodDef __pyx_mdef_11pygame_sdl2_5color_5Color_13correct_gamma_lambda1 = {"lambda1", (PyCFunction)__pyx_pw_11pygame_sdl2_5color_5Color_13correct_gamma_lambda1, METH_O, 0};
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_13correct_gamma_lambda1(PyObject *__pyx_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda1 (wrapper)", 0);
  __pyx_r = __pyx_lambda_funcdef_lambda1(__pyx_self, ((PyObject *)__pyx_v_x));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda1(PyObject *__pyx_self, PyObject *__pyx_v_x) {
  struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *__pyx_cur_scope;
  struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda1", 0);
  __pyx_outer_scope = (struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_x, __pyx_float_255_0, 255.0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(!__pyx_cur_scope->__pyx_v_gamma)) { __Pyx_RaiseClosureNameError("gamma"); __PYX_ERR(0, 453, __pyx_L1_error) }
  __pyx_t_2 = __pyx_cur_scope->__pyx_v_gamma;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyNumber_Power2(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_3, __pyx_int_255); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_round, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyNumber_Int(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.correct_gamma.lambda1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":452
 *         return self.r / 255.0, self.g / 255.0, self.b / 255.0, self.a / 255.0
 * 
 *     def correct_gamma(self, gamma):             # <<<<<<<<<<<<<<
 *         m = map(lambda x: int(round(pow(x / 255.0, gamma) * 255)), tuple(self))
 *         c = type(self)(tuple(m))
 */

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_40correct_gamma(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_gamma) {
  struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *__pyx_cur_scope;
  PyObject *__pyx_v_m = NULL;
  PyObject *__pyx_v_c = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("correct_gamma", 0);
  __pyx_cur_scope = (struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *)__pyx_tp_new_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma(__pyx_ptype_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 452, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_gamma = __pyx_v_gamma;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_gamma);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_gamma);

  /* "pygame_sdl2/color.pyx":453
 * 
 *     def correct_gamma(self, gamma):
 *         m = map(lambda x: int(round(pow(x / 255.0, gamma) * 255)), tuple(self))             # <<<<<<<<<<<<<<
 *         c = type(self)(tuple(m))
 *         return c
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_11pygame_sdl2_5color_5Color_13correct_gamma_lambda1, 0, __pyx_n_s_correct_gamma_locals_lambda, ((PyObject*)__pyx_cur_scope), __pyx_n_s_pygame_sdl2_color, __pyx_d, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PySequence_Tuple(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_map, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_m = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pygame_sdl2/color.pyx":454
 *     def correct_gamma(self, gamma):
 *         m = map(lambda x: int(round(pow(x / 255.0, gamma) * 255)), tuple(self))
 *         c = type(self)(tuple(m))             # <<<<<<<<<<<<<<
 *         return c
 * 
 */
  __pyx_t_3 = __Pyx_PySequence_Tuple(__pyx_v_m); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
  __pyx_t_1 = ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))); __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_c = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pygame_sdl2/color.pyx":455
 *         m = map(lambda x: int(round(pow(x / 255.0, gamma) * 255)), tuple(self))
 *         c = type(self)(tuple(m))
 *         return c             # <<<<<<<<<<<<<<
 * 
 *     def set_length(self, n):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_c);
  __pyx_r = __pyx_v_c;
  goto __pyx_L0;

  /* "pygame_sdl2/color.pyx":452
 *         return self.r / 255.0, self.g / 255.0, self.b / 255.0, self.a / 255.0
 * 
 *     def correct_gamma(self, gamma):             # <<<<<<<<<<<<<<
 *         m = map(lambda x: int(round(pow(x / 255.0, gamma) * 255)), tuple(self))
 *         c = type(self)(tuple(m))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.correct_gamma", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_m);
  __Pyx_XDECREF(__pyx_v_c);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pyx":457
 *         return c
 * 
 *     def set_length(self, n):             # <<<<<<<<<<<<<<
 *         if n > 4 or n < 1:
 *             raise ValueError(n)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_43set_length(PyObject *__pyx_v_self, PyObject *__pyx_v_n); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_43set_length(PyObject *__pyx_v_self, PyObject *__pyx_v_n) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_length (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_42set_length(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_n));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_42set_length(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_n) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  uint8_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_length", 0);

  /* "pygame_sdl2/color.pyx":458
 * 
 *     def set_length(self, n):
 *         if n > 4 or n < 1:             # <<<<<<<<<<<<<<
 *             raise ValueError(n)
 *         self.length = n
 */
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_n, __pyx_int_4, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 458, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_n, __pyx_int_1, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 458, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "pygame_sdl2/color.pyx":459
 *     def set_length(self, n):
 *         if n > 4 or n < 1:
 *             raise ValueError(n)             # <<<<<<<<<<<<<<
 *         self.length = n
 */
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_n); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 459, __pyx_L1_error)

    /* "pygame_sdl2/color.pyx":458
 * 
 *     def set_length(self, n):
 *         if n > 4 or n < 1:             # <<<<<<<<<<<<<<
 *             raise ValueError(n)
 *         self.length = n
 */
  }

  /* "pygame_sdl2/color.pyx":460
 *         if n > 4 or n < 1:
 *             raise ValueError(n)
 *         self.length = n             # <<<<<<<<<<<<<<
 */
  __pyx_t_4 = __Pyx_PyInt_As_uint8_t(__pyx_v_n); if (unlikely((__pyx_t_4 == ((uint8_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 460, __pyx_L1_error)
  __pyx_v_self->length = __pyx_t_4;

  /* "pygame_sdl2/color.pyx":457
 *         return c
 * 
 *     def set_length(self, n):             # <<<<<<<<<<<<<<
 *         if n > 4 or n < 1:
 *             raise ValueError(n)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.set_length", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pygame_sdl2/color.pxd":30
 *     cdef object __weakref__
 * 
 *     cdef public Uint8 r, g, b, a             # <<<<<<<<<<<<<<
 *     cdef uint8_t length
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_1r_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_1r_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_1r___get__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_1r___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->r); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.r.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_1r_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_1r_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_1r_2__set__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_1r_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Uint8 __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_uint8_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(1, 30, __pyx_L1_error)
  __pyx_v_self->r = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pygame_sdl2.color.Color.r.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_1g_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_1g_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_1g___get__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_1g___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->g); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.g.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_1g_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_1g_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_1g_2__set__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_1g_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Uint8 __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_uint8_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(1, 30, __pyx_L1_error)
  __pyx_v_self->g = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pygame_sdl2.color.Color.g.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_1b_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_1b_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_1b___get__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_1b___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->b); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.b.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_1b_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_1b_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_1b_2__set__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_1b_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Uint8 __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_uint8_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(1, 30, __pyx_L1_error)
  __pyx_v_self->b = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pygame_sdl2.color.Color.b.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_1a_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pygame_sdl2_5color_5Color_1a_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_1a___get__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pygame_sdl2_5color_5Color_1a___get__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint8_t(__pyx_v_self->a); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pygame_sdl2.color.Color.a.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_11pygame_sdl2_5color_5Color_1a_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_11pygame_sdl2_5color_5Color_1a_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pygame_sdl2_5color_5Color_1a_2__set__(((struct __pyx_obj_11pygame_sdl2_5color_Color *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pygame_sdl2_5color_5Color_1a_2__set__(struct __pyx_obj_11pygame_sdl2_5color_Color *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Uint8 __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_uint8_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((Uint8)-1)) && PyErr_Occurred())) __PYX_ERR(1, 30, __pyx_L1_error)
  __pyx_v_self->a = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pygame_sdl2.color.Color.a.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_11pygame_sdl2_5color_Color __pyx_vtable_11pygame_sdl2_5color_Color;

static PyObject *__pyx_tp_new_11pygame_sdl2_5color_Color(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_11pygame_sdl2_5color_Color *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_11pygame_sdl2_5color_Color *)o);
  p->__pyx_vtab = __pyx_vtabptr_11pygame_sdl2_5color_Color;
  if (unlikely(__pyx_pw_11pygame_sdl2_5color_5Color_3__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_11pygame_sdl2_5color_Color(PyObject *o) {
  struct __pyx_obj_11pygame_sdl2_5color_Color *p = (struct __pyx_obj_11pygame_sdl2_5color_Color *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  if (p->__weakref__) PyObject_ClearWeakRefs(o);
  (*Py_TYPE(o)->tp_free)(o);
}
static PyObject *__pyx_sq_item_11pygame_sdl2_5color_Color(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_11pygame_sdl2_5color_Color(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_11pygame_sdl2_5color_5Color_21__setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}
#define __pyx_nb_add_11pygame_sdl2_5color_Color __pyx_pw_11pygame_sdl2_5color_5Color_29__add__
#define __pyx_nb_subtract_11pygame_sdl2_5color_Color __pyx_pw_11pygame_sdl2_5color_5Color_31__sub__
#define __pyx_nb_multiply_11pygame_sdl2_5color_Color __pyx_pw_11pygame_sdl2_5color_5Color_27__mul__
#define __pyx_nb_divide_11pygame_sdl2_5color_Color __pyx_pw_11pygame_sdl2_5color_5Color_35__div__
#define __pyx_nb_remainder_11pygame_sdl2_5color_Color __pyx_pw_11pygame_sdl2_5color_5Color_33__mod__
#define __pyx_nb_floor_divide_11pygame_sdl2_5color_Color __pyx_pw_11pygame_sdl2_5color_5Color_37__floordiv__

static PyObject *__pyx_getprop_11pygame_sdl2_5color_5Color_cmy(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pygame_sdl2_5color_5Color_3cmy_1__get__(o);
}

static int __pyx_setprop_11pygame_sdl2_5color_5Color_cmy(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pygame_sdl2_5color_5Color_3cmy_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_11pygame_sdl2_5color_5Color_hsva(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pygame_sdl2_5color_5Color_4hsva_1__get__(o);
}

static int __pyx_setprop_11pygame_sdl2_5color_5Color_hsva(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pygame_sdl2_5color_5Color_4hsva_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_11pygame_sdl2_5color_5Color_hsla(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pygame_sdl2_5color_5Color_4hsla_1__get__(o);
}

static int __pyx_setprop_11pygame_sdl2_5color_5Color_hsla(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pygame_sdl2_5color_5Color_4hsla_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_11pygame_sdl2_5color_5Color_i1i2i3(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pygame_sdl2_5color_5Color_6i1i2i3_1__get__(o);
}

static int __pyx_setprop_11pygame_sdl2_5color_5Color_i1i2i3(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pygame_sdl2_5color_5Color_6i1i2i3_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_11pygame_sdl2_5color_5Color_r(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pygame_sdl2_5color_5Color_1r_1__get__(o);
}

static int __pyx_setprop_11pygame_sdl2_5color_5Color_r(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pygame_sdl2_5color_5Color_1r_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_11pygame_sdl2_5color_5Color_g(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pygame_sdl2_5color_5Color_1g_1__get__(o);
}

static int __pyx_setprop_11pygame_sdl2_5color_5Color_g(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pygame_sdl2_5color_5Color_1g_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_11pygame_sdl2_5color_5Color_b(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pygame_sdl2_5color_5Color_1b_1__get__(o);
}

static int __pyx_setprop_11pygame_sdl2_5color_5Color_b(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pygame_sdl2_5color_5Color_1b_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_11pygame_sdl2_5color_5Color_a(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pygame_sdl2_5color_5Color_1a_1__get__(o);
}

static int __pyx_setprop_11pygame_sdl2_5color_5Color_a(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pygame_sdl2_5color_5Color_1a_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_11pygame_sdl2_5color_Color[] = {
  {"__reduce__", (PyCFunction)__pyx_pw_11pygame_sdl2_5color_5Color_17__reduce__, METH_NOARGS, 0},
  {"__setstate__", (PyCFunction)__pyx_pw_11pygame_sdl2_5color_5Color_19__setstate__, METH_O, 0},
  {"normalize", (PyCFunction)__pyx_pw_11pygame_sdl2_5color_5Color_39normalize, METH_NOARGS, 0},
  {"correct_gamma", (PyCFunction)__pyx_pw_11pygame_sdl2_5color_5Color_41correct_gamma, METH_O, 0},
  {"set_length", (PyCFunction)__pyx_pw_11pygame_sdl2_5color_5Color_43set_length, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_11pygame_sdl2_5color_Color[] = {
  {(char *)"cmy", __pyx_getprop_11pygame_sdl2_5color_5Color_cmy, __pyx_setprop_11pygame_sdl2_5color_5Color_cmy, (char *)0, 0},
  {(char *)"hsva", __pyx_getprop_11pygame_sdl2_5color_5Color_hsva, __pyx_setprop_11pygame_sdl2_5color_5Color_hsva, (char *)0, 0},
  {(char *)"hsla", __pyx_getprop_11pygame_sdl2_5color_5Color_hsla, __pyx_setprop_11pygame_sdl2_5color_5Color_hsla, (char *)0, 0},
  {(char *)"i1i2i3", __pyx_getprop_11pygame_sdl2_5color_5Color_i1i2i3, __pyx_setprop_11pygame_sdl2_5color_5Color_i1i2i3, (char *)0, 0},
  {(char *)"r", __pyx_getprop_11pygame_sdl2_5color_5Color_r, __pyx_setprop_11pygame_sdl2_5color_5Color_r, (char *)0, 0},
  {(char *)"g", __pyx_getprop_11pygame_sdl2_5color_5Color_g, __pyx_setprop_11pygame_sdl2_5color_5Color_g, (char *)0, 0},
  {(char *)"b", __pyx_getprop_11pygame_sdl2_5color_5Color_b, __pyx_setprop_11pygame_sdl2_5color_5Color_b, (char *)0, 0},
  {(char *)"a", __pyx_getprop_11pygame_sdl2_5color_5Color_a, __pyx_setprop_11pygame_sdl2_5color_5Color_a, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_Color = {
  __pyx_nb_add_11pygame_sdl2_5color_Color, /*nb_add*/
  __pyx_nb_subtract_11pygame_sdl2_5color_Color, /*nb_subtract*/
  __pyx_nb_multiply_11pygame_sdl2_5color_Color, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  __pyx_nb_divide_11pygame_sdl2_5color_Color, /*nb_divide*/
  #endif
  __pyx_nb_remainder_11pygame_sdl2_5color_Color, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  __pyx_pw_11pygame_sdl2_5color_5Color_9__int__, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  __pyx_pw_11pygame_sdl2_5color_5Color_9__int__, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  __pyx_pw_11pygame_sdl2_5color_5Color_15__float__, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  __pyx_pw_11pygame_sdl2_5color_5Color_13__oct__, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  __pyx_pw_11pygame_sdl2_5color_5Color_11__hex__, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  __pyx_nb_floor_divide_11pygame_sdl2_5color_Color, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PySequenceMethods __pyx_tp_as_sequence_Color = {
  __pyx_pw_11pygame_sdl2_5color_5Color_25__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_11pygame_sdl2_5color_Color, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_Color = {
  __pyx_pw_11pygame_sdl2_5color_5Color_25__len__, /*mp_length*/
  __pyx_pw_11pygame_sdl2_5color_5Color_23__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_11pygame_sdl2_5color_Color, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_11pygame_sdl2_5color_Color = {
  PyVarObject_HEAD_INIT(0, 0)
  "pygame_sdl2.color.Color", /*tp_name*/
  sizeof(struct __pyx_obj_11pygame_sdl2_5color_Color), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_11pygame_sdl2_5color_Color, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_11pygame_sdl2_5color_5Color_7__repr__, /*tp_repr*/
  &__pyx_tp_as_number_Color, /*tp_as_number*/
  &__pyx_tp_as_sequence_Color, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_Color, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_pw_11pygame_sdl2_5color_5Color_1__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_11pygame_sdl2_5color_Color, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_11pygame_sdl2_5color_Color, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_11pygame_sdl2_5color_5Color_5__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_11pygame_sdl2_5color_Color, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *__pyx_freelist_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma[8];
static int __pyx_freecount_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma = 0;

static PyObject *__pyx_tp_new_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma)))) {
    o = (PyObject*)__pyx_freelist_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma[--__pyx_freecount_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma];
    memset(o, 0, sizeof(struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma(PyObject *o) {
  struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *p = (struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_gamma);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma)))) {
    __pyx_freelist_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma[__pyx_freecount_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma++] = ((struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *p = (struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *)o;
  if (p->__pyx_v_gamma) {
    e = (*v)(p->__pyx_v_gamma, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *p = (struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma *)o;
  tmp = ((PyObject*)p->__pyx_v_gamma);
  p->__pyx_v_gamma = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma = {
  PyVarObject_HEAD_INIT(0, 0)
  "pygame_sdl2.color.__pyx_scope_struct__correct_gamma", /*tp_name*/
  sizeof(struct __pyx_obj_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma, /*tp_traverse*/
  __pyx_tp_clear_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_color(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_color},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "color",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_08x, __pyx_k_08x, sizeof(__pyx_k_08x), 0, 1, 0, 0},
  {&__pyx_kp_u_0x, __pyx_k_0x, sizeof(__pyx_k_0x), 0, 1, 0, 0},
  {&__pyx_n_u_BBB, __pyx_k_BBB, sizeof(__pyx_k_BBB), 0, 1, 0, 1},
  {&__pyx_n_u_BBBB, __pyx_k_BBBB, sizeof(__pyx_k_BBBB), 0, 1, 0, 1},
  {&__pyx_n_s_Color, __pyx_k_Color, sizeof(__pyx_k_Color), 0, 0, 1, 1},
  {&__pyx_n_s_Color_from_hex_locals_lambda, __pyx_k_Color_from_hex_locals_lambda, sizeof(__pyx_k_Color_from_hex_locals_lambda), 0, 0, 1, 1},
  {&__pyx_kp_u_Expected_a_color, __pyx_k_Expected_a_color, sizeof(__pyx_k_Expected_a_color), 0, 1, 0, 0},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_kp_u_L, __pyx_k_L, sizeof(__pyx_k_L), 0, 1, 0, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_u__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 1, 0, 0},
  {&__pyx_kp_u__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 1, 0, 0},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_u_aliceblue, __pyx_k_aliceblue, sizeof(__pyx_k_aliceblue), 0, 1, 0, 1},
  {&__pyx_n_u_antiquewhite, __pyx_k_antiquewhite, sizeof(__pyx_k_antiquewhite), 0, 1, 0, 1},
  {&__pyx_n_u_antiquewhite1, __pyx_k_antiquewhite1, sizeof(__pyx_k_antiquewhite1), 0, 1, 0, 1},
  {&__pyx_n_u_antiquewhite2, __pyx_k_antiquewhite2, sizeof(__pyx_k_antiquewhite2), 0, 1, 0, 1},
  {&__pyx_n_u_antiquewhite3, __pyx_k_antiquewhite3, sizeof(__pyx_k_antiquewhite3), 0, 1, 0, 1},
  {&__pyx_n_u_antiquewhite4, __pyx_k_antiquewhite4, sizeof(__pyx_k_antiquewhite4), 0, 1, 0, 1},
  {&__pyx_n_u_aquamarine, __pyx_k_aquamarine, sizeof(__pyx_k_aquamarine), 0, 1, 0, 1},
  {&__pyx_n_u_aquamarine1, __pyx_k_aquamarine1, sizeof(__pyx_k_aquamarine1), 0, 1, 0, 1},
  {&__pyx_n_u_aquamarine2, __pyx_k_aquamarine2, sizeof(__pyx_k_aquamarine2), 0, 1, 0, 1},
  {&__pyx_n_u_aquamarine3, __pyx_k_aquamarine3, sizeof(__pyx_k_aquamarine3), 0, 1, 0, 1},
  {&__pyx_n_u_aquamarine4, __pyx_k_aquamarine4, sizeof(__pyx_k_aquamarine4), 0, 1, 0, 1},
  {&__pyx_n_u_azure, __pyx_k_azure, sizeof(__pyx_k_azure), 0, 1, 0, 1},
  {&__pyx_n_u_azure1, __pyx_k_azure1, sizeof(__pyx_k_azure1), 0, 1, 0, 1},
  {&__pyx_n_u_azure2, __pyx_k_azure2, sizeof(__pyx_k_azure2), 0, 1, 0, 1},
  {&__pyx_n_u_azure3, __pyx_k_azure3, sizeof(__pyx_k_azure3), 0, 1, 0, 1},
  {&__pyx_n_u_azure4, __pyx_k_azure4, sizeof(__pyx_k_azure4), 0, 1, 0, 1},
  {&__pyx_n_s_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 0, 1, 1},
  {&__pyx_n_u_beige, __pyx_k_beige, sizeof(__pyx_k_beige), 0, 1, 0, 1},
  {&__pyx_n_s_binascii, __pyx_k_binascii, sizeof(__pyx_k_binascii), 0, 0, 1, 1},
  {&__pyx_n_u_bisque, __pyx_k_bisque, sizeof(__pyx_k_bisque), 0, 1, 0, 1},
  {&__pyx_n_u_bisque1, __pyx_k_bisque1, sizeof(__pyx_k_bisque1), 0, 1, 0, 1},
  {&__pyx_n_u_bisque2, __pyx_k_bisque2, sizeof(__pyx_k_bisque2), 0, 1, 0, 1},
  {&__pyx_n_u_bisque3, __pyx_k_bisque3, sizeof(__pyx_k_bisque3), 0, 1, 0, 1},
  {&__pyx_n_u_bisque4, __pyx_k_bisque4, sizeof(__pyx_k_bisque4), 0, 1, 0, 1},
  {&__pyx_n_u_black, __pyx_k_black, sizeof(__pyx_k_black), 0, 1, 0, 1},
  {&__pyx_n_u_blanchedalmond, __pyx_k_blanchedalmond, sizeof(__pyx_k_blanchedalmond), 0, 1, 0, 1},
  {&__pyx_n_u_blue, __pyx_k_blue, sizeof(__pyx_k_blue), 0, 1, 0, 1},
  {&__pyx_n_u_blue1, __pyx_k_blue1, sizeof(__pyx_k_blue1), 0, 1, 0, 1},
  {&__pyx_n_u_blue2, __pyx_k_blue2, sizeof(__pyx_k_blue2), 0, 1, 0, 1},
  {&__pyx_n_u_blue3, __pyx_k_blue3, sizeof(__pyx_k_blue3), 0, 1, 0, 1},
  {&__pyx_n_u_blue4, __pyx_k_blue4, sizeof(__pyx_k_blue4), 0, 1, 0, 1},
  {&__pyx_n_u_blueviolet, __pyx_k_blueviolet, sizeof(__pyx_k_blueviolet), 0, 1, 0, 1},
  {&__pyx_n_u_brown, __pyx_k_brown, sizeof(__pyx_k_brown), 0, 1, 0, 1},
  {&__pyx_n_u_brown1, __pyx_k_brown1, sizeof(__pyx_k_brown1), 0, 1, 0, 1},
  {&__pyx_n_u_brown2, __pyx_k_brown2, sizeof(__pyx_k_brown2), 0, 1, 0, 1},
  {&__pyx_n_u_brown3, __pyx_k_brown3, sizeof(__pyx_k_brown3), 0, 1, 0, 1},
  {&__pyx_n_u_brown4, __pyx_k_brown4, sizeof(__pyx_k_brown4), 0, 1, 0, 1},
  {&__pyx_n_u_burlywood, __pyx_k_burlywood, sizeof(__pyx_k_burlywood), 0, 1, 0, 1},
  {&__pyx_n_u_burlywood1, __pyx_k_burlywood1, sizeof(__pyx_k_burlywood1), 0, 1, 0, 1},
  {&__pyx_n_u_burlywood2, __pyx_k_burlywood2, sizeof(__pyx_k_burlywood2), 0, 1, 0, 1},
  {&__pyx_n_u_burlywood3, __pyx_k_burlywood3, sizeof(__pyx_k_burlywood3), 0, 1, 0, 1},
  {&__pyx_n_u_burlywood4, __pyx_k_burlywood4, sizeof(__pyx_k_burlywood4), 0, 1, 0, 1},
  {&__pyx_n_u_cadetblue, __pyx_k_cadetblue, sizeof(__pyx_k_cadetblue), 0, 1, 0, 1},
  {&__pyx_n_u_cadetblue1, __pyx_k_cadetblue1, sizeof(__pyx_k_cadetblue1), 0, 1, 0, 1},
  {&__pyx_n_u_cadetblue2, __pyx_k_cadetblue2, sizeof(__pyx_k_cadetblue2), 0, 1, 0, 1},
  {&__pyx_n_u_cadetblue3, __pyx_k_cadetblue3, sizeof(__pyx_k_cadetblue3), 0, 1, 0, 1},
  {&__pyx_n_u_cadetblue4, __pyx_k_cadetblue4, sizeof(__pyx_k_cadetblue4), 0, 1, 0, 1},
  {&__pyx_n_u_chartreuse, __pyx_k_chartreuse, sizeof(__pyx_k_chartreuse), 0, 1, 0, 1},
  {&__pyx_n_u_chartreuse1, __pyx_k_chartreuse1, sizeof(__pyx_k_chartreuse1), 0, 1, 0, 1},
  {&__pyx_n_u_chartreuse2, __pyx_k_chartreuse2, sizeof(__pyx_k_chartreuse2), 0, 1, 0, 1},
  {&__pyx_n_u_chartreuse3, __pyx_k_chartreuse3, sizeof(__pyx_k_chartreuse3), 0, 1, 0, 1},
  {&__pyx_n_u_chartreuse4, __pyx_k_chartreuse4, sizeof(__pyx_k_chartreuse4), 0, 1, 0, 1},
  {&__pyx_n_u_chocolate, __pyx_k_chocolate, sizeof(__pyx_k_chocolate), 0, 1, 0, 1},
  {&__pyx_n_u_chocolate1, __pyx_k_chocolate1, sizeof(__pyx_k_chocolate1), 0, 1, 0, 1},
  {&__pyx_n_u_chocolate2, __pyx_k_chocolate2, sizeof(__pyx_k_chocolate2), 0, 1, 0, 1},
  {&__pyx_n_u_chocolate3, __pyx_k_chocolate3, sizeof(__pyx_k_chocolate3), 0, 1, 0, 1},
  {&__pyx_n_u_chocolate4, __pyx_k_chocolate4, sizeof(__pyx_k_chocolate4), 0, 1, 0, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_u_coral, __pyx_k_coral, sizeof(__pyx_k_coral), 0, 1, 0, 1},
  {&__pyx_n_u_coral1, __pyx_k_coral1, sizeof(__pyx_k_coral1), 0, 1, 0, 1},
  {&__pyx_n_u_coral2, __pyx_k_coral2, sizeof(__pyx_k_coral2), 0, 1, 0, 1},
  {&__pyx_n_u_coral3, __pyx_k_coral3, sizeof(__pyx_k_coral3), 0, 1, 0, 1},
  {&__pyx_n_u_coral4, __pyx_k_coral4, sizeof(__pyx_k_coral4), 0, 1, 0, 1},
  {&__pyx_n_u_cornflowerblue, __pyx_k_cornflowerblue, sizeof(__pyx_k_cornflowerblue), 0, 1, 0, 1},
  {&__pyx_n_u_cornsilk, __pyx_k_cornsilk, sizeof(__pyx_k_cornsilk), 0, 1, 0, 1},
  {&__pyx_n_u_cornsilk1, __pyx_k_cornsilk1, sizeof(__pyx_k_cornsilk1), 0, 1, 0, 1},
  {&__pyx_n_u_cornsilk2, __pyx_k_cornsilk2, sizeof(__pyx_k_cornsilk2), 0, 1, 0, 1},
  {&__pyx_n_u_cornsilk3, __pyx_k_cornsilk3, sizeof(__pyx_k_cornsilk3), 0, 1, 0, 1},
  {&__pyx_n_u_cornsilk4, __pyx_k_cornsilk4, sizeof(__pyx_k_cornsilk4), 0, 1, 0, 1},
  {&__pyx_n_s_correct_gamma_locals_lambda, __pyx_k_correct_gamma_locals_lambda, sizeof(__pyx_k_correct_gamma_locals_lambda), 0, 0, 1, 1},
  {&__pyx_n_u_cyan, __pyx_k_cyan, sizeof(__pyx_k_cyan), 0, 1, 0, 1},
  {&__pyx_n_u_cyan1, __pyx_k_cyan1, sizeof(__pyx_k_cyan1), 0, 1, 0, 1},
  {&__pyx_n_u_cyan2, __pyx_k_cyan2, sizeof(__pyx_k_cyan2), 0, 1, 0, 1},
  {&__pyx_n_u_cyan3, __pyx_k_cyan3, sizeof(__pyx_k_cyan3), 0, 1, 0, 1},
  {&__pyx_n_u_cyan4, __pyx_k_cyan4, sizeof(__pyx_k_cyan4), 0, 1, 0, 1},
  {&__pyx_n_u_darkblue, __pyx_k_darkblue, sizeof(__pyx_k_darkblue), 0, 1, 0, 1},
  {&__pyx_n_u_darkcyan, __pyx_k_darkcyan, sizeof(__pyx_k_darkcyan), 0, 1, 0, 1},
  {&__pyx_n_u_darkgoldenrod, __pyx_k_darkgoldenrod, sizeof(__pyx_k_darkgoldenrod), 0, 1, 0, 1},
  {&__pyx_n_u_darkgoldenrod1, __pyx_k_darkgoldenrod1, sizeof(__pyx_k_darkgoldenrod1), 0, 1, 0, 1},
  {&__pyx_n_u_darkgoldenrod2, __pyx_k_darkgoldenrod2, sizeof(__pyx_k_darkgoldenrod2), 0, 1, 0, 1},
  {&__pyx_n_u_darkgoldenrod3, __pyx_k_darkgoldenrod3, sizeof(__pyx_k_darkgoldenrod3), 0, 1, 0, 1},
  {&__pyx_n_u_darkgoldenrod4, __pyx_k_darkgoldenrod4, sizeof(__pyx_k_darkgoldenrod4), 0, 1, 0, 1},
  {&__pyx_n_u_darkgray, __pyx_k_darkgray, sizeof(__pyx_k_darkgray), 0, 1, 0, 1},
  {&__pyx_n_u_darkgreen, __pyx_k_darkgreen, sizeof(__pyx_k_darkgreen), 0, 1, 0, 1},
  {&__pyx_n_u_darkgrey, __pyx_k_darkgrey, sizeof(__pyx_k_darkgrey), 0, 1, 0, 1},
  {&__pyx_n_u_darkkhaki, __pyx_k_darkkhaki, sizeof(__pyx_k_darkkhaki), 0, 1, 0, 1},
  {&__pyx_n_u_darkmagenta, __pyx_k_darkmagenta, sizeof(__pyx_k_darkmagenta), 0, 1, 0, 1},
  {&__pyx_n_u_darkolivegreen, __pyx_k_darkolivegreen, sizeof(__pyx_k_darkolivegreen), 0, 1, 0, 1},
  {&__pyx_n_u_darkolivegreen1, __pyx_k_darkolivegreen1, sizeof(__pyx_k_darkolivegreen1), 0, 1, 0, 1},
  {&__pyx_n_u_darkolivegreen2, __pyx_k_darkolivegreen2, sizeof(__pyx_k_darkolivegreen2), 0, 1, 0, 1},
  {&__pyx_n_u_darkolivegreen3, __pyx_k_darkolivegreen3, sizeof(__pyx_k_darkolivegreen3), 0, 1, 0, 1},
  {&__pyx_n_u_darkolivegreen4, __pyx_k_darkolivegreen4, sizeof(__pyx_k_darkolivegreen4), 0, 1, 0, 1},
  {&__pyx_n_u_darkorange, __pyx_k_darkorange, sizeof(__pyx_k_darkorange), 0, 1, 0, 1},
  {&__pyx_n_u_darkorange1, __pyx_k_darkorange1, sizeof(__pyx_k_darkorange1), 0, 1, 0, 1},
  {&__pyx_n_u_darkorange2, __pyx_k_darkorange2, sizeof(__pyx_k_darkorange2), 0, 1, 0, 1},
  {&__pyx_n_u_darkorange3, __pyx_k_darkorange3, sizeof(__pyx_k_darkorange3), 0, 1, 0, 1},
  {&__pyx_n_u_darkorange4, __pyx_k_darkorange4, sizeof(__pyx_k_darkorange4), 0, 1, 0, 1},
  {&__pyx_n_u_darkorchid, __pyx_k_darkorchid, sizeof(__pyx_k_darkorchid), 0, 1, 0, 1},
  {&__pyx_n_u_darkorchid1, __pyx_k_darkorchid1, sizeof(__pyx_k_darkorchid1), 0, 1, 0, 1},
  {&__pyx_n_u_darkorchid2, __pyx_k_darkorchid2, sizeof(__pyx_k_darkorchid2), 0, 1, 0, 1},
  {&__pyx_n_u_darkorchid3, __pyx_k_darkorchid3, sizeof(__pyx_k_darkorchid3), 0, 1, 0, 1},
  {&__pyx_n_u_darkorchid4, __pyx_k_darkorchid4, sizeof(__pyx_k_darkorchid4), 0, 1, 0, 1},
  {&__pyx_n_u_darkred, __pyx_k_darkred, sizeof(__pyx_k_darkred), 0, 1, 0, 1},
  {&__pyx_n_u_darksalmon, __pyx_k_darksalmon, sizeof(__pyx_k_darksalmon), 0, 1, 0, 1},
  {&__pyx_n_u_darkseagreen, __pyx_k_darkseagreen, sizeof(__pyx_k_darkseagreen), 0, 1, 0, 1},
  {&__pyx_n_u_darkseagreen1, __pyx_k_darkseagreen1, sizeof(__pyx_k_darkseagreen1), 0, 1, 0, 1},
  {&__pyx_n_u_darkseagreen2, __pyx_k_darkseagreen2, sizeof(__pyx_k_darkseagreen2), 0, 1, 0, 1},
  {&__pyx_n_u_darkseagreen3, __pyx_k_darkseagreen3, sizeof(__pyx_k_darkseagreen3), 0, 1, 0, 1},
  {&__pyx_n_u_darkseagreen4, __pyx_k_darkseagreen4, sizeof(__pyx_k_darkseagreen4), 0, 1, 0, 1},
  {&__pyx_n_u_darkslateblue, __pyx_k_darkslateblue, sizeof(__pyx_k_darkslateblue), 0, 1, 0, 1},
  {&__pyx_n_u_darkslategray, __pyx_k_darkslategray, sizeof(__pyx_k_darkslategray), 0, 1, 0, 1},
  {&__pyx_n_u_darkslategray1, __pyx_k_darkslategray1, sizeof(__pyx_k_darkslategray1), 0, 1, 0, 1},
  {&__pyx_n_u_darkslategray2, __pyx_k_darkslategray2, sizeof(__pyx_k_darkslategray2), 0, 1, 0, 1},
  {&__pyx_n_u_darkslategray3, __pyx_k_darkslategray3, sizeof(__pyx_k_darkslategray3), 0, 1, 0, 1},
  {&__pyx_n_u_darkslategray4, __pyx_k_darkslategray4, sizeof(__pyx_k_darkslategray4), 0, 1, 0, 1},
  {&__pyx_n_u_darkslategrey, __pyx_k_darkslategrey, sizeof(__pyx_k_darkslategrey), 0, 1, 0, 1},
  {&__pyx_n_u_darkturquoise, __pyx_k_darkturquoise, sizeof(__pyx_k_darkturquoise), 0, 1, 0, 1},
  {&__pyx_n_u_darkviolet, __pyx_k_darkviolet, sizeof(__pyx_k_darkviolet), 0, 1, 0, 1},
  {&__pyx_n_u_debianred, __pyx_k_debianred, sizeof(__pyx_k_debianred), 0, 1, 0, 1},
  {&__pyx_n_u_deeppink, __pyx_k_deeppink, sizeof(__pyx_k_deeppink), 0, 1, 0, 1},
  {&__pyx_n_u_deeppink1, __pyx_k_deeppink1, sizeof(__pyx_k_deeppink1), 0, 1, 0, 1},
  {&__pyx_n_u_deeppink2, __pyx_k_deeppink2, sizeof(__pyx_k_deeppink2), 0, 1, 0, 1},
  {&__pyx_n_u_deeppink3, __pyx_k_deeppink3, sizeof(__pyx_k_deeppink3), 0, 1, 0, 1},
  {&__pyx_n_u_deeppink4, __pyx_k_deeppink4, sizeof(__pyx_k_deeppink4), 0, 1, 0, 1},
  {&__pyx_n_u_deepskyblue, __pyx_k_deepskyblue, sizeof(__pyx_k_deepskyblue), 0, 1, 0, 1},
  {&__pyx_n_u_deepskyblue1, __pyx_k_deepskyblue1, sizeof(__pyx_k_deepskyblue1), 0, 1, 0, 1},
  {&__pyx_n_u_deepskyblue2, __pyx_k_deepskyblue2, sizeof(__pyx_k_deepskyblue2), 0, 1, 0, 1},
  {&__pyx_n_u_deepskyblue3, __pyx_k_deepskyblue3, sizeof(__pyx_k_deepskyblue3), 0, 1, 0, 1},
  {&__pyx_n_u_deepskyblue4, __pyx_k_deepskyblue4, sizeof(__pyx_k_deepskyblue4), 0, 1, 0, 1},
  {&__pyx_n_u_dimgray, __pyx_k_dimgray, sizeof(__pyx_k_dimgray), 0, 1, 0, 1},
  {&__pyx_n_u_dimgrey, __pyx_k_dimgrey, sizeof(__pyx_k_dimgrey), 0, 1, 0, 1},
  {&__pyx_n_s_div, __pyx_k_div, sizeof(__pyx_k_div), 0, 0, 1, 1},
  {&__pyx_n_u_dodgerblue, __pyx_k_dodgerblue, sizeof(__pyx_k_dodgerblue), 0, 1, 0, 1},
  {&__pyx_n_u_dodgerblue1, __pyx_k_dodgerblue1, sizeof(__pyx_k_dodgerblue1), 0, 1, 0, 1},
  {&__pyx_n_u_dodgerblue2, __pyx_k_dodgerblue2, sizeof(__pyx_k_dodgerblue2), 0, 1, 0, 1},
  {&__pyx_n_u_dodgerblue3, __pyx_k_dodgerblue3, sizeof(__pyx_k_dodgerblue3), 0, 1, 0, 1},
  {&__pyx_n_u_dodgerblue4, __pyx_k_dodgerblue4, sizeof(__pyx_k_dodgerblue4), 0, 1, 0, 1},
  {&__pyx_n_u_firebrick, __pyx_k_firebrick, sizeof(__pyx_k_firebrick), 0, 1, 0, 1},
  {&__pyx_n_u_firebrick1, __pyx_k_firebrick1, sizeof(__pyx_k_firebrick1), 0, 1, 0, 1},
  {&__pyx_n_u_firebrick2, __pyx_k_firebrick2, sizeof(__pyx_k_firebrick2), 0, 1, 0, 1},
  {&__pyx_n_u_firebrick3, __pyx_k_firebrick3, sizeof(__pyx_k_firebrick3), 0, 1, 0, 1},
  {&__pyx_n_u_firebrick4, __pyx_k_firebrick4, sizeof(__pyx_k_firebrick4), 0, 1, 0, 1},
  {&__pyx_n_s_floordiv___locals_div, __pyx_k_floordiv___locals_div, sizeof(__pyx_k_floordiv___locals_div), 0, 0, 1, 1},
  {&__pyx_n_u_floralwhite, __pyx_k_floralwhite, sizeof(__pyx_k_floralwhite), 0, 1, 0, 1},
  {&__pyx_n_u_forestgreen, __pyx_k_forestgreen, sizeof(__pyx_k_forestgreen), 0, 1, 0, 1},
  {&__pyx_n_s_g, __pyx_k_g, sizeof(__pyx_k_g), 0, 0, 1, 1},
  {&__pyx_n_u_gainsboro, __pyx_k_gainsboro, sizeof(__pyx_k_gainsboro), 0, 1, 0, 1},
  {&__pyx_n_u_ghostwhite, __pyx_k_ghostwhite, sizeof(__pyx_k_ghostwhite), 0, 1, 0, 1},
  {&__pyx_n_u_gold, __pyx_k_gold, sizeof(__pyx_k_gold), 0, 1, 0, 1},
  {&__pyx_n_u_gold1, __pyx_k_gold1, sizeof(__pyx_k_gold1), 0, 1, 0, 1},
  {&__pyx_n_u_gold2, __pyx_k_gold2, sizeof(__pyx_k_gold2), 0, 1, 0, 1},
  {&__pyx_n_u_gold3, __pyx_k_gold3, sizeof(__pyx_k_gold3), 0, 1, 0, 1},
  {&__pyx_n_u_gold4, __pyx_k_gold4, sizeof(__pyx_k_gold4), 0, 1, 0, 1},
  {&__pyx_n_u_goldenrod, __pyx_k_goldenrod, sizeof(__pyx_k_goldenrod), 0, 1, 0, 1},
  {&__pyx_n_u_goldenrod1, __pyx_k_goldenrod1, sizeof(__pyx_k_goldenrod1), 0, 1, 0, 1},
  {&__pyx_n_u_goldenrod2, __pyx_k_goldenrod2, sizeof(__pyx_k_goldenrod2), 0, 1, 0, 1},
  {&__pyx_n_u_goldenrod3, __pyx_k_goldenrod3, sizeof(__pyx_k_goldenrod3), 0, 1, 0, 1},
  {&__pyx_n_u_goldenrod4, __pyx_k_goldenrod4, sizeof(__pyx_k_goldenrod4), 0, 1, 0, 1},
  {&__pyx_n_u_gray, __pyx_k_gray, sizeof(__pyx_k_gray), 0, 1, 0, 1},
  {&__pyx_n_u_gray0, __pyx_k_gray0, sizeof(__pyx_k_gray0), 0, 1, 0, 1},
  {&__pyx_n_u_gray1, __pyx_k_gray1, sizeof(__pyx_k_gray1), 0, 1, 0, 1},
  {&__pyx_n_u_gray10, __pyx_k_gray10, sizeof(__pyx_k_gray10), 0, 1, 0, 1},
  {&__pyx_n_u_gray100, __pyx_k_gray100, sizeof(__pyx_k_gray100), 0, 1, 0, 1},
  {&__pyx_n_u_gray11, __pyx_k_gray11, sizeof(__pyx_k_gray11), 0, 1, 0, 1},
  {&__pyx_n_u_gray12, __pyx_k_gray12, sizeof(__pyx_k_gray12), 0, 1, 0, 1},
  {&__pyx_n_u_gray13, __pyx_k_gray13, sizeof(__pyx_k_gray13), 0, 1, 0, 1},
  {&__pyx_n_u_gray14, __pyx_k_gray14, sizeof(__pyx_k_gray14), 0, 1, 0, 1},
  {&__pyx_n_u_gray15, __pyx_k_gray15, sizeof(__pyx_k_gray15), 0, 1, 0, 1},
  {&__pyx_n_u_gray16, __pyx_k_gray16, sizeof(__pyx_k_gray16), 0, 1, 0, 1},
  {&__pyx_n_u_gray17, __pyx_k_gray17, sizeof(__pyx_k_gray17), 0, 1, 0, 1},
  {&__pyx_n_u_gray18, __pyx_k_gray18, sizeof(__pyx_k_gray18), 0, 1, 0, 1},
  {&__pyx_n_u_gray19, __pyx_k_gray19, sizeof(__pyx_k_gray19), 0, 1, 0, 1},
  {&__pyx_n_u_gray2, __pyx_k_gray2, sizeof(__pyx_k_gray2), 0, 1, 0, 1},
  {&__pyx_n_u_gray20, __pyx_k_gray20, sizeof(__pyx_k_gray20), 0, 1, 0, 1},
  {&__pyx_n_u_gray21, __pyx_k_gray21, sizeof(__pyx_k_gray21), 0, 1, 0, 1},
  {&__pyx_n_u_gray22, __pyx_k_gray22, sizeof(__pyx_k_gray22), 0, 1, 0, 1},
  {&__pyx_n_u_gray23, __pyx_k_gray23, sizeof(__pyx_k_gray23), 0, 1, 0, 1},
  {&__pyx_n_u_gray24, __pyx_k_gray24, sizeof(__pyx_k_gray24), 0, 1, 0, 1},
  {&__pyx_n_u_gray25, __pyx_k_gray25, sizeof(__pyx_k_gray25), 0, 1, 0, 1},
  {&__pyx_n_u_gray26, __pyx_k_gray26, sizeof(__pyx_k_gray26), 0, 1, 0, 1},
  {&__pyx_n_u_gray27, __pyx_k_gray27, sizeof(__pyx_k_gray27), 0, 1, 0, 1},
  {&__pyx_n_u_gray28, __pyx_k_gray28, sizeof(__pyx_k_gray28), 0, 1, 0, 1},
  {&__pyx_n_u_gray29, __pyx_k_gray29, sizeof(__pyx_k_gray29), 0, 1, 0, 1},
  {&__pyx_n_u_gray3, __pyx_k_gray3, sizeof(__pyx_k_gray3), 0, 1, 0, 1},
  {&__pyx_n_u_gray30, __pyx_k_gray30, sizeof(__pyx_k_gray30), 0, 1, 0, 1},
  {&__pyx_n_u_gray31, __pyx_k_gray31, sizeof(__pyx_k_gray31), 0, 1, 0, 1},
  {&__pyx_n_u_gray32, __pyx_k_gray32, sizeof(__pyx_k_gray32), 0, 1, 0, 1},
  {&__pyx_n_u_gray33, __pyx_k_gray33, sizeof(__pyx_k_gray33), 0, 1, 0, 1},
  {&__pyx_n_u_gray34, __pyx_k_gray34, sizeof(__pyx_k_gray34), 0, 1, 0, 1},
  {&__pyx_n_u_gray35, __pyx_k_gray35, sizeof(__pyx_k_gray35), 0, 1, 0, 1},
  {&__pyx_n_u_gray36, __pyx_k_gray36, sizeof(__pyx_k_gray36), 0, 1, 0, 1},
  {&__pyx_n_u_gray37, __pyx_k_gray37, sizeof(__pyx_k_gray37), 0, 1, 0, 1},
  {&__pyx_n_u_gray38, __pyx_k_gray38, sizeof(__pyx_k_gray38), 0, 1, 0, 1},
  {&__pyx_n_u_gray39, __pyx_k_gray39, sizeof(__pyx_k_gray39), 0, 1, 0, 1},
  {&__pyx_n_u_gray4, __pyx_k_gray4, sizeof(__pyx_k_gray4), 0, 1, 0, 1},
  {&__pyx_n_u_gray40, __pyx_k_gray40, sizeof(__pyx_k_gray40), 0, 1, 0, 1},
  {&__pyx_n_u_gray41, __pyx_k_gray41, sizeof(__pyx_k_gray41), 0, 1, 0, 1},
  {&__pyx_n_u_gray42, __pyx_k_gray42, sizeof(__pyx_k_gray42), 0, 1, 0, 1},
  {&__pyx_n_u_gray43, __pyx_k_gray43, sizeof(__pyx_k_gray43), 0, 1, 0, 1},
  {&__pyx_n_u_gray44, __pyx_k_gray44, sizeof(__pyx_k_gray44), 0, 1, 0, 1},
  {&__pyx_n_u_gray45, __pyx_k_gray45, sizeof(__pyx_k_gray45), 0, 1, 0, 1},
  {&__pyx_n_u_gray46, __pyx_k_gray46, sizeof(__pyx_k_gray46), 0, 1, 0, 1},
  {&__pyx_n_u_gray47, __pyx_k_gray47, sizeof(__pyx_k_gray47), 0, 1, 0, 1},
  {&__pyx_n_u_gray48, __pyx_k_gray48, sizeof(__pyx_k_gray48), 0, 1, 0, 1},
  {&__pyx_n_u_gray49, __pyx_k_gray49, sizeof(__pyx_k_gray49), 0, 1, 0, 1},
  {&__pyx_n_u_gray5, __pyx_k_gray5, sizeof(__pyx_k_gray5), 0, 1, 0, 1},
  {&__pyx_n_u_gray50, __pyx_k_gray50, sizeof(__pyx_k_gray50), 0, 1, 0, 1},
  {&__pyx_n_u_gray51, __pyx_k_gray51, sizeof(__pyx_k_gray51), 0, 1, 0, 1},
  {&__pyx_n_u_gray52, __pyx_k_gray52, sizeof(__pyx_k_gray52), 0, 1, 0, 1},
  {&__pyx_n_u_gray53, __pyx_k_gray53, sizeof(__pyx_k_gray53), 0, 1, 0, 1},
  {&__pyx_n_u_gray54, __pyx_k_gray54, sizeof(__pyx_k_gray54), 0, 1, 0, 1},
  {&__pyx_n_u_gray55, __pyx_k_gray55, sizeof(__pyx_k_gray55), 0, 1, 0, 1},
  {&__pyx_n_u_gray56, __pyx_k_gray56, sizeof(__pyx_k_gray56), 0, 1, 0, 1},
  {&__pyx_n_u_gray57, __pyx_k_gray57, sizeof(__pyx_k_gray57), 0, 1, 0, 1},
  {&__pyx_n_u_gray58, __pyx_k_gray58, sizeof(__pyx_k_gray58), 0, 1, 0, 1},
  {&__pyx_n_u_gray59, __pyx_k_gray59, sizeof(__pyx_k_gray59), 0, 1, 0, 1},
  {&__pyx_n_u_gray6, __pyx_k_gray6, sizeof(__pyx_k_gray6), 0, 1, 0, 1},
  {&__pyx_n_u_gray60, __pyx_k_gray60, sizeof(__pyx_k_gray60), 0, 1, 0, 1},
  {&__pyx_n_u_gray61, __pyx_k_gray61, sizeof(__pyx_k_gray61), 0, 1, 0, 1},
  {&__pyx_n_u_gray62, __pyx_k_gray62, sizeof(__pyx_k_gray62), 0, 1, 0, 1},
  {&__pyx_n_u_gray63, __pyx_k_gray63, sizeof(__pyx_k_gray63), 0, 1, 0, 1},
  {&__pyx_n_u_gray64, __pyx_k_gray64, sizeof(__pyx_k_gray64), 0, 1, 0, 1},
  {&__pyx_n_u_gray65, __pyx_k_gray65, sizeof(__pyx_k_gray65), 0, 1, 0, 1},
  {&__pyx_n_u_gray66, __pyx_k_gray66, sizeof(__pyx_k_gray66), 0, 1, 0, 1},
  {&__pyx_n_u_gray67, __pyx_k_gray67, sizeof(__pyx_k_gray67), 0, 1, 0, 1},
  {&__pyx_n_u_gray68, __pyx_k_gray68, sizeof(__pyx_k_gray68), 0, 1, 0, 1},
  {&__pyx_n_u_gray69, __pyx_k_gray69, sizeof(__pyx_k_gray69), 0, 1, 0, 1},
  {&__pyx_n_u_gray7, __pyx_k_gray7, sizeof(__pyx_k_gray7), 0, 1, 0, 1},
  {&__pyx_n_u_gray70, __pyx_k_gray70, sizeof(__pyx_k_gray70), 0, 1, 0, 1},
  {&__pyx_n_u_gray71, __pyx_k_gray71, sizeof(__pyx_k_gray71), 0, 1, 0, 1},
  {&__pyx_n_u_gray72, __pyx_k_gray72, sizeof(__pyx_k_gray72), 0, 1, 0, 1},
  {&__pyx_n_u_gray73, __pyx_k_gray73, sizeof(__pyx_k_gray73), 0, 1, 0, 1},
  {&__pyx_n_u_gray74, __pyx_k_gray74, sizeof(__pyx_k_gray74), 0, 1, 0, 1},
  {&__pyx_n_u_gray75, __pyx_k_gray75, sizeof(__pyx_k_gray75), 0, 1, 0, 1},
  {&__pyx_n_u_gray76, __pyx_k_gray76, sizeof(__pyx_k_gray76), 0, 1, 0, 1},
  {&__pyx_n_u_gray77, __pyx_k_gray77, sizeof(__pyx_k_gray77), 0, 1, 0, 1},
  {&__pyx_n_u_gray78, __pyx_k_gray78, sizeof(__pyx_k_gray78), 0, 1, 0, 1},
  {&__pyx_n_u_gray79, __pyx_k_gray79, sizeof(__pyx_k_gray79), 0, 1, 0, 1},
  {&__pyx_n_u_gray8, __pyx_k_gray8, sizeof(__pyx_k_gray8), 0, 1, 0, 1},
  {&__pyx_n_u_gray80, __pyx_k_gray80, sizeof(__pyx_k_gray80), 0, 1, 0, 1},
  {&__pyx_n_u_gray81, __pyx_k_gray81, sizeof(__pyx_k_gray81), 0, 1, 0, 1},
  {&__pyx_n_u_gray82, __pyx_k_gray82, sizeof(__pyx_k_gray82), 0, 1, 0, 1},
  {&__pyx_n_u_gray83, __pyx_k_gray83, sizeof(__pyx_k_gray83), 0, 1, 0, 1},
  {&__pyx_n_u_gray84, __pyx_k_gray84, sizeof(__pyx_k_gray84), 0, 1, 0, 1},
  {&__pyx_n_u_gray85, __pyx_k_gray85, sizeof(__pyx_k_gray85), 0, 1, 0, 1},
  {&__pyx_n_u_gray86, __pyx_k_gray86, sizeof(__pyx_k_gray86), 0, 1, 0, 1},
  {&__pyx_n_u_gray87, __pyx_k_gray87, sizeof(__pyx_k_gray87), 0, 1, 0, 1},
  {&__pyx_n_u_gray88, __pyx_k_gray88, sizeof(__pyx_k_gray88), 0, 1, 0, 1},
  {&__pyx_n_u_gray89, __pyx_k_gray89, sizeof(__pyx_k_gray89), 0, 1, 0, 1},
  {&__pyx_n_u_gray9, __pyx_k_gray9, sizeof(__pyx_k_gray9), 0, 1, 0, 1},
  {&__pyx_n_u_gray90, __pyx_k_gray90, sizeof(__pyx_k_gray90), 0, 1, 0, 1},
  {&__pyx_n_u_gray91, __pyx_k_gray91, sizeof(__pyx_k_gray91), 0, 1, 0, 1},
  {&__pyx_n_u_gray92, __pyx_k_gray92, sizeof(__pyx_k_gray92), 0, 1, 0, 1},
  {&__pyx_n_u_gray93, __pyx_k_gray93, sizeof(__pyx_k_gray93), 0, 1, 0, 1},
  {&__pyx_n_u_gray94, __pyx_k_gray94, sizeof(__pyx_k_gray94), 0, 1, 0, 1},
  {&__pyx_n_u_gray95, __pyx_k_gray95, sizeof(__pyx_k_gray95), 0, 1, 0, 1},
  {&__pyx_n_u_gray96, __pyx_k_gray96, sizeof(__pyx_k_gray96), 0, 1, 0, 1},
  {&__pyx_n_u_gray97, __pyx_k_gray97, sizeof(__pyx_k_gray97), 0, 1, 0, 1},
  {&__pyx_n_u_gray98, __pyx_k_gray98, sizeof(__pyx_k_gray98), 0, 1, 0, 1},
  {&__pyx_n_u_gray99, __pyx_k_gray99, sizeof(__pyx_k_gray99), 0, 1, 0, 1},
  {&__pyx_n_u_green, __pyx_k_green, sizeof(__pyx_k_green), 0, 1, 0, 1},
  {&__pyx_n_u_green1, __pyx_k_green1, sizeof(__pyx_k_green1), 0, 1, 0, 1},
  {&__pyx_n_u_green2, __pyx_k_green2, sizeof(__pyx_k_green2), 0, 1, 0, 1},
  {&__pyx_n_u_green3, __pyx_k_green3, sizeof(__pyx_k_green3), 0, 1, 0, 1},
  {&__pyx_n_u_green4, __pyx_k_green4, sizeof(__pyx_k_green4), 0, 1, 0, 1},
  {&__pyx_n_u_greenyellow, __pyx_k_greenyellow, sizeof(__pyx_k_greenyellow), 0, 1, 0, 1},
  {&__pyx_n_u_grey, __pyx_k_grey, sizeof(__pyx_k_grey), 0, 1, 0, 1},
  {&__pyx_n_u_grey0, __pyx_k_grey0, sizeof(__pyx_k_grey0), 0, 1, 0, 1},
  {&__pyx_n_u_grey1, __pyx_k_grey1, sizeof(__pyx_k_grey1), 0, 1, 0, 1},
  {&__pyx_n_u_grey10, __pyx_k_grey10, sizeof(__pyx_k_grey10), 0, 1, 0, 1},
  {&__pyx_n_u_grey100, __pyx_k_grey100, sizeof(__pyx_k_grey100), 0, 1, 0, 1},
  {&__pyx_n_u_grey11, __pyx_k_grey11, sizeof(__pyx_k_grey11), 0, 1, 0, 1},
  {&__pyx_n_u_grey12, __pyx_k_grey12, sizeof(__pyx_k_grey12), 0, 1, 0, 1},
  {&__pyx_n_u_grey13, __pyx_k_grey13, sizeof(__pyx_k_grey13), 0, 1, 0, 1},
  {&__pyx_n_u_grey14, __pyx_k_grey14, sizeof(__pyx_k_grey14), 0, 1, 0, 1},
  {&__pyx_n_u_grey15, __pyx_k_grey15, sizeof(__pyx_k_grey15), 0, 1, 0, 1},
  {&__pyx_n_u_grey16, __pyx_k_grey16, sizeof(__pyx_k_grey16), 0, 1, 0, 1},
  {&__pyx_n_u_grey17, __pyx_k_grey17, sizeof(__pyx_k_grey17), 0, 1, 0, 1},
  {&__pyx_n_u_grey18, __pyx_k_grey18, sizeof(__pyx_k_grey18), 0, 1, 0, 1},
  {&__pyx_n_u_grey19, __pyx_k_grey19, sizeof(__pyx_k_grey19), 0, 1, 0, 1},
  {&__pyx_n_u_grey2, __pyx_k_grey2, sizeof(__pyx_k_grey2), 0, 1, 0, 1},
  {&__pyx_n_u_grey20, __pyx_k_grey20, sizeof(__pyx_k_grey20), 0, 1, 0, 1},
  {&__pyx_n_u_grey21, __pyx_k_grey21, sizeof(__pyx_k_grey21), 0, 1, 0, 1},
  {&__pyx_n_u_grey22, __pyx_k_grey22, sizeof(__pyx_k_grey22), 0, 1, 0, 1},
  {&__pyx_n_u_grey23, __pyx_k_grey23, sizeof(__pyx_k_grey23), 0, 1, 0, 1},
  {&__pyx_n_u_grey24, __pyx_k_grey24, sizeof(__pyx_k_grey24), 0, 1, 0, 1},
  {&__pyx_n_u_grey25, __pyx_k_grey25, sizeof(__pyx_k_grey25), 0, 1, 0, 1},
  {&__pyx_n_u_grey26, __pyx_k_grey26, sizeof(__pyx_k_grey26), 0, 1, 0, 1},
  {&__pyx_n_u_grey27, __pyx_k_grey27, sizeof(__pyx_k_grey27), 0, 1, 0, 1},
  {&__pyx_n_u_grey28, __pyx_k_grey28, sizeof(__pyx_k_grey28), 0, 1, 0, 1},
  {&__pyx_n_u_grey29, __pyx_k_grey29, sizeof(__pyx_k_grey29), 0, 1, 0, 1},
  {&__pyx_n_u_grey3, __pyx_k_grey3, sizeof(__pyx_k_grey3), 0, 1, 0, 1},
  {&__pyx_n_u_grey30, __pyx_k_grey30, sizeof(__pyx_k_grey30), 0, 1, 0, 1},
  {&__pyx_n_u_grey31, __pyx_k_grey31, sizeof(__pyx_k_grey31), 0, 1, 0, 1},
  {&__pyx_n_u_grey32, __pyx_k_grey32, sizeof(__pyx_k_grey32), 0, 1, 0, 1},
  {&__pyx_n_u_grey33, __pyx_k_grey33, sizeof(__pyx_k_grey33), 0, 1, 0, 1},
  {&__pyx_n_u_grey34, __pyx_k_grey34, sizeof(__pyx_k_grey34), 0, 1, 0, 1},
  {&__pyx_n_u_grey35, __pyx_k_grey35, sizeof(__pyx_k_grey35), 0, 1, 0, 1},
  {&__pyx_n_u_grey36, __pyx_k_grey36, sizeof(__pyx_k_grey36), 0, 1, 0, 1},
  {&__pyx_n_u_grey37, __pyx_k_grey37, sizeof(__pyx_k_grey37), 0, 1, 0, 1},
  {&__pyx_n_u_grey38, __pyx_k_grey38, sizeof(__pyx_k_grey38), 0, 1, 0, 1},
  {&__pyx_n_u_grey39, __pyx_k_grey39, sizeof(__pyx_k_grey39), 0, 1, 0, 1},
  {&__pyx_n_u_grey4, __pyx_k_grey4, sizeof(__pyx_k_grey4), 0, 1, 0, 1},
  {&__pyx_n_u_grey40, __pyx_k_grey40, sizeof(__pyx_k_grey40), 0, 1, 0, 1},
  {&__pyx_n_u_grey41, __pyx_k_grey41, sizeof(__pyx_k_grey41), 0, 1, 0, 1},
  {&__pyx_n_u_grey42, __pyx_k_grey42, sizeof(__pyx_k_grey42), 0, 1, 0, 1},
  {&__pyx_n_u_grey43, __pyx_k_grey43, sizeof(__pyx_k_grey43), 0, 1, 0, 1},
  {&__pyx_n_u_grey44, __pyx_k_grey44, sizeof(__pyx_k_grey44), 0, 1, 0, 1},
  {&__pyx_n_u_grey45, __pyx_k_grey45, sizeof(__pyx_k_grey45), 0, 1, 0, 1},
  {&__pyx_n_u_grey46, __pyx_k_grey46, sizeof(__pyx_k_grey46), 0, 1, 0, 1},
  {&__pyx_n_u_grey47, __pyx_k_grey47, sizeof(__pyx_k_grey47), 0, 1, 0, 1},
  {&__pyx_n_u_grey48, __pyx_k_grey48, sizeof(__pyx_k_grey48), 0, 1, 0, 1},
  {&__pyx_n_u_grey49, __pyx_k_grey49, sizeof(__pyx_k_grey49), 0, 1, 0, 1},
  {&__pyx_n_u_grey5, __pyx_k_grey5, sizeof(__pyx_k_grey5), 0, 1, 0, 1},
  {&__pyx_n_u_grey50, __pyx_k_grey50, sizeof(__pyx_k_grey50), 0, 1, 0, 1},
  {&__pyx_n_u_grey51, __pyx_k_grey51, sizeof(__pyx_k_grey51), 0, 1, 0, 1},
  {&__pyx_n_u_grey52, __pyx_k_grey52, sizeof(__pyx_k_grey52), 0, 1, 0, 1},
  {&__pyx_n_u_grey53, __pyx_k_grey53, sizeof(__pyx_k_grey53), 0, 1, 0, 1},
  {&__pyx_n_u_grey54, __pyx_k_grey54, sizeof(__pyx_k_grey54), 0, 1, 0, 1},
  {&__pyx_n_u_grey55, __pyx_k_grey55, sizeof(__pyx_k_grey55), 0, 1, 0, 1},
  {&__pyx_n_u_grey56, __pyx_k_grey56, sizeof(__pyx_k_grey56), 0, 1, 0, 1},
  {&__pyx_n_u_grey57, __pyx_k_grey57, sizeof(__pyx_k_grey57), 0, 1, 0, 1},
  {&__pyx_n_u_grey58, __pyx_k_grey58, sizeof(__pyx_k_grey58), 0, 1, 0, 1},
  {&__pyx_n_u_grey59, __pyx_k_grey59, sizeof(__pyx_k_grey59), 0, 1, 0, 1},
  {&__pyx_n_u_grey6, __pyx_k_grey6, sizeof(__pyx_k_grey6), 0, 1, 0, 1},
  {&__pyx_n_u_grey60, __pyx_k_grey60, sizeof(__pyx_k_grey60), 0, 1, 0, 1},
  {&__pyx_n_u_grey61, __pyx_k_grey61, sizeof(__pyx_k_grey61), 0, 1, 0, 1},
  {&__pyx_n_u_grey62, __pyx_k_grey62, sizeof(__pyx_k_grey62), 0, 1, 0, 1},
  {&__pyx_n_u_grey63, __pyx_k_grey63, sizeof(__pyx_k_grey63), 0, 1, 0, 1},
  {&__pyx_n_u_grey64, __pyx_k_grey64, sizeof(__pyx_k_grey64), 0, 1, 0, 1},
  {&__pyx_n_u_grey65, __pyx_k_grey65, sizeof(__pyx_k_grey65), 0, 1, 0, 1},
  {&__pyx_n_u_grey66, __pyx_k_grey66, sizeof(__pyx_k_grey66), 0, 1, 0, 1},
  {&__pyx_n_u_grey67, __pyx_k_grey67, sizeof(__pyx_k_grey67), 0, 1, 0, 1},
  {&__pyx_n_u_grey68, __pyx_k_grey68, sizeof(__pyx_k_grey68), 0, 1, 0, 1},
  {&__pyx_n_u_grey69, __pyx_k_grey69, sizeof(__pyx_k_grey69), 0, 1, 0, 1},
  {&__pyx_n_u_grey7, __pyx_k_grey7, sizeof(__pyx_k_grey7), 0, 1, 0, 1},
  {&__pyx_n_u_grey70, __pyx_k_grey70, sizeof(__pyx_k_grey70), 0, 1, 0, 1},
  {&__pyx_n_u_grey71, __pyx_k_grey71, sizeof(__pyx_k_grey71), 0, 1, 0, 1},
  {&__pyx_n_u_grey72, __pyx_k_grey72, sizeof(__pyx_k_grey72), 0, 1, 0, 1},
  {&__pyx_n_u_grey73, __pyx_k_grey73, sizeof(__pyx_k_grey73), 0, 1, 0, 1},
  {&__pyx_n_u_grey74, __pyx_k_grey74, sizeof(__pyx_k_grey74), 0, 1, 0, 1},
  {&__pyx_n_u_grey75, __pyx_k_grey75, sizeof(__pyx_k_grey75), 0, 1, 0, 1},
  {&__pyx_n_u_grey76, __pyx_k_grey76, sizeof(__pyx_k_grey76), 0, 1, 0, 1},
  {&__pyx_n_u_grey77, __pyx_k_grey77, sizeof(__pyx_k_grey77), 0, 1, 0, 1},
  {&__pyx_n_u_grey78, __pyx_k_grey78, sizeof(__pyx_k_grey78), 0, 1, 0, 1},
  {&__pyx_n_u_grey79, __pyx_k_grey79, sizeof(__pyx_k_grey79), 0, 1, 0, 1},
  {&__pyx_n_u_grey8, __pyx_k_grey8, sizeof(__pyx_k_grey8), 0, 1, 0, 1},
  {&__pyx_n_u_grey80, __pyx_k_grey80, sizeof(__pyx_k_grey80), 0, 1, 0, 1},
  {&__pyx_n_u_grey81, __pyx_k_grey81, sizeof(__pyx_k_grey81), 0, 1, 0, 1},
  {&__pyx_n_u_grey82, __pyx_k_grey82, sizeof(__pyx_k_grey82), 0, 1, 0, 1},
  {&__pyx_n_u_grey83, __pyx_k_grey83, sizeof(__pyx_k_grey83), 0, 1, 0, 1},
  {&__pyx_n_u_grey84, __pyx_k_grey84, sizeof(__pyx_k_grey84), 0, 1, 0, 1},
  {&__pyx_n_u_grey85, __pyx_k_grey85, sizeof(__pyx_k_grey85), 0, 1, 0, 1},
  {&__pyx_n_u_grey86, __pyx_k_grey86, sizeof(__pyx_k_grey86), 0, 1, 0, 1},
  {&__pyx_n_u_grey87, __pyx_k_grey87, sizeof(__pyx_k_grey87), 0, 1, 0, 1},
  {&__pyx_n_u_grey88, __pyx_k_grey88, sizeof(__pyx_k_grey88), 0, 1, 0, 1},
  {&__pyx_n_u_grey89, __pyx_k_grey89, sizeof(__pyx_k_grey89), 0, 1, 0, 1},
  {&__pyx_n_u_grey9, __pyx_k_grey9, sizeof(__pyx_k_grey9), 0, 1, 0, 1},
  {&__pyx_n_u_grey90, __pyx_k_grey90, sizeof(__pyx_k_grey90), 0, 1, 0, 1},
  {&__pyx_n_u_grey91, __pyx_k_grey91, sizeof(__pyx_k_grey91), 0, 1, 0, 1},
  {&__pyx_n_u_grey92, __pyx_k_grey92, sizeof(__pyx_k_grey92), 0, 1, 0, 1},
  {&__pyx_n_u_grey93, __pyx_k_grey93, sizeof(__pyx_k_grey93), 0, 1, 0, 1},
  {&__pyx_n_u_grey94, __pyx_k_grey94, sizeof(__pyx_k_grey94), 0, 1, 0, 1},
  {&__pyx_n_u_grey95, __pyx_k_grey95, sizeof(__pyx_k_grey95), 0, 1, 0, 1},
  {&__pyx_n_u_grey96, __pyx_k_grey96, sizeof(__pyx_k_grey96), 0, 1, 0, 1},
  {&__pyx_n_u_grey97, __pyx_k_grey97, sizeof(__pyx_k_grey97), 0, 1, 0, 1},
  {&__pyx_n_u_grey98, __pyx_k_grey98, sizeof(__pyx_k_grey98), 0, 1, 0, 1},
  {&__pyx_n_u_grey99, __pyx_k_grey99, sizeof(__pyx_k_grey99), 0, 1, 0, 1},
  {&__pyx_n_s_hex, __pyx_k_hex, sizeof(__pyx_k_hex), 0, 0, 1, 1},
  {&__pyx_n_u_honeydew, __pyx_k_honeydew, sizeof(__pyx_k_honeydew), 0, 1, 0, 1},
  {&__pyx_n_u_honeydew1, __pyx_k_honeydew1, sizeof(__pyx_k_honeydew1), 0, 1, 0, 1},
  {&__pyx_n_u_honeydew2, __pyx_k_honeydew2, sizeof(__pyx_k_honeydew2), 0, 1, 0, 1},
  {&__pyx_n_u_honeydew3, __pyx_k_honeydew3, sizeof(__pyx_k_honeydew3), 0, 1, 0, 1},
  {&__pyx_n_u_honeydew4, __pyx_k_honeydew4, sizeof(__pyx_k_honeydew4), 0, 1, 0, 1},
  {&__pyx_n_u_hotpink, __pyx_k_hotpink, sizeof(__pyx_k_hotpink), 0, 1, 0, 1},
  {&__pyx_n_u_hotpink1, __pyx_k_hotpink1, sizeof(__pyx_k_hotpink1), 0, 1, 0, 1},
  {&__pyx_n_u_hotpink2, __pyx_k_hotpink2, sizeof(__pyx_k_hotpink2), 0, 1, 0, 1},
  {&__pyx_n_u_hotpink3, __pyx_k_hotpink3, sizeof(__pyx_k_hotpink3), 0, 1, 0, 1},
  {&__pyx_n_u_hotpink4, __pyx_k_hotpink4, sizeof(__pyx_k_hotpink4), 0, 1, 0, 1},
  {&__pyx_n_s_hsva, __pyx_k_hsva, sizeof(__pyx_k_hsva), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_u_indianred, __pyx_k_indianred, sizeof(__pyx_k_indianred), 0, 1, 0, 1},
  {&__pyx_n_u_indianred1, __pyx_k_indianred1, sizeof(__pyx_k_indianred1), 0, 1, 0, 1},
  {&__pyx_n_u_indianred2, __pyx_k_indianred2, sizeof(__pyx_k_indianred2), 0, 1, 0, 1},
  {&__pyx_n_u_indianred3, __pyx_k_indianred3, sizeof(__pyx_k_indianred3), 0, 1, 0, 1},
  {&__pyx_n_u_indianred4, __pyx_k_indianred4, sizeof(__pyx_k_indianred4), 0, 1, 0, 1},
  {&__pyx_n_u_ivory, __pyx_k_ivory, sizeof(__pyx_k_ivory), 0, 1, 0, 1},
  {&__pyx_n_u_ivory1, __pyx_k_ivory1, sizeof(__pyx_k_ivory1), 0, 1, 0, 1},
  {&__pyx_n_u_ivory2, __pyx_k_ivory2, sizeof(__pyx_k_ivory2), 0, 1, 0, 1},
  {&__pyx_n_u_ivory3, __pyx_k_ivory3, sizeof(__pyx_k_ivory3), 0, 1, 0, 1},
  {&__pyx_n_u_ivory4, __pyx_k_ivory4, sizeof(__pyx_k_ivory4), 0, 1, 0, 1},
  {&__pyx_n_u_khaki, __pyx_k_khaki, sizeof(__pyx_k_khaki), 0, 1, 0, 1},
  {&__pyx_n_u_khaki1, __pyx_k_khaki1, sizeof(__pyx_k_khaki1), 0, 1, 0, 1},
  {&__pyx_n_u_khaki2, __pyx_k_khaki2, sizeof(__pyx_k_khaki2), 0, 1, 0, 1},
  {&__pyx_n_u_khaki3, __pyx_k_khaki3, sizeof(__pyx_k_khaki3), 0, 1, 0, 1},
  {&__pyx_n_u_khaki4, __pyx_k_khaki4, sizeof(__pyx_k_khaki4), 0, 1, 0, 1},
  {&__pyx_n_s_l, __pyx_k_l, sizeof(__pyx_k_l), 0, 0, 1, 1},
  {&__pyx_n_u_lavender, __pyx_k_lavender, sizeof(__pyx_k_lavender), 0, 1, 0, 1},
  {&__pyx_n_u_lavenderblush, __pyx_k_lavenderblush, sizeof(__pyx_k_lavenderblush), 0, 1, 0, 1},
  {&__pyx_n_u_lavenderblush1, __pyx_k_lavenderblush1, sizeof(__pyx_k_lavenderblush1), 0, 1, 0, 1},
  {&__pyx_n_u_lavenderblush2, __pyx_k_lavenderblush2, sizeof(__pyx_k_lavenderblush2), 0, 1, 0, 1},
  {&__pyx_n_u_lavenderblush3, __pyx_k_lavenderblush3, sizeof(__pyx_k_lavenderblush3), 0, 1, 0, 1},
  {&__pyx_n_u_lavenderblush4, __pyx_k_lavenderblush4, sizeof(__pyx_k_lavenderblush4), 0, 1, 0, 1},
  {&__pyx_n_u_lawngreen, __pyx_k_lawngreen, sizeof(__pyx_k_lawngreen), 0, 1, 0, 1},
  {&__pyx_n_u_lemonchiffon, __pyx_k_lemonchiffon, sizeof(__pyx_k_lemonchiffon), 0, 1, 0, 1},
  {&__pyx_n_u_lemonchiffon1, __pyx_k_lemonchiffon1, sizeof(__pyx_k_lemonchiffon1), 0, 1, 0, 1},
  {&__pyx_n_u_lemonchiffon2, __pyx_k_lemonchiffon2, sizeof(__pyx_k_lemonchiffon2), 0, 1, 0, 1},
  {&__pyx_n_u_lemonchiffon3, __pyx_k_lemonchiffon3, sizeof(__pyx_k_lemonchiffon3), 0, 1, 0, 1},
  {&__pyx_n_u_lemonchiffon4, __pyx_k_lemonchiffon4, sizeof(__pyx_k_lemonchiffon4), 0, 1, 0, 1},
  {&__pyx_n_u_lightblue, __pyx_k_lightblue, sizeof(__pyx_k_lightblue), 0, 1, 0, 1},
  {&__pyx_n_u_lightblue1, __pyx_k_lightblue1, sizeof(__pyx_k_lightblue1), 0, 1, 0, 1},
  {&__pyx_n_u_lightblue2, __pyx_k_lightblue2, sizeof(__pyx_k_lightblue2), 0, 1, 0, 1},
  {&__pyx_n_u_lightblue3, __pyx_k_lightblue3, sizeof(__pyx_k_lightblue3), 0, 1, 0, 1},
  {&__pyx_n_u_lightblue4, __pyx_k_lightblue4, sizeof(__pyx_k_lightblue4), 0, 1, 0, 1},
  {&__pyx_n_u_lightcoral, __pyx_k_lightcoral, sizeof(__pyx_k_lightcoral), 0, 1, 0, 1},
  {&__pyx_n_u_lightcyan, __pyx_k_lightcyan, sizeof(__pyx_k_lightcyan), 0, 1, 0, 1},
  {&__pyx_n_u_lightcyan1, __pyx_k_lightcyan1, sizeof(__pyx_k_lightcyan1), 0, 1, 0, 1},
  {&__pyx_n_u_lightcyan2, __pyx_k_lightcyan2, sizeof(__pyx_k_lightcyan2), 0, 1, 0, 1},
  {&__pyx_n_u_lightcyan3, __pyx_k_lightcyan3, sizeof(__pyx_k_lightcyan3), 0, 1, 0, 1},
  {&__pyx_n_u_lightcyan4, __pyx_k_lightcyan4, sizeof(__pyx_k_lightcyan4), 0, 1, 0, 1},
  {&__pyx_n_u_lightgoldenrod, __pyx_k_lightgoldenrod, sizeof(__pyx_k_lightgoldenrod), 0, 1, 0, 1},
  {&__pyx_n_u_lightgoldenrod1, __pyx_k_lightgoldenrod1, sizeof(__pyx_k_lightgoldenrod1), 0, 1, 0, 1},
  {&__pyx_n_u_lightgoldenrod2, __pyx_k_lightgoldenrod2, sizeof(__pyx_k_lightgoldenrod2), 0, 1, 0, 1},
  {&__pyx_n_u_lightgoldenrod3, __pyx_k_lightgoldenrod3, sizeof(__pyx_k_lightgoldenrod3), 0, 1, 0, 1},
  {&__pyx_n_u_lightgoldenrod4, __pyx_k_lightgoldenrod4, sizeof(__pyx_k_lightgoldenrod4), 0, 1, 0, 1},
  {&__pyx_n_u_lightgoldenrodyellow, __pyx_k_lightgoldenrodyellow, sizeof(__pyx_k_lightgoldenrodyellow), 0, 1, 0, 1},
  {&__pyx_n_u_lightgray, __pyx_k_lightgray, sizeof(__pyx_k_lightgray), 0, 1, 0, 1},
  {&__pyx_n_u_lightgreen, __pyx_k_lightgreen, sizeof(__pyx_k_lightgreen), 0, 1, 0, 1},
  {&__pyx_n_u_lightgrey, __pyx_k_lightgrey, sizeof(__pyx_k_lightgrey), 0, 1, 0, 1},
  {&__pyx_n_u_lightpink, __pyx_k_lightpink, sizeof(__pyx_k_lightpink), 0, 1, 0, 1},
  {&__pyx_n_u_lightpink1, __pyx_k_lightpink1, sizeof(__pyx_k_lightpink1), 0, 1, 0, 1},
  {&__pyx_n_u_lightpink2, __pyx_k_lightpink2, sizeof(__pyx_k_lightpink2), 0, 1, 0, 1},
  {&__pyx_n_u_lightpink3, __pyx_k_lightpink3, sizeof(__pyx_k_lightpink3), 0, 1, 0, 1},
  {&__pyx_n_u_lightpink4, __pyx_k_lightpink4, sizeof(__pyx_k_lightpink4), 0, 1, 0, 1},
  {&__pyx_n_u_lightsalmon, __pyx_k_lightsalmon, sizeof(__pyx_k_lightsalmon), 0, 1, 0, 1},
  {&__pyx_n_u_lightsalmon1, __pyx_k_lightsalmon1, sizeof(__pyx_k_lightsalmon1), 0, 1, 0, 1},
  {&__pyx_n_u_lightsalmon2, __pyx_k_lightsalmon2, sizeof(__pyx_k_lightsalmon2), 0, 1, 0, 1},
  {&__pyx_n_u_lightsalmon3, __pyx_k_lightsalmon3, sizeof(__pyx_k_lightsalmon3), 0, 1, 0, 1},
  {&__pyx_n_u_lightsalmon4, __pyx_k_lightsalmon4, sizeof(__pyx_k_lightsalmon4), 0, 1, 0, 1},
  {&__pyx_n_u_lightseagreen, __pyx_k_lightseagreen, sizeof(__pyx_k_lightseagreen), 0, 1, 0, 1},
  {&__pyx_n_u_lightskyblue, __pyx_k_lightskyblue, sizeof(__pyx_k_lightskyblue), 0, 1, 0, 1},
  {&__pyx_n_u_lightskyblue1, __pyx_k_lightskyblue1, sizeof(__pyx_k_lightskyblue1), 0, 1, 0, 1},
  {&__pyx_n_u_lightskyblue2, __pyx_k_lightskyblue2, sizeof(__pyx_k_lightskyblue2), 0, 1, 0, 1},
  {&__pyx_n_u_lightskyblue3, __pyx_k_lightskyblue3, sizeof(__pyx_k_lightskyblue3), 0, 1, 0, 1},
  {&__pyx_n_u_lightskyblue4, __pyx_k_lightskyblue4, sizeof(__pyx_k_lightskyblue4), 0, 1, 0, 1},
  {&__pyx_n_u_lightslateblue, __pyx_k_lightslateblue, sizeof(__pyx_k_lightslateblue), 0, 1, 0, 1},
  {&__pyx_n_u_lightslategray, __pyx_k_lightslategray, sizeof(__pyx_k_lightslategray), 0, 1, 0, 1},
  {&__pyx_n_u_lightslategrey, __pyx_k_lightslategrey, sizeof(__pyx_k_lightslategrey), 0, 1, 0, 1},
  {&__pyx_n_u_lightsteelblue, __pyx_k_lightsteelblue, sizeof(__pyx_k_lightsteelblue), 0, 1, 0, 1},
  {&__pyx_n_u_lightsteelblue1, __pyx_k_lightsteelblue1, sizeof(__pyx_k_lightsteelblue1), 0, 1, 0, 1},
  {&__pyx_n_u_lightsteelblue2, __pyx_k_lightsteelblue2, sizeof(__pyx_k_lightsteelblue2), 0, 1, 0, 1},
  {&__pyx_n_u_lightsteelblue3, __pyx_k_lightsteelblue3, sizeof(__pyx_k_lightsteelblue3), 0, 1, 0, 1},
  {&__pyx_n_u_lightsteelblue4, __pyx_k_lightsteelblue4, sizeof(__pyx_k_lightsteelblue4), 0, 1, 0, 1},
  {&__pyx_n_u_lightyellow, __pyx_k_lightyellow, sizeof(__pyx_k_lightyellow), 0, 1, 0, 1},
  {&__pyx_n_u_lightyellow1, __pyx_k_lightyellow1, sizeof(__pyx_k_lightyellow1), 0, 1, 0, 1},
  {&__pyx_n_u_lightyellow2, __pyx_k_lightyellow2, sizeof(__pyx_k_lightyellow2), 0, 1, 0, 1},
  {&__pyx_n_u_lightyellow3, __pyx_k_lightyellow3, sizeof(__pyx_k_lightyellow3), 0, 1, 0, 1},
  {&__pyx_n_u_lightyellow4, __pyx_k_lightyellow4, sizeof(__pyx_k_lightyellow4), 0, 1, 0, 1},
  {&__pyx_n_u_limegreen, __pyx_k_limegreen, sizeof(__pyx_k_limegreen), 0, 1, 0, 1},
  {&__pyx_n_u_linen, __pyx_k_linen, sizeof(__pyx_k_linen), 0, 1, 0, 1},
  {&__pyx_n_s_lower, __pyx_k_lower, sizeof(__pyx_k_lower), 0, 0, 1, 1},
  {&__pyx_n_u_magenta, __pyx_k_magenta, sizeof(__pyx_k_magenta), 0, 1, 0, 1},
  {&__pyx_n_u_magenta1, __pyx_k_magenta1, sizeof(__pyx_k_magenta1), 0, 1, 0, 1},
  {&__pyx_n_u_magenta2, __pyx_k_magenta2, sizeof(__pyx_k_magenta2), 0, 1, 0, 1},
  {&__pyx_n_u_magenta3, __pyx_k_magenta3, sizeof(__pyx_k_magenta3), 0, 1, 0, 1},
  {&__pyx_n_u_magenta4, __pyx_k_magenta4, sizeof(__pyx_k_magenta4), 0, 1, 0, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_map, __pyx_k_map, sizeof(__pyx_k_map), 0, 0, 1, 1},
  {&__pyx_n_u_maroon, __pyx_k_maroon, sizeof(__pyx_k_maroon), 0, 1, 0, 1},
  {&__pyx_n_u_maroon1, __pyx_k_maroon1, sizeof(__pyx_k_maroon1), 0, 1, 0, 1},
  {&__pyx_n_u_maroon2, __pyx_k_maroon2, sizeof(__pyx_k_maroon2), 0, 1, 0, 1},
  {&__pyx_n_u_maroon3, __pyx_k_maroon3, sizeof(__pyx_k_maroon3), 0, 1, 0, 1},
  {&__pyx_n_u_maroon4, __pyx_k_maroon4, sizeof(__pyx_k_maroon4), 0, 1, 0, 1},
  {&__pyx_n_u_mediumaquamarine, __pyx_k_mediumaquamarine, sizeof(__pyx_k_mediumaquamarine), 0, 1, 0, 1},
  {&__pyx_n_u_mediumblue, __pyx_k_mediumblue, sizeof(__pyx_k_mediumblue), 0, 1, 0, 1},
  {&__pyx_n_u_mediumorchid, __pyx_k_mediumorchid, sizeof(__pyx_k_mediumorchid), 0, 1, 0, 1},
  {&__pyx_n_u_mediumorchid1, __pyx_k_mediumorchid1, sizeof(__pyx_k_mediumorchid1), 0, 1, 0, 1},
  {&__pyx_n_u_mediumorchid2, __pyx_k_mediumorchid2, sizeof(__pyx_k_mediumorchid2), 0, 1, 0, 1},
  {&__pyx_n_u_mediumorchid3, __pyx_k_mediumorchid3, sizeof(__pyx_k_mediumorchid3), 0, 1, 0, 1},
  {&__pyx_n_u_mediumorchid4, __pyx_k_mediumorchid4, sizeof(__pyx_k_mediumorchid4), 0, 1, 0, 1},
  {&__pyx_n_u_mediumpurple, __pyx_k_mediumpurple, sizeof(__pyx_k_mediumpurple), 0, 1, 0, 1},
  {&__pyx_n_u_mediumpurple1, __pyx_k_mediumpurple1, sizeof(__pyx_k_mediumpurple1), 0, 1, 0, 1},
  {&__pyx_n_u_mediumpurple2, __pyx_k_mediumpurple2, sizeof(__pyx_k_mediumpurple2), 0, 1, 0, 1},
  {&__pyx_n_u_mediumpurple3, __pyx_k_mediumpurple3, sizeof(__pyx_k_mediumpurple3), 0, 1, 0, 1},
  {&__pyx_n_u_mediumpurple4, __pyx_k_mediumpurple4, sizeof(__pyx_k_mediumpurple4), 0, 1, 0, 1},
  {&__pyx_n_u_mediumseagreen, __pyx_k_mediumseagreen, sizeof(__pyx_k_mediumseagreen), 0, 1, 0, 1},
  {&__pyx_n_u_mediumslateblue, __pyx_k_mediumslateblue, sizeof(__pyx_k_mediumslateblue), 0, 1, 0, 1},
  {&__pyx_n_u_mediumspringgreen, __pyx_k_mediumspringgreen, sizeof(__pyx_k_mediumspringgreen), 0, 1, 0, 1},
  {&__pyx_n_u_mediumturquoise, __pyx_k_mediumturquoise, sizeof(__pyx_k_mediumturquoise), 0, 1, 0, 1},
  {&__pyx_n_u_mediumvioletred, __pyx_k_mediumvioletred, sizeof(__pyx_k_mediumvioletred), 0, 1, 0, 1},
  {&__pyx_n_u_midnightblue, __pyx_k_midnightblue, sizeof(__pyx_k_midnightblue), 0, 1, 0, 1},
  {&__pyx_n_u_mintcream, __pyx_k_mintcream, sizeof(__pyx_k_mintcream), 0, 1, 0, 1},
  {&__pyx_n_u_mistyrose, __pyx_k_mistyrose, sizeof(__pyx_k_mistyrose), 0, 1, 0, 1},
  {&__pyx_n_u_mistyrose1, __pyx_k_mistyrose1, sizeof(__pyx_k_mistyrose1), 0, 1, 0, 1},
  {&__pyx_n_u_mistyrose2, __pyx_k_mistyrose2, sizeof(__pyx_k_mistyrose2), 0, 1, 0, 1},
  {&__pyx_n_u_mistyrose3, __pyx_k_mistyrose3, sizeof(__pyx_k_mistyrose3), 0, 1, 0, 1},
  {&__pyx_n_u_mistyrose4, __pyx_k_mistyrose4, sizeof(__pyx_k_mistyrose4), 0, 1, 0, 1},
  {&__pyx_n_u_moccasin, __pyx_k_moccasin, sizeof(__pyx_k_moccasin), 0, 1, 0, 1},
  {&__pyx_n_s_mod, __pyx_k_mod, sizeof(__pyx_k_mod), 0, 0, 1, 1},
  {&__pyx_n_s_mod___locals_mod, __pyx_k_mod___locals_mod, sizeof(__pyx_k_mod___locals_mod), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_u_navajowhite, __pyx_k_navajowhite, sizeof(__pyx_k_navajowhite), 0, 1, 0, 1},
  {&__pyx_n_u_navajowhite1, __pyx_k_navajowhite1, sizeof(__pyx_k_navajowhite1), 0, 1, 0, 1},
  {&__pyx_n_u_navajowhite2, __pyx_k_navajowhite2, sizeof(__pyx_k_navajowhite2), 0, 1, 0, 1},
  {&__pyx_n_u_navajowhite3, __pyx_k_navajowhite3, sizeof(__pyx_k_navajowhite3), 0, 1, 0, 1},
  {&__pyx_n_u_navajowhite4, __pyx_k_navajowhite4, sizeof(__pyx_k_navajowhite4), 0, 1, 0, 1},
  {&__pyx_n_u_navy, __pyx_k_navy, sizeof(__pyx_k_navy), 0, 1, 0, 1},
  {&__pyx_n_u_navyblue, __pyx_k_navyblue, sizeof(__pyx_k_navyblue), 0, 1, 0, 1},
  {&__pyx_n_s_oct, __pyx_k_oct, sizeof(__pyx_k_oct), 0, 0, 1, 1},
  {&__pyx_n_u_oldlace, __pyx_k_oldlace, sizeof(__pyx_k_oldlace), 0, 1, 0, 1},
  {&__pyx_n_u_olivedrab, __pyx_k_olivedrab, sizeof(__pyx_k_olivedrab), 0, 1, 0, 1},
  {&__pyx_n_u_olivedrab1, __pyx_k_olivedrab1, sizeof(__pyx_k_olivedrab1), 0, 1, 0, 1},
  {&__pyx_n_u_olivedrab2, __pyx_k_olivedrab2, sizeof(__pyx_k_olivedrab2), 0, 1, 0, 1},
  {&__pyx_n_u_olivedrab3, __pyx_k_olivedrab3, sizeof(__pyx_k_olivedrab3), 0, 1, 0, 1},
  {&__pyx_n_u_olivedrab4, __pyx_k_olivedrab4, sizeof(__pyx_k_olivedrab4), 0, 1, 0, 1},
  {&__pyx_n_u_orange, __pyx_k_orange, sizeof(__pyx_k_orange), 0, 1, 0, 1},
  {&__pyx_n_u_orange1, __pyx_k_orange1, sizeof(__pyx_k_orange1), 0, 1, 0, 1},
  {&__pyx_n_u_orange2, __pyx_k_orange2, sizeof(__pyx_k_orange2), 0, 1, 0, 1},
  {&__pyx_n_u_orange3, __pyx_k_orange3, sizeof(__pyx_k_orange3), 0, 1, 0, 1},
  {&__pyx_n_u_orange4, __pyx_k_orange4, sizeof(__pyx_k_orange4), 0, 1, 0, 1},
  {&__pyx_n_u_orangered, __pyx_k_orangered, sizeof(__pyx_k_orangered), 0, 1, 0, 1},
  {&__pyx_n_u_orangered1, __pyx_k_orangered1, sizeof(__pyx_k_orangered1), 0, 1, 0, 1},
  {&__pyx_n_u_orangered2, __pyx_k_orangered2, sizeof(__pyx_k_orangered2), 0, 1, 0, 1},
  {&__pyx_n_u_orangered3, __pyx_k_orangered3, sizeof(__pyx_k_orangered3), 0, 1, 0, 1},
  {&__pyx_n_u_orangered4, __pyx_k_orangered4, sizeof(__pyx_k_orangered4), 0, 1, 0, 1},
  {&__pyx_n_u_orchid, __pyx_k_orchid, sizeof(__pyx_k_orchid), 0, 1, 0, 1},
  {&__pyx_n_u_orchid1, __pyx_k_orchid1, sizeof(__pyx_k_orchid1), 0, 1, 0, 1},
  {&__pyx_n_u_orchid2, __pyx_k_orchid2, sizeof(__pyx_k_orchid2), 0, 1, 0, 1},
  {&__pyx_n_u_orchid3, __pyx_k_orchid3, sizeof(__pyx_k_orchid3), 0, 1, 0, 1},
  {&__pyx_n_u_orchid4, __pyx_k_orchid4, sizeof(__pyx_k_orchid4), 0, 1, 0, 1},
  {&__pyx_n_s_pack, __pyx_k_pack, sizeof(__pyx_k_pack), 0, 0, 1, 1},
  {&__pyx_n_u_palegoldenrod, __pyx_k_palegoldenrod, sizeof(__pyx_k_palegoldenrod), 0, 1, 0, 1},
  {&__pyx_n_u_palegreen, __pyx_k_palegreen, sizeof(__pyx_k_palegreen), 0, 1, 0, 1},
  {&__pyx_n_u_palegreen1, __pyx_k_palegreen1, sizeof(__pyx_k_palegreen1), 0, 1, 0, 1},
  {&__pyx_n_u_palegreen2, __pyx_k_palegreen2, sizeof(__pyx_k_palegreen2), 0, 1, 0, 1},
  {&__pyx_n_u_palegreen3, __pyx_k_palegreen3, sizeof(__pyx_k_palegreen3), 0, 1, 0, 1},
  {&__pyx_n_u_palegreen4, __pyx_k_palegreen4, sizeof(__pyx_k_palegreen4), 0, 1, 0, 1},
  {&__pyx_n_u_paleturquoise, __pyx_k_paleturquoise, sizeof(__pyx_k_paleturquoise), 0, 1, 0, 1},
  {&__pyx_n_u_paleturquoise1, __pyx_k_paleturquoise1, sizeof(__pyx_k_paleturquoise1), 0, 1, 0, 1},
  {&__pyx_n_u_paleturquoise2, __pyx_k_paleturquoise2, sizeof(__pyx_k_paleturquoise2), 0, 1, 0, 1},
  {&__pyx_n_u_paleturquoise3, __pyx_k_paleturquoise3, sizeof(__pyx_k_paleturquoise3), 0, 1, 0, 1},
  {&__pyx_n_u_paleturquoise4, __pyx_k_paleturquoise4, sizeof(__pyx_k_paleturquoise4), 0, 1, 0, 1},
  {&__pyx_n_u_palevioletred, __pyx_k_palevioletred, sizeof(__pyx_k_palevioletred), 0, 1, 0, 1},
  {&__pyx_n_u_palevioletred1, __pyx_k_palevioletred1, sizeof(__pyx_k_palevioletred1), 0, 1, 0, 1},
  {&__pyx_n_u_palevioletred2, __pyx_k_palevioletred2, sizeof(__pyx_k_palevioletred2), 0, 1, 0, 1},
  {&__pyx_n_u_palevioletred3, __pyx_k_palevioletred3, sizeof(__pyx_k_palevioletred3), 0, 1, 0, 1},
  {&__pyx_n_u_palevioletred4, __pyx_k_palevioletred4, sizeof(__pyx_k_palevioletred4), 0, 1, 0, 1},
  {&__pyx_n_u_papayawhip, __pyx_k_papayawhip, sizeof(__pyx_k_papayawhip), 0, 1, 0, 1},
  {&__pyx_n_u_peachpuff, __pyx_k_peachpuff, sizeof(__pyx_k_peachpuff), 0, 1, 0, 1},
  {&__pyx_n_u_peachpuff1, __pyx_k_peachpuff1, sizeof(__pyx_k_peachpuff1), 0, 1, 0, 1},
  {&__pyx_n_u_peachpuff2, __pyx_k_peachpuff2, sizeof(__pyx_k_peachpuff2), 0, 1, 0, 1},
  {&__pyx_n_u_peachpuff3, __pyx_k_peachpuff3, sizeof(__pyx_k_peachpuff3), 0, 1, 0, 1},
  {&__pyx_n_u_peachpuff4, __pyx_k_peachpuff4, sizeof(__pyx_k_peachpuff4), 0, 1, 0, 1},
  {&__pyx_n_u_peru, __pyx_k_peru, sizeof(__pyx_k_peru), 0, 1, 0, 1},
  {&__pyx_n_u_pink, __pyx_k_pink, sizeof(__pyx_k_pink), 0, 1, 0, 1},
  {&__pyx_n_u_pink1, __pyx_k_pink1, sizeof(__pyx_k_pink1), 0, 1, 0, 1},
  {&__pyx_n_u_pink2, __pyx_k_pink2, sizeof(__pyx_k_pink2), 0, 1, 0, 1},
  {&__pyx_n_u_pink3, __pyx_k_pink3, sizeof(__pyx_k_pink3), 0, 1, 0, 1},
  {&__pyx_n_u_pink4, __pyx_k_pink4, sizeof(__pyx_k_pink4), 0, 1, 0, 1},
  {&__pyx_n_u_plum, __pyx_k_plum, sizeof(__pyx_k_plum), 0, 1, 0, 1},
  {&__pyx_n_u_plum1, __pyx_k_plum1, sizeof(__pyx_k_plum1), 0, 1, 0, 1},
  {&__pyx_n_u_plum2, __pyx_k_plum2, sizeof(__pyx_k_plum2), 0, 1, 0, 1},
  {&__pyx_n_u_plum3, __pyx_k_plum3, sizeof(__pyx_k_plum3), 0, 1, 0, 1},
  {&__pyx_n_u_plum4, __pyx_k_plum4, sizeof(__pyx_k_plum4), 0, 1, 0, 1},
  {&__pyx_n_u_powderblue, __pyx_k_powderblue, sizeof(__pyx_k_powderblue), 0, 1, 0, 1},
  {&__pyx_n_u_purple, __pyx_k_purple, sizeof(__pyx_k_purple), 0, 1, 0, 1},
  {&__pyx_n_u_purple1, __pyx_k_purple1, sizeof(__pyx_k_purple1), 0, 1, 0, 1},
  {&__pyx_n_u_purple2, __pyx_k_purple2, sizeof(__pyx_k_purple2), 0, 1, 0, 1},
  {&__pyx_n_u_purple3, __pyx_k_purple3, sizeof(__pyx_k_purple3), 0, 1, 0, 1},
  {&__pyx_n_u_purple4, __pyx_k_purple4, sizeof(__pyx_k_purple4), 0, 1, 0, 1},
  {&__pyx_n_s_pygame_sdl2_color, __pyx_k_pygame_sdl2_color, sizeof(__pyx_k_pygame_sdl2_color), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 0, 1, 1},
  {&__pyx_n_u_red, __pyx_k_red, sizeof(__pyx_k_red), 0, 1, 0, 1},
  {&__pyx_n_u_red1, __pyx_k_red1, sizeof(__pyx_k_red1), 0, 1, 0, 1},
  {&__pyx_n_u_red2, __pyx_k_red2, sizeof(__pyx_k_red2), 0, 1, 0, 1},
  {&__pyx_n_u_red3, __pyx_k_red3, sizeof(__pyx_k_red3), 0, 1, 0, 1},
  {&__pyx_n_u_red4, __pyx_k_red4, sizeof(__pyx_k_red4), 0, 1, 0, 1},
  {&__pyx_n_u_rgba, __pyx_k_rgba, sizeof(__pyx_k_rgba), 0, 1, 0, 1},
  {&__pyx_n_u_rosybrown, __pyx_k_rosybrown, sizeof(__pyx_k_rosybrown), 0, 1, 0, 1},
  {&__pyx_n_u_rosybrown1, __pyx_k_rosybrown1, sizeof(__pyx_k_rosybrown1), 0, 1, 0, 1},
  {&__pyx_n_u_rosybrown2, __pyx_k_rosybrown2, sizeof(__pyx_k_rosybrown2), 0, 1, 0, 1},
  {&__pyx_n_u_rosybrown3, __pyx_k_rosybrown3, sizeof(__pyx_k_rosybrown3), 0, 1, 0, 1},
  {&__pyx_n_u_rosybrown4, __pyx_k_rosybrown4, sizeof(__pyx_k_rosybrown4), 0, 1, 0, 1},
  {&__pyx_n_s_round, __pyx_k_round, sizeof(__pyx_k_round), 0, 0, 1, 1},
  {&__pyx_n_u_royalblue, __pyx_k_royalblue, sizeof(__pyx_k_royalblue), 0, 1, 0, 1},
  {&__pyx_n_u_royalblue1, __pyx_k_royalblue1, sizeof(__pyx_k_royalblue1), 0, 1, 0, 1},
  {&__pyx_n_u_royalblue2, __pyx_k_royalblue2, sizeof(__pyx_k_royalblue2), 0, 1, 0, 1},
  {&__pyx_n_u_royalblue3, __pyx_k_royalblue3, sizeof(__pyx_k_royalblue3), 0, 1, 0, 1},
  {&__pyx_n_u_royalblue4, __pyx_k_royalblue4, sizeof(__pyx_k_royalblue4), 0, 1, 0, 1},
  {&__pyx_n_u_saddlebrown, __pyx_k_saddlebrown, sizeof(__pyx_k_saddlebrown), 0, 1, 0, 1},
  {&__pyx_n_u_salmon, __pyx_k_salmon, sizeof(__pyx_k_salmon), 0, 1, 0, 1},
  {&__pyx_n_u_salmon1, __pyx_k_salmon1, sizeof(__pyx_k_salmon1), 0, 1, 0, 1},
  {&__pyx_n_u_salmon2, __pyx_k_salmon2, sizeof(__pyx_k_salmon2), 0, 1, 0, 1},
  {&__pyx_n_u_salmon3, __pyx_k_salmon3, sizeof(__pyx_k_salmon3), 0, 1, 0, 1},
  {&__pyx_n_u_salmon4, __pyx_k_salmon4, sizeof(__pyx_k_salmon4), 0, 1, 0, 1},
  {&__pyx_n_u_sandybrown, __pyx_k_sandybrown, sizeof(__pyx_k_sandybrown), 0, 1, 0, 1},
  {&__pyx_n_u_seagreen, __pyx_k_seagreen, sizeof(__pyx_k_seagreen), 0, 1, 0, 1},
  {&__pyx_n_u_seagreen1, __pyx_k_seagreen1, sizeof(__pyx_k_seagreen1), 0, 1, 0, 1},
  {&__pyx_n_u_seagreen2, __pyx_k_seagreen2, sizeof(__pyx_k_seagreen2), 0, 1, 0, 1},
  {&__pyx_n_u_seagreen3, __pyx_k_seagreen3, sizeof(__pyx_k_seagreen3), 0, 1, 0, 1},
  {&__pyx_n_u_seagreen4, __pyx_k_seagreen4, sizeof(__pyx_k_seagreen4), 0, 1, 0, 1},
  {&__pyx_n_u_seashell, __pyx_k_seashell, sizeof(__pyx_k_seashell), 0, 1, 0, 1},
  {&__pyx_n_u_seashell1, __pyx_k_seashell1, sizeof(__pyx_k_seashell1), 0, 1, 0, 1},
  {&__pyx_n_u_seashell2, __pyx_k_seashell2, sizeof(__pyx_k_seashell2), 0, 1, 0, 1},
  {&__pyx_n_u_seashell3, __pyx_k_seashell3, sizeof(__pyx_k_seashell3), 0, 1, 0, 1},
  {&__pyx_n_u_seashell4, __pyx_k_seashell4, sizeof(__pyx_k_seashell4), 0, 1, 0, 1},
  {&__pyx_n_u_sienna, __pyx_k_sienna, sizeof(__pyx_k_sienna), 0, 1, 0, 1},
  {&__pyx_n_u_sienna1, __pyx_k_sienna1, sizeof(__pyx_k_sienna1), 0, 1, 0, 1},
  {&__pyx_n_u_sienna2, __pyx_k_sienna2, sizeof(__pyx_k_sienna2), 0, 1, 0, 1},
  {&__pyx_n_u_sienna3, __pyx_k_sienna3, sizeof(__pyx_k_sienna3), 0, 1, 0, 1},
  {&__pyx_n_u_sienna4, __pyx_k_sienna4, sizeof(__pyx_k_sienna4), 0, 1, 0, 1},
  {&__pyx_n_u_skyblue, __pyx_k_skyblue, sizeof(__pyx_k_skyblue), 0, 1, 0, 1},
  {&__pyx_n_u_skyblue1, __pyx_k_skyblue1, sizeof(__pyx_k_skyblue1), 0, 1, 0, 1},
  {&__pyx_n_u_skyblue2, __pyx_k_skyblue2, sizeof(__pyx_k_skyblue2), 0, 1, 0, 1},
  {&__pyx_n_u_skyblue3, __pyx_k_skyblue3, sizeof(__pyx_k_skyblue3), 0, 1, 0, 1},
  {&__pyx_n_u_skyblue4, __pyx_k_skyblue4, sizeof(__pyx_k_skyblue4), 0, 1, 0, 1},
  {&__pyx_n_u_slateblue, __pyx_k_slateblue, sizeof(__pyx_k_slateblue), 0, 1, 0, 1},
  {&__pyx_n_u_slateblue1, __pyx_k_slateblue1, sizeof(__pyx_k_slateblue1), 0, 1, 0, 1},
  {&__pyx_n_u_slateblue2, __pyx_k_slateblue2, sizeof(__pyx_k_slateblue2), 0, 1, 0, 1},
  {&__pyx_n_u_slateblue3, __pyx_k_slateblue3, sizeof(__pyx_k_slateblue3), 0, 1, 0, 1},
  {&__pyx_n_u_slateblue4, __pyx_k_slateblue4, sizeof(__pyx_k_slateblue4), 0, 1, 0, 1},
  {&__pyx_n_u_slategray, __pyx_k_slategray, sizeof(__pyx_k_slategray), 0, 1, 0, 1},
  {&__pyx_n_u_slategray1, __pyx_k_slategray1, sizeof(__pyx_k_slategray1), 0, 1, 0, 1},
  {&__pyx_n_u_slategray2, __pyx_k_slategray2, sizeof(__pyx_k_slategray2), 0, 1, 0, 1},
  {&__pyx_n_u_slategray3, __pyx_k_slategray3, sizeof(__pyx_k_slategray3), 0, 1, 0, 1},
  {&__pyx_n_u_slategray4, __pyx_k_slategray4, sizeof(__pyx_k_slategray4), 0, 1, 0, 1},
  {&__pyx_n_u_slategrey, __pyx_k_slategrey, sizeof(__pyx_k_slategrey), 0, 1, 0, 1},
  {&__pyx_n_u_snow, __pyx_k_snow, sizeof(__pyx_k_snow), 0, 1, 0, 1},
  {&__pyx_n_u_snow1, __pyx_k_snow1, sizeof(__pyx_k_snow1), 0, 1, 0, 1},
  {&__pyx_n_u_snow2, __pyx_k_snow2, sizeof(__pyx_k_snow2), 0, 1, 0, 1},
  {&__pyx_n_u_snow3, __pyx_k_snow3, sizeof(__pyx_k_snow3), 0, 1, 0, 1},
  {&__pyx_n_u_snow4, __pyx_k_snow4, sizeof(__pyx_k_snow4), 0, 1, 0, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_u_springgreen, __pyx_k_springgreen, sizeof(__pyx_k_springgreen), 0, 1, 0, 1},
  {&__pyx_n_u_springgreen1, __pyx_k_springgreen1, sizeof(__pyx_k_springgreen1), 0, 1, 0, 1},
  {&__pyx_n_u_springgreen2, __pyx_k_springgreen2, sizeof(__pyx_k_springgreen2), 0, 1, 0, 1},
  {&__pyx_n_u_springgreen3, __pyx_k_springgreen3, sizeof(__pyx_k_springgreen3), 0, 1, 0, 1},
  {&__pyx_n_u_springgreen4, __pyx_k_springgreen4, sizeof(__pyx_k_springgreen4), 0, 1, 0, 1},
  {&__pyx_kp_s_src_pygame_sdl2_color_pyx, __pyx_k_src_pygame_sdl2_color_pyx, sizeof(__pyx_k_src_pygame_sdl2_color_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_startswith, __pyx_k_startswith, sizeof(__pyx_k_startswith), 0, 0, 1, 1},
  {&__pyx_n_u_steelblue, __pyx_k_steelblue, sizeof(__pyx_k_steelblue), 0, 1, 0, 1},
  {&__pyx_n_u_steelblue1, __pyx_k_steelblue1, sizeof(__pyx_k_steelblue1), 0, 1, 0, 1},
  {&__pyx_n_u_steelblue2, __pyx_k_steelblue2, sizeof(__pyx_k_steelblue2), 0, 1, 0, 1},
  {&__pyx_n_u_steelblue3, __pyx_k_steelblue3, sizeof(__pyx_k_steelblue3), 0, 1, 0, 1},
  {&__pyx_n_u_steelblue4, __pyx_k_steelblue4, sizeof(__pyx_k_steelblue4), 0, 1, 0, 1},
  {&__pyx_n_s_struct, __pyx_k_struct, sizeof(__pyx_k_struct), 0, 0, 1, 1},
  {&__pyx_n_u_tan, __pyx_k_tan, sizeof(__pyx_k_tan), 0, 1, 0, 1},
  {&__pyx_n_u_tan1, __pyx_k_tan1, sizeof(__pyx_k_tan1), 0, 1, 0, 1},
  {&__pyx_n_u_tan2, __pyx_k_tan2, sizeof(__pyx_k_tan2), 0, 1, 0, 1},
  {&__pyx_n_u_tan3, __pyx_k_tan3, sizeof(__pyx_k_tan3), 0, 1, 0, 1},
  {&__pyx_n_u_tan4, __pyx_k_tan4, sizeof(__pyx_k_tan4), 0, 1, 0, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_u_thistle, __pyx_k_thistle, sizeof(__pyx_k_thistle), 0, 1, 0, 1},
  {&__pyx_n_u_thistle1, __pyx_k_thistle1, sizeof(__pyx_k_thistle1), 0, 1, 0, 1},
  {&__pyx_n_u_thistle2, __pyx_k_thistle2, sizeof(__pyx_k_thistle2), 0, 1, 0, 1},
  {&__pyx_n_u_thistle3, __pyx_k_thistle3, sizeof(__pyx_k_thistle3), 0, 1, 0, 1},
  {&__pyx_n_u_thistle4, __pyx_k_thistle4, sizeof(__pyx_k_thistle4), 0, 1, 0, 1},
  {&__pyx_n_u_tomato, __pyx_k_tomato, sizeof(__pyx_k_tomato), 0, 1, 0, 1},
  {&__pyx_n_u_tomato1, __pyx_k_tomato1, sizeof(__pyx_k_tomato1), 0, 1, 0, 1},
  {&__pyx_n_u_tomato2, __pyx_k_tomato2, sizeof(__pyx_k_tomato2), 0, 1, 0, 1},
  {&__pyx_n_u_tomato3, __pyx_k_tomato3, sizeof(__pyx_k_tomato3), 0, 1, 0, 1},
  {&__pyx_n_u_tomato4, __pyx_k_tomato4, sizeof(__pyx_k_tomato4), 0, 1, 0, 1},
  {&__pyx_n_u_turquoise, __pyx_k_turquoise, sizeof(__pyx_k_turquoise), 0, 1, 0, 1},
  {&__pyx_n_u_turquoise1, __pyx_k_turquoise1, sizeof(__pyx_k_turquoise1), 0, 1, 0, 1},
  {&__pyx_n_u_turquoise2, __pyx_k_turquoise2, sizeof(__pyx_k_turquoise2), 0, 1, 0, 1},
  {&__pyx_n_u_turquoise3, __pyx_k_turquoise3, sizeof(__pyx_k_turquoise3), 0, 1, 0, 1},
  {&__pyx_n_u_turquoise4, __pyx_k_turquoise4, sizeof(__pyx_k_turquoise4), 0, 1, 0, 1},
  {&__pyx_n_s_unhexlify, __pyx_k_unhexlify, sizeof(__pyx_k_unhexlify), 0, 0, 1, 1},
  {&__pyx_n_s_unpack, __pyx_k_unpack, sizeof(__pyx_k_unpack), 0, 0, 1, 1},
  {&__pyx_n_u_violet, __pyx_k_violet, sizeof(__pyx_k_violet), 0, 1, 0, 1},
  {&__pyx_n_u_violetred, __pyx_k_violetred, sizeof(__pyx_k_violetred), 0, 1, 0, 1},
  {&__pyx_n_u_violetred1, __pyx_k_violetred1, sizeof(__pyx_k_violetred1), 0, 1, 0, 1},
  {&__pyx_n_u_violetred2, __pyx_k_violetred2, sizeof(__pyx_k_violetred2), 0, 1, 0, 1},
  {&__pyx_n_u_violetred3, __pyx_k_violetred3, sizeof(__pyx_k_violetred3), 0, 1, 0, 1},
  {&__pyx_n_u_violetred4, __pyx_k_violetred4, sizeof(__pyx_k_violetred4), 0, 1, 0, 1},
  {&__pyx_n_u_wheat, __pyx_k_wheat, sizeof(__pyx_k_wheat), 0, 1, 0, 1},
  {&__pyx_n_u_wheat1, __pyx_k_wheat1, sizeof(__pyx_k_wheat1), 0, 1, 0, 1},
  {&__pyx_n_u_wheat2, __pyx_k_wheat2, sizeof(__pyx_k_wheat2), 0, 1, 0, 1},
  {&__pyx_n_u_wheat3, __pyx_k_wheat3, sizeof(__pyx_k_wheat3), 0, 1, 0, 1},
  {&__pyx_n_u_wheat4, __pyx_k_wheat4, sizeof(__pyx_k_wheat4), 0, 1, 0, 1},
  {&__pyx_n_u_white, __pyx_k_white, sizeof(__pyx_k_white), 0, 1, 0, 1},
  {&__pyx_n_u_whitesmoke, __pyx_k_whitesmoke, sizeof(__pyx_k_whitesmoke), 0, 1, 0, 1},
  {&__pyx_n_u_yellow, __pyx_k_yellow, sizeof(__pyx_k_yellow), 0, 1, 0, 1},
  {&__pyx_n_u_yellow1, __pyx_k_yellow1, sizeof(__pyx_k_yellow1), 0, 1, 0, 1},
  {&__pyx_n_u_yellow2, __pyx_k_yellow2, sizeof(__pyx_k_yellow2), 0, 1, 0, 1},
  {&__pyx_n_u_yellow3, __pyx_k_yellow3, sizeof(__pyx_k_yellow3), 0, 1, 0, 1},
  {&__pyx_n_u_yellow4, __pyx_k_yellow4, sizeof(__pyx_k_yellow4), 0, 1, 0, 1},
  {&__pyx_n_u_yellowgreen, __pyx_k_yellowgreen, sizeof(__pyx_k_yellowgreen), 0, 1, 0, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 41, __pyx_L1_error)
  __pyx_builtin_map = __Pyx_GetBuiltinName(__pyx_n_s_map); if (!__pyx_builtin_map) __PYX_ERR(0, 73, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 82, __pyx_L1_error)
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 94, __pyx_L1_error)
  __pyx_builtin_hex = __Pyx_GetBuiltinName(__pyx_n_s_hex); if (!__pyx_builtin_hex) __PYX_ERR(0, 153, __pyx_L1_error)
  __pyx_builtin_oct = __Pyx_GetBuiltinName(__pyx_n_s_oct); if (!__pyx_builtin_oct) __PYX_ERR(0, 156, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(0, 173, __pyx_L1_error)
  __pyx_builtin_round = __Pyx_GetBuiltinName(__pyx_n_s_round); if (!__pyx_builtin_round) __PYX_ERR(0, 453, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "pygame_sdl2/color.pyx":41
 *         return color
 *     else:
 *         raise TypeError("Expected a color.")             # <<<<<<<<<<<<<<
 * 
 *     return SDL_MapRGBA(surface.format, r, g, b, a)
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_Expected_a_color); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "pygame_sdl2/color.pyx":123
 *             if isinstance(c, basestring):
 *                 if c.startswith('#'):
 *                     self.from_hex(c[1:])             # <<<<<<<<<<<<<<
 *                 elif c.startswith('0x'):
 *                     self.from_hex(c[2:])
 */
  __pyx_slice__4 = PySlice_New(__pyx_int_1, Py_None, Py_None); if (unlikely(!__pyx_slice__4)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__4);
  __Pyx_GIVEREF(__pyx_slice__4);

  /* "pygame_sdl2/color.pyx":125
 *                     self.from_hex(c[1:])
 *                 elif c.startswith('0x'):
 *                     self.from_hex(c[2:])             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.from_name(c)
 */
  __pyx_slice__5 = PySlice_New(__pyx_int_2, Py_None, Py_None); if (unlikely(!__pyx_slice__5)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__5);
  __Pyx_GIVEREF(__pyx_slice__5);

  /* "pygame_sdl2/color.pyx":228
 *     def __mod__(self not None, Color rhs not None):
 * 
 *         def mod(l, r):             # <<<<<<<<<<<<<<
 *             if r == 0:
 *                 return 0
 */
  __pyx_tuple__6 = PyTuple_Pack(2, __pyx_n_s_l, __pyx_n_s_r); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_pygame_sdl2_color_pyx, __pyx_n_s_mod, 228, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(0, 228, __pyx_L1_error)

  /* "pygame_sdl2/color.pyx":251
 *     def __floordiv__(self not None, Color rhs not None):
 * 
 *         def div(a, b):             # <<<<<<<<<<<<<<
 *             if b == 0:
 *                 return b
 */
  __pyx_tuple__8 = PyTuple_Pack(2, __pyx_n_s_a, __pyx_n_s_b); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_pygame_sdl2_color_pyx, __pyx_n_s_div, 251, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) __PYX_ERR(0, 251, __pyx_L1_error)

  /* "include/color_dict.pxi":5
 * 
 * cdef object colors = {
 *     'aliceblue' : (240, 248, 255),             # <<<<<<<<<<<<<<
 *     'antiquewhite' : (250, 235, 215),
 *     'antiquewhite1' : (255, 239, 219),
 */
  __pyx_tuple__10 = PyTuple_Pack(3, __pyx_int_240, __pyx_int_248, __pyx_int_255); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "include/color_dict.pxi":6
 * cdef object colors = {
 *     'aliceblue' : (240, 248, 255),
 *     'antiquewhite' : (250, 235, 215),             # <<<<<<<<<<<<<<
 *     'antiquewhite1' : (255, 239, 219),
 *     'antiquewhite2' : (238, 223, 204),
 */
  __pyx_tuple__11 = PyTuple_Pack(3, __pyx_int_250, __pyx_int_235, __pyx_int_215); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "include/color_dict.pxi":7
 *     'aliceblue' : (240, 248, 255),
 *     'antiquewhite' : (250, 235, 215),
 *     'antiquewhite1' : (255, 239, 219),             # <<<<<<<<<<<<<<
 *     'antiquewhite2' : (238, 223, 204),
 *     'antiquewhite3' : (205, 192, 176),
 */
  __pyx_tuple__12 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_239, __pyx_int_219); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "include/color_dict.pxi":8
 *     'antiquewhite' : (250, 235, 215),
 *     'antiquewhite1' : (255, 239, 219),
 *     'antiquewhite2' : (238, 223, 204),             # <<<<<<<<<<<<<<
 *     'antiquewhite3' : (205, 192, 176),
 *     'antiquewhite4' : (139, 131, 120),
 */
  __pyx_tuple__13 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_223, __pyx_int_204); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(2, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "include/color_dict.pxi":9
 *     'antiquewhite1' : (255, 239, 219),
 *     'antiquewhite2' : (238, 223, 204),
 *     'antiquewhite3' : (205, 192, 176),             # <<<<<<<<<<<<<<
 *     'antiquewhite4' : (139, 131, 120),
 *     'aquamarine' : (127, 255, 212),
 */
  __pyx_tuple__14 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_192, __pyx_int_176); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "include/color_dict.pxi":10
 *     'antiquewhite2' : (238, 223, 204),
 *     'antiquewhite3' : (205, 192, 176),
 *     'antiquewhite4' : (139, 131, 120),             # <<<<<<<<<<<<<<
 *     'aquamarine' : (127, 255, 212),
 *     'aquamarine1' : (127, 255, 212),
 */
  __pyx_tuple__15 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_131, __pyx_int_120); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "include/color_dict.pxi":11
 *     'antiquewhite3' : (205, 192, 176),
 *     'antiquewhite4' : (139, 131, 120),
 *     'aquamarine' : (127, 255, 212),             # <<<<<<<<<<<<<<
 *     'aquamarine1' : (127, 255, 212),
 *     'aquamarine2' : (118, 238, 198),
 */
  __pyx_tuple__16 = PyTuple_Pack(3, __pyx_int_127, __pyx_int_255, __pyx_int_212); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(2, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "include/color_dict.pxi":13
 *     'aquamarine' : (127, 255, 212),
 *     'aquamarine1' : (127, 255, 212),
 *     'aquamarine2' : (118, 238, 198),             # <<<<<<<<<<<<<<
 *     'aquamarine3' : (102, 205, 170),
 *     'aquamarine4' : (69, 139, 116),
 */
  __pyx_tuple__17 = PyTuple_Pack(3, __pyx_int_118, __pyx_int_238, __pyx_int_198); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(2, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "include/color_dict.pxi":14
 *     'aquamarine1' : (127, 255, 212),
 *     'aquamarine2' : (118, 238, 198),
 *     'aquamarine3' : (102, 205, 170),             # <<<<<<<<<<<<<<
 *     'aquamarine4' : (69, 139, 116),
 *     'azure' : (240, 255, 255),
 */
  __pyx_tuple__18 = PyTuple_Pack(3, __pyx_int_102, __pyx_int_205, __pyx_int_170); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(2, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "include/color_dict.pxi":15
 *     'aquamarine2' : (118, 238, 198),
 *     'aquamarine3' : (102, 205, 170),
 *     'aquamarine4' : (69, 139, 116),             # <<<<<<<<<<<<<<
 *     'azure' : (240, 255, 255),
 *     'azure1' : (240, 255, 255),
 */
  __pyx_tuple__19 = PyTuple_Pack(3, __pyx_int_69, __pyx_int_139, __pyx_int_116); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "include/color_dict.pxi":16
 *     'aquamarine3' : (102, 205, 170),
 *     'aquamarine4' : (69, 139, 116),
 *     'azure' : (240, 255, 255),             # <<<<<<<<<<<<<<
 *     'azure1' : (240, 255, 255),
 *     'azure2' : (224, 238, 238),
 */
  __pyx_tuple__20 = PyTuple_Pack(3, __pyx_int_240, __pyx_int_255, __pyx_int_255); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "include/color_dict.pxi":18
 *     'azure' : (240, 255, 255),
 *     'azure1' : (240, 255, 255),
 *     'azure2' : (224, 238, 238),             # <<<<<<<<<<<<<<
 *     'azure3' : (193, 205, 205),
 *     'azure4' : (131, 139, 139),
 */
  __pyx_tuple__21 = PyTuple_Pack(3, __pyx_int_224, __pyx_int_238, __pyx_int_238); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(2, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "include/color_dict.pxi":19
 *     'azure1' : (240, 255, 255),
 *     'azure2' : (224, 238, 238),
 *     'azure3' : (193, 205, 205),             # <<<<<<<<<<<<<<
 *     'azure4' : (131, 139, 139),
 *     'beige' : (245, 245, 220),
 */
  __pyx_tuple__22 = PyTuple_Pack(3, __pyx_int_193, __pyx_int_205, __pyx_int_205); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(2, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "include/color_dict.pxi":20
 *     'azure2' : (224, 238, 238),
 *     'azure3' : (193, 205, 205),
 *     'azure4' : (131, 139, 139),             # <<<<<<<<<<<<<<
 *     'beige' : (245, 245, 220),
 *     'bisque' : (255, 228, 196),
 */
  __pyx_tuple__23 = PyTuple_Pack(3, __pyx_int_131, __pyx_int_139, __pyx_int_139); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(2, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "include/color_dict.pxi":21
 *     'azure3' : (193, 205, 205),
 *     'azure4' : (131, 139, 139),
 *     'beige' : (245, 245, 220),             # <<<<<<<<<<<<<<
 *     'bisque' : (255, 228, 196),
 *     'bisque1' : (255, 228, 196),
 */
  __pyx_tuple__24 = PyTuple_Pack(3, __pyx_int_245, __pyx_int_245, __pyx_int_220); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(2, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "include/color_dict.pxi":22
 *     'azure4' : (131, 139, 139),
 *     'beige' : (245, 245, 220),
 *     'bisque' : (255, 228, 196),             # <<<<<<<<<<<<<<
 *     'bisque1' : (255, 228, 196),
 *     'bisque2' : (238, 213, 183),
 */
  __pyx_tuple__25 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_228, __pyx_int_196); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(2, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "include/color_dict.pxi":24
 *     'bisque' : (255, 228, 196),
 *     'bisque1' : (255, 228, 196),
 *     'bisque2' : (238, 213, 183),             # <<<<<<<<<<<<<<
 *     'bisque3' : (205, 183, 158),
 *     'bisque4' : (139, 125, 107),
 */
  __pyx_tuple__26 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_213, __pyx_int_183); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(2, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "include/color_dict.pxi":25
 *     'bisque1' : (255, 228, 196),
 *     'bisque2' : (238, 213, 183),
 *     'bisque3' : (205, 183, 158),             # <<<<<<<<<<<<<<
 *     'bisque4' : (139, 125, 107),
 *     'black' : (0, 0, 0),
 */
  __pyx_tuple__27 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_183, __pyx_int_158); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(2, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "include/color_dict.pxi":26
 *     'bisque2' : (238, 213, 183),
 *     'bisque3' : (205, 183, 158),
 *     'bisque4' : (139, 125, 107),             # <<<<<<<<<<<<<<
 *     'black' : (0, 0, 0),
 *     'blanchedalmond' : (255, 235, 205),
 */
  __pyx_tuple__28 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_125, __pyx_int_107); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(2, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "include/color_dict.pxi":27
 *     'bisque3' : (205, 183, 158),
 *     'bisque4' : (139, 125, 107),
 *     'black' : (0, 0, 0),             # <<<<<<<<<<<<<<
 *     'blanchedalmond' : (255, 235, 205),
 *     'blue' : (0, 0, 255),
 */
  __pyx_tuple__29 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_0, __pyx_int_0); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(2, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "include/color_dict.pxi":28
 *     'bisque4' : (139, 125, 107),
 *     'black' : (0, 0, 0),
 *     'blanchedalmond' : (255, 235, 205),             # <<<<<<<<<<<<<<
 *     'blue' : (0, 0, 255),
 *     'blue1' : (0, 0, 255),
 */
  __pyx_tuple__30 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_235, __pyx_int_205); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(2, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "include/color_dict.pxi":29
 *     'black' : (0, 0, 0),
 *     'blanchedalmond' : (255, 235, 205),
 *     'blue' : (0, 0, 255),             # <<<<<<<<<<<<<<
 *     'blue1' : (0, 0, 255),
 *     'blue2' : (0, 0, 238),
 */
  __pyx_tuple__31 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_0, __pyx_int_255); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(2, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "include/color_dict.pxi":31
 *     'blue' : (0, 0, 255),
 *     'blue1' : (0, 0, 255),
 *     'blue2' : (0, 0, 238),             # <<<<<<<<<<<<<<
 *     'blue3' : (0, 0, 205),
 *     'blue4' : (0, 0, 139),
 */
  __pyx_tuple__32 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_0, __pyx_int_238); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(2, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);

  /* "include/color_dict.pxi":32
 *     'blue1' : (0, 0, 255),
 *     'blue2' : (0, 0, 238),
 *     'blue3' : (0, 0, 205),             # <<<<<<<<<<<<<<
 *     'blue4' : (0, 0, 139),
 *     'blueviolet' : (138, 43, 226),
 */
  __pyx_tuple__33 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_0, __pyx_int_205); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(2, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "include/color_dict.pxi":33
 *     'blue2' : (0, 0, 238),
 *     'blue3' : (0, 0, 205),
 *     'blue4' : (0, 0, 139),             # <<<<<<<<<<<<<<
 *     'blueviolet' : (138, 43, 226),
 *     'brown' : (165, 42, 42),
 */
  __pyx_tuple__34 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_0, __pyx_int_139); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(2, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);

  /* "include/color_dict.pxi":34
 *     'blue3' : (0, 0, 205),
 *     'blue4' : (0, 0, 139),
 *     'blueviolet' : (138, 43, 226),             # <<<<<<<<<<<<<<
 *     'brown' : (165, 42, 42),
 *     'brown1' : (255, 64, 64),
 */
  __pyx_tuple__35 = PyTuple_Pack(3, __pyx_int_138, __pyx_int_43, __pyx_int_226); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(2, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);

  /* "include/color_dict.pxi":35
 *     'blue4' : (0, 0, 139),
 *     'blueviolet' : (138, 43, 226),
 *     'brown' : (165, 42, 42),             # <<<<<<<<<<<<<<
 *     'brown1' : (255, 64, 64),
 *     'brown2' : (238, 59, 59),
 */
  __pyx_tuple__36 = PyTuple_Pack(3, __pyx_int_165, __pyx_int_42, __pyx_int_42); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(2, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);

  /* "include/color_dict.pxi":36
 *     'blueviolet' : (138, 43, 226),
 *     'brown' : (165, 42, 42),
 *     'brown1' : (255, 64, 64),             # <<<<<<<<<<<<<<
 *     'brown2' : (238, 59, 59),
 *     'brown3' : (205, 51, 51),
 */
  __pyx_tuple__37 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_64, __pyx_int_64); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(2, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);

  /* "include/color_dict.pxi":37
 *     'brown' : (165, 42, 42),
 *     'brown1' : (255, 64, 64),
 *     'brown2' : (238, 59, 59),             # <<<<<<<<<<<<<<
 *     'brown3' : (205, 51, 51),
 *     'brown4' : (139, 35, 35),
 */
  __pyx_tuple__38 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_59, __pyx_int_59); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(2, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);

  /* "include/color_dict.pxi":38
 *     'brown1' : (255, 64, 64),
 *     'brown2' : (238, 59, 59),
 *     'brown3' : (205, 51, 51),             # <<<<<<<<<<<<<<
 *     'brown4' : (139, 35, 35),
 *     'burlywood' : (222, 184, 135),
 */
  __pyx_tuple__39 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_51, __pyx_int_51); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(2, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);

  /* "include/color_dict.pxi":39
 *     'brown2' : (238, 59, 59),
 *     'brown3' : (205, 51, 51),
 *     'brown4' : (139, 35, 35),             # <<<<<<<<<<<<<<
 *     'burlywood' : (222, 184, 135),
 *     'burlywood1' : (255, 211, 155),
 */
  __pyx_tuple__40 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_35, __pyx_int_35); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(2, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);

  /* "include/color_dict.pxi":40
 *     'brown3' : (205, 51, 51),
 *     'brown4' : (139, 35, 35),
 *     'burlywood' : (222, 184, 135),             # <<<<<<<<<<<<<<
 *     'burlywood1' : (255, 211, 155),
 *     'burlywood2' : (238, 197, 145),
 */
  __pyx_tuple__41 = PyTuple_Pack(3, __pyx_int_222, __pyx_int_184, __pyx_int_135); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(2, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);

  /* "include/color_dict.pxi":41
 *     'brown4' : (139, 35, 35),
 *     'burlywood' : (222, 184, 135),
 *     'burlywood1' : (255, 211, 155),             # <<<<<<<<<<<<<<
 *     'burlywood2' : (238, 197, 145),
 *     'burlywood3' : (205, 170, 125),
 */
  __pyx_tuple__42 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_211, __pyx_int_155); if (unlikely(!__pyx_tuple__42)) __PYX_ERR(2, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);

  /* "include/color_dict.pxi":42
 *     'burlywood' : (222, 184, 135),
 *     'burlywood1' : (255, 211, 155),
 *     'burlywood2' : (238, 197, 145),             # <<<<<<<<<<<<<<
 *     'burlywood3' : (205, 170, 125),
 *     'burlywood4' : (139, 115, 85),
 */
  __pyx_tuple__43 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_197, __pyx_int_145); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(2, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);

  /* "include/color_dict.pxi":43
 *     'burlywood1' : (255, 211, 155),
 *     'burlywood2' : (238, 197, 145),
 *     'burlywood3' : (205, 170, 125),             # <<<<<<<<<<<<<<
 *     'burlywood4' : (139, 115, 85),
 *     'cadetblue' : (95, 158, 160),
 */
  __pyx_tuple__44 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_170, __pyx_int_125); if (unlikely(!__pyx_tuple__44)) __PYX_ERR(2, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);

  /* "include/color_dict.pxi":44
 *     'burlywood2' : (238, 197, 145),
 *     'burlywood3' : (205, 170, 125),
 *     'burlywood4' : (139, 115, 85),             # <<<<<<<<<<<<<<
 *     'cadetblue' : (95, 158, 160),
 *     'cadetblue1' : (152, 245, 255),
 */
  __pyx_tuple__45 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_115, __pyx_int_85); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(2, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);

  /* "include/color_dict.pxi":45
 *     'burlywood3' : (205, 170, 125),
 *     'burlywood4' : (139, 115, 85),
 *     'cadetblue' : (95, 158, 160),             # <<<<<<<<<<<<<<
 *     'cadetblue1' : (152, 245, 255),
 *     'cadetblue2' : (142, 229, 238),
 */
  __pyx_tuple__46 = PyTuple_Pack(3, __pyx_int_95, __pyx_int_158, __pyx_int_160); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(2, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);

  /* "include/color_dict.pxi":46
 *     'burlywood4' : (139, 115, 85),
 *     'cadetblue' : (95, 158, 160),
 *     'cadetblue1' : (152, 245, 255),             # <<<<<<<<<<<<<<
 *     'cadetblue2' : (142, 229, 238),
 *     'cadetblue3' : (122, 197, 205),
 */
  __pyx_tuple__47 = PyTuple_Pack(3, __pyx_int_152, __pyx_int_245, __pyx_int_255); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(2, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);

  /* "include/color_dict.pxi":47
 *     'cadetblue' : (95, 158, 160),
 *     'cadetblue1' : (152, 245, 255),
 *     'cadetblue2' : (142, 229, 238),             # <<<<<<<<<<<<<<
 *     'cadetblue3' : (122, 197, 205),
 *     'cadetblue4' : (83, 134, 139),
 */
  __pyx_tuple__48 = PyTuple_Pack(3, __pyx_int_142, __pyx_int_229, __pyx_int_238); if (unlikely(!__pyx_tuple__48)) __PYX_ERR(2, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);

  /* "include/color_dict.pxi":48
 *     'cadetblue1' : (152, 245, 255),
 *     'cadetblue2' : (142, 229, 238),
 *     'cadetblue3' : (122, 197, 205),             # <<<<<<<<<<<<<<
 *     'cadetblue4' : (83, 134, 139),
 *     'chartreuse' : (127, 255, 0),
 */
  __pyx_tuple__49 = PyTuple_Pack(3, __pyx_int_122, __pyx_int_197, __pyx_int_205); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(2, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);

  /* "include/color_dict.pxi":49
 *     'cadetblue2' : (142, 229, 238),
 *     'cadetblue3' : (122, 197, 205),
 *     'cadetblue4' : (83, 134, 139),             # <<<<<<<<<<<<<<
 *     'chartreuse' : (127, 255, 0),
 *     'chartreuse1' : (127, 255, 0),
 */
  __pyx_tuple__50 = PyTuple_Pack(3, __pyx_int_83, __pyx_int_134, __pyx_int_139); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(2, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);

  /* "include/color_dict.pxi":50
 *     'cadetblue3' : (122, 197, 205),
 *     'cadetblue4' : (83, 134, 139),
 *     'chartreuse' : (127, 255, 0),             # <<<<<<<<<<<<<<
 *     'chartreuse1' : (127, 255, 0),
 *     'chartreuse2' : (118, 238, 0),
 */
  __pyx_tuple__51 = PyTuple_Pack(3, __pyx_int_127, __pyx_int_255, __pyx_int_0); if (unlikely(!__pyx_tuple__51)) __PYX_ERR(2, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);

  /* "include/color_dict.pxi":52
 *     'chartreuse' : (127, 255, 0),
 *     'chartreuse1' : (127, 255, 0),
 *     'chartreuse2' : (118, 238, 0),             # <<<<<<<<<<<<<<
 *     'chartreuse3' : (102, 205, 0),
 *     'chartreuse4' : (69, 139, 0),
 */
  __pyx_tuple__52 = PyTuple_Pack(3, __pyx_int_118, __pyx_int_238, __pyx_int_0); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(2, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);

  /* "include/color_dict.pxi":53
 *     'chartreuse1' : (127, 255, 0),
 *     'chartreuse2' : (118, 238, 0),
 *     'chartreuse3' : (102, 205, 0),             # <<<<<<<<<<<<<<
 *     'chartreuse4' : (69, 139, 0),
 *     'chocolate' : (210, 105, 30),
 */
  __pyx_tuple__53 = PyTuple_Pack(3, __pyx_int_102, __pyx_int_205, __pyx_int_0); if (unlikely(!__pyx_tuple__53)) __PYX_ERR(2, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);

  /* "include/color_dict.pxi":54
 *     'chartreuse2' : (118, 238, 0),
 *     'chartreuse3' : (102, 205, 0),
 *     'chartreuse4' : (69, 139, 0),             # <<<<<<<<<<<<<<
 *     'chocolate' : (210, 105, 30),
 *     'chocolate1' : (255, 127, 36),
 */
  __pyx_tuple__54 = PyTuple_Pack(3, __pyx_int_69, __pyx_int_139, __pyx_int_0); if (unlikely(!__pyx_tuple__54)) __PYX_ERR(2, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);

  /* "include/color_dict.pxi":55
 *     'chartreuse3' : (102, 205, 0),
 *     'chartreuse4' : (69, 139, 0),
 *     'chocolate' : (210, 105, 30),             # <<<<<<<<<<<<<<
 *     'chocolate1' : (255, 127, 36),
 *     'chocolate2' : (238, 118, 33),
 */
  __pyx_tuple__55 = PyTuple_Pack(3, __pyx_int_210, __pyx_int_105, __pyx_int_30); if (unlikely(!__pyx_tuple__55)) __PYX_ERR(2, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);

  /* "include/color_dict.pxi":56
 *     'chartreuse4' : (69, 139, 0),
 *     'chocolate' : (210, 105, 30),
 *     'chocolate1' : (255, 127, 36),             # <<<<<<<<<<<<<<
 *     'chocolate2' : (238, 118, 33),
 *     'chocolate3' : (205, 102, 29),
 */
  __pyx_tuple__56 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_127, __pyx_int_36); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(2, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);

  /* "include/color_dict.pxi":57
 *     'chocolate' : (210, 105, 30),
 *     'chocolate1' : (255, 127, 36),
 *     'chocolate2' : (238, 118, 33),             # <<<<<<<<<<<<<<
 *     'chocolate3' : (205, 102, 29),
 *     'chocolate4' : (139, 69, 19),
 */
  __pyx_tuple__57 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_118, __pyx_int_33); if (unlikely(!__pyx_tuple__57)) __PYX_ERR(2, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);

  /* "include/color_dict.pxi":58
 *     'chocolate1' : (255, 127, 36),
 *     'chocolate2' : (238, 118, 33),
 *     'chocolate3' : (205, 102, 29),             # <<<<<<<<<<<<<<
 *     'chocolate4' : (139, 69, 19),
 *     'coral' : (255, 127, 80),
 */
  __pyx_tuple__58 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_102, __pyx_int_29); if (unlikely(!__pyx_tuple__58)) __PYX_ERR(2, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);

  /* "include/color_dict.pxi":59
 *     'chocolate2' : (238, 118, 33),
 *     'chocolate3' : (205, 102, 29),
 *     'chocolate4' : (139, 69, 19),             # <<<<<<<<<<<<<<
 *     'coral' : (255, 127, 80),
 *     'coral1' : (255, 114, 86),
 */
  __pyx_tuple__59 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_69, __pyx_int_19); if (unlikely(!__pyx_tuple__59)) __PYX_ERR(2, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);

  /* "include/color_dict.pxi":60
 *     'chocolate3' : (205, 102, 29),
 *     'chocolate4' : (139, 69, 19),
 *     'coral' : (255, 127, 80),             # <<<<<<<<<<<<<<
 *     'coral1' : (255, 114, 86),
 *     'coral2' : (238, 106, 80),
 */
  __pyx_tuple__60 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_127, __pyx_int_80); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(2, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);

  /* "include/color_dict.pxi":61
 *     'chocolate4' : (139, 69, 19),
 *     'coral' : (255, 127, 80),
 *     'coral1' : (255, 114, 86),             # <<<<<<<<<<<<<<
 *     'coral2' : (238, 106, 80),
 *     'coral3' : (205, 91, 69),
 */
  __pyx_tuple__61 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_114, __pyx_int_86); if (unlikely(!__pyx_tuple__61)) __PYX_ERR(2, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__61);
  __Pyx_GIVEREF(__pyx_tuple__61);

  /* "include/color_dict.pxi":62
 *     'coral' : (255, 127, 80),
 *     'coral1' : (255, 114, 86),
 *     'coral2' : (238, 106, 80),             # <<<<<<<<<<<<<<
 *     'coral3' : (205, 91, 69),
 *     'coral4' : (139, 62, 47),
 */
  __pyx_tuple__62 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_106, __pyx_int_80); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(2, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);

  /* "include/color_dict.pxi":63
 *     'coral1' : (255, 114, 86),
 *     'coral2' : (238, 106, 80),
 *     'coral3' : (205, 91, 69),             # <<<<<<<<<<<<<<
 *     'coral4' : (139, 62, 47),
 *     'cornflowerblue' : (100, 149, 237),
 */
  __pyx_tuple__63 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_91, __pyx_int_69); if (unlikely(!__pyx_tuple__63)) __PYX_ERR(2, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__63);
  __Pyx_GIVEREF(__pyx_tuple__63);

  /* "include/color_dict.pxi":64
 *     'coral2' : (238, 106, 80),
 *     'coral3' : (205, 91, 69),
 *     'coral4' : (139, 62, 47),             # <<<<<<<<<<<<<<
 *     'cornflowerblue' : (100, 149, 237),
 *     'cornsilk' : (255, 248, 220),
 */
  __pyx_tuple__64 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_62, __pyx_int_47); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(2, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);

  /* "include/color_dict.pxi":65
 *     'coral3' : (205, 91, 69),
 *     'coral4' : (139, 62, 47),
 *     'cornflowerblue' : (100, 149, 237),             # <<<<<<<<<<<<<<
 *     'cornsilk' : (255, 248, 220),
 *     'cornsilk1' : (255, 248, 220),
 */
  __pyx_tuple__65 = PyTuple_Pack(3, __pyx_int_100, __pyx_int_149, __pyx_int_237); if (unlikely(!__pyx_tuple__65)) __PYX_ERR(2, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);

  /* "include/color_dict.pxi":66
 *     'coral4' : (139, 62, 47),
 *     'cornflowerblue' : (100, 149, 237),
 *     'cornsilk' : (255, 248, 220),             # <<<<<<<<<<<<<<
 *     'cornsilk1' : (255, 248, 220),
 *     'cornsilk2' : (238, 232, 205),
 */
  __pyx_tuple__66 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_248, __pyx_int_220); if (unlikely(!__pyx_tuple__66)) __PYX_ERR(2, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);

  /* "include/color_dict.pxi":68
 *     'cornsilk' : (255, 248, 220),
 *     'cornsilk1' : (255, 248, 220),
 *     'cornsilk2' : (238, 232, 205),             # <<<<<<<<<<<<<<
 *     'cornsilk3' : (205, 200, 177),
 *     'cornsilk4' : (139, 136, 120),
 */
  __pyx_tuple__67 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_232, __pyx_int_205); if (unlikely(!__pyx_tuple__67)) __PYX_ERR(2, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);

  /* "include/color_dict.pxi":69
 *     'cornsilk1' : (255, 248, 220),
 *     'cornsilk2' : (238, 232, 205),
 *     'cornsilk3' : (205, 200, 177),             # <<<<<<<<<<<<<<
 *     'cornsilk4' : (139, 136, 120),
 *     'cyan' : (0, 255, 255),
 */
  __pyx_tuple__68 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_200, __pyx_int_177); if (unlikely(!__pyx_tuple__68)) __PYX_ERR(2, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);

  /* "include/color_dict.pxi":70
 *     'cornsilk2' : (238, 232, 205),
 *     'cornsilk3' : (205, 200, 177),
 *     'cornsilk4' : (139, 136, 120),             # <<<<<<<<<<<<<<
 *     'cyan' : (0, 255, 255),
 *     'cyan1' : (0, 255, 255),
 */
  __pyx_tuple__69 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_136, __pyx_int_120); if (unlikely(!__pyx_tuple__69)) __PYX_ERR(2, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__69);
  __Pyx_GIVEREF(__pyx_tuple__69);

  /* "include/color_dict.pxi":71
 *     'cornsilk3' : (205, 200, 177),
 *     'cornsilk4' : (139, 136, 120),
 *     'cyan' : (0, 255, 255),             # <<<<<<<<<<<<<<
 *     'cyan1' : (0, 255, 255),
 *     'cyan2' : (0, 238, 238),
 */
  __pyx_tuple__70 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_255, __pyx_int_255); if (unlikely(!__pyx_tuple__70)) __PYX_ERR(2, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);

  /* "include/color_dict.pxi":73
 *     'cyan' : (0, 255, 255),
 *     'cyan1' : (0, 255, 255),
 *     'cyan2' : (0, 238, 238),             # <<<<<<<<<<<<<<
 *     'cyan3' : (0, 205, 205),
 *     'cyan4' : (0, 139, 139),
 */
  __pyx_tuple__71 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_238, __pyx_int_238); if (unlikely(!__pyx_tuple__71)) __PYX_ERR(2, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__71);
  __Pyx_GIVEREF(__pyx_tuple__71);

  /* "include/color_dict.pxi":74
 *     'cyan1' : (0, 255, 255),
 *     'cyan2' : (0, 238, 238),
 *     'cyan3' : (0, 205, 205),             # <<<<<<<<<<<<<<
 *     'cyan4' : (0, 139, 139),
 *     'darkblue' : (0, 0, 139),
 */
  __pyx_tuple__72 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_205, __pyx_int_205); if (unlikely(!__pyx_tuple__72)) __PYX_ERR(2, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);

  /* "include/color_dict.pxi":75
 *     'cyan2' : (0, 238, 238),
 *     'cyan3' : (0, 205, 205),
 *     'cyan4' : (0, 139, 139),             # <<<<<<<<<<<<<<
 *     'darkblue' : (0, 0, 139),
 *     'darkcyan' : (0, 139, 139),
 */
  __pyx_tuple__73 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_139, __pyx_int_139); if (unlikely(!__pyx_tuple__73)) __PYX_ERR(2, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__73);
  __Pyx_GIVEREF(__pyx_tuple__73);

  /* "include/color_dict.pxi":78
 *     'darkblue' : (0, 0, 139),
 *     'darkcyan' : (0, 139, 139),
 *     'darkgoldenrod' : (184, 134, 11),             # <<<<<<<<<<<<<<
 *     'darkgoldenrod1' : (255, 185, 15),
 *     'darkgoldenrod2' : (238, 173, 14),
 */
  __pyx_tuple__74 = PyTuple_Pack(3, __pyx_int_184, __pyx_int_134, __pyx_int_11); if (unlikely(!__pyx_tuple__74)) __PYX_ERR(2, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);

  /* "include/color_dict.pxi":79
 *     'darkcyan' : (0, 139, 139),
 *     'darkgoldenrod' : (184, 134, 11),
 *     'darkgoldenrod1' : (255, 185, 15),             # <<<<<<<<<<<<<<
 *     'darkgoldenrod2' : (238, 173, 14),
 *     'darkgoldenrod3' : (205, 149, 12),
 */
  __pyx_tuple__75 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_185, __pyx_int_15); if (unlikely(!__pyx_tuple__75)) __PYX_ERR(2, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__75);
  __Pyx_GIVEREF(__pyx_tuple__75);

  /* "include/color_dict.pxi":80
 *     'darkgoldenrod' : (184, 134, 11),
 *     'darkgoldenrod1' : (255, 185, 15),
 *     'darkgoldenrod2' : (238, 173, 14),             # <<<<<<<<<<<<<<
 *     'darkgoldenrod3' : (205, 149, 12),
 *     'darkgoldenrod4' : (139, 101, 8),
 */
  __pyx_tuple__76 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_173, __pyx_int_14); if (unlikely(!__pyx_tuple__76)) __PYX_ERR(2, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__76);
  __Pyx_GIVEREF(__pyx_tuple__76);

  /* "include/color_dict.pxi":81
 *     'darkgoldenrod1' : (255, 185, 15),
 *     'darkgoldenrod2' : (238, 173, 14),
 *     'darkgoldenrod3' : (205, 149, 12),             # <<<<<<<<<<<<<<
 *     'darkgoldenrod4' : (139, 101, 8),
 *     'darkgray' : (169, 169, 169),
 */
  __pyx_tuple__77 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_149, __pyx_int_12); if (unlikely(!__pyx_tuple__77)) __PYX_ERR(2, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__77);
  __Pyx_GIVEREF(__pyx_tuple__77);

  /* "include/color_dict.pxi":82
 *     'darkgoldenrod2' : (238, 173, 14),
 *     'darkgoldenrod3' : (205, 149, 12),
 *     'darkgoldenrod4' : (139, 101, 8),             # <<<<<<<<<<<<<<
 *     'darkgray' : (169, 169, 169),
 *     'darkgreen' : (0, 100, 0),
 */
  __pyx_tuple__78 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_101, __pyx_int_8); if (unlikely(!__pyx_tuple__78)) __PYX_ERR(2, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__78);
  __Pyx_GIVEREF(__pyx_tuple__78);

  /* "include/color_dict.pxi":83
 *     'darkgoldenrod3' : (205, 149, 12),
 *     'darkgoldenrod4' : (139, 101, 8),
 *     'darkgray' : (169, 169, 169),             # <<<<<<<<<<<<<<
 *     'darkgreen' : (0, 100, 0),
 *     'darkgrey' : (169, 169, 169),
 */
  __pyx_tuple__79 = PyTuple_Pack(3, __pyx_int_169, __pyx_int_169, __pyx_int_169); if (unlikely(!__pyx_tuple__79)) __PYX_ERR(2, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__79);
  __Pyx_GIVEREF(__pyx_tuple__79);

  /* "include/color_dict.pxi":84
 *     'darkgoldenrod4' : (139, 101, 8),
 *     'darkgray' : (169, 169, 169),
 *     'darkgreen' : (0, 100, 0),             # <<<<<<<<<<<<<<
 *     'darkgrey' : (169, 169, 169),
 *     'darkkhaki' : (189, 183, 107),
 */
  __pyx_tuple__80 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_100, __pyx_int_0); if (unlikely(!__pyx_tuple__80)) __PYX_ERR(2, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__80);
  __Pyx_GIVEREF(__pyx_tuple__80);

  /* "include/color_dict.pxi":86
 *     'darkgreen' : (0, 100, 0),
 *     'darkgrey' : (169, 169, 169),
 *     'darkkhaki' : (189, 183, 107),             # <<<<<<<<<<<<<<
 *     'darkmagenta' : (139, 0, 139),
 *     'darkolivegreen' : (85, 107, 47),
 */
  __pyx_tuple__81 = PyTuple_Pack(3, __pyx_int_189, __pyx_int_183, __pyx_int_107); if (unlikely(!__pyx_tuple__81)) __PYX_ERR(2, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__81);
  __Pyx_GIVEREF(__pyx_tuple__81);

  /* "include/color_dict.pxi":87
 *     'darkgrey' : (169, 169, 169),
 *     'darkkhaki' : (189, 183, 107),
 *     'darkmagenta' : (139, 0, 139),             # <<<<<<<<<<<<<<
 *     'darkolivegreen' : (85, 107, 47),
 *     'darkolivegreen1' : (202, 255, 112),
 */
  __pyx_tuple__82 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_0, __pyx_int_139); if (unlikely(!__pyx_tuple__82)) __PYX_ERR(2, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__82);
  __Pyx_GIVEREF(__pyx_tuple__82);

  /* "include/color_dict.pxi":88
 *     'darkkhaki' : (189, 183, 107),
 *     'darkmagenta' : (139, 0, 139),
 *     'darkolivegreen' : (85, 107, 47),             # <<<<<<<<<<<<<<
 *     'darkolivegreen1' : (202, 255, 112),
 *     'darkolivegreen2' : (188, 238, 104),
 */
  __pyx_tuple__83 = PyTuple_Pack(3, __pyx_int_85, __pyx_int_107, __pyx_int_47); if (unlikely(!__pyx_tuple__83)) __PYX_ERR(2, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__83);
  __Pyx_GIVEREF(__pyx_tuple__83);

  /* "include/color_dict.pxi":89
 *     'darkmagenta' : (139, 0, 139),
 *     'darkolivegreen' : (85, 107, 47),
 *     'darkolivegreen1' : (202, 255, 112),             # <<<<<<<<<<<<<<
 *     'darkolivegreen2' : (188, 238, 104),
 *     'darkolivegreen3' : (162, 205, 90),
 */
  __pyx_tuple__84 = PyTuple_Pack(3, __pyx_int_202, __pyx_int_255, __pyx_int_112); if (unlikely(!__pyx_tuple__84)) __PYX_ERR(2, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__84);
  __Pyx_GIVEREF(__pyx_tuple__84);

  /* "include/color_dict.pxi":90
 *     'darkolivegreen' : (85, 107, 47),
 *     'darkolivegreen1' : (202, 255, 112),
 *     'darkolivegreen2' : (188, 238, 104),             # <<<<<<<<<<<<<<
 *     'darkolivegreen3' : (162, 205, 90),
 *     'darkolivegreen4' : (110, 139, 61),
 */
  __pyx_tuple__85 = PyTuple_Pack(3, __pyx_int_188, __pyx_int_238, __pyx_int_104); if (unlikely(!__pyx_tuple__85)) __PYX_ERR(2, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__85);
  __Pyx_GIVEREF(__pyx_tuple__85);

  /* "include/color_dict.pxi":91
 *     'darkolivegreen1' : (202, 255, 112),
 *     'darkolivegreen2' : (188, 238, 104),
 *     'darkolivegreen3' : (162, 205, 90),             # <<<<<<<<<<<<<<
 *     'darkolivegreen4' : (110, 139, 61),
 *     'darkorange' : (255, 140, 0),
 */
  __pyx_tuple__86 = PyTuple_Pack(3, __pyx_int_162, __pyx_int_205, __pyx_int_90); if (unlikely(!__pyx_tuple__86)) __PYX_ERR(2, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);

  /* "include/color_dict.pxi":92
 *     'darkolivegreen2' : (188, 238, 104),
 *     'darkolivegreen3' : (162, 205, 90),
 *     'darkolivegreen4' : (110, 139, 61),             # <<<<<<<<<<<<<<
 *     'darkorange' : (255, 140, 0),
 *     'darkorange1' : (255, 127, 0),
 */
  __pyx_tuple__87 = PyTuple_Pack(3, __pyx_int_110, __pyx_int_139, __pyx_int_61); if (unlikely(!__pyx_tuple__87)) __PYX_ERR(2, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__87);
  __Pyx_GIVEREF(__pyx_tuple__87);

  /* "include/color_dict.pxi":93
 *     'darkolivegreen3' : (162, 205, 90),
 *     'darkolivegreen4' : (110, 139, 61),
 *     'darkorange' : (255, 140, 0),             # <<<<<<<<<<<<<<
 *     'darkorange1' : (255, 127, 0),
 *     'darkorange2' : (238, 118, 0),
 */
  __pyx_tuple__88 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_140, __pyx_int_0); if (unlikely(!__pyx_tuple__88)) __PYX_ERR(2, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_tuple__88);

  /* "include/color_dict.pxi":94
 *     'darkolivegreen4' : (110, 139, 61),
 *     'darkorange' : (255, 140, 0),
 *     'darkorange1' : (255, 127, 0),             # <<<<<<<<<<<<<<
 *     'darkorange2' : (238, 118, 0),
 *     'darkorange3' : (205, 102, 0),
 */
  __pyx_tuple__89 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_127, __pyx_int_0); if (unlikely(!__pyx_tuple__89)) __PYX_ERR(2, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_tuple__89);

  /* "include/color_dict.pxi":95
 *     'darkorange' : (255, 140, 0),
 *     'darkorange1' : (255, 127, 0),
 *     'darkorange2' : (238, 118, 0),             # <<<<<<<<<<<<<<
 *     'darkorange3' : (205, 102, 0),
 *     'darkorange4' : (139, 69, 0),
 */
  __pyx_tuple__90 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_118, __pyx_int_0); if (unlikely(!__pyx_tuple__90)) __PYX_ERR(2, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_tuple__90);

  /* "include/color_dict.pxi":96
 *     'darkorange1' : (255, 127, 0),
 *     'darkorange2' : (238, 118, 0),
 *     'darkorange3' : (205, 102, 0),             # <<<<<<<<<<<<<<
 *     'darkorange4' : (139, 69, 0),
 *     'darkorchid' : (153, 50, 204),
 */
  __pyx_tuple__91 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_102, __pyx_int_0); if (unlikely(!__pyx_tuple__91)) __PYX_ERR(2, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__91);
  __Pyx_GIVEREF(__pyx_tuple__91);

  /* "include/color_dict.pxi":97
 *     'darkorange2' : (238, 118, 0),
 *     'darkorange3' : (205, 102, 0),
 *     'darkorange4' : (139, 69, 0),             # <<<<<<<<<<<<<<
 *     'darkorchid' : (153, 50, 204),
 *     'darkorchid1' : (191, 62, 255),
 */
  __pyx_tuple__92 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_69, __pyx_int_0); if (unlikely(!__pyx_tuple__92)) __PYX_ERR(2, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__92);
  __Pyx_GIVEREF(__pyx_tuple__92);

  /* "include/color_dict.pxi":98
 *     'darkorange3' : (205, 102, 0),
 *     'darkorange4' : (139, 69, 0),
 *     'darkorchid' : (153, 50, 204),             # <<<<<<<<<<<<<<
 *     'darkorchid1' : (191, 62, 255),
 *     'darkorchid2' : (178, 58, 238),
 */
  __pyx_tuple__93 = PyTuple_Pack(3, __pyx_int_153, __pyx_int_50, __pyx_int_204); if (unlikely(!__pyx_tuple__93)) __PYX_ERR(2, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__93);
  __Pyx_GIVEREF(__pyx_tuple__93);

  /* "include/color_dict.pxi":99
 *     'darkorange4' : (139, 69, 0),
 *     'darkorchid' : (153, 50, 204),
 *     'darkorchid1' : (191, 62, 255),             # <<<<<<<<<<<<<<
 *     'darkorchid2' : (178, 58, 238),
 *     'darkorchid3' : (154, 50, 205),
 */
  __pyx_tuple__94 = PyTuple_Pack(3, __pyx_int_191, __pyx_int_62, __pyx_int_255); if (unlikely(!__pyx_tuple__94)) __PYX_ERR(2, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__94);
  __Pyx_GIVEREF(__pyx_tuple__94);

  /* "include/color_dict.pxi":100
 *     'darkorchid' : (153, 50, 204),
 *     'darkorchid1' : (191, 62, 255),
 *     'darkorchid2' : (178, 58, 238),             # <<<<<<<<<<<<<<
 *     'darkorchid3' : (154, 50, 205),
 *     'darkorchid4' : (104, 34, 139),
 */
  __pyx_tuple__95 = PyTuple_Pack(3, __pyx_int_178, __pyx_int_58, __pyx_int_238); if (unlikely(!__pyx_tuple__95)) __PYX_ERR(2, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__95);
  __Pyx_GIVEREF(__pyx_tuple__95);

  /* "include/color_dict.pxi":101
 *     'darkorchid1' : (191, 62, 255),
 *     'darkorchid2' : (178, 58, 238),
 *     'darkorchid3' : (154, 50, 205),             # <<<<<<<<<<<<<<
 *     'darkorchid4' : (104, 34, 139),
 *     'darkred' : (139, 0, 0),
 */
  __pyx_tuple__96 = PyTuple_Pack(3, __pyx_int_154, __pyx_int_50, __pyx_int_205); if (unlikely(!__pyx_tuple__96)) __PYX_ERR(2, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__96);
  __Pyx_GIVEREF(__pyx_tuple__96);

  /* "include/color_dict.pxi":102
 *     'darkorchid2' : (178, 58, 238),
 *     'darkorchid3' : (154, 50, 205),
 *     'darkorchid4' : (104, 34, 139),             # <<<<<<<<<<<<<<
 *     'darkred' : (139, 0, 0),
 *     'darksalmon' : (233, 150, 122),
 */
  __pyx_tuple__97 = PyTuple_Pack(3, __pyx_int_104, __pyx_int_34, __pyx_int_139); if (unlikely(!__pyx_tuple__97)) __PYX_ERR(2, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__97);
  __Pyx_GIVEREF(__pyx_tuple__97);

  /* "include/color_dict.pxi":103
 *     'darkorchid3' : (154, 50, 205),
 *     'darkorchid4' : (104, 34, 139),
 *     'darkred' : (139, 0, 0),             # <<<<<<<<<<<<<<
 *     'darksalmon' : (233, 150, 122),
 *     'darkseagreen' : (143, 188, 143),
 */
  __pyx_tuple__98 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_0, __pyx_int_0); if (unlikely(!__pyx_tuple__98)) __PYX_ERR(2, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__98);
  __Pyx_GIVEREF(__pyx_tuple__98);

  /* "include/color_dict.pxi":104
 *     'darkorchid4' : (104, 34, 139),
 *     'darkred' : (139, 0, 0),
 *     'darksalmon' : (233, 150, 122),             # <<<<<<<<<<<<<<
 *     'darkseagreen' : (143, 188, 143),
 *     'darkseagreen1' : (193, 255, 193),
 */
  __pyx_tuple__99 = PyTuple_Pack(3, __pyx_int_233, __pyx_int_150, __pyx_int_122); if (unlikely(!__pyx_tuple__99)) __PYX_ERR(2, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__99);
  __Pyx_GIVEREF(__pyx_tuple__99);

  /* "include/color_dict.pxi":105
 *     'darkred' : (139, 0, 0),
 *     'darksalmon' : (233, 150, 122),
 *     'darkseagreen' : (143, 188, 143),             # <<<<<<<<<<<<<<
 *     'darkseagreen1' : (193, 255, 193),
 *     'darkseagreen2' : (180, 238, 180),
 */
  __pyx_tuple__100 = PyTuple_Pack(3, __pyx_int_143, __pyx_int_188, __pyx_int_143); if (unlikely(!__pyx_tuple__100)) __PYX_ERR(2, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__100);
  __Pyx_GIVEREF(__pyx_tuple__100);

  /* "include/color_dict.pxi":106
 *     'darksalmon' : (233, 150, 122),
 *     'darkseagreen' : (143, 188, 143),
 *     'darkseagreen1' : (193, 255, 193),             # <<<<<<<<<<<<<<
 *     'darkseagreen2' : (180, 238, 180),
 *     'darkseagreen3' : (155, 205, 155),
 */
  __pyx_tuple__101 = PyTuple_Pack(3, __pyx_int_193, __pyx_int_255, __pyx_int_193); if (unlikely(!__pyx_tuple__101)) __PYX_ERR(2, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__101);
  __Pyx_GIVEREF(__pyx_tuple__101);

  /* "include/color_dict.pxi":107
 *     'darkseagreen' : (143, 188, 143),
 *     'darkseagreen1' : (193, 255, 193),
 *     'darkseagreen2' : (180, 238, 180),             # <<<<<<<<<<<<<<
 *     'darkseagreen3' : (155, 205, 155),
 *     'darkseagreen4' : (105, 139, 105),
 */
  __pyx_tuple__102 = PyTuple_Pack(3, __pyx_int_180, __pyx_int_238, __pyx_int_180); if (unlikely(!__pyx_tuple__102)) __PYX_ERR(2, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__102);
  __Pyx_GIVEREF(__pyx_tuple__102);

  /* "include/color_dict.pxi":108
 *     'darkseagreen1' : (193, 255, 193),
 *     'darkseagreen2' : (180, 238, 180),
 *     'darkseagreen3' : (155, 205, 155),             # <<<<<<<<<<<<<<
 *     'darkseagreen4' : (105, 139, 105),
 *     'darkslateblue' : (72, 61, 139),
 */
  __pyx_tuple__103 = PyTuple_Pack(3, __pyx_int_155, __pyx_int_205, __pyx_int_155); if (unlikely(!__pyx_tuple__103)) __PYX_ERR(2, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__103);
  __Pyx_GIVEREF(__pyx_tuple__103);

  /* "include/color_dict.pxi":109
 *     'darkseagreen2' : (180, 238, 180),
 *     'darkseagreen3' : (155, 205, 155),
 *     'darkseagreen4' : (105, 139, 105),             # <<<<<<<<<<<<<<
 *     'darkslateblue' : (72, 61, 139),
 *     'darkslategray' : (47, 79, 79),
 */
  __pyx_tuple__104 = PyTuple_Pack(3, __pyx_int_105, __pyx_int_139, __pyx_int_105); if (unlikely(!__pyx_tuple__104)) __PYX_ERR(2, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__104);
  __Pyx_GIVEREF(__pyx_tuple__104);

  /* "include/color_dict.pxi":110
 *     'darkseagreen3' : (155, 205, 155),
 *     'darkseagreen4' : (105, 139, 105),
 *     'darkslateblue' : (72, 61, 139),             # <<<<<<<<<<<<<<
 *     'darkslategray' : (47, 79, 79),
 *     'darkslategray1' : (151, 255, 255),
 */
  __pyx_tuple__105 = PyTuple_Pack(3, __pyx_int_72, __pyx_int_61, __pyx_int_139); if (unlikely(!__pyx_tuple__105)) __PYX_ERR(2, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__105);
  __Pyx_GIVEREF(__pyx_tuple__105);

  /* "include/color_dict.pxi":111
 *     'darkseagreen4' : (105, 139, 105),
 *     'darkslateblue' : (72, 61, 139),
 *     'darkslategray' : (47, 79, 79),             # <<<<<<<<<<<<<<
 *     'darkslategray1' : (151, 255, 255),
 *     'darkslategray2' : (141, 238, 238),
 */
  __pyx_tuple__106 = PyTuple_Pack(3, __pyx_int_47, __pyx_int_79, __pyx_int_79); if (unlikely(!__pyx_tuple__106)) __PYX_ERR(2, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__106);
  __Pyx_GIVEREF(__pyx_tuple__106);

  /* "include/color_dict.pxi":112
 *     'darkslateblue' : (72, 61, 139),
 *     'darkslategray' : (47, 79, 79),
 *     'darkslategray1' : (151, 255, 255),             # <<<<<<<<<<<<<<
 *     'darkslategray2' : (141, 238, 238),
 *     'darkslategray3' : (121, 205, 205),
 */
  __pyx_tuple__107 = PyTuple_Pack(3, __pyx_int_151, __pyx_int_255, __pyx_int_255); if (unlikely(!__pyx_tuple__107)) __PYX_ERR(2, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__107);
  __Pyx_GIVEREF(__pyx_tuple__107);

  /* "include/color_dict.pxi":113
 *     'darkslategray' : (47, 79, 79),
 *     'darkslategray1' : (151, 255, 255),
 *     'darkslategray2' : (141, 238, 238),             # <<<<<<<<<<<<<<
 *     'darkslategray3' : (121, 205, 205),
 *     'darkslategray4' : (82, 139, 139),
 */
  __pyx_tuple__108 = PyTuple_Pack(3, __pyx_int_141, __pyx_int_238, __pyx_int_238); if (unlikely(!__pyx_tuple__108)) __PYX_ERR(2, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__108);
  __Pyx_GIVEREF(__pyx_tuple__108);

  /* "include/color_dict.pxi":114
 *     'darkslategray1' : (151, 255, 255),
 *     'darkslategray2' : (141, 238, 238),
 *     'darkslategray3' : (121, 205, 205),             # <<<<<<<<<<<<<<
 *     'darkslategray4' : (82, 139, 139),
 *     'darkslategrey' : (47, 79, 79),
 */
  __pyx_tuple__109 = PyTuple_Pack(3, __pyx_int_121, __pyx_int_205, __pyx_int_205); if (unlikely(!__pyx_tuple__109)) __PYX_ERR(2, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__109);
  __Pyx_GIVEREF(__pyx_tuple__109);

  /* "include/color_dict.pxi":115
 *     'darkslategray2' : (141, 238, 238),
 *     'darkslategray3' : (121, 205, 205),
 *     'darkslategray4' : (82, 139, 139),             # <<<<<<<<<<<<<<
 *     'darkslategrey' : (47, 79, 79),
 *     'darkturquoise' : (0, 206, 209),
 */
  __pyx_tuple__110 = PyTuple_Pack(3, __pyx_int_82, __pyx_int_139, __pyx_int_139); if (unlikely(!__pyx_tuple__110)) __PYX_ERR(2, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__110);
  __Pyx_GIVEREF(__pyx_tuple__110);

  /* "include/color_dict.pxi":117
 *     'darkslategray4' : (82, 139, 139),
 *     'darkslategrey' : (47, 79, 79),
 *     'darkturquoise' : (0, 206, 209),             # <<<<<<<<<<<<<<
 *     'darkviolet' : (148, 0, 211),
 *     'debianred' : (215, 7, 81),
 */
  __pyx_tuple__111 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_206, __pyx_int_209); if (unlikely(!__pyx_tuple__111)) __PYX_ERR(2, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__111);
  __Pyx_GIVEREF(__pyx_tuple__111);

  /* "include/color_dict.pxi":118
 *     'darkslategrey' : (47, 79, 79),
 *     'darkturquoise' : (0, 206, 209),
 *     'darkviolet' : (148, 0, 211),             # <<<<<<<<<<<<<<
 *     'debianred' : (215, 7, 81),
 *     'deeppink' : (255, 20, 147),
 */
  __pyx_tuple__112 = PyTuple_Pack(3, __pyx_int_148, __pyx_int_0, __pyx_int_211); if (unlikely(!__pyx_tuple__112)) __PYX_ERR(2, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__112);
  __Pyx_GIVEREF(__pyx_tuple__112);

  /* "include/color_dict.pxi":119
 *     'darkturquoise' : (0, 206, 209),
 *     'darkviolet' : (148, 0, 211),
 *     'debianred' : (215, 7, 81),             # <<<<<<<<<<<<<<
 *     'deeppink' : (255, 20, 147),
 *     'deeppink1' : (255, 20, 147),
 */
  __pyx_tuple__113 = PyTuple_Pack(3, __pyx_int_215, __pyx_int_7, __pyx_int_81); if (unlikely(!__pyx_tuple__113)) __PYX_ERR(2, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__113);
  __Pyx_GIVEREF(__pyx_tuple__113);

  /* "include/color_dict.pxi":120
 *     'darkviolet' : (148, 0, 211),
 *     'debianred' : (215, 7, 81),
 *     'deeppink' : (255, 20, 147),             # <<<<<<<<<<<<<<
 *     'deeppink1' : (255, 20, 147),
 *     'deeppink2' : (238, 18, 137),
 */
  __pyx_tuple__114 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_20, __pyx_int_147); if (unlikely(!__pyx_tuple__114)) __PYX_ERR(2, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__114);
  __Pyx_GIVEREF(__pyx_tuple__114);

  /* "include/color_dict.pxi":122
 *     'deeppink' : (255, 20, 147),
 *     'deeppink1' : (255, 20, 147),
 *     'deeppink2' : (238, 18, 137),             # <<<<<<<<<<<<<<
 *     'deeppink3' : (205, 16, 118),
 *     'deeppink4' : (139, 10, 80),
 */
  __pyx_tuple__115 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_18, __pyx_int_137); if (unlikely(!__pyx_tuple__115)) __PYX_ERR(2, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__115);
  __Pyx_GIVEREF(__pyx_tuple__115);

  /* "include/color_dict.pxi":123
 *     'deeppink1' : (255, 20, 147),
 *     'deeppink2' : (238, 18, 137),
 *     'deeppink3' : (205, 16, 118),             # <<<<<<<<<<<<<<
 *     'deeppink4' : (139, 10, 80),
 *     'deepskyblue' : (0, 191, 255),
 */
  __pyx_tuple__116 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_16, __pyx_int_118); if (unlikely(!__pyx_tuple__116)) __PYX_ERR(2, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__116);
  __Pyx_GIVEREF(__pyx_tuple__116);

  /* "include/color_dict.pxi":124
 *     'deeppink2' : (238, 18, 137),
 *     'deeppink3' : (205, 16, 118),
 *     'deeppink4' : (139, 10, 80),             # <<<<<<<<<<<<<<
 *     'deepskyblue' : (0, 191, 255),
 *     'deepskyblue1' : (0, 191, 255),
 */
  __pyx_tuple__117 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_10, __pyx_int_80); if (unlikely(!__pyx_tuple__117)) __PYX_ERR(2, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__117);
  __Pyx_GIVEREF(__pyx_tuple__117);

  /* "include/color_dict.pxi":125
 *     'deeppink3' : (205, 16, 118),
 *     'deeppink4' : (139, 10, 80),
 *     'deepskyblue' : (0, 191, 255),             # <<<<<<<<<<<<<<
 *     'deepskyblue1' : (0, 191, 255),
 *     'deepskyblue2' : (0, 178, 238),
 */
  __pyx_tuple__118 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_191, __pyx_int_255); if (unlikely(!__pyx_tuple__118)) __PYX_ERR(2, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__118);
  __Pyx_GIVEREF(__pyx_tuple__118);

  /* "include/color_dict.pxi":127
 *     'deepskyblue' : (0, 191, 255),
 *     'deepskyblue1' : (0, 191, 255),
 *     'deepskyblue2' : (0, 178, 238),             # <<<<<<<<<<<<<<
 *     'deepskyblue3' : (0, 154, 205),
 *     'deepskyblue4' : (0, 104, 139),
 */
  __pyx_tuple__119 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_178, __pyx_int_238); if (unlikely(!__pyx_tuple__119)) __PYX_ERR(2, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__119);
  __Pyx_GIVEREF(__pyx_tuple__119);

  /* "include/color_dict.pxi":128
 *     'deepskyblue1' : (0, 191, 255),
 *     'deepskyblue2' : (0, 178, 238),
 *     'deepskyblue3' : (0, 154, 205),             # <<<<<<<<<<<<<<
 *     'deepskyblue4' : (0, 104, 139),
 *     'dimgray' : (105, 105, 105),
 */
  __pyx_tuple__120 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_154, __pyx_int_205); if (unlikely(!__pyx_tuple__120)) __PYX_ERR(2, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__120);
  __Pyx_GIVEREF(__pyx_tuple__120);

  /* "include/color_dict.pxi":129
 *     'deepskyblue2' : (0, 178, 238),
 *     'deepskyblue3' : (0, 154, 205),
 *     'deepskyblue4' : (0, 104, 139),             # <<<<<<<<<<<<<<
 *     'dimgray' : (105, 105, 105),
 *     'dimgrey' : (105, 105, 105),
 */
  __pyx_tuple__121 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_104, __pyx_int_139); if (unlikely(!__pyx_tuple__121)) __PYX_ERR(2, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__121);
  __Pyx_GIVEREF(__pyx_tuple__121);

  /* "include/color_dict.pxi":130
 *     'deepskyblue3' : (0, 154, 205),
 *     'deepskyblue4' : (0, 104, 139),
 *     'dimgray' : (105, 105, 105),             # <<<<<<<<<<<<<<
 *     'dimgrey' : (105, 105, 105),
 *     'dodgerblue' : (30, 144, 255),
 */
  __pyx_tuple__122 = PyTuple_Pack(3, __pyx_int_105, __pyx_int_105, __pyx_int_105); if (unlikely(!__pyx_tuple__122)) __PYX_ERR(2, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__122);
  __Pyx_GIVEREF(__pyx_tuple__122);

  /* "include/color_dict.pxi":132
 *     'dimgray' : (105, 105, 105),
 *     'dimgrey' : (105, 105, 105),
 *     'dodgerblue' : (30, 144, 255),             # <<<<<<<<<<<<<<
 *     'dodgerblue1' : (30, 144, 255),
 *     'dodgerblue2' : (28, 134, 238),
 */
  __pyx_tuple__123 = PyTuple_Pack(3, __pyx_int_30, __pyx_int_144, __pyx_int_255); if (unlikely(!__pyx_tuple__123)) __PYX_ERR(2, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__123);
  __Pyx_GIVEREF(__pyx_tuple__123);

  /* "include/color_dict.pxi":134
 *     'dodgerblue' : (30, 144, 255),
 *     'dodgerblue1' : (30, 144, 255),
 *     'dodgerblue2' : (28, 134, 238),             # <<<<<<<<<<<<<<
 *     'dodgerblue3' : (24, 116, 205),
 *     'dodgerblue4' : (16, 78, 139),
 */
  __pyx_tuple__124 = PyTuple_Pack(3, __pyx_int_28, __pyx_int_134, __pyx_int_238); if (unlikely(!__pyx_tuple__124)) __PYX_ERR(2, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__124);
  __Pyx_GIVEREF(__pyx_tuple__124);

  /* "include/color_dict.pxi":135
 *     'dodgerblue1' : (30, 144, 255),
 *     'dodgerblue2' : (28, 134, 238),
 *     'dodgerblue3' : (24, 116, 205),             # <<<<<<<<<<<<<<
 *     'dodgerblue4' : (16, 78, 139),
 *     'firebrick' : (178, 34, 34),
 */
  __pyx_tuple__125 = PyTuple_Pack(3, __pyx_int_24, __pyx_int_116, __pyx_int_205); if (unlikely(!__pyx_tuple__125)) __PYX_ERR(2, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__125);
  __Pyx_GIVEREF(__pyx_tuple__125);

  /* "include/color_dict.pxi":136
 *     'dodgerblue2' : (28, 134, 238),
 *     'dodgerblue3' : (24, 116, 205),
 *     'dodgerblue4' : (16, 78, 139),             # <<<<<<<<<<<<<<
 *     'firebrick' : (178, 34, 34),
 *     'firebrick1' : (255, 48, 48),
 */
  __pyx_tuple__126 = PyTuple_Pack(3, __pyx_int_16, __pyx_int_78, __pyx_int_139); if (unlikely(!__pyx_tuple__126)) __PYX_ERR(2, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__126);
  __Pyx_GIVEREF(__pyx_tuple__126);

  /* "include/color_dict.pxi":137
 *     'dodgerblue3' : (24, 116, 205),
 *     'dodgerblue4' : (16, 78, 139),
 *     'firebrick' : (178, 34, 34),             # <<<<<<<<<<<<<<
 *     'firebrick1' : (255, 48, 48),
 *     'firebrick2' : (238, 44, 44),
 */
  __pyx_tuple__127 = PyTuple_Pack(3, __pyx_int_178, __pyx_int_34, __pyx_int_34); if (unlikely(!__pyx_tuple__127)) __PYX_ERR(2, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__127);
  __Pyx_GIVEREF(__pyx_tuple__127);

  /* "include/color_dict.pxi":138
 *     'dodgerblue4' : (16, 78, 139),
 *     'firebrick' : (178, 34, 34),
 *     'firebrick1' : (255, 48, 48),             # <<<<<<<<<<<<<<
 *     'firebrick2' : (238, 44, 44),
 *     'firebrick3' : (205, 38, 38),
 */
  __pyx_tuple__128 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_48, __pyx_int_48); if (unlikely(!__pyx_tuple__128)) __PYX_ERR(2, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__128);
  __Pyx_GIVEREF(__pyx_tuple__128);

  /* "include/color_dict.pxi":139
 *     'firebrick' : (178, 34, 34),
 *     'firebrick1' : (255, 48, 48),
 *     'firebrick2' : (238, 44, 44),             # <<<<<<<<<<<<<<
 *     'firebrick3' : (205, 38, 38),
 *     'firebrick4' : (139, 26, 26),
 */
  __pyx_tuple__129 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_44, __pyx_int_44); if (unlikely(!__pyx_tuple__129)) __PYX_ERR(2, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__129);
  __Pyx_GIVEREF(__pyx_tuple__129);

  /* "include/color_dict.pxi":140
 *     'firebrick1' : (255, 48, 48),
 *     'firebrick2' : (238, 44, 44),
 *     'firebrick3' : (205, 38, 38),             # <<<<<<<<<<<<<<
 *     'firebrick4' : (139, 26, 26),
 *     'floralwhite' : (255, 250, 240),
 */
  __pyx_tuple__130 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_38, __pyx_int_38); if (unlikely(!__pyx_tuple__130)) __PYX_ERR(2, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__130);
  __Pyx_GIVEREF(__pyx_tuple__130);

  /* "include/color_dict.pxi":141
 *     'firebrick2' : (238, 44, 44),
 *     'firebrick3' : (205, 38, 38),
 *     'firebrick4' : (139, 26, 26),             # <<<<<<<<<<<<<<
 *     'floralwhite' : (255, 250, 240),
 *     'forestgreen' : (34, 139, 34),
 */
  __pyx_tuple__131 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_26, __pyx_int_26); if (unlikely(!__pyx_tuple__131)) __PYX_ERR(2, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__131);
  __Pyx_GIVEREF(__pyx_tuple__131);

  /* "include/color_dict.pxi":142
 *     'firebrick3' : (205, 38, 38),
 *     'firebrick4' : (139, 26, 26),
 *     'floralwhite' : (255, 250, 240),             # <<<<<<<<<<<<<<
 *     'forestgreen' : (34, 139, 34),
 *     'gainsboro' : (220, 220, 220),
 */
  __pyx_tuple__132 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_250, __pyx_int_240); if (unlikely(!__pyx_tuple__132)) __PYX_ERR(2, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__132);
  __Pyx_GIVEREF(__pyx_tuple__132);

  /* "include/color_dict.pxi":143
 *     'firebrick4' : (139, 26, 26),
 *     'floralwhite' : (255, 250, 240),
 *     'forestgreen' : (34, 139, 34),             # <<<<<<<<<<<<<<
 *     'gainsboro' : (220, 220, 220),
 *     'ghostwhite' : (248, 248, 255),
 */
  __pyx_tuple__133 = PyTuple_Pack(3, __pyx_int_34, __pyx_int_139, __pyx_int_34); if (unlikely(!__pyx_tuple__133)) __PYX_ERR(2, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__133);
  __Pyx_GIVEREF(__pyx_tuple__133);

  /* "include/color_dict.pxi":144
 *     'floralwhite' : (255, 250, 240),
 *     'forestgreen' : (34, 139, 34),
 *     'gainsboro' : (220, 220, 220),             # <<<<<<<<<<<<<<
 *     'ghostwhite' : (248, 248, 255),
 *     'gold' : (255, 215, 0),
 */
  __pyx_tuple__134 = PyTuple_Pack(3, __pyx_int_220, __pyx_int_220, __pyx_int_220); if (unlikely(!__pyx_tuple__134)) __PYX_ERR(2, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__134);
  __Pyx_GIVEREF(__pyx_tuple__134);

  /* "include/color_dict.pxi":145
 *     'forestgreen' : (34, 139, 34),
 *     'gainsboro' : (220, 220, 220),
 *     'ghostwhite' : (248, 248, 255),             # <<<<<<<<<<<<<<
 *     'gold' : (255, 215, 0),
 *     'gold1' : (255, 215, 0),
 */
  __pyx_tuple__135 = PyTuple_Pack(3, __pyx_int_248, __pyx_int_248, __pyx_int_255); if (unlikely(!__pyx_tuple__135)) __PYX_ERR(2, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__135);
  __Pyx_GIVEREF(__pyx_tuple__135);

  /* "include/color_dict.pxi":146
 *     'gainsboro' : (220, 220, 220),
 *     'ghostwhite' : (248, 248, 255),
 *     'gold' : (255, 215, 0),             # <<<<<<<<<<<<<<
 *     'gold1' : (255, 215, 0),
 *     'gold2' : (238, 201, 0),
 */
  __pyx_tuple__136 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_215, __pyx_int_0); if (unlikely(!__pyx_tuple__136)) __PYX_ERR(2, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__136);
  __Pyx_GIVEREF(__pyx_tuple__136);

  /* "include/color_dict.pxi":148
 *     'gold' : (255, 215, 0),
 *     'gold1' : (255, 215, 0),
 *     'gold2' : (238, 201, 0),             # <<<<<<<<<<<<<<
 *     'gold3' : (205, 173, 0),
 *     'gold4' : (139, 117, 0),
 */
  __pyx_tuple__137 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_201, __pyx_int_0); if (unlikely(!__pyx_tuple__137)) __PYX_ERR(2, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__137);
  __Pyx_GIVEREF(__pyx_tuple__137);

  /* "include/color_dict.pxi":149
 *     'gold1' : (255, 215, 0),
 *     'gold2' : (238, 201, 0),
 *     'gold3' : (205, 173, 0),             # <<<<<<<<<<<<<<
 *     'gold4' : (139, 117, 0),
 *     'goldenrod' : (218, 165, 32),
 */
  __pyx_tuple__138 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_173, __pyx_int_0); if (unlikely(!__pyx_tuple__138)) __PYX_ERR(2, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__138);
  __Pyx_GIVEREF(__pyx_tuple__138);

  /* "include/color_dict.pxi":150
 *     'gold2' : (238, 201, 0),
 *     'gold3' : (205, 173, 0),
 *     'gold4' : (139, 117, 0),             # <<<<<<<<<<<<<<
 *     'goldenrod' : (218, 165, 32),
 *     'goldenrod1' : (255, 193, 37),
 */
  __pyx_tuple__139 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_117, __pyx_int_0); if (unlikely(!__pyx_tuple__139)) __PYX_ERR(2, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__139);
  __Pyx_GIVEREF(__pyx_tuple__139);

  /* "include/color_dict.pxi":151
 *     'gold3' : (205, 173, 0),
 *     'gold4' : (139, 117, 0),
 *     'goldenrod' : (218, 165, 32),             # <<<<<<<<<<<<<<
 *     'goldenrod1' : (255, 193, 37),
 *     'goldenrod2' : (238, 180, 34),
 */
  __pyx_tuple__140 = PyTuple_Pack(3, __pyx_int_218, __pyx_int_165, __pyx_int_32); if (unlikely(!__pyx_tuple__140)) __PYX_ERR(2, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__140);
  __Pyx_GIVEREF(__pyx_tuple__140);

  /* "include/color_dict.pxi":152
 *     'gold4' : (139, 117, 0),
 *     'goldenrod' : (218, 165, 32),
 *     'goldenrod1' : (255, 193, 37),             # <<<<<<<<<<<<<<
 *     'goldenrod2' : (238, 180, 34),
 *     'goldenrod3' : (205, 155, 29),
 */
  __pyx_tuple__141 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_193, __pyx_int_37); if (unlikely(!__pyx_tuple__141)) __PYX_ERR(2, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__141);
  __Pyx_GIVEREF(__pyx_tuple__141);

  /* "include/color_dict.pxi":153
 *     'goldenrod' : (218, 165, 32),
 *     'goldenrod1' : (255, 193, 37),
 *     'goldenrod2' : (238, 180, 34),             # <<<<<<<<<<<<<<
 *     'goldenrod3' : (205, 155, 29),
 *     'goldenrod4' : (139, 105, 20),
 */
  __pyx_tuple__142 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_180, __pyx_int_34); if (unlikely(!__pyx_tuple__142)) __PYX_ERR(2, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__142);
  __Pyx_GIVEREF(__pyx_tuple__142);

  /* "include/color_dict.pxi":154
 *     'goldenrod1' : (255, 193, 37),
 *     'goldenrod2' : (238, 180, 34),
 *     'goldenrod3' : (205, 155, 29),             # <<<<<<<<<<<<<<
 *     'goldenrod4' : (139, 105, 20),
 *     'gray' : (190, 190, 190),
 */
  __pyx_tuple__143 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_155, __pyx_int_29); if (unlikely(!__pyx_tuple__143)) __PYX_ERR(2, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__143);
  __Pyx_GIVEREF(__pyx_tuple__143);

  /* "include/color_dict.pxi":155
 *     'goldenrod2' : (238, 180, 34),
 *     'goldenrod3' : (205, 155, 29),
 *     'goldenrod4' : (139, 105, 20),             # <<<<<<<<<<<<<<
 *     'gray' : (190, 190, 190),
 *     'gray0' : (0, 0, 0),
 */
  __pyx_tuple__144 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_105, __pyx_int_20); if (unlikely(!__pyx_tuple__144)) __PYX_ERR(2, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__144);
  __Pyx_GIVEREF(__pyx_tuple__144);

  /* "include/color_dict.pxi":156
 *     'goldenrod3' : (205, 155, 29),
 *     'goldenrod4' : (139, 105, 20),
 *     'gray' : (190, 190, 190),             # <<<<<<<<<<<<<<
 *     'gray0' : (0, 0, 0),
 *     'gray1' : (3, 3, 3),
 */
  __pyx_tuple__145 = PyTuple_Pack(3, __pyx_int_190, __pyx_int_190, __pyx_int_190); if (unlikely(!__pyx_tuple__145)) __PYX_ERR(2, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__145);
  __Pyx_GIVEREF(__pyx_tuple__145);

  /* "include/color_dict.pxi":158
 *     'gray' : (190, 190, 190),
 *     'gray0' : (0, 0, 0),
 *     'gray1' : (3, 3, 3),             # <<<<<<<<<<<<<<
 *     'gray10' : (26, 26, 26),
 *     'gray100' : (255, 255, 255),
 */
  __pyx_tuple__146 = PyTuple_Pack(3, __pyx_int_3, __pyx_int_3, __pyx_int_3); if (unlikely(!__pyx_tuple__146)) __PYX_ERR(2, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__146);
  __Pyx_GIVEREF(__pyx_tuple__146);

  /* "include/color_dict.pxi":159
 *     'gray0' : (0, 0, 0),
 *     'gray1' : (3, 3, 3),
 *     'gray10' : (26, 26, 26),             # <<<<<<<<<<<<<<
 *     'gray100' : (255, 255, 255),
 *     'gray11' : (28, 28, 28),
 */
  __pyx_tuple__147 = PyTuple_Pack(3, __pyx_int_26, __pyx_int_26, __pyx_int_26); if (unlikely(!__pyx_tuple__147)) __PYX_ERR(2, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__147);
  __Pyx_GIVEREF(__pyx_tuple__147);

  /* "include/color_dict.pxi":160
 *     'gray1' : (3, 3, 3),
 *     'gray10' : (26, 26, 26),
 *     'gray100' : (255, 255, 255),             # <<<<<<<<<<<<<<
 *     'gray11' : (28, 28, 28),
 *     'gray12' : (31, 31, 31),
 */
  __pyx_tuple__148 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_255, __pyx_int_255); if (unlikely(!__pyx_tuple__148)) __PYX_ERR(2, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__148);
  __Pyx_GIVEREF(__pyx_tuple__148);

  /* "include/color_dict.pxi":161
 *     'gray10' : (26, 26, 26),
 *     'gray100' : (255, 255, 255),
 *     'gray11' : (28, 28, 28),             # <<<<<<<<<<<<<<
 *     'gray12' : (31, 31, 31),
 *     'gray13' : (33, 33, 33),
 */
  __pyx_tuple__149 = PyTuple_Pack(3, __pyx_int_28, __pyx_int_28, __pyx_int_28); if (unlikely(!__pyx_tuple__149)) __PYX_ERR(2, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__149);
  __Pyx_GIVEREF(__pyx_tuple__149);

  /* "include/color_dict.pxi":162
 *     'gray100' : (255, 255, 255),
 *     'gray11' : (28, 28, 28),
 *     'gray12' : (31, 31, 31),             # <<<<<<<<<<<<<<
 *     'gray13' : (33, 33, 33),
 *     'gray14' : (36, 36, 36),
 */
  __pyx_tuple__150 = PyTuple_Pack(3, __pyx_int_31, __pyx_int_31, __pyx_int_31); if (unlikely(!__pyx_tuple__150)) __PYX_ERR(2, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__150);
  __Pyx_GIVEREF(__pyx_tuple__150);

  /* "include/color_dict.pxi":163
 *     'gray11' : (28, 28, 28),
 *     'gray12' : (31, 31, 31),
 *     'gray13' : (33, 33, 33),             # <<<<<<<<<<<<<<
 *     'gray14' : (36, 36, 36),
 *     'gray15' : (38, 38, 38),
 */
  __pyx_tuple__151 = PyTuple_Pack(3, __pyx_int_33, __pyx_int_33, __pyx_int_33); if (unlikely(!__pyx_tuple__151)) __PYX_ERR(2, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__151);
  __Pyx_GIVEREF(__pyx_tuple__151);

  /* "include/color_dict.pxi":164
 *     'gray12' : (31, 31, 31),
 *     'gray13' : (33, 33, 33),
 *     'gray14' : (36, 36, 36),             # <<<<<<<<<<<<<<
 *     'gray15' : (38, 38, 38),
 *     'gray16' : (41, 41, 41),
 */
  __pyx_tuple__152 = PyTuple_Pack(3, __pyx_int_36, __pyx_int_36, __pyx_int_36); if (unlikely(!__pyx_tuple__152)) __PYX_ERR(2, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__152);
  __Pyx_GIVEREF(__pyx_tuple__152);

  /* "include/color_dict.pxi":165
 *     'gray13' : (33, 33, 33),
 *     'gray14' : (36, 36, 36),
 *     'gray15' : (38, 38, 38),             # <<<<<<<<<<<<<<
 *     'gray16' : (41, 41, 41),
 *     'gray17' : (43, 43, 43),
 */
  __pyx_tuple__153 = PyTuple_Pack(3, __pyx_int_38, __pyx_int_38, __pyx_int_38); if (unlikely(!__pyx_tuple__153)) __PYX_ERR(2, 165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__153);
  __Pyx_GIVEREF(__pyx_tuple__153);

  /* "include/color_dict.pxi":166
 *     'gray14' : (36, 36, 36),
 *     'gray15' : (38, 38, 38),
 *     'gray16' : (41, 41, 41),             # <<<<<<<<<<<<<<
 *     'gray17' : (43, 43, 43),
 *     'gray18' : (46, 46, 46),
 */
  __pyx_tuple__154 = PyTuple_Pack(3, __pyx_int_41, __pyx_int_41, __pyx_int_41); if (unlikely(!__pyx_tuple__154)) __PYX_ERR(2, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__154);
  __Pyx_GIVEREF(__pyx_tuple__154);

  /* "include/color_dict.pxi":167
 *     'gray15' : (38, 38, 38),
 *     'gray16' : (41, 41, 41),
 *     'gray17' : (43, 43, 43),             # <<<<<<<<<<<<<<
 *     'gray18' : (46, 46, 46),
 *     'gray19' : (48, 48, 48),
 */
  __pyx_tuple__155 = PyTuple_Pack(3, __pyx_int_43, __pyx_int_43, __pyx_int_43); if (unlikely(!__pyx_tuple__155)) __PYX_ERR(2, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__155);
  __Pyx_GIVEREF(__pyx_tuple__155);

  /* "include/color_dict.pxi":168
 *     'gray16' : (41, 41, 41),
 *     'gray17' : (43, 43, 43),
 *     'gray18' : (46, 46, 46),             # <<<<<<<<<<<<<<
 *     'gray19' : (48, 48, 48),
 *     'gray2' : (5, 5, 5),
 */
  __pyx_tuple__156 = PyTuple_Pack(3, __pyx_int_46, __pyx_int_46, __pyx_int_46); if (unlikely(!__pyx_tuple__156)) __PYX_ERR(2, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__156);
  __Pyx_GIVEREF(__pyx_tuple__156);

  /* "include/color_dict.pxi":169
 *     'gray17' : (43, 43, 43),
 *     'gray18' : (46, 46, 46),
 *     'gray19' : (48, 48, 48),             # <<<<<<<<<<<<<<
 *     'gray2' : (5, 5, 5),
 *     'gray20' : (51, 51, 51),
 */
  __pyx_tuple__157 = PyTuple_Pack(3, __pyx_int_48, __pyx_int_48, __pyx_int_48); if (unlikely(!__pyx_tuple__157)) __PYX_ERR(2, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__157);
  __Pyx_GIVEREF(__pyx_tuple__157);

  /* "include/color_dict.pxi":170
 *     'gray18' : (46, 46, 46),
 *     'gray19' : (48, 48, 48),
 *     'gray2' : (5, 5, 5),             # <<<<<<<<<<<<<<
 *     'gray20' : (51, 51, 51),
 *     'gray21' : (54, 54, 54),
 */
  __pyx_tuple__158 = PyTuple_Pack(3, __pyx_int_5, __pyx_int_5, __pyx_int_5); if (unlikely(!__pyx_tuple__158)) __PYX_ERR(2, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__158);
  __Pyx_GIVEREF(__pyx_tuple__158);

  /* "include/color_dict.pxi":171
 *     'gray19' : (48, 48, 48),
 *     'gray2' : (5, 5, 5),
 *     'gray20' : (51, 51, 51),             # <<<<<<<<<<<<<<
 *     'gray21' : (54, 54, 54),
 *     'gray22' : (56, 56, 56),
 */
  __pyx_tuple__159 = PyTuple_Pack(3, __pyx_int_51, __pyx_int_51, __pyx_int_51); if (unlikely(!__pyx_tuple__159)) __PYX_ERR(2, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__159);
  __Pyx_GIVEREF(__pyx_tuple__159);

  /* "include/color_dict.pxi":172
 *     'gray2' : (5, 5, 5),
 *     'gray20' : (51, 51, 51),
 *     'gray21' : (54, 54, 54),             # <<<<<<<<<<<<<<
 *     'gray22' : (56, 56, 56),
 *     'gray23' : (59, 59, 59),
 */
  __pyx_tuple__160 = PyTuple_Pack(3, __pyx_int_54, __pyx_int_54, __pyx_int_54); if (unlikely(!__pyx_tuple__160)) __PYX_ERR(2, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__160);
  __Pyx_GIVEREF(__pyx_tuple__160);

  /* "include/color_dict.pxi":173
 *     'gray20' : (51, 51, 51),
 *     'gray21' : (54, 54, 54),
 *     'gray22' : (56, 56, 56),             # <<<<<<<<<<<<<<
 *     'gray23' : (59, 59, 59),
 *     'gray24' : (61, 61, 61),
 */
  __pyx_tuple__161 = PyTuple_Pack(3, __pyx_int_56, __pyx_int_56, __pyx_int_56); if (unlikely(!__pyx_tuple__161)) __PYX_ERR(2, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__161);
  __Pyx_GIVEREF(__pyx_tuple__161);

  /* "include/color_dict.pxi":174
 *     'gray21' : (54, 54, 54),
 *     'gray22' : (56, 56, 56),
 *     'gray23' : (59, 59, 59),             # <<<<<<<<<<<<<<
 *     'gray24' : (61, 61, 61),
 *     'gray25' : (64, 64, 64),
 */
  __pyx_tuple__162 = PyTuple_Pack(3, __pyx_int_59, __pyx_int_59, __pyx_int_59); if (unlikely(!__pyx_tuple__162)) __PYX_ERR(2, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__162);
  __Pyx_GIVEREF(__pyx_tuple__162);

  /* "include/color_dict.pxi":175
 *     'gray22' : (56, 56, 56),
 *     'gray23' : (59, 59, 59),
 *     'gray24' : (61, 61, 61),             # <<<<<<<<<<<<<<
 *     'gray25' : (64, 64, 64),
 *     'gray26' : (66, 66, 66),
 */
  __pyx_tuple__163 = PyTuple_Pack(3, __pyx_int_61, __pyx_int_61, __pyx_int_61); if (unlikely(!__pyx_tuple__163)) __PYX_ERR(2, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__163);
  __Pyx_GIVEREF(__pyx_tuple__163);

  /* "include/color_dict.pxi":176
 *     'gray23' : (59, 59, 59),
 *     'gray24' : (61, 61, 61),
 *     'gray25' : (64, 64, 64),             # <<<<<<<<<<<<<<
 *     'gray26' : (66, 66, 66),
 *     'gray27' : (69, 69, 69),
 */
  __pyx_tuple__164 = PyTuple_Pack(3, __pyx_int_64, __pyx_int_64, __pyx_int_64); if (unlikely(!__pyx_tuple__164)) __PYX_ERR(2, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__164);
  __Pyx_GIVEREF(__pyx_tuple__164);

  /* "include/color_dict.pxi":177
 *     'gray24' : (61, 61, 61),
 *     'gray25' : (64, 64, 64),
 *     'gray26' : (66, 66, 66),             # <<<<<<<<<<<<<<
 *     'gray27' : (69, 69, 69),
 *     'gray28' : (71, 71, 71),
 */
  __pyx_tuple__165 = PyTuple_Pack(3, __pyx_int_66, __pyx_int_66, __pyx_int_66); if (unlikely(!__pyx_tuple__165)) __PYX_ERR(2, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__165);
  __Pyx_GIVEREF(__pyx_tuple__165);

  /* "include/color_dict.pxi":178
 *     'gray25' : (64, 64, 64),
 *     'gray26' : (66, 66, 66),
 *     'gray27' : (69, 69, 69),             # <<<<<<<<<<<<<<
 *     'gray28' : (71, 71, 71),
 *     'gray29' : (74, 74, 74),
 */
  __pyx_tuple__166 = PyTuple_Pack(3, __pyx_int_69, __pyx_int_69, __pyx_int_69); if (unlikely(!__pyx_tuple__166)) __PYX_ERR(2, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__166);
  __Pyx_GIVEREF(__pyx_tuple__166);

  /* "include/color_dict.pxi":179
 *     'gray26' : (66, 66, 66),
 *     'gray27' : (69, 69, 69),
 *     'gray28' : (71, 71, 71),             # <<<<<<<<<<<<<<
 *     'gray29' : (74, 74, 74),
 *     'gray3' : (8, 8, 8),
 */
  __pyx_tuple__167 = PyTuple_Pack(3, __pyx_int_71, __pyx_int_71, __pyx_int_71); if (unlikely(!__pyx_tuple__167)) __PYX_ERR(2, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__167);
  __Pyx_GIVEREF(__pyx_tuple__167);

  /* "include/color_dict.pxi":180
 *     'gray27' : (69, 69, 69),
 *     'gray28' : (71, 71, 71),
 *     'gray29' : (74, 74, 74),             # <<<<<<<<<<<<<<
 *     'gray3' : (8, 8, 8),
 *     'gray30' : (77, 77, 77),
 */
  __pyx_tuple__168 = PyTuple_Pack(3, __pyx_int_74, __pyx_int_74, __pyx_int_74); if (unlikely(!__pyx_tuple__168)) __PYX_ERR(2, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__168);
  __Pyx_GIVEREF(__pyx_tuple__168);

  /* "include/color_dict.pxi":181
 *     'gray28' : (71, 71, 71),
 *     'gray29' : (74, 74, 74),
 *     'gray3' : (8, 8, 8),             # <<<<<<<<<<<<<<
 *     'gray30' : (77, 77, 77),
 *     'gray31' : (79, 79, 79),
 */
  __pyx_tuple__169 = PyTuple_Pack(3, __pyx_int_8, __pyx_int_8, __pyx_int_8); if (unlikely(!__pyx_tuple__169)) __PYX_ERR(2, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__169);
  __Pyx_GIVEREF(__pyx_tuple__169);

  /* "include/color_dict.pxi":182
 *     'gray29' : (74, 74, 74),
 *     'gray3' : (8, 8, 8),
 *     'gray30' : (77, 77, 77),             # <<<<<<<<<<<<<<
 *     'gray31' : (79, 79, 79),
 *     'gray32' : (82, 82, 82),
 */
  __pyx_tuple__170 = PyTuple_Pack(3, __pyx_int_77, __pyx_int_77, __pyx_int_77); if (unlikely(!__pyx_tuple__170)) __PYX_ERR(2, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__170);
  __Pyx_GIVEREF(__pyx_tuple__170);

  /* "include/color_dict.pxi":183
 *     'gray3' : (8, 8, 8),
 *     'gray30' : (77, 77, 77),
 *     'gray31' : (79, 79, 79),             # <<<<<<<<<<<<<<
 *     'gray32' : (82, 82, 82),
 *     'gray33' : (84, 84, 84),
 */
  __pyx_tuple__171 = PyTuple_Pack(3, __pyx_int_79, __pyx_int_79, __pyx_int_79); if (unlikely(!__pyx_tuple__171)) __PYX_ERR(2, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__171);
  __Pyx_GIVEREF(__pyx_tuple__171);

  /* "include/color_dict.pxi":184
 *     'gray30' : (77, 77, 77),
 *     'gray31' : (79, 79, 79),
 *     'gray32' : (82, 82, 82),             # <<<<<<<<<<<<<<
 *     'gray33' : (84, 84, 84),
 *     'gray34' : (87, 87, 87),
 */
  __pyx_tuple__172 = PyTuple_Pack(3, __pyx_int_82, __pyx_int_82, __pyx_int_82); if (unlikely(!__pyx_tuple__172)) __PYX_ERR(2, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__172);
  __Pyx_GIVEREF(__pyx_tuple__172);

  /* "include/color_dict.pxi":185
 *     'gray31' : (79, 79, 79),
 *     'gray32' : (82, 82, 82),
 *     'gray33' : (84, 84, 84),             # <<<<<<<<<<<<<<
 *     'gray34' : (87, 87, 87),
 *     'gray35' : (89, 89, 89),
 */
  __pyx_tuple__173 = PyTuple_Pack(3, __pyx_int_84, __pyx_int_84, __pyx_int_84); if (unlikely(!__pyx_tuple__173)) __PYX_ERR(2, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__173);
  __Pyx_GIVEREF(__pyx_tuple__173);

  /* "include/color_dict.pxi":186
 *     'gray32' : (82, 82, 82),
 *     'gray33' : (84, 84, 84),
 *     'gray34' : (87, 87, 87),             # <<<<<<<<<<<<<<
 *     'gray35' : (89, 89, 89),
 *     'gray36' : (92, 92, 92),
 */
  __pyx_tuple__174 = PyTuple_Pack(3, __pyx_int_87, __pyx_int_87, __pyx_int_87); if (unlikely(!__pyx_tuple__174)) __PYX_ERR(2, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__174);
  __Pyx_GIVEREF(__pyx_tuple__174);

  /* "include/color_dict.pxi":187
 *     'gray33' : (84, 84, 84),
 *     'gray34' : (87, 87, 87),
 *     'gray35' : (89, 89, 89),             # <<<<<<<<<<<<<<
 *     'gray36' : (92, 92, 92),
 *     'gray37' : (94, 94, 94),
 */
  __pyx_tuple__175 = PyTuple_Pack(3, __pyx_int_89, __pyx_int_89, __pyx_int_89); if (unlikely(!__pyx_tuple__175)) __PYX_ERR(2, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__175);
  __Pyx_GIVEREF(__pyx_tuple__175);

  /* "include/color_dict.pxi":188
 *     'gray34' : (87, 87, 87),
 *     'gray35' : (89, 89, 89),
 *     'gray36' : (92, 92, 92),             # <<<<<<<<<<<<<<
 *     'gray37' : (94, 94, 94),
 *     'gray38' : (97, 97, 97),
 */
  __pyx_tuple__176 = PyTuple_Pack(3, __pyx_int_92, __pyx_int_92, __pyx_int_92); if (unlikely(!__pyx_tuple__176)) __PYX_ERR(2, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__176);
  __Pyx_GIVEREF(__pyx_tuple__176);

  /* "include/color_dict.pxi":189
 *     'gray35' : (89, 89, 89),
 *     'gray36' : (92, 92, 92),
 *     'gray37' : (94, 94, 94),             # <<<<<<<<<<<<<<
 *     'gray38' : (97, 97, 97),
 *     'gray39' : (99, 99, 99),
 */
  __pyx_tuple__177 = PyTuple_Pack(3, __pyx_int_94, __pyx_int_94, __pyx_int_94); if (unlikely(!__pyx_tuple__177)) __PYX_ERR(2, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__177);
  __Pyx_GIVEREF(__pyx_tuple__177);

  /* "include/color_dict.pxi":190
 *     'gray36' : (92, 92, 92),
 *     'gray37' : (94, 94, 94),
 *     'gray38' : (97, 97, 97),             # <<<<<<<<<<<<<<
 *     'gray39' : (99, 99, 99),
 *     'gray4' : (10, 10, 10),
 */
  __pyx_tuple__178 = PyTuple_Pack(3, __pyx_int_97, __pyx_int_97, __pyx_int_97); if (unlikely(!__pyx_tuple__178)) __PYX_ERR(2, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__178);
  __Pyx_GIVEREF(__pyx_tuple__178);

  /* "include/color_dict.pxi":191
 *     'gray37' : (94, 94, 94),
 *     'gray38' : (97, 97, 97),
 *     'gray39' : (99, 99, 99),             # <<<<<<<<<<<<<<
 *     'gray4' : (10, 10, 10),
 *     'gray40' : (102, 102, 102),
 */
  __pyx_tuple__179 = PyTuple_Pack(3, __pyx_int_99, __pyx_int_99, __pyx_int_99); if (unlikely(!__pyx_tuple__179)) __PYX_ERR(2, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__179);
  __Pyx_GIVEREF(__pyx_tuple__179);

  /* "include/color_dict.pxi":192
 *     'gray38' : (97, 97, 97),
 *     'gray39' : (99, 99, 99),
 *     'gray4' : (10, 10, 10),             # <<<<<<<<<<<<<<
 *     'gray40' : (102, 102, 102),
 *     'gray41' : (105, 105, 105),
 */
  __pyx_tuple__180 = PyTuple_Pack(3, __pyx_int_10, __pyx_int_10, __pyx_int_10); if (unlikely(!__pyx_tuple__180)) __PYX_ERR(2, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__180);
  __Pyx_GIVEREF(__pyx_tuple__180);

  /* "include/color_dict.pxi":193
 *     'gray39' : (99, 99, 99),
 *     'gray4' : (10, 10, 10),
 *     'gray40' : (102, 102, 102),             # <<<<<<<<<<<<<<
 *     'gray41' : (105, 105, 105),
 *     'gray42' : (107, 107, 107),
 */
  __pyx_tuple__181 = PyTuple_Pack(3, __pyx_int_102, __pyx_int_102, __pyx_int_102); if (unlikely(!__pyx_tuple__181)) __PYX_ERR(2, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__181);
  __Pyx_GIVEREF(__pyx_tuple__181);

  /* "include/color_dict.pxi":195
 *     'gray40' : (102, 102, 102),
 *     'gray41' : (105, 105, 105),
 *     'gray42' : (107, 107, 107),             # <<<<<<<<<<<<<<
 *     'gray43' : (110, 110, 110),
 *     'gray44' : (112, 112, 112),
 */
  __pyx_tuple__182 = PyTuple_Pack(3, __pyx_int_107, __pyx_int_107, __pyx_int_107); if (unlikely(!__pyx_tuple__182)) __PYX_ERR(2, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__182);
  __Pyx_GIVEREF(__pyx_tuple__182);

  /* "include/color_dict.pxi":196
 *     'gray41' : (105, 105, 105),
 *     'gray42' : (107, 107, 107),
 *     'gray43' : (110, 110, 110),             # <<<<<<<<<<<<<<
 *     'gray44' : (112, 112, 112),
 *     'gray45' : (115, 115, 115),
 */
  __pyx_tuple__183 = PyTuple_Pack(3, __pyx_int_110, __pyx_int_110, __pyx_int_110); if (unlikely(!__pyx_tuple__183)) __PYX_ERR(2, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__183);
  __Pyx_GIVEREF(__pyx_tuple__183);

  /* "include/color_dict.pxi":197
 *     'gray42' : (107, 107, 107),
 *     'gray43' : (110, 110, 110),
 *     'gray44' : (112, 112, 112),             # <<<<<<<<<<<<<<
 *     'gray45' : (115, 115, 115),
 *     'gray46' : (117, 117, 117),
 */
  __pyx_tuple__184 = PyTuple_Pack(3, __pyx_int_112, __pyx_int_112, __pyx_int_112); if (unlikely(!__pyx_tuple__184)) __PYX_ERR(2, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__184);
  __Pyx_GIVEREF(__pyx_tuple__184);

  /* "include/color_dict.pxi":198
 *     'gray43' : (110, 110, 110),
 *     'gray44' : (112, 112, 112),
 *     'gray45' : (115, 115, 115),             # <<<<<<<<<<<<<<
 *     'gray46' : (117, 117, 117),
 *     'gray47' : (120, 120, 120),
 */
  __pyx_tuple__185 = PyTuple_Pack(3, __pyx_int_115, __pyx_int_115, __pyx_int_115); if (unlikely(!__pyx_tuple__185)) __PYX_ERR(2, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__185);
  __Pyx_GIVEREF(__pyx_tuple__185);

  /* "include/color_dict.pxi":199
 *     'gray44' : (112, 112, 112),
 *     'gray45' : (115, 115, 115),
 *     'gray46' : (117, 117, 117),             # <<<<<<<<<<<<<<
 *     'gray47' : (120, 120, 120),
 *     'gray48' : (122, 122, 122),
 */
  __pyx_tuple__186 = PyTuple_Pack(3, __pyx_int_117, __pyx_int_117, __pyx_int_117); if (unlikely(!__pyx_tuple__186)) __PYX_ERR(2, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__186);
  __Pyx_GIVEREF(__pyx_tuple__186);

  /* "include/color_dict.pxi":200
 *     'gray45' : (115, 115, 115),
 *     'gray46' : (117, 117, 117),
 *     'gray47' : (120, 120, 120),             # <<<<<<<<<<<<<<
 *     'gray48' : (122, 122, 122),
 *     'gray49' : (125, 125, 125),
 */
  __pyx_tuple__187 = PyTuple_Pack(3, __pyx_int_120, __pyx_int_120, __pyx_int_120); if (unlikely(!__pyx_tuple__187)) __PYX_ERR(2, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__187);
  __Pyx_GIVEREF(__pyx_tuple__187);

  /* "include/color_dict.pxi":201
 *     'gray46' : (117, 117, 117),
 *     'gray47' : (120, 120, 120),
 *     'gray48' : (122, 122, 122),             # <<<<<<<<<<<<<<
 *     'gray49' : (125, 125, 125),
 *     'gray5' : (13, 13, 13),
 */
  __pyx_tuple__188 = PyTuple_Pack(3, __pyx_int_122, __pyx_int_122, __pyx_int_122); if (unlikely(!__pyx_tuple__188)) __PYX_ERR(2, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__188);
  __Pyx_GIVEREF(__pyx_tuple__188);

  /* "include/color_dict.pxi":202
 *     'gray47' : (120, 120, 120),
 *     'gray48' : (122, 122, 122),
 *     'gray49' : (125, 125, 125),             # <<<<<<<<<<<<<<
 *     'gray5' : (13, 13, 13),
 *     'gray50' : (127, 127, 127),
 */
  __pyx_tuple__189 = PyTuple_Pack(3, __pyx_int_125, __pyx_int_125, __pyx_int_125); if (unlikely(!__pyx_tuple__189)) __PYX_ERR(2, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__189);
  __Pyx_GIVEREF(__pyx_tuple__189);

  /* "include/color_dict.pxi":203
 *     'gray48' : (122, 122, 122),
 *     'gray49' : (125, 125, 125),
 *     'gray5' : (13, 13, 13),             # <<<<<<<<<<<<<<
 *     'gray50' : (127, 127, 127),
 *     'gray51' : (130, 130, 130),
 */
  __pyx_tuple__190 = PyTuple_Pack(3, __pyx_int_13, __pyx_int_13, __pyx_int_13); if (unlikely(!__pyx_tuple__190)) __PYX_ERR(2, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__190);
  __Pyx_GIVEREF(__pyx_tuple__190);

  /* "include/color_dict.pxi":204
 *     'gray49' : (125, 125, 125),
 *     'gray5' : (13, 13, 13),
 *     'gray50' : (127, 127, 127),             # <<<<<<<<<<<<<<
 *     'gray51' : (130, 130, 130),
 *     'gray52' : (133, 133, 133),
 */
  __pyx_tuple__191 = PyTuple_Pack(3, __pyx_int_127, __pyx_int_127, __pyx_int_127); if (unlikely(!__pyx_tuple__191)) __PYX_ERR(2, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__191);
  __Pyx_GIVEREF(__pyx_tuple__191);

  /* "include/color_dict.pxi":205
 *     'gray5' : (13, 13, 13),
 *     'gray50' : (127, 127, 127),
 *     'gray51' : (130, 130, 130),             # <<<<<<<<<<<<<<
 *     'gray52' : (133, 133, 133),
 *     'gray53' : (135, 135, 135),
 */
  __pyx_tuple__192 = PyTuple_Pack(3, __pyx_int_130, __pyx_int_130, __pyx_int_130); if (unlikely(!__pyx_tuple__192)) __PYX_ERR(2, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__192);
  __Pyx_GIVEREF(__pyx_tuple__192);

  /* "include/color_dict.pxi":206
 *     'gray50' : (127, 127, 127),
 *     'gray51' : (130, 130, 130),
 *     'gray52' : (133, 133, 133),             # <<<<<<<<<<<<<<
 *     'gray53' : (135, 135, 135),
 *     'gray54' : (138, 138, 138),
 */
  __pyx_tuple__193 = PyTuple_Pack(3, __pyx_int_133, __pyx_int_133, __pyx_int_133); if (unlikely(!__pyx_tuple__193)) __PYX_ERR(2, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__193);
  __Pyx_GIVEREF(__pyx_tuple__193);

  /* "include/color_dict.pxi":207
 *     'gray51' : (130, 130, 130),
 *     'gray52' : (133, 133, 133),
 *     'gray53' : (135, 135, 135),             # <<<<<<<<<<<<<<
 *     'gray54' : (138, 138, 138),
 *     'gray55' : (140, 140, 140),
 */
  __pyx_tuple__194 = PyTuple_Pack(3, __pyx_int_135, __pyx_int_135, __pyx_int_135); if (unlikely(!__pyx_tuple__194)) __PYX_ERR(2, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__194);
  __Pyx_GIVEREF(__pyx_tuple__194);

  /* "include/color_dict.pxi":208
 *     'gray52' : (133, 133, 133),
 *     'gray53' : (135, 135, 135),
 *     'gray54' : (138, 138, 138),             # <<<<<<<<<<<<<<
 *     'gray55' : (140, 140, 140),
 *     'gray56' : (143, 143, 143),
 */
  __pyx_tuple__195 = PyTuple_Pack(3, __pyx_int_138, __pyx_int_138, __pyx_int_138); if (unlikely(!__pyx_tuple__195)) __PYX_ERR(2, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__195);
  __Pyx_GIVEREF(__pyx_tuple__195);

  /* "include/color_dict.pxi":209
 *     'gray53' : (135, 135, 135),
 *     'gray54' : (138, 138, 138),
 *     'gray55' : (140, 140, 140),             # <<<<<<<<<<<<<<
 *     'gray56' : (143, 143, 143),
 *     'gray57' : (145, 145, 145),
 */
  __pyx_tuple__196 = PyTuple_Pack(3, __pyx_int_140, __pyx_int_140, __pyx_int_140); if (unlikely(!__pyx_tuple__196)) __PYX_ERR(2, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__196);
  __Pyx_GIVEREF(__pyx_tuple__196);

  /* "include/color_dict.pxi":210
 *     'gray54' : (138, 138, 138),
 *     'gray55' : (140, 140, 140),
 *     'gray56' : (143, 143, 143),             # <<<<<<<<<<<<<<
 *     'gray57' : (145, 145, 145),
 *     'gray58' : (148, 148, 148),
 */
  __pyx_tuple__197 = PyTuple_Pack(3, __pyx_int_143, __pyx_int_143, __pyx_int_143); if (unlikely(!__pyx_tuple__197)) __PYX_ERR(2, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__197);
  __Pyx_GIVEREF(__pyx_tuple__197);

  /* "include/color_dict.pxi":211
 *     'gray55' : (140, 140, 140),
 *     'gray56' : (143, 143, 143),
 *     'gray57' : (145, 145, 145),             # <<<<<<<<<<<<<<
 *     'gray58' : (148, 148, 148),
 *     'gray59' : (150, 150, 150),
 */
  __pyx_tuple__198 = PyTuple_Pack(3, __pyx_int_145, __pyx_int_145, __pyx_int_145); if (unlikely(!__pyx_tuple__198)) __PYX_ERR(2, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__198);
  __Pyx_GIVEREF(__pyx_tuple__198);

  /* "include/color_dict.pxi":212
 *     'gray56' : (143, 143, 143),
 *     'gray57' : (145, 145, 145),
 *     'gray58' : (148, 148, 148),             # <<<<<<<<<<<<<<
 *     'gray59' : (150, 150, 150),
 *     'gray6' : (15, 15, 15),
 */
  __pyx_tuple__199 = PyTuple_Pack(3, __pyx_int_148, __pyx_int_148, __pyx_int_148); if (unlikely(!__pyx_tuple__199)) __PYX_ERR(2, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__199);
  __Pyx_GIVEREF(__pyx_tuple__199);

  /* "include/color_dict.pxi":213
 *     'gray57' : (145, 145, 145),
 *     'gray58' : (148, 148, 148),
 *     'gray59' : (150, 150, 150),             # <<<<<<<<<<<<<<
 *     'gray6' : (15, 15, 15),
 *     'gray60' : (153, 153, 153),
 */
  __pyx_tuple__200 = PyTuple_Pack(3, __pyx_int_150, __pyx_int_150, __pyx_int_150); if (unlikely(!__pyx_tuple__200)) __PYX_ERR(2, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__200);
  __Pyx_GIVEREF(__pyx_tuple__200);

  /* "include/color_dict.pxi":214
 *     'gray58' : (148, 148, 148),
 *     'gray59' : (150, 150, 150),
 *     'gray6' : (15, 15, 15),             # <<<<<<<<<<<<<<
 *     'gray60' : (153, 153, 153),
 *     'gray61' : (156, 156, 156),
 */
  __pyx_tuple__201 = PyTuple_Pack(3, __pyx_int_15, __pyx_int_15, __pyx_int_15); if (unlikely(!__pyx_tuple__201)) __PYX_ERR(2, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__201);
  __Pyx_GIVEREF(__pyx_tuple__201);

  /* "include/color_dict.pxi":215
 *     'gray59' : (150, 150, 150),
 *     'gray6' : (15, 15, 15),
 *     'gray60' : (153, 153, 153),             # <<<<<<<<<<<<<<
 *     'gray61' : (156, 156, 156),
 *     'gray62' : (158, 158, 158),
 */
  __pyx_tuple__202 = PyTuple_Pack(3, __pyx_int_153, __pyx_int_153, __pyx_int_153); if (unlikely(!__pyx_tuple__202)) __PYX_ERR(2, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__202);
  __Pyx_GIVEREF(__pyx_tuple__202);

  /* "include/color_dict.pxi":216
 *     'gray6' : (15, 15, 15),
 *     'gray60' : (153, 153, 153),
 *     'gray61' : (156, 156, 156),             # <<<<<<<<<<<<<<
 *     'gray62' : (158, 158, 158),
 *     'gray63' : (161, 161, 161),
 */
  __pyx_tuple__203 = PyTuple_Pack(3, __pyx_int_156, __pyx_int_156, __pyx_int_156); if (unlikely(!__pyx_tuple__203)) __PYX_ERR(2, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__203);
  __Pyx_GIVEREF(__pyx_tuple__203);

  /* "include/color_dict.pxi":217
 *     'gray60' : (153, 153, 153),
 *     'gray61' : (156, 156, 156),
 *     'gray62' : (158, 158, 158),             # <<<<<<<<<<<<<<
 *     'gray63' : (161, 161, 161),
 *     'gray64' : (163, 163, 163),
 */
  __pyx_tuple__204 = PyTuple_Pack(3, __pyx_int_158, __pyx_int_158, __pyx_int_158); if (unlikely(!__pyx_tuple__204)) __PYX_ERR(2, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__204);
  __Pyx_GIVEREF(__pyx_tuple__204);

  /* "include/color_dict.pxi":218
 *     'gray61' : (156, 156, 156),
 *     'gray62' : (158, 158, 158),
 *     'gray63' : (161, 161, 161),             # <<<<<<<<<<<<<<
 *     'gray64' : (163, 163, 163),
 *     'gray65' : (166, 166, 166),
 */
  __pyx_tuple__205 = PyTuple_Pack(3, __pyx_int_161, __pyx_int_161, __pyx_int_161); if (unlikely(!__pyx_tuple__205)) __PYX_ERR(2, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__205);
  __Pyx_GIVEREF(__pyx_tuple__205);

  /* "include/color_dict.pxi":219
 *     'gray62' : (158, 158, 158),
 *     'gray63' : (161, 161, 161),
 *     'gray64' : (163, 163, 163),             # <<<<<<<<<<<<<<
 *     'gray65' : (166, 166, 166),
 *     'gray66' : (168, 168, 168),
 */
  __pyx_tuple__206 = PyTuple_Pack(3, __pyx_int_163, __pyx_int_163, __pyx_int_163); if (unlikely(!__pyx_tuple__206)) __PYX_ERR(2, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__206);
  __Pyx_GIVEREF(__pyx_tuple__206);

  /* "include/color_dict.pxi":220
 *     'gray63' : (161, 161, 161),
 *     'gray64' : (163, 163, 163),
 *     'gray65' : (166, 166, 166),             # <<<<<<<<<<<<<<
 *     'gray66' : (168, 168, 168),
 *     'gray67' : (171, 171, 171),
 */
  __pyx_tuple__207 = PyTuple_Pack(3, __pyx_int_166, __pyx_int_166, __pyx_int_166); if (unlikely(!__pyx_tuple__207)) __PYX_ERR(2, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__207);
  __Pyx_GIVEREF(__pyx_tuple__207);

  /* "include/color_dict.pxi":221
 *     'gray64' : (163, 163, 163),
 *     'gray65' : (166, 166, 166),
 *     'gray66' : (168, 168, 168),             # <<<<<<<<<<<<<<
 *     'gray67' : (171, 171, 171),
 *     'gray68' : (173, 173, 173),
 */
  __pyx_tuple__208 = PyTuple_Pack(3, __pyx_int_168, __pyx_int_168, __pyx_int_168); if (unlikely(!__pyx_tuple__208)) __PYX_ERR(2, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__208);
  __Pyx_GIVEREF(__pyx_tuple__208);

  /* "include/color_dict.pxi":222
 *     'gray65' : (166, 166, 166),
 *     'gray66' : (168, 168, 168),
 *     'gray67' : (171, 171, 171),             # <<<<<<<<<<<<<<
 *     'gray68' : (173, 173, 173),
 *     'gray69' : (176, 176, 176),
 */
  __pyx_tuple__209 = PyTuple_Pack(3, __pyx_int_171, __pyx_int_171, __pyx_int_171); if (unlikely(!__pyx_tuple__209)) __PYX_ERR(2, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__209);
  __Pyx_GIVEREF(__pyx_tuple__209);

  /* "include/color_dict.pxi":223
 *     'gray66' : (168, 168, 168),
 *     'gray67' : (171, 171, 171),
 *     'gray68' : (173, 173, 173),             # <<<<<<<<<<<<<<
 *     'gray69' : (176, 176, 176),
 *     'gray7' : (18, 18, 18),
 */
  __pyx_tuple__210 = PyTuple_Pack(3, __pyx_int_173, __pyx_int_173, __pyx_int_173); if (unlikely(!__pyx_tuple__210)) __PYX_ERR(2, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__210);
  __Pyx_GIVEREF(__pyx_tuple__210);

  /* "include/color_dict.pxi":224
 *     'gray67' : (171, 171, 171),
 *     'gray68' : (173, 173, 173),
 *     'gray69' : (176, 176, 176),             # <<<<<<<<<<<<<<
 *     'gray7' : (18, 18, 18),
 *     'gray70' : (179, 179, 179),
 */
  __pyx_tuple__211 = PyTuple_Pack(3, __pyx_int_176, __pyx_int_176, __pyx_int_176); if (unlikely(!__pyx_tuple__211)) __PYX_ERR(2, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__211);
  __Pyx_GIVEREF(__pyx_tuple__211);

  /* "include/color_dict.pxi":225
 *     'gray68' : (173, 173, 173),
 *     'gray69' : (176, 176, 176),
 *     'gray7' : (18, 18, 18),             # <<<<<<<<<<<<<<
 *     'gray70' : (179, 179, 179),
 *     'gray71' : (181, 181, 181),
 */
  __pyx_tuple__212 = PyTuple_Pack(3, __pyx_int_18, __pyx_int_18, __pyx_int_18); if (unlikely(!__pyx_tuple__212)) __PYX_ERR(2, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__212);
  __Pyx_GIVEREF(__pyx_tuple__212);

  /* "include/color_dict.pxi":226
 *     'gray69' : (176, 176, 176),
 *     'gray7' : (18, 18, 18),
 *     'gray70' : (179, 179, 179),             # <<<<<<<<<<<<<<
 *     'gray71' : (181, 181, 181),
 *     'gray72' : (184, 184, 184),
 */
  __pyx_tuple__213 = PyTuple_Pack(3, __pyx_int_179, __pyx_int_179, __pyx_int_179); if (unlikely(!__pyx_tuple__213)) __PYX_ERR(2, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__213);
  __Pyx_GIVEREF(__pyx_tuple__213);

  /* "include/color_dict.pxi":227
 *     'gray7' : (18, 18, 18),
 *     'gray70' : (179, 179, 179),
 *     'gray71' : (181, 181, 181),             # <<<<<<<<<<<<<<
 *     'gray72' : (184, 184, 184),
 *     'gray73' : (186, 186, 186),
 */
  __pyx_tuple__214 = PyTuple_Pack(3, __pyx_int_181, __pyx_int_181, __pyx_int_181); if (unlikely(!__pyx_tuple__214)) __PYX_ERR(2, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__214);
  __Pyx_GIVEREF(__pyx_tuple__214);

  /* "include/color_dict.pxi":228
 *     'gray70' : (179, 179, 179),
 *     'gray71' : (181, 181, 181),
 *     'gray72' : (184, 184, 184),             # <<<<<<<<<<<<<<
 *     'gray73' : (186, 186, 186),
 *     'gray74' : (189, 189, 189),
 */
  __pyx_tuple__215 = PyTuple_Pack(3, __pyx_int_184, __pyx_int_184, __pyx_int_184); if (unlikely(!__pyx_tuple__215)) __PYX_ERR(2, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__215);
  __Pyx_GIVEREF(__pyx_tuple__215);

  /* "include/color_dict.pxi":229
 *     'gray71' : (181, 181, 181),
 *     'gray72' : (184, 184, 184),
 *     'gray73' : (186, 186, 186),             # <<<<<<<<<<<<<<
 *     'gray74' : (189, 189, 189),
 *     'gray75' : (191, 191, 191),
 */
  __pyx_tuple__216 = PyTuple_Pack(3, __pyx_int_186, __pyx_int_186, __pyx_int_186); if (unlikely(!__pyx_tuple__216)) __PYX_ERR(2, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__216);
  __Pyx_GIVEREF(__pyx_tuple__216);

  /* "include/color_dict.pxi":230
 *     'gray72' : (184, 184, 184),
 *     'gray73' : (186, 186, 186),
 *     'gray74' : (189, 189, 189),             # <<<<<<<<<<<<<<
 *     'gray75' : (191, 191, 191),
 *     'gray76' : (194, 194, 194),
 */
  __pyx_tuple__217 = PyTuple_Pack(3, __pyx_int_189, __pyx_int_189, __pyx_int_189); if (unlikely(!__pyx_tuple__217)) __PYX_ERR(2, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__217);
  __Pyx_GIVEREF(__pyx_tuple__217);

  /* "include/color_dict.pxi":231
 *     'gray73' : (186, 186, 186),
 *     'gray74' : (189, 189, 189),
 *     'gray75' : (191, 191, 191),             # <<<<<<<<<<<<<<
 *     'gray76' : (194, 194, 194),
 *     'gray77' : (196, 196, 196),
 */
  __pyx_tuple__218 = PyTuple_Pack(3, __pyx_int_191, __pyx_int_191, __pyx_int_191); if (unlikely(!__pyx_tuple__218)) __PYX_ERR(2, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__218);
  __Pyx_GIVEREF(__pyx_tuple__218);

  /* "include/color_dict.pxi":232
 *     'gray74' : (189, 189, 189),
 *     'gray75' : (191, 191, 191),
 *     'gray76' : (194, 194, 194),             # <<<<<<<<<<<<<<
 *     'gray77' : (196, 196, 196),
 *     'gray78' : (199, 199, 199),
 */
  __pyx_tuple__219 = PyTuple_Pack(3, __pyx_int_194, __pyx_int_194, __pyx_int_194); if (unlikely(!__pyx_tuple__219)) __PYX_ERR(2, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__219);
  __Pyx_GIVEREF(__pyx_tuple__219);

  /* "include/color_dict.pxi":233
 *     'gray75' : (191, 191, 191),
 *     'gray76' : (194, 194, 194),
 *     'gray77' : (196, 196, 196),             # <<<<<<<<<<<<<<
 *     'gray78' : (199, 199, 199),
 *     'gray79' : (201, 201, 201),
 */
  __pyx_tuple__220 = PyTuple_Pack(3, __pyx_int_196, __pyx_int_196, __pyx_int_196); if (unlikely(!__pyx_tuple__220)) __PYX_ERR(2, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__220);
  __Pyx_GIVEREF(__pyx_tuple__220);

  /* "include/color_dict.pxi":234
 *     'gray76' : (194, 194, 194),
 *     'gray77' : (196, 196, 196),
 *     'gray78' : (199, 199, 199),             # <<<<<<<<<<<<<<
 *     'gray79' : (201, 201, 201),
 *     'gray8' : (20, 20, 20),
 */
  __pyx_tuple__221 = PyTuple_Pack(3, __pyx_int_199, __pyx_int_199, __pyx_int_199); if (unlikely(!__pyx_tuple__221)) __PYX_ERR(2, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__221);
  __Pyx_GIVEREF(__pyx_tuple__221);

  /* "include/color_dict.pxi":235
 *     'gray77' : (196, 196, 196),
 *     'gray78' : (199, 199, 199),
 *     'gray79' : (201, 201, 201),             # <<<<<<<<<<<<<<
 *     'gray8' : (20, 20, 20),
 *     'gray80' : (204, 204, 204),
 */
  __pyx_tuple__222 = PyTuple_Pack(3, __pyx_int_201, __pyx_int_201, __pyx_int_201); if (unlikely(!__pyx_tuple__222)) __PYX_ERR(2, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__222);
  __Pyx_GIVEREF(__pyx_tuple__222);

  /* "include/color_dict.pxi":236
 *     'gray78' : (199, 199, 199),
 *     'gray79' : (201, 201, 201),
 *     'gray8' : (20, 20, 20),             # <<<<<<<<<<<<<<
 *     'gray80' : (204, 204, 204),
 *     'gray81' : (207, 207, 207),
 */
  __pyx_tuple__223 = PyTuple_Pack(3, __pyx_int_20, __pyx_int_20, __pyx_int_20); if (unlikely(!__pyx_tuple__223)) __PYX_ERR(2, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__223);
  __Pyx_GIVEREF(__pyx_tuple__223);

  /* "include/color_dict.pxi":237
 *     'gray79' : (201, 201, 201),
 *     'gray8' : (20, 20, 20),
 *     'gray80' : (204, 204, 204),             # <<<<<<<<<<<<<<
 *     'gray81' : (207, 207, 207),
 *     'gray82' : (209, 209, 209),
 */
  __pyx_tuple__224 = PyTuple_Pack(3, __pyx_int_204, __pyx_int_204, __pyx_int_204); if (unlikely(!__pyx_tuple__224)) __PYX_ERR(2, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__224);
  __Pyx_GIVEREF(__pyx_tuple__224);

  /* "include/color_dict.pxi":238
 *     'gray8' : (20, 20, 20),
 *     'gray80' : (204, 204, 204),
 *     'gray81' : (207, 207, 207),             # <<<<<<<<<<<<<<
 *     'gray82' : (209, 209, 209),
 *     'gray83' : (212, 212, 212),
 */
  __pyx_tuple__225 = PyTuple_Pack(3, __pyx_int_207, __pyx_int_207, __pyx_int_207); if (unlikely(!__pyx_tuple__225)) __PYX_ERR(2, 238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__225);
  __Pyx_GIVEREF(__pyx_tuple__225);

  /* "include/color_dict.pxi":239
 *     'gray80' : (204, 204, 204),
 *     'gray81' : (207, 207, 207),
 *     'gray82' : (209, 209, 209),             # <<<<<<<<<<<<<<
 *     'gray83' : (212, 212, 212),
 *     'gray84' : (214, 214, 214),
 */
  __pyx_tuple__226 = PyTuple_Pack(3, __pyx_int_209, __pyx_int_209, __pyx_int_209); if (unlikely(!__pyx_tuple__226)) __PYX_ERR(2, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__226);
  __Pyx_GIVEREF(__pyx_tuple__226);

  /* "include/color_dict.pxi":240
 *     'gray81' : (207, 207, 207),
 *     'gray82' : (209, 209, 209),
 *     'gray83' : (212, 212, 212),             # <<<<<<<<<<<<<<
 *     'gray84' : (214, 214, 214),
 *     'gray85' : (217, 217, 217),
 */
  __pyx_tuple__227 = PyTuple_Pack(3, __pyx_int_212, __pyx_int_212, __pyx_int_212); if (unlikely(!__pyx_tuple__227)) __PYX_ERR(2, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__227);
  __Pyx_GIVEREF(__pyx_tuple__227);

  /* "include/color_dict.pxi":241
 *     'gray82' : (209, 209, 209),
 *     'gray83' : (212, 212, 212),
 *     'gray84' : (214, 214, 214),             # <<<<<<<<<<<<<<
 *     'gray85' : (217, 217, 217),
 *     'gray86' : (219, 219, 219),
 */
  __pyx_tuple__228 = PyTuple_Pack(3, __pyx_int_214, __pyx_int_214, __pyx_int_214); if (unlikely(!__pyx_tuple__228)) __PYX_ERR(2, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__228);
  __Pyx_GIVEREF(__pyx_tuple__228);

  /* "include/color_dict.pxi":242
 *     'gray83' : (212, 212, 212),
 *     'gray84' : (214, 214, 214),
 *     'gray85' : (217, 217, 217),             # <<<<<<<<<<<<<<
 *     'gray86' : (219, 219, 219),
 *     'gray87' : (222, 222, 222),
 */
  __pyx_tuple__229 = PyTuple_Pack(3, __pyx_int_217, __pyx_int_217, __pyx_int_217); if (unlikely(!__pyx_tuple__229)) __PYX_ERR(2, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__229);
  __Pyx_GIVEREF(__pyx_tuple__229);

  /* "include/color_dict.pxi":243
 *     'gray84' : (214, 214, 214),
 *     'gray85' : (217, 217, 217),
 *     'gray86' : (219, 219, 219),             # <<<<<<<<<<<<<<
 *     'gray87' : (222, 222, 222),
 *     'gray88' : (224, 224, 224),
 */
  __pyx_tuple__230 = PyTuple_Pack(3, __pyx_int_219, __pyx_int_219, __pyx_int_219); if (unlikely(!__pyx_tuple__230)) __PYX_ERR(2, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__230);
  __Pyx_GIVEREF(__pyx_tuple__230);

  /* "include/color_dict.pxi":244
 *     'gray85' : (217, 217, 217),
 *     'gray86' : (219, 219, 219),
 *     'gray87' : (222, 222, 222),             # <<<<<<<<<<<<<<
 *     'gray88' : (224, 224, 224),
 *     'gray89' : (227, 227, 227),
 */
  __pyx_tuple__231 = PyTuple_Pack(3, __pyx_int_222, __pyx_int_222, __pyx_int_222); if (unlikely(!__pyx_tuple__231)) __PYX_ERR(2, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__231);
  __Pyx_GIVEREF(__pyx_tuple__231);

  /* "include/color_dict.pxi":245
 *     'gray86' : (219, 219, 219),
 *     'gray87' : (222, 222, 222),
 *     'gray88' : (224, 224, 224),             # <<<<<<<<<<<<<<
 *     'gray89' : (227, 227, 227),
 *     'gray9' : (23, 23, 23),
 */
  __pyx_tuple__232 = PyTuple_Pack(3, __pyx_int_224, __pyx_int_224, __pyx_int_224); if (unlikely(!__pyx_tuple__232)) __PYX_ERR(2, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__232);
  __Pyx_GIVEREF(__pyx_tuple__232);

  /* "include/color_dict.pxi":246
 *     'gray87' : (222, 222, 222),
 *     'gray88' : (224, 224, 224),
 *     'gray89' : (227, 227, 227),             # <<<<<<<<<<<<<<
 *     'gray9' : (23, 23, 23),
 *     'gray90' : (229, 229, 229),
 */
  __pyx_tuple__233 = PyTuple_Pack(3, __pyx_int_227, __pyx_int_227, __pyx_int_227); if (unlikely(!__pyx_tuple__233)) __PYX_ERR(2, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__233);
  __Pyx_GIVEREF(__pyx_tuple__233);

  /* "include/color_dict.pxi":247
 *     'gray88' : (224, 224, 224),
 *     'gray89' : (227, 227, 227),
 *     'gray9' : (23, 23, 23),             # <<<<<<<<<<<<<<
 *     'gray90' : (229, 229, 229),
 *     'gray91' : (232, 232, 232),
 */
  __pyx_tuple__234 = PyTuple_Pack(3, __pyx_int_23, __pyx_int_23, __pyx_int_23); if (unlikely(!__pyx_tuple__234)) __PYX_ERR(2, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__234);
  __Pyx_GIVEREF(__pyx_tuple__234);

  /* "include/color_dict.pxi":248
 *     'gray89' : (227, 227, 227),
 *     'gray9' : (23, 23, 23),
 *     'gray90' : (229, 229, 229),             # <<<<<<<<<<<<<<
 *     'gray91' : (232, 232, 232),
 *     'gray92' : (235, 235, 235),
 */
  __pyx_tuple__235 = PyTuple_Pack(3, __pyx_int_229, __pyx_int_229, __pyx_int_229); if (unlikely(!__pyx_tuple__235)) __PYX_ERR(2, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__235);
  __Pyx_GIVEREF(__pyx_tuple__235);

  /* "include/color_dict.pxi":249
 *     'gray9' : (23, 23, 23),
 *     'gray90' : (229, 229, 229),
 *     'gray91' : (232, 232, 232),             # <<<<<<<<<<<<<<
 *     'gray92' : (235, 235, 235),
 *     'gray93' : (237, 237, 237),
 */
  __pyx_tuple__236 = PyTuple_Pack(3, __pyx_int_232, __pyx_int_232, __pyx_int_232); if (unlikely(!__pyx_tuple__236)) __PYX_ERR(2, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__236);
  __Pyx_GIVEREF(__pyx_tuple__236);

  /* "include/color_dict.pxi":250
 *     'gray90' : (229, 229, 229),
 *     'gray91' : (232, 232, 232),
 *     'gray92' : (235, 235, 235),             # <<<<<<<<<<<<<<
 *     'gray93' : (237, 237, 237),
 *     'gray94' : (240, 240, 240),
 */
  __pyx_tuple__237 = PyTuple_Pack(3, __pyx_int_235, __pyx_int_235, __pyx_int_235); if (unlikely(!__pyx_tuple__237)) __PYX_ERR(2, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__237);
  __Pyx_GIVEREF(__pyx_tuple__237);

  /* "include/color_dict.pxi":251
 *     'gray91' : (232, 232, 232),
 *     'gray92' : (235, 235, 235),
 *     'gray93' : (237, 237, 237),             # <<<<<<<<<<<<<<
 *     'gray94' : (240, 240, 240),
 *     'gray95' : (242, 242, 242),
 */
  __pyx_tuple__238 = PyTuple_Pack(3, __pyx_int_237, __pyx_int_237, __pyx_int_237); if (unlikely(!__pyx_tuple__238)) __PYX_ERR(2, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__238);
  __Pyx_GIVEREF(__pyx_tuple__238);

  /* "include/color_dict.pxi":252
 *     'gray92' : (235, 235, 235),
 *     'gray93' : (237, 237, 237),
 *     'gray94' : (240, 240, 240),             # <<<<<<<<<<<<<<
 *     'gray95' : (242, 242, 242),
 *     'gray96' : (245, 245, 245),
 */
  __pyx_tuple__239 = PyTuple_Pack(3, __pyx_int_240, __pyx_int_240, __pyx_int_240); if (unlikely(!__pyx_tuple__239)) __PYX_ERR(2, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__239);
  __Pyx_GIVEREF(__pyx_tuple__239);

  /* "include/color_dict.pxi":253
 *     'gray93' : (237, 237, 237),
 *     'gray94' : (240, 240, 240),
 *     'gray95' : (242, 242, 242),             # <<<<<<<<<<<<<<
 *     'gray96' : (245, 245, 245),
 *     'gray97' : (247, 247, 247),
 */
  __pyx_tuple__240 = PyTuple_Pack(3, __pyx_int_242, __pyx_int_242, __pyx_int_242); if (unlikely(!__pyx_tuple__240)) __PYX_ERR(2, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__240);
  __Pyx_GIVEREF(__pyx_tuple__240);

  /* "include/color_dict.pxi":254
 *     'gray94' : (240, 240, 240),
 *     'gray95' : (242, 242, 242),
 *     'gray96' : (245, 245, 245),             # <<<<<<<<<<<<<<
 *     'gray97' : (247, 247, 247),
 *     'gray98' : (250, 250, 250),
 */
  __pyx_tuple__241 = PyTuple_Pack(3, __pyx_int_245, __pyx_int_245, __pyx_int_245); if (unlikely(!__pyx_tuple__241)) __PYX_ERR(2, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__241);
  __Pyx_GIVEREF(__pyx_tuple__241);

  /* "include/color_dict.pxi":255
 *     'gray95' : (242, 242, 242),
 *     'gray96' : (245, 245, 245),
 *     'gray97' : (247, 247, 247),             # <<<<<<<<<<<<<<
 *     'gray98' : (250, 250, 250),
 *     'gray99' : (252, 252, 252),
 */
  __pyx_tuple__242 = PyTuple_Pack(3, __pyx_int_247, __pyx_int_247, __pyx_int_247); if (unlikely(!__pyx_tuple__242)) __PYX_ERR(2, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__242);
  __Pyx_GIVEREF(__pyx_tuple__242);

  /* "include/color_dict.pxi":256
 *     'gray96' : (245, 245, 245),
 *     'gray97' : (247, 247, 247),
 *     'gray98' : (250, 250, 250),             # <<<<<<<<<<<<<<
 *     'gray99' : (252, 252, 252),
 *     'green' : (0, 255, 0),
 */
  __pyx_tuple__243 = PyTuple_Pack(3, __pyx_int_250, __pyx_int_250, __pyx_int_250); if (unlikely(!__pyx_tuple__243)) __PYX_ERR(2, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__243);
  __Pyx_GIVEREF(__pyx_tuple__243);

  /* "include/color_dict.pxi":257
 *     'gray97' : (247, 247, 247),
 *     'gray98' : (250, 250, 250),
 *     'gray99' : (252, 252, 252),             # <<<<<<<<<<<<<<
 *     'green' : (0, 255, 0),
 *     'green1' : (0, 255, 0),
 */
  __pyx_tuple__244 = PyTuple_Pack(3, __pyx_int_252, __pyx_int_252, __pyx_int_252); if (unlikely(!__pyx_tuple__244)) __PYX_ERR(2, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__244);
  __Pyx_GIVEREF(__pyx_tuple__244);

  /* "include/color_dict.pxi":258
 *     'gray98' : (250, 250, 250),
 *     'gray99' : (252, 252, 252),
 *     'green' : (0, 255, 0),             # <<<<<<<<<<<<<<
 *     'green1' : (0, 255, 0),
 *     'green2' : (0, 238, 0),
 */
  __pyx_tuple__245 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_255, __pyx_int_0); if (unlikely(!__pyx_tuple__245)) __PYX_ERR(2, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__245);
  __Pyx_GIVEREF(__pyx_tuple__245);

  /* "include/color_dict.pxi":260
 *     'green' : (0, 255, 0),
 *     'green1' : (0, 255, 0),
 *     'green2' : (0, 238, 0),             # <<<<<<<<<<<<<<
 *     'green3' : (0, 205, 0),
 *     'green4' : (0, 139, 0),
 */
  __pyx_tuple__246 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_238, __pyx_int_0); if (unlikely(!__pyx_tuple__246)) __PYX_ERR(2, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__246);
  __Pyx_GIVEREF(__pyx_tuple__246);

  /* "include/color_dict.pxi":261
 *     'green1' : (0, 255, 0),
 *     'green2' : (0, 238, 0),
 *     'green3' : (0, 205, 0),             # <<<<<<<<<<<<<<
 *     'green4' : (0, 139, 0),
 *     'greenyellow' : (173, 255, 47),
 */
  __pyx_tuple__247 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_205, __pyx_int_0); if (unlikely(!__pyx_tuple__247)) __PYX_ERR(2, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__247);
  __Pyx_GIVEREF(__pyx_tuple__247);

  /* "include/color_dict.pxi":262
 *     'green2' : (0, 238, 0),
 *     'green3' : (0, 205, 0),
 *     'green4' : (0, 139, 0),             # <<<<<<<<<<<<<<
 *     'greenyellow' : (173, 255, 47),
 *     'grey' : (190, 190, 190),
 */
  __pyx_tuple__248 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_139, __pyx_int_0); if (unlikely(!__pyx_tuple__248)) __PYX_ERR(2, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__248);
  __Pyx_GIVEREF(__pyx_tuple__248);

  /* "include/color_dict.pxi":263
 *     'green3' : (0, 205, 0),
 *     'green4' : (0, 139, 0),
 *     'greenyellow' : (173, 255, 47),             # <<<<<<<<<<<<<<
 *     'grey' : (190, 190, 190),
 *     'grey0' : (0, 0, 0),
 */
  __pyx_tuple__249 = PyTuple_Pack(3, __pyx_int_173, __pyx_int_255, __pyx_int_47); if (unlikely(!__pyx_tuple__249)) __PYX_ERR(2, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__249);
  __Pyx_GIVEREF(__pyx_tuple__249);

  /* "include/color_dict.pxi":366
 *     'grey98' : (250, 250, 250),
 *     'grey99' : (252, 252, 252),
 *     'honeydew' : (240, 255, 240),             # <<<<<<<<<<<<<<
 *     'honeydew1' : (240, 255, 240),
 *     'honeydew2' : (224, 238, 224),
 */
  __pyx_tuple__250 = PyTuple_Pack(3, __pyx_int_240, __pyx_int_255, __pyx_int_240); if (unlikely(!__pyx_tuple__250)) __PYX_ERR(2, 366, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__250);
  __Pyx_GIVEREF(__pyx_tuple__250);

  /* "include/color_dict.pxi":368
 *     'honeydew' : (240, 255, 240),
 *     'honeydew1' : (240, 255, 240),
 *     'honeydew2' : (224, 238, 224),             # <<<<<<<<<<<<<<
 *     'honeydew3' : (193, 205, 193),
 *     'honeydew4' : (131, 139, 131),
 */
  __pyx_tuple__251 = PyTuple_Pack(3, __pyx_int_224, __pyx_int_238, __pyx_int_224); if (unlikely(!__pyx_tuple__251)) __PYX_ERR(2, 368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__251);
  __Pyx_GIVEREF(__pyx_tuple__251);

  /* "include/color_dict.pxi":369
 *     'honeydew1' : (240, 255, 240),
 *     'honeydew2' : (224, 238, 224),
 *     'honeydew3' : (193, 205, 193),             # <<<<<<<<<<<<<<
 *     'honeydew4' : (131, 139, 131),
 *     'hotpink' : (255, 105, 180),
 */
  __pyx_tuple__252 = PyTuple_Pack(3, __pyx_int_193, __pyx_int_205, __pyx_int_193); if (unlikely(!__pyx_tuple__252)) __PYX_ERR(2, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__252);
  __Pyx_GIVEREF(__pyx_tuple__252);

  /* "include/color_dict.pxi":370
 *     'honeydew2' : (224, 238, 224),
 *     'honeydew3' : (193, 205, 193),
 *     'honeydew4' : (131, 139, 131),             # <<<<<<<<<<<<<<
 *     'hotpink' : (255, 105, 180),
 *     'hotpink1' : (255, 110, 180),
 */
  __pyx_tuple__253 = PyTuple_Pack(3, __pyx_int_131, __pyx_int_139, __pyx_int_131); if (unlikely(!__pyx_tuple__253)) __PYX_ERR(2, 370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__253);
  __Pyx_GIVEREF(__pyx_tuple__253);

  /* "include/color_dict.pxi":371
 *     'honeydew3' : (193, 205, 193),
 *     'honeydew4' : (131, 139, 131),
 *     'hotpink' : (255, 105, 180),             # <<<<<<<<<<<<<<
 *     'hotpink1' : (255, 110, 180),
 *     'hotpink2' : (238, 106, 167),
 */
  __pyx_tuple__254 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_105, __pyx_int_180); if (unlikely(!__pyx_tuple__254)) __PYX_ERR(2, 371, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__254);
  __Pyx_GIVEREF(__pyx_tuple__254);

  /* "include/color_dict.pxi":372
 *     'honeydew4' : (131, 139, 131),
 *     'hotpink' : (255, 105, 180),
 *     'hotpink1' : (255, 110, 180),             # <<<<<<<<<<<<<<
 *     'hotpink2' : (238, 106, 167),
 *     'hotpink3' : (205, 96, 144),
 */
  __pyx_tuple__255 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_110, __pyx_int_180); if (unlikely(!__pyx_tuple__255)) __PYX_ERR(2, 372, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__255);
  __Pyx_GIVEREF(__pyx_tuple__255);

  /* "include/color_dict.pxi":373
 *     'hotpink' : (255, 105, 180),
 *     'hotpink1' : (255, 110, 180),
 *     'hotpink2' : (238, 106, 167),             # <<<<<<<<<<<<<<
 *     'hotpink3' : (205, 96, 144),
 *     'hotpink4' : (139, 58, 98),
 */
  __pyx_tuple__256 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_106, __pyx_int_167); if (unlikely(!__pyx_tuple__256)) __PYX_ERR(2, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__256);
  __Pyx_GIVEREF(__pyx_tuple__256);

  /* "include/color_dict.pxi":374
 *     'hotpink1' : (255, 110, 180),
 *     'hotpink2' : (238, 106, 167),
 *     'hotpink3' : (205, 96, 144),             # <<<<<<<<<<<<<<
 *     'hotpink4' : (139, 58, 98),
 *     'indianred' : (205, 92, 92),
 */
  __pyx_tuple__257 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_96, __pyx_int_144); if (unlikely(!__pyx_tuple__257)) __PYX_ERR(2, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__257);
  __Pyx_GIVEREF(__pyx_tuple__257);

  /* "include/color_dict.pxi":375
 *     'hotpink2' : (238, 106, 167),
 *     'hotpink3' : (205, 96, 144),
 *     'hotpink4' : (139, 58, 98),             # <<<<<<<<<<<<<<
 *     'indianred' : (205, 92, 92),
 *     'indianred1' : (255, 106, 106),
 */
  __pyx_tuple__258 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_58, __pyx_int_98); if (unlikely(!__pyx_tuple__258)) __PYX_ERR(2, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__258);
  __Pyx_GIVEREF(__pyx_tuple__258);

  /* "include/color_dict.pxi":376
 *     'hotpink3' : (205, 96, 144),
 *     'hotpink4' : (139, 58, 98),
 *     'indianred' : (205, 92, 92),             # <<<<<<<<<<<<<<
 *     'indianred1' : (255, 106, 106),
 *     'indianred2' : (238, 99, 99),
 */
  __pyx_tuple__259 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_92, __pyx_int_92); if (unlikely(!__pyx_tuple__259)) __PYX_ERR(2, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__259);
  __Pyx_GIVEREF(__pyx_tuple__259);

  /* "include/color_dict.pxi":377
 *     'hotpink4' : (139, 58, 98),
 *     'indianred' : (205, 92, 92),
 *     'indianred1' : (255, 106, 106),             # <<<<<<<<<<<<<<
 *     'indianred2' : (238, 99, 99),
 *     'indianred3' : (205, 85, 85),
 */
  __pyx_tuple__260 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_106, __pyx_int_106); if (unlikely(!__pyx_tuple__260)) __PYX_ERR(2, 377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__260);
  __Pyx_GIVEREF(__pyx_tuple__260);

  /* "include/color_dict.pxi":378
 *     'indianred' : (205, 92, 92),
 *     'indianred1' : (255, 106, 106),
 *     'indianred2' : (238, 99, 99),             # <<<<<<<<<<<<<<
 *     'indianred3' : (205, 85, 85),
 *     'indianred4' : (139, 58, 58),
 */
  __pyx_tuple__261 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_99, __pyx_int_99); if (unlikely(!__pyx_tuple__261)) __PYX_ERR(2, 378, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__261);
  __Pyx_GIVEREF(__pyx_tuple__261);

  /* "include/color_dict.pxi":379
 *     'indianred1' : (255, 106, 106),
 *     'indianred2' : (238, 99, 99),
 *     'indianred3' : (205, 85, 85),             # <<<<<<<<<<<<<<
 *     'indianred4' : (139, 58, 58),
 *     'ivory' : (255, 255, 240),
 */
  __pyx_tuple__262 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_85, __pyx_int_85); if (unlikely(!__pyx_tuple__262)) __PYX_ERR(2, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__262);
  __Pyx_GIVEREF(__pyx_tuple__262);

  /* "include/color_dict.pxi":380
 *     'indianred2' : (238, 99, 99),
 *     'indianred3' : (205, 85, 85),
 *     'indianred4' : (139, 58, 58),             # <<<<<<<<<<<<<<
 *     'ivory' : (255, 255, 240),
 *     'ivory1' : (255, 255, 240),
 */
  __pyx_tuple__263 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_58, __pyx_int_58); if (unlikely(!__pyx_tuple__263)) __PYX_ERR(2, 380, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__263);
  __Pyx_GIVEREF(__pyx_tuple__263);

  /* "include/color_dict.pxi":381
 *     'indianred3' : (205, 85, 85),
 *     'indianred4' : (139, 58, 58),
 *     'ivory' : (255, 255, 240),             # <<<<<<<<<<<<<<
 *     'ivory1' : (255, 255, 240),
 *     'ivory2' : (238, 238, 224),
 */
  __pyx_tuple__264 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_255, __pyx_int_240); if (unlikely(!__pyx_tuple__264)) __PYX_ERR(2, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__264);
  __Pyx_GIVEREF(__pyx_tuple__264);

  /* "include/color_dict.pxi":383
 *     'ivory' : (255, 255, 240),
 *     'ivory1' : (255, 255, 240),
 *     'ivory2' : (238, 238, 224),             # <<<<<<<<<<<<<<
 *     'ivory3' : (205, 205, 193),
 *     'ivory4' : (139, 139, 131),
 */
  __pyx_tuple__265 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_238, __pyx_int_224); if (unlikely(!__pyx_tuple__265)) __PYX_ERR(2, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__265);
  __Pyx_GIVEREF(__pyx_tuple__265);

  /* "include/color_dict.pxi":384
 *     'ivory1' : (255, 255, 240),
 *     'ivory2' : (238, 238, 224),
 *     'ivory3' : (205, 205, 193),             # <<<<<<<<<<<<<<
 *     'ivory4' : (139, 139, 131),
 *     'khaki' : (240, 230, 140),
 */
  __pyx_tuple__266 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_205, __pyx_int_193); if (unlikely(!__pyx_tuple__266)) __PYX_ERR(2, 384, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__266);
  __Pyx_GIVEREF(__pyx_tuple__266);

  /* "include/color_dict.pxi":385
 *     'ivory2' : (238, 238, 224),
 *     'ivory3' : (205, 205, 193),
 *     'ivory4' : (139, 139, 131),             # <<<<<<<<<<<<<<
 *     'khaki' : (240, 230, 140),
 *     'khaki1' : (255, 246, 143),
 */
  __pyx_tuple__267 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_139, __pyx_int_131); if (unlikely(!__pyx_tuple__267)) __PYX_ERR(2, 385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__267);
  __Pyx_GIVEREF(__pyx_tuple__267);

  /* "include/color_dict.pxi":386
 *     'ivory3' : (205, 205, 193),
 *     'ivory4' : (139, 139, 131),
 *     'khaki' : (240, 230, 140),             # <<<<<<<<<<<<<<
 *     'khaki1' : (255, 246, 143),
 *     'khaki2' : (238, 230, 133),
 */
  __pyx_tuple__268 = PyTuple_Pack(3, __pyx_int_240, __pyx_int_230, __pyx_int_140); if (unlikely(!__pyx_tuple__268)) __PYX_ERR(2, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__268);
  __Pyx_GIVEREF(__pyx_tuple__268);

  /* "include/color_dict.pxi":387
 *     'ivory4' : (139, 139, 131),
 *     'khaki' : (240, 230, 140),
 *     'khaki1' : (255, 246, 143),             # <<<<<<<<<<<<<<
 *     'khaki2' : (238, 230, 133),
 *     'khaki3' : (205, 198, 115),
 */
  __pyx_tuple__269 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_246, __pyx_int_143); if (unlikely(!__pyx_tuple__269)) __PYX_ERR(2, 387, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__269);
  __Pyx_GIVEREF(__pyx_tuple__269);

  /* "include/color_dict.pxi":388
 *     'khaki' : (240, 230, 140),
 *     'khaki1' : (255, 246, 143),
 *     'khaki2' : (238, 230, 133),             # <<<<<<<<<<<<<<
 *     'khaki3' : (205, 198, 115),
 *     'khaki4' : (139, 134, 78),
 */
  __pyx_tuple__270 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_230, __pyx_int_133); if (unlikely(!__pyx_tuple__270)) __PYX_ERR(2, 388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__270);
  __Pyx_GIVEREF(__pyx_tuple__270);

  /* "include/color_dict.pxi":389
 *     'khaki1' : (255, 246, 143),
 *     'khaki2' : (238, 230, 133),
 *     'khaki3' : (205, 198, 115),             # <<<<<<<<<<<<<<
 *     'khaki4' : (139, 134, 78),
 *     'lavender' : (230, 230, 250),
 */
  __pyx_tuple__271 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_198, __pyx_int_115); if (unlikely(!__pyx_tuple__271)) __PYX_ERR(2, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__271);
  __Pyx_GIVEREF(__pyx_tuple__271);

  /* "include/color_dict.pxi":390
 *     'khaki2' : (238, 230, 133),
 *     'khaki3' : (205, 198, 115),
 *     'khaki4' : (139, 134, 78),             # <<<<<<<<<<<<<<
 *     'lavender' : (230, 230, 250),
 *     'lavenderblush' : (255, 240, 245),
 */
  __pyx_tuple__272 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_134, __pyx_int_78); if (unlikely(!__pyx_tuple__272)) __PYX_ERR(2, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__272);
  __Pyx_GIVEREF(__pyx_tuple__272);

  /* "include/color_dict.pxi":391
 *     'khaki3' : (205, 198, 115),
 *     'khaki4' : (139, 134, 78),
 *     'lavender' : (230, 230, 250),             # <<<<<<<<<<<<<<
 *     'lavenderblush' : (255, 240, 245),
 *     'lavenderblush1' : (255, 240, 245),
 */
  __pyx_tuple__273 = PyTuple_Pack(3, __pyx_int_230, __pyx_int_230, __pyx_int_250); if (unlikely(!__pyx_tuple__273)) __PYX_ERR(2, 391, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__273);
  __Pyx_GIVEREF(__pyx_tuple__273);

  /* "include/color_dict.pxi":392
 *     'khaki4' : (139, 134, 78),
 *     'lavender' : (230, 230, 250),
 *     'lavenderblush' : (255, 240, 245),             # <<<<<<<<<<<<<<
 *     'lavenderblush1' : (255, 240, 245),
 *     'lavenderblush2' : (238, 224, 229),
 */
  __pyx_tuple__274 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_240, __pyx_int_245); if (unlikely(!__pyx_tuple__274)) __PYX_ERR(2, 392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__274);
  __Pyx_GIVEREF(__pyx_tuple__274);

  /* "include/color_dict.pxi":394
 *     'lavenderblush' : (255, 240, 245),
 *     'lavenderblush1' : (255, 240, 245),
 *     'lavenderblush2' : (238, 224, 229),             # <<<<<<<<<<<<<<
 *     'lavenderblush3' : (205, 193, 197),
 *     'lavenderblush4' : (139, 131, 134),
 */
  __pyx_tuple__275 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_224, __pyx_int_229); if (unlikely(!__pyx_tuple__275)) __PYX_ERR(2, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__275);
  __Pyx_GIVEREF(__pyx_tuple__275);

  /* "include/color_dict.pxi":395
 *     'lavenderblush1' : (255, 240, 245),
 *     'lavenderblush2' : (238, 224, 229),
 *     'lavenderblush3' : (205, 193, 197),             # <<<<<<<<<<<<<<
 *     'lavenderblush4' : (139, 131, 134),
 *     'lawngreen' : (124, 252, 0),
 */
  __pyx_tuple__276 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_193, __pyx_int_197); if (unlikely(!__pyx_tuple__276)) __PYX_ERR(2, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__276);
  __Pyx_GIVEREF(__pyx_tuple__276);

  /* "include/color_dict.pxi":396
 *     'lavenderblush2' : (238, 224, 229),
 *     'lavenderblush3' : (205, 193, 197),
 *     'lavenderblush4' : (139, 131, 134),             # <<<<<<<<<<<<<<
 *     'lawngreen' : (124, 252, 0),
 *     'lemonchiffon' : (255, 250, 205),
 */
  __pyx_tuple__277 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_131, __pyx_int_134); if (unlikely(!__pyx_tuple__277)) __PYX_ERR(2, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__277);
  __Pyx_GIVEREF(__pyx_tuple__277);

  /* "include/color_dict.pxi":397
 *     'lavenderblush3' : (205, 193, 197),
 *     'lavenderblush4' : (139, 131, 134),
 *     'lawngreen' : (124, 252, 0),             # <<<<<<<<<<<<<<
 *     'lemonchiffon' : (255, 250, 205),
 *     'lemonchiffon1' : (255, 250, 205),
 */
  __pyx_tuple__278 = PyTuple_Pack(3, __pyx_int_124, __pyx_int_252, __pyx_int_0); if (unlikely(!__pyx_tuple__278)) __PYX_ERR(2, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__278);
  __Pyx_GIVEREF(__pyx_tuple__278);

  /* "include/color_dict.pxi":398
 *     'lavenderblush4' : (139, 131, 134),
 *     'lawngreen' : (124, 252, 0),
 *     'lemonchiffon' : (255, 250, 205),             # <<<<<<<<<<<<<<
 *     'lemonchiffon1' : (255, 250, 205),
 *     'lemonchiffon2' : (238, 233, 191),
 */
  __pyx_tuple__279 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_250, __pyx_int_205); if (unlikely(!__pyx_tuple__279)) __PYX_ERR(2, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__279);
  __Pyx_GIVEREF(__pyx_tuple__279);

  /* "include/color_dict.pxi":400
 *     'lemonchiffon' : (255, 250, 205),
 *     'lemonchiffon1' : (255, 250, 205),
 *     'lemonchiffon2' : (238, 233, 191),             # <<<<<<<<<<<<<<
 *     'lemonchiffon3' : (205, 201, 165),
 *     'lemonchiffon4' : (139, 137, 112),
 */
  __pyx_tuple__280 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_233, __pyx_int_191); if (unlikely(!__pyx_tuple__280)) __PYX_ERR(2, 400, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__280);
  __Pyx_GIVEREF(__pyx_tuple__280);

  /* "include/color_dict.pxi":401
 *     'lemonchiffon1' : (255, 250, 205),
 *     'lemonchiffon2' : (238, 233, 191),
 *     'lemonchiffon3' : (205, 201, 165),             # <<<<<<<<<<<<<<
 *     'lemonchiffon4' : (139, 137, 112),
 *     'lightblue' : (173, 216, 230),
 */
  __pyx_tuple__281 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_201, __pyx_int_165); if (unlikely(!__pyx_tuple__281)) __PYX_ERR(2, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__281);
  __Pyx_GIVEREF(__pyx_tuple__281);

  /* "include/color_dict.pxi":402
 *     'lemonchiffon2' : (238, 233, 191),
 *     'lemonchiffon3' : (205, 201, 165),
 *     'lemonchiffon4' : (139, 137, 112),             # <<<<<<<<<<<<<<
 *     'lightblue' : (173, 216, 230),
 *     'lightblue1' : (191, 239, 255),
 */
  __pyx_tuple__282 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_137, __pyx_int_112); if (unlikely(!__pyx_tuple__282)) __PYX_ERR(2, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__282);
  __Pyx_GIVEREF(__pyx_tuple__282);

  /* "include/color_dict.pxi":403
 *     'lemonchiffon3' : (205, 201, 165),
 *     'lemonchiffon4' : (139, 137, 112),
 *     'lightblue' : (173, 216, 230),             # <<<<<<<<<<<<<<
 *     'lightblue1' : (191, 239, 255),
 *     'lightblue2' : (178, 223, 238),
 */
  __pyx_tuple__283 = PyTuple_Pack(3, __pyx_int_173, __pyx_int_216, __pyx_int_230); if (unlikely(!__pyx_tuple__283)) __PYX_ERR(2, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__283);
  __Pyx_GIVEREF(__pyx_tuple__283);

  /* "include/color_dict.pxi":404
 *     'lemonchiffon4' : (139, 137, 112),
 *     'lightblue' : (173, 216, 230),
 *     'lightblue1' : (191, 239, 255),             # <<<<<<<<<<<<<<
 *     'lightblue2' : (178, 223, 238),
 *     'lightblue3' : (154, 192, 205),
 */
  __pyx_tuple__284 = PyTuple_Pack(3, __pyx_int_191, __pyx_int_239, __pyx_int_255); if (unlikely(!__pyx_tuple__284)) __PYX_ERR(2, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__284);
  __Pyx_GIVEREF(__pyx_tuple__284);

  /* "include/color_dict.pxi":405
 *     'lightblue' : (173, 216, 230),
 *     'lightblue1' : (191, 239, 255),
 *     'lightblue2' : (178, 223, 238),             # <<<<<<<<<<<<<<
 *     'lightblue3' : (154, 192, 205),
 *     'lightblue4' : (104, 131, 139),
 */
  __pyx_tuple__285 = PyTuple_Pack(3, __pyx_int_178, __pyx_int_223, __pyx_int_238); if (unlikely(!__pyx_tuple__285)) __PYX_ERR(2, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__285);
  __Pyx_GIVEREF(__pyx_tuple__285);

  /* "include/color_dict.pxi":406
 *     'lightblue1' : (191, 239, 255),
 *     'lightblue2' : (178, 223, 238),
 *     'lightblue3' : (154, 192, 205),             # <<<<<<<<<<<<<<
 *     'lightblue4' : (104, 131, 139),
 *     'lightcoral' : (240, 128, 128),
 */
  __pyx_tuple__286 = PyTuple_Pack(3, __pyx_int_154, __pyx_int_192, __pyx_int_205); if (unlikely(!__pyx_tuple__286)) __PYX_ERR(2, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__286);
  __Pyx_GIVEREF(__pyx_tuple__286);

  /* "include/color_dict.pxi":407
 *     'lightblue2' : (178, 223, 238),
 *     'lightblue3' : (154, 192, 205),
 *     'lightblue4' : (104, 131, 139),             # <<<<<<<<<<<<<<
 *     'lightcoral' : (240, 128, 128),
 *     'lightcyan' : (224, 255, 255),
 */
  __pyx_tuple__287 = PyTuple_Pack(3, __pyx_int_104, __pyx_int_131, __pyx_int_139); if (unlikely(!__pyx_tuple__287)) __PYX_ERR(2, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__287);
  __Pyx_GIVEREF(__pyx_tuple__287);

  /* "include/color_dict.pxi":408
 *     'lightblue3' : (154, 192, 205),
 *     'lightblue4' : (104, 131, 139),
 *     'lightcoral' : (240, 128, 128),             # <<<<<<<<<<<<<<
 *     'lightcyan' : (224, 255, 255),
 *     'lightcyan1' : (224, 255, 255),
 */
  __pyx_tuple__288 = PyTuple_Pack(3, __pyx_int_240, __pyx_int_128, __pyx_int_128); if (unlikely(!__pyx_tuple__288)) __PYX_ERR(2, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__288);
  __Pyx_GIVEREF(__pyx_tuple__288);

  /* "include/color_dict.pxi":409
 *     'lightblue4' : (104, 131, 139),
 *     'lightcoral' : (240, 128, 128),
 *     'lightcyan' : (224, 255, 255),             # <<<<<<<<<<<<<<
 *     'lightcyan1' : (224, 255, 255),
 *     'lightcyan2' : (209, 238, 238),
 */
  __pyx_tuple__289 = PyTuple_Pack(3, __pyx_int_224, __pyx_int_255, __pyx_int_255); if (unlikely(!__pyx_tuple__289)) __PYX_ERR(2, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__289);
  __Pyx_GIVEREF(__pyx_tuple__289);

  /* "include/color_dict.pxi":411
 *     'lightcyan' : (224, 255, 255),
 *     'lightcyan1' : (224, 255, 255),
 *     'lightcyan2' : (209, 238, 238),             # <<<<<<<<<<<<<<
 *     'lightcyan3' : (180, 205, 205),
 *     'lightcyan4' : (122, 139, 139),
 */
  __pyx_tuple__290 = PyTuple_Pack(3, __pyx_int_209, __pyx_int_238, __pyx_int_238); if (unlikely(!__pyx_tuple__290)) __PYX_ERR(2, 411, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__290);
  __Pyx_GIVEREF(__pyx_tuple__290);

  /* "include/color_dict.pxi":412
 *     'lightcyan1' : (224, 255, 255),
 *     'lightcyan2' : (209, 238, 238),
 *     'lightcyan3' : (180, 205, 205),             # <<<<<<<<<<<<<<
 *     'lightcyan4' : (122, 139, 139),
 *     'lightgoldenrod' : (238, 221, 130),
 */
  __pyx_tuple__291 = PyTuple_Pack(3, __pyx_int_180, __pyx_int_205, __pyx_int_205); if (unlikely(!__pyx_tuple__291)) __PYX_ERR(2, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__291);
  __Pyx_GIVEREF(__pyx_tuple__291);

  /* "include/color_dict.pxi":413
 *     'lightcyan2' : (209, 238, 238),
 *     'lightcyan3' : (180, 205, 205),
 *     'lightcyan4' : (122, 139, 139),             # <<<<<<<<<<<<<<
 *     'lightgoldenrod' : (238, 221, 130),
 *     'lightgoldenrod1' : (255, 236, 139),
 */
  __pyx_tuple__292 = PyTuple_Pack(3, __pyx_int_122, __pyx_int_139, __pyx_int_139); if (unlikely(!__pyx_tuple__292)) __PYX_ERR(2, 413, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__292);
  __Pyx_GIVEREF(__pyx_tuple__292);

  /* "include/color_dict.pxi":414
 *     'lightcyan3' : (180, 205, 205),
 *     'lightcyan4' : (122, 139, 139),
 *     'lightgoldenrod' : (238, 221, 130),             # <<<<<<<<<<<<<<
 *     'lightgoldenrod1' : (255, 236, 139),
 *     'lightgoldenrod2' : (238, 220, 130),
 */
  __pyx_tuple__293 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_221, __pyx_int_130); if (unlikely(!__pyx_tuple__293)) __PYX_ERR(2, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__293);
  __Pyx_GIVEREF(__pyx_tuple__293);

  /* "include/color_dict.pxi":415
 *     'lightcyan4' : (122, 139, 139),
 *     'lightgoldenrod' : (238, 221, 130),
 *     'lightgoldenrod1' : (255, 236, 139),             # <<<<<<<<<<<<<<
 *     'lightgoldenrod2' : (238, 220, 130),
 *     'lightgoldenrod3' : (205, 190, 112),
 */
  __pyx_tuple__294 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_236, __pyx_int_139); if (unlikely(!__pyx_tuple__294)) __PYX_ERR(2, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__294);
  __Pyx_GIVEREF(__pyx_tuple__294);

  /* "include/color_dict.pxi":416
 *     'lightgoldenrod' : (238, 221, 130),
 *     'lightgoldenrod1' : (255, 236, 139),
 *     'lightgoldenrod2' : (238, 220, 130),             # <<<<<<<<<<<<<<
 *     'lightgoldenrod3' : (205, 190, 112),
 *     'lightgoldenrod4' : (139, 129, 76),
 */
  __pyx_tuple__295 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_220, __pyx_int_130); if (unlikely(!__pyx_tuple__295)) __PYX_ERR(2, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__295);
  __Pyx_GIVEREF(__pyx_tuple__295);

  /* "include/color_dict.pxi":417
 *     'lightgoldenrod1' : (255, 236, 139),
 *     'lightgoldenrod2' : (238, 220, 130),
 *     'lightgoldenrod3' : (205, 190, 112),             # <<<<<<<<<<<<<<
 *     'lightgoldenrod4' : (139, 129, 76),
 *     'lightgoldenrodyellow' : (250, 250, 210),
 */
  __pyx_tuple__296 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_190, __pyx_int_112); if (unlikely(!__pyx_tuple__296)) __PYX_ERR(2, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__296);
  __Pyx_GIVEREF(__pyx_tuple__296);

  /* "include/color_dict.pxi":418
 *     'lightgoldenrod2' : (238, 220, 130),
 *     'lightgoldenrod3' : (205, 190, 112),
 *     'lightgoldenrod4' : (139, 129, 76),             # <<<<<<<<<<<<<<
 *     'lightgoldenrodyellow' : (250, 250, 210),
 *     'lightgray' : (211, 211, 211),
 */
  __pyx_tuple__297 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_129, __pyx_int_76); if (unlikely(!__pyx_tuple__297)) __PYX_ERR(2, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__297);
  __Pyx_GIVEREF(__pyx_tuple__297);

  /* "include/color_dict.pxi":419
 *     'lightgoldenrod3' : (205, 190, 112),
 *     'lightgoldenrod4' : (139, 129, 76),
 *     'lightgoldenrodyellow' : (250, 250, 210),             # <<<<<<<<<<<<<<
 *     'lightgray' : (211, 211, 211),
 *     'lightgreen' : (144, 238, 144),
 */
  __pyx_tuple__298 = PyTuple_Pack(3, __pyx_int_250, __pyx_int_250, __pyx_int_210); if (unlikely(!__pyx_tuple__298)) __PYX_ERR(2, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__298);
  __Pyx_GIVEREF(__pyx_tuple__298);

  /* "include/color_dict.pxi":420
 *     'lightgoldenrod4' : (139, 129, 76),
 *     'lightgoldenrodyellow' : (250, 250, 210),
 *     'lightgray' : (211, 211, 211),             # <<<<<<<<<<<<<<
 *     'lightgreen' : (144, 238, 144),
 *     'lightgrey' : (211, 211, 211),
 */
  __pyx_tuple__299 = PyTuple_Pack(3, __pyx_int_211, __pyx_int_211, __pyx_int_211); if (unlikely(!__pyx_tuple__299)) __PYX_ERR(2, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__299);
  __Pyx_GIVEREF(__pyx_tuple__299);

  /* "include/color_dict.pxi":421
 *     'lightgoldenrodyellow' : (250, 250, 210),
 *     'lightgray' : (211, 211, 211),
 *     'lightgreen' : (144, 238, 144),             # <<<<<<<<<<<<<<
 *     'lightgrey' : (211, 211, 211),
 *     'lightpink' : (255, 182, 193),
 */
  __pyx_tuple__300 = PyTuple_Pack(3, __pyx_int_144, __pyx_int_238, __pyx_int_144); if (unlikely(!__pyx_tuple__300)) __PYX_ERR(2, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__300);
  __Pyx_GIVEREF(__pyx_tuple__300);

  /* "include/color_dict.pxi":423
 *     'lightgreen' : (144, 238, 144),
 *     'lightgrey' : (211, 211, 211),
 *     'lightpink' : (255, 182, 193),             # <<<<<<<<<<<<<<
 *     'lightpink1' : (255, 174, 185),
 *     'lightpink2' : (238, 162, 173),
 */
  __pyx_tuple__301 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_182, __pyx_int_193); if (unlikely(!__pyx_tuple__301)) __PYX_ERR(2, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__301);
  __Pyx_GIVEREF(__pyx_tuple__301);

  /* "include/color_dict.pxi":424
 *     'lightgrey' : (211, 211, 211),
 *     'lightpink' : (255, 182, 193),
 *     'lightpink1' : (255, 174, 185),             # <<<<<<<<<<<<<<
 *     'lightpink2' : (238, 162, 173),
 *     'lightpink3' : (205, 140, 149),
 */
  __pyx_tuple__302 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_174, __pyx_int_185); if (unlikely(!__pyx_tuple__302)) __PYX_ERR(2, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__302);
  __Pyx_GIVEREF(__pyx_tuple__302);

  /* "include/color_dict.pxi":425
 *     'lightpink' : (255, 182, 193),
 *     'lightpink1' : (255, 174, 185),
 *     'lightpink2' : (238, 162, 173),             # <<<<<<<<<<<<<<
 *     'lightpink3' : (205, 140, 149),
 *     'lightpink4' : (139, 95, 101),
 */
  __pyx_tuple__303 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_162, __pyx_int_173); if (unlikely(!__pyx_tuple__303)) __PYX_ERR(2, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__303);
  __Pyx_GIVEREF(__pyx_tuple__303);

  /* "include/color_dict.pxi":426
 *     'lightpink1' : (255, 174, 185),
 *     'lightpink2' : (238, 162, 173),
 *     'lightpink3' : (205, 140, 149),             # <<<<<<<<<<<<<<
 *     'lightpink4' : (139, 95, 101),
 *     'lightsalmon' : (255, 160, 122),
 */
  __pyx_tuple__304 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_140, __pyx_int_149); if (unlikely(!__pyx_tuple__304)) __PYX_ERR(2, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__304);
  __Pyx_GIVEREF(__pyx_tuple__304);

  /* "include/color_dict.pxi":427
 *     'lightpink2' : (238, 162, 173),
 *     'lightpink3' : (205, 140, 149),
 *     'lightpink4' : (139, 95, 101),             # <<<<<<<<<<<<<<
 *     'lightsalmon' : (255, 160, 122),
 *     'lightsalmon1' : (255, 160, 122),
 */
  __pyx_tuple__305 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_95, __pyx_int_101); if (unlikely(!__pyx_tuple__305)) __PYX_ERR(2, 427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__305);
  __Pyx_GIVEREF(__pyx_tuple__305);

  /* "include/color_dict.pxi":428
 *     'lightpink3' : (205, 140, 149),
 *     'lightpink4' : (139, 95, 101),
 *     'lightsalmon' : (255, 160, 122),             # <<<<<<<<<<<<<<
 *     'lightsalmon1' : (255, 160, 122),
 *     'lightsalmon2' : (238, 149, 114),
 */
  __pyx_tuple__306 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_160, __pyx_int_122); if (unlikely(!__pyx_tuple__306)) __PYX_ERR(2, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__306);
  __Pyx_GIVEREF(__pyx_tuple__306);

  /* "include/color_dict.pxi":430
 *     'lightsalmon' : (255, 160, 122),
 *     'lightsalmon1' : (255, 160, 122),
 *     'lightsalmon2' : (238, 149, 114),             # <<<<<<<<<<<<<<
 *     'lightsalmon3' : (205, 129, 98),
 *     'lightsalmon4' : (139, 87, 66),
 */
  __pyx_tuple__307 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_149, __pyx_int_114); if (unlikely(!__pyx_tuple__307)) __PYX_ERR(2, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__307);
  __Pyx_GIVEREF(__pyx_tuple__307);

  /* "include/color_dict.pxi":431
 *     'lightsalmon1' : (255, 160, 122),
 *     'lightsalmon2' : (238, 149, 114),
 *     'lightsalmon3' : (205, 129, 98),             # <<<<<<<<<<<<<<
 *     'lightsalmon4' : (139, 87, 66),
 *     'lightseagreen' : (32, 178, 170),
 */
  __pyx_tuple__308 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_129, __pyx_int_98); if (unlikely(!__pyx_tuple__308)) __PYX_ERR(2, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__308);
  __Pyx_GIVEREF(__pyx_tuple__308);

  /* "include/color_dict.pxi":432
 *     'lightsalmon2' : (238, 149, 114),
 *     'lightsalmon3' : (205, 129, 98),
 *     'lightsalmon4' : (139, 87, 66),             # <<<<<<<<<<<<<<
 *     'lightseagreen' : (32, 178, 170),
 *     'lightskyblue' : (135, 206, 250),
 */
  __pyx_tuple__309 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_87, __pyx_int_66); if (unlikely(!__pyx_tuple__309)) __PYX_ERR(2, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__309);
  __Pyx_GIVEREF(__pyx_tuple__309);

  /* "include/color_dict.pxi":433
 *     'lightsalmon3' : (205, 129, 98),
 *     'lightsalmon4' : (139, 87, 66),
 *     'lightseagreen' : (32, 178, 170),             # <<<<<<<<<<<<<<
 *     'lightskyblue' : (135, 206, 250),
 *     'lightskyblue1' : (176, 226, 255),
 */
  __pyx_tuple__310 = PyTuple_Pack(3, __pyx_int_32, __pyx_int_178, __pyx_int_170); if (unlikely(!__pyx_tuple__310)) __PYX_ERR(2, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__310);
  __Pyx_GIVEREF(__pyx_tuple__310);

  /* "include/color_dict.pxi":434
 *     'lightsalmon4' : (139, 87, 66),
 *     'lightseagreen' : (32, 178, 170),
 *     'lightskyblue' : (135, 206, 250),             # <<<<<<<<<<<<<<
 *     'lightskyblue1' : (176, 226, 255),
 *     'lightskyblue2' : (164, 211, 238),
 */
  __pyx_tuple__311 = PyTuple_Pack(3, __pyx_int_135, __pyx_int_206, __pyx_int_250); if (unlikely(!__pyx_tuple__311)) __PYX_ERR(2, 434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__311);
  __Pyx_GIVEREF(__pyx_tuple__311);

  /* "include/color_dict.pxi":435
 *     'lightseagreen' : (32, 178, 170),
 *     'lightskyblue' : (135, 206, 250),
 *     'lightskyblue1' : (176, 226, 255),             # <<<<<<<<<<<<<<
 *     'lightskyblue2' : (164, 211, 238),
 *     'lightskyblue3' : (141, 182, 205),
 */
  __pyx_tuple__312 = PyTuple_Pack(3, __pyx_int_176, __pyx_int_226, __pyx_int_255); if (unlikely(!__pyx_tuple__312)) __PYX_ERR(2, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__312);
  __Pyx_GIVEREF(__pyx_tuple__312);

  /* "include/color_dict.pxi":436
 *     'lightskyblue' : (135, 206, 250),
 *     'lightskyblue1' : (176, 226, 255),
 *     'lightskyblue2' : (164, 211, 238),             # <<<<<<<<<<<<<<
 *     'lightskyblue3' : (141, 182, 205),
 *     'lightskyblue4' : (96, 123, 139),
 */
  __pyx_tuple__313 = PyTuple_Pack(3, __pyx_int_164, __pyx_int_211, __pyx_int_238); if (unlikely(!__pyx_tuple__313)) __PYX_ERR(2, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__313);
  __Pyx_GIVEREF(__pyx_tuple__313);

  /* "include/color_dict.pxi":437
 *     'lightskyblue1' : (176, 226, 255),
 *     'lightskyblue2' : (164, 211, 238),
 *     'lightskyblue3' : (141, 182, 205),             # <<<<<<<<<<<<<<
 *     'lightskyblue4' : (96, 123, 139),
 *     'lightslateblue' : (132, 112, 255),
 */
  __pyx_tuple__314 = PyTuple_Pack(3, __pyx_int_141, __pyx_int_182, __pyx_int_205); if (unlikely(!__pyx_tuple__314)) __PYX_ERR(2, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__314);
  __Pyx_GIVEREF(__pyx_tuple__314);

  /* "include/color_dict.pxi":438
 *     'lightskyblue2' : (164, 211, 238),
 *     'lightskyblue3' : (141, 182, 205),
 *     'lightskyblue4' : (96, 123, 139),             # <<<<<<<<<<<<<<
 *     'lightslateblue' : (132, 112, 255),
 *     'lightslategray' : (119, 136, 153),
 */
  __pyx_tuple__315 = PyTuple_Pack(3, __pyx_int_96, __pyx_int_123, __pyx_int_139); if (unlikely(!__pyx_tuple__315)) __PYX_ERR(2, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__315);
  __Pyx_GIVEREF(__pyx_tuple__315);

  /* "include/color_dict.pxi":439
 *     'lightskyblue3' : (141, 182, 205),
 *     'lightskyblue4' : (96, 123, 139),
 *     'lightslateblue' : (132, 112, 255),             # <<<<<<<<<<<<<<
 *     'lightslategray' : (119, 136, 153),
 *     'lightslategrey' : (119, 136, 153),
 */
  __pyx_tuple__316 = PyTuple_Pack(3, __pyx_int_132, __pyx_int_112, __pyx_int_255); if (unlikely(!__pyx_tuple__316)) __PYX_ERR(2, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__316);
  __Pyx_GIVEREF(__pyx_tuple__316);

  /* "include/color_dict.pxi":440
 *     'lightskyblue4' : (96, 123, 139),
 *     'lightslateblue' : (132, 112, 255),
 *     'lightslategray' : (119, 136, 153),             # <<<<<<<<<<<<<<
 *     'lightslategrey' : (119, 136, 153),
 *     'lightsteelblue' : (176, 196, 222),
 */
  __pyx_tuple__317 = PyTuple_Pack(3, __pyx_int_119, __pyx_int_136, __pyx_int_153); if (unlikely(!__pyx_tuple__317)) __PYX_ERR(2, 440, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__317);
  __Pyx_GIVEREF(__pyx_tuple__317);

  /* "include/color_dict.pxi":442
 *     'lightslategray' : (119, 136, 153),
 *     'lightslategrey' : (119, 136, 153),
 *     'lightsteelblue' : (176, 196, 222),             # <<<<<<<<<<<<<<
 *     'lightsteelblue1' : (202, 225, 255),
 *     'lightsteelblue2' : (188, 210, 238),
 */
  __pyx_tuple__318 = PyTuple_Pack(3, __pyx_int_176, __pyx_int_196, __pyx_int_222); if (unlikely(!__pyx_tuple__318)) __PYX_ERR(2, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__318);
  __Pyx_GIVEREF(__pyx_tuple__318);

  /* "include/color_dict.pxi":443
 *     'lightslategrey' : (119, 136, 153),
 *     'lightsteelblue' : (176, 196, 222),
 *     'lightsteelblue1' : (202, 225, 255),             # <<<<<<<<<<<<<<
 *     'lightsteelblue2' : (188, 210, 238),
 *     'lightsteelblue3' : (162, 181, 205),
 */
  __pyx_tuple__319 = PyTuple_Pack(3, __pyx_int_202, __pyx_int_225, __pyx_int_255); if (unlikely(!__pyx_tuple__319)) __PYX_ERR(2, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__319);
  __Pyx_GIVEREF(__pyx_tuple__319);

  /* "include/color_dict.pxi":444
 *     'lightsteelblue' : (176, 196, 222),
 *     'lightsteelblue1' : (202, 225, 255),
 *     'lightsteelblue2' : (188, 210, 238),             # <<<<<<<<<<<<<<
 *     'lightsteelblue3' : (162, 181, 205),
 *     'lightsteelblue4' : (110, 123, 139),
 */
  __pyx_tuple__320 = PyTuple_Pack(3, __pyx_int_188, __pyx_int_210, __pyx_int_238); if (unlikely(!__pyx_tuple__320)) __PYX_ERR(2, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__320);
  __Pyx_GIVEREF(__pyx_tuple__320);

  /* "include/color_dict.pxi":445
 *     'lightsteelblue1' : (202, 225, 255),
 *     'lightsteelblue2' : (188, 210, 238),
 *     'lightsteelblue3' : (162, 181, 205),             # <<<<<<<<<<<<<<
 *     'lightsteelblue4' : (110, 123, 139),
 *     'lightyellow' : (255, 255, 224),
 */
  __pyx_tuple__321 = PyTuple_Pack(3, __pyx_int_162, __pyx_int_181, __pyx_int_205); if (unlikely(!__pyx_tuple__321)) __PYX_ERR(2, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__321);
  __Pyx_GIVEREF(__pyx_tuple__321);

  /* "include/color_dict.pxi":446
 *     'lightsteelblue2' : (188, 210, 238),
 *     'lightsteelblue3' : (162, 181, 205),
 *     'lightsteelblue4' : (110, 123, 139),             # <<<<<<<<<<<<<<
 *     'lightyellow' : (255, 255, 224),
 *     'lightyellow1' : (255, 255, 224),
 */
  __pyx_tuple__322 = PyTuple_Pack(3, __pyx_int_110, __pyx_int_123, __pyx_int_139); if (unlikely(!__pyx_tuple__322)) __PYX_ERR(2, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__322);
  __Pyx_GIVEREF(__pyx_tuple__322);

  /* "include/color_dict.pxi":447
 *     'lightsteelblue3' : (162, 181, 205),
 *     'lightsteelblue4' : (110, 123, 139),
 *     'lightyellow' : (255, 255, 224),             # <<<<<<<<<<<<<<
 *     'lightyellow1' : (255, 255, 224),
 *     'lightyellow2' : (238, 238, 209),
 */
  __pyx_tuple__323 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_255, __pyx_int_224); if (unlikely(!__pyx_tuple__323)) __PYX_ERR(2, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__323);
  __Pyx_GIVEREF(__pyx_tuple__323);

  /* "include/color_dict.pxi":449
 *     'lightyellow' : (255, 255, 224),
 *     'lightyellow1' : (255, 255, 224),
 *     'lightyellow2' : (238, 238, 209),             # <<<<<<<<<<<<<<
 *     'lightyellow3' : (205, 205, 180),
 *     'lightyellow4' : (139, 139, 122),
 */
  __pyx_tuple__324 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_238, __pyx_int_209); if (unlikely(!__pyx_tuple__324)) __PYX_ERR(2, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__324);
  __Pyx_GIVEREF(__pyx_tuple__324);

  /* "include/color_dict.pxi":450
 *     'lightyellow1' : (255, 255, 224),
 *     'lightyellow2' : (238, 238, 209),
 *     'lightyellow3' : (205, 205, 180),             # <<<<<<<<<<<<<<
 *     'lightyellow4' : (139, 139, 122),
 *     'limegreen' : (50, 205, 50),
 */
  __pyx_tuple__325 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_205, __pyx_int_180); if (unlikely(!__pyx_tuple__325)) __PYX_ERR(2, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__325);
  __Pyx_GIVEREF(__pyx_tuple__325);

  /* "include/color_dict.pxi":451
 *     'lightyellow2' : (238, 238, 209),
 *     'lightyellow3' : (205, 205, 180),
 *     'lightyellow4' : (139, 139, 122),             # <<<<<<<<<<<<<<
 *     'limegreen' : (50, 205, 50),
 *     'linen' : (250, 240, 230),
 */
  __pyx_tuple__326 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_139, __pyx_int_122); if (unlikely(!__pyx_tuple__326)) __PYX_ERR(2, 451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__326);
  __Pyx_GIVEREF(__pyx_tuple__326);

  /* "include/color_dict.pxi":452
 *     'lightyellow3' : (205, 205, 180),
 *     'lightyellow4' : (139, 139, 122),
 *     'limegreen' : (50, 205, 50),             # <<<<<<<<<<<<<<
 *     'linen' : (250, 240, 230),
 *     'magenta' : (255, 0, 255),
 */
  __pyx_tuple__327 = PyTuple_Pack(3, __pyx_int_50, __pyx_int_205, __pyx_int_50); if (unlikely(!__pyx_tuple__327)) __PYX_ERR(2, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__327);
  __Pyx_GIVEREF(__pyx_tuple__327);

  /* "include/color_dict.pxi":453
 *     'lightyellow4' : (139, 139, 122),
 *     'limegreen' : (50, 205, 50),
 *     'linen' : (250, 240, 230),             # <<<<<<<<<<<<<<
 *     'magenta' : (255, 0, 255),
 *     'magenta1' : (255, 0, 255),
 */
  __pyx_tuple__328 = PyTuple_Pack(3, __pyx_int_250, __pyx_int_240, __pyx_int_230); if (unlikely(!__pyx_tuple__328)) __PYX_ERR(2, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__328);
  __Pyx_GIVEREF(__pyx_tuple__328);

  /* "include/color_dict.pxi":454
 *     'limegreen' : (50, 205, 50),
 *     'linen' : (250, 240, 230),
 *     'magenta' : (255, 0, 255),             # <<<<<<<<<<<<<<
 *     'magenta1' : (255, 0, 255),
 *     'magenta2' : (238, 0, 238),
 */
  __pyx_tuple__329 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_0, __pyx_int_255); if (unlikely(!__pyx_tuple__329)) __PYX_ERR(2, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__329);
  __Pyx_GIVEREF(__pyx_tuple__329);

  /* "include/color_dict.pxi":456
 *     'magenta' : (255, 0, 255),
 *     'magenta1' : (255, 0, 255),
 *     'magenta2' : (238, 0, 238),             # <<<<<<<<<<<<<<
 *     'magenta3' : (205, 0, 205),
 *     'magenta4' : (139, 0, 139),
 */
  __pyx_tuple__330 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_0, __pyx_int_238); if (unlikely(!__pyx_tuple__330)) __PYX_ERR(2, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__330);
  __Pyx_GIVEREF(__pyx_tuple__330);

  /* "include/color_dict.pxi":457
 *     'magenta1' : (255, 0, 255),
 *     'magenta2' : (238, 0, 238),
 *     'magenta3' : (205, 0, 205),             # <<<<<<<<<<<<<<
 *     'magenta4' : (139, 0, 139),
 *     'maroon' : (176, 48, 96),
 */
  __pyx_tuple__331 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_0, __pyx_int_205); if (unlikely(!__pyx_tuple__331)) __PYX_ERR(2, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__331);
  __Pyx_GIVEREF(__pyx_tuple__331);

  /* "include/color_dict.pxi":459
 *     'magenta3' : (205, 0, 205),
 *     'magenta4' : (139, 0, 139),
 *     'maroon' : (176, 48, 96),             # <<<<<<<<<<<<<<
 *     'maroon1' : (255, 52, 179),
 *     'maroon2' : (238, 48, 167),
 */
  __pyx_tuple__332 = PyTuple_Pack(3, __pyx_int_176, __pyx_int_48, __pyx_int_96); if (unlikely(!__pyx_tuple__332)) __PYX_ERR(2, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__332);
  __Pyx_GIVEREF(__pyx_tuple__332);

  /* "include/color_dict.pxi":460
 *     'magenta4' : (139, 0, 139),
 *     'maroon' : (176, 48, 96),
 *     'maroon1' : (255, 52, 179),             # <<<<<<<<<<<<<<
 *     'maroon2' : (238, 48, 167),
 *     'maroon3' : (205, 41, 144),
 */
  __pyx_tuple__333 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_52, __pyx_int_179); if (unlikely(!__pyx_tuple__333)) __PYX_ERR(2, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__333);
  __Pyx_GIVEREF(__pyx_tuple__333);

  /* "include/color_dict.pxi":461
 *     'maroon' : (176, 48, 96),
 *     'maroon1' : (255, 52, 179),
 *     'maroon2' : (238, 48, 167),             # <<<<<<<<<<<<<<
 *     'maroon3' : (205, 41, 144),
 *     'maroon4' : (139, 28, 98),
 */
  __pyx_tuple__334 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_48, __pyx_int_167); if (unlikely(!__pyx_tuple__334)) __PYX_ERR(2, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__334);
  __Pyx_GIVEREF(__pyx_tuple__334);

  /* "include/color_dict.pxi":462
 *     'maroon1' : (255, 52, 179),
 *     'maroon2' : (238, 48, 167),
 *     'maroon3' : (205, 41, 144),             # <<<<<<<<<<<<<<
 *     'maroon4' : (139, 28, 98),
 *     'mediumaquamarine' : (102, 205, 170),
 */
  __pyx_tuple__335 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_41, __pyx_int_144); if (unlikely(!__pyx_tuple__335)) __PYX_ERR(2, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__335);
  __Pyx_GIVEREF(__pyx_tuple__335);

  /* "include/color_dict.pxi":463
 *     'maroon2' : (238, 48, 167),
 *     'maroon3' : (205, 41, 144),
 *     'maroon4' : (139, 28, 98),             # <<<<<<<<<<<<<<
 *     'mediumaquamarine' : (102, 205, 170),
 *     'mediumblue' : (0, 0, 205),
 */
  __pyx_tuple__336 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_28, __pyx_int_98); if (unlikely(!__pyx_tuple__336)) __PYX_ERR(2, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__336);
  __Pyx_GIVEREF(__pyx_tuple__336);

  /* "include/color_dict.pxi":466
 *     'mediumaquamarine' : (102, 205, 170),
 *     'mediumblue' : (0, 0, 205),
 *     'mediumorchid' : (186, 85, 211),             # <<<<<<<<<<<<<<
 *     'mediumorchid1' : (224, 102, 255),
 *     'mediumorchid2' : (209, 95, 238),
 */
  __pyx_tuple__337 = PyTuple_Pack(3, __pyx_int_186, __pyx_int_85, __pyx_int_211); if (unlikely(!__pyx_tuple__337)) __PYX_ERR(2, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__337);
  __Pyx_GIVEREF(__pyx_tuple__337);

  /* "include/color_dict.pxi":467
 *     'mediumblue' : (0, 0, 205),
 *     'mediumorchid' : (186, 85, 211),
 *     'mediumorchid1' : (224, 102, 255),             # <<<<<<<<<<<<<<
 *     'mediumorchid2' : (209, 95, 238),
 *     'mediumorchid3' : (180, 82, 205),
 */
  __pyx_tuple__338 = PyTuple_Pack(3, __pyx_int_224, __pyx_int_102, __pyx_int_255); if (unlikely(!__pyx_tuple__338)) __PYX_ERR(2, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__338);
  __Pyx_GIVEREF(__pyx_tuple__338);

  /* "include/color_dict.pxi":468
 *     'mediumorchid' : (186, 85, 211),
 *     'mediumorchid1' : (224, 102, 255),
 *     'mediumorchid2' : (209, 95, 238),             # <<<<<<<<<<<<<<
 *     'mediumorchid3' : (180, 82, 205),
 *     'mediumorchid4' : (122, 55, 139),
 */
  __pyx_tuple__339 = PyTuple_Pack(3, __pyx_int_209, __pyx_int_95, __pyx_int_238); if (unlikely(!__pyx_tuple__339)) __PYX_ERR(2, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__339);
  __Pyx_GIVEREF(__pyx_tuple__339);

  /* "include/color_dict.pxi":469
 *     'mediumorchid1' : (224, 102, 255),
 *     'mediumorchid2' : (209, 95, 238),
 *     'mediumorchid3' : (180, 82, 205),             # <<<<<<<<<<<<<<
 *     'mediumorchid4' : (122, 55, 139),
 *     'mediumpurple' : (147, 112, 219),
 */
  __pyx_tuple__340 = PyTuple_Pack(3, __pyx_int_180, __pyx_int_82, __pyx_int_205); if (unlikely(!__pyx_tuple__340)) __PYX_ERR(2, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__340);
  __Pyx_GIVEREF(__pyx_tuple__340);

  /* "include/color_dict.pxi":470
 *     'mediumorchid2' : (209, 95, 238),
 *     'mediumorchid3' : (180, 82, 205),
 *     'mediumorchid4' : (122, 55, 139),             # <<<<<<<<<<<<<<
 *     'mediumpurple' : (147, 112, 219),
 *     'mediumpurple1' : (171, 130, 255),
 */
  __pyx_tuple__341 = PyTuple_Pack(3, __pyx_int_122, __pyx_int_55, __pyx_int_139); if (unlikely(!__pyx_tuple__341)) __PYX_ERR(2, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__341);
  __Pyx_GIVEREF(__pyx_tuple__341);

  /* "include/color_dict.pxi":471
 *     'mediumorchid3' : (180, 82, 205),
 *     'mediumorchid4' : (122, 55, 139),
 *     'mediumpurple' : (147, 112, 219),             # <<<<<<<<<<<<<<
 *     'mediumpurple1' : (171, 130, 255),
 *     'mediumpurple2' : (159, 121, 238),
 */
  __pyx_tuple__342 = PyTuple_Pack(3, __pyx_int_147, __pyx_int_112, __pyx_int_219); if (unlikely(!__pyx_tuple__342)) __PYX_ERR(2, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__342);
  __Pyx_GIVEREF(__pyx_tuple__342);

  /* "include/color_dict.pxi":472
 *     'mediumorchid4' : (122, 55, 139),
 *     'mediumpurple' : (147, 112, 219),
 *     'mediumpurple1' : (171, 130, 255),             # <<<<<<<<<<<<<<
 *     'mediumpurple2' : (159, 121, 238),
 *     'mediumpurple3' : (137, 104, 205),
 */
  __pyx_tuple__343 = PyTuple_Pack(3, __pyx_int_171, __pyx_int_130, __pyx_int_255); if (unlikely(!__pyx_tuple__343)) __PYX_ERR(2, 472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__343);
  __Pyx_GIVEREF(__pyx_tuple__343);

  /* "include/color_dict.pxi":473
 *     'mediumpurple' : (147, 112, 219),
 *     'mediumpurple1' : (171, 130, 255),
 *     'mediumpurple2' : (159, 121, 238),             # <<<<<<<<<<<<<<
 *     'mediumpurple3' : (137, 104, 205),
 *     'mediumpurple4' : (93, 71, 139),
 */
  __pyx_tuple__344 = PyTuple_Pack(3, __pyx_int_159, __pyx_int_121, __pyx_int_238); if (unlikely(!__pyx_tuple__344)) __PYX_ERR(2, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__344);
  __Pyx_GIVEREF(__pyx_tuple__344);

  /* "include/color_dict.pxi":474
 *     'mediumpurple1' : (171, 130, 255),
 *     'mediumpurple2' : (159, 121, 238),
 *     'mediumpurple3' : (137, 104, 205),             # <<<<<<<<<<<<<<
 *     'mediumpurple4' : (93, 71, 139),
 *     'mediumseagreen' : (60, 179, 113),
 */
  __pyx_tuple__345 = PyTuple_Pack(3, __pyx_int_137, __pyx_int_104, __pyx_int_205); if (unlikely(!__pyx_tuple__345)) __PYX_ERR(2, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__345);
  __Pyx_GIVEREF(__pyx_tuple__345);

  /* "include/color_dict.pxi":475
 *     'mediumpurple2' : (159, 121, 238),
 *     'mediumpurple3' : (137, 104, 205),
 *     'mediumpurple4' : (93, 71, 139),             # <<<<<<<<<<<<<<
 *     'mediumseagreen' : (60, 179, 113),
 *     'mediumslateblue' : (123, 104, 238),
 */
  __pyx_tuple__346 = PyTuple_Pack(3, __pyx_int_93, __pyx_int_71, __pyx_int_139); if (unlikely(!__pyx_tuple__346)) __PYX_ERR(2, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__346);
  __Pyx_GIVEREF(__pyx_tuple__346);

  /* "include/color_dict.pxi":476
 *     'mediumpurple3' : (137, 104, 205),
 *     'mediumpurple4' : (93, 71, 139),
 *     'mediumseagreen' : (60, 179, 113),             # <<<<<<<<<<<<<<
 *     'mediumslateblue' : (123, 104, 238),
 *     'mediumspringgreen' : (0, 250, 154),
 */
  __pyx_tuple__347 = PyTuple_Pack(3, __pyx_int_60, __pyx_int_179, __pyx_int_113); if (unlikely(!__pyx_tuple__347)) __PYX_ERR(2, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__347);
  __Pyx_GIVEREF(__pyx_tuple__347);

  /* "include/color_dict.pxi":477
 *     'mediumpurple4' : (93, 71, 139),
 *     'mediumseagreen' : (60, 179, 113),
 *     'mediumslateblue' : (123, 104, 238),             # <<<<<<<<<<<<<<
 *     'mediumspringgreen' : (0, 250, 154),
 *     'mediumturquoise' : (72, 209, 204),
 */
  __pyx_tuple__348 = PyTuple_Pack(3, __pyx_int_123, __pyx_int_104, __pyx_int_238); if (unlikely(!__pyx_tuple__348)) __PYX_ERR(2, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__348);
  __Pyx_GIVEREF(__pyx_tuple__348);

  /* "include/color_dict.pxi":478
 *     'mediumseagreen' : (60, 179, 113),
 *     'mediumslateblue' : (123, 104, 238),
 *     'mediumspringgreen' : (0, 250, 154),             # <<<<<<<<<<<<<<
 *     'mediumturquoise' : (72, 209, 204),
 *     'mediumvioletred' : (199, 21, 133),
 */
  __pyx_tuple__349 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_250, __pyx_int_154); if (unlikely(!__pyx_tuple__349)) __PYX_ERR(2, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__349);
  __Pyx_GIVEREF(__pyx_tuple__349);

  /* "include/color_dict.pxi":479
 *     'mediumslateblue' : (123, 104, 238),
 *     'mediumspringgreen' : (0, 250, 154),
 *     'mediumturquoise' : (72, 209, 204),             # <<<<<<<<<<<<<<
 *     'mediumvioletred' : (199, 21, 133),
 *     'midnightblue' : (25, 25, 112),
 */
  __pyx_tuple__350 = PyTuple_Pack(3, __pyx_int_72, __pyx_int_209, __pyx_int_204); if (unlikely(!__pyx_tuple__350)) __PYX_ERR(2, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__350);
  __Pyx_GIVEREF(__pyx_tuple__350);

  /* "include/color_dict.pxi":480
 *     'mediumspringgreen' : (0, 250, 154),
 *     'mediumturquoise' : (72, 209, 204),
 *     'mediumvioletred' : (199, 21, 133),             # <<<<<<<<<<<<<<
 *     'midnightblue' : (25, 25, 112),
 *     'mintcream' : (245, 255, 250),
 */
  __pyx_tuple__351 = PyTuple_Pack(3, __pyx_int_199, __pyx_int_21, __pyx_int_133); if (unlikely(!__pyx_tuple__351)) __PYX_ERR(2, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__351);
  __Pyx_GIVEREF(__pyx_tuple__351);

  /* "include/color_dict.pxi":481
 *     'mediumturquoise' : (72, 209, 204),
 *     'mediumvioletred' : (199, 21, 133),
 *     'midnightblue' : (25, 25, 112),             # <<<<<<<<<<<<<<
 *     'mintcream' : (245, 255, 250),
 *     'mistyrose' : (255, 228, 225),
 */
  __pyx_tuple__352 = PyTuple_Pack(3, __pyx_int_25, __pyx_int_25, __pyx_int_112); if (unlikely(!__pyx_tuple__352)) __PYX_ERR(2, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__352);
  __Pyx_GIVEREF(__pyx_tuple__352);

  /* "include/color_dict.pxi":482
 *     'mediumvioletred' : (199, 21, 133),
 *     'midnightblue' : (25, 25, 112),
 *     'mintcream' : (245, 255, 250),             # <<<<<<<<<<<<<<
 *     'mistyrose' : (255, 228, 225),
 *     'mistyrose1' : (255, 228, 225),
 */
  __pyx_tuple__353 = PyTuple_Pack(3, __pyx_int_245, __pyx_int_255, __pyx_int_250); if (unlikely(!__pyx_tuple__353)) __PYX_ERR(2, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__353);
  __Pyx_GIVEREF(__pyx_tuple__353);

  /* "include/color_dict.pxi":483
 *     'midnightblue' : (25, 25, 112),
 *     'mintcream' : (245, 255, 250),
 *     'mistyrose' : (255, 228, 225),             # <<<<<<<<<<<<<<
 *     'mistyrose1' : (255, 228, 225),
 *     'mistyrose2' : (238, 213, 210),
 */
  __pyx_tuple__354 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_228, __pyx_int_225); if (unlikely(!__pyx_tuple__354)) __PYX_ERR(2, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__354);
  __Pyx_GIVEREF(__pyx_tuple__354);

  /* "include/color_dict.pxi":485
 *     'mistyrose' : (255, 228, 225),
 *     'mistyrose1' : (255, 228, 225),
 *     'mistyrose2' : (238, 213, 210),             # <<<<<<<<<<<<<<
 *     'mistyrose3' : (205, 183, 181),
 *     'mistyrose4' : (139, 125, 123),
 */
  __pyx_tuple__355 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_213, __pyx_int_210); if (unlikely(!__pyx_tuple__355)) __PYX_ERR(2, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__355);
  __Pyx_GIVEREF(__pyx_tuple__355);

  /* "include/color_dict.pxi":486
 *     'mistyrose1' : (255, 228, 225),
 *     'mistyrose2' : (238, 213, 210),
 *     'mistyrose3' : (205, 183, 181),             # <<<<<<<<<<<<<<
 *     'mistyrose4' : (139, 125, 123),
 *     'moccasin' : (255, 228, 181),
 */
  __pyx_tuple__356 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_183, __pyx_int_181); if (unlikely(!__pyx_tuple__356)) __PYX_ERR(2, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__356);
  __Pyx_GIVEREF(__pyx_tuple__356);

  /* "include/color_dict.pxi":487
 *     'mistyrose2' : (238, 213, 210),
 *     'mistyrose3' : (205, 183, 181),
 *     'mistyrose4' : (139, 125, 123),             # <<<<<<<<<<<<<<
 *     'moccasin' : (255, 228, 181),
 *     'navajowhite' : (255, 222, 173),
 */
  __pyx_tuple__357 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_125, __pyx_int_123); if (unlikely(!__pyx_tuple__357)) __PYX_ERR(2, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__357);
  __Pyx_GIVEREF(__pyx_tuple__357);

  /* "include/color_dict.pxi":488
 *     'mistyrose3' : (205, 183, 181),
 *     'mistyrose4' : (139, 125, 123),
 *     'moccasin' : (255, 228, 181),             # <<<<<<<<<<<<<<
 *     'navajowhite' : (255, 222, 173),
 *     'navajowhite1' : (255, 222, 173),
 */
  __pyx_tuple__358 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_228, __pyx_int_181); if (unlikely(!__pyx_tuple__358)) __PYX_ERR(2, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__358);
  __Pyx_GIVEREF(__pyx_tuple__358);

  /* "include/color_dict.pxi":489
 *     'mistyrose4' : (139, 125, 123),
 *     'moccasin' : (255, 228, 181),
 *     'navajowhite' : (255, 222, 173),             # <<<<<<<<<<<<<<
 *     'navajowhite1' : (255, 222, 173),
 *     'navajowhite2' : (238, 207, 161),
 */
  __pyx_tuple__359 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_222, __pyx_int_173); if (unlikely(!__pyx_tuple__359)) __PYX_ERR(2, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__359);
  __Pyx_GIVEREF(__pyx_tuple__359);

  /* "include/color_dict.pxi":491
 *     'navajowhite' : (255, 222, 173),
 *     'navajowhite1' : (255, 222, 173),
 *     'navajowhite2' : (238, 207, 161),             # <<<<<<<<<<<<<<
 *     'navajowhite3' : (205, 179, 139),
 *     'navajowhite4' : (139, 121, 94),
 */
  __pyx_tuple__360 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_207, __pyx_int_161); if (unlikely(!__pyx_tuple__360)) __PYX_ERR(2, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__360);
  __Pyx_GIVEREF(__pyx_tuple__360);

  /* "include/color_dict.pxi":492
 *     'navajowhite1' : (255, 222, 173),
 *     'navajowhite2' : (238, 207, 161),
 *     'navajowhite3' : (205, 179, 139),             # <<<<<<<<<<<<<<
 *     'navajowhite4' : (139, 121, 94),
 *     'navy' : (0, 0, 128),
 */
  __pyx_tuple__361 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_179, __pyx_int_139); if (unlikely(!__pyx_tuple__361)) __PYX_ERR(2, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__361);
  __Pyx_GIVEREF(__pyx_tuple__361);

  /* "include/color_dict.pxi":493
 *     'navajowhite2' : (238, 207, 161),
 *     'navajowhite3' : (205, 179, 139),
 *     'navajowhite4' : (139, 121, 94),             # <<<<<<<<<<<<<<
 *     'navy' : (0, 0, 128),
 *     'navyblue' : (0, 0, 128),
 */
  __pyx_tuple__362 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_121, __pyx_int_94); if (unlikely(!__pyx_tuple__362)) __PYX_ERR(2, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__362);
  __Pyx_GIVEREF(__pyx_tuple__362);

  /* "include/color_dict.pxi":494
 *     'navajowhite3' : (205, 179, 139),
 *     'navajowhite4' : (139, 121, 94),
 *     'navy' : (0, 0, 128),             # <<<<<<<<<<<<<<
 *     'navyblue' : (0, 0, 128),
 *     'oldlace' : (253, 245, 230),
 */
  __pyx_tuple__363 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_0, __pyx_int_128); if (unlikely(!__pyx_tuple__363)) __PYX_ERR(2, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__363);
  __Pyx_GIVEREF(__pyx_tuple__363);

  /* "include/color_dict.pxi":496
 *     'navy' : (0, 0, 128),
 *     'navyblue' : (0, 0, 128),
 *     'oldlace' : (253, 245, 230),             # <<<<<<<<<<<<<<
 *     'olivedrab' : (107, 142, 35),
 *     'olivedrab1' : (192, 255, 62),
 */
  __pyx_tuple__364 = PyTuple_Pack(3, __pyx_int_253, __pyx_int_245, __pyx_int_230); if (unlikely(!__pyx_tuple__364)) __PYX_ERR(2, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__364);
  __Pyx_GIVEREF(__pyx_tuple__364);

  /* "include/color_dict.pxi":497
 *     'navyblue' : (0, 0, 128),
 *     'oldlace' : (253, 245, 230),
 *     'olivedrab' : (107, 142, 35),             # <<<<<<<<<<<<<<
 *     'olivedrab1' : (192, 255, 62),
 *     'olivedrab2' : (179, 238, 58),
 */
  __pyx_tuple__365 = PyTuple_Pack(3, __pyx_int_107, __pyx_int_142, __pyx_int_35); if (unlikely(!__pyx_tuple__365)) __PYX_ERR(2, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__365);
  __Pyx_GIVEREF(__pyx_tuple__365);

  /* "include/color_dict.pxi":498
 *     'oldlace' : (253, 245, 230),
 *     'olivedrab' : (107, 142, 35),
 *     'olivedrab1' : (192, 255, 62),             # <<<<<<<<<<<<<<
 *     'olivedrab2' : (179, 238, 58),
 *     'olivedrab3' : (154, 205, 50),
 */
  __pyx_tuple__366 = PyTuple_Pack(3, __pyx_int_192, __pyx_int_255, __pyx_int_62); if (unlikely(!__pyx_tuple__366)) __PYX_ERR(2, 498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__366);
  __Pyx_GIVEREF(__pyx_tuple__366);

  /* "include/color_dict.pxi":499
 *     'olivedrab' : (107, 142, 35),
 *     'olivedrab1' : (192, 255, 62),
 *     'olivedrab2' : (179, 238, 58),             # <<<<<<<<<<<<<<
 *     'olivedrab3' : (154, 205, 50),
 *     'olivedrab4' : (105, 139, 34),
 */
  __pyx_tuple__367 = PyTuple_Pack(3, __pyx_int_179, __pyx_int_238, __pyx_int_58); if (unlikely(!__pyx_tuple__367)) __PYX_ERR(2, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__367);
  __Pyx_GIVEREF(__pyx_tuple__367);

  /* "include/color_dict.pxi":500
 *     'olivedrab1' : (192, 255, 62),
 *     'olivedrab2' : (179, 238, 58),
 *     'olivedrab3' : (154, 205, 50),             # <<<<<<<<<<<<<<
 *     'olivedrab4' : (105, 139, 34),
 *     'orange' : (255, 165, 0),
 */
  __pyx_tuple__368 = PyTuple_Pack(3, __pyx_int_154, __pyx_int_205, __pyx_int_50); if (unlikely(!__pyx_tuple__368)) __PYX_ERR(2, 500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__368);
  __Pyx_GIVEREF(__pyx_tuple__368);

  /* "include/color_dict.pxi":501
 *     'olivedrab2' : (179, 238, 58),
 *     'olivedrab3' : (154, 205, 50),
 *     'olivedrab4' : (105, 139, 34),             # <<<<<<<<<<<<<<
 *     'orange' : (255, 165, 0),
 *     'orange1' : (255, 165, 0),
 */
  __pyx_tuple__369 = PyTuple_Pack(3, __pyx_int_105, __pyx_int_139, __pyx_int_34); if (unlikely(!__pyx_tuple__369)) __PYX_ERR(2, 501, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__369);
  __Pyx_GIVEREF(__pyx_tuple__369);

  /* "include/color_dict.pxi":502
 *     'olivedrab3' : (154, 205, 50),
 *     'olivedrab4' : (105, 139, 34),
 *     'orange' : (255, 165, 0),             # <<<<<<<<<<<<<<
 *     'orange1' : (255, 165, 0),
 *     'orange2' : (238, 154, 0),
 */
  __pyx_tuple__370 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_165, __pyx_int_0); if (unlikely(!__pyx_tuple__370)) __PYX_ERR(2, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__370);
  __Pyx_GIVEREF(__pyx_tuple__370);

  /* "include/color_dict.pxi":504
 *     'orange' : (255, 165, 0),
 *     'orange1' : (255, 165, 0),
 *     'orange2' : (238, 154, 0),             # <<<<<<<<<<<<<<
 *     'orange3' : (205, 133, 0),
 *     'orange4' : (139, 90, 0),
 */
  __pyx_tuple__371 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_154, __pyx_int_0); if (unlikely(!__pyx_tuple__371)) __PYX_ERR(2, 504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__371);
  __Pyx_GIVEREF(__pyx_tuple__371);

  /* "include/color_dict.pxi":505
 *     'orange1' : (255, 165, 0),
 *     'orange2' : (238, 154, 0),
 *     'orange3' : (205, 133, 0),             # <<<<<<<<<<<<<<
 *     'orange4' : (139, 90, 0),
 *     'orangered' : (255, 69, 0),
 */
  __pyx_tuple__372 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_133, __pyx_int_0); if (unlikely(!__pyx_tuple__372)) __PYX_ERR(2, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__372);
  __Pyx_GIVEREF(__pyx_tuple__372);

  /* "include/color_dict.pxi":506
 *     'orange2' : (238, 154, 0),
 *     'orange3' : (205, 133, 0),
 *     'orange4' : (139, 90, 0),             # <<<<<<<<<<<<<<
 *     'orangered' : (255, 69, 0),
 *     'orangered1' : (255, 69, 0),
 */
  __pyx_tuple__373 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_90, __pyx_int_0); if (unlikely(!__pyx_tuple__373)) __PYX_ERR(2, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__373);
  __Pyx_GIVEREF(__pyx_tuple__373);

  /* "include/color_dict.pxi":507
 *     'orange3' : (205, 133, 0),
 *     'orange4' : (139, 90, 0),
 *     'orangered' : (255, 69, 0),             # <<<<<<<<<<<<<<
 *     'orangered1' : (255, 69, 0),
 *     'orangered2' : (238, 64, 0),
 */
  __pyx_tuple__374 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_69, __pyx_int_0); if (unlikely(!__pyx_tuple__374)) __PYX_ERR(2, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__374);
  __Pyx_GIVEREF(__pyx_tuple__374);

  /* "include/color_dict.pxi":509
 *     'orangered' : (255, 69, 0),
 *     'orangered1' : (255, 69, 0),
 *     'orangered2' : (238, 64, 0),             # <<<<<<<<<<<<<<
 *     'orangered3' : (205, 55, 0),
 *     'orangered4' : (139, 37, 0),
 */
  __pyx_tuple__375 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_64, __pyx_int_0); if (unlikely(!__pyx_tuple__375)) __PYX_ERR(2, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__375);
  __Pyx_GIVEREF(__pyx_tuple__375);

  /* "include/color_dict.pxi":510
 *     'orangered1' : (255, 69, 0),
 *     'orangered2' : (238, 64, 0),
 *     'orangered3' : (205, 55, 0),             # <<<<<<<<<<<<<<
 *     'orangered4' : (139, 37, 0),
 *     'orchid' : (218, 112, 214),
 */
  __pyx_tuple__376 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_55, __pyx_int_0); if (unlikely(!__pyx_tuple__376)) __PYX_ERR(2, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__376);
  __Pyx_GIVEREF(__pyx_tuple__376);

  /* "include/color_dict.pxi":511
 *     'orangered2' : (238, 64, 0),
 *     'orangered3' : (205, 55, 0),
 *     'orangered4' : (139, 37, 0),             # <<<<<<<<<<<<<<
 *     'orchid' : (218, 112, 214),
 *     'orchid1' : (255, 131, 250),
 */
  __pyx_tuple__377 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_37, __pyx_int_0); if (unlikely(!__pyx_tuple__377)) __PYX_ERR(2, 511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__377);
  __Pyx_GIVEREF(__pyx_tuple__377);

  /* "include/color_dict.pxi":512
 *     'orangered3' : (205, 55, 0),
 *     'orangered4' : (139, 37, 0),
 *     'orchid' : (218, 112, 214),             # <<<<<<<<<<<<<<
 *     'orchid1' : (255, 131, 250),
 *     'orchid2' : (238, 122, 233),
 */
  __pyx_tuple__378 = PyTuple_Pack(3, __pyx_int_218, __pyx_int_112, __pyx_int_214); if (unlikely(!__pyx_tuple__378)) __PYX_ERR(2, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__378);
  __Pyx_GIVEREF(__pyx_tuple__378);

  /* "include/color_dict.pxi":513
 *     'orangered4' : (139, 37, 0),
 *     'orchid' : (218, 112, 214),
 *     'orchid1' : (255, 131, 250),             # <<<<<<<<<<<<<<
 *     'orchid2' : (238, 122, 233),
 *     'orchid3' : (205, 105, 201),
 */
  __pyx_tuple__379 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_131, __pyx_int_250); if (unlikely(!__pyx_tuple__379)) __PYX_ERR(2, 513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__379);
  __Pyx_GIVEREF(__pyx_tuple__379);

  /* "include/color_dict.pxi":514
 *     'orchid' : (218, 112, 214),
 *     'orchid1' : (255, 131, 250),
 *     'orchid2' : (238, 122, 233),             # <<<<<<<<<<<<<<
 *     'orchid3' : (205, 105, 201),
 *     'orchid4' : (139, 71, 137),
 */
  __pyx_tuple__380 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_122, __pyx_int_233); if (unlikely(!__pyx_tuple__380)) __PYX_ERR(2, 514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__380);
  __Pyx_GIVEREF(__pyx_tuple__380);

  /* "include/color_dict.pxi":515
 *     'orchid1' : (255, 131, 250),
 *     'orchid2' : (238, 122, 233),
 *     'orchid3' : (205, 105, 201),             # <<<<<<<<<<<<<<
 *     'orchid4' : (139, 71, 137),
 *     'palegoldenrod' : (238, 232, 170),
 */
  __pyx_tuple__381 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_105, __pyx_int_201); if (unlikely(!__pyx_tuple__381)) __PYX_ERR(2, 515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__381);
  __Pyx_GIVEREF(__pyx_tuple__381);

  /* "include/color_dict.pxi":516
 *     'orchid2' : (238, 122, 233),
 *     'orchid3' : (205, 105, 201),
 *     'orchid4' : (139, 71, 137),             # <<<<<<<<<<<<<<
 *     'palegoldenrod' : (238, 232, 170),
 *     'palegreen' : (152, 251, 152),
 */
  __pyx_tuple__382 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_71, __pyx_int_137); if (unlikely(!__pyx_tuple__382)) __PYX_ERR(2, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__382);
  __Pyx_GIVEREF(__pyx_tuple__382);

  /* "include/color_dict.pxi":517
 *     'orchid3' : (205, 105, 201),
 *     'orchid4' : (139, 71, 137),
 *     'palegoldenrod' : (238, 232, 170),             # <<<<<<<<<<<<<<
 *     'palegreen' : (152, 251, 152),
 *     'palegreen1' : (154, 255, 154),
 */
  __pyx_tuple__383 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_232, __pyx_int_170); if (unlikely(!__pyx_tuple__383)) __PYX_ERR(2, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__383);
  __Pyx_GIVEREF(__pyx_tuple__383);

  /* "include/color_dict.pxi":518
 *     'orchid4' : (139, 71, 137),
 *     'palegoldenrod' : (238, 232, 170),
 *     'palegreen' : (152, 251, 152),             # <<<<<<<<<<<<<<
 *     'palegreen1' : (154, 255, 154),
 *     'palegreen2' : (144, 238, 144),
 */
  __pyx_tuple__384 = PyTuple_Pack(3, __pyx_int_152, __pyx_int_251, __pyx_int_152); if (unlikely(!__pyx_tuple__384)) __PYX_ERR(2, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__384);
  __Pyx_GIVEREF(__pyx_tuple__384);

  /* "include/color_dict.pxi":519
 *     'palegoldenrod' : (238, 232, 170),
 *     'palegreen' : (152, 251, 152),
 *     'palegreen1' : (154, 255, 154),             # <<<<<<<<<<<<<<
 *     'palegreen2' : (144, 238, 144),
 *     'palegreen3' : (124, 205, 124),
 */
  __pyx_tuple__385 = PyTuple_Pack(3, __pyx_int_154, __pyx_int_255, __pyx_int_154); if (unlikely(!__pyx_tuple__385)) __PYX_ERR(2, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__385);
  __Pyx_GIVEREF(__pyx_tuple__385);

  /* "include/color_dict.pxi":521
 *     'palegreen1' : (154, 255, 154),
 *     'palegreen2' : (144, 238, 144),
 *     'palegreen3' : (124, 205, 124),             # <<<<<<<<<<<<<<
 *     'palegreen4' : (84, 139, 84),
 *     'paleturquoise' : (175, 238, 238),
 */
  __pyx_tuple__386 = PyTuple_Pack(3, __pyx_int_124, __pyx_int_205, __pyx_int_124); if (unlikely(!__pyx_tuple__386)) __PYX_ERR(2, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__386);
  __Pyx_GIVEREF(__pyx_tuple__386);

  /* "include/color_dict.pxi":522
 *     'palegreen2' : (144, 238, 144),
 *     'palegreen3' : (124, 205, 124),
 *     'palegreen4' : (84, 139, 84),             # <<<<<<<<<<<<<<
 *     'paleturquoise' : (175, 238, 238),
 *     'paleturquoise1' : (187, 255, 255),
 */
  __pyx_tuple__387 = PyTuple_Pack(3, __pyx_int_84, __pyx_int_139, __pyx_int_84); if (unlikely(!__pyx_tuple__387)) __PYX_ERR(2, 522, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__387);
  __Pyx_GIVEREF(__pyx_tuple__387);

  /* "include/color_dict.pxi":523
 *     'palegreen3' : (124, 205, 124),
 *     'palegreen4' : (84, 139, 84),
 *     'paleturquoise' : (175, 238, 238),             # <<<<<<<<<<<<<<
 *     'paleturquoise1' : (187, 255, 255),
 *     'paleturquoise2' : (174, 238, 238),
 */
  __pyx_tuple__388 = PyTuple_Pack(3, __pyx_int_175, __pyx_int_238, __pyx_int_238); if (unlikely(!__pyx_tuple__388)) __PYX_ERR(2, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__388);
  __Pyx_GIVEREF(__pyx_tuple__388);

  /* "include/color_dict.pxi":524
 *     'palegreen4' : (84, 139, 84),
 *     'paleturquoise' : (175, 238, 238),
 *     'paleturquoise1' : (187, 255, 255),             # <<<<<<<<<<<<<<
 *     'paleturquoise2' : (174, 238, 238),
 *     'paleturquoise3' : (150, 205, 205),
 */
  __pyx_tuple__389 = PyTuple_Pack(3, __pyx_int_187, __pyx_int_255, __pyx_int_255); if (unlikely(!__pyx_tuple__389)) __PYX_ERR(2, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__389);
  __Pyx_GIVEREF(__pyx_tuple__389);

  /* "include/color_dict.pxi":525
 *     'paleturquoise' : (175, 238, 238),
 *     'paleturquoise1' : (187, 255, 255),
 *     'paleturquoise2' : (174, 238, 238),             # <<<<<<<<<<<<<<
 *     'paleturquoise3' : (150, 205, 205),
 *     'paleturquoise4' : (102, 139, 139),
 */
  __pyx_tuple__390 = PyTuple_Pack(3, __pyx_int_174, __pyx_int_238, __pyx_int_238); if (unlikely(!__pyx_tuple__390)) __PYX_ERR(2, 525, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__390);
  __Pyx_GIVEREF(__pyx_tuple__390);

  /* "include/color_dict.pxi":526
 *     'paleturquoise1' : (187, 255, 255),
 *     'paleturquoise2' : (174, 238, 238),
 *     'paleturquoise3' : (150, 205, 205),             # <<<<<<<<<<<<<<
 *     'paleturquoise4' : (102, 139, 139),
 *     'palevioletred' : (219, 112, 147),
 */
  __pyx_tuple__391 = PyTuple_Pack(3, __pyx_int_150, __pyx_int_205, __pyx_int_205); if (unlikely(!__pyx_tuple__391)) __PYX_ERR(2, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__391);
  __Pyx_GIVEREF(__pyx_tuple__391);

  /* "include/color_dict.pxi":527
 *     'paleturquoise2' : (174, 238, 238),
 *     'paleturquoise3' : (150, 205, 205),
 *     'paleturquoise4' : (102, 139, 139),             # <<<<<<<<<<<<<<
 *     'palevioletred' : (219, 112, 147),
 *     'palevioletred1' : (255, 130, 171),
 */
  __pyx_tuple__392 = PyTuple_Pack(3, __pyx_int_102, __pyx_int_139, __pyx_int_139); if (unlikely(!__pyx_tuple__392)) __PYX_ERR(2, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__392);
  __Pyx_GIVEREF(__pyx_tuple__392);

  /* "include/color_dict.pxi":528
 *     'paleturquoise3' : (150, 205, 205),
 *     'paleturquoise4' : (102, 139, 139),
 *     'palevioletred' : (219, 112, 147),             # <<<<<<<<<<<<<<
 *     'palevioletred1' : (255, 130, 171),
 *     'palevioletred2' : (238, 121, 159),
 */
  __pyx_tuple__393 = PyTuple_Pack(3, __pyx_int_219, __pyx_int_112, __pyx_int_147); if (unlikely(!__pyx_tuple__393)) __PYX_ERR(2, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__393);
  __Pyx_GIVEREF(__pyx_tuple__393);

  /* "include/color_dict.pxi":529
 *     'paleturquoise4' : (102, 139, 139),
 *     'palevioletred' : (219, 112, 147),
 *     'palevioletred1' : (255, 130, 171),             # <<<<<<<<<<<<<<
 *     'palevioletred2' : (238, 121, 159),
 *     'palevioletred3' : (205, 104, 137),
 */
  __pyx_tuple__394 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_130, __pyx_int_171); if (unlikely(!__pyx_tuple__394)) __PYX_ERR(2, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__394);
  __Pyx_GIVEREF(__pyx_tuple__394);

  /* "include/color_dict.pxi":530
 *     'palevioletred' : (219, 112, 147),
 *     'palevioletred1' : (255, 130, 171),
 *     'palevioletred2' : (238, 121, 159),             # <<<<<<<<<<<<<<
 *     'palevioletred3' : (205, 104, 137),
 *     'palevioletred4' : (139, 71, 93),
 */
  __pyx_tuple__395 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_121, __pyx_int_159); if (unlikely(!__pyx_tuple__395)) __PYX_ERR(2, 530, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__395);
  __Pyx_GIVEREF(__pyx_tuple__395);

  /* "include/color_dict.pxi":531
 *     'palevioletred1' : (255, 130, 171),
 *     'palevioletred2' : (238, 121, 159),
 *     'palevioletred3' : (205, 104, 137),             # <<<<<<<<<<<<<<
 *     'palevioletred4' : (139, 71, 93),
 *     'papayawhip' : (255, 239, 213),
 */
  __pyx_tuple__396 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_104, __pyx_int_137); if (unlikely(!__pyx_tuple__396)) __PYX_ERR(2, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__396);
  __Pyx_GIVEREF(__pyx_tuple__396);

  /* "include/color_dict.pxi":532
 *     'palevioletred2' : (238, 121, 159),
 *     'palevioletred3' : (205, 104, 137),
 *     'palevioletred4' : (139, 71, 93),             # <<<<<<<<<<<<<<
 *     'papayawhip' : (255, 239, 213),
 *     'peachpuff' : (255, 218, 185),
 */
  __pyx_tuple__397 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_71, __pyx_int_93); if (unlikely(!__pyx_tuple__397)) __PYX_ERR(2, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__397);
  __Pyx_GIVEREF(__pyx_tuple__397);

  /* "include/color_dict.pxi":533
 *     'palevioletred3' : (205, 104, 137),
 *     'palevioletred4' : (139, 71, 93),
 *     'papayawhip' : (255, 239, 213),             # <<<<<<<<<<<<<<
 *     'peachpuff' : (255, 218, 185),
 *     'peachpuff1' : (255, 218, 185),
 */
  __pyx_tuple__398 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_239, __pyx_int_213); if (unlikely(!__pyx_tuple__398)) __PYX_ERR(2, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__398);
  __Pyx_GIVEREF(__pyx_tuple__398);

  /* "include/color_dict.pxi":534
 *     'palevioletred4' : (139, 71, 93),
 *     'papayawhip' : (255, 239, 213),
 *     'peachpuff' : (255, 218, 185),             # <<<<<<<<<<<<<<
 *     'peachpuff1' : (255, 218, 185),
 *     'peachpuff2' : (238, 203, 173),
 */
  __pyx_tuple__399 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_218, __pyx_int_185); if (unlikely(!__pyx_tuple__399)) __PYX_ERR(2, 534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__399);
  __Pyx_GIVEREF(__pyx_tuple__399);

  /* "include/color_dict.pxi":536
 *     'peachpuff' : (255, 218, 185),
 *     'peachpuff1' : (255, 218, 185),
 *     'peachpuff2' : (238, 203, 173),             # <<<<<<<<<<<<<<
 *     'peachpuff3' : (205, 175, 149),
 *     'peachpuff4' : (139, 119, 101),
 */
  __pyx_tuple__400 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_203, __pyx_int_173); if (unlikely(!__pyx_tuple__400)) __PYX_ERR(2, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__400);
  __Pyx_GIVEREF(__pyx_tuple__400);

  /* "include/color_dict.pxi":537
 *     'peachpuff1' : (255, 218, 185),
 *     'peachpuff2' : (238, 203, 173),
 *     'peachpuff3' : (205, 175, 149),             # <<<<<<<<<<<<<<
 *     'peachpuff4' : (139, 119, 101),
 *     'peru' : (205, 133, 63),
 */
  __pyx_tuple__401 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_175, __pyx_int_149); if (unlikely(!__pyx_tuple__401)) __PYX_ERR(2, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__401);
  __Pyx_GIVEREF(__pyx_tuple__401);

  /* "include/color_dict.pxi":538
 *     'peachpuff2' : (238, 203, 173),
 *     'peachpuff3' : (205, 175, 149),
 *     'peachpuff4' : (139, 119, 101),             # <<<<<<<<<<<<<<
 *     'peru' : (205, 133, 63),
 *     'pink' : (255, 192, 203),
 */
  __pyx_tuple__402 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_119, __pyx_int_101); if (unlikely(!__pyx_tuple__402)) __PYX_ERR(2, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__402);
  __Pyx_GIVEREF(__pyx_tuple__402);

  /* "include/color_dict.pxi":539
 *     'peachpuff3' : (205, 175, 149),
 *     'peachpuff4' : (139, 119, 101),
 *     'peru' : (205, 133, 63),             # <<<<<<<<<<<<<<
 *     'pink' : (255, 192, 203),
 *     'pink1' : (255, 181, 197),
 */
  __pyx_tuple__403 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_133, __pyx_int_63); if (unlikely(!__pyx_tuple__403)) __PYX_ERR(2, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__403);
  __Pyx_GIVEREF(__pyx_tuple__403);

  /* "include/color_dict.pxi":540
 *     'peachpuff4' : (139, 119, 101),
 *     'peru' : (205, 133, 63),
 *     'pink' : (255, 192, 203),             # <<<<<<<<<<<<<<
 *     'pink1' : (255, 181, 197),
 *     'pink2' : (238, 169, 184),
 */
  __pyx_tuple__404 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_192, __pyx_int_203); if (unlikely(!__pyx_tuple__404)) __PYX_ERR(2, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__404);
  __Pyx_GIVEREF(__pyx_tuple__404);

  /* "include/color_dict.pxi":541
 *     'peru' : (205, 133, 63),
 *     'pink' : (255, 192, 203),
 *     'pink1' : (255, 181, 197),             # <<<<<<<<<<<<<<
 *     'pink2' : (238, 169, 184),
 *     'pink3' : (205, 145, 158),
 */
  __pyx_tuple__405 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_181, __pyx_int_197); if (unlikely(!__pyx_tuple__405)) __PYX_ERR(2, 541, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__405);
  __Pyx_GIVEREF(__pyx_tuple__405);

  /* "include/color_dict.pxi":542
 *     'pink' : (255, 192, 203),
 *     'pink1' : (255, 181, 197),
 *     'pink2' : (238, 169, 184),             # <<<<<<<<<<<<<<
 *     'pink3' : (205, 145, 158),
 *     'pink4' : (139, 99, 108),
 */
  __pyx_tuple__406 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_169, __pyx_int_184); if (unlikely(!__pyx_tuple__406)) __PYX_ERR(2, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__406);
  __Pyx_GIVEREF(__pyx_tuple__406);

  /* "include/color_dict.pxi":543
 *     'pink1' : (255, 181, 197),
 *     'pink2' : (238, 169, 184),
 *     'pink3' : (205, 145, 158),             # <<<<<<<<<<<<<<
 *     'pink4' : (139, 99, 108),
 *     'plum' : (221, 160, 221),
 */
  __pyx_tuple__407 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_145, __pyx_int_158); if (unlikely(!__pyx_tuple__407)) __PYX_ERR(2, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__407);
  __Pyx_GIVEREF(__pyx_tuple__407);

  /* "include/color_dict.pxi":544
 *     'pink2' : (238, 169, 184),
 *     'pink3' : (205, 145, 158),
 *     'pink4' : (139, 99, 108),             # <<<<<<<<<<<<<<
 *     'plum' : (221, 160, 221),
 *     'plum1' : (255, 187, 255),
 */
  __pyx_tuple__408 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_99, __pyx_int_108); if (unlikely(!__pyx_tuple__408)) __PYX_ERR(2, 544, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__408);
  __Pyx_GIVEREF(__pyx_tuple__408);

  /* "include/color_dict.pxi":545
 *     'pink3' : (205, 145, 158),
 *     'pink4' : (139, 99, 108),
 *     'plum' : (221, 160, 221),             # <<<<<<<<<<<<<<
 *     'plum1' : (255, 187, 255),
 *     'plum2' : (238, 174, 238),
 */
  __pyx_tuple__409 = PyTuple_Pack(3, __pyx_int_221, __pyx_int_160, __pyx_int_221); if (unlikely(!__pyx_tuple__409)) __PYX_ERR(2, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__409);
  __Pyx_GIVEREF(__pyx_tuple__409);

  /* "include/color_dict.pxi":546
 *     'pink4' : (139, 99, 108),
 *     'plum' : (221, 160, 221),
 *     'plum1' : (255, 187, 255),             # <<<<<<<<<<<<<<
 *     'plum2' : (238, 174, 238),
 *     'plum3' : (205, 150, 205),
 */
  __pyx_tuple__410 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_187, __pyx_int_255); if (unlikely(!__pyx_tuple__410)) __PYX_ERR(2, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__410);
  __Pyx_GIVEREF(__pyx_tuple__410);

  /* "include/color_dict.pxi":547
 *     'plum' : (221, 160, 221),
 *     'plum1' : (255, 187, 255),
 *     'plum2' : (238, 174, 238),             # <<<<<<<<<<<<<<
 *     'plum3' : (205, 150, 205),
 *     'plum4' : (139, 102, 139),
 */
  __pyx_tuple__411 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_174, __pyx_int_238); if (unlikely(!__pyx_tuple__411)) __PYX_ERR(2, 547, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__411);
  __Pyx_GIVEREF(__pyx_tuple__411);

  /* "include/color_dict.pxi":548
 *     'plum1' : (255, 187, 255),
 *     'plum2' : (238, 174, 238),
 *     'plum3' : (205, 150, 205),             # <<<<<<<<<<<<<<
 *     'plum4' : (139, 102, 139),
 *     'powderblue' : (176, 224, 230),
 */
  __pyx_tuple__412 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_150, __pyx_int_205); if (unlikely(!__pyx_tuple__412)) __PYX_ERR(2, 548, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__412);
  __Pyx_GIVEREF(__pyx_tuple__412);

  /* "include/color_dict.pxi":549
 *     'plum2' : (238, 174, 238),
 *     'plum3' : (205, 150, 205),
 *     'plum4' : (139, 102, 139),             # <<<<<<<<<<<<<<
 *     'powderblue' : (176, 224, 230),
 *     'purple' : (160, 32, 240),
 */
  __pyx_tuple__413 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_102, __pyx_int_139); if (unlikely(!__pyx_tuple__413)) __PYX_ERR(2, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__413);
  __Pyx_GIVEREF(__pyx_tuple__413);

  /* "include/color_dict.pxi":550
 *     'plum3' : (205, 150, 205),
 *     'plum4' : (139, 102, 139),
 *     'powderblue' : (176, 224, 230),             # <<<<<<<<<<<<<<
 *     'purple' : (160, 32, 240),
 *     'purple1' : (155, 48, 255),
 */
  __pyx_tuple__414 = PyTuple_Pack(3, __pyx_int_176, __pyx_int_224, __pyx_int_230); if (unlikely(!__pyx_tuple__414)) __PYX_ERR(2, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__414);
  __Pyx_GIVEREF(__pyx_tuple__414);

  /* "include/color_dict.pxi":551
 *     'plum4' : (139, 102, 139),
 *     'powderblue' : (176, 224, 230),
 *     'purple' : (160, 32, 240),             # <<<<<<<<<<<<<<
 *     'purple1' : (155, 48, 255),
 *     'purple2' : (145, 44, 238),
 */
  __pyx_tuple__415 = PyTuple_Pack(3, __pyx_int_160, __pyx_int_32, __pyx_int_240); if (unlikely(!__pyx_tuple__415)) __PYX_ERR(2, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__415);
  __Pyx_GIVEREF(__pyx_tuple__415);

  /* "include/color_dict.pxi":552
 *     'powderblue' : (176, 224, 230),
 *     'purple' : (160, 32, 240),
 *     'purple1' : (155, 48, 255),             # <<<<<<<<<<<<<<
 *     'purple2' : (145, 44, 238),
 *     'purple3' : (125, 38, 205),
 */
  __pyx_tuple__416 = PyTuple_Pack(3, __pyx_int_155, __pyx_int_48, __pyx_int_255); if (unlikely(!__pyx_tuple__416)) __PYX_ERR(2, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__416);
  __Pyx_GIVEREF(__pyx_tuple__416);

  /* "include/color_dict.pxi":553
 *     'purple' : (160, 32, 240),
 *     'purple1' : (155, 48, 255),
 *     'purple2' : (145, 44, 238),             # <<<<<<<<<<<<<<
 *     'purple3' : (125, 38, 205),
 *     'purple4' : (85, 26, 139),
 */
  __pyx_tuple__417 = PyTuple_Pack(3, __pyx_int_145, __pyx_int_44, __pyx_int_238); if (unlikely(!__pyx_tuple__417)) __PYX_ERR(2, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__417);
  __Pyx_GIVEREF(__pyx_tuple__417);

  /* "include/color_dict.pxi":554
 *     'purple1' : (155, 48, 255),
 *     'purple2' : (145, 44, 238),
 *     'purple3' : (125, 38, 205),             # <<<<<<<<<<<<<<
 *     'purple4' : (85, 26, 139),
 *     'red' : (255, 0, 0),
 */
  __pyx_tuple__418 = PyTuple_Pack(3, __pyx_int_125, __pyx_int_38, __pyx_int_205); if (unlikely(!__pyx_tuple__418)) __PYX_ERR(2, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__418);
  __Pyx_GIVEREF(__pyx_tuple__418);

  /* "include/color_dict.pxi":555
 *     'purple2' : (145, 44, 238),
 *     'purple3' : (125, 38, 205),
 *     'purple4' : (85, 26, 139),             # <<<<<<<<<<<<<<
 *     'red' : (255, 0, 0),
 *     'red1' : (255, 0, 0),
 */
  __pyx_tuple__419 = PyTuple_Pack(3, __pyx_int_85, __pyx_int_26, __pyx_int_139); if (unlikely(!__pyx_tuple__419)) __PYX_ERR(2, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__419);
  __Pyx_GIVEREF(__pyx_tuple__419);

  /* "include/color_dict.pxi":556
 *     'purple3' : (125, 38, 205),
 *     'purple4' : (85, 26, 139),
 *     'red' : (255, 0, 0),             # <<<<<<<<<<<<<<
 *     'red1' : (255, 0, 0),
 *     'red2' : (238, 0, 0),
 */
  __pyx_tuple__420 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_0, __pyx_int_0); if (unlikely(!__pyx_tuple__420)) __PYX_ERR(2, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__420);
  __Pyx_GIVEREF(__pyx_tuple__420);

  /* "include/color_dict.pxi":558
 *     'red' : (255, 0, 0),
 *     'red1' : (255, 0, 0),
 *     'red2' : (238, 0, 0),             # <<<<<<<<<<<<<<
 *     'red3' : (205, 0, 0),
 *     'red4' : (139, 0, 0),
 */
  __pyx_tuple__421 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_0, __pyx_int_0); if (unlikely(!__pyx_tuple__421)) __PYX_ERR(2, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__421);
  __Pyx_GIVEREF(__pyx_tuple__421);

  /* "include/color_dict.pxi":559
 *     'red1' : (255, 0, 0),
 *     'red2' : (238, 0, 0),
 *     'red3' : (205, 0, 0),             # <<<<<<<<<<<<<<
 *     'red4' : (139, 0, 0),
 *     'rosybrown' : (188, 143, 143),
 */
  __pyx_tuple__422 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_0, __pyx_int_0); if (unlikely(!__pyx_tuple__422)) __PYX_ERR(2, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__422);
  __Pyx_GIVEREF(__pyx_tuple__422);

  /* "include/color_dict.pxi":561
 *     'red3' : (205, 0, 0),
 *     'red4' : (139, 0, 0),
 *     'rosybrown' : (188, 143, 143),             # <<<<<<<<<<<<<<
 *     'rosybrown1' : (255, 193, 193),
 *     'rosybrown2' : (238, 180, 180),
 */
  __pyx_tuple__423 = PyTuple_Pack(3, __pyx_int_188, __pyx_int_143, __pyx_int_143); if (unlikely(!__pyx_tuple__423)) __PYX_ERR(2, 561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__423);
  __Pyx_GIVEREF(__pyx_tuple__423);

  /* "include/color_dict.pxi":562
 *     'red4' : (139, 0, 0),
 *     'rosybrown' : (188, 143, 143),
 *     'rosybrown1' : (255, 193, 193),             # <<<<<<<<<<<<<<
 *     'rosybrown2' : (238, 180, 180),
 *     'rosybrown3' : (205, 155, 155),
 */
  __pyx_tuple__424 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_193, __pyx_int_193); if (unlikely(!__pyx_tuple__424)) __PYX_ERR(2, 562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__424);
  __Pyx_GIVEREF(__pyx_tuple__424);

  /* "include/color_dict.pxi":563
 *     'rosybrown' : (188, 143, 143),
 *     'rosybrown1' : (255, 193, 193),
 *     'rosybrown2' : (238, 180, 180),             # <<<<<<<<<<<<<<
 *     'rosybrown3' : (205, 155, 155),
 *     'rosybrown4' : (139, 105, 105),
 */
  __pyx_tuple__425 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_180, __pyx_int_180); if (unlikely(!__pyx_tuple__425)) __PYX_ERR(2, 563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__425);
  __Pyx_GIVEREF(__pyx_tuple__425);

  /* "include/color_dict.pxi":564
 *     'rosybrown1' : (255, 193, 193),
 *     'rosybrown2' : (238, 180, 180),
 *     'rosybrown3' : (205, 155, 155),             # <<<<<<<<<<<<<<
 *     'rosybrown4' : (139, 105, 105),
 *     'royalblue' : (65, 105, 225),
 */
  __pyx_tuple__426 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_155, __pyx_int_155); if (unlikely(!__pyx_tuple__426)) __PYX_ERR(2, 564, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__426);
  __Pyx_GIVEREF(__pyx_tuple__426);

  /* "include/color_dict.pxi":565
 *     'rosybrown2' : (238, 180, 180),
 *     'rosybrown3' : (205, 155, 155),
 *     'rosybrown4' : (139, 105, 105),             # <<<<<<<<<<<<<<
 *     'royalblue' : (65, 105, 225),
 *     'royalblue1' : (72, 118, 255),
 */
  __pyx_tuple__427 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_105, __pyx_int_105); if (unlikely(!__pyx_tuple__427)) __PYX_ERR(2, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__427);
  __Pyx_GIVEREF(__pyx_tuple__427);

  /* "include/color_dict.pxi":566
 *     'rosybrown3' : (205, 155, 155),
 *     'rosybrown4' : (139, 105, 105),
 *     'royalblue' : (65, 105, 225),             # <<<<<<<<<<<<<<
 *     'royalblue1' : (72, 118, 255),
 *     'royalblue2' : (67, 110, 238),
 */
  __pyx_tuple__428 = PyTuple_Pack(3, __pyx_int_65, __pyx_int_105, __pyx_int_225); if (unlikely(!__pyx_tuple__428)) __PYX_ERR(2, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__428);
  __Pyx_GIVEREF(__pyx_tuple__428);

  /* "include/color_dict.pxi":567
 *     'rosybrown4' : (139, 105, 105),
 *     'royalblue' : (65, 105, 225),
 *     'royalblue1' : (72, 118, 255),             # <<<<<<<<<<<<<<
 *     'royalblue2' : (67, 110, 238),
 *     'royalblue3' : (58, 95, 205),
 */
  __pyx_tuple__429 = PyTuple_Pack(3, __pyx_int_72, __pyx_int_118, __pyx_int_255); if (unlikely(!__pyx_tuple__429)) __PYX_ERR(2, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__429);
  __Pyx_GIVEREF(__pyx_tuple__429);

  /* "include/color_dict.pxi":568
 *     'royalblue' : (65, 105, 225),
 *     'royalblue1' : (72, 118, 255),
 *     'royalblue2' : (67, 110, 238),             # <<<<<<<<<<<<<<
 *     'royalblue3' : (58, 95, 205),
 *     'royalblue4' : (39, 64, 139),
 */
  __pyx_tuple__430 = PyTuple_Pack(3, __pyx_int_67, __pyx_int_110, __pyx_int_238); if (unlikely(!__pyx_tuple__430)) __PYX_ERR(2, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__430);
  __Pyx_GIVEREF(__pyx_tuple__430);

  /* "include/color_dict.pxi":569
 *     'royalblue1' : (72, 118, 255),
 *     'royalblue2' : (67, 110, 238),
 *     'royalblue3' : (58, 95, 205),             # <<<<<<<<<<<<<<
 *     'royalblue4' : (39, 64, 139),
 *     'saddlebrown' : (139, 69, 19),
 */
  __pyx_tuple__431 = PyTuple_Pack(3, __pyx_int_58, __pyx_int_95, __pyx_int_205); if (unlikely(!__pyx_tuple__431)) __PYX_ERR(2, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__431);
  __Pyx_GIVEREF(__pyx_tuple__431);

  /* "include/color_dict.pxi":570
 *     'royalblue2' : (67, 110, 238),
 *     'royalblue3' : (58, 95, 205),
 *     'royalblue4' : (39, 64, 139),             # <<<<<<<<<<<<<<
 *     'saddlebrown' : (139, 69, 19),
 *     'salmon' : (250, 128, 114),
 */
  __pyx_tuple__432 = PyTuple_Pack(3, __pyx_int_39, __pyx_int_64, __pyx_int_139); if (unlikely(!__pyx_tuple__432)) __PYX_ERR(2, 570, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__432);
  __Pyx_GIVEREF(__pyx_tuple__432);

  /* "include/color_dict.pxi":572
 *     'royalblue4' : (39, 64, 139),
 *     'saddlebrown' : (139, 69, 19),
 *     'salmon' : (250, 128, 114),             # <<<<<<<<<<<<<<
 *     'salmon1' : (255, 140, 105),
 *     'salmon2' : (238, 130, 98),
 */
  __pyx_tuple__433 = PyTuple_Pack(3, __pyx_int_250, __pyx_int_128, __pyx_int_114); if (unlikely(!__pyx_tuple__433)) __PYX_ERR(2, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__433);
  __Pyx_GIVEREF(__pyx_tuple__433);

  /* "include/color_dict.pxi":573
 *     'saddlebrown' : (139, 69, 19),
 *     'salmon' : (250, 128, 114),
 *     'salmon1' : (255, 140, 105),             # <<<<<<<<<<<<<<
 *     'salmon2' : (238, 130, 98),
 *     'salmon3' : (205, 112, 84),
 */
  __pyx_tuple__434 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_140, __pyx_int_105); if (unlikely(!__pyx_tuple__434)) __PYX_ERR(2, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__434);
  __Pyx_GIVEREF(__pyx_tuple__434);

  /* "include/color_dict.pxi":574
 *     'salmon' : (250, 128, 114),
 *     'salmon1' : (255, 140, 105),
 *     'salmon2' : (238, 130, 98),             # <<<<<<<<<<<<<<
 *     'salmon3' : (205, 112, 84),
 *     'salmon4' : (139, 76, 57),
 */
  __pyx_tuple__435 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_130, __pyx_int_98); if (unlikely(!__pyx_tuple__435)) __PYX_ERR(2, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__435);
  __Pyx_GIVEREF(__pyx_tuple__435);

  /* "include/color_dict.pxi":575
 *     'salmon1' : (255, 140, 105),
 *     'salmon2' : (238, 130, 98),
 *     'salmon3' : (205, 112, 84),             # <<<<<<<<<<<<<<
 *     'salmon4' : (139, 76, 57),
 *     'sandybrown' : (244, 164, 96),
 */
  __pyx_tuple__436 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_112, __pyx_int_84); if (unlikely(!__pyx_tuple__436)) __PYX_ERR(2, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__436);
  __Pyx_GIVEREF(__pyx_tuple__436);

  /* "include/color_dict.pxi":576
 *     'salmon2' : (238, 130, 98),
 *     'salmon3' : (205, 112, 84),
 *     'salmon4' : (139, 76, 57),             # <<<<<<<<<<<<<<
 *     'sandybrown' : (244, 164, 96),
 *     'seagreen' : (46, 139, 87),
 */
  __pyx_tuple__437 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_76, __pyx_int_57); if (unlikely(!__pyx_tuple__437)) __PYX_ERR(2, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__437);
  __Pyx_GIVEREF(__pyx_tuple__437);

  /* "include/color_dict.pxi":577
 *     'salmon3' : (205, 112, 84),
 *     'salmon4' : (139, 76, 57),
 *     'sandybrown' : (244, 164, 96),             # <<<<<<<<<<<<<<
 *     'seagreen' : (46, 139, 87),
 *     'seagreen1' : (84, 255, 159),
 */
  __pyx_tuple__438 = PyTuple_Pack(3, __pyx_int_244, __pyx_int_164, __pyx_int_96); if (unlikely(!__pyx_tuple__438)) __PYX_ERR(2, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__438);
  __Pyx_GIVEREF(__pyx_tuple__438);

  /* "include/color_dict.pxi":578
 *     'salmon4' : (139, 76, 57),
 *     'sandybrown' : (244, 164, 96),
 *     'seagreen' : (46, 139, 87),             # <<<<<<<<<<<<<<
 *     'seagreen1' : (84, 255, 159),
 *     'seagreen2' : (78, 238, 148),
 */
  __pyx_tuple__439 = PyTuple_Pack(3, __pyx_int_46, __pyx_int_139, __pyx_int_87); if (unlikely(!__pyx_tuple__439)) __PYX_ERR(2, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__439);
  __Pyx_GIVEREF(__pyx_tuple__439);

  /* "include/color_dict.pxi":579
 *     'sandybrown' : (244, 164, 96),
 *     'seagreen' : (46, 139, 87),
 *     'seagreen1' : (84, 255, 159),             # <<<<<<<<<<<<<<
 *     'seagreen2' : (78, 238, 148),
 *     'seagreen3' : (67, 205, 128),
 */
  __pyx_tuple__440 = PyTuple_Pack(3, __pyx_int_84, __pyx_int_255, __pyx_int_159); if (unlikely(!__pyx_tuple__440)) __PYX_ERR(2, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__440);
  __Pyx_GIVEREF(__pyx_tuple__440);

  /* "include/color_dict.pxi":580
 *     'seagreen' : (46, 139, 87),
 *     'seagreen1' : (84, 255, 159),
 *     'seagreen2' : (78, 238, 148),             # <<<<<<<<<<<<<<
 *     'seagreen3' : (67, 205, 128),
 *     'seagreen4' : (46, 139, 87),
 */
  __pyx_tuple__441 = PyTuple_Pack(3, __pyx_int_78, __pyx_int_238, __pyx_int_148); if (unlikely(!__pyx_tuple__441)) __PYX_ERR(2, 580, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__441);
  __Pyx_GIVEREF(__pyx_tuple__441);

  /* "include/color_dict.pxi":581
 *     'seagreen1' : (84, 255, 159),
 *     'seagreen2' : (78, 238, 148),
 *     'seagreen3' : (67, 205, 128),             # <<<<<<<<<<<<<<
 *     'seagreen4' : (46, 139, 87),
 *     'seashell' : (255, 245, 238),
 */
  __pyx_tuple__442 = PyTuple_Pack(3, __pyx_int_67, __pyx_int_205, __pyx_int_128); if (unlikely(!__pyx_tuple__442)) __PYX_ERR(2, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__442);
  __Pyx_GIVEREF(__pyx_tuple__442);

  /* "include/color_dict.pxi":583
 *     'seagreen3' : (67, 205, 128),
 *     'seagreen4' : (46, 139, 87),
 *     'seashell' : (255, 245, 238),             # <<<<<<<<<<<<<<
 *     'seashell1' : (255, 245, 238),
 *     'seashell2' : (238, 229, 222),
 */
  __pyx_tuple__443 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_245, __pyx_int_238); if (unlikely(!__pyx_tuple__443)) __PYX_ERR(2, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__443);
  __Pyx_GIVEREF(__pyx_tuple__443);

  /* "include/color_dict.pxi":585
 *     'seashell' : (255, 245, 238),
 *     'seashell1' : (255, 245, 238),
 *     'seashell2' : (238, 229, 222),             # <<<<<<<<<<<<<<
 *     'seashell3' : (205, 197, 191),
 *     'seashell4' : (139, 134, 130),
 */
  __pyx_tuple__444 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_229, __pyx_int_222); if (unlikely(!__pyx_tuple__444)) __PYX_ERR(2, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__444);
  __Pyx_GIVEREF(__pyx_tuple__444);

  /* "include/color_dict.pxi":586
 *     'seashell1' : (255, 245, 238),
 *     'seashell2' : (238, 229, 222),
 *     'seashell3' : (205, 197, 191),             # <<<<<<<<<<<<<<
 *     'seashell4' : (139, 134, 130),
 *     'sienna' : (160, 82, 45),
 */
  __pyx_tuple__445 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_197, __pyx_int_191); if (unlikely(!__pyx_tuple__445)) __PYX_ERR(2, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__445);
  __Pyx_GIVEREF(__pyx_tuple__445);

  /* "include/color_dict.pxi":587
 *     'seashell2' : (238, 229, 222),
 *     'seashell3' : (205, 197, 191),
 *     'seashell4' : (139, 134, 130),             # <<<<<<<<<<<<<<
 *     'sienna' : (160, 82, 45),
 *     'sienna1' : (255, 130, 71),
 */
  __pyx_tuple__446 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_134, __pyx_int_130); if (unlikely(!__pyx_tuple__446)) __PYX_ERR(2, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__446);
  __Pyx_GIVEREF(__pyx_tuple__446);

  /* "include/color_dict.pxi":588
 *     'seashell3' : (205, 197, 191),
 *     'seashell4' : (139, 134, 130),
 *     'sienna' : (160, 82, 45),             # <<<<<<<<<<<<<<
 *     'sienna1' : (255, 130, 71),
 *     'sienna2' : (238, 121, 66),
 */
  __pyx_tuple__447 = PyTuple_Pack(3, __pyx_int_160, __pyx_int_82, __pyx_int_45); if (unlikely(!__pyx_tuple__447)) __PYX_ERR(2, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__447);
  __Pyx_GIVEREF(__pyx_tuple__447);

  /* "include/color_dict.pxi":589
 *     'seashell4' : (139, 134, 130),
 *     'sienna' : (160, 82, 45),
 *     'sienna1' : (255, 130, 71),             # <<<<<<<<<<<<<<
 *     'sienna2' : (238, 121, 66),
 *     'sienna3' : (205, 104, 57),
 */
  __pyx_tuple__448 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_130, __pyx_int_71); if (unlikely(!__pyx_tuple__448)) __PYX_ERR(2, 589, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__448);
  __Pyx_GIVEREF(__pyx_tuple__448);

  /* "include/color_dict.pxi":590
 *     'sienna' : (160, 82, 45),
 *     'sienna1' : (255, 130, 71),
 *     'sienna2' : (238, 121, 66),             # <<<<<<<<<<<<<<
 *     'sienna3' : (205, 104, 57),
 *     'sienna4' : (139, 71, 38),
 */
  __pyx_tuple__449 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_121, __pyx_int_66); if (unlikely(!__pyx_tuple__449)) __PYX_ERR(2, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__449);
  __Pyx_GIVEREF(__pyx_tuple__449);

  /* "include/color_dict.pxi":591
 *     'sienna1' : (255, 130, 71),
 *     'sienna2' : (238, 121, 66),
 *     'sienna3' : (205, 104, 57),             # <<<<<<<<<<<<<<
 *     'sienna4' : (139, 71, 38),
 *     'skyblue' : (135, 206, 235),
 */
  __pyx_tuple__450 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_104, __pyx_int_57); if (unlikely(!__pyx_tuple__450)) __PYX_ERR(2, 591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__450);
  __Pyx_GIVEREF(__pyx_tuple__450);

  /* "include/color_dict.pxi":592
 *     'sienna2' : (238, 121, 66),
 *     'sienna3' : (205, 104, 57),
 *     'sienna4' : (139, 71, 38),             # <<<<<<<<<<<<<<
 *     'skyblue' : (135, 206, 235),
 *     'skyblue1' : (135, 206, 255),
 */
  __pyx_tuple__451 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_71, __pyx_int_38); if (unlikely(!__pyx_tuple__451)) __PYX_ERR(2, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__451);
  __Pyx_GIVEREF(__pyx_tuple__451);

  /* "include/color_dict.pxi":593
 *     'sienna3' : (205, 104, 57),
 *     'sienna4' : (139, 71, 38),
 *     'skyblue' : (135, 206, 235),             # <<<<<<<<<<<<<<
 *     'skyblue1' : (135, 206, 255),
 *     'skyblue2' : (126, 192, 238),
 */
  __pyx_tuple__452 = PyTuple_Pack(3, __pyx_int_135, __pyx_int_206, __pyx_int_235); if (unlikely(!__pyx_tuple__452)) __PYX_ERR(2, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__452);
  __Pyx_GIVEREF(__pyx_tuple__452);

  /* "include/color_dict.pxi":594
 *     'sienna4' : (139, 71, 38),
 *     'skyblue' : (135, 206, 235),
 *     'skyblue1' : (135, 206, 255),             # <<<<<<<<<<<<<<
 *     'skyblue2' : (126, 192, 238),
 *     'skyblue3' : (108, 166, 205),
 */
  __pyx_tuple__453 = PyTuple_Pack(3, __pyx_int_135, __pyx_int_206, __pyx_int_255); if (unlikely(!__pyx_tuple__453)) __PYX_ERR(2, 594, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__453);
  __Pyx_GIVEREF(__pyx_tuple__453);

  /* "include/color_dict.pxi":595
 *     'skyblue' : (135, 206, 235),
 *     'skyblue1' : (135, 206, 255),
 *     'skyblue2' : (126, 192, 238),             # <<<<<<<<<<<<<<
 *     'skyblue3' : (108, 166, 205),
 *     'skyblue4' : (74, 112, 139),
 */
  __pyx_tuple__454 = PyTuple_Pack(3, __pyx_int_126, __pyx_int_192, __pyx_int_238); if (unlikely(!__pyx_tuple__454)) __PYX_ERR(2, 595, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__454);
  __Pyx_GIVEREF(__pyx_tuple__454);

  /* "include/color_dict.pxi":596
 *     'skyblue1' : (135, 206, 255),
 *     'skyblue2' : (126, 192, 238),
 *     'skyblue3' : (108, 166, 205),             # <<<<<<<<<<<<<<
 *     'skyblue4' : (74, 112, 139),
 *     'slateblue' : (106, 90, 205),
 */
  __pyx_tuple__455 = PyTuple_Pack(3, __pyx_int_108, __pyx_int_166, __pyx_int_205); if (unlikely(!__pyx_tuple__455)) __PYX_ERR(2, 596, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__455);
  __Pyx_GIVEREF(__pyx_tuple__455);

  /* "include/color_dict.pxi":597
 *     'skyblue2' : (126, 192, 238),
 *     'skyblue3' : (108, 166, 205),
 *     'skyblue4' : (74, 112, 139),             # <<<<<<<<<<<<<<
 *     'slateblue' : (106, 90, 205),
 *     'slateblue1' : (131, 111, 255),
 */
  __pyx_tuple__456 = PyTuple_Pack(3, __pyx_int_74, __pyx_int_112, __pyx_int_139); if (unlikely(!__pyx_tuple__456)) __PYX_ERR(2, 597, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__456);
  __Pyx_GIVEREF(__pyx_tuple__456);

  /* "include/color_dict.pxi":598
 *     'skyblue3' : (108, 166, 205),
 *     'skyblue4' : (74, 112, 139),
 *     'slateblue' : (106, 90, 205),             # <<<<<<<<<<<<<<
 *     'slateblue1' : (131, 111, 255),
 *     'slateblue2' : (122, 103, 238),
 */
  __pyx_tuple__457 = PyTuple_Pack(3, __pyx_int_106, __pyx_int_90, __pyx_int_205); if (unlikely(!__pyx_tuple__457)) __PYX_ERR(2, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__457);
  __Pyx_GIVEREF(__pyx_tuple__457);

  /* "include/color_dict.pxi":599
 *     'skyblue4' : (74, 112, 139),
 *     'slateblue' : (106, 90, 205),
 *     'slateblue1' : (131, 111, 255),             # <<<<<<<<<<<<<<
 *     'slateblue2' : (122, 103, 238),
 *     'slateblue3' : (105, 89, 205),
 */
  __pyx_tuple__458 = PyTuple_Pack(3, __pyx_int_131, __pyx_int_111, __pyx_int_255); if (unlikely(!__pyx_tuple__458)) __PYX_ERR(2, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__458);
  __Pyx_GIVEREF(__pyx_tuple__458);

  /* "include/color_dict.pxi":600
 *     'slateblue' : (106, 90, 205),
 *     'slateblue1' : (131, 111, 255),
 *     'slateblue2' : (122, 103, 238),             # <<<<<<<<<<<<<<
 *     'slateblue3' : (105, 89, 205),
 *     'slateblue4' : (71, 60, 139),
 */
  __pyx_tuple__459 = PyTuple_Pack(3, __pyx_int_122, __pyx_int_103, __pyx_int_238); if (unlikely(!__pyx_tuple__459)) __PYX_ERR(2, 600, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__459);
  __Pyx_GIVEREF(__pyx_tuple__459);

  /* "include/color_dict.pxi":601
 *     'slateblue1' : (131, 111, 255),
 *     'slateblue2' : (122, 103, 238),
 *     'slateblue3' : (105, 89, 205),             # <<<<<<<<<<<<<<
 *     'slateblue4' : (71, 60, 139),
 *     'slategray' : (112, 128, 144),
 */
  __pyx_tuple__460 = PyTuple_Pack(3, __pyx_int_105, __pyx_int_89, __pyx_int_205); if (unlikely(!__pyx_tuple__460)) __PYX_ERR(2, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__460);
  __Pyx_GIVEREF(__pyx_tuple__460);

  /* "include/color_dict.pxi":602
 *     'slateblue2' : (122, 103, 238),
 *     'slateblue3' : (105, 89, 205),
 *     'slateblue4' : (71, 60, 139),             # <<<<<<<<<<<<<<
 *     'slategray' : (112, 128, 144),
 *     'slategray1' : (198, 226, 255),
 */
  __pyx_tuple__461 = PyTuple_Pack(3, __pyx_int_71, __pyx_int_60, __pyx_int_139); if (unlikely(!__pyx_tuple__461)) __PYX_ERR(2, 602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__461);
  __Pyx_GIVEREF(__pyx_tuple__461);

  /* "include/color_dict.pxi":603
 *     'slateblue3' : (105, 89, 205),
 *     'slateblue4' : (71, 60, 139),
 *     'slategray' : (112, 128, 144),             # <<<<<<<<<<<<<<
 *     'slategray1' : (198, 226, 255),
 *     'slategray2' : (185, 211, 238),
 */
  __pyx_tuple__462 = PyTuple_Pack(3, __pyx_int_112, __pyx_int_128, __pyx_int_144); if (unlikely(!__pyx_tuple__462)) __PYX_ERR(2, 603, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__462);
  __Pyx_GIVEREF(__pyx_tuple__462);

  /* "include/color_dict.pxi":604
 *     'slateblue4' : (71, 60, 139),
 *     'slategray' : (112, 128, 144),
 *     'slategray1' : (198, 226, 255),             # <<<<<<<<<<<<<<
 *     'slategray2' : (185, 211, 238),
 *     'slategray3' : (159, 182, 205),
 */
  __pyx_tuple__463 = PyTuple_Pack(3, __pyx_int_198, __pyx_int_226, __pyx_int_255); if (unlikely(!__pyx_tuple__463)) __PYX_ERR(2, 604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__463);
  __Pyx_GIVEREF(__pyx_tuple__463);

  /* "include/color_dict.pxi":605
 *     'slategray' : (112, 128, 144),
 *     'slategray1' : (198, 226, 255),
 *     'slategray2' : (185, 211, 238),             # <<<<<<<<<<<<<<
 *     'slategray3' : (159, 182, 205),
 *     'slategray4' : (108, 123, 139),
 */
  __pyx_tuple__464 = PyTuple_Pack(3, __pyx_int_185, __pyx_int_211, __pyx_int_238); if (unlikely(!__pyx_tuple__464)) __PYX_ERR(2, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__464);
  __Pyx_GIVEREF(__pyx_tuple__464);

  /* "include/color_dict.pxi":606
 *     'slategray1' : (198, 226, 255),
 *     'slategray2' : (185, 211, 238),
 *     'slategray3' : (159, 182, 205),             # <<<<<<<<<<<<<<
 *     'slategray4' : (108, 123, 139),
 *     'slategrey' : (112, 128, 144),
 */
  __pyx_tuple__465 = PyTuple_Pack(3, __pyx_int_159, __pyx_int_182, __pyx_int_205); if (unlikely(!__pyx_tuple__465)) __PYX_ERR(2, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__465);
  __Pyx_GIVEREF(__pyx_tuple__465);

  /* "include/color_dict.pxi":607
 *     'slategray2' : (185, 211, 238),
 *     'slategray3' : (159, 182, 205),
 *     'slategray4' : (108, 123, 139),             # <<<<<<<<<<<<<<
 *     'slategrey' : (112, 128, 144),
 *     'snow' : (255, 250, 250),
 */
  __pyx_tuple__466 = PyTuple_Pack(3, __pyx_int_108, __pyx_int_123, __pyx_int_139); if (unlikely(!__pyx_tuple__466)) __PYX_ERR(2, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__466);
  __Pyx_GIVEREF(__pyx_tuple__466);

  /* "include/color_dict.pxi":609
 *     'slategray4' : (108, 123, 139),
 *     'slategrey' : (112, 128, 144),
 *     'snow' : (255, 250, 250),             # <<<<<<<<<<<<<<
 *     'snow1' : (255, 250, 250),
 *     'snow2' : (238, 233, 233),
 */
  __pyx_tuple__467 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_250, __pyx_int_250); if (unlikely(!__pyx_tuple__467)) __PYX_ERR(2, 609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__467);
  __Pyx_GIVEREF(__pyx_tuple__467);

  /* "include/color_dict.pxi":611
 *     'snow' : (255, 250, 250),
 *     'snow1' : (255, 250, 250),
 *     'snow2' : (238, 233, 233),             # <<<<<<<<<<<<<<
 *     'snow3' : (205, 201, 201),
 *     'snow4' : (139, 137, 137),
 */
  __pyx_tuple__468 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_233, __pyx_int_233); if (unlikely(!__pyx_tuple__468)) __PYX_ERR(2, 611, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__468);
  __Pyx_GIVEREF(__pyx_tuple__468);

  /* "include/color_dict.pxi":612
 *     'snow1' : (255, 250, 250),
 *     'snow2' : (238, 233, 233),
 *     'snow3' : (205, 201, 201),             # <<<<<<<<<<<<<<
 *     'snow4' : (139, 137, 137),
 *     'springgreen' : (0, 255, 127),
 */
  __pyx_tuple__469 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_201, __pyx_int_201); if (unlikely(!__pyx_tuple__469)) __PYX_ERR(2, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__469);
  __Pyx_GIVEREF(__pyx_tuple__469);

  /* "include/color_dict.pxi":613
 *     'snow2' : (238, 233, 233),
 *     'snow3' : (205, 201, 201),
 *     'snow4' : (139, 137, 137),             # <<<<<<<<<<<<<<
 *     'springgreen' : (0, 255, 127),
 *     'springgreen1' : (0, 255, 127),
 */
  __pyx_tuple__470 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_137, __pyx_int_137); if (unlikely(!__pyx_tuple__470)) __PYX_ERR(2, 613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__470);
  __Pyx_GIVEREF(__pyx_tuple__470);

  /* "include/color_dict.pxi":614
 *     'snow3' : (205, 201, 201),
 *     'snow4' : (139, 137, 137),
 *     'springgreen' : (0, 255, 127),             # <<<<<<<<<<<<<<
 *     'springgreen1' : (0, 255, 127),
 *     'springgreen2' : (0, 238, 118),
 */
  __pyx_tuple__471 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_255, __pyx_int_127); if (unlikely(!__pyx_tuple__471)) __PYX_ERR(2, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__471);
  __Pyx_GIVEREF(__pyx_tuple__471);

  /* "include/color_dict.pxi":616
 *     'springgreen' : (0, 255, 127),
 *     'springgreen1' : (0, 255, 127),
 *     'springgreen2' : (0, 238, 118),             # <<<<<<<<<<<<<<
 *     'springgreen3' : (0, 205, 102),
 *     'springgreen4' : (0, 139, 69),
 */
  __pyx_tuple__472 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_238, __pyx_int_118); if (unlikely(!__pyx_tuple__472)) __PYX_ERR(2, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__472);
  __Pyx_GIVEREF(__pyx_tuple__472);

  /* "include/color_dict.pxi":617
 *     'springgreen1' : (0, 255, 127),
 *     'springgreen2' : (0, 238, 118),
 *     'springgreen3' : (0, 205, 102),             # <<<<<<<<<<<<<<
 *     'springgreen4' : (0, 139, 69),
 *     'steelblue' : (70, 130, 180),
 */
  __pyx_tuple__473 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_205, __pyx_int_102); if (unlikely(!__pyx_tuple__473)) __PYX_ERR(2, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__473);
  __Pyx_GIVEREF(__pyx_tuple__473);

  /* "include/color_dict.pxi":618
 *     'springgreen2' : (0, 238, 118),
 *     'springgreen3' : (0, 205, 102),
 *     'springgreen4' : (0, 139, 69),             # <<<<<<<<<<<<<<
 *     'steelblue' : (70, 130, 180),
 *     'steelblue1' : (99, 184, 255),
 */
  __pyx_tuple__474 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_139, __pyx_int_69); if (unlikely(!__pyx_tuple__474)) __PYX_ERR(2, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__474);
  __Pyx_GIVEREF(__pyx_tuple__474);

  /* "include/color_dict.pxi":619
 *     'springgreen3' : (0, 205, 102),
 *     'springgreen4' : (0, 139, 69),
 *     'steelblue' : (70, 130, 180),             # <<<<<<<<<<<<<<
 *     'steelblue1' : (99, 184, 255),
 *     'steelblue2' : (92, 172, 238),
 */
  __pyx_tuple__475 = PyTuple_Pack(3, __pyx_int_70, __pyx_int_130, __pyx_int_180); if (unlikely(!__pyx_tuple__475)) __PYX_ERR(2, 619, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__475);
  __Pyx_GIVEREF(__pyx_tuple__475);

  /* "include/color_dict.pxi":620
 *     'springgreen4' : (0, 139, 69),
 *     'steelblue' : (70, 130, 180),
 *     'steelblue1' : (99, 184, 255),             # <<<<<<<<<<<<<<
 *     'steelblue2' : (92, 172, 238),
 *     'steelblue3' : (79, 148, 205),
 */
  __pyx_tuple__476 = PyTuple_Pack(3, __pyx_int_99, __pyx_int_184, __pyx_int_255); if (unlikely(!__pyx_tuple__476)) __PYX_ERR(2, 620, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__476);
  __Pyx_GIVEREF(__pyx_tuple__476);

  /* "include/color_dict.pxi":621
 *     'steelblue' : (70, 130, 180),
 *     'steelblue1' : (99, 184, 255),
 *     'steelblue2' : (92, 172, 238),             # <<<<<<<<<<<<<<
 *     'steelblue3' : (79, 148, 205),
 *     'steelblue4' : (54, 100, 139),
 */
  __pyx_tuple__477 = PyTuple_Pack(3, __pyx_int_92, __pyx_int_172, __pyx_int_238); if (unlikely(!__pyx_tuple__477)) __PYX_ERR(2, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__477);
  __Pyx_GIVEREF(__pyx_tuple__477);

  /* "include/color_dict.pxi":622
 *     'steelblue1' : (99, 184, 255),
 *     'steelblue2' : (92, 172, 238),
 *     'steelblue3' : (79, 148, 205),             # <<<<<<<<<<<<<<
 *     'steelblue4' : (54, 100, 139),
 *     'tan' : (210, 180, 140),
 */
  __pyx_tuple__478 = PyTuple_Pack(3, __pyx_int_79, __pyx_int_148, __pyx_int_205); if (unlikely(!__pyx_tuple__478)) __PYX_ERR(2, 622, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__478);
  __Pyx_GIVEREF(__pyx_tuple__478);

  /* "include/color_dict.pxi":623
 *     'steelblue2' : (92, 172, 238),
 *     'steelblue3' : (79, 148, 205),
 *     'steelblue4' : (54, 100, 139),             # <<<<<<<<<<<<<<
 *     'tan' : (210, 180, 140),
 *     'tan1' : (255, 165, 79),
 */
  __pyx_tuple__479 = PyTuple_Pack(3, __pyx_int_54, __pyx_int_100, __pyx_int_139); if (unlikely(!__pyx_tuple__479)) __PYX_ERR(2, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__479);
  __Pyx_GIVEREF(__pyx_tuple__479);

  /* "include/color_dict.pxi":624
 *     'steelblue3' : (79, 148, 205),
 *     'steelblue4' : (54, 100, 139),
 *     'tan' : (210, 180, 140),             # <<<<<<<<<<<<<<
 *     'tan1' : (255, 165, 79),
 *     'tan2' : (238, 154, 73),
 */
  __pyx_tuple__480 = PyTuple_Pack(3, __pyx_int_210, __pyx_int_180, __pyx_int_140); if (unlikely(!__pyx_tuple__480)) __PYX_ERR(2, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__480);
  __Pyx_GIVEREF(__pyx_tuple__480);

  /* "include/color_dict.pxi":625
 *     'steelblue4' : (54, 100, 139),
 *     'tan' : (210, 180, 140),
 *     'tan1' : (255, 165, 79),             # <<<<<<<<<<<<<<
 *     'tan2' : (238, 154, 73),
 *     'tan3' : (205, 133, 63),
 */
  __pyx_tuple__481 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_165, __pyx_int_79); if (unlikely(!__pyx_tuple__481)) __PYX_ERR(2, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__481);
  __Pyx_GIVEREF(__pyx_tuple__481);

  /* "include/color_dict.pxi":626
 *     'tan' : (210, 180, 140),
 *     'tan1' : (255, 165, 79),
 *     'tan2' : (238, 154, 73),             # <<<<<<<<<<<<<<
 *     'tan3' : (205, 133, 63),
 *     'tan4' : (139, 90, 43),
 */
  __pyx_tuple__482 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_154, __pyx_int_73); if (unlikely(!__pyx_tuple__482)) __PYX_ERR(2, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__482);
  __Pyx_GIVEREF(__pyx_tuple__482);

  /* "include/color_dict.pxi":628
 *     'tan2' : (238, 154, 73),
 *     'tan3' : (205, 133, 63),
 *     'tan4' : (139, 90, 43),             # <<<<<<<<<<<<<<
 *     'thistle' : (216, 191, 216),
 *     'thistle1' : (255, 225, 255),
 */
  __pyx_tuple__483 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_90, __pyx_int_43); if (unlikely(!__pyx_tuple__483)) __PYX_ERR(2, 628, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__483);
  __Pyx_GIVEREF(__pyx_tuple__483);

  /* "include/color_dict.pxi":629
 *     'tan3' : (205, 133, 63),
 *     'tan4' : (139, 90, 43),
 *     'thistle' : (216, 191, 216),             # <<<<<<<<<<<<<<
 *     'thistle1' : (255, 225, 255),
 *     'thistle2' : (238, 210, 238),
 */
  __pyx_tuple__484 = PyTuple_Pack(3, __pyx_int_216, __pyx_int_191, __pyx_int_216); if (unlikely(!__pyx_tuple__484)) __PYX_ERR(2, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__484);
  __Pyx_GIVEREF(__pyx_tuple__484);

  /* "include/color_dict.pxi":630
 *     'tan4' : (139, 90, 43),
 *     'thistle' : (216, 191, 216),
 *     'thistle1' : (255, 225, 255),             # <<<<<<<<<<<<<<
 *     'thistle2' : (238, 210, 238),
 *     'thistle3' : (205, 181, 205),
 */
  __pyx_tuple__485 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_225, __pyx_int_255); if (unlikely(!__pyx_tuple__485)) __PYX_ERR(2, 630, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__485);
  __Pyx_GIVEREF(__pyx_tuple__485);

  /* "include/color_dict.pxi":631
 *     'thistle' : (216, 191, 216),
 *     'thistle1' : (255, 225, 255),
 *     'thistle2' : (238, 210, 238),             # <<<<<<<<<<<<<<
 *     'thistle3' : (205, 181, 205),
 *     'thistle4' : (139, 123, 139),
 */
  __pyx_tuple__486 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_210, __pyx_int_238); if (unlikely(!__pyx_tuple__486)) __PYX_ERR(2, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__486);
  __Pyx_GIVEREF(__pyx_tuple__486);

  /* "include/color_dict.pxi":632
 *     'thistle1' : (255, 225, 255),
 *     'thistle2' : (238, 210, 238),
 *     'thistle3' : (205, 181, 205),             # <<<<<<<<<<<<<<
 *     'thistle4' : (139, 123, 139),
 *     'tomato' : (255, 99, 71),
 */
  __pyx_tuple__487 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_181, __pyx_int_205); if (unlikely(!__pyx_tuple__487)) __PYX_ERR(2, 632, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__487);
  __Pyx_GIVEREF(__pyx_tuple__487);

  /* "include/color_dict.pxi":633
 *     'thistle2' : (238, 210, 238),
 *     'thistle3' : (205, 181, 205),
 *     'thistle4' : (139, 123, 139),             # <<<<<<<<<<<<<<
 *     'tomato' : (255, 99, 71),
 *     'tomato1' : (255, 99, 71),
 */
  __pyx_tuple__488 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_123, __pyx_int_139); if (unlikely(!__pyx_tuple__488)) __PYX_ERR(2, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__488);
  __Pyx_GIVEREF(__pyx_tuple__488);

  /* "include/color_dict.pxi":634
 *     'thistle3' : (205, 181, 205),
 *     'thistle4' : (139, 123, 139),
 *     'tomato' : (255, 99, 71),             # <<<<<<<<<<<<<<
 *     'tomato1' : (255, 99, 71),
 *     'tomato2' : (238, 92, 66),
 */
  __pyx_tuple__489 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_99, __pyx_int_71); if (unlikely(!__pyx_tuple__489)) __PYX_ERR(2, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__489);
  __Pyx_GIVEREF(__pyx_tuple__489);

  /* "include/color_dict.pxi":636
 *     'tomato' : (255, 99, 71),
 *     'tomato1' : (255, 99, 71),
 *     'tomato2' : (238, 92, 66),             # <<<<<<<<<<<<<<
 *     'tomato3' : (205, 79, 57),
 *     'tomato4' : (139, 54, 38),
 */
  __pyx_tuple__490 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_92, __pyx_int_66); if (unlikely(!__pyx_tuple__490)) __PYX_ERR(2, 636, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__490);
  __Pyx_GIVEREF(__pyx_tuple__490);

  /* "include/color_dict.pxi":637
 *     'tomato1' : (255, 99, 71),
 *     'tomato2' : (238, 92, 66),
 *     'tomato3' : (205, 79, 57),             # <<<<<<<<<<<<<<
 *     'tomato4' : (139, 54, 38),
 *     'turquoise' : (64, 224, 208),
 */
  __pyx_tuple__491 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_79, __pyx_int_57); if (unlikely(!__pyx_tuple__491)) __PYX_ERR(2, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__491);
  __Pyx_GIVEREF(__pyx_tuple__491);

  /* "include/color_dict.pxi":638
 *     'tomato2' : (238, 92, 66),
 *     'tomato3' : (205, 79, 57),
 *     'tomato4' : (139, 54, 38),             # <<<<<<<<<<<<<<
 *     'turquoise' : (64, 224, 208),
 *     'turquoise1' : (0, 245, 255),
 */
  __pyx_tuple__492 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_54, __pyx_int_38); if (unlikely(!__pyx_tuple__492)) __PYX_ERR(2, 638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__492);
  __Pyx_GIVEREF(__pyx_tuple__492);

  /* "include/color_dict.pxi":639
 *     'tomato3' : (205, 79, 57),
 *     'tomato4' : (139, 54, 38),
 *     'turquoise' : (64, 224, 208),             # <<<<<<<<<<<<<<
 *     'turquoise1' : (0, 245, 255),
 *     'turquoise2' : (0, 229, 238),
 */
  __pyx_tuple__493 = PyTuple_Pack(3, __pyx_int_64, __pyx_int_224, __pyx_int_208); if (unlikely(!__pyx_tuple__493)) __PYX_ERR(2, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__493);
  __Pyx_GIVEREF(__pyx_tuple__493);

  /* "include/color_dict.pxi":640
 *     'tomato4' : (139, 54, 38),
 *     'turquoise' : (64, 224, 208),
 *     'turquoise1' : (0, 245, 255),             # <<<<<<<<<<<<<<
 *     'turquoise2' : (0, 229, 238),
 *     'turquoise3' : (0, 197, 205),
 */
  __pyx_tuple__494 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_245, __pyx_int_255); if (unlikely(!__pyx_tuple__494)) __PYX_ERR(2, 640, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__494);
  __Pyx_GIVEREF(__pyx_tuple__494);

  /* "include/color_dict.pxi":641
 *     'turquoise' : (64, 224, 208),
 *     'turquoise1' : (0, 245, 255),
 *     'turquoise2' : (0, 229, 238),             # <<<<<<<<<<<<<<
 *     'turquoise3' : (0, 197, 205),
 *     'turquoise4' : (0, 134, 139),
 */
  __pyx_tuple__495 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_229, __pyx_int_238); if (unlikely(!__pyx_tuple__495)) __PYX_ERR(2, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__495);
  __Pyx_GIVEREF(__pyx_tuple__495);

  /* "include/color_dict.pxi":642
 *     'turquoise1' : (0, 245, 255),
 *     'turquoise2' : (0, 229, 238),
 *     'turquoise3' : (0, 197, 205),             # <<<<<<<<<<<<<<
 *     'turquoise4' : (0, 134, 139),
 *     'violet' : (238, 130, 238),
 */
  __pyx_tuple__496 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_197, __pyx_int_205); if (unlikely(!__pyx_tuple__496)) __PYX_ERR(2, 642, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__496);
  __Pyx_GIVEREF(__pyx_tuple__496);

  /* "include/color_dict.pxi":643
 *     'turquoise2' : (0, 229, 238),
 *     'turquoise3' : (0, 197, 205),
 *     'turquoise4' : (0, 134, 139),             # <<<<<<<<<<<<<<
 *     'violet' : (238, 130, 238),
 *     'violetred' : (208, 32, 144),
 */
  __pyx_tuple__497 = PyTuple_Pack(3, __pyx_int_0, __pyx_int_134, __pyx_int_139); if (unlikely(!__pyx_tuple__497)) __PYX_ERR(2, 643, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__497);
  __Pyx_GIVEREF(__pyx_tuple__497);

  /* "include/color_dict.pxi":644
 *     'turquoise3' : (0, 197, 205),
 *     'turquoise4' : (0, 134, 139),
 *     'violet' : (238, 130, 238),             # <<<<<<<<<<<<<<
 *     'violetred' : (208, 32, 144),
 *     'violetred1' : (255, 62, 150),
 */
  __pyx_tuple__498 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_130, __pyx_int_238); if (unlikely(!__pyx_tuple__498)) __PYX_ERR(2, 644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__498);
  __Pyx_GIVEREF(__pyx_tuple__498);

  /* "include/color_dict.pxi":645
 *     'turquoise4' : (0, 134, 139),
 *     'violet' : (238, 130, 238),
 *     'violetred' : (208, 32, 144),             # <<<<<<<<<<<<<<
 *     'violetred1' : (255, 62, 150),
 *     'violetred2' : (238, 58, 140),
 */
  __pyx_tuple__499 = PyTuple_Pack(3, __pyx_int_208, __pyx_int_32, __pyx_int_144); if (unlikely(!__pyx_tuple__499)) __PYX_ERR(2, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__499);
  __Pyx_GIVEREF(__pyx_tuple__499);

  /* "include/color_dict.pxi":646
 *     'violet' : (238, 130, 238),
 *     'violetred' : (208, 32, 144),
 *     'violetred1' : (255, 62, 150),             # <<<<<<<<<<<<<<
 *     'violetred2' : (238, 58, 140),
 *     'violetred3' : (205, 50, 120),
 */
  __pyx_tuple__500 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_62, __pyx_int_150); if (unlikely(!__pyx_tuple__500)) __PYX_ERR(2, 646, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__500);
  __Pyx_GIVEREF(__pyx_tuple__500);

  /* "include/color_dict.pxi":647
 *     'violetred' : (208, 32, 144),
 *     'violetred1' : (255, 62, 150),
 *     'violetred2' : (238, 58, 140),             # <<<<<<<<<<<<<<
 *     'violetred3' : (205, 50, 120),
 *     'violetred4' : (139, 34, 82),
 */
  __pyx_tuple__501 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_58, __pyx_int_140); if (unlikely(!__pyx_tuple__501)) __PYX_ERR(2, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__501);
  __Pyx_GIVEREF(__pyx_tuple__501);

  /* "include/color_dict.pxi":648
 *     'violetred1' : (255, 62, 150),
 *     'violetred2' : (238, 58, 140),
 *     'violetred3' : (205, 50, 120),             # <<<<<<<<<<<<<<
 *     'violetred4' : (139, 34, 82),
 *     'wheat' : (245, 222, 179),
 */
  __pyx_tuple__502 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_50, __pyx_int_120); if (unlikely(!__pyx_tuple__502)) __PYX_ERR(2, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__502);
  __Pyx_GIVEREF(__pyx_tuple__502);

  /* "include/color_dict.pxi":649
 *     'violetred2' : (238, 58, 140),
 *     'violetred3' : (205, 50, 120),
 *     'violetred4' : (139, 34, 82),             # <<<<<<<<<<<<<<
 *     'wheat' : (245, 222, 179),
 *     'wheat1' : (255, 231, 186),
 */
  __pyx_tuple__503 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_34, __pyx_int_82); if (unlikely(!__pyx_tuple__503)) __PYX_ERR(2, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__503);
  __Pyx_GIVEREF(__pyx_tuple__503);

  /* "include/color_dict.pxi":650
 *     'violetred3' : (205, 50, 120),
 *     'violetred4' : (139, 34, 82),
 *     'wheat' : (245, 222, 179),             # <<<<<<<<<<<<<<
 *     'wheat1' : (255, 231, 186),
 *     'wheat2' : (238, 216, 174),
 */
  __pyx_tuple__504 = PyTuple_Pack(3, __pyx_int_245, __pyx_int_222, __pyx_int_179); if (unlikely(!__pyx_tuple__504)) __PYX_ERR(2, 650, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__504);
  __Pyx_GIVEREF(__pyx_tuple__504);

  /* "include/color_dict.pxi":651
 *     'violetred4' : (139, 34, 82),
 *     'wheat' : (245, 222, 179),
 *     'wheat1' : (255, 231, 186),             # <<<<<<<<<<<<<<
 *     'wheat2' : (238, 216, 174),
 *     'wheat3' : (205, 186, 150),
 */
  __pyx_tuple__505 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_231, __pyx_int_186); if (unlikely(!__pyx_tuple__505)) __PYX_ERR(2, 651, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__505);
  __Pyx_GIVEREF(__pyx_tuple__505);

  /* "include/color_dict.pxi":652
 *     'wheat' : (245, 222, 179),
 *     'wheat1' : (255, 231, 186),
 *     'wheat2' : (238, 216, 174),             # <<<<<<<<<<<<<<
 *     'wheat3' : (205, 186, 150),
 *     'wheat4' : (139, 126, 102),
 */
  __pyx_tuple__506 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_216, __pyx_int_174); if (unlikely(!__pyx_tuple__506)) __PYX_ERR(2, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__506);
  __Pyx_GIVEREF(__pyx_tuple__506);

  /* "include/color_dict.pxi":653
 *     'wheat1' : (255, 231, 186),
 *     'wheat2' : (238, 216, 174),
 *     'wheat3' : (205, 186, 150),             # <<<<<<<<<<<<<<
 *     'wheat4' : (139, 126, 102),
 *     'white' : (255, 255, 255),
 */
  __pyx_tuple__507 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_186, __pyx_int_150); if (unlikely(!__pyx_tuple__507)) __PYX_ERR(2, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__507);
  __Pyx_GIVEREF(__pyx_tuple__507);

  /* "include/color_dict.pxi":654
 *     'wheat2' : (238, 216, 174),
 *     'wheat3' : (205, 186, 150),
 *     'wheat4' : (139, 126, 102),             # <<<<<<<<<<<<<<
 *     'white' : (255, 255, 255),
 *     'whitesmoke' : (245, 245, 245),
 */
  __pyx_tuple__508 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_126, __pyx_int_102); if (unlikely(!__pyx_tuple__508)) __PYX_ERR(2, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__508);
  __Pyx_GIVEREF(__pyx_tuple__508);

  /* "include/color_dict.pxi":657
 *     'white' : (255, 255, 255),
 *     'whitesmoke' : (245, 245, 245),
 *     'yellow' : (255, 255, 0),             # <<<<<<<<<<<<<<
 *     'yellow1' : (255, 255, 0),
 *     'yellow2' : (238, 238, 0),
 */
  __pyx_tuple__509 = PyTuple_Pack(3, __pyx_int_255, __pyx_int_255, __pyx_int_0); if (unlikely(!__pyx_tuple__509)) __PYX_ERR(2, 657, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__509);
  __Pyx_GIVEREF(__pyx_tuple__509);

  /* "include/color_dict.pxi":659
 *     'yellow' : (255, 255, 0),
 *     'yellow1' : (255, 255, 0),
 *     'yellow2' : (238, 238, 0),             # <<<<<<<<<<<<<<
 *     'yellow3' : (205, 205, 0),
 *     'yellow4' : (139, 139, 0),
 */
  __pyx_tuple__510 = PyTuple_Pack(3, __pyx_int_238, __pyx_int_238, __pyx_int_0); if (unlikely(!__pyx_tuple__510)) __PYX_ERR(2, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__510);
  __Pyx_GIVEREF(__pyx_tuple__510);

  /* "include/color_dict.pxi":660
 *     'yellow1' : (255, 255, 0),
 *     'yellow2' : (238, 238, 0),
 *     'yellow3' : (205, 205, 0),             # <<<<<<<<<<<<<<
 *     'yellow4' : (139, 139, 0),
 *     'yellowgreen' : (154, 205, 50),
 */
  __pyx_tuple__511 = PyTuple_Pack(3, __pyx_int_205, __pyx_int_205, __pyx_int_0); if (unlikely(!__pyx_tuple__511)) __PYX_ERR(2, 660, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__511);
  __Pyx_GIVEREF(__pyx_tuple__511);

  /* "include/color_dict.pxi":661
 *     'yellow2' : (238, 238, 0),
 *     'yellow3' : (205, 205, 0),
 *     'yellow4' : (139, 139, 0),             # <<<<<<<<<<<<<<
 *     'yellowgreen' : (154, 205, 50),
 * }
 */
  __pyx_tuple__512 = PyTuple_Pack(3, __pyx_int_139, __pyx_int_139, __pyx_int_0); if (unlikely(!__pyx_tuple__512)) __PYX_ERR(2, 661, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__512);
  __Pyx_GIVEREF(__pyx_tuple__512);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_255_0 = PyFloat_FromDouble(255.0); if (unlikely(!__pyx_float_255_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_360_0 = PyFloat_FromDouble(360.0); if (unlikely(!__pyx_float_360_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_7 = PyInt_FromLong(7); if (unlikely(!__pyx_int_7)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_8 = PyInt_FromLong(8); if (unlikely(!__pyx_int_8)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_10 = PyInt_FromLong(10); if (unlikely(!__pyx_int_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_11 = PyInt_FromLong(11); if (unlikely(!__pyx_int_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_12 = PyInt_FromLong(12); if (unlikely(!__pyx_int_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_13 = PyInt_FromLong(13); if (unlikely(!__pyx_int_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_14 = PyInt_FromLong(14); if (unlikely(!__pyx_int_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_15 = PyInt_FromLong(15); if (unlikely(!__pyx_int_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16 = PyInt_FromLong(16); if (unlikely(!__pyx_int_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_18 = PyInt_FromLong(18); if (unlikely(!__pyx_int_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_19 = PyInt_FromLong(19); if (unlikely(!__pyx_int_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_20 = PyInt_FromLong(20); if (unlikely(!__pyx_int_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_21 = PyInt_FromLong(21); if (unlikely(!__pyx_int_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_23 = PyInt_FromLong(23); if (unlikely(!__pyx_int_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_24 = PyInt_FromLong(24); if (unlikely(!__pyx_int_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_25 = PyInt_FromLong(25); if (unlikely(!__pyx_int_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_26 = PyInt_FromLong(26); if (unlikely(!__pyx_int_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_28 = PyInt_FromLong(28); if (unlikely(!__pyx_int_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_29 = PyInt_FromLong(29); if (unlikely(!__pyx_int_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_30 = PyInt_FromLong(30); if (unlikely(!__pyx_int_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_31 = PyInt_FromLong(31); if (unlikely(!__pyx_int_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_32 = PyInt_FromLong(32); if (unlikely(!__pyx_int_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_33 = PyInt_FromLong(33); if (unlikely(!__pyx_int_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_34 = PyInt_FromLong(34); if (unlikely(!__pyx_int_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_35 = PyInt_FromLong(35); if (unlikely(!__pyx_int_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_36 = PyInt_FromLong(36); if (unlikely(!__pyx_int_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_37 = PyInt_FromLong(37); if (unlikely(!__pyx_int_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_38 = PyInt_FromLong(38); if (unlikely(!__pyx_int_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_39 = PyInt_FromLong(39); if (unlikely(!__pyx_int_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_41 = PyInt_FromLong(41); if (unlikely(!__pyx_int_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_42 = PyInt_FromLong(42); if (unlikely(!__pyx_int_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_43 = PyInt_FromLong(43); if (unlikely(!__pyx_int_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_44 = PyInt_FromLong(44); if (unlikely(!__pyx_int_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_45 = PyInt_FromLong(45); if (unlikely(!__pyx_int_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_46 = PyInt_FromLong(46); if (unlikely(!__pyx_int_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_47 = PyInt_FromLong(47); if (unlikely(!__pyx_int_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_48 = PyInt_FromLong(48); if (unlikely(!__pyx_int_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_50 = PyInt_FromLong(50); if (unlikely(!__pyx_int_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_51 = PyInt_FromLong(51); if (unlikely(!__pyx_int_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_52 = PyInt_FromLong(52); if (unlikely(!__pyx_int_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_54 = PyInt_FromLong(54); if (unlikely(!__pyx_int_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_55 = PyInt_FromLong(55); if (unlikely(!__pyx_int_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_56 = PyInt_FromLong(56); if (unlikely(!__pyx_int_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_57 = PyInt_FromLong(57); if (unlikely(!__pyx_int_57)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_58 = PyInt_FromLong(58); if (unlikely(!__pyx_int_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_59 = PyInt_FromLong(59); if (unlikely(!__pyx_int_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_60 = PyInt_FromLong(60); if (unlikely(!__pyx_int_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_61 = PyInt_FromLong(61); if (unlikely(!__pyx_int_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_62 = PyInt_FromLong(62); if (unlikely(!__pyx_int_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_63 = PyInt_FromLong(63); if (unlikely(!__pyx_int_63)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_64 = PyInt_FromLong(64); if (unlikely(!__pyx_int_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_65 = PyInt_FromLong(65); if (unlikely(!__pyx_int_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_66 = PyInt_FromLong(66); if (unlikely(!__pyx_int_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_67 = PyInt_FromLong(67); if (unlikely(!__pyx_int_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_69 = PyInt_FromLong(69); if (unlikely(!__pyx_int_69)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_70 = PyInt_FromLong(70); if (unlikely(!__pyx_int_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_71 = PyInt_FromLong(71); if (unlikely(!__pyx_int_71)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_72 = PyInt_FromLong(72); if (unlikely(!__pyx_int_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_73 = PyInt_FromLong(73); if (unlikely(!__pyx_int_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_74 = PyInt_FromLong(74); if (unlikely(!__pyx_int_74)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_76 = PyInt_FromLong(76); if (unlikely(!__pyx_int_76)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_77 = PyInt_FromLong(77); if (unlikely(!__pyx_int_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_78 = PyInt_FromLong(78); if (unlikely(!__pyx_int_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_79 = PyInt_FromLong(79); if (unlikely(!__pyx_int_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_80 = PyInt_FromLong(80); if (unlikely(!__pyx_int_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_81 = PyInt_FromLong(81); if (unlikely(!__pyx_int_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_82 = PyInt_FromLong(82); if (unlikely(!__pyx_int_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_83 = PyInt_FromLong(83); if (unlikely(!__pyx_int_83)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_84 = PyInt_FromLong(84); if (unlikely(!__pyx_int_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_85 = PyInt_FromLong(85); if (unlikely(!__pyx_int_85)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_86 = PyInt_FromLong(86); if (unlikely(!__pyx_int_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_87 = PyInt_FromLong(87); if (unlikely(!__pyx_int_87)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_89 = PyInt_FromLong(89); if (unlikely(!__pyx_int_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_90 = PyInt_FromLong(90); if (unlikely(!__pyx_int_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_91 = PyInt_FromLong(91); if (unlikely(!__pyx_int_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_92 = PyInt_FromLong(92); if (unlikely(!__pyx_int_92)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_93 = PyInt_FromLong(93); if (unlikely(!__pyx_int_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_94 = PyInt_FromLong(94); if (unlikely(!__pyx_int_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_95 = PyInt_FromLong(95); if (unlikely(!__pyx_int_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_96 = PyInt_FromLong(96); if (unlikely(!__pyx_int_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_97 = PyInt_FromLong(97); if (unlikely(!__pyx_int_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_98 = PyInt_FromLong(98); if (unlikely(!__pyx_int_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_99 = PyInt_FromLong(99); if (unlikely(!__pyx_int_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_100 = PyInt_FromLong(100); if (unlikely(!__pyx_int_100)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_101 = PyInt_FromLong(101); if (unlikely(!__pyx_int_101)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_102 = PyInt_FromLong(102); if (unlikely(!__pyx_int_102)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_103 = PyInt_FromLong(103); if (unlikely(!__pyx_int_103)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_104 = PyInt_FromLong(104); if (unlikely(!__pyx_int_104)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_105 = PyInt_FromLong(105); if (unlikely(!__pyx_int_105)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_106 = PyInt_FromLong(106); if (unlikely(!__pyx_int_106)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_107 = PyInt_FromLong(107); if (unlikely(!__pyx_int_107)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_108 = PyInt_FromLong(108); if (unlikely(!__pyx_int_108)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_110 = PyInt_FromLong(110); if (unlikely(!__pyx_int_110)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_111 = PyInt_FromLong(111); if (unlikely(!__pyx_int_111)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_112 = PyInt_FromLong(112); if (unlikely(!__pyx_int_112)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_113 = PyInt_FromLong(113); if (unlikely(!__pyx_int_113)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_114 = PyInt_FromLong(114); if (unlikely(!__pyx_int_114)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_115 = PyInt_FromLong(115); if (unlikely(!__pyx_int_115)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_116 = PyInt_FromLong(116); if (unlikely(!__pyx_int_116)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_117 = PyInt_FromLong(117); if (unlikely(!__pyx_int_117)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_118 = PyInt_FromLong(118); if (unlikely(!__pyx_int_118)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_119 = PyInt_FromLong(119); if (unlikely(!__pyx_int_119)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_120 = PyInt_FromLong(120); if (unlikely(!__pyx_int_120)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_121 = PyInt_FromLong(121); if (unlikely(!__pyx_int_121)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_122 = PyInt_FromLong(122); if (unlikely(!__pyx_int_122)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_123 = PyInt_FromLong(123); if (unlikely(!__pyx_int_123)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_124 = PyInt_FromLong(124); if (unlikely(!__pyx_int_124)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_125 = PyInt_FromLong(125); if (unlikely(!__pyx_int_125)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_126 = PyInt_FromLong(126); if (unlikely(!__pyx_int_126)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_127 = PyInt_FromLong(127); if (unlikely(!__pyx_int_127)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_128 = PyInt_FromLong(128); if (unlikely(!__pyx_int_128)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_129 = PyInt_FromLong(129); if (unlikely(!__pyx_int_129)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_130 = PyInt_FromLong(130); if (unlikely(!__pyx_int_130)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_131 = PyInt_FromLong(131); if (unlikely(!__pyx_int_131)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_132 = PyInt_FromLong(132); if (unlikely(!__pyx_int_132)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_133 = PyInt_FromLong(133); if (unlikely(!__pyx_int_133)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_134 = PyInt_FromLong(134); if (unlikely(!__pyx_int_134)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_135 = PyInt_FromLong(135); if (unlikely(!__pyx_int_135)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_136 = PyInt_FromLong(136); if (unlikely(!__pyx_int_136)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_137 = PyInt_FromLong(137); if (unlikely(!__pyx_int_137)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_138 = PyInt_FromLong(138); if (unlikely(!__pyx_int_138)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_139 = PyInt_FromLong(139); if (unlikely(!__pyx_int_139)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_140 = PyInt_FromLong(140); if (unlikely(!__pyx_int_140)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_141 = PyInt_FromLong(141); if (unlikely(!__pyx_int_141)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_142 = PyInt_FromLong(142); if (unlikely(!__pyx_int_142)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_143 = PyInt_FromLong(143); if (unlikely(!__pyx_int_143)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_144 = PyInt_FromLong(144); if (unlikely(!__pyx_int_144)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_145 = PyInt_FromLong(145); if (unlikely(!__pyx_int_145)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_147 = PyInt_FromLong(147); if (unlikely(!__pyx_int_147)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_148 = PyInt_FromLong(148); if (unlikely(!__pyx_int_148)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_149 = PyInt_FromLong(149); if (unlikely(!__pyx_int_149)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_150 = PyInt_FromLong(150); if (unlikely(!__pyx_int_150)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_151 = PyInt_FromLong(151); if (unlikely(!__pyx_int_151)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_152 = PyInt_FromLong(152); if (unlikely(!__pyx_int_152)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_153 = PyInt_FromLong(153); if (unlikely(!__pyx_int_153)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_154 = PyInt_FromLong(154); if (unlikely(!__pyx_int_154)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_155 = PyInt_FromLong(155); if (unlikely(!__pyx_int_155)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_156 = PyInt_FromLong(156); if (unlikely(!__pyx_int_156)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_158 = PyInt_FromLong(158); if (unlikely(!__pyx_int_158)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_159 = PyInt_FromLong(159); if (unlikely(!__pyx_int_159)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_160 = PyInt_FromLong(160); if (unlikely(!__pyx_int_160)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_161 = PyInt_FromLong(161); if (unlikely(!__pyx_int_161)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_162 = PyInt_FromLong(162); if (unlikely(!__pyx_int_162)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_163 = PyInt_FromLong(163); if (unlikely(!__pyx_int_163)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_164 = PyInt_FromLong(164); if (unlikely(!__pyx_int_164)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_165 = PyInt_FromLong(165); if (unlikely(!__pyx_int_165)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_166 = PyInt_FromLong(166); if (unlikely(!__pyx_int_166)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_167 = PyInt_FromLong(167); if (unlikely(!__pyx_int_167)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_168 = PyInt_FromLong(168); if (unlikely(!__pyx_int_168)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_169 = PyInt_FromLong(169); if (unlikely(!__pyx_int_169)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_170 = PyInt_FromLong(170); if (unlikely(!__pyx_int_170)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_171 = PyInt_FromLong(171); if (unlikely(!__pyx_int_171)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_172 = PyInt_FromLong(172); if (unlikely(!__pyx_int_172)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_173 = PyInt_FromLong(173); if (unlikely(!__pyx_int_173)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_174 = PyInt_FromLong(174); if (unlikely(!__pyx_int_174)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_175 = PyInt_FromLong(175); if (unlikely(!__pyx_int_175)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_176 = PyInt_FromLong(176); if (unlikely(!__pyx_int_176)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_177 = PyInt_FromLong(177); if (unlikely(!__pyx_int_177)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_178 = PyInt_FromLong(178); if (unlikely(!__pyx_int_178)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_179 = PyInt_FromLong(179); if (unlikely(!__pyx_int_179)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_180 = PyInt_FromLong(180); if (unlikely(!__pyx_int_180)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_181 = PyInt_FromLong(181); if (unlikely(!__pyx_int_181)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_182 = PyInt_FromLong(182); if (unlikely(!__pyx_int_182)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_183 = PyInt_FromLong(183); if (unlikely(!__pyx_int_183)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_184 = PyInt_FromLong(184); if (unlikely(!__pyx_int_184)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_185 = PyInt_FromLong(185); if (unlikely(!__pyx_int_185)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_186 = PyInt_FromLong(186); if (unlikely(!__pyx_int_186)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_187 = PyInt_FromLong(187); if (unlikely(!__pyx_int_187)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_188 = PyInt_FromLong(188); if (unlikely(!__pyx_int_188)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_189 = PyInt_FromLong(189); if (unlikely(!__pyx_int_189)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_190 = PyInt_FromLong(190); if (unlikely(!__pyx_int_190)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_191 = PyInt_FromLong(191); if (unlikely(!__pyx_int_191)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_192 = PyInt_FromLong(192); if (unlikely(!__pyx_int_192)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_193 = PyInt_FromLong(193); if (unlikely(!__pyx_int_193)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_194 = PyInt_FromLong(194); if (unlikely(!__pyx_int_194)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_196 = PyInt_FromLong(196); if (unlikely(!__pyx_int_196)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_197 = PyInt_FromLong(197); if (unlikely(!__pyx_int_197)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_198 = PyInt_FromLong(198); if (unlikely(!__pyx_int_198)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_199 = PyInt_FromLong(199); if (unlikely(!__pyx_int_199)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_200 = PyInt_FromLong(200); if (unlikely(!__pyx_int_200)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_201 = PyInt_FromLong(201); if (unlikely(!__pyx_int_201)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_202 = PyInt_FromLong(202); if (unlikely(!__pyx_int_202)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_203 = PyInt_FromLong(203); if (unlikely(!__pyx_int_203)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_204 = PyInt_FromLong(204); if (unlikely(!__pyx_int_204)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_205 = PyInt_FromLong(205); if (unlikely(!__pyx_int_205)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_206 = PyInt_FromLong(206); if (unlikely(!__pyx_int_206)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_207 = PyInt_FromLong(207); if (unlikely(!__pyx_int_207)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_208 = PyInt_FromLong(208); if (unlikely(!__pyx_int_208)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_209 = PyInt_FromLong(209); if (unlikely(!__pyx_int_209)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_210 = PyInt_FromLong(210); if (unlikely(!__pyx_int_210)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_211 = PyInt_FromLong(211); if (unlikely(!__pyx_int_211)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_212 = PyInt_FromLong(212); if (unlikely(!__pyx_int_212)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_213 = PyInt_FromLong(213); if (unlikely(!__pyx_int_213)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_214 = PyInt_FromLong(214); if (unlikely(!__pyx_int_214)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_215 = PyInt_FromLong(215); if (unlikely(!__pyx_int_215)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_216 = PyInt_FromLong(216); if (unlikely(!__pyx_int_216)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_217 = PyInt_FromLong(217); if (unlikely(!__pyx_int_217)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_218 = PyInt_FromLong(218); if (unlikely(!__pyx_int_218)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_219 = PyInt_FromLong(219); if (unlikely(!__pyx_int_219)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_220 = PyInt_FromLong(220); if (unlikely(!__pyx_int_220)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_221 = PyInt_FromLong(221); if (unlikely(!__pyx_int_221)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_222 = PyInt_FromLong(222); if (unlikely(!__pyx_int_222)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_223 = PyInt_FromLong(223); if (unlikely(!__pyx_int_223)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_224 = PyInt_FromLong(224); if (unlikely(!__pyx_int_224)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_225 = PyInt_FromLong(225); if (unlikely(!__pyx_int_225)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_226 = PyInt_FromLong(226); if (unlikely(!__pyx_int_226)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_227 = PyInt_FromLong(227); if (unlikely(!__pyx_int_227)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_228 = PyInt_FromLong(228); if (unlikely(!__pyx_int_228)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_229 = PyInt_FromLong(229); if (unlikely(!__pyx_int_229)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_230 = PyInt_FromLong(230); if (unlikely(!__pyx_int_230)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_231 = PyInt_FromLong(231); if (unlikely(!__pyx_int_231)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_232 = PyInt_FromLong(232); if (unlikely(!__pyx_int_232)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_233 = PyInt_FromLong(233); if (unlikely(!__pyx_int_233)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_235 = PyInt_FromLong(235); if (unlikely(!__pyx_int_235)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_236 = PyInt_FromLong(236); if (unlikely(!__pyx_int_236)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_237 = PyInt_FromLong(237); if (unlikely(!__pyx_int_237)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_238 = PyInt_FromLong(238); if (unlikely(!__pyx_int_238)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_239 = PyInt_FromLong(239); if (unlikely(!__pyx_int_239)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_240 = PyInt_FromLong(240); if (unlikely(!__pyx_int_240)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_242 = PyInt_FromLong(242); if (unlikely(!__pyx_int_242)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_244 = PyInt_FromLong(244); if (unlikely(!__pyx_int_244)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_245 = PyInt_FromLong(245); if (unlikely(!__pyx_int_245)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_246 = PyInt_FromLong(246); if (unlikely(!__pyx_int_246)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_247 = PyInt_FromLong(247); if (unlikely(!__pyx_int_247)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_248 = PyInt_FromLong(248); if (unlikely(!__pyx_int_248)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_250 = PyInt_FromLong(250); if (unlikely(!__pyx_int_250)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_251 = PyInt_FromLong(251); if (unlikely(!__pyx_int_251)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_252 = PyInt_FromLong(252); if (unlikely(!__pyx_int_252)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_253 = PyInt_FromLong(253); if (unlikely(!__pyx_int_253)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_255 = PyInt_FromLong(255); if (unlikely(!__pyx_int_255)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_11pygame_sdl2_5color_colors = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("map_color", (void (*)(void))__pyx_f_11pygame_sdl2_5color_map_color, "Uint32 (SDL_Surface *, PyObject *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("get_color", (void (*)(void))__pyx_f_11pygame_sdl2_5color_get_color, "PyObject *(Uint32, SDL_Surface *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("to_sdl_color", (void (*)(void))__pyx_f_11pygame_sdl2_5color_to_sdl_color, "PyObject *(PyObject *, SDL_Color *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_11pygame_sdl2_5color_Color = &__pyx_vtable_11pygame_sdl2_5color_Color;
  __pyx_vtable_11pygame_sdl2_5color_Color.from_rgba = (PyObject *(*)(struct __pyx_obj_11pygame_sdl2_5color_Color *, Uint8, Uint8, Uint8, Uint8))__pyx_f_11pygame_sdl2_5color_5Color_from_rgba;
  __pyx_vtable_11pygame_sdl2_5color_Color.from_name = (PyObject *(*)(struct __pyx_obj_11pygame_sdl2_5color_Color *, PyObject *))__pyx_f_11pygame_sdl2_5color_5Color_from_name;
  __pyx_vtable_11pygame_sdl2_5color_Color.from_hex = (PyObject *(*)(struct __pyx_obj_11pygame_sdl2_5color_Color *, PyObject *))__pyx_f_11pygame_sdl2_5color_5Color_from_hex;
  if (PyType_Ready(&__pyx_type_11pygame_sdl2_5color_Color) < 0) __PYX_ERR(0, 63, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_11pygame_sdl2_5color_Color.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_11pygame_sdl2_5color_Color.tp_dictoffset && __pyx_type_11pygame_sdl2_5color_Color.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_11pygame_sdl2_5color_Color.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_11pygame_sdl2_5color_Color.tp_dict, __pyx_vtabptr_11pygame_sdl2_5color_Color) < 0) __PYX_ERR(0, 63, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Color, (PyObject *)&__pyx_type_11pygame_sdl2_5color_Color) < 0) __PYX_ERR(0, 63, __pyx_L1_error)
  if (__pyx_type_11pygame_sdl2_5color_Color.tp_weaklistoffset == 0) __pyx_type_11pygame_sdl2_5color_Color.tp_weaklistoffset = offsetof(struct __pyx_obj_11pygame_sdl2_5color_Color, __weakref__);
  __pyx_ptype_11pygame_sdl2_5color_Color = &__pyx_type_11pygame_sdl2_5color_Color;
  if (PyType_Ready(&__pyx_type_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma) < 0) __PYX_ERR(0, 452, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma.tp_dictoffset && __pyx_type_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma = &__pyx_type_11pygame_sdl2_5color___pyx_scope_struct__correct_gamma;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initcolor(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initcolor(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_color(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_color(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_color(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'color' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_color(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("color", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pygame_sdl2__color) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pygame_sdl2.color")) {
      if (unlikely(PyDict_SetItemString(modules, "pygame_sdl2.color", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  if (unlikely(__Pyx_modinit_function_export_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_type_import_code();
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "pygame_sdl2/color.pyx":21
 * 
 * from sdl2 cimport *
 * import binascii             # <<<<<<<<<<<<<<
 * import struct
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_binascii, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_binascii, __pyx_t_1) < 0) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":22
 * from sdl2 cimport *
 * import binascii
 * import struct             # <<<<<<<<<<<<<<
 * 
 * include "color_dict.pxi"
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_struct, __pyx_t_1) < 0) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/color_dict.pxi":5
 * 
 * cdef object colors = {
 *     'aliceblue' : (240, 248, 255),             # <<<<<<<<<<<<<<
 *     'antiquewhite' : (250, 235, 215),
 *     'antiquewhite1' : (255, 239, 219),
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(658); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_aliceblue, __pyx_tuple__10) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":6
 * cdef object colors = {
 *     'aliceblue' : (240, 248, 255),
 *     'antiquewhite' : (250, 235, 215),             # <<<<<<<<<<<<<<
 *     'antiquewhite1' : (255, 239, 219),
 *     'antiquewhite2' : (238, 223, 204),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_antiquewhite, __pyx_tuple__11) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":7
 *     'aliceblue' : (240, 248, 255),
 *     'antiquewhite' : (250, 235, 215),
 *     'antiquewhite1' : (255, 239, 219),             # <<<<<<<<<<<<<<
 *     'antiquewhite2' : (238, 223, 204),
 *     'antiquewhite3' : (205, 192, 176),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_antiquewhite1, __pyx_tuple__12) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":8
 *     'antiquewhite' : (250, 235, 215),
 *     'antiquewhite1' : (255, 239, 219),
 *     'antiquewhite2' : (238, 223, 204),             # <<<<<<<<<<<<<<
 *     'antiquewhite3' : (205, 192, 176),
 *     'antiquewhite4' : (139, 131, 120),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_antiquewhite2, __pyx_tuple__13) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":9
 *     'antiquewhite1' : (255, 239, 219),
 *     'antiquewhite2' : (238, 223, 204),
 *     'antiquewhite3' : (205, 192, 176),             # <<<<<<<<<<<<<<
 *     'antiquewhite4' : (139, 131, 120),
 *     'aquamarine' : (127, 255, 212),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_antiquewhite3, __pyx_tuple__14) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":10
 *     'antiquewhite2' : (238, 223, 204),
 *     'antiquewhite3' : (205, 192, 176),
 *     'antiquewhite4' : (139, 131, 120),             # <<<<<<<<<<<<<<
 *     'aquamarine' : (127, 255, 212),
 *     'aquamarine1' : (127, 255, 212),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_antiquewhite4, __pyx_tuple__15) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":11
 *     'antiquewhite3' : (205, 192, 176),
 *     'antiquewhite4' : (139, 131, 120),
 *     'aquamarine' : (127, 255, 212),             # <<<<<<<<<<<<<<
 *     'aquamarine1' : (127, 255, 212),
 *     'aquamarine2' : (118, 238, 198),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_aquamarine, __pyx_tuple__16) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":12
 *     'antiquewhite4' : (139, 131, 120),
 *     'aquamarine' : (127, 255, 212),
 *     'aquamarine1' : (127, 255, 212),             # <<<<<<<<<<<<<<
 *     'aquamarine2' : (118, 238, 198),
 *     'aquamarine3' : (102, 205, 170),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_aquamarine1, __pyx_tuple__16) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":13
 *     'aquamarine' : (127, 255, 212),
 *     'aquamarine1' : (127, 255, 212),
 *     'aquamarine2' : (118, 238, 198),             # <<<<<<<<<<<<<<
 *     'aquamarine3' : (102, 205, 170),
 *     'aquamarine4' : (69, 139, 116),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_aquamarine2, __pyx_tuple__17) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":14
 *     'aquamarine1' : (127, 255, 212),
 *     'aquamarine2' : (118, 238, 198),
 *     'aquamarine3' : (102, 205, 170),             # <<<<<<<<<<<<<<
 *     'aquamarine4' : (69, 139, 116),
 *     'azure' : (240, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_aquamarine3, __pyx_tuple__18) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":15
 *     'aquamarine2' : (118, 238, 198),
 *     'aquamarine3' : (102, 205, 170),
 *     'aquamarine4' : (69, 139, 116),             # <<<<<<<<<<<<<<
 *     'azure' : (240, 255, 255),
 *     'azure1' : (240, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_aquamarine4, __pyx_tuple__19) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":16
 *     'aquamarine3' : (102, 205, 170),
 *     'aquamarine4' : (69, 139, 116),
 *     'azure' : (240, 255, 255),             # <<<<<<<<<<<<<<
 *     'azure1' : (240, 255, 255),
 *     'azure2' : (224, 238, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_azure, __pyx_tuple__20) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":17
 *     'aquamarine4' : (69, 139, 116),
 *     'azure' : (240, 255, 255),
 *     'azure1' : (240, 255, 255),             # <<<<<<<<<<<<<<
 *     'azure2' : (224, 238, 238),
 *     'azure3' : (193, 205, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_azure1, __pyx_tuple__20) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":18
 *     'azure' : (240, 255, 255),
 *     'azure1' : (240, 255, 255),
 *     'azure2' : (224, 238, 238),             # <<<<<<<<<<<<<<
 *     'azure3' : (193, 205, 205),
 *     'azure4' : (131, 139, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_azure2, __pyx_tuple__21) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":19
 *     'azure1' : (240, 255, 255),
 *     'azure2' : (224, 238, 238),
 *     'azure3' : (193, 205, 205),             # <<<<<<<<<<<<<<
 *     'azure4' : (131, 139, 139),
 *     'beige' : (245, 245, 220),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_azure3, __pyx_tuple__22) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":20
 *     'azure2' : (224, 238, 238),
 *     'azure3' : (193, 205, 205),
 *     'azure4' : (131, 139, 139),             # <<<<<<<<<<<<<<
 *     'beige' : (245, 245, 220),
 *     'bisque' : (255, 228, 196),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_azure4, __pyx_tuple__23) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":21
 *     'azure3' : (193, 205, 205),
 *     'azure4' : (131, 139, 139),
 *     'beige' : (245, 245, 220),             # <<<<<<<<<<<<<<
 *     'bisque' : (255, 228, 196),
 *     'bisque1' : (255, 228, 196),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_beige, __pyx_tuple__24) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":22
 *     'azure4' : (131, 139, 139),
 *     'beige' : (245, 245, 220),
 *     'bisque' : (255, 228, 196),             # <<<<<<<<<<<<<<
 *     'bisque1' : (255, 228, 196),
 *     'bisque2' : (238, 213, 183),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_bisque, __pyx_tuple__25) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":23
 *     'beige' : (245, 245, 220),
 *     'bisque' : (255, 228, 196),
 *     'bisque1' : (255, 228, 196),             # <<<<<<<<<<<<<<
 *     'bisque2' : (238, 213, 183),
 *     'bisque3' : (205, 183, 158),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_bisque1, __pyx_tuple__25) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":24
 *     'bisque' : (255, 228, 196),
 *     'bisque1' : (255, 228, 196),
 *     'bisque2' : (238, 213, 183),             # <<<<<<<<<<<<<<
 *     'bisque3' : (205, 183, 158),
 *     'bisque4' : (139, 125, 107),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_bisque2, __pyx_tuple__26) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":25
 *     'bisque1' : (255, 228, 196),
 *     'bisque2' : (238, 213, 183),
 *     'bisque3' : (205, 183, 158),             # <<<<<<<<<<<<<<
 *     'bisque4' : (139, 125, 107),
 *     'black' : (0, 0, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_bisque3, __pyx_tuple__27) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":26
 *     'bisque2' : (238, 213, 183),
 *     'bisque3' : (205, 183, 158),
 *     'bisque4' : (139, 125, 107),             # <<<<<<<<<<<<<<
 *     'black' : (0, 0, 0),
 *     'blanchedalmond' : (255, 235, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_bisque4, __pyx_tuple__28) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":27
 *     'bisque3' : (205, 183, 158),
 *     'bisque4' : (139, 125, 107),
 *     'black' : (0, 0, 0),             # <<<<<<<<<<<<<<
 *     'blanchedalmond' : (255, 235, 205),
 *     'blue' : (0, 0, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_black, __pyx_tuple__29) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":28
 *     'bisque4' : (139, 125, 107),
 *     'black' : (0, 0, 0),
 *     'blanchedalmond' : (255, 235, 205),             # <<<<<<<<<<<<<<
 *     'blue' : (0, 0, 255),
 *     'blue1' : (0, 0, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_blanchedalmond, __pyx_tuple__30) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":29
 *     'black' : (0, 0, 0),
 *     'blanchedalmond' : (255, 235, 205),
 *     'blue' : (0, 0, 255),             # <<<<<<<<<<<<<<
 *     'blue1' : (0, 0, 255),
 *     'blue2' : (0, 0, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_blue, __pyx_tuple__31) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":30
 *     'blanchedalmond' : (255, 235, 205),
 *     'blue' : (0, 0, 255),
 *     'blue1' : (0, 0, 255),             # <<<<<<<<<<<<<<
 *     'blue2' : (0, 0, 238),
 *     'blue3' : (0, 0, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_blue1, __pyx_tuple__31) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":31
 *     'blue' : (0, 0, 255),
 *     'blue1' : (0, 0, 255),
 *     'blue2' : (0, 0, 238),             # <<<<<<<<<<<<<<
 *     'blue3' : (0, 0, 205),
 *     'blue4' : (0, 0, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_blue2, __pyx_tuple__32) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":32
 *     'blue1' : (0, 0, 255),
 *     'blue2' : (0, 0, 238),
 *     'blue3' : (0, 0, 205),             # <<<<<<<<<<<<<<
 *     'blue4' : (0, 0, 139),
 *     'blueviolet' : (138, 43, 226),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_blue3, __pyx_tuple__33) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":33
 *     'blue2' : (0, 0, 238),
 *     'blue3' : (0, 0, 205),
 *     'blue4' : (0, 0, 139),             # <<<<<<<<<<<<<<
 *     'blueviolet' : (138, 43, 226),
 *     'brown' : (165, 42, 42),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_blue4, __pyx_tuple__34) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":34
 *     'blue3' : (0, 0, 205),
 *     'blue4' : (0, 0, 139),
 *     'blueviolet' : (138, 43, 226),             # <<<<<<<<<<<<<<
 *     'brown' : (165, 42, 42),
 *     'brown1' : (255, 64, 64),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_blueviolet, __pyx_tuple__35) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":35
 *     'blue4' : (0, 0, 139),
 *     'blueviolet' : (138, 43, 226),
 *     'brown' : (165, 42, 42),             # <<<<<<<<<<<<<<
 *     'brown1' : (255, 64, 64),
 *     'brown2' : (238, 59, 59),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_brown, __pyx_tuple__36) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":36
 *     'blueviolet' : (138, 43, 226),
 *     'brown' : (165, 42, 42),
 *     'brown1' : (255, 64, 64),             # <<<<<<<<<<<<<<
 *     'brown2' : (238, 59, 59),
 *     'brown3' : (205, 51, 51),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_brown1, __pyx_tuple__37) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":37
 *     'brown' : (165, 42, 42),
 *     'brown1' : (255, 64, 64),
 *     'brown2' : (238, 59, 59),             # <<<<<<<<<<<<<<
 *     'brown3' : (205, 51, 51),
 *     'brown4' : (139, 35, 35),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_brown2, __pyx_tuple__38) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":38
 *     'brown1' : (255, 64, 64),
 *     'brown2' : (238, 59, 59),
 *     'brown3' : (205, 51, 51),             # <<<<<<<<<<<<<<
 *     'brown4' : (139, 35, 35),
 *     'burlywood' : (222, 184, 135),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_brown3, __pyx_tuple__39) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":39
 *     'brown2' : (238, 59, 59),
 *     'brown3' : (205, 51, 51),
 *     'brown4' : (139, 35, 35),             # <<<<<<<<<<<<<<
 *     'burlywood' : (222, 184, 135),
 *     'burlywood1' : (255, 211, 155),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_brown4, __pyx_tuple__40) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":40
 *     'brown3' : (205, 51, 51),
 *     'brown4' : (139, 35, 35),
 *     'burlywood' : (222, 184, 135),             # <<<<<<<<<<<<<<
 *     'burlywood1' : (255, 211, 155),
 *     'burlywood2' : (238, 197, 145),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_burlywood, __pyx_tuple__41) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":41
 *     'brown4' : (139, 35, 35),
 *     'burlywood' : (222, 184, 135),
 *     'burlywood1' : (255, 211, 155),             # <<<<<<<<<<<<<<
 *     'burlywood2' : (238, 197, 145),
 *     'burlywood3' : (205, 170, 125),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_burlywood1, __pyx_tuple__42) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":42
 *     'burlywood' : (222, 184, 135),
 *     'burlywood1' : (255, 211, 155),
 *     'burlywood2' : (238, 197, 145),             # <<<<<<<<<<<<<<
 *     'burlywood3' : (205, 170, 125),
 *     'burlywood4' : (139, 115, 85),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_burlywood2, __pyx_tuple__43) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":43
 *     'burlywood1' : (255, 211, 155),
 *     'burlywood2' : (238, 197, 145),
 *     'burlywood3' : (205, 170, 125),             # <<<<<<<<<<<<<<
 *     'burlywood4' : (139, 115, 85),
 *     'cadetblue' : (95, 158, 160),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_burlywood3, __pyx_tuple__44) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":44
 *     'burlywood2' : (238, 197, 145),
 *     'burlywood3' : (205, 170, 125),
 *     'burlywood4' : (139, 115, 85),             # <<<<<<<<<<<<<<
 *     'cadetblue' : (95, 158, 160),
 *     'cadetblue1' : (152, 245, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_burlywood4, __pyx_tuple__45) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":45
 *     'burlywood3' : (205, 170, 125),
 *     'burlywood4' : (139, 115, 85),
 *     'cadetblue' : (95, 158, 160),             # <<<<<<<<<<<<<<
 *     'cadetblue1' : (152, 245, 255),
 *     'cadetblue2' : (142, 229, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cadetblue, __pyx_tuple__46) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":46
 *     'burlywood4' : (139, 115, 85),
 *     'cadetblue' : (95, 158, 160),
 *     'cadetblue1' : (152, 245, 255),             # <<<<<<<<<<<<<<
 *     'cadetblue2' : (142, 229, 238),
 *     'cadetblue3' : (122, 197, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cadetblue1, __pyx_tuple__47) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":47
 *     'cadetblue' : (95, 158, 160),
 *     'cadetblue1' : (152, 245, 255),
 *     'cadetblue2' : (142, 229, 238),             # <<<<<<<<<<<<<<
 *     'cadetblue3' : (122, 197, 205),
 *     'cadetblue4' : (83, 134, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cadetblue2, __pyx_tuple__48) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":48
 *     'cadetblue1' : (152, 245, 255),
 *     'cadetblue2' : (142, 229, 238),
 *     'cadetblue3' : (122, 197, 205),             # <<<<<<<<<<<<<<
 *     'cadetblue4' : (83, 134, 139),
 *     'chartreuse' : (127, 255, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cadetblue3, __pyx_tuple__49) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":49
 *     'cadetblue2' : (142, 229, 238),
 *     'cadetblue3' : (122, 197, 205),
 *     'cadetblue4' : (83, 134, 139),             # <<<<<<<<<<<<<<
 *     'chartreuse' : (127, 255, 0),
 *     'chartreuse1' : (127, 255, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cadetblue4, __pyx_tuple__50) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":50
 *     'cadetblue3' : (122, 197, 205),
 *     'cadetblue4' : (83, 134, 139),
 *     'chartreuse' : (127, 255, 0),             # <<<<<<<<<<<<<<
 *     'chartreuse1' : (127, 255, 0),
 *     'chartreuse2' : (118, 238, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_chartreuse, __pyx_tuple__51) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":51
 *     'cadetblue4' : (83, 134, 139),
 *     'chartreuse' : (127, 255, 0),
 *     'chartreuse1' : (127, 255, 0),             # <<<<<<<<<<<<<<
 *     'chartreuse2' : (118, 238, 0),
 *     'chartreuse3' : (102, 205, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_chartreuse1, __pyx_tuple__51) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":52
 *     'chartreuse' : (127, 255, 0),
 *     'chartreuse1' : (127, 255, 0),
 *     'chartreuse2' : (118, 238, 0),             # <<<<<<<<<<<<<<
 *     'chartreuse3' : (102, 205, 0),
 *     'chartreuse4' : (69, 139, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_chartreuse2, __pyx_tuple__52) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":53
 *     'chartreuse1' : (127, 255, 0),
 *     'chartreuse2' : (118, 238, 0),
 *     'chartreuse3' : (102, 205, 0),             # <<<<<<<<<<<<<<
 *     'chartreuse4' : (69, 139, 0),
 *     'chocolate' : (210, 105, 30),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_chartreuse3, __pyx_tuple__53) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":54
 *     'chartreuse2' : (118, 238, 0),
 *     'chartreuse3' : (102, 205, 0),
 *     'chartreuse4' : (69, 139, 0),             # <<<<<<<<<<<<<<
 *     'chocolate' : (210, 105, 30),
 *     'chocolate1' : (255, 127, 36),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_chartreuse4, __pyx_tuple__54) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":55
 *     'chartreuse3' : (102, 205, 0),
 *     'chartreuse4' : (69, 139, 0),
 *     'chocolate' : (210, 105, 30),             # <<<<<<<<<<<<<<
 *     'chocolate1' : (255, 127, 36),
 *     'chocolate2' : (238, 118, 33),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_chocolate, __pyx_tuple__55) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":56
 *     'chartreuse4' : (69, 139, 0),
 *     'chocolate' : (210, 105, 30),
 *     'chocolate1' : (255, 127, 36),             # <<<<<<<<<<<<<<
 *     'chocolate2' : (238, 118, 33),
 *     'chocolate3' : (205, 102, 29),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_chocolate1, __pyx_tuple__56) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":57
 *     'chocolate' : (210, 105, 30),
 *     'chocolate1' : (255, 127, 36),
 *     'chocolate2' : (238, 118, 33),             # <<<<<<<<<<<<<<
 *     'chocolate3' : (205, 102, 29),
 *     'chocolate4' : (139, 69, 19),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_chocolate2, __pyx_tuple__57) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":58
 *     'chocolate1' : (255, 127, 36),
 *     'chocolate2' : (238, 118, 33),
 *     'chocolate3' : (205, 102, 29),             # <<<<<<<<<<<<<<
 *     'chocolate4' : (139, 69, 19),
 *     'coral' : (255, 127, 80),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_chocolate3, __pyx_tuple__58) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":59
 *     'chocolate2' : (238, 118, 33),
 *     'chocolate3' : (205, 102, 29),
 *     'chocolate4' : (139, 69, 19),             # <<<<<<<<<<<<<<
 *     'coral' : (255, 127, 80),
 *     'coral1' : (255, 114, 86),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_chocolate4, __pyx_tuple__59) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":60
 *     'chocolate3' : (205, 102, 29),
 *     'chocolate4' : (139, 69, 19),
 *     'coral' : (255, 127, 80),             # <<<<<<<<<<<<<<
 *     'coral1' : (255, 114, 86),
 *     'coral2' : (238, 106, 80),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_coral, __pyx_tuple__60) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":61
 *     'chocolate4' : (139, 69, 19),
 *     'coral' : (255, 127, 80),
 *     'coral1' : (255, 114, 86),             # <<<<<<<<<<<<<<
 *     'coral2' : (238, 106, 80),
 *     'coral3' : (205, 91, 69),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_coral1, __pyx_tuple__61) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":62
 *     'coral' : (255, 127, 80),
 *     'coral1' : (255, 114, 86),
 *     'coral2' : (238, 106, 80),             # <<<<<<<<<<<<<<
 *     'coral3' : (205, 91, 69),
 *     'coral4' : (139, 62, 47),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_coral2, __pyx_tuple__62) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":63
 *     'coral1' : (255, 114, 86),
 *     'coral2' : (238, 106, 80),
 *     'coral3' : (205, 91, 69),             # <<<<<<<<<<<<<<
 *     'coral4' : (139, 62, 47),
 *     'cornflowerblue' : (100, 149, 237),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_coral3, __pyx_tuple__63) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":64
 *     'coral2' : (238, 106, 80),
 *     'coral3' : (205, 91, 69),
 *     'coral4' : (139, 62, 47),             # <<<<<<<<<<<<<<
 *     'cornflowerblue' : (100, 149, 237),
 *     'cornsilk' : (255, 248, 220),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_coral4, __pyx_tuple__64) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":65
 *     'coral3' : (205, 91, 69),
 *     'coral4' : (139, 62, 47),
 *     'cornflowerblue' : (100, 149, 237),             # <<<<<<<<<<<<<<
 *     'cornsilk' : (255, 248, 220),
 *     'cornsilk1' : (255, 248, 220),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cornflowerblue, __pyx_tuple__65) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":66
 *     'coral4' : (139, 62, 47),
 *     'cornflowerblue' : (100, 149, 237),
 *     'cornsilk' : (255, 248, 220),             # <<<<<<<<<<<<<<
 *     'cornsilk1' : (255, 248, 220),
 *     'cornsilk2' : (238, 232, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cornsilk, __pyx_tuple__66) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":67
 *     'cornflowerblue' : (100, 149, 237),
 *     'cornsilk' : (255, 248, 220),
 *     'cornsilk1' : (255, 248, 220),             # <<<<<<<<<<<<<<
 *     'cornsilk2' : (238, 232, 205),
 *     'cornsilk3' : (205, 200, 177),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cornsilk1, __pyx_tuple__66) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":68
 *     'cornsilk' : (255, 248, 220),
 *     'cornsilk1' : (255, 248, 220),
 *     'cornsilk2' : (238, 232, 205),             # <<<<<<<<<<<<<<
 *     'cornsilk3' : (205, 200, 177),
 *     'cornsilk4' : (139, 136, 120),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cornsilk2, __pyx_tuple__67) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":69
 *     'cornsilk1' : (255, 248, 220),
 *     'cornsilk2' : (238, 232, 205),
 *     'cornsilk3' : (205, 200, 177),             # <<<<<<<<<<<<<<
 *     'cornsilk4' : (139, 136, 120),
 *     'cyan' : (0, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cornsilk3, __pyx_tuple__68) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":70
 *     'cornsilk2' : (238, 232, 205),
 *     'cornsilk3' : (205, 200, 177),
 *     'cornsilk4' : (139, 136, 120),             # <<<<<<<<<<<<<<
 *     'cyan' : (0, 255, 255),
 *     'cyan1' : (0, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cornsilk4, __pyx_tuple__69) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":71
 *     'cornsilk3' : (205, 200, 177),
 *     'cornsilk4' : (139, 136, 120),
 *     'cyan' : (0, 255, 255),             # <<<<<<<<<<<<<<
 *     'cyan1' : (0, 255, 255),
 *     'cyan2' : (0, 238, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cyan, __pyx_tuple__70) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":72
 *     'cornsilk4' : (139, 136, 120),
 *     'cyan' : (0, 255, 255),
 *     'cyan1' : (0, 255, 255),             # <<<<<<<<<<<<<<
 *     'cyan2' : (0, 238, 238),
 *     'cyan3' : (0, 205, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cyan1, __pyx_tuple__70) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":73
 *     'cyan' : (0, 255, 255),
 *     'cyan1' : (0, 255, 255),
 *     'cyan2' : (0, 238, 238),             # <<<<<<<<<<<<<<
 *     'cyan3' : (0, 205, 205),
 *     'cyan4' : (0, 139, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cyan2, __pyx_tuple__71) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":74
 *     'cyan1' : (0, 255, 255),
 *     'cyan2' : (0, 238, 238),
 *     'cyan3' : (0, 205, 205),             # <<<<<<<<<<<<<<
 *     'cyan4' : (0, 139, 139),
 *     'darkblue' : (0, 0, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cyan3, __pyx_tuple__72) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":75
 *     'cyan2' : (0, 238, 238),
 *     'cyan3' : (0, 205, 205),
 *     'cyan4' : (0, 139, 139),             # <<<<<<<<<<<<<<
 *     'darkblue' : (0, 0, 139),
 *     'darkcyan' : (0, 139, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_cyan4, __pyx_tuple__73) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":76
 *     'cyan3' : (0, 205, 205),
 *     'cyan4' : (0, 139, 139),
 *     'darkblue' : (0, 0, 139),             # <<<<<<<<<<<<<<
 *     'darkcyan' : (0, 139, 139),
 *     'darkgoldenrod' : (184, 134, 11),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkblue, __pyx_tuple__34) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":77
 *     'cyan4' : (0, 139, 139),
 *     'darkblue' : (0, 0, 139),
 *     'darkcyan' : (0, 139, 139),             # <<<<<<<<<<<<<<
 *     'darkgoldenrod' : (184, 134, 11),
 *     'darkgoldenrod1' : (255, 185, 15),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkcyan, __pyx_tuple__73) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":78
 *     'darkblue' : (0, 0, 139),
 *     'darkcyan' : (0, 139, 139),
 *     'darkgoldenrod' : (184, 134, 11),             # <<<<<<<<<<<<<<
 *     'darkgoldenrod1' : (255, 185, 15),
 *     'darkgoldenrod2' : (238, 173, 14),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkgoldenrod, __pyx_tuple__74) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":79
 *     'darkcyan' : (0, 139, 139),
 *     'darkgoldenrod' : (184, 134, 11),
 *     'darkgoldenrod1' : (255, 185, 15),             # <<<<<<<<<<<<<<
 *     'darkgoldenrod2' : (238, 173, 14),
 *     'darkgoldenrod3' : (205, 149, 12),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkgoldenrod1, __pyx_tuple__75) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":80
 *     'darkgoldenrod' : (184, 134, 11),
 *     'darkgoldenrod1' : (255, 185, 15),
 *     'darkgoldenrod2' : (238, 173, 14),             # <<<<<<<<<<<<<<
 *     'darkgoldenrod3' : (205, 149, 12),
 *     'darkgoldenrod4' : (139, 101, 8),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkgoldenrod2, __pyx_tuple__76) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":81
 *     'darkgoldenrod1' : (255, 185, 15),
 *     'darkgoldenrod2' : (238, 173, 14),
 *     'darkgoldenrod3' : (205, 149, 12),             # <<<<<<<<<<<<<<
 *     'darkgoldenrod4' : (139, 101, 8),
 *     'darkgray' : (169, 169, 169),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkgoldenrod3, __pyx_tuple__77) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":82
 *     'darkgoldenrod2' : (238, 173, 14),
 *     'darkgoldenrod3' : (205, 149, 12),
 *     'darkgoldenrod4' : (139, 101, 8),             # <<<<<<<<<<<<<<
 *     'darkgray' : (169, 169, 169),
 *     'darkgreen' : (0, 100, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkgoldenrod4, __pyx_tuple__78) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":83
 *     'darkgoldenrod3' : (205, 149, 12),
 *     'darkgoldenrod4' : (139, 101, 8),
 *     'darkgray' : (169, 169, 169),             # <<<<<<<<<<<<<<
 *     'darkgreen' : (0, 100, 0),
 *     'darkgrey' : (169, 169, 169),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkgray, __pyx_tuple__79) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":84
 *     'darkgoldenrod4' : (139, 101, 8),
 *     'darkgray' : (169, 169, 169),
 *     'darkgreen' : (0, 100, 0),             # <<<<<<<<<<<<<<
 *     'darkgrey' : (169, 169, 169),
 *     'darkkhaki' : (189, 183, 107),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkgreen, __pyx_tuple__80) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":85
 *     'darkgray' : (169, 169, 169),
 *     'darkgreen' : (0, 100, 0),
 *     'darkgrey' : (169, 169, 169),             # <<<<<<<<<<<<<<
 *     'darkkhaki' : (189, 183, 107),
 *     'darkmagenta' : (139, 0, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkgrey, __pyx_tuple__79) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":86
 *     'darkgreen' : (0, 100, 0),
 *     'darkgrey' : (169, 169, 169),
 *     'darkkhaki' : (189, 183, 107),             # <<<<<<<<<<<<<<
 *     'darkmagenta' : (139, 0, 139),
 *     'darkolivegreen' : (85, 107, 47),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkkhaki, __pyx_tuple__81) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":87
 *     'darkgrey' : (169, 169, 169),
 *     'darkkhaki' : (189, 183, 107),
 *     'darkmagenta' : (139, 0, 139),             # <<<<<<<<<<<<<<
 *     'darkolivegreen' : (85, 107, 47),
 *     'darkolivegreen1' : (202, 255, 112),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkmagenta, __pyx_tuple__82) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":88
 *     'darkkhaki' : (189, 183, 107),
 *     'darkmagenta' : (139, 0, 139),
 *     'darkolivegreen' : (85, 107, 47),             # <<<<<<<<<<<<<<
 *     'darkolivegreen1' : (202, 255, 112),
 *     'darkolivegreen2' : (188, 238, 104),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkolivegreen, __pyx_tuple__83) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":89
 *     'darkmagenta' : (139, 0, 139),
 *     'darkolivegreen' : (85, 107, 47),
 *     'darkolivegreen1' : (202, 255, 112),             # <<<<<<<<<<<<<<
 *     'darkolivegreen2' : (188, 238, 104),
 *     'darkolivegreen3' : (162, 205, 90),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkolivegreen1, __pyx_tuple__84) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":90
 *     'darkolivegreen' : (85, 107, 47),
 *     'darkolivegreen1' : (202, 255, 112),
 *     'darkolivegreen2' : (188, 238, 104),             # <<<<<<<<<<<<<<
 *     'darkolivegreen3' : (162, 205, 90),
 *     'darkolivegreen4' : (110, 139, 61),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkolivegreen2, __pyx_tuple__85) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":91
 *     'darkolivegreen1' : (202, 255, 112),
 *     'darkolivegreen2' : (188, 238, 104),
 *     'darkolivegreen3' : (162, 205, 90),             # <<<<<<<<<<<<<<
 *     'darkolivegreen4' : (110, 139, 61),
 *     'darkorange' : (255, 140, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkolivegreen3, __pyx_tuple__86) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":92
 *     'darkolivegreen2' : (188, 238, 104),
 *     'darkolivegreen3' : (162, 205, 90),
 *     'darkolivegreen4' : (110, 139, 61),             # <<<<<<<<<<<<<<
 *     'darkorange' : (255, 140, 0),
 *     'darkorange1' : (255, 127, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkolivegreen4, __pyx_tuple__87) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":93
 *     'darkolivegreen3' : (162, 205, 90),
 *     'darkolivegreen4' : (110, 139, 61),
 *     'darkorange' : (255, 140, 0),             # <<<<<<<<<<<<<<
 *     'darkorange1' : (255, 127, 0),
 *     'darkorange2' : (238, 118, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkorange, __pyx_tuple__88) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":94
 *     'darkolivegreen4' : (110, 139, 61),
 *     'darkorange' : (255, 140, 0),
 *     'darkorange1' : (255, 127, 0),             # <<<<<<<<<<<<<<
 *     'darkorange2' : (238, 118, 0),
 *     'darkorange3' : (205, 102, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkorange1, __pyx_tuple__89) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":95
 *     'darkorange' : (255, 140, 0),
 *     'darkorange1' : (255, 127, 0),
 *     'darkorange2' : (238, 118, 0),             # <<<<<<<<<<<<<<
 *     'darkorange3' : (205, 102, 0),
 *     'darkorange4' : (139, 69, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkorange2, __pyx_tuple__90) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":96
 *     'darkorange1' : (255, 127, 0),
 *     'darkorange2' : (238, 118, 0),
 *     'darkorange3' : (205, 102, 0),             # <<<<<<<<<<<<<<
 *     'darkorange4' : (139, 69, 0),
 *     'darkorchid' : (153, 50, 204),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkorange3, __pyx_tuple__91) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":97
 *     'darkorange2' : (238, 118, 0),
 *     'darkorange3' : (205, 102, 0),
 *     'darkorange4' : (139, 69, 0),             # <<<<<<<<<<<<<<
 *     'darkorchid' : (153, 50, 204),
 *     'darkorchid1' : (191, 62, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkorange4, __pyx_tuple__92) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":98
 *     'darkorange3' : (205, 102, 0),
 *     'darkorange4' : (139, 69, 0),
 *     'darkorchid' : (153, 50, 204),             # <<<<<<<<<<<<<<
 *     'darkorchid1' : (191, 62, 255),
 *     'darkorchid2' : (178, 58, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkorchid, __pyx_tuple__93) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":99
 *     'darkorange4' : (139, 69, 0),
 *     'darkorchid' : (153, 50, 204),
 *     'darkorchid1' : (191, 62, 255),             # <<<<<<<<<<<<<<
 *     'darkorchid2' : (178, 58, 238),
 *     'darkorchid3' : (154, 50, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkorchid1, __pyx_tuple__94) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":100
 *     'darkorchid' : (153, 50, 204),
 *     'darkorchid1' : (191, 62, 255),
 *     'darkorchid2' : (178, 58, 238),             # <<<<<<<<<<<<<<
 *     'darkorchid3' : (154, 50, 205),
 *     'darkorchid4' : (104, 34, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkorchid2, __pyx_tuple__95) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":101
 *     'darkorchid1' : (191, 62, 255),
 *     'darkorchid2' : (178, 58, 238),
 *     'darkorchid3' : (154, 50, 205),             # <<<<<<<<<<<<<<
 *     'darkorchid4' : (104, 34, 139),
 *     'darkred' : (139, 0, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkorchid3, __pyx_tuple__96) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":102
 *     'darkorchid2' : (178, 58, 238),
 *     'darkorchid3' : (154, 50, 205),
 *     'darkorchid4' : (104, 34, 139),             # <<<<<<<<<<<<<<
 *     'darkred' : (139, 0, 0),
 *     'darksalmon' : (233, 150, 122),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkorchid4, __pyx_tuple__97) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":103
 *     'darkorchid3' : (154, 50, 205),
 *     'darkorchid4' : (104, 34, 139),
 *     'darkred' : (139, 0, 0),             # <<<<<<<<<<<<<<
 *     'darksalmon' : (233, 150, 122),
 *     'darkseagreen' : (143, 188, 143),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkred, __pyx_tuple__98) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":104
 *     'darkorchid4' : (104, 34, 139),
 *     'darkred' : (139, 0, 0),
 *     'darksalmon' : (233, 150, 122),             # <<<<<<<<<<<<<<
 *     'darkseagreen' : (143, 188, 143),
 *     'darkseagreen1' : (193, 255, 193),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darksalmon, __pyx_tuple__99) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":105
 *     'darkred' : (139, 0, 0),
 *     'darksalmon' : (233, 150, 122),
 *     'darkseagreen' : (143, 188, 143),             # <<<<<<<<<<<<<<
 *     'darkseagreen1' : (193, 255, 193),
 *     'darkseagreen2' : (180, 238, 180),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkseagreen, __pyx_tuple__100) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":106
 *     'darksalmon' : (233, 150, 122),
 *     'darkseagreen' : (143, 188, 143),
 *     'darkseagreen1' : (193, 255, 193),             # <<<<<<<<<<<<<<
 *     'darkseagreen2' : (180, 238, 180),
 *     'darkseagreen3' : (155, 205, 155),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkseagreen1, __pyx_tuple__101) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":107
 *     'darkseagreen' : (143, 188, 143),
 *     'darkseagreen1' : (193, 255, 193),
 *     'darkseagreen2' : (180, 238, 180),             # <<<<<<<<<<<<<<
 *     'darkseagreen3' : (155, 205, 155),
 *     'darkseagreen4' : (105, 139, 105),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkseagreen2, __pyx_tuple__102) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":108
 *     'darkseagreen1' : (193, 255, 193),
 *     'darkseagreen2' : (180, 238, 180),
 *     'darkseagreen3' : (155, 205, 155),             # <<<<<<<<<<<<<<
 *     'darkseagreen4' : (105, 139, 105),
 *     'darkslateblue' : (72, 61, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkseagreen3, __pyx_tuple__103) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":109
 *     'darkseagreen2' : (180, 238, 180),
 *     'darkseagreen3' : (155, 205, 155),
 *     'darkseagreen4' : (105, 139, 105),             # <<<<<<<<<<<<<<
 *     'darkslateblue' : (72, 61, 139),
 *     'darkslategray' : (47, 79, 79),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkseagreen4, __pyx_tuple__104) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":110
 *     'darkseagreen3' : (155, 205, 155),
 *     'darkseagreen4' : (105, 139, 105),
 *     'darkslateblue' : (72, 61, 139),             # <<<<<<<<<<<<<<
 *     'darkslategray' : (47, 79, 79),
 *     'darkslategray1' : (151, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkslateblue, __pyx_tuple__105) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":111
 *     'darkseagreen4' : (105, 139, 105),
 *     'darkslateblue' : (72, 61, 139),
 *     'darkslategray' : (47, 79, 79),             # <<<<<<<<<<<<<<
 *     'darkslategray1' : (151, 255, 255),
 *     'darkslategray2' : (141, 238, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkslategray, __pyx_tuple__106) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":112
 *     'darkslateblue' : (72, 61, 139),
 *     'darkslategray' : (47, 79, 79),
 *     'darkslategray1' : (151, 255, 255),             # <<<<<<<<<<<<<<
 *     'darkslategray2' : (141, 238, 238),
 *     'darkslategray3' : (121, 205, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkslategray1, __pyx_tuple__107) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":113
 *     'darkslategray' : (47, 79, 79),
 *     'darkslategray1' : (151, 255, 255),
 *     'darkslategray2' : (141, 238, 238),             # <<<<<<<<<<<<<<
 *     'darkslategray3' : (121, 205, 205),
 *     'darkslategray4' : (82, 139, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkslategray2, __pyx_tuple__108) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":114
 *     'darkslategray1' : (151, 255, 255),
 *     'darkslategray2' : (141, 238, 238),
 *     'darkslategray3' : (121, 205, 205),             # <<<<<<<<<<<<<<
 *     'darkslategray4' : (82, 139, 139),
 *     'darkslategrey' : (47, 79, 79),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkslategray3, __pyx_tuple__109) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":115
 *     'darkslategray2' : (141, 238, 238),
 *     'darkslategray3' : (121, 205, 205),
 *     'darkslategray4' : (82, 139, 139),             # <<<<<<<<<<<<<<
 *     'darkslategrey' : (47, 79, 79),
 *     'darkturquoise' : (0, 206, 209),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkslategray4, __pyx_tuple__110) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":116
 *     'darkslategray3' : (121, 205, 205),
 *     'darkslategray4' : (82, 139, 139),
 *     'darkslategrey' : (47, 79, 79),             # <<<<<<<<<<<<<<
 *     'darkturquoise' : (0, 206, 209),
 *     'darkviolet' : (148, 0, 211),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkslategrey, __pyx_tuple__106) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":117
 *     'darkslategray4' : (82, 139, 139),
 *     'darkslategrey' : (47, 79, 79),
 *     'darkturquoise' : (0, 206, 209),             # <<<<<<<<<<<<<<
 *     'darkviolet' : (148, 0, 211),
 *     'debianred' : (215, 7, 81),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkturquoise, __pyx_tuple__111) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":118
 *     'darkslategrey' : (47, 79, 79),
 *     'darkturquoise' : (0, 206, 209),
 *     'darkviolet' : (148, 0, 211),             # <<<<<<<<<<<<<<
 *     'debianred' : (215, 7, 81),
 *     'deeppink' : (255, 20, 147),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_darkviolet, __pyx_tuple__112) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":119
 *     'darkturquoise' : (0, 206, 209),
 *     'darkviolet' : (148, 0, 211),
 *     'debianred' : (215, 7, 81),             # <<<<<<<<<<<<<<
 *     'deeppink' : (255, 20, 147),
 *     'deeppink1' : (255, 20, 147),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_debianred, __pyx_tuple__113) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":120
 *     'darkviolet' : (148, 0, 211),
 *     'debianred' : (215, 7, 81),
 *     'deeppink' : (255, 20, 147),             # <<<<<<<<<<<<<<
 *     'deeppink1' : (255, 20, 147),
 *     'deeppink2' : (238, 18, 137),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_deeppink, __pyx_tuple__114) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":121
 *     'debianred' : (215, 7, 81),
 *     'deeppink' : (255, 20, 147),
 *     'deeppink1' : (255, 20, 147),             # <<<<<<<<<<<<<<
 *     'deeppink2' : (238, 18, 137),
 *     'deeppink3' : (205, 16, 118),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_deeppink1, __pyx_tuple__114) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":122
 *     'deeppink' : (255, 20, 147),
 *     'deeppink1' : (255, 20, 147),
 *     'deeppink2' : (238, 18, 137),             # <<<<<<<<<<<<<<
 *     'deeppink3' : (205, 16, 118),
 *     'deeppink4' : (139, 10, 80),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_deeppink2, __pyx_tuple__115) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":123
 *     'deeppink1' : (255, 20, 147),
 *     'deeppink2' : (238, 18, 137),
 *     'deeppink3' : (205, 16, 118),             # <<<<<<<<<<<<<<
 *     'deeppink4' : (139, 10, 80),
 *     'deepskyblue' : (0, 191, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_deeppink3, __pyx_tuple__116) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":124
 *     'deeppink2' : (238, 18, 137),
 *     'deeppink3' : (205, 16, 118),
 *     'deeppink4' : (139, 10, 80),             # <<<<<<<<<<<<<<
 *     'deepskyblue' : (0, 191, 255),
 *     'deepskyblue1' : (0, 191, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_deeppink4, __pyx_tuple__117) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":125
 *     'deeppink3' : (205, 16, 118),
 *     'deeppink4' : (139, 10, 80),
 *     'deepskyblue' : (0, 191, 255),             # <<<<<<<<<<<<<<
 *     'deepskyblue1' : (0, 191, 255),
 *     'deepskyblue2' : (0, 178, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_deepskyblue, __pyx_tuple__118) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":126
 *     'deeppink4' : (139, 10, 80),
 *     'deepskyblue' : (0, 191, 255),
 *     'deepskyblue1' : (0, 191, 255),             # <<<<<<<<<<<<<<
 *     'deepskyblue2' : (0, 178, 238),
 *     'deepskyblue3' : (0, 154, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_deepskyblue1, __pyx_tuple__118) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":127
 *     'deepskyblue' : (0, 191, 255),
 *     'deepskyblue1' : (0, 191, 255),
 *     'deepskyblue2' : (0, 178, 238),             # <<<<<<<<<<<<<<
 *     'deepskyblue3' : (0, 154, 205),
 *     'deepskyblue4' : (0, 104, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_deepskyblue2, __pyx_tuple__119) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":128
 *     'deepskyblue1' : (0, 191, 255),
 *     'deepskyblue2' : (0, 178, 238),
 *     'deepskyblue3' : (0, 154, 205),             # <<<<<<<<<<<<<<
 *     'deepskyblue4' : (0, 104, 139),
 *     'dimgray' : (105, 105, 105),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_deepskyblue3, __pyx_tuple__120) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":129
 *     'deepskyblue2' : (0, 178, 238),
 *     'deepskyblue3' : (0, 154, 205),
 *     'deepskyblue4' : (0, 104, 139),             # <<<<<<<<<<<<<<
 *     'dimgray' : (105, 105, 105),
 *     'dimgrey' : (105, 105, 105),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_deepskyblue4, __pyx_tuple__121) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":130
 *     'deepskyblue3' : (0, 154, 205),
 *     'deepskyblue4' : (0, 104, 139),
 *     'dimgray' : (105, 105, 105),             # <<<<<<<<<<<<<<
 *     'dimgrey' : (105, 105, 105),
 *     'dodgerblue' : (30, 144, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_dimgray, __pyx_tuple__122) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":131
 *     'deepskyblue4' : (0, 104, 139),
 *     'dimgray' : (105, 105, 105),
 *     'dimgrey' : (105, 105, 105),             # <<<<<<<<<<<<<<
 *     'dodgerblue' : (30, 144, 255),
 *     'dodgerblue1' : (30, 144, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_dimgrey, __pyx_tuple__122) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":132
 *     'dimgray' : (105, 105, 105),
 *     'dimgrey' : (105, 105, 105),
 *     'dodgerblue' : (30, 144, 255),             # <<<<<<<<<<<<<<
 *     'dodgerblue1' : (30, 144, 255),
 *     'dodgerblue2' : (28, 134, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_dodgerblue, __pyx_tuple__123) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":133
 *     'dimgrey' : (105, 105, 105),
 *     'dodgerblue' : (30, 144, 255),
 *     'dodgerblue1' : (30, 144, 255),             # <<<<<<<<<<<<<<
 *     'dodgerblue2' : (28, 134, 238),
 *     'dodgerblue3' : (24, 116, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_dodgerblue1, __pyx_tuple__123) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":134
 *     'dodgerblue' : (30, 144, 255),
 *     'dodgerblue1' : (30, 144, 255),
 *     'dodgerblue2' : (28, 134, 238),             # <<<<<<<<<<<<<<
 *     'dodgerblue3' : (24, 116, 205),
 *     'dodgerblue4' : (16, 78, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_dodgerblue2, __pyx_tuple__124) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":135
 *     'dodgerblue1' : (30, 144, 255),
 *     'dodgerblue2' : (28, 134, 238),
 *     'dodgerblue3' : (24, 116, 205),             # <<<<<<<<<<<<<<
 *     'dodgerblue4' : (16, 78, 139),
 *     'firebrick' : (178, 34, 34),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_dodgerblue3, __pyx_tuple__125) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":136
 *     'dodgerblue2' : (28, 134, 238),
 *     'dodgerblue3' : (24, 116, 205),
 *     'dodgerblue4' : (16, 78, 139),             # <<<<<<<<<<<<<<
 *     'firebrick' : (178, 34, 34),
 *     'firebrick1' : (255, 48, 48),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_dodgerblue4, __pyx_tuple__126) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":137
 *     'dodgerblue3' : (24, 116, 205),
 *     'dodgerblue4' : (16, 78, 139),
 *     'firebrick' : (178, 34, 34),             # <<<<<<<<<<<<<<
 *     'firebrick1' : (255, 48, 48),
 *     'firebrick2' : (238, 44, 44),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_firebrick, __pyx_tuple__127) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":138
 *     'dodgerblue4' : (16, 78, 139),
 *     'firebrick' : (178, 34, 34),
 *     'firebrick1' : (255, 48, 48),             # <<<<<<<<<<<<<<
 *     'firebrick2' : (238, 44, 44),
 *     'firebrick3' : (205, 38, 38),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_firebrick1, __pyx_tuple__128) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":139
 *     'firebrick' : (178, 34, 34),
 *     'firebrick1' : (255, 48, 48),
 *     'firebrick2' : (238, 44, 44),             # <<<<<<<<<<<<<<
 *     'firebrick3' : (205, 38, 38),
 *     'firebrick4' : (139, 26, 26),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_firebrick2, __pyx_tuple__129) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":140
 *     'firebrick1' : (255, 48, 48),
 *     'firebrick2' : (238, 44, 44),
 *     'firebrick3' : (205, 38, 38),             # <<<<<<<<<<<<<<
 *     'firebrick4' : (139, 26, 26),
 *     'floralwhite' : (255, 250, 240),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_firebrick3, __pyx_tuple__130) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":141
 *     'firebrick2' : (238, 44, 44),
 *     'firebrick3' : (205, 38, 38),
 *     'firebrick4' : (139, 26, 26),             # <<<<<<<<<<<<<<
 *     'floralwhite' : (255, 250, 240),
 *     'forestgreen' : (34, 139, 34),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_firebrick4, __pyx_tuple__131) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":142
 *     'firebrick3' : (205, 38, 38),
 *     'firebrick4' : (139, 26, 26),
 *     'floralwhite' : (255, 250, 240),             # <<<<<<<<<<<<<<
 *     'forestgreen' : (34, 139, 34),
 *     'gainsboro' : (220, 220, 220),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_floralwhite, __pyx_tuple__132) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":143
 *     'firebrick4' : (139, 26, 26),
 *     'floralwhite' : (255, 250, 240),
 *     'forestgreen' : (34, 139, 34),             # <<<<<<<<<<<<<<
 *     'gainsboro' : (220, 220, 220),
 *     'ghostwhite' : (248, 248, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_forestgreen, __pyx_tuple__133) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":144
 *     'floralwhite' : (255, 250, 240),
 *     'forestgreen' : (34, 139, 34),
 *     'gainsboro' : (220, 220, 220),             # <<<<<<<<<<<<<<
 *     'ghostwhite' : (248, 248, 255),
 *     'gold' : (255, 215, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gainsboro, __pyx_tuple__134) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":145
 *     'forestgreen' : (34, 139, 34),
 *     'gainsboro' : (220, 220, 220),
 *     'ghostwhite' : (248, 248, 255),             # <<<<<<<<<<<<<<
 *     'gold' : (255, 215, 0),
 *     'gold1' : (255, 215, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_ghostwhite, __pyx_tuple__135) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":146
 *     'gainsboro' : (220, 220, 220),
 *     'ghostwhite' : (248, 248, 255),
 *     'gold' : (255, 215, 0),             # <<<<<<<<<<<<<<
 *     'gold1' : (255, 215, 0),
 *     'gold2' : (238, 201, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gold, __pyx_tuple__136) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":147
 *     'ghostwhite' : (248, 248, 255),
 *     'gold' : (255, 215, 0),
 *     'gold1' : (255, 215, 0),             # <<<<<<<<<<<<<<
 *     'gold2' : (238, 201, 0),
 *     'gold3' : (205, 173, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gold1, __pyx_tuple__136) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":148
 *     'gold' : (255, 215, 0),
 *     'gold1' : (255, 215, 0),
 *     'gold2' : (238, 201, 0),             # <<<<<<<<<<<<<<
 *     'gold3' : (205, 173, 0),
 *     'gold4' : (139, 117, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gold2, __pyx_tuple__137) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":149
 *     'gold1' : (255, 215, 0),
 *     'gold2' : (238, 201, 0),
 *     'gold3' : (205, 173, 0),             # <<<<<<<<<<<<<<
 *     'gold4' : (139, 117, 0),
 *     'goldenrod' : (218, 165, 32),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gold3, __pyx_tuple__138) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":150
 *     'gold2' : (238, 201, 0),
 *     'gold3' : (205, 173, 0),
 *     'gold4' : (139, 117, 0),             # <<<<<<<<<<<<<<
 *     'goldenrod' : (218, 165, 32),
 *     'goldenrod1' : (255, 193, 37),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gold4, __pyx_tuple__139) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":151
 *     'gold3' : (205, 173, 0),
 *     'gold4' : (139, 117, 0),
 *     'goldenrod' : (218, 165, 32),             # <<<<<<<<<<<<<<
 *     'goldenrod1' : (255, 193, 37),
 *     'goldenrod2' : (238, 180, 34),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_goldenrod, __pyx_tuple__140) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":152
 *     'gold4' : (139, 117, 0),
 *     'goldenrod' : (218, 165, 32),
 *     'goldenrod1' : (255, 193, 37),             # <<<<<<<<<<<<<<
 *     'goldenrod2' : (238, 180, 34),
 *     'goldenrod3' : (205, 155, 29),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_goldenrod1, __pyx_tuple__141) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":153
 *     'goldenrod' : (218, 165, 32),
 *     'goldenrod1' : (255, 193, 37),
 *     'goldenrod2' : (238, 180, 34),             # <<<<<<<<<<<<<<
 *     'goldenrod3' : (205, 155, 29),
 *     'goldenrod4' : (139, 105, 20),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_goldenrod2, __pyx_tuple__142) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":154
 *     'goldenrod1' : (255, 193, 37),
 *     'goldenrod2' : (238, 180, 34),
 *     'goldenrod3' : (205, 155, 29),             # <<<<<<<<<<<<<<
 *     'goldenrod4' : (139, 105, 20),
 *     'gray' : (190, 190, 190),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_goldenrod3, __pyx_tuple__143) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":155
 *     'goldenrod2' : (238, 180, 34),
 *     'goldenrod3' : (205, 155, 29),
 *     'goldenrod4' : (139, 105, 20),             # <<<<<<<<<<<<<<
 *     'gray' : (190, 190, 190),
 *     'gray0' : (0, 0, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_goldenrod4, __pyx_tuple__144) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":156
 *     'goldenrod3' : (205, 155, 29),
 *     'goldenrod4' : (139, 105, 20),
 *     'gray' : (190, 190, 190),             # <<<<<<<<<<<<<<
 *     'gray0' : (0, 0, 0),
 *     'gray1' : (3, 3, 3),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray, __pyx_tuple__145) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":157
 *     'goldenrod4' : (139, 105, 20),
 *     'gray' : (190, 190, 190),
 *     'gray0' : (0, 0, 0),             # <<<<<<<<<<<<<<
 *     'gray1' : (3, 3, 3),
 *     'gray10' : (26, 26, 26),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray0, __pyx_tuple__29) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":158
 *     'gray' : (190, 190, 190),
 *     'gray0' : (0, 0, 0),
 *     'gray1' : (3, 3, 3),             # <<<<<<<<<<<<<<
 *     'gray10' : (26, 26, 26),
 *     'gray100' : (255, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray1, __pyx_tuple__146) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":159
 *     'gray0' : (0, 0, 0),
 *     'gray1' : (3, 3, 3),
 *     'gray10' : (26, 26, 26),             # <<<<<<<<<<<<<<
 *     'gray100' : (255, 255, 255),
 *     'gray11' : (28, 28, 28),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray10, __pyx_tuple__147) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":160
 *     'gray1' : (3, 3, 3),
 *     'gray10' : (26, 26, 26),
 *     'gray100' : (255, 255, 255),             # <<<<<<<<<<<<<<
 *     'gray11' : (28, 28, 28),
 *     'gray12' : (31, 31, 31),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray100, __pyx_tuple__148) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":161
 *     'gray10' : (26, 26, 26),
 *     'gray100' : (255, 255, 255),
 *     'gray11' : (28, 28, 28),             # <<<<<<<<<<<<<<
 *     'gray12' : (31, 31, 31),
 *     'gray13' : (33, 33, 33),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray11, __pyx_tuple__149) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":162
 *     'gray100' : (255, 255, 255),
 *     'gray11' : (28, 28, 28),
 *     'gray12' : (31, 31, 31),             # <<<<<<<<<<<<<<
 *     'gray13' : (33, 33, 33),
 *     'gray14' : (36, 36, 36),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray12, __pyx_tuple__150) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":163
 *     'gray11' : (28, 28, 28),
 *     'gray12' : (31, 31, 31),
 *     'gray13' : (33, 33, 33),             # <<<<<<<<<<<<<<
 *     'gray14' : (36, 36, 36),
 *     'gray15' : (38, 38, 38),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray13, __pyx_tuple__151) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":164
 *     'gray12' : (31, 31, 31),
 *     'gray13' : (33, 33, 33),
 *     'gray14' : (36, 36, 36),             # <<<<<<<<<<<<<<
 *     'gray15' : (38, 38, 38),
 *     'gray16' : (41, 41, 41),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray14, __pyx_tuple__152) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":165
 *     'gray13' : (33, 33, 33),
 *     'gray14' : (36, 36, 36),
 *     'gray15' : (38, 38, 38),             # <<<<<<<<<<<<<<
 *     'gray16' : (41, 41, 41),
 *     'gray17' : (43, 43, 43),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray15, __pyx_tuple__153) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":166
 *     'gray14' : (36, 36, 36),
 *     'gray15' : (38, 38, 38),
 *     'gray16' : (41, 41, 41),             # <<<<<<<<<<<<<<
 *     'gray17' : (43, 43, 43),
 *     'gray18' : (46, 46, 46),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray16, __pyx_tuple__154) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":167
 *     'gray15' : (38, 38, 38),
 *     'gray16' : (41, 41, 41),
 *     'gray17' : (43, 43, 43),             # <<<<<<<<<<<<<<
 *     'gray18' : (46, 46, 46),
 *     'gray19' : (48, 48, 48),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray17, __pyx_tuple__155) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":168
 *     'gray16' : (41, 41, 41),
 *     'gray17' : (43, 43, 43),
 *     'gray18' : (46, 46, 46),             # <<<<<<<<<<<<<<
 *     'gray19' : (48, 48, 48),
 *     'gray2' : (5, 5, 5),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray18, __pyx_tuple__156) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":169
 *     'gray17' : (43, 43, 43),
 *     'gray18' : (46, 46, 46),
 *     'gray19' : (48, 48, 48),             # <<<<<<<<<<<<<<
 *     'gray2' : (5, 5, 5),
 *     'gray20' : (51, 51, 51),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray19, __pyx_tuple__157) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":170
 *     'gray18' : (46, 46, 46),
 *     'gray19' : (48, 48, 48),
 *     'gray2' : (5, 5, 5),             # <<<<<<<<<<<<<<
 *     'gray20' : (51, 51, 51),
 *     'gray21' : (54, 54, 54),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray2, __pyx_tuple__158) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":171
 *     'gray19' : (48, 48, 48),
 *     'gray2' : (5, 5, 5),
 *     'gray20' : (51, 51, 51),             # <<<<<<<<<<<<<<
 *     'gray21' : (54, 54, 54),
 *     'gray22' : (56, 56, 56),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray20, __pyx_tuple__159) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":172
 *     'gray2' : (5, 5, 5),
 *     'gray20' : (51, 51, 51),
 *     'gray21' : (54, 54, 54),             # <<<<<<<<<<<<<<
 *     'gray22' : (56, 56, 56),
 *     'gray23' : (59, 59, 59),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray21, __pyx_tuple__160) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":173
 *     'gray20' : (51, 51, 51),
 *     'gray21' : (54, 54, 54),
 *     'gray22' : (56, 56, 56),             # <<<<<<<<<<<<<<
 *     'gray23' : (59, 59, 59),
 *     'gray24' : (61, 61, 61),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray22, __pyx_tuple__161) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":174
 *     'gray21' : (54, 54, 54),
 *     'gray22' : (56, 56, 56),
 *     'gray23' : (59, 59, 59),             # <<<<<<<<<<<<<<
 *     'gray24' : (61, 61, 61),
 *     'gray25' : (64, 64, 64),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray23, __pyx_tuple__162) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":175
 *     'gray22' : (56, 56, 56),
 *     'gray23' : (59, 59, 59),
 *     'gray24' : (61, 61, 61),             # <<<<<<<<<<<<<<
 *     'gray25' : (64, 64, 64),
 *     'gray26' : (66, 66, 66),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray24, __pyx_tuple__163) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":176
 *     'gray23' : (59, 59, 59),
 *     'gray24' : (61, 61, 61),
 *     'gray25' : (64, 64, 64),             # <<<<<<<<<<<<<<
 *     'gray26' : (66, 66, 66),
 *     'gray27' : (69, 69, 69),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray25, __pyx_tuple__164) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":177
 *     'gray24' : (61, 61, 61),
 *     'gray25' : (64, 64, 64),
 *     'gray26' : (66, 66, 66),             # <<<<<<<<<<<<<<
 *     'gray27' : (69, 69, 69),
 *     'gray28' : (71, 71, 71),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray26, __pyx_tuple__165) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":178
 *     'gray25' : (64, 64, 64),
 *     'gray26' : (66, 66, 66),
 *     'gray27' : (69, 69, 69),             # <<<<<<<<<<<<<<
 *     'gray28' : (71, 71, 71),
 *     'gray29' : (74, 74, 74),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray27, __pyx_tuple__166) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":179
 *     'gray26' : (66, 66, 66),
 *     'gray27' : (69, 69, 69),
 *     'gray28' : (71, 71, 71),             # <<<<<<<<<<<<<<
 *     'gray29' : (74, 74, 74),
 *     'gray3' : (8, 8, 8),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray28, __pyx_tuple__167) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":180
 *     'gray27' : (69, 69, 69),
 *     'gray28' : (71, 71, 71),
 *     'gray29' : (74, 74, 74),             # <<<<<<<<<<<<<<
 *     'gray3' : (8, 8, 8),
 *     'gray30' : (77, 77, 77),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray29, __pyx_tuple__168) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":181
 *     'gray28' : (71, 71, 71),
 *     'gray29' : (74, 74, 74),
 *     'gray3' : (8, 8, 8),             # <<<<<<<<<<<<<<
 *     'gray30' : (77, 77, 77),
 *     'gray31' : (79, 79, 79),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray3, __pyx_tuple__169) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":182
 *     'gray29' : (74, 74, 74),
 *     'gray3' : (8, 8, 8),
 *     'gray30' : (77, 77, 77),             # <<<<<<<<<<<<<<
 *     'gray31' : (79, 79, 79),
 *     'gray32' : (82, 82, 82),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray30, __pyx_tuple__170) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":183
 *     'gray3' : (8, 8, 8),
 *     'gray30' : (77, 77, 77),
 *     'gray31' : (79, 79, 79),             # <<<<<<<<<<<<<<
 *     'gray32' : (82, 82, 82),
 *     'gray33' : (84, 84, 84),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray31, __pyx_tuple__171) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":184
 *     'gray30' : (77, 77, 77),
 *     'gray31' : (79, 79, 79),
 *     'gray32' : (82, 82, 82),             # <<<<<<<<<<<<<<
 *     'gray33' : (84, 84, 84),
 *     'gray34' : (87, 87, 87),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray32, __pyx_tuple__172) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":185
 *     'gray31' : (79, 79, 79),
 *     'gray32' : (82, 82, 82),
 *     'gray33' : (84, 84, 84),             # <<<<<<<<<<<<<<
 *     'gray34' : (87, 87, 87),
 *     'gray35' : (89, 89, 89),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray33, __pyx_tuple__173) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":186
 *     'gray32' : (82, 82, 82),
 *     'gray33' : (84, 84, 84),
 *     'gray34' : (87, 87, 87),             # <<<<<<<<<<<<<<
 *     'gray35' : (89, 89, 89),
 *     'gray36' : (92, 92, 92),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray34, __pyx_tuple__174) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":187
 *     'gray33' : (84, 84, 84),
 *     'gray34' : (87, 87, 87),
 *     'gray35' : (89, 89, 89),             # <<<<<<<<<<<<<<
 *     'gray36' : (92, 92, 92),
 *     'gray37' : (94, 94, 94),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray35, __pyx_tuple__175) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":188
 *     'gray34' : (87, 87, 87),
 *     'gray35' : (89, 89, 89),
 *     'gray36' : (92, 92, 92),             # <<<<<<<<<<<<<<
 *     'gray37' : (94, 94, 94),
 *     'gray38' : (97, 97, 97),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray36, __pyx_tuple__176) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":189
 *     'gray35' : (89, 89, 89),
 *     'gray36' : (92, 92, 92),
 *     'gray37' : (94, 94, 94),             # <<<<<<<<<<<<<<
 *     'gray38' : (97, 97, 97),
 *     'gray39' : (99, 99, 99),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray37, __pyx_tuple__177) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":190
 *     'gray36' : (92, 92, 92),
 *     'gray37' : (94, 94, 94),
 *     'gray38' : (97, 97, 97),             # <<<<<<<<<<<<<<
 *     'gray39' : (99, 99, 99),
 *     'gray4' : (10, 10, 10),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray38, __pyx_tuple__178) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":191
 *     'gray37' : (94, 94, 94),
 *     'gray38' : (97, 97, 97),
 *     'gray39' : (99, 99, 99),             # <<<<<<<<<<<<<<
 *     'gray4' : (10, 10, 10),
 *     'gray40' : (102, 102, 102),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray39, __pyx_tuple__179) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":192
 *     'gray38' : (97, 97, 97),
 *     'gray39' : (99, 99, 99),
 *     'gray4' : (10, 10, 10),             # <<<<<<<<<<<<<<
 *     'gray40' : (102, 102, 102),
 *     'gray41' : (105, 105, 105),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray4, __pyx_tuple__180) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":193
 *     'gray39' : (99, 99, 99),
 *     'gray4' : (10, 10, 10),
 *     'gray40' : (102, 102, 102),             # <<<<<<<<<<<<<<
 *     'gray41' : (105, 105, 105),
 *     'gray42' : (107, 107, 107),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray40, __pyx_tuple__181) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":194
 *     'gray4' : (10, 10, 10),
 *     'gray40' : (102, 102, 102),
 *     'gray41' : (105, 105, 105),             # <<<<<<<<<<<<<<
 *     'gray42' : (107, 107, 107),
 *     'gray43' : (110, 110, 110),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray41, __pyx_tuple__122) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":195
 *     'gray40' : (102, 102, 102),
 *     'gray41' : (105, 105, 105),
 *     'gray42' : (107, 107, 107),             # <<<<<<<<<<<<<<
 *     'gray43' : (110, 110, 110),
 *     'gray44' : (112, 112, 112),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray42, __pyx_tuple__182) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":196
 *     'gray41' : (105, 105, 105),
 *     'gray42' : (107, 107, 107),
 *     'gray43' : (110, 110, 110),             # <<<<<<<<<<<<<<
 *     'gray44' : (112, 112, 112),
 *     'gray45' : (115, 115, 115),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray43, __pyx_tuple__183) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":197
 *     'gray42' : (107, 107, 107),
 *     'gray43' : (110, 110, 110),
 *     'gray44' : (112, 112, 112),             # <<<<<<<<<<<<<<
 *     'gray45' : (115, 115, 115),
 *     'gray46' : (117, 117, 117),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray44, __pyx_tuple__184) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":198
 *     'gray43' : (110, 110, 110),
 *     'gray44' : (112, 112, 112),
 *     'gray45' : (115, 115, 115),             # <<<<<<<<<<<<<<
 *     'gray46' : (117, 117, 117),
 *     'gray47' : (120, 120, 120),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray45, __pyx_tuple__185) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":199
 *     'gray44' : (112, 112, 112),
 *     'gray45' : (115, 115, 115),
 *     'gray46' : (117, 117, 117),             # <<<<<<<<<<<<<<
 *     'gray47' : (120, 120, 120),
 *     'gray48' : (122, 122, 122),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray46, __pyx_tuple__186) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":200
 *     'gray45' : (115, 115, 115),
 *     'gray46' : (117, 117, 117),
 *     'gray47' : (120, 120, 120),             # <<<<<<<<<<<<<<
 *     'gray48' : (122, 122, 122),
 *     'gray49' : (125, 125, 125),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray47, __pyx_tuple__187) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":201
 *     'gray46' : (117, 117, 117),
 *     'gray47' : (120, 120, 120),
 *     'gray48' : (122, 122, 122),             # <<<<<<<<<<<<<<
 *     'gray49' : (125, 125, 125),
 *     'gray5' : (13, 13, 13),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray48, __pyx_tuple__188) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":202
 *     'gray47' : (120, 120, 120),
 *     'gray48' : (122, 122, 122),
 *     'gray49' : (125, 125, 125),             # <<<<<<<<<<<<<<
 *     'gray5' : (13, 13, 13),
 *     'gray50' : (127, 127, 127),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray49, __pyx_tuple__189) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":203
 *     'gray48' : (122, 122, 122),
 *     'gray49' : (125, 125, 125),
 *     'gray5' : (13, 13, 13),             # <<<<<<<<<<<<<<
 *     'gray50' : (127, 127, 127),
 *     'gray51' : (130, 130, 130),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray5, __pyx_tuple__190) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":204
 *     'gray49' : (125, 125, 125),
 *     'gray5' : (13, 13, 13),
 *     'gray50' : (127, 127, 127),             # <<<<<<<<<<<<<<
 *     'gray51' : (130, 130, 130),
 *     'gray52' : (133, 133, 133),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray50, __pyx_tuple__191) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":205
 *     'gray5' : (13, 13, 13),
 *     'gray50' : (127, 127, 127),
 *     'gray51' : (130, 130, 130),             # <<<<<<<<<<<<<<
 *     'gray52' : (133, 133, 133),
 *     'gray53' : (135, 135, 135),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray51, __pyx_tuple__192) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":206
 *     'gray50' : (127, 127, 127),
 *     'gray51' : (130, 130, 130),
 *     'gray52' : (133, 133, 133),             # <<<<<<<<<<<<<<
 *     'gray53' : (135, 135, 135),
 *     'gray54' : (138, 138, 138),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray52, __pyx_tuple__193) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":207
 *     'gray51' : (130, 130, 130),
 *     'gray52' : (133, 133, 133),
 *     'gray53' : (135, 135, 135),             # <<<<<<<<<<<<<<
 *     'gray54' : (138, 138, 138),
 *     'gray55' : (140, 140, 140),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray53, __pyx_tuple__194) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":208
 *     'gray52' : (133, 133, 133),
 *     'gray53' : (135, 135, 135),
 *     'gray54' : (138, 138, 138),             # <<<<<<<<<<<<<<
 *     'gray55' : (140, 140, 140),
 *     'gray56' : (143, 143, 143),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray54, __pyx_tuple__195) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":209
 *     'gray53' : (135, 135, 135),
 *     'gray54' : (138, 138, 138),
 *     'gray55' : (140, 140, 140),             # <<<<<<<<<<<<<<
 *     'gray56' : (143, 143, 143),
 *     'gray57' : (145, 145, 145),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray55, __pyx_tuple__196) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":210
 *     'gray54' : (138, 138, 138),
 *     'gray55' : (140, 140, 140),
 *     'gray56' : (143, 143, 143),             # <<<<<<<<<<<<<<
 *     'gray57' : (145, 145, 145),
 *     'gray58' : (148, 148, 148),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray56, __pyx_tuple__197) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":211
 *     'gray55' : (140, 140, 140),
 *     'gray56' : (143, 143, 143),
 *     'gray57' : (145, 145, 145),             # <<<<<<<<<<<<<<
 *     'gray58' : (148, 148, 148),
 *     'gray59' : (150, 150, 150),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray57, __pyx_tuple__198) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":212
 *     'gray56' : (143, 143, 143),
 *     'gray57' : (145, 145, 145),
 *     'gray58' : (148, 148, 148),             # <<<<<<<<<<<<<<
 *     'gray59' : (150, 150, 150),
 *     'gray6' : (15, 15, 15),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray58, __pyx_tuple__199) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":213
 *     'gray57' : (145, 145, 145),
 *     'gray58' : (148, 148, 148),
 *     'gray59' : (150, 150, 150),             # <<<<<<<<<<<<<<
 *     'gray6' : (15, 15, 15),
 *     'gray60' : (153, 153, 153),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray59, __pyx_tuple__200) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":214
 *     'gray58' : (148, 148, 148),
 *     'gray59' : (150, 150, 150),
 *     'gray6' : (15, 15, 15),             # <<<<<<<<<<<<<<
 *     'gray60' : (153, 153, 153),
 *     'gray61' : (156, 156, 156),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray6, __pyx_tuple__201) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":215
 *     'gray59' : (150, 150, 150),
 *     'gray6' : (15, 15, 15),
 *     'gray60' : (153, 153, 153),             # <<<<<<<<<<<<<<
 *     'gray61' : (156, 156, 156),
 *     'gray62' : (158, 158, 158),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray60, __pyx_tuple__202) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":216
 *     'gray6' : (15, 15, 15),
 *     'gray60' : (153, 153, 153),
 *     'gray61' : (156, 156, 156),             # <<<<<<<<<<<<<<
 *     'gray62' : (158, 158, 158),
 *     'gray63' : (161, 161, 161),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray61, __pyx_tuple__203) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":217
 *     'gray60' : (153, 153, 153),
 *     'gray61' : (156, 156, 156),
 *     'gray62' : (158, 158, 158),             # <<<<<<<<<<<<<<
 *     'gray63' : (161, 161, 161),
 *     'gray64' : (163, 163, 163),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray62, __pyx_tuple__204) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":218
 *     'gray61' : (156, 156, 156),
 *     'gray62' : (158, 158, 158),
 *     'gray63' : (161, 161, 161),             # <<<<<<<<<<<<<<
 *     'gray64' : (163, 163, 163),
 *     'gray65' : (166, 166, 166),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray63, __pyx_tuple__205) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":219
 *     'gray62' : (158, 158, 158),
 *     'gray63' : (161, 161, 161),
 *     'gray64' : (163, 163, 163),             # <<<<<<<<<<<<<<
 *     'gray65' : (166, 166, 166),
 *     'gray66' : (168, 168, 168),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray64, __pyx_tuple__206) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":220
 *     'gray63' : (161, 161, 161),
 *     'gray64' : (163, 163, 163),
 *     'gray65' : (166, 166, 166),             # <<<<<<<<<<<<<<
 *     'gray66' : (168, 168, 168),
 *     'gray67' : (171, 171, 171),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray65, __pyx_tuple__207) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":221
 *     'gray64' : (163, 163, 163),
 *     'gray65' : (166, 166, 166),
 *     'gray66' : (168, 168, 168),             # <<<<<<<<<<<<<<
 *     'gray67' : (171, 171, 171),
 *     'gray68' : (173, 173, 173),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray66, __pyx_tuple__208) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":222
 *     'gray65' : (166, 166, 166),
 *     'gray66' : (168, 168, 168),
 *     'gray67' : (171, 171, 171),             # <<<<<<<<<<<<<<
 *     'gray68' : (173, 173, 173),
 *     'gray69' : (176, 176, 176),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray67, __pyx_tuple__209) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":223
 *     'gray66' : (168, 168, 168),
 *     'gray67' : (171, 171, 171),
 *     'gray68' : (173, 173, 173),             # <<<<<<<<<<<<<<
 *     'gray69' : (176, 176, 176),
 *     'gray7' : (18, 18, 18),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray68, __pyx_tuple__210) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":224
 *     'gray67' : (171, 171, 171),
 *     'gray68' : (173, 173, 173),
 *     'gray69' : (176, 176, 176),             # <<<<<<<<<<<<<<
 *     'gray7' : (18, 18, 18),
 *     'gray70' : (179, 179, 179),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray69, __pyx_tuple__211) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":225
 *     'gray68' : (173, 173, 173),
 *     'gray69' : (176, 176, 176),
 *     'gray7' : (18, 18, 18),             # <<<<<<<<<<<<<<
 *     'gray70' : (179, 179, 179),
 *     'gray71' : (181, 181, 181),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray7, __pyx_tuple__212) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":226
 *     'gray69' : (176, 176, 176),
 *     'gray7' : (18, 18, 18),
 *     'gray70' : (179, 179, 179),             # <<<<<<<<<<<<<<
 *     'gray71' : (181, 181, 181),
 *     'gray72' : (184, 184, 184),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray70, __pyx_tuple__213) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":227
 *     'gray7' : (18, 18, 18),
 *     'gray70' : (179, 179, 179),
 *     'gray71' : (181, 181, 181),             # <<<<<<<<<<<<<<
 *     'gray72' : (184, 184, 184),
 *     'gray73' : (186, 186, 186),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray71, __pyx_tuple__214) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":228
 *     'gray70' : (179, 179, 179),
 *     'gray71' : (181, 181, 181),
 *     'gray72' : (184, 184, 184),             # <<<<<<<<<<<<<<
 *     'gray73' : (186, 186, 186),
 *     'gray74' : (189, 189, 189),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray72, __pyx_tuple__215) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":229
 *     'gray71' : (181, 181, 181),
 *     'gray72' : (184, 184, 184),
 *     'gray73' : (186, 186, 186),             # <<<<<<<<<<<<<<
 *     'gray74' : (189, 189, 189),
 *     'gray75' : (191, 191, 191),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray73, __pyx_tuple__216) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":230
 *     'gray72' : (184, 184, 184),
 *     'gray73' : (186, 186, 186),
 *     'gray74' : (189, 189, 189),             # <<<<<<<<<<<<<<
 *     'gray75' : (191, 191, 191),
 *     'gray76' : (194, 194, 194),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray74, __pyx_tuple__217) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":231
 *     'gray73' : (186, 186, 186),
 *     'gray74' : (189, 189, 189),
 *     'gray75' : (191, 191, 191),             # <<<<<<<<<<<<<<
 *     'gray76' : (194, 194, 194),
 *     'gray77' : (196, 196, 196),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray75, __pyx_tuple__218) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":232
 *     'gray74' : (189, 189, 189),
 *     'gray75' : (191, 191, 191),
 *     'gray76' : (194, 194, 194),             # <<<<<<<<<<<<<<
 *     'gray77' : (196, 196, 196),
 *     'gray78' : (199, 199, 199),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray76, __pyx_tuple__219) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":233
 *     'gray75' : (191, 191, 191),
 *     'gray76' : (194, 194, 194),
 *     'gray77' : (196, 196, 196),             # <<<<<<<<<<<<<<
 *     'gray78' : (199, 199, 199),
 *     'gray79' : (201, 201, 201),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray77, __pyx_tuple__220) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":234
 *     'gray76' : (194, 194, 194),
 *     'gray77' : (196, 196, 196),
 *     'gray78' : (199, 199, 199),             # <<<<<<<<<<<<<<
 *     'gray79' : (201, 201, 201),
 *     'gray8' : (20, 20, 20),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray78, __pyx_tuple__221) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":235
 *     'gray77' : (196, 196, 196),
 *     'gray78' : (199, 199, 199),
 *     'gray79' : (201, 201, 201),             # <<<<<<<<<<<<<<
 *     'gray8' : (20, 20, 20),
 *     'gray80' : (204, 204, 204),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray79, __pyx_tuple__222) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":236
 *     'gray78' : (199, 199, 199),
 *     'gray79' : (201, 201, 201),
 *     'gray8' : (20, 20, 20),             # <<<<<<<<<<<<<<
 *     'gray80' : (204, 204, 204),
 *     'gray81' : (207, 207, 207),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray8, __pyx_tuple__223) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":237
 *     'gray79' : (201, 201, 201),
 *     'gray8' : (20, 20, 20),
 *     'gray80' : (204, 204, 204),             # <<<<<<<<<<<<<<
 *     'gray81' : (207, 207, 207),
 *     'gray82' : (209, 209, 209),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray80, __pyx_tuple__224) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":238
 *     'gray8' : (20, 20, 20),
 *     'gray80' : (204, 204, 204),
 *     'gray81' : (207, 207, 207),             # <<<<<<<<<<<<<<
 *     'gray82' : (209, 209, 209),
 *     'gray83' : (212, 212, 212),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray81, __pyx_tuple__225) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":239
 *     'gray80' : (204, 204, 204),
 *     'gray81' : (207, 207, 207),
 *     'gray82' : (209, 209, 209),             # <<<<<<<<<<<<<<
 *     'gray83' : (212, 212, 212),
 *     'gray84' : (214, 214, 214),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray82, __pyx_tuple__226) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":240
 *     'gray81' : (207, 207, 207),
 *     'gray82' : (209, 209, 209),
 *     'gray83' : (212, 212, 212),             # <<<<<<<<<<<<<<
 *     'gray84' : (214, 214, 214),
 *     'gray85' : (217, 217, 217),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray83, __pyx_tuple__227) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":241
 *     'gray82' : (209, 209, 209),
 *     'gray83' : (212, 212, 212),
 *     'gray84' : (214, 214, 214),             # <<<<<<<<<<<<<<
 *     'gray85' : (217, 217, 217),
 *     'gray86' : (219, 219, 219),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray84, __pyx_tuple__228) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":242
 *     'gray83' : (212, 212, 212),
 *     'gray84' : (214, 214, 214),
 *     'gray85' : (217, 217, 217),             # <<<<<<<<<<<<<<
 *     'gray86' : (219, 219, 219),
 *     'gray87' : (222, 222, 222),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray85, __pyx_tuple__229) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":243
 *     'gray84' : (214, 214, 214),
 *     'gray85' : (217, 217, 217),
 *     'gray86' : (219, 219, 219),             # <<<<<<<<<<<<<<
 *     'gray87' : (222, 222, 222),
 *     'gray88' : (224, 224, 224),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray86, __pyx_tuple__230) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":244
 *     'gray85' : (217, 217, 217),
 *     'gray86' : (219, 219, 219),
 *     'gray87' : (222, 222, 222),             # <<<<<<<<<<<<<<
 *     'gray88' : (224, 224, 224),
 *     'gray89' : (227, 227, 227),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray87, __pyx_tuple__231) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":245
 *     'gray86' : (219, 219, 219),
 *     'gray87' : (222, 222, 222),
 *     'gray88' : (224, 224, 224),             # <<<<<<<<<<<<<<
 *     'gray89' : (227, 227, 227),
 *     'gray9' : (23, 23, 23),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray88, __pyx_tuple__232) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":246
 *     'gray87' : (222, 222, 222),
 *     'gray88' : (224, 224, 224),
 *     'gray89' : (227, 227, 227),             # <<<<<<<<<<<<<<
 *     'gray9' : (23, 23, 23),
 *     'gray90' : (229, 229, 229),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray89, __pyx_tuple__233) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":247
 *     'gray88' : (224, 224, 224),
 *     'gray89' : (227, 227, 227),
 *     'gray9' : (23, 23, 23),             # <<<<<<<<<<<<<<
 *     'gray90' : (229, 229, 229),
 *     'gray91' : (232, 232, 232),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray9, __pyx_tuple__234) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":248
 *     'gray89' : (227, 227, 227),
 *     'gray9' : (23, 23, 23),
 *     'gray90' : (229, 229, 229),             # <<<<<<<<<<<<<<
 *     'gray91' : (232, 232, 232),
 *     'gray92' : (235, 235, 235),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray90, __pyx_tuple__235) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":249
 *     'gray9' : (23, 23, 23),
 *     'gray90' : (229, 229, 229),
 *     'gray91' : (232, 232, 232),             # <<<<<<<<<<<<<<
 *     'gray92' : (235, 235, 235),
 *     'gray93' : (237, 237, 237),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray91, __pyx_tuple__236) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":250
 *     'gray90' : (229, 229, 229),
 *     'gray91' : (232, 232, 232),
 *     'gray92' : (235, 235, 235),             # <<<<<<<<<<<<<<
 *     'gray93' : (237, 237, 237),
 *     'gray94' : (240, 240, 240),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray92, __pyx_tuple__237) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":251
 *     'gray91' : (232, 232, 232),
 *     'gray92' : (235, 235, 235),
 *     'gray93' : (237, 237, 237),             # <<<<<<<<<<<<<<
 *     'gray94' : (240, 240, 240),
 *     'gray95' : (242, 242, 242),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray93, __pyx_tuple__238) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":252
 *     'gray92' : (235, 235, 235),
 *     'gray93' : (237, 237, 237),
 *     'gray94' : (240, 240, 240),             # <<<<<<<<<<<<<<
 *     'gray95' : (242, 242, 242),
 *     'gray96' : (245, 245, 245),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray94, __pyx_tuple__239) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":253
 *     'gray93' : (237, 237, 237),
 *     'gray94' : (240, 240, 240),
 *     'gray95' : (242, 242, 242),             # <<<<<<<<<<<<<<
 *     'gray96' : (245, 245, 245),
 *     'gray97' : (247, 247, 247),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray95, __pyx_tuple__240) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":254
 *     'gray94' : (240, 240, 240),
 *     'gray95' : (242, 242, 242),
 *     'gray96' : (245, 245, 245),             # <<<<<<<<<<<<<<
 *     'gray97' : (247, 247, 247),
 *     'gray98' : (250, 250, 250),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray96, __pyx_tuple__241) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":255
 *     'gray95' : (242, 242, 242),
 *     'gray96' : (245, 245, 245),
 *     'gray97' : (247, 247, 247),             # <<<<<<<<<<<<<<
 *     'gray98' : (250, 250, 250),
 *     'gray99' : (252, 252, 252),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray97, __pyx_tuple__242) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":256
 *     'gray96' : (245, 245, 245),
 *     'gray97' : (247, 247, 247),
 *     'gray98' : (250, 250, 250),             # <<<<<<<<<<<<<<
 *     'gray99' : (252, 252, 252),
 *     'green' : (0, 255, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray98, __pyx_tuple__243) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":257
 *     'gray97' : (247, 247, 247),
 *     'gray98' : (250, 250, 250),
 *     'gray99' : (252, 252, 252),             # <<<<<<<<<<<<<<
 *     'green' : (0, 255, 0),
 *     'green1' : (0, 255, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_gray99, __pyx_tuple__244) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":258
 *     'gray98' : (250, 250, 250),
 *     'gray99' : (252, 252, 252),
 *     'green' : (0, 255, 0),             # <<<<<<<<<<<<<<
 *     'green1' : (0, 255, 0),
 *     'green2' : (0, 238, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_green, __pyx_tuple__245) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":259
 *     'gray99' : (252, 252, 252),
 *     'green' : (0, 255, 0),
 *     'green1' : (0, 255, 0),             # <<<<<<<<<<<<<<
 *     'green2' : (0, 238, 0),
 *     'green3' : (0, 205, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_green1, __pyx_tuple__245) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":260
 *     'green' : (0, 255, 0),
 *     'green1' : (0, 255, 0),
 *     'green2' : (0, 238, 0),             # <<<<<<<<<<<<<<
 *     'green3' : (0, 205, 0),
 *     'green4' : (0, 139, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_green2, __pyx_tuple__246) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":261
 *     'green1' : (0, 255, 0),
 *     'green2' : (0, 238, 0),
 *     'green3' : (0, 205, 0),             # <<<<<<<<<<<<<<
 *     'green4' : (0, 139, 0),
 *     'greenyellow' : (173, 255, 47),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_green3, __pyx_tuple__247) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":262
 *     'green2' : (0, 238, 0),
 *     'green3' : (0, 205, 0),
 *     'green4' : (0, 139, 0),             # <<<<<<<<<<<<<<
 *     'greenyellow' : (173, 255, 47),
 *     'grey' : (190, 190, 190),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_green4, __pyx_tuple__248) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":263
 *     'green3' : (0, 205, 0),
 *     'green4' : (0, 139, 0),
 *     'greenyellow' : (173, 255, 47),             # <<<<<<<<<<<<<<
 *     'grey' : (190, 190, 190),
 *     'grey0' : (0, 0, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_greenyellow, __pyx_tuple__249) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":264
 *     'green4' : (0, 139, 0),
 *     'greenyellow' : (173, 255, 47),
 *     'grey' : (190, 190, 190),             # <<<<<<<<<<<<<<
 *     'grey0' : (0, 0, 0),
 *     'grey1' : (3, 3, 3),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey, __pyx_tuple__145) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":265
 *     'greenyellow' : (173, 255, 47),
 *     'grey' : (190, 190, 190),
 *     'grey0' : (0, 0, 0),             # <<<<<<<<<<<<<<
 *     'grey1' : (3, 3, 3),
 *     'grey10' : (26, 26, 26),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey0, __pyx_tuple__29) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":266
 *     'grey' : (190, 190, 190),
 *     'grey0' : (0, 0, 0),
 *     'grey1' : (3, 3, 3),             # <<<<<<<<<<<<<<
 *     'grey10' : (26, 26, 26),
 *     'grey100' : (255, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey1, __pyx_tuple__146) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":267
 *     'grey0' : (0, 0, 0),
 *     'grey1' : (3, 3, 3),
 *     'grey10' : (26, 26, 26),             # <<<<<<<<<<<<<<
 *     'grey100' : (255, 255, 255),
 *     'grey11' : (28, 28, 28),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey10, __pyx_tuple__147) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":268
 *     'grey1' : (3, 3, 3),
 *     'grey10' : (26, 26, 26),
 *     'grey100' : (255, 255, 255),             # <<<<<<<<<<<<<<
 *     'grey11' : (28, 28, 28),
 *     'grey12' : (31, 31, 31),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey100, __pyx_tuple__148) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":269
 *     'grey10' : (26, 26, 26),
 *     'grey100' : (255, 255, 255),
 *     'grey11' : (28, 28, 28),             # <<<<<<<<<<<<<<
 *     'grey12' : (31, 31, 31),
 *     'grey13' : (33, 33, 33),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey11, __pyx_tuple__149) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":270
 *     'grey100' : (255, 255, 255),
 *     'grey11' : (28, 28, 28),
 *     'grey12' : (31, 31, 31),             # <<<<<<<<<<<<<<
 *     'grey13' : (33, 33, 33),
 *     'grey14' : (36, 36, 36),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey12, __pyx_tuple__150) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":271
 *     'grey11' : (28, 28, 28),
 *     'grey12' : (31, 31, 31),
 *     'grey13' : (33, 33, 33),             # <<<<<<<<<<<<<<
 *     'grey14' : (36, 36, 36),
 *     'grey15' : (38, 38, 38),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey13, __pyx_tuple__151) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":272
 *     'grey12' : (31, 31, 31),
 *     'grey13' : (33, 33, 33),
 *     'grey14' : (36, 36, 36),             # <<<<<<<<<<<<<<
 *     'grey15' : (38, 38, 38),
 *     'grey16' : (41, 41, 41),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey14, __pyx_tuple__152) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":273
 *     'grey13' : (33, 33, 33),
 *     'grey14' : (36, 36, 36),
 *     'grey15' : (38, 38, 38),             # <<<<<<<<<<<<<<
 *     'grey16' : (41, 41, 41),
 *     'grey17' : (43, 43, 43),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey15, __pyx_tuple__153) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":274
 *     'grey14' : (36, 36, 36),
 *     'grey15' : (38, 38, 38),
 *     'grey16' : (41, 41, 41),             # <<<<<<<<<<<<<<
 *     'grey17' : (43, 43, 43),
 *     'grey18' : (46, 46, 46),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey16, __pyx_tuple__154) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":275
 *     'grey15' : (38, 38, 38),
 *     'grey16' : (41, 41, 41),
 *     'grey17' : (43, 43, 43),             # <<<<<<<<<<<<<<
 *     'grey18' : (46, 46, 46),
 *     'grey19' : (48, 48, 48),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey17, __pyx_tuple__155) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":276
 *     'grey16' : (41, 41, 41),
 *     'grey17' : (43, 43, 43),
 *     'grey18' : (46, 46, 46),             # <<<<<<<<<<<<<<
 *     'grey19' : (48, 48, 48),
 *     'grey2' : (5, 5, 5),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey18, __pyx_tuple__156) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":277
 *     'grey17' : (43, 43, 43),
 *     'grey18' : (46, 46, 46),
 *     'grey19' : (48, 48, 48),             # <<<<<<<<<<<<<<
 *     'grey2' : (5, 5, 5),
 *     'grey20' : (51, 51, 51),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey19, __pyx_tuple__157) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":278
 *     'grey18' : (46, 46, 46),
 *     'grey19' : (48, 48, 48),
 *     'grey2' : (5, 5, 5),             # <<<<<<<<<<<<<<
 *     'grey20' : (51, 51, 51),
 *     'grey21' : (54, 54, 54),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey2, __pyx_tuple__158) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":279
 *     'grey19' : (48, 48, 48),
 *     'grey2' : (5, 5, 5),
 *     'grey20' : (51, 51, 51),             # <<<<<<<<<<<<<<
 *     'grey21' : (54, 54, 54),
 *     'grey22' : (56, 56, 56),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey20, __pyx_tuple__159) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":280
 *     'grey2' : (5, 5, 5),
 *     'grey20' : (51, 51, 51),
 *     'grey21' : (54, 54, 54),             # <<<<<<<<<<<<<<
 *     'grey22' : (56, 56, 56),
 *     'grey23' : (59, 59, 59),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey21, __pyx_tuple__160) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":281
 *     'grey20' : (51, 51, 51),
 *     'grey21' : (54, 54, 54),
 *     'grey22' : (56, 56, 56),             # <<<<<<<<<<<<<<
 *     'grey23' : (59, 59, 59),
 *     'grey24' : (61, 61, 61),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey22, __pyx_tuple__161) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":282
 *     'grey21' : (54, 54, 54),
 *     'grey22' : (56, 56, 56),
 *     'grey23' : (59, 59, 59),             # <<<<<<<<<<<<<<
 *     'grey24' : (61, 61, 61),
 *     'grey25' : (64, 64, 64),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey23, __pyx_tuple__162) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":283
 *     'grey22' : (56, 56, 56),
 *     'grey23' : (59, 59, 59),
 *     'grey24' : (61, 61, 61),             # <<<<<<<<<<<<<<
 *     'grey25' : (64, 64, 64),
 *     'grey26' : (66, 66, 66),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey24, __pyx_tuple__163) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":284
 *     'grey23' : (59, 59, 59),
 *     'grey24' : (61, 61, 61),
 *     'grey25' : (64, 64, 64),             # <<<<<<<<<<<<<<
 *     'grey26' : (66, 66, 66),
 *     'grey27' : (69, 69, 69),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey25, __pyx_tuple__164) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":285
 *     'grey24' : (61, 61, 61),
 *     'grey25' : (64, 64, 64),
 *     'grey26' : (66, 66, 66),             # <<<<<<<<<<<<<<
 *     'grey27' : (69, 69, 69),
 *     'grey28' : (71, 71, 71),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey26, __pyx_tuple__165) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":286
 *     'grey25' : (64, 64, 64),
 *     'grey26' : (66, 66, 66),
 *     'grey27' : (69, 69, 69),             # <<<<<<<<<<<<<<
 *     'grey28' : (71, 71, 71),
 *     'grey29' : (74, 74, 74),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey27, __pyx_tuple__166) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":287
 *     'grey26' : (66, 66, 66),
 *     'grey27' : (69, 69, 69),
 *     'grey28' : (71, 71, 71),             # <<<<<<<<<<<<<<
 *     'grey29' : (74, 74, 74),
 *     'grey3' : (8, 8, 8),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey28, __pyx_tuple__167) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":288
 *     'grey27' : (69, 69, 69),
 *     'grey28' : (71, 71, 71),
 *     'grey29' : (74, 74, 74),             # <<<<<<<<<<<<<<
 *     'grey3' : (8, 8, 8),
 *     'grey30' : (77, 77, 77),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey29, __pyx_tuple__168) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":289
 *     'grey28' : (71, 71, 71),
 *     'grey29' : (74, 74, 74),
 *     'grey3' : (8, 8, 8),             # <<<<<<<<<<<<<<
 *     'grey30' : (77, 77, 77),
 *     'grey31' : (79, 79, 79),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey3, __pyx_tuple__169) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":290
 *     'grey29' : (74, 74, 74),
 *     'grey3' : (8, 8, 8),
 *     'grey30' : (77, 77, 77),             # <<<<<<<<<<<<<<
 *     'grey31' : (79, 79, 79),
 *     'grey32' : (82, 82, 82),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey30, __pyx_tuple__170) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":291
 *     'grey3' : (8, 8, 8),
 *     'grey30' : (77, 77, 77),
 *     'grey31' : (79, 79, 79),             # <<<<<<<<<<<<<<
 *     'grey32' : (82, 82, 82),
 *     'grey33' : (84, 84, 84),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey31, __pyx_tuple__171) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":292
 *     'grey30' : (77, 77, 77),
 *     'grey31' : (79, 79, 79),
 *     'grey32' : (82, 82, 82),             # <<<<<<<<<<<<<<
 *     'grey33' : (84, 84, 84),
 *     'grey34' : (87, 87, 87),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey32, __pyx_tuple__172) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":293
 *     'grey31' : (79, 79, 79),
 *     'grey32' : (82, 82, 82),
 *     'grey33' : (84, 84, 84),             # <<<<<<<<<<<<<<
 *     'grey34' : (87, 87, 87),
 *     'grey35' : (89, 89, 89),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey33, __pyx_tuple__173) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":294
 *     'grey32' : (82, 82, 82),
 *     'grey33' : (84, 84, 84),
 *     'grey34' : (87, 87, 87),             # <<<<<<<<<<<<<<
 *     'grey35' : (89, 89, 89),
 *     'grey36' : (92, 92, 92),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey34, __pyx_tuple__174) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":295
 *     'grey33' : (84, 84, 84),
 *     'grey34' : (87, 87, 87),
 *     'grey35' : (89, 89, 89),             # <<<<<<<<<<<<<<
 *     'grey36' : (92, 92, 92),
 *     'grey37' : (94, 94, 94),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey35, __pyx_tuple__175) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":296
 *     'grey34' : (87, 87, 87),
 *     'grey35' : (89, 89, 89),
 *     'grey36' : (92, 92, 92),             # <<<<<<<<<<<<<<
 *     'grey37' : (94, 94, 94),
 *     'grey38' : (97, 97, 97),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey36, __pyx_tuple__176) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":297
 *     'grey35' : (89, 89, 89),
 *     'grey36' : (92, 92, 92),
 *     'grey37' : (94, 94, 94),             # <<<<<<<<<<<<<<
 *     'grey38' : (97, 97, 97),
 *     'grey39' : (99, 99, 99),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey37, __pyx_tuple__177) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":298
 *     'grey36' : (92, 92, 92),
 *     'grey37' : (94, 94, 94),
 *     'grey38' : (97, 97, 97),             # <<<<<<<<<<<<<<
 *     'grey39' : (99, 99, 99),
 *     'grey4' : (10, 10, 10),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey38, __pyx_tuple__178) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":299
 *     'grey37' : (94, 94, 94),
 *     'grey38' : (97, 97, 97),
 *     'grey39' : (99, 99, 99),             # <<<<<<<<<<<<<<
 *     'grey4' : (10, 10, 10),
 *     'grey40' : (102, 102, 102),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey39, __pyx_tuple__179) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":300
 *     'grey38' : (97, 97, 97),
 *     'grey39' : (99, 99, 99),
 *     'grey4' : (10, 10, 10),             # <<<<<<<<<<<<<<
 *     'grey40' : (102, 102, 102),
 *     'grey41' : (105, 105, 105),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey4, __pyx_tuple__180) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":301
 *     'grey39' : (99, 99, 99),
 *     'grey4' : (10, 10, 10),
 *     'grey40' : (102, 102, 102),             # <<<<<<<<<<<<<<
 *     'grey41' : (105, 105, 105),
 *     'grey42' : (107, 107, 107),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey40, __pyx_tuple__181) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":302
 *     'grey4' : (10, 10, 10),
 *     'grey40' : (102, 102, 102),
 *     'grey41' : (105, 105, 105),             # <<<<<<<<<<<<<<
 *     'grey42' : (107, 107, 107),
 *     'grey43' : (110, 110, 110),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey41, __pyx_tuple__122) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":303
 *     'grey40' : (102, 102, 102),
 *     'grey41' : (105, 105, 105),
 *     'grey42' : (107, 107, 107),             # <<<<<<<<<<<<<<
 *     'grey43' : (110, 110, 110),
 *     'grey44' : (112, 112, 112),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey42, __pyx_tuple__182) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":304
 *     'grey41' : (105, 105, 105),
 *     'grey42' : (107, 107, 107),
 *     'grey43' : (110, 110, 110),             # <<<<<<<<<<<<<<
 *     'grey44' : (112, 112, 112),
 *     'grey45' : (115, 115, 115),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey43, __pyx_tuple__183) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":305
 *     'grey42' : (107, 107, 107),
 *     'grey43' : (110, 110, 110),
 *     'grey44' : (112, 112, 112),             # <<<<<<<<<<<<<<
 *     'grey45' : (115, 115, 115),
 *     'grey46' : (117, 117, 117),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey44, __pyx_tuple__184) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":306
 *     'grey43' : (110, 110, 110),
 *     'grey44' : (112, 112, 112),
 *     'grey45' : (115, 115, 115),             # <<<<<<<<<<<<<<
 *     'grey46' : (117, 117, 117),
 *     'grey47' : (120, 120, 120),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey45, __pyx_tuple__185) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":307
 *     'grey44' : (112, 112, 112),
 *     'grey45' : (115, 115, 115),
 *     'grey46' : (117, 117, 117),             # <<<<<<<<<<<<<<
 *     'grey47' : (120, 120, 120),
 *     'grey48' : (122, 122, 122),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey46, __pyx_tuple__186) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":308
 *     'grey45' : (115, 115, 115),
 *     'grey46' : (117, 117, 117),
 *     'grey47' : (120, 120, 120),             # <<<<<<<<<<<<<<
 *     'grey48' : (122, 122, 122),
 *     'grey49' : (125, 125, 125),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey47, __pyx_tuple__187) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":309
 *     'grey46' : (117, 117, 117),
 *     'grey47' : (120, 120, 120),
 *     'grey48' : (122, 122, 122),             # <<<<<<<<<<<<<<
 *     'grey49' : (125, 125, 125),
 *     'grey5' : (13, 13, 13),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey48, __pyx_tuple__188) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":310
 *     'grey47' : (120, 120, 120),
 *     'grey48' : (122, 122, 122),
 *     'grey49' : (125, 125, 125),             # <<<<<<<<<<<<<<
 *     'grey5' : (13, 13, 13),
 *     'grey50' : (127, 127, 127),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey49, __pyx_tuple__189) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":311
 *     'grey48' : (122, 122, 122),
 *     'grey49' : (125, 125, 125),
 *     'grey5' : (13, 13, 13),             # <<<<<<<<<<<<<<
 *     'grey50' : (127, 127, 127),
 *     'grey51' : (130, 130, 130),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey5, __pyx_tuple__190) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":312
 *     'grey49' : (125, 125, 125),
 *     'grey5' : (13, 13, 13),
 *     'grey50' : (127, 127, 127),             # <<<<<<<<<<<<<<
 *     'grey51' : (130, 130, 130),
 *     'grey52' : (133, 133, 133),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey50, __pyx_tuple__191) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":313
 *     'grey5' : (13, 13, 13),
 *     'grey50' : (127, 127, 127),
 *     'grey51' : (130, 130, 130),             # <<<<<<<<<<<<<<
 *     'grey52' : (133, 133, 133),
 *     'grey53' : (135, 135, 135),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey51, __pyx_tuple__192) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":314
 *     'grey50' : (127, 127, 127),
 *     'grey51' : (130, 130, 130),
 *     'grey52' : (133, 133, 133),             # <<<<<<<<<<<<<<
 *     'grey53' : (135, 135, 135),
 *     'grey54' : (138, 138, 138),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey52, __pyx_tuple__193) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":315
 *     'grey51' : (130, 130, 130),
 *     'grey52' : (133, 133, 133),
 *     'grey53' : (135, 135, 135),             # <<<<<<<<<<<<<<
 *     'grey54' : (138, 138, 138),
 *     'grey55' : (140, 140, 140),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey53, __pyx_tuple__194) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":316
 *     'grey52' : (133, 133, 133),
 *     'grey53' : (135, 135, 135),
 *     'grey54' : (138, 138, 138),             # <<<<<<<<<<<<<<
 *     'grey55' : (140, 140, 140),
 *     'grey56' : (143, 143, 143),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey54, __pyx_tuple__195) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":317
 *     'grey53' : (135, 135, 135),
 *     'grey54' : (138, 138, 138),
 *     'grey55' : (140, 140, 140),             # <<<<<<<<<<<<<<
 *     'grey56' : (143, 143, 143),
 *     'grey57' : (145, 145, 145),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey55, __pyx_tuple__196) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":318
 *     'grey54' : (138, 138, 138),
 *     'grey55' : (140, 140, 140),
 *     'grey56' : (143, 143, 143),             # <<<<<<<<<<<<<<
 *     'grey57' : (145, 145, 145),
 *     'grey58' : (148, 148, 148),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey56, __pyx_tuple__197) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":319
 *     'grey55' : (140, 140, 140),
 *     'grey56' : (143, 143, 143),
 *     'grey57' : (145, 145, 145),             # <<<<<<<<<<<<<<
 *     'grey58' : (148, 148, 148),
 *     'grey59' : (150, 150, 150),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey57, __pyx_tuple__198) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":320
 *     'grey56' : (143, 143, 143),
 *     'grey57' : (145, 145, 145),
 *     'grey58' : (148, 148, 148),             # <<<<<<<<<<<<<<
 *     'grey59' : (150, 150, 150),
 *     'grey6' : (15, 15, 15),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey58, __pyx_tuple__199) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":321
 *     'grey57' : (145, 145, 145),
 *     'grey58' : (148, 148, 148),
 *     'grey59' : (150, 150, 150),             # <<<<<<<<<<<<<<
 *     'grey6' : (15, 15, 15),
 *     'grey60' : (153, 153, 153),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey59, __pyx_tuple__200) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":322
 *     'grey58' : (148, 148, 148),
 *     'grey59' : (150, 150, 150),
 *     'grey6' : (15, 15, 15),             # <<<<<<<<<<<<<<
 *     'grey60' : (153, 153, 153),
 *     'grey61' : (156, 156, 156),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey6, __pyx_tuple__201) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":323
 *     'grey59' : (150, 150, 150),
 *     'grey6' : (15, 15, 15),
 *     'grey60' : (153, 153, 153),             # <<<<<<<<<<<<<<
 *     'grey61' : (156, 156, 156),
 *     'grey62' : (158, 158, 158),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey60, __pyx_tuple__202) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":324
 *     'grey6' : (15, 15, 15),
 *     'grey60' : (153, 153, 153),
 *     'grey61' : (156, 156, 156),             # <<<<<<<<<<<<<<
 *     'grey62' : (158, 158, 158),
 *     'grey63' : (161, 161, 161),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey61, __pyx_tuple__203) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":325
 *     'grey60' : (153, 153, 153),
 *     'grey61' : (156, 156, 156),
 *     'grey62' : (158, 158, 158),             # <<<<<<<<<<<<<<
 *     'grey63' : (161, 161, 161),
 *     'grey64' : (163, 163, 163),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey62, __pyx_tuple__204) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":326
 *     'grey61' : (156, 156, 156),
 *     'grey62' : (158, 158, 158),
 *     'grey63' : (161, 161, 161),             # <<<<<<<<<<<<<<
 *     'grey64' : (163, 163, 163),
 *     'grey65' : (166, 166, 166),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey63, __pyx_tuple__205) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":327
 *     'grey62' : (158, 158, 158),
 *     'grey63' : (161, 161, 161),
 *     'grey64' : (163, 163, 163),             # <<<<<<<<<<<<<<
 *     'grey65' : (166, 166, 166),
 *     'grey66' : (168, 168, 168),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey64, __pyx_tuple__206) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":328
 *     'grey63' : (161, 161, 161),
 *     'grey64' : (163, 163, 163),
 *     'grey65' : (166, 166, 166),             # <<<<<<<<<<<<<<
 *     'grey66' : (168, 168, 168),
 *     'grey67' : (171, 171, 171),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey65, __pyx_tuple__207) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":329
 *     'grey64' : (163, 163, 163),
 *     'grey65' : (166, 166, 166),
 *     'grey66' : (168, 168, 168),             # <<<<<<<<<<<<<<
 *     'grey67' : (171, 171, 171),
 *     'grey68' : (173, 173, 173),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey66, __pyx_tuple__208) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":330
 *     'grey65' : (166, 166, 166),
 *     'grey66' : (168, 168, 168),
 *     'grey67' : (171, 171, 171),             # <<<<<<<<<<<<<<
 *     'grey68' : (173, 173, 173),
 *     'grey69' : (176, 176, 176),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey67, __pyx_tuple__209) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":331
 *     'grey66' : (168, 168, 168),
 *     'grey67' : (171, 171, 171),
 *     'grey68' : (173, 173, 173),             # <<<<<<<<<<<<<<
 *     'grey69' : (176, 176, 176),
 *     'grey7' : (18, 18, 18),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey68, __pyx_tuple__210) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":332
 *     'grey67' : (171, 171, 171),
 *     'grey68' : (173, 173, 173),
 *     'grey69' : (176, 176, 176),             # <<<<<<<<<<<<<<
 *     'grey7' : (18, 18, 18),
 *     'grey70' : (179, 179, 179),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey69, __pyx_tuple__211) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":333
 *     'grey68' : (173, 173, 173),
 *     'grey69' : (176, 176, 176),
 *     'grey7' : (18, 18, 18),             # <<<<<<<<<<<<<<
 *     'grey70' : (179, 179, 179),
 *     'grey71' : (181, 181, 181),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey7, __pyx_tuple__212) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":334
 *     'grey69' : (176, 176, 176),
 *     'grey7' : (18, 18, 18),
 *     'grey70' : (179, 179, 179),             # <<<<<<<<<<<<<<
 *     'grey71' : (181, 181, 181),
 *     'grey72' : (184, 184, 184),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey70, __pyx_tuple__213) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":335
 *     'grey7' : (18, 18, 18),
 *     'grey70' : (179, 179, 179),
 *     'grey71' : (181, 181, 181),             # <<<<<<<<<<<<<<
 *     'grey72' : (184, 184, 184),
 *     'grey73' : (186, 186, 186),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey71, __pyx_tuple__214) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":336
 *     'grey70' : (179, 179, 179),
 *     'grey71' : (181, 181, 181),
 *     'grey72' : (184, 184, 184),             # <<<<<<<<<<<<<<
 *     'grey73' : (186, 186, 186),
 *     'grey74' : (189, 189, 189),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey72, __pyx_tuple__215) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":337
 *     'grey71' : (181, 181, 181),
 *     'grey72' : (184, 184, 184),
 *     'grey73' : (186, 186, 186),             # <<<<<<<<<<<<<<
 *     'grey74' : (189, 189, 189),
 *     'grey75' : (191, 191, 191),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey73, __pyx_tuple__216) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":338
 *     'grey72' : (184, 184, 184),
 *     'grey73' : (186, 186, 186),
 *     'grey74' : (189, 189, 189),             # <<<<<<<<<<<<<<
 *     'grey75' : (191, 191, 191),
 *     'grey76' : (194, 194, 194),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey74, __pyx_tuple__217) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":339
 *     'grey73' : (186, 186, 186),
 *     'grey74' : (189, 189, 189),
 *     'grey75' : (191, 191, 191),             # <<<<<<<<<<<<<<
 *     'grey76' : (194, 194, 194),
 *     'grey77' : (196, 196, 196),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey75, __pyx_tuple__218) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":340
 *     'grey74' : (189, 189, 189),
 *     'grey75' : (191, 191, 191),
 *     'grey76' : (194, 194, 194),             # <<<<<<<<<<<<<<
 *     'grey77' : (196, 196, 196),
 *     'grey78' : (199, 199, 199),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey76, __pyx_tuple__219) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":341
 *     'grey75' : (191, 191, 191),
 *     'grey76' : (194, 194, 194),
 *     'grey77' : (196, 196, 196),             # <<<<<<<<<<<<<<
 *     'grey78' : (199, 199, 199),
 *     'grey79' : (201, 201, 201),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey77, __pyx_tuple__220) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":342
 *     'grey76' : (194, 194, 194),
 *     'grey77' : (196, 196, 196),
 *     'grey78' : (199, 199, 199),             # <<<<<<<<<<<<<<
 *     'grey79' : (201, 201, 201),
 *     'grey8' : (20, 20, 20),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey78, __pyx_tuple__221) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":343
 *     'grey77' : (196, 196, 196),
 *     'grey78' : (199, 199, 199),
 *     'grey79' : (201, 201, 201),             # <<<<<<<<<<<<<<
 *     'grey8' : (20, 20, 20),
 *     'grey80' : (204, 204, 204),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey79, __pyx_tuple__222) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":344
 *     'grey78' : (199, 199, 199),
 *     'grey79' : (201, 201, 201),
 *     'grey8' : (20, 20, 20),             # <<<<<<<<<<<<<<
 *     'grey80' : (204, 204, 204),
 *     'grey81' : (207, 207, 207),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey8, __pyx_tuple__223) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":345
 *     'grey79' : (201, 201, 201),
 *     'grey8' : (20, 20, 20),
 *     'grey80' : (204, 204, 204),             # <<<<<<<<<<<<<<
 *     'grey81' : (207, 207, 207),
 *     'grey82' : (209, 209, 209),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey80, __pyx_tuple__224) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":346
 *     'grey8' : (20, 20, 20),
 *     'grey80' : (204, 204, 204),
 *     'grey81' : (207, 207, 207),             # <<<<<<<<<<<<<<
 *     'grey82' : (209, 209, 209),
 *     'grey83' : (212, 212, 212),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey81, __pyx_tuple__225) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":347
 *     'grey80' : (204, 204, 204),
 *     'grey81' : (207, 207, 207),
 *     'grey82' : (209, 209, 209),             # <<<<<<<<<<<<<<
 *     'grey83' : (212, 212, 212),
 *     'grey84' : (214, 214, 214),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey82, __pyx_tuple__226) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":348
 *     'grey81' : (207, 207, 207),
 *     'grey82' : (209, 209, 209),
 *     'grey83' : (212, 212, 212),             # <<<<<<<<<<<<<<
 *     'grey84' : (214, 214, 214),
 *     'grey85' : (217, 217, 217),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey83, __pyx_tuple__227) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":349
 *     'grey82' : (209, 209, 209),
 *     'grey83' : (212, 212, 212),
 *     'grey84' : (214, 214, 214),             # <<<<<<<<<<<<<<
 *     'grey85' : (217, 217, 217),
 *     'grey86' : (219, 219, 219),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey84, __pyx_tuple__228) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":350
 *     'grey83' : (212, 212, 212),
 *     'grey84' : (214, 214, 214),
 *     'grey85' : (217, 217, 217),             # <<<<<<<<<<<<<<
 *     'grey86' : (219, 219, 219),
 *     'grey87' : (222, 222, 222),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey85, __pyx_tuple__229) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":351
 *     'grey84' : (214, 214, 214),
 *     'grey85' : (217, 217, 217),
 *     'grey86' : (219, 219, 219),             # <<<<<<<<<<<<<<
 *     'grey87' : (222, 222, 222),
 *     'grey88' : (224, 224, 224),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey86, __pyx_tuple__230) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":352
 *     'grey85' : (217, 217, 217),
 *     'grey86' : (219, 219, 219),
 *     'grey87' : (222, 222, 222),             # <<<<<<<<<<<<<<
 *     'grey88' : (224, 224, 224),
 *     'grey89' : (227, 227, 227),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey87, __pyx_tuple__231) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":353
 *     'grey86' : (219, 219, 219),
 *     'grey87' : (222, 222, 222),
 *     'grey88' : (224, 224, 224),             # <<<<<<<<<<<<<<
 *     'grey89' : (227, 227, 227),
 *     'grey9' : (23, 23, 23),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey88, __pyx_tuple__232) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":354
 *     'grey87' : (222, 222, 222),
 *     'grey88' : (224, 224, 224),
 *     'grey89' : (227, 227, 227),             # <<<<<<<<<<<<<<
 *     'grey9' : (23, 23, 23),
 *     'grey90' : (229, 229, 229),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey89, __pyx_tuple__233) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":355
 *     'grey88' : (224, 224, 224),
 *     'grey89' : (227, 227, 227),
 *     'grey9' : (23, 23, 23),             # <<<<<<<<<<<<<<
 *     'grey90' : (229, 229, 229),
 *     'grey91' : (232, 232, 232),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey9, __pyx_tuple__234) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":356
 *     'grey89' : (227, 227, 227),
 *     'grey9' : (23, 23, 23),
 *     'grey90' : (229, 229, 229),             # <<<<<<<<<<<<<<
 *     'grey91' : (232, 232, 232),
 *     'grey92' : (235, 235, 235),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey90, __pyx_tuple__235) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":357
 *     'grey9' : (23, 23, 23),
 *     'grey90' : (229, 229, 229),
 *     'grey91' : (232, 232, 232),             # <<<<<<<<<<<<<<
 *     'grey92' : (235, 235, 235),
 *     'grey93' : (237, 237, 237),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey91, __pyx_tuple__236) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":358
 *     'grey90' : (229, 229, 229),
 *     'grey91' : (232, 232, 232),
 *     'grey92' : (235, 235, 235),             # <<<<<<<<<<<<<<
 *     'grey93' : (237, 237, 237),
 *     'grey94' : (240, 240, 240),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey92, __pyx_tuple__237) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":359
 *     'grey91' : (232, 232, 232),
 *     'grey92' : (235, 235, 235),
 *     'grey93' : (237, 237, 237),             # <<<<<<<<<<<<<<
 *     'grey94' : (240, 240, 240),
 *     'grey95' : (242, 242, 242),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey93, __pyx_tuple__238) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":360
 *     'grey92' : (235, 235, 235),
 *     'grey93' : (237, 237, 237),
 *     'grey94' : (240, 240, 240),             # <<<<<<<<<<<<<<
 *     'grey95' : (242, 242, 242),
 *     'grey96' : (245, 245, 245),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey94, __pyx_tuple__239) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":361
 *     'grey93' : (237, 237, 237),
 *     'grey94' : (240, 240, 240),
 *     'grey95' : (242, 242, 242),             # <<<<<<<<<<<<<<
 *     'grey96' : (245, 245, 245),
 *     'grey97' : (247, 247, 247),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey95, __pyx_tuple__240) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":362
 *     'grey94' : (240, 240, 240),
 *     'grey95' : (242, 242, 242),
 *     'grey96' : (245, 245, 245),             # <<<<<<<<<<<<<<
 *     'grey97' : (247, 247, 247),
 *     'grey98' : (250, 250, 250),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey96, __pyx_tuple__241) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":363
 *     'grey95' : (242, 242, 242),
 *     'grey96' : (245, 245, 245),
 *     'grey97' : (247, 247, 247),             # <<<<<<<<<<<<<<
 *     'grey98' : (250, 250, 250),
 *     'grey99' : (252, 252, 252),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey97, __pyx_tuple__242) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":364
 *     'grey96' : (245, 245, 245),
 *     'grey97' : (247, 247, 247),
 *     'grey98' : (250, 250, 250),             # <<<<<<<<<<<<<<
 *     'grey99' : (252, 252, 252),
 *     'honeydew' : (240, 255, 240),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey98, __pyx_tuple__243) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":365
 *     'grey97' : (247, 247, 247),
 *     'grey98' : (250, 250, 250),
 *     'grey99' : (252, 252, 252),             # <<<<<<<<<<<<<<
 *     'honeydew' : (240, 255, 240),
 *     'honeydew1' : (240, 255, 240),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_grey99, __pyx_tuple__244) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":366
 *     'grey98' : (250, 250, 250),
 *     'grey99' : (252, 252, 252),
 *     'honeydew' : (240, 255, 240),             # <<<<<<<<<<<<<<
 *     'honeydew1' : (240, 255, 240),
 *     'honeydew2' : (224, 238, 224),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_honeydew, __pyx_tuple__250) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":367
 *     'grey99' : (252, 252, 252),
 *     'honeydew' : (240, 255, 240),
 *     'honeydew1' : (240, 255, 240),             # <<<<<<<<<<<<<<
 *     'honeydew2' : (224, 238, 224),
 *     'honeydew3' : (193, 205, 193),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_honeydew1, __pyx_tuple__250) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":368
 *     'honeydew' : (240, 255, 240),
 *     'honeydew1' : (240, 255, 240),
 *     'honeydew2' : (224, 238, 224),             # <<<<<<<<<<<<<<
 *     'honeydew3' : (193, 205, 193),
 *     'honeydew4' : (131, 139, 131),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_honeydew2, __pyx_tuple__251) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":369
 *     'honeydew1' : (240, 255, 240),
 *     'honeydew2' : (224, 238, 224),
 *     'honeydew3' : (193, 205, 193),             # <<<<<<<<<<<<<<
 *     'honeydew4' : (131, 139, 131),
 *     'hotpink' : (255, 105, 180),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_honeydew3, __pyx_tuple__252) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":370
 *     'honeydew2' : (224, 238, 224),
 *     'honeydew3' : (193, 205, 193),
 *     'honeydew4' : (131, 139, 131),             # <<<<<<<<<<<<<<
 *     'hotpink' : (255, 105, 180),
 *     'hotpink1' : (255, 110, 180),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_honeydew4, __pyx_tuple__253) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":371
 *     'honeydew3' : (193, 205, 193),
 *     'honeydew4' : (131, 139, 131),
 *     'hotpink' : (255, 105, 180),             # <<<<<<<<<<<<<<
 *     'hotpink1' : (255, 110, 180),
 *     'hotpink2' : (238, 106, 167),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_hotpink, __pyx_tuple__254) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":372
 *     'honeydew4' : (131, 139, 131),
 *     'hotpink' : (255, 105, 180),
 *     'hotpink1' : (255, 110, 180),             # <<<<<<<<<<<<<<
 *     'hotpink2' : (238, 106, 167),
 *     'hotpink3' : (205, 96, 144),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_hotpink1, __pyx_tuple__255) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":373
 *     'hotpink' : (255, 105, 180),
 *     'hotpink1' : (255, 110, 180),
 *     'hotpink2' : (238, 106, 167),             # <<<<<<<<<<<<<<
 *     'hotpink3' : (205, 96, 144),
 *     'hotpink4' : (139, 58, 98),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_hotpink2, __pyx_tuple__256) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":374
 *     'hotpink1' : (255, 110, 180),
 *     'hotpink2' : (238, 106, 167),
 *     'hotpink3' : (205, 96, 144),             # <<<<<<<<<<<<<<
 *     'hotpink4' : (139, 58, 98),
 *     'indianred' : (205, 92, 92),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_hotpink3, __pyx_tuple__257) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":375
 *     'hotpink2' : (238, 106, 167),
 *     'hotpink3' : (205, 96, 144),
 *     'hotpink4' : (139, 58, 98),             # <<<<<<<<<<<<<<
 *     'indianred' : (205, 92, 92),
 *     'indianred1' : (255, 106, 106),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_hotpink4, __pyx_tuple__258) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":376
 *     'hotpink3' : (205, 96, 144),
 *     'hotpink4' : (139, 58, 98),
 *     'indianred' : (205, 92, 92),             # <<<<<<<<<<<<<<
 *     'indianred1' : (255, 106, 106),
 *     'indianred2' : (238, 99, 99),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_indianred, __pyx_tuple__259) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":377
 *     'hotpink4' : (139, 58, 98),
 *     'indianred' : (205, 92, 92),
 *     'indianred1' : (255, 106, 106),             # <<<<<<<<<<<<<<
 *     'indianred2' : (238, 99, 99),
 *     'indianred3' : (205, 85, 85),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_indianred1, __pyx_tuple__260) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":378
 *     'indianred' : (205, 92, 92),
 *     'indianred1' : (255, 106, 106),
 *     'indianred2' : (238, 99, 99),             # <<<<<<<<<<<<<<
 *     'indianred3' : (205, 85, 85),
 *     'indianred4' : (139, 58, 58),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_indianred2, __pyx_tuple__261) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":379
 *     'indianred1' : (255, 106, 106),
 *     'indianred2' : (238, 99, 99),
 *     'indianred3' : (205, 85, 85),             # <<<<<<<<<<<<<<
 *     'indianred4' : (139, 58, 58),
 *     'ivory' : (255, 255, 240),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_indianred3, __pyx_tuple__262) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":380
 *     'indianred2' : (238, 99, 99),
 *     'indianred3' : (205, 85, 85),
 *     'indianred4' : (139, 58, 58),             # <<<<<<<<<<<<<<
 *     'ivory' : (255, 255, 240),
 *     'ivory1' : (255, 255, 240),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_indianred4, __pyx_tuple__263) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":381
 *     'indianred3' : (205, 85, 85),
 *     'indianred4' : (139, 58, 58),
 *     'ivory' : (255, 255, 240),             # <<<<<<<<<<<<<<
 *     'ivory1' : (255, 255, 240),
 *     'ivory2' : (238, 238, 224),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_ivory, __pyx_tuple__264) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":382
 *     'indianred4' : (139, 58, 58),
 *     'ivory' : (255, 255, 240),
 *     'ivory1' : (255, 255, 240),             # <<<<<<<<<<<<<<
 *     'ivory2' : (238, 238, 224),
 *     'ivory3' : (205, 205, 193),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_ivory1, __pyx_tuple__264) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":383
 *     'ivory' : (255, 255, 240),
 *     'ivory1' : (255, 255, 240),
 *     'ivory2' : (238, 238, 224),             # <<<<<<<<<<<<<<
 *     'ivory3' : (205, 205, 193),
 *     'ivory4' : (139, 139, 131),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_ivory2, __pyx_tuple__265) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":384
 *     'ivory1' : (255, 255, 240),
 *     'ivory2' : (238, 238, 224),
 *     'ivory3' : (205, 205, 193),             # <<<<<<<<<<<<<<
 *     'ivory4' : (139, 139, 131),
 *     'khaki' : (240, 230, 140),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_ivory3, __pyx_tuple__266) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":385
 *     'ivory2' : (238, 238, 224),
 *     'ivory3' : (205, 205, 193),
 *     'ivory4' : (139, 139, 131),             # <<<<<<<<<<<<<<
 *     'khaki' : (240, 230, 140),
 *     'khaki1' : (255, 246, 143),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_ivory4, __pyx_tuple__267) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":386
 *     'ivory3' : (205, 205, 193),
 *     'ivory4' : (139, 139, 131),
 *     'khaki' : (240, 230, 140),             # <<<<<<<<<<<<<<
 *     'khaki1' : (255, 246, 143),
 *     'khaki2' : (238, 230, 133),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_khaki, __pyx_tuple__268) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":387
 *     'ivory4' : (139, 139, 131),
 *     'khaki' : (240, 230, 140),
 *     'khaki1' : (255, 246, 143),             # <<<<<<<<<<<<<<
 *     'khaki2' : (238, 230, 133),
 *     'khaki3' : (205, 198, 115),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_khaki1, __pyx_tuple__269) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":388
 *     'khaki' : (240, 230, 140),
 *     'khaki1' : (255, 246, 143),
 *     'khaki2' : (238, 230, 133),             # <<<<<<<<<<<<<<
 *     'khaki3' : (205, 198, 115),
 *     'khaki4' : (139, 134, 78),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_khaki2, __pyx_tuple__270) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":389
 *     'khaki1' : (255, 246, 143),
 *     'khaki2' : (238, 230, 133),
 *     'khaki3' : (205, 198, 115),             # <<<<<<<<<<<<<<
 *     'khaki4' : (139, 134, 78),
 *     'lavender' : (230, 230, 250),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_khaki3, __pyx_tuple__271) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":390
 *     'khaki2' : (238, 230, 133),
 *     'khaki3' : (205, 198, 115),
 *     'khaki4' : (139, 134, 78),             # <<<<<<<<<<<<<<
 *     'lavender' : (230, 230, 250),
 *     'lavenderblush' : (255, 240, 245),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_khaki4, __pyx_tuple__272) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":391
 *     'khaki3' : (205, 198, 115),
 *     'khaki4' : (139, 134, 78),
 *     'lavender' : (230, 230, 250),             # <<<<<<<<<<<<<<
 *     'lavenderblush' : (255, 240, 245),
 *     'lavenderblush1' : (255, 240, 245),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lavender, __pyx_tuple__273) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":392
 *     'khaki4' : (139, 134, 78),
 *     'lavender' : (230, 230, 250),
 *     'lavenderblush' : (255, 240, 245),             # <<<<<<<<<<<<<<
 *     'lavenderblush1' : (255, 240, 245),
 *     'lavenderblush2' : (238, 224, 229),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lavenderblush, __pyx_tuple__274) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":393
 *     'lavender' : (230, 230, 250),
 *     'lavenderblush' : (255, 240, 245),
 *     'lavenderblush1' : (255, 240, 245),             # <<<<<<<<<<<<<<
 *     'lavenderblush2' : (238, 224, 229),
 *     'lavenderblush3' : (205, 193, 197),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lavenderblush1, __pyx_tuple__274) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":394
 *     'lavenderblush' : (255, 240, 245),
 *     'lavenderblush1' : (255, 240, 245),
 *     'lavenderblush2' : (238, 224, 229),             # <<<<<<<<<<<<<<
 *     'lavenderblush3' : (205, 193, 197),
 *     'lavenderblush4' : (139, 131, 134),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lavenderblush2, __pyx_tuple__275) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":395
 *     'lavenderblush1' : (255, 240, 245),
 *     'lavenderblush2' : (238, 224, 229),
 *     'lavenderblush3' : (205, 193, 197),             # <<<<<<<<<<<<<<
 *     'lavenderblush4' : (139, 131, 134),
 *     'lawngreen' : (124, 252, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lavenderblush3, __pyx_tuple__276) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":396
 *     'lavenderblush2' : (238, 224, 229),
 *     'lavenderblush3' : (205, 193, 197),
 *     'lavenderblush4' : (139, 131, 134),             # <<<<<<<<<<<<<<
 *     'lawngreen' : (124, 252, 0),
 *     'lemonchiffon' : (255, 250, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lavenderblush4, __pyx_tuple__277) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":397
 *     'lavenderblush3' : (205, 193, 197),
 *     'lavenderblush4' : (139, 131, 134),
 *     'lawngreen' : (124, 252, 0),             # <<<<<<<<<<<<<<
 *     'lemonchiffon' : (255, 250, 205),
 *     'lemonchiffon1' : (255, 250, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lawngreen, __pyx_tuple__278) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":398
 *     'lavenderblush4' : (139, 131, 134),
 *     'lawngreen' : (124, 252, 0),
 *     'lemonchiffon' : (255, 250, 205),             # <<<<<<<<<<<<<<
 *     'lemonchiffon1' : (255, 250, 205),
 *     'lemonchiffon2' : (238, 233, 191),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lemonchiffon, __pyx_tuple__279) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":399
 *     'lawngreen' : (124, 252, 0),
 *     'lemonchiffon' : (255, 250, 205),
 *     'lemonchiffon1' : (255, 250, 205),             # <<<<<<<<<<<<<<
 *     'lemonchiffon2' : (238, 233, 191),
 *     'lemonchiffon3' : (205, 201, 165),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lemonchiffon1, __pyx_tuple__279) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":400
 *     'lemonchiffon' : (255, 250, 205),
 *     'lemonchiffon1' : (255, 250, 205),
 *     'lemonchiffon2' : (238, 233, 191),             # <<<<<<<<<<<<<<
 *     'lemonchiffon3' : (205, 201, 165),
 *     'lemonchiffon4' : (139, 137, 112),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lemonchiffon2, __pyx_tuple__280) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":401
 *     'lemonchiffon1' : (255, 250, 205),
 *     'lemonchiffon2' : (238, 233, 191),
 *     'lemonchiffon3' : (205, 201, 165),             # <<<<<<<<<<<<<<
 *     'lemonchiffon4' : (139, 137, 112),
 *     'lightblue' : (173, 216, 230),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lemonchiffon3, __pyx_tuple__281) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":402
 *     'lemonchiffon2' : (238, 233, 191),
 *     'lemonchiffon3' : (205, 201, 165),
 *     'lemonchiffon4' : (139, 137, 112),             # <<<<<<<<<<<<<<
 *     'lightblue' : (173, 216, 230),
 *     'lightblue1' : (191, 239, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lemonchiffon4, __pyx_tuple__282) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":403
 *     'lemonchiffon3' : (205, 201, 165),
 *     'lemonchiffon4' : (139, 137, 112),
 *     'lightblue' : (173, 216, 230),             # <<<<<<<<<<<<<<
 *     'lightblue1' : (191, 239, 255),
 *     'lightblue2' : (178, 223, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightblue, __pyx_tuple__283) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":404
 *     'lemonchiffon4' : (139, 137, 112),
 *     'lightblue' : (173, 216, 230),
 *     'lightblue1' : (191, 239, 255),             # <<<<<<<<<<<<<<
 *     'lightblue2' : (178, 223, 238),
 *     'lightblue3' : (154, 192, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightblue1, __pyx_tuple__284) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":405
 *     'lightblue' : (173, 216, 230),
 *     'lightblue1' : (191, 239, 255),
 *     'lightblue2' : (178, 223, 238),             # <<<<<<<<<<<<<<
 *     'lightblue3' : (154, 192, 205),
 *     'lightblue4' : (104, 131, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightblue2, __pyx_tuple__285) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":406
 *     'lightblue1' : (191, 239, 255),
 *     'lightblue2' : (178, 223, 238),
 *     'lightblue3' : (154, 192, 205),             # <<<<<<<<<<<<<<
 *     'lightblue4' : (104, 131, 139),
 *     'lightcoral' : (240, 128, 128),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightblue3, __pyx_tuple__286) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":407
 *     'lightblue2' : (178, 223, 238),
 *     'lightblue3' : (154, 192, 205),
 *     'lightblue4' : (104, 131, 139),             # <<<<<<<<<<<<<<
 *     'lightcoral' : (240, 128, 128),
 *     'lightcyan' : (224, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightblue4, __pyx_tuple__287) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":408
 *     'lightblue3' : (154, 192, 205),
 *     'lightblue4' : (104, 131, 139),
 *     'lightcoral' : (240, 128, 128),             # <<<<<<<<<<<<<<
 *     'lightcyan' : (224, 255, 255),
 *     'lightcyan1' : (224, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightcoral, __pyx_tuple__288) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":409
 *     'lightblue4' : (104, 131, 139),
 *     'lightcoral' : (240, 128, 128),
 *     'lightcyan' : (224, 255, 255),             # <<<<<<<<<<<<<<
 *     'lightcyan1' : (224, 255, 255),
 *     'lightcyan2' : (209, 238, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightcyan, __pyx_tuple__289) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":410
 *     'lightcoral' : (240, 128, 128),
 *     'lightcyan' : (224, 255, 255),
 *     'lightcyan1' : (224, 255, 255),             # <<<<<<<<<<<<<<
 *     'lightcyan2' : (209, 238, 238),
 *     'lightcyan3' : (180, 205, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightcyan1, __pyx_tuple__289) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":411
 *     'lightcyan' : (224, 255, 255),
 *     'lightcyan1' : (224, 255, 255),
 *     'lightcyan2' : (209, 238, 238),             # <<<<<<<<<<<<<<
 *     'lightcyan3' : (180, 205, 205),
 *     'lightcyan4' : (122, 139, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightcyan2, __pyx_tuple__290) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":412
 *     'lightcyan1' : (224, 255, 255),
 *     'lightcyan2' : (209, 238, 238),
 *     'lightcyan3' : (180, 205, 205),             # <<<<<<<<<<<<<<
 *     'lightcyan4' : (122, 139, 139),
 *     'lightgoldenrod' : (238, 221, 130),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightcyan3, __pyx_tuple__291) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":413
 *     'lightcyan2' : (209, 238, 238),
 *     'lightcyan3' : (180, 205, 205),
 *     'lightcyan4' : (122, 139, 139),             # <<<<<<<<<<<<<<
 *     'lightgoldenrod' : (238, 221, 130),
 *     'lightgoldenrod1' : (255, 236, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightcyan4, __pyx_tuple__292) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":414
 *     'lightcyan3' : (180, 205, 205),
 *     'lightcyan4' : (122, 139, 139),
 *     'lightgoldenrod' : (238, 221, 130),             # <<<<<<<<<<<<<<
 *     'lightgoldenrod1' : (255, 236, 139),
 *     'lightgoldenrod2' : (238, 220, 130),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightgoldenrod, __pyx_tuple__293) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":415
 *     'lightcyan4' : (122, 139, 139),
 *     'lightgoldenrod' : (238, 221, 130),
 *     'lightgoldenrod1' : (255, 236, 139),             # <<<<<<<<<<<<<<
 *     'lightgoldenrod2' : (238, 220, 130),
 *     'lightgoldenrod3' : (205, 190, 112),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightgoldenrod1, __pyx_tuple__294) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":416
 *     'lightgoldenrod' : (238, 221, 130),
 *     'lightgoldenrod1' : (255, 236, 139),
 *     'lightgoldenrod2' : (238, 220, 130),             # <<<<<<<<<<<<<<
 *     'lightgoldenrod3' : (205, 190, 112),
 *     'lightgoldenrod4' : (139, 129, 76),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightgoldenrod2, __pyx_tuple__295) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":417
 *     'lightgoldenrod1' : (255, 236, 139),
 *     'lightgoldenrod2' : (238, 220, 130),
 *     'lightgoldenrod3' : (205, 190, 112),             # <<<<<<<<<<<<<<
 *     'lightgoldenrod4' : (139, 129, 76),
 *     'lightgoldenrodyellow' : (250, 250, 210),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightgoldenrod3, __pyx_tuple__296) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":418
 *     'lightgoldenrod2' : (238, 220, 130),
 *     'lightgoldenrod3' : (205, 190, 112),
 *     'lightgoldenrod4' : (139, 129, 76),             # <<<<<<<<<<<<<<
 *     'lightgoldenrodyellow' : (250, 250, 210),
 *     'lightgray' : (211, 211, 211),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightgoldenrod4, __pyx_tuple__297) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":419
 *     'lightgoldenrod3' : (205, 190, 112),
 *     'lightgoldenrod4' : (139, 129, 76),
 *     'lightgoldenrodyellow' : (250, 250, 210),             # <<<<<<<<<<<<<<
 *     'lightgray' : (211, 211, 211),
 *     'lightgreen' : (144, 238, 144),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightgoldenrodyellow, __pyx_tuple__298) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":420
 *     'lightgoldenrod4' : (139, 129, 76),
 *     'lightgoldenrodyellow' : (250, 250, 210),
 *     'lightgray' : (211, 211, 211),             # <<<<<<<<<<<<<<
 *     'lightgreen' : (144, 238, 144),
 *     'lightgrey' : (211, 211, 211),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightgray, __pyx_tuple__299) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":421
 *     'lightgoldenrodyellow' : (250, 250, 210),
 *     'lightgray' : (211, 211, 211),
 *     'lightgreen' : (144, 238, 144),             # <<<<<<<<<<<<<<
 *     'lightgrey' : (211, 211, 211),
 *     'lightpink' : (255, 182, 193),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightgreen, __pyx_tuple__300) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":422
 *     'lightgray' : (211, 211, 211),
 *     'lightgreen' : (144, 238, 144),
 *     'lightgrey' : (211, 211, 211),             # <<<<<<<<<<<<<<
 *     'lightpink' : (255, 182, 193),
 *     'lightpink1' : (255, 174, 185),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightgrey, __pyx_tuple__299) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":423
 *     'lightgreen' : (144, 238, 144),
 *     'lightgrey' : (211, 211, 211),
 *     'lightpink' : (255, 182, 193),             # <<<<<<<<<<<<<<
 *     'lightpink1' : (255, 174, 185),
 *     'lightpink2' : (238, 162, 173),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightpink, __pyx_tuple__301) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":424
 *     'lightgrey' : (211, 211, 211),
 *     'lightpink' : (255, 182, 193),
 *     'lightpink1' : (255, 174, 185),             # <<<<<<<<<<<<<<
 *     'lightpink2' : (238, 162, 173),
 *     'lightpink3' : (205, 140, 149),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightpink1, __pyx_tuple__302) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":425
 *     'lightpink' : (255, 182, 193),
 *     'lightpink1' : (255, 174, 185),
 *     'lightpink2' : (238, 162, 173),             # <<<<<<<<<<<<<<
 *     'lightpink3' : (205, 140, 149),
 *     'lightpink4' : (139, 95, 101),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightpink2, __pyx_tuple__303) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":426
 *     'lightpink1' : (255, 174, 185),
 *     'lightpink2' : (238, 162, 173),
 *     'lightpink3' : (205, 140, 149),             # <<<<<<<<<<<<<<
 *     'lightpink4' : (139, 95, 101),
 *     'lightsalmon' : (255, 160, 122),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightpink3, __pyx_tuple__304) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":427
 *     'lightpink2' : (238, 162, 173),
 *     'lightpink3' : (205, 140, 149),
 *     'lightpink4' : (139, 95, 101),             # <<<<<<<<<<<<<<
 *     'lightsalmon' : (255, 160, 122),
 *     'lightsalmon1' : (255, 160, 122),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightpink4, __pyx_tuple__305) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":428
 *     'lightpink3' : (205, 140, 149),
 *     'lightpink4' : (139, 95, 101),
 *     'lightsalmon' : (255, 160, 122),             # <<<<<<<<<<<<<<
 *     'lightsalmon1' : (255, 160, 122),
 *     'lightsalmon2' : (238, 149, 114),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightsalmon, __pyx_tuple__306) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":429
 *     'lightpink4' : (139, 95, 101),
 *     'lightsalmon' : (255, 160, 122),
 *     'lightsalmon1' : (255, 160, 122),             # <<<<<<<<<<<<<<
 *     'lightsalmon2' : (238, 149, 114),
 *     'lightsalmon3' : (205, 129, 98),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightsalmon1, __pyx_tuple__306) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":430
 *     'lightsalmon' : (255, 160, 122),
 *     'lightsalmon1' : (255, 160, 122),
 *     'lightsalmon2' : (238, 149, 114),             # <<<<<<<<<<<<<<
 *     'lightsalmon3' : (205, 129, 98),
 *     'lightsalmon4' : (139, 87, 66),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightsalmon2, __pyx_tuple__307) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":431
 *     'lightsalmon1' : (255, 160, 122),
 *     'lightsalmon2' : (238, 149, 114),
 *     'lightsalmon3' : (205, 129, 98),             # <<<<<<<<<<<<<<
 *     'lightsalmon4' : (139, 87, 66),
 *     'lightseagreen' : (32, 178, 170),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightsalmon3, __pyx_tuple__308) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":432
 *     'lightsalmon2' : (238, 149, 114),
 *     'lightsalmon3' : (205, 129, 98),
 *     'lightsalmon4' : (139, 87, 66),             # <<<<<<<<<<<<<<
 *     'lightseagreen' : (32, 178, 170),
 *     'lightskyblue' : (135, 206, 250),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightsalmon4, __pyx_tuple__309) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":433
 *     'lightsalmon3' : (205, 129, 98),
 *     'lightsalmon4' : (139, 87, 66),
 *     'lightseagreen' : (32, 178, 170),             # <<<<<<<<<<<<<<
 *     'lightskyblue' : (135, 206, 250),
 *     'lightskyblue1' : (176, 226, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightseagreen, __pyx_tuple__310) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":434
 *     'lightsalmon4' : (139, 87, 66),
 *     'lightseagreen' : (32, 178, 170),
 *     'lightskyblue' : (135, 206, 250),             # <<<<<<<<<<<<<<
 *     'lightskyblue1' : (176, 226, 255),
 *     'lightskyblue2' : (164, 211, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightskyblue, __pyx_tuple__311) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":435
 *     'lightseagreen' : (32, 178, 170),
 *     'lightskyblue' : (135, 206, 250),
 *     'lightskyblue1' : (176, 226, 255),             # <<<<<<<<<<<<<<
 *     'lightskyblue2' : (164, 211, 238),
 *     'lightskyblue3' : (141, 182, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightskyblue1, __pyx_tuple__312) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":436
 *     'lightskyblue' : (135, 206, 250),
 *     'lightskyblue1' : (176, 226, 255),
 *     'lightskyblue2' : (164, 211, 238),             # <<<<<<<<<<<<<<
 *     'lightskyblue3' : (141, 182, 205),
 *     'lightskyblue4' : (96, 123, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightskyblue2, __pyx_tuple__313) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":437
 *     'lightskyblue1' : (176, 226, 255),
 *     'lightskyblue2' : (164, 211, 238),
 *     'lightskyblue3' : (141, 182, 205),             # <<<<<<<<<<<<<<
 *     'lightskyblue4' : (96, 123, 139),
 *     'lightslateblue' : (132, 112, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightskyblue3, __pyx_tuple__314) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":438
 *     'lightskyblue2' : (164, 211, 238),
 *     'lightskyblue3' : (141, 182, 205),
 *     'lightskyblue4' : (96, 123, 139),             # <<<<<<<<<<<<<<
 *     'lightslateblue' : (132, 112, 255),
 *     'lightslategray' : (119, 136, 153),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightskyblue4, __pyx_tuple__315) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":439
 *     'lightskyblue3' : (141, 182, 205),
 *     'lightskyblue4' : (96, 123, 139),
 *     'lightslateblue' : (132, 112, 255),             # <<<<<<<<<<<<<<
 *     'lightslategray' : (119, 136, 153),
 *     'lightslategrey' : (119, 136, 153),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightslateblue, __pyx_tuple__316) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":440
 *     'lightskyblue4' : (96, 123, 139),
 *     'lightslateblue' : (132, 112, 255),
 *     'lightslategray' : (119, 136, 153),             # <<<<<<<<<<<<<<
 *     'lightslategrey' : (119, 136, 153),
 *     'lightsteelblue' : (176, 196, 222),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightslategray, __pyx_tuple__317) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":441
 *     'lightslateblue' : (132, 112, 255),
 *     'lightslategray' : (119, 136, 153),
 *     'lightslategrey' : (119, 136, 153),             # <<<<<<<<<<<<<<
 *     'lightsteelblue' : (176, 196, 222),
 *     'lightsteelblue1' : (202, 225, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightslategrey, __pyx_tuple__317) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":442
 *     'lightslategray' : (119, 136, 153),
 *     'lightslategrey' : (119, 136, 153),
 *     'lightsteelblue' : (176, 196, 222),             # <<<<<<<<<<<<<<
 *     'lightsteelblue1' : (202, 225, 255),
 *     'lightsteelblue2' : (188, 210, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightsteelblue, __pyx_tuple__318) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":443
 *     'lightslategrey' : (119, 136, 153),
 *     'lightsteelblue' : (176, 196, 222),
 *     'lightsteelblue1' : (202, 225, 255),             # <<<<<<<<<<<<<<
 *     'lightsteelblue2' : (188, 210, 238),
 *     'lightsteelblue3' : (162, 181, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightsteelblue1, __pyx_tuple__319) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":444
 *     'lightsteelblue' : (176, 196, 222),
 *     'lightsteelblue1' : (202, 225, 255),
 *     'lightsteelblue2' : (188, 210, 238),             # <<<<<<<<<<<<<<
 *     'lightsteelblue3' : (162, 181, 205),
 *     'lightsteelblue4' : (110, 123, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightsteelblue2, __pyx_tuple__320) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":445
 *     'lightsteelblue1' : (202, 225, 255),
 *     'lightsteelblue2' : (188, 210, 238),
 *     'lightsteelblue3' : (162, 181, 205),             # <<<<<<<<<<<<<<
 *     'lightsteelblue4' : (110, 123, 139),
 *     'lightyellow' : (255, 255, 224),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightsteelblue3, __pyx_tuple__321) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":446
 *     'lightsteelblue2' : (188, 210, 238),
 *     'lightsteelblue3' : (162, 181, 205),
 *     'lightsteelblue4' : (110, 123, 139),             # <<<<<<<<<<<<<<
 *     'lightyellow' : (255, 255, 224),
 *     'lightyellow1' : (255, 255, 224),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightsteelblue4, __pyx_tuple__322) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":447
 *     'lightsteelblue3' : (162, 181, 205),
 *     'lightsteelblue4' : (110, 123, 139),
 *     'lightyellow' : (255, 255, 224),             # <<<<<<<<<<<<<<
 *     'lightyellow1' : (255, 255, 224),
 *     'lightyellow2' : (238, 238, 209),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightyellow, __pyx_tuple__323) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":448
 *     'lightsteelblue4' : (110, 123, 139),
 *     'lightyellow' : (255, 255, 224),
 *     'lightyellow1' : (255, 255, 224),             # <<<<<<<<<<<<<<
 *     'lightyellow2' : (238, 238, 209),
 *     'lightyellow3' : (205, 205, 180),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightyellow1, __pyx_tuple__323) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":449
 *     'lightyellow' : (255, 255, 224),
 *     'lightyellow1' : (255, 255, 224),
 *     'lightyellow2' : (238, 238, 209),             # <<<<<<<<<<<<<<
 *     'lightyellow3' : (205, 205, 180),
 *     'lightyellow4' : (139, 139, 122),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightyellow2, __pyx_tuple__324) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":450
 *     'lightyellow1' : (255, 255, 224),
 *     'lightyellow2' : (238, 238, 209),
 *     'lightyellow3' : (205, 205, 180),             # <<<<<<<<<<<<<<
 *     'lightyellow4' : (139, 139, 122),
 *     'limegreen' : (50, 205, 50),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightyellow3, __pyx_tuple__325) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":451
 *     'lightyellow2' : (238, 238, 209),
 *     'lightyellow3' : (205, 205, 180),
 *     'lightyellow4' : (139, 139, 122),             # <<<<<<<<<<<<<<
 *     'limegreen' : (50, 205, 50),
 *     'linen' : (250, 240, 230),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_lightyellow4, __pyx_tuple__326) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":452
 *     'lightyellow3' : (205, 205, 180),
 *     'lightyellow4' : (139, 139, 122),
 *     'limegreen' : (50, 205, 50),             # <<<<<<<<<<<<<<
 *     'linen' : (250, 240, 230),
 *     'magenta' : (255, 0, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_limegreen, __pyx_tuple__327) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":453
 *     'lightyellow4' : (139, 139, 122),
 *     'limegreen' : (50, 205, 50),
 *     'linen' : (250, 240, 230),             # <<<<<<<<<<<<<<
 *     'magenta' : (255, 0, 255),
 *     'magenta1' : (255, 0, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_linen, __pyx_tuple__328) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":454
 *     'limegreen' : (50, 205, 50),
 *     'linen' : (250, 240, 230),
 *     'magenta' : (255, 0, 255),             # <<<<<<<<<<<<<<
 *     'magenta1' : (255, 0, 255),
 *     'magenta2' : (238, 0, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_magenta, __pyx_tuple__329) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":455
 *     'linen' : (250, 240, 230),
 *     'magenta' : (255, 0, 255),
 *     'magenta1' : (255, 0, 255),             # <<<<<<<<<<<<<<
 *     'magenta2' : (238, 0, 238),
 *     'magenta3' : (205, 0, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_magenta1, __pyx_tuple__329) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":456
 *     'magenta' : (255, 0, 255),
 *     'magenta1' : (255, 0, 255),
 *     'magenta2' : (238, 0, 238),             # <<<<<<<<<<<<<<
 *     'magenta3' : (205, 0, 205),
 *     'magenta4' : (139, 0, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_magenta2, __pyx_tuple__330) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":457
 *     'magenta1' : (255, 0, 255),
 *     'magenta2' : (238, 0, 238),
 *     'magenta3' : (205, 0, 205),             # <<<<<<<<<<<<<<
 *     'magenta4' : (139, 0, 139),
 *     'maroon' : (176, 48, 96),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_magenta3, __pyx_tuple__331) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":458
 *     'magenta2' : (238, 0, 238),
 *     'magenta3' : (205, 0, 205),
 *     'magenta4' : (139, 0, 139),             # <<<<<<<<<<<<<<
 *     'maroon' : (176, 48, 96),
 *     'maroon1' : (255, 52, 179),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_magenta4, __pyx_tuple__82) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":459
 *     'magenta3' : (205, 0, 205),
 *     'magenta4' : (139, 0, 139),
 *     'maroon' : (176, 48, 96),             # <<<<<<<<<<<<<<
 *     'maroon1' : (255, 52, 179),
 *     'maroon2' : (238, 48, 167),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_maroon, __pyx_tuple__332) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":460
 *     'magenta4' : (139, 0, 139),
 *     'maroon' : (176, 48, 96),
 *     'maroon1' : (255, 52, 179),             # <<<<<<<<<<<<<<
 *     'maroon2' : (238, 48, 167),
 *     'maroon3' : (205, 41, 144),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_maroon1, __pyx_tuple__333) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":461
 *     'maroon' : (176, 48, 96),
 *     'maroon1' : (255, 52, 179),
 *     'maroon2' : (238, 48, 167),             # <<<<<<<<<<<<<<
 *     'maroon3' : (205, 41, 144),
 *     'maroon4' : (139, 28, 98),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_maroon2, __pyx_tuple__334) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":462
 *     'maroon1' : (255, 52, 179),
 *     'maroon2' : (238, 48, 167),
 *     'maroon3' : (205, 41, 144),             # <<<<<<<<<<<<<<
 *     'maroon4' : (139, 28, 98),
 *     'mediumaquamarine' : (102, 205, 170),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_maroon3, __pyx_tuple__335) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":463
 *     'maroon2' : (238, 48, 167),
 *     'maroon3' : (205, 41, 144),
 *     'maroon4' : (139, 28, 98),             # <<<<<<<<<<<<<<
 *     'mediumaquamarine' : (102, 205, 170),
 *     'mediumblue' : (0, 0, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_maroon4, __pyx_tuple__336) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":464
 *     'maroon3' : (205, 41, 144),
 *     'maroon4' : (139, 28, 98),
 *     'mediumaquamarine' : (102, 205, 170),             # <<<<<<<<<<<<<<
 *     'mediumblue' : (0, 0, 205),
 *     'mediumorchid' : (186, 85, 211),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumaquamarine, __pyx_tuple__18) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":465
 *     'maroon4' : (139, 28, 98),
 *     'mediumaquamarine' : (102, 205, 170),
 *     'mediumblue' : (0, 0, 205),             # <<<<<<<<<<<<<<
 *     'mediumorchid' : (186, 85, 211),
 *     'mediumorchid1' : (224, 102, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumblue, __pyx_tuple__33) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":466
 *     'mediumaquamarine' : (102, 205, 170),
 *     'mediumblue' : (0, 0, 205),
 *     'mediumorchid' : (186, 85, 211),             # <<<<<<<<<<<<<<
 *     'mediumorchid1' : (224, 102, 255),
 *     'mediumorchid2' : (209, 95, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumorchid, __pyx_tuple__337) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":467
 *     'mediumblue' : (0, 0, 205),
 *     'mediumorchid' : (186, 85, 211),
 *     'mediumorchid1' : (224, 102, 255),             # <<<<<<<<<<<<<<
 *     'mediumorchid2' : (209, 95, 238),
 *     'mediumorchid3' : (180, 82, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumorchid1, __pyx_tuple__338) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":468
 *     'mediumorchid' : (186, 85, 211),
 *     'mediumorchid1' : (224, 102, 255),
 *     'mediumorchid2' : (209, 95, 238),             # <<<<<<<<<<<<<<
 *     'mediumorchid3' : (180, 82, 205),
 *     'mediumorchid4' : (122, 55, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumorchid2, __pyx_tuple__339) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":469
 *     'mediumorchid1' : (224, 102, 255),
 *     'mediumorchid2' : (209, 95, 238),
 *     'mediumorchid3' : (180, 82, 205),             # <<<<<<<<<<<<<<
 *     'mediumorchid4' : (122, 55, 139),
 *     'mediumpurple' : (147, 112, 219),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumorchid3, __pyx_tuple__340) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":470
 *     'mediumorchid2' : (209, 95, 238),
 *     'mediumorchid3' : (180, 82, 205),
 *     'mediumorchid4' : (122, 55, 139),             # <<<<<<<<<<<<<<
 *     'mediumpurple' : (147, 112, 219),
 *     'mediumpurple1' : (171, 130, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumorchid4, __pyx_tuple__341) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":471
 *     'mediumorchid3' : (180, 82, 205),
 *     'mediumorchid4' : (122, 55, 139),
 *     'mediumpurple' : (147, 112, 219),             # <<<<<<<<<<<<<<
 *     'mediumpurple1' : (171, 130, 255),
 *     'mediumpurple2' : (159, 121, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumpurple, __pyx_tuple__342) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":472
 *     'mediumorchid4' : (122, 55, 139),
 *     'mediumpurple' : (147, 112, 219),
 *     'mediumpurple1' : (171, 130, 255),             # <<<<<<<<<<<<<<
 *     'mediumpurple2' : (159, 121, 238),
 *     'mediumpurple3' : (137, 104, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumpurple1, __pyx_tuple__343) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":473
 *     'mediumpurple' : (147, 112, 219),
 *     'mediumpurple1' : (171, 130, 255),
 *     'mediumpurple2' : (159, 121, 238),             # <<<<<<<<<<<<<<
 *     'mediumpurple3' : (137, 104, 205),
 *     'mediumpurple4' : (93, 71, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumpurple2, __pyx_tuple__344) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":474
 *     'mediumpurple1' : (171, 130, 255),
 *     'mediumpurple2' : (159, 121, 238),
 *     'mediumpurple3' : (137, 104, 205),             # <<<<<<<<<<<<<<
 *     'mediumpurple4' : (93, 71, 139),
 *     'mediumseagreen' : (60, 179, 113),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumpurple3, __pyx_tuple__345) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":475
 *     'mediumpurple2' : (159, 121, 238),
 *     'mediumpurple3' : (137, 104, 205),
 *     'mediumpurple4' : (93, 71, 139),             # <<<<<<<<<<<<<<
 *     'mediumseagreen' : (60, 179, 113),
 *     'mediumslateblue' : (123, 104, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumpurple4, __pyx_tuple__346) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":476
 *     'mediumpurple3' : (137, 104, 205),
 *     'mediumpurple4' : (93, 71, 139),
 *     'mediumseagreen' : (60, 179, 113),             # <<<<<<<<<<<<<<
 *     'mediumslateblue' : (123, 104, 238),
 *     'mediumspringgreen' : (0, 250, 154),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumseagreen, __pyx_tuple__347) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":477
 *     'mediumpurple4' : (93, 71, 139),
 *     'mediumseagreen' : (60, 179, 113),
 *     'mediumslateblue' : (123, 104, 238),             # <<<<<<<<<<<<<<
 *     'mediumspringgreen' : (0, 250, 154),
 *     'mediumturquoise' : (72, 209, 204),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumslateblue, __pyx_tuple__348) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":478
 *     'mediumseagreen' : (60, 179, 113),
 *     'mediumslateblue' : (123, 104, 238),
 *     'mediumspringgreen' : (0, 250, 154),             # <<<<<<<<<<<<<<
 *     'mediumturquoise' : (72, 209, 204),
 *     'mediumvioletred' : (199, 21, 133),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumspringgreen, __pyx_tuple__349) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":479
 *     'mediumslateblue' : (123, 104, 238),
 *     'mediumspringgreen' : (0, 250, 154),
 *     'mediumturquoise' : (72, 209, 204),             # <<<<<<<<<<<<<<
 *     'mediumvioletred' : (199, 21, 133),
 *     'midnightblue' : (25, 25, 112),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumturquoise, __pyx_tuple__350) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":480
 *     'mediumspringgreen' : (0, 250, 154),
 *     'mediumturquoise' : (72, 209, 204),
 *     'mediumvioletred' : (199, 21, 133),             # <<<<<<<<<<<<<<
 *     'midnightblue' : (25, 25, 112),
 *     'mintcream' : (245, 255, 250),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mediumvioletred, __pyx_tuple__351) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":481
 *     'mediumturquoise' : (72, 209, 204),
 *     'mediumvioletred' : (199, 21, 133),
 *     'midnightblue' : (25, 25, 112),             # <<<<<<<<<<<<<<
 *     'mintcream' : (245, 255, 250),
 *     'mistyrose' : (255, 228, 225),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_midnightblue, __pyx_tuple__352) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":482
 *     'mediumvioletred' : (199, 21, 133),
 *     'midnightblue' : (25, 25, 112),
 *     'mintcream' : (245, 255, 250),             # <<<<<<<<<<<<<<
 *     'mistyrose' : (255, 228, 225),
 *     'mistyrose1' : (255, 228, 225),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mintcream, __pyx_tuple__353) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":483
 *     'midnightblue' : (25, 25, 112),
 *     'mintcream' : (245, 255, 250),
 *     'mistyrose' : (255, 228, 225),             # <<<<<<<<<<<<<<
 *     'mistyrose1' : (255, 228, 225),
 *     'mistyrose2' : (238, 213, 210),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mistyrose, __pyx_tuple__354) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":484
 *     'mintcream' : (245, 255, 250),
 *     'mistyrose' : (255, 228, 225),
 *     'mistyrose1' : (255, 228, 225),             # <<<<<<<<<<<<<<
 *     'mistyrose2' : (238, 213, 210),
 *     'mistyrose3' : (205, 183, 181),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mistyrose1, __pyx_tuple__354) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":485
 *     'mistyrose' : (255, 228, 225),
 *     'mistyrose1' : (255, 228, 225),
 *     'mistyrose2' : (238, 213, 210),             # <<<<<<<<<<<<<<
 *     'mistyrose3' : (205, 183, 181),
 *     'mistyrose4' : (139, 125, 123),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mistyrose2, __pyx_tuple__355) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":486
 *     'mistyrose1' : (255, 228, 225),
 *     'mistyrose2' : (238, 213, 210),
 *     'mistyrose3' : (205, 183, 181),             # <<<<<<<<<<<<<<
 *     'mistyrose4' : (139, 125, 123),
 *     'moccasin' : (255, 228, 181),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mistyrose3, __pyx_tuple__356) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":487
 *     'mistyrose2' : (238, 213, 210),
 *     'mistyrose3' : (205, 183, 181),
 *     'mistyrose4' : (139, 125, 123),             # <<<<<<<<<<<<<<
 *     'moccasin' : (255, 228, 181),
 *     'navajowhite' : (255, 222, 173),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_mistyrose4, __pyx_tuple__357) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":488
 *     'mistyrose3' : (205, 183, 181),
 *     'mistyrose4' : (139, 125, 123),
 *     'moccasin' : (255, 228, 181),             # <<<<<<<<<<<<<<
 *     'navajowhite' : (255, 222, 173),
 *     'navajowhite1' : (255, 222, 173),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_moccasin, __pyx_tuple__358) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":489
 *     'mistyrose4' : (139, 125, 123),
 *     'moccasin' : (255, 228, 181),
 *     'navajowhite' : (255, 222, 173),             # <<<<<<<<<<<<<<
 *     'navajowhite1' : (255, 222, 173),
 *     'navajowhite2' : (238, 207, 161),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_navajowhite, __pyx_tuple__359) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":490
 *     'moccasin' : (255, 228, 181),
 *     'navajowhite' : (255, 222, 173),
 *     'navajowhite1' : (255, 222, 173),             # <<<<<<<<<<<<<<
 *     'navajowhite2' : (238, 207, 161),
 *     'navajowhite3' : (205, 179, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_navajowhite1, __pyx_tuple__359) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":491
 *     'navajowhite' : (255, 222, 173),
 *     'navajowhite1' : (255, 222, 173),
 *     'navajowhite2' : (238, 207, 161),             # <<<<<<<<<<<<<<
 *     'navajowhite3' : (205, 179, 139),
 *     'navajowhite4' : (139, 121, 94),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_navajowhite2, __pyx_tuple__360) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":492
 *     'navajowhite1' : (255, 222, 173),
 *     'navajowhite2' : (238, 207, 161),
 *     'navajowhite3' : (205, 179, 139),             # <<<<<<<<<<<<<<
 *     'navajowhite4' : (139, 121, 94),
 *     'navy' : (0, 0, 128),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_navajowhite3, __pyx_tuple__361) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":493
 *     'navajowhite2' : (238, 207, 161),
 *     'navajowhite3' : (205, 179, 139),
 *     'navajowhite4' : (139, 121, 94),             # <<<<<<<<<<<<<<
 *     'navy' : (0, 0, 128),
 *     'navyblue' : (0, 0, 128),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_navajowhite4, __pyx_tuple__362) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":494
 *     'navajowhite3' : (205, 179, 139),
 *     'navajowhite4' : (139, 121, 94),
 *     'navy' : (0, 0, 128),             # <<<<<<<<<<<<<<
 *     'navyblue' : (0, 0, 128),
 *     'oldlace' : (253, 245, 230),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_navy, __pyx_tuple__363) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":495
 *     'navajowhite4' : (139, 121, 94),
 *     'navy' : (0, 0, 128),
 *     'navyblue' : (0, 0, 128),             # <<<<<<<<<<<<<<
 *     'oldlace' : (253, 245, 230),
 *     'olivedrab' : (107, 142, 35),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_navyblue, __pyx_tuple__363) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":496
 *     'navy' : (0, 0, 128),
 *     'navyblue' : (0, 0, 128),
 *     'oldlace' : (253, 245, 230),             # <<<<<<<<<<<<<<
 *     'olivedrab' : (107, 142, 35),
 *     'olivedrab1' : (192, 255, 62),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_oldlace, __pyx_tuple__364) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":497
 *     'navyblue' : (0, 0, 128),
 *     'oldlace' : (253, 245, 230),
 *     'olivedrab' : (107, 142, 35),             # <<<<<<<<<<<<<<
 *     'olivedrab1' : (192, 255, 62),
 *     'olivedrab2' : (179, 238, 58),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_olivedrab, __pyx_tuple__365) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":498
 *     'oldlace' : (253, 245, 230),
 *     'olivedrab' : (107, 142, 35),
 *     'olivedrab1' : (192, 255, 62),             # <<<<<<<<<<<<<<
 *     'olivedrab2' : (179, 238, 58),
 *     'olivedrab3' : (154, 205, 50),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_olivedrab1, __pyx_tuple__366) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":499
 *     'olivedrab' : (107, 142, 35),
 *     'olivedrab1' : (192, 255, 62),
 *     'olivedrab2' : (179, 238, 58),             # <<<<<<<<<<<<<<
 *     'olivedrab3' : (154, 205, 50),
 *     'olivedrab4' : (105, 139, 34),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_olivedrab2, __pyx_tuple__367) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":500
 *     'olivedrab1' : (192, 255, 62),
 *     'olivedrab2' : (179, 238, 58),
 *     'olivedrab3' : (154, 205, 50),             # <<<<<<<<<<<<<<
 *     'olivedrab4' : (105, 139, 34),
 *     'orange' : (255, 165, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_olivedrab3, __pyx_tuple__368) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":501
 *     'olivedrab2' : (179, 238, 58),
 *     'olivedrab3' : (154, 205, 50),
 *     'olivedrab4' : (105, 139, 34),             # <<<<<<<<<<<<<<
 *     'orange' : (255, 165, 0),
 *     'orange1' : (255, 165, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_olivedrab4, __pyx_tuple__369) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":502
 *     'olivedrab3' : (154, 205, 50),
 *     'olivedrab4' : (105, 139, 34),
 *     'orange' : (255, 165, 0),             # <<<<<<<<<<<<<<
 *     'orange1' : (255, 165, 0),
 *     'orange2' : (238, 154, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orange, __pyx_tuple__370) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":503
 *     'olivedrab4' : (105, 139, 34),
 *     'orange' : (255, 165, 0),
 *     'orange1' : (255, 165, 0),             # <<<<<<<<<<<<<<
 *     'orange2' : (238, 154, 0),
 *     'orange3' : (205, 133, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orange1, __pyx_tuple__370) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":504
 *     'orange' : (255, 165, 0),
 *     'orange1' : (255, 165, 0),
 *     'orange2' : (238, 154, 0),             # <<<<<<<<<<<<<<
 *     'orange3' : (205, 133, 0),
 *     'orange4' : (139, 90, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orange2, __pyx_tuple__371) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":505
 *     'orange1' : (255, 165, 0),
 *     'orange2' : (238, 154, 0),
 *     'orange3' : (205, 133, 0),             # <<<<<<<<<<<<<<
 *     'orange4' : (139, 90, 0),
 *     'orangered' : (255, 69, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orange3, __pyx_tuple__372) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":506
 *     'orange2' : (238, 154, 0),
 *     'orange3' : (205, 133, 0),
 *     'orange4' : (139, 90, 0),             # <<<<<<<<<<<<<<
 *     'orangered' : (255, 69, 0),
 *     'orangered1' : (255, 69, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orange4, __pyx_tuple__373) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":507
 *     'orange3' : (205, 133, 0),
 *     'orange4' : (139, 90, 0),
 *     'orangered' : (255, 69, 0),             # <<<<<<<<<<<<<<
 *     'orangered1' : (255, 69, 0),
 *     'orangered2' : (238, 64, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orangered, __pyx_tuple__374) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":508
 *     'orange4' : (139, 90, 0),
 *     'orangered' : (255, 69, 0),
 *     'orangered1' : (255, 69, 0),             # <<<<<<<<<<<<<<
 *     'orangered2' : (238, 64, 0),
 *     'orangered3' : (205, 55, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orangered1, __pyx_tuple__374) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":509
 *     'orangered' : (255, 69, 0),
 *     'orangered1' : (255, 69, 0),
 *     'orangered2' : (238, 64, 0),             # <<<<<<<<<<<<<<
 *     'orangered3' : (205, 55, 0),
 *     'orangered4' : (139, 37, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orangered2, __pyx_tuple__375) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":510
 *     'orangered1' : (255, 69, 0),
 *     'orangered2' : (238, 64, 0),
 *     'orangered3' : (205, 55, 0),             # <<<<<<<<<<<<<<
 *     'orangered4' : (139, 37, 0),
 *     'orchid' : (218, 112, 214),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orangered3, __pyx_tuple__376) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":511
 *     'orangered2' : (238, 64, 0),
 *     'orangered3' : (205, 55, 0),
 *     'orangered4' : (139, 37, 0),             # <<<<<<<<<<<<<<
 *     'orchid' : (218, 112, 214),
 *     'orchid1' : (255, 131, 250),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orangered4, __pyx_tuple__377) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":512
 *     'orangered3' : (205, 55, 0),
 *     'orangered4' : (139, 37, 0),
 *     'orchid' : (218, 112, 214),             # <<<<<<<<<<<<<<
 *     'orchid1' : (255, 131, 250),
 *     'orchid2' : (238, 122, 233),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orchid, __pyx_tuple__378) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":513
 *     'orangered4' : (139, 37, 0),
 *     'orchid' : (218, 112, 214),
 *     'orchid1' : (255, 131, 250),             # <<<<<<<<<<<<<<
 *     'orchid2' : (238, 122, 233),
 *     'orchid3' : (205, 105, 201),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orchid1, __pyx_tuple__379) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":514
 *     'orchid' : (218, 112, 214),
 *     'orchid1' : (255, 131, 250),
 *     'orchid2' : (238, 122, 233),             # <<<<<<<<<<<<<<
 *     'orchid3' : (205, 105, 201),
 *     'orchid4' : (139, 71, 137),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orchid2, __pyx_tuple__380) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":515
 *     'orchid1' : (255, 131, 250),
 *     'orchid2' : (238, 122, 233),
 *     'orchid3' : (205, 105, 201),             # <<<<<<<<<<<<<<
 *     'orchid4' : (139, 71, 137),
 *     'palegoldenrod' : (238, 232, 170),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orchid3, __pyx_tuple__381) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":516
 *     'orchid2' : (238, 122, 233),
 *     'orchid3' : (205, 105, 201),
 *     'orchid4' : (139, 71, 137),             # <<<<<<<<<<<<<<
 *     'palegoldenrod' : (238, 232, 170),
 *     'palegreen' : (152, 251, 152),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_orchid4, __pyx_tuple__382) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":517
 *     'orchid3' : (205, 105, 201),
 *     'orchid4' : (139, 71, 137),
 *     'palegoldenrod' : (238, 232, 170),             # <<<<<<<<<<<<<<
 *     'palegreen' : (152, 251, 152),
 *     'palegreen1' : (154, 255, 154),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palegoldenrod, __pyx_tuple__383) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":518
 *     'orchid4' : (139, 71, 137),
 *     'palegoldenrod' : (238, 232, 170),
 *     'palegreen' : (152, 251, 152),             # <<<<<<<<<<<<<<
 *     'palegreen1' : (154, 255, 154),
 *     'palegreen2' : (144, 238, 144),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palegreen, __pyx_tuple__384) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":519
 *     'palegoldenrod' : (238, 232, 170),
 *     'palegreen' : (152, 251, 152),
 *     'palegreen1' : (154, 255, 154),             # <<<<<<<<<<<<<<
 *     'palegreen2' : (144, 238, 144),
 *     'palegreen3' : (124, 205, 124),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palegreen1, __pyx_tuple__385) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":520
 *     'palegreen' : (152, 251, 152),
 *     'palegreen1' : (154, 255, 154),
 *     'palegreen2' : (144, 238, 144),             # <<<<<<<<<<<<<<
 *     'palegreen3' : (124, 205, 124),
 *     'palegreen4' : (84, 139, 84),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palegreen2, __pyx_tuple__300) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":521
 *     'palegreen1' : (154, 255, 154),
 *     'palegreen2' : (144, 238, 144),
 *     'palegreen3' : (124, 205, 124),             # <<<<<<<<<<<<<<
 *     'palegreen4' : (84, 139, 84),
 *     'paleturquoise' : (175, 238, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palegreen3, __pyx_tuple__386) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":522
 *     'palegreen2' : (144, 238, 144),
 *     'palegreen3' : (124, 205, 124),
 *     'palegreen4' : (84, 139, 84),             # <<<<<<<<<<<<<<
 *     'paleturquoise' : (175, 238, 238),
 *     'paleturquoise1' : (187, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palegreen4, __pyx_tuple__387) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":523
 *     'palegreen3' : (124, 205, 124),
 *     'palegreen4' : (84, 139, 84),
 *     'paleturquoise' : (175, 238, 238),             # <<<<<<<<<<<<<<
 *     'paleturquoise1' : (187, 255, 255),
 *     'paleturquoise2' : (174, 238, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_paleturquoise, __pyx_tuple__388) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":524
 *     'palegreen4' : (84, 139, 84),
 *     'paleturquoise' : (175, 238, 238),
 *     'paleturquoise1' : (187, 255, 255),             # <<<<<<<<<<<<<<
 *     'paleturquoise2' : (174, 238, 238),
 *     'paleturquoise3' : (150, 205, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_paleturquoise1, __pyx_tuple__389) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":525
 *     'paleturquoise' : (175, 238, 238),
 *     'paleturquoise1' : (187, 255, 255),
 *     'paleturquoise2' : (174, 238, 238),             # <<<<<<<<<<<<<<
 *     'paleturquoise3' : (150, 205, 205),
 *     'paleturquoise4' : (102, 139, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_paleturquoise2, __pyx_tuple__390) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":526
 *     'paleturquoise1' : (187, 255, 255),
 *     'paleturquoise2' : (174, 238, 238),
 *     'paleturquoise3' : (150, 205, 205),             # <<<<<<<<<<<<<<
 *     'paleturquoise4' : (102, 139, 139),
 *     'palevioletred' : (219, 112, 147),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_paleturquoise3, __pyx_tuple__391) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":527
 *     'paleturquoise2' : (174, 238, 238),
 *     'paleturquoise3' : (150, 205, 205),
 *     'paleturquoise4' : (102, 139, 139),             # <<<<<<<<<<<<<<
 *     'palevioletred' : (219, 112, 147),
 *     'palevioletred1' : (255, 130, 171),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_paleturquoise4, __pyx_tuple__392) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":528
 *     'paleturquoise3' : (150, 205, 205),
 *     'paleturquoise4' : (102, 139, 139),
 *     'palevioletred' : (219, 112, 147),             # <<<<<<<<<<<<<<
 *     'palevioletred1' : (255, 130, 171),
 *     'palevioletred2' : (238, 121, 159),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palevioletred, __pyx_tuple__393) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":529
 *     'paleturquoise4' : (102, 139, 139),
 *     'palevioletred' : (219, 112, 147),
 *     'palevioletred1' : (255, 130, 171),             # <<<<<<<<<<<<<<
 *     'palevioletred2' : (238, 121, 159),
 *     'palevioletred3' : (205, 104, 137),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palevioletred1, __pyx_tuple__394) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":530
 *     'palevioletred' : (219, 112, 147),
 *     'palevioletred1' : (255, 130, 171),
 *     'palevioletred2' : (238, 121, 159),             # <<<<<<<<<<<<<<
 *     'palevioletred3' : (205, 104, 137),
 *     'palevioletred4' : (139, 71, 93),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palevioletred2, __pyx_tuple__395) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":531
 *     'palevioletred1' : (255, 130, 171),
 *     'palevioletred2' : (238, 121, 159),
 *     'palevioletred3' : (205, 104, 137),             # <<<<<<<<<<<<<<
 *     'palevioletred4' : (139, 71, 93),
 *     'papayawhip' : (255, 239, 213),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palevioletred3, __pyx_tuple__396) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":532
 *     'palevioletred2' : (238, 121, 159),
 *     'palevioletred3' : (205, 104, 137),
 *     'palevioletred4' : (139, 71, 93),             # <<<<<<<<<<<<<<
 *     'papayawhip' : (255, 239, 213),
 *     'peachpuff' : (255, 218, 185),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_palevioletred4, __pyx_tuple__397) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":533
 *     'palevioletred3' : (205, 104, 137),
 *     'palevioletred4' : (139, 71, 93),
 *     'papayawhip' : (255, 239, 213),             # <<<<<<<<<<<<<<
 *     'peachpuff' : (255, 218, 185),
 *     'peachpuff1' : (255, 218, 185),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_papayawhip, __pyx_tuple__398) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":534
 *     'palevioletred4' : (139, 71, 93),
 *     'papayawhip' : (255, 239, 213),
 *     'peachpuff' : (255, 218, 185),             # <<<<<<<<<<<<<<
 *     'peachpuff1' : (255, 218, 185),
 *     'peachpuff2' : (238, 203, 173),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_peachpuff, __pyx_tuple__399) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":535
 *     'papayawhip' : (255, 239, 213),
 *     'peachpuff' : (255, 218, 185),
 *     'peachpuff1' : (255, 218, 185),             # <<<<<<<<<<<<<<
 *     'peachpuff2' : (238, 203, 173),
 *     'peachpuff3' : (205, 175, 149),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_peachpuff1, __pyx_tuple__399) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":536
 *     'peachpuff' : (255, 218, 185),
 *     'peachpuff1' : (255, 218, 185),
 *     'peachpuff2' : (238, 203, 173),             # <<<<<<<<<<<<<<
 *     'peachpuff3' : (205, 175, 149),
 *     'peachpuff4' : (139, 119, 101),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_peachpuff2, __pyx_tuple__400) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":537
 *     'peachpuff1' : (255, 218, 185),
 *     'peachpuff2' : (238, 203, 173),
 *     'peachpuff3' : (205, 175, 149),             # <<<<<<<<<<<<<<
 *     'peachpuff4' : (139, 119, 101),
 *     'peru' : (205, 133, 63),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_peachpuff3, __pyx_tuple__401) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":538
 *     'peachpuff2' : (238, 203, 173),
 *     'peachpuff3' : (205, 175, 149),
 *     'peachpuff4' : (139, 119, 101),             # <<<<<<<<<<<<<<
 *     'peru' : (205, 133, 63),
 *     'pink' : (255, 192, 203),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_peachpuff4, __pyx_tuple__402) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":539
 *     'peachpuff3' : (205, 175, 149),
 *     'peachpuff4' : (139, 119, 101),
 *     'peru' : (205, 133, 63),             # <<<<<<<<<<<<<<
 *     'pink' : (255, 192, 203),
 *     'pink1' : (255, 181, 197),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_peru, __pyx_tuple__403) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":540
 *     'peachpuff4' : (139, 119, 101),
 *     'peru' : (205, 133, 63),
 *     'pink' : (255, 192, 203),             # <<<<<<<<<<<<<<
 *     'pink1' : (255, 181, 197),
 *     'pink2' : (238, 169, 184),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_pink, __pyx_tuple__404) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":541
 *     'peru' : (205, 133, 63),
 *     'pink' : (255, 192, 203),
 *     'pink1' : (255, 181, 197),             # <<<<<<<<<<<<<<
 *     'pink2' : (238, 169, 184),
 *     'pink3' : (205, 145, 158),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_pink1, __pyx_tuple__405) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":542
 *     'pink' : (255, 192, 203),
 *     'pink1' : (255, 181, 197),
 *     'pink2' : (238, 169, 184),             # <<<<<<<<<<<<<<
 *     'pink3' : (205, 145, 158),
 *     'pink4' : (139, 99, 108),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_pink2, __pyx_tuple__406) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":543
 *     'pink1' : (255, 181, 197),
 *     'pink2' : (238, 169, 184),
 *     'pink3' : (205, 145, 158),             # <<<<<<<<<<<<<<
 *     'pink4' : (139, 99, 108),
 *     'plum' : (221, 160, 221),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_pink3, __pyx_tuple__407) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":544
 *     'pink2' : (238, 169, 184),
 *     'pink3' : (205, 145, 158),
 *     'pink4' : (139, 99, 108),             # <<<<<<<<<<<<<<
 *     'plum' : (221, 160, 221),
 *     'plum1' : (255, 187, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_pink4, __pyx_tuple__408) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":545
 *     'pink3' : (205, 145, 158),
 *     'pink4' : (139, 99, 108),
 *     'plum' : (221, 160, 221),             # <<<<<<<<<<<<<<
 *     'plum1' : (255, 187, 255),
 *     'plum2' : (238, 174, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_plum, __pyx_tuple__409) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":546
 *     'pink4' : (139, 99, 108),
 *     'plum' : (221, 160, 221),
 *     'plum1' : (255, 187, 255),             # <<<<<<<<<<<<<<
 *     'plum2' : (238, 174, 238),
 *     'plum3' : (205, 150, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_plum1, __pyx_tuple__410) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":547
 *     'plum' : (221, 160, 221),
 *     'plum1' : (255, 187, 255),
 *     'plum2' : (238, 174, 238),             # <<<<<<<<<<<<<<
 *     'plum3' : (205, 150, 205),
 *     'plum4' : (139, 102, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_plum2, __pyx_tuple__411) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":548
 *     'plum1' : (255, 187, 255),
 *     'plum2' : (238, 174, 238),
 *     'plum3' : (205, 150, 205),             # <<<<<<<<<<<<<<
 *     'plum4' : (139, 102, 139),
 *     'powderblue' : (176, 224, 230),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_plum3, __pyx_tuple__412) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":549
 *     'plum2' : (238, 174, 238),
 *     'plum3' : (205, 150, 205),
 *     'plum4' : (139, 102, 139),             # <<<<<<<<<<<<<<
 *     'powderblue' : (176, 224, 230),
 *     'purple' : (160, 32, 240),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_plum4, __pyx_tuple__413) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":550
 *     'plum3' : (205, 150, 205),
 *     'plum4' : (139, 102, 139),
 *     'powderblue' : (176, 224, 230),             # <<<<<<<<<<<<<<
 *     'purple' : (160, 32, 240),
 *     'purple1' : (155, 48, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_powderblue, __pyx_tuple__414) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":551
 *     'plum4' : (139, 102, 139),
 *     'powderblue' : (176, 224, 230),
 *     'purple' : (160, 32, 240),             # <<<<<<<<<<<<<<
 *     'purple1' : (155, 48, 255),
 *     'purple2' : (145, 44, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_purple, __pyx_tuple__415) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":552
 *     'powderblue' : (176, 224, 230),
 *     'purple' : (160, 32, 240),
 *     'purple1' : (155, 48, 255),             # <<<<<<<<<<<<<<
 *     'purple2' : (145, 44, 238),
 *     'purple3' : (125, 38, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_purple1, __pyx_tuple__416) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":553
 *     'purple' : (160, 32, 240),
 *     'purple1' : (155, 48, 255),
 *     'purple2' : (145, 44, 238),             # <<<<<<<<<<<<<<
 *     'purple3' : (125, 38, 205),
 *     'purple4' : (85, 26, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_purple2, __pyx_tuple__417) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":554
 *     'purple1' : (155, 48, 255),
 *     'purple2' : (145, 44, 238),
 *     'purple3' : (125, 38, 205),             # <<<<<<<<<<<<<<
 *     'purple4' : (85, 26, 139),
 *     'red' : (255, 0, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_purple3, __pyx_tuple__418) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":555
 *     'purple2' : (145, 44, 238),
 *     'purple3' : (125, 38, 205),
 *     'purple4' : (85, 26, 139),             # <<<<<<<<<<<<<<
 *     'red' : (255, 0, 0),
 *     'red1' : (255, 0, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_purple4, __pyx_tuple__419) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":556
 *     'purple3' : (125, 38, 205),
 *     'purple4' : (85, 26, 139),
 *     'red' : (255, 0, 0),             # <<<<<<<<<<<<<<
 *     'red1' : (255, 0, 0),
 *     'red2' : (238, 0, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_red, __pyx_tuple__420) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":557
 *     'purple4' : (85, 26, 139),
 *     'red' : (255, 0, 0),
 *     'red1' : (255, 0, 0),             # <<<<<<<<<<<<<<
 *     'red2' : (238, 0, 0),
 *     'red3' : (205, 0, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_red1, __pyx_tuple__420) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":558
 *     'red' : (255, 0, 0),
 *     'red1' : (255, 0, 0),
 *     'red2' : (238, 0, 0),             # <<<<<<<<<<<<<<
 *     'red3' : (205, 0, 0),
 *     'red4' : (139, 0, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_red2, __pyx_tuple__421) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":559
 *     'red1' : (255, 0, 0),
 *     'red2' : (238, 0, 0),
 *     'red3' : (205, 0, 0),             # <<<<<<<<<<<<<<
 *     'red4' : (139, 0, 0),
 *     'rosybrown' : (188, 143, 143),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_red3, __pyx_tuple__422) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":560
 *     'red2' : (238, 0, 0),
 *     'red3' : (205, 0, 0),
 *     'red4' : (139, 0, 0),             # <<<<<<<<<<<<<<
 *     'rosybrown' : (188, 143, 143),
 *     'rosybrown1' : (255, 193, 193),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_red4, __pyx_tuple__98) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":561
 *     'red3' : (205, 0, 0),
 *     'red4' : (139, 0, 0),
 *     'rosybrown' : (188, 143, 143),             # <<<<<<<<<<<<<<
 *     'rosybrown1' : (255, 193, 193),
 *     'rosybrown2' : (238, 180, 180),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_rosybrown, __pyx_tuple__423) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":562
 *     'red4' : (139, 0, 0),
 *     'rosybrown' : (188, 143, 143),
 *     'rosybrown1' : (255, 193, 193),             # <<<<<<<<<<<<<<
 *     'rosybrown2' : (238, 180, 180),
 *     'rosybrown3' : (205, 155, 155),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_rosybrown1, __pyx_tuple__424) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":563
 *     'rosybrown' : (188, 143, 143),
 *     'rosybrown1' : (255, 193, 193),
 *     'rosybrown2' : (238, 180, 180),             # <<<<<<<<<<<<<<
 *     'rosybrown3' : (205, 155, 155),
 *     'rosybrown4' : (139, 105, 105),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_rosybrown2, __pyx_tuple__425) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":564
 *     'rosybrown1' : (255, 193, 193),
 *     'rosybrown2' : (238, 180, 180),
 *     'rosybrown3' : (205, 155, 155),             # <<<<<<<<<<<<<<
 *     'rosybrown4' : (139, 105, 105),
 *     'royalblue' : (65, 105, 225),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_rosybrown3, __pyx_tuple__426) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":565
 *     'rosybrown2' : (238, 180, 180),
 *     'rosybrown3' : (205, 155, 155),
 *     'rosybrown4' : (139, 105, 105),             # <<<<<<<<<<<<<<
 *     'royalblue' : (65, 105, 225),
 *     'royalblue1' : (72, 118, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_rosybrown4, __pyx_tuple__427) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":566
 *     'rosybrown3' : (205, 155, 155),
 *     'rosybrown4' : (139, 105, 105),
 *     'royalblue' : (65, 105, 225),             # <<<<<<<<<<<<<<
 *     'royalblue1' : (72, 118, 255),
 *     'royalblue2' : (67, 110, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_royalblue, __pyx_tuple__428) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":567
 *     'rosybrown4' : (139, 105, 105),
 *     'royalblue' : (65, 105, 225),
 *     'royalblue1' : (72, 118, 255),             # <<<<<<<<<<<<<<
 *     'royalblue2' : (67, 110, 238),
 *     'royalblue3' : (58, 95, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_royalblue1, __pyx_tuple__429) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":568
 *     'royalblue' : (65, 105, 225),
 *     'royalblue1' : (72, 118, 255),
 *     'royalblue2' : (67, 110, 238),             # <<<<<<<<<<<<<<
 *     'royalblue3' : (58, 95, 205),
 *     'royalblue4' : (39, 64, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_royalblue2, __pyx_tuple__430) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":569
 *     'royalblue1' : (72, 118, 255),
 *     'royalblue2' : (67, 110, 238),
 *     'royalblue3' : (58, 95, 205),             # <<<<<<<<<<<<<<
 *     'royalblue4' : (39, 64, 139),
 *     'saddlebrown' : (139, 69, 19),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_royalblue3, __pyx_tuple__431) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":570
 *     'royalblue2' : (67, 110, 238),
 *     'royalblue3' : (58, 95, 205),
 *     'royalblue4' : (39, 64, 139),             # <<<<<<<<<<<<<<
 *     'saddlebrown' : (139, 69, 19),
 *     'salmon' : (250, 128, 114),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_royalblue4, __pyx_tuple__432) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":571
 *     'royalblue3' : (58, 95, 205),
 *     'royalblue4' : (39, 64, 139),
 *     'saddlebrown' : (139, 69, 19),             # <<<<<<<<<<<<<<
 *     'salmon' : (250, 128, 114),
 *     'salmon1' : (255, 140, 105),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_saddlebrown, __pyx_tuple__59) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":572
 *     'royalblue4' : (39, 64, 139),
 *     'saddlebrown' : (139, 69, 19),
 *     'salmon' : (250, 128, 114),             # <<<<<<<<<<<<<<
 *     'salmon1' : (255, 140, 105),
 *     'salmon2' : (238, 130, 98),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_salmon, __pyx_tuple__433) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":573
 *     'saddlebrown' : (139, 69, 19),
 *     'salmon' : (250, 128, 114),
 *     'salmon1' : (255, 140, 105),             # <<<<<<<<<<<<<<
 *     'salmon2' : (238, 130, 98),
 *     'salmon3' : (205, 112, 84),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_salmon1, __pyx_tuple__434) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":574
 *     'salmon' : (250, 128, 114),
 *     'salmon1' : (255, 140, 105),
 *     'salmon2' : (238, 130, 98),             # <<<<<<<<<<<<<<
 *     'salmon3' : (205, 112, 84),
 *     'salmon4' : (139, 76, 57),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_salmon2, __pyx_tuple__435) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":575
 *     'salmon1' : (255, 140, 105),
 *     'salmon2' : (238, 130, 98),
 *     'salmon3' : (205, 112, 84),             # <<<<<<<<<<<<<<
 *     'salmon4' : (139, 76, 57),
 *     'sandybrown' : (244, 164, 96),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_salmon3, __pyx_tuple__436) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":576
 *     'salmon2' : (238, 130, 98),
 *     'salmon3' : (205, 112, 84),
 *     'salmon4' : (139, 76, 57),             # <<<<<<<<<<<<<<
 *     'sandybrown' : (244, 164, 96),
 *     'seagreen' : (46, 139, 87),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_salmon4, __pyx_tuple__437) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":577
 *     'salmon3' : (205, 112, 84),
 *     'salmon4' : (139, 76, 57),
 *     'sandybrown' : (244, 164, 96),             # <<<<<<<<<<<<<<
 *     'seagreen' : (46, 139, 87),
 *     'seagreen1' : (84, 255, 159),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sandybrown, __pyx_tuple__438) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":578
 *     'salmon4' : (139, 76, 57),
 *     'sandybrown' : (244, 164, 96),
 *     'seagreen' : (46, 139, 87),             # <<<<<<<<<<<<<<
 *     'seagreen1' : (84, 255, 159),
 *     'seagreen2' : (78, 238, 148),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_seagreen, __pyx_tuple__439) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":579
 *     'sandybrown' : (244, 164, 96),
 *     'seagreen' : (46, 139, 87),
 *     'seagreen1' : (84, 255, 159),             # <<<<<<<<<<<<<<
 *     'seagreen2' : (78, 238, 148),
 *     'seagreen3' : (67, 205, 128),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_seagreen1, __pyx_tuple__440) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":580
 *     'seagreen' : (46, 139, 87),
 *     'seagreen1' : (84, 255, 159),
 *     'seagreen2' : (78, 238, 148),             # <<<<<<<<<<<<<<
 *     'seagreen3' : (67, 205, 128),
 *     'seagreen4' : (46, 139, 87),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_seagreen2, __pyx_tuple__441) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":581
 *     'seagreen1' : (84, 255, 159),
 *     'seagreen2' : (78, 238, 148),
 *     'seagreen3' : (67, 205, 128),             # <<<<<<<<<<<<<<
 *     'seagreen4' : (46, 139, 87),
 *     'seashell' : (255, 245, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_seagreen3, __pyx_tuple__442) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":582
 *     'seagreen2' : (78, 238, 148),
 *     'seagreen3' : (67, 205, 128),
 *     'seagreen4' : (46, 139, 87),             # <<<<<<<<<<<<<<
 *     'seashell' : (255, 245, 238),
 *     'seashell1' : (255, 245, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_seagreen4, __pyx_tuple__439) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":583
 *     'seagreen3' : (67, 205, 128),
 *     'seagreen4' : (46, 139, 87),
 *     'seashell' : (255, 245, 238),             # <<<<<<<<<<<<<<
 *     'seashell1' : (255, 245, 238),
 *     'seashell2' : (238, 229, 222),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_seashell, __pyx_tuple__443) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":584
 *     'seagreen4' : (46, 139, 87),
 *     'seashell' : (255, 245, 238),
 *     'seashell1' : (255, 245, 238),             # <<<<<<<<<<<<<<
 *     'seashell2' : (238, 229, 222),
 *     'seashell3' : (205, 197, 191),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_seashell1, __pyx_tuple__443) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":585
 *     'seashell' : (255, 245, 238),
 *     'seashell1' : (255, 245, 238),
 *     'seashell2' : (238, 229, 222),             # <<<<<<<<<<<<<<
 *     'seashell3' : (205, 197, 191),
 *     'seashell4' : (139, 134, 130),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_seashell2, __pyx_tuple__444) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":586
 *     'seashell1' : (255, 245, 238),
 *     'seashell2' : (238, 229, 222),
 *     'seashell3' : (205, 197, 191),             # <<<<<<<<<<<<<<
 *     'seashell4' : (139, 134, 130),
 *     'sienna' : (160, 82, 45),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_seashell3, __pyx_tuple__445) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":587
 *     'seashell2' : (238, 229, 222),
 *     'seashell3' : (205, 197, 191),
 *     'seashell4' : (139, 134, 130),             # <<<<<<<<<<<<<<
 *     'sienna' : (160, 82, 45),
 *     'sienna1' : (255, 130, 71),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_seashell4, __pyx_tuple__446) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":588
 *     'seashell3' : (205, 197, 191),
 *     'seashell4' : (139, 134, 130),
 *     'sienna' : (160, 82, 45),             # <<<<<<<<<<<<<<
 *     'sienna1' : (255, 130, 71),
 *     'sienna2' : (238, 121, 66),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sienna, __pyx_tuple__447) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":589
 *     'seashell4' : (139, 134, 130),
 *     'sienna' : (160, 82, 45),
 *     'sienna1' : (255, 130, 71),             # <<<<<<<<<<<<<<
 *     'sienna2' : (238, 121, 66),
 *     'sienna3' : (205, 104, 57),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sienna1, __pyx_tuple__448) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":590
 *     'sienna' : (160, 82, 45),
 *     'sienna1' : (255, 130, 71),
 *     'sienna2' : (238, 121, 66),             # <<<<<<<<<<<<<<
 *     'sienna3' : (205, 104, 57),
 *     'sienna4' : (139, 71, 38),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sienna2, __pyx_tuple__449) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":591
 *     'sienna1' : (255, 130, 71),
 *     'sienna2' : (238, 121, 66),
 *     'sienna3' : (205, 104, 57),             # <<<<<<<<<<<<<<
 *     'sienna4' : (139, 71, 38),
 *     'skyblue' : (135, 206, 235),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sienna3, __pyx_tuple__450) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":592
 *     'sienna2' : (238, 121, 66),
 *     'sienna3' : (205, 104, 57),
 *     'sienna4' : (139, 71, 38),             # <<<<<<<<<<<<<<
 *     'skyblue' : (135, 206, 235),
 *     'skyblue1' : (135, 206, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_sienna4, __pyx_tuple__451) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":593
 *     'sienna3' : (205, 104, 57),
 *     'sienna4' : (139, 71, 38),
 *     'skyblue' : (135, 206, 235),             # <<<<<<<<<<<<<<
 *     'skyblue1' : (135, 206, 255),
 *     'skyblue2' : (126, 192, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_skyblue, __pyx_tuple__452) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":594
 *     'sienna4' : (139, 71, 38),
 *     'skyblue' : (135, 206, 235),
 *     'skyblue1' : (135, 206, 255),             # <<<<<<<<<<<<<<
 *     'skyblue2' : (126, 192, 238),
 *     'skyblue3' : (108, 166, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_skyblue1, __pyx_tuple__453) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":595
 *     'skyblue' : (135, 206, 235),
 *     'skyblue1' : (135, 206, 255),
 *     'skyblue2' : (126, 192, 238),             # <<<<<<<<<<<<<<
 *     'skyblue3' : (108, 166, 205),
 *     'skyblue4' : (74, 112, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_skyblue2, __pyx_tuple__454) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":596
 *     'skyblue1' : (135, 206, 255),
 *     'skyblue2' : (126, 192, 238),
 *     'skyblue3' : (108, 166, 205),             # <<<<<<<<<<<<<<
 *     'skyblue4' : (74, 112, 139),
 *     'slateblue' : (106, 90, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_skyblue3, __pyx_tuple__455) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":597
 *     'skyblue2' : (126, 192, 238),
 *     'skyblue3' : (108, 166, 205),
 *     'skyblue4' : (74, 112, 139),             # <<<<<<<<<<<<<<
 *     'slateblue' : (106, 90, 205),
 *     'slateblue1' : (131, 111, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_skyblue4, __pyx_tuple__456) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":598
 *     'skyblue3' : (108, 166, 205),
 *     'skyblue4' : (74, 112, 139),
 *     'slateblue' : (106, 90, 205),             # <<<<<<<<<<<<<<
 *     'slateblue1' : (131, 111, 255),
 *     'slateblue2' : (122, 103, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slateblue, __pyx_tuple__457) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":599
 *     'skyblue4' : (74, 112, 139),
 *     'slateblue' : (106, 90, 205),
 *     'slateblue1' : (131, 111, 255),             # <<<<<<<<<<<<<<
 *     'slateblue2' : (122, 103, 238),
 *     'slateblue3' : (105, 89, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slateblue1, __pyx_tuple__458) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":600
 *     'slateblue' : (106, 90, 205),
 *     'slateblue1' : (131, 111, 255),
 *     'slateblue2' : (122, 103, 238),             # <<<<<<<<<<<<<<
 *     'slateblue3' : (105, 89, 205),
 *     'slateblue4' : (71, 60, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slateblue2, __pyx_tuple__459) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":601
 *     'slateblue1' : (131, 111, 255),
 *     'slateblue2' : (122, 103, 238),
 *     'slateblue3' : (105, 89, 205),             # <<<<<<<<<<<<<<
 *     'slateblue4' : (71, 60, 139),
 *     'slategray' : (112, 128, 144),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slateblue3, __pyx_tuple__460) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":602
 *     'slateblue2' : (122, 103, 238),
 *     'slateblue3' : (105, 89, 205),
 *     'slateblue4' : (71, 60, 139),             # <<<<<<<<<<<<<<
 *     'slategray' : (112, 128, 144),
 *     'slategray1' : (198, 226, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slateblue4, __pyx_tuple__461) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":603
 *     'slateblue3' : (105, 89, 205),
 *     'slateblue4' : (71, 60, 139),
 *     'slategray' : (112, 128, 144),             # <<<<<<<<<<<<<<
 *     'slategray1' : (198, 226, 255),
 *     'slategray2' : (185, 211, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slategray, __pyx_tuple__462) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":604
 *     'slateblue4' : (71, 60, 139),
 *     'slategray' : (112, 128, 144),
 *     'slategray1' : (198, 226, 255),             # <<<<<<<<<<<<<<
 *     'slategray2' : (185, 211, 238),
 *     'slategray3' : (159, 182, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slategray1, __pyx_tuple__463) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":605
 *     'slategray' : (112, 128, 144),
 *     'slategray1' : (198, 226, 255),
 *     'slategray2' : (185, 211, 238),             # <<<<<<<<<<<<<<
 *     'slategray3' : (159, 182, 205),
 *     'slategray4' : (108, 123, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slategray2, __pyx_tuple__464) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":606
 *     'slategray1' : (198, 226, 255),
 *     'slategray2' : (185, 211, 238),
 *     'slategray3' : (159, 182, 205),             # <<<<<<<<<<<<<<
 *     'slategray4' : (108, 123, 139),
 *     'slategrey' : (112, 128, 144),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slategray3, __pyx_tuple__465) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":607
 *     'slategray2' : (185, 211, 238),
 *     'slategray3' : (159, 182, 205),
 *     'slategray4' : (108, 123, 139),             # <<<<<<<<<<<<<<
 *     'slategrey' : (112, 128, 144),
 *     'snow' : (255, 250, 250),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slategray4, __pyx_tuple__466) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":608
 *     'slategray3' : (159, 182, 205),
 *     'slategray4' : (108, 123, 139),
 *     'slategrey' : (112, 128, 144),             # <<<<<<<<<<<<<<
 *     'snow' : (255, 250, 250),
 *     'snow1' : (255, 250, 250),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_slategrey, __pyx_tuple__462) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":609
 *     'slategray4' : (108, 123, 139),
 *     'slategrey' : (112, 128, 144),
 *     'snow' : (255, 250, 250),             # <<<<<<<<<<<<<<
 *     'snow1' : (255, 250, 250),
 *     'snow2' : (238, 233, 233),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_snow, __pyx_tuple__467) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":610
 *     'slategrey' : (112, 128, 144),
 *     'snow' : (255, 250, 250),
 *     'snow1' : (255, 250, 250),             # <<<<<<<<<<<<<<
 *     'snow2' : (238, 233, 233),
 *     'snow3' : (205, 201, 201),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_snow1, __pyx_tuple__467) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":611
 *     'snow' : (255, 250, 250),
 *     'snow1' : (255, 250, 250),
 *     'snow2' : (238, 233, 233),             # <<<<<<<<<<<<<<
 *     'snow3' : (205, 201, 201),
 *     'snow4' : (139, 137, 137),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_snow2, __pyx_tuple__468) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":612
 *     'snow1' : (255, 250, 250),
 *     'snow2' : (238, 233, 233),
 *     'snow3' : (205, 201, 201),             # <<<<<<<<<<<<<<
 *     'snow4' : (139, 137, 137),
 *     'springgreen' : (0, 255, 127),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_snow3, __pyx_tuple__469) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":613
 *     'snow2' : (238, 233, 233),
 *     'snow3' : (205, 201, 201),
 *     'snow4' : (139, 137, 137),             # <<<<<<<<<<<<<<
 *     'springgreen' : (0, 255, 127),
 *     'springgreen1' : (0, 255, 127),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_snow4, __pyx_tuple__470) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":614
 *     'snow3' : (205, 201, 201),
 *     'snow4' : (139, 137, 137),
 *     'springgreen' : (0, 255, 127),             # <<<<<<<<<<<<<<
 *     'springgreen1' : (0, 255, 127),
 *     'springgreen2' : (0, 238, 118),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_springgreen, __pyx_tuple__471) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":615
 *     'snow4' : (139, 137, 137),
 *     'springgreen' : (0, 255, 127),
 *     'springgreen1' : (0, 255, 127),             # <<<<<<<<<<<<<<
 *     'springgreen2' : (0, 238, 118),
 *     'springgreen3' : (0, 205, 102),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_springgreen1, __pyx_tuple__471) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":616
 *     'springgreen' : (0, 255, 127),
 *     'springgreen1' : (0, 255, 127),
 *     'springgreen2' : (0, 238, 118),             # <<<<<<<<<<<<<<
 *     'springgreen3' : (0, 205, 102),
 *     'springgreen4' : (0, 139, 69),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_springgreen2, __pyx_tuple__472) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":617
 *     'springgreen1' : (0, 255, 127),
 *     'springgreen2' : (0, 238, 118),
 *     'springgreen3' : (0, 205, 102),             # <<<<<<<<<<<<<<
 *     'springgreen4' : (0, 139, 69),
 *     'steelblue' : (70, 130, 180),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_springgreen3, __pyx_tuple__473) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":618
 *     'springgreen2' : (0, 238, 118),
 *     'springgreen3' : (0, 205, 102),
 *     'springgreen4' : (0, 139, 69),             # <<<<<<<<<<<<<<
 *     'steelblue' : (70, 130, 180),
 *     'steelblue1' : (99, 184, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_springgreen4, __pyx_tuple__474) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":619
 *     'springgreen3' : (0, 205, 102),
 *     'springgreen4' : (0, 139, 69),
 *     'steelblue' : (70, 130, 180),             # <<<<<<<<<<<<<<
 *     'steelblue1' : (99, 184, 255),
 *     'steelblue2' : (92, 172, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_steelblue, __pyx_tuple__475) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":620
 *     'springgreen4' : (0, 139, 69),
 *     'steelblue' : (70, 130, 180),
 *     'steelblue1' : (99, 184, 255),             # <<<<<<<<<<<<<<
 *     'steelblue2' : (92, 172, 238),
 *     'steelblue3' : (79, 148, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_steelblue1, __pyx_tuple__476) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":621
 *     'steelblue' : (70, 130, 180),
 *     'steelblue1' : (99, 184, 255),
 *     'steelblue2' : (92, 172, 238),             # <<<<<<<<<<<<<<
 *     'steelblue3' : (79, 148, 205),
 *     'steelblue4' : (54, 100, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_steelblue2, __pyx_tuple__477) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":622
 *     'steelblue1' : (99, 184, 255),
 *     'steelblue2' : (92, 172, 238),
 *     'steelblue3' : (79, 148, 205),             # <<<<<<<<<<<<<<
 *     'steelblue4' : (54, 100, 139),
 *     'tan' : (210, 180, 140),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_steelblue3, __pyx_tuple__478) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":623
 *     'steelblue2' : (92, 172, 238),
 *     'steelblue3' : (79, 148, 205),
 *     'steelblue4' : (54, 100, 139),             # <<<<<<<<<<<<<<
 *     'tan' : (210, 180, 140),
 *     'tan1' : (255, 165, 79),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_steelblue4, __pyx_tuple__479) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":624
 *     'steelblue3' : (79, 148, 205),
 *     'steelblue4' : (54, 100, 139),
 *     'tan' : (210, 180, 140),             # <<<<<<<<<<<<<<
 *     'tan1' : (255, 165, 79),
 *     'tan2' : (238, 154, 73),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tan, __pyx_tuple__480) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":625
 *     'steelblue4' : (54, 100, 139),
 *     'tan' : (210, 180, 140),
 *     'tan1' : (255, 165, 79),             # <<<<<<<<<<<<<<
 *     'tan2' : (238, 154, 73),
 *     'tan3' : (205, 133, 63),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tan1, __pyx_tuple__481) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":626
 *     'tan' : (210, 180, 140),
 *     'tan1' : (255, 165, 79),
 *     'tan2' : (238, 154, 73),             # <<<<<<<<<<<<<<
 *     'tan3' : (205, 133, 63),
 *     'tan4' : (139, 90, 43),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tan2, __pyx_tuple__482) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":627
 *     'tan1' : (255, 165, 79),
 *     'tan2' : (238, 154, 73),
 *     'tan3' : (205, 133, 63),             # <<<<<<<<<<<<<<
 *     'tan4' : (139, 90, 43),
 *     'thistle' : (216, 191, 216),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tan3, __pyx_tuple__403) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":628
 *     'tan2' : (238, 154, 73),
 *     'tan3' : (205, 133, 63),
 *     'tan4' : (139, 90, 43),             # <<<<<<<<<<<<<<
 *     'thistle' : (216, 191, 216),
 *     'thistle1' : (255, 225, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tan4, __pyx_tuple__483) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":629
 *     'tan3' : (205, 133, 63),
 *     'tan4' : (139, 90, 43),
 *     'thistle' : (216, 191, 216),             # <<<<<<<<<<<<<<
 *     'thistle1' : (255, 225, 255),
 *     'thistle2' : (238, 210, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_thistle, __pyx_tuple__484) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":630
 *     'tan4' : (139, 90, 43),
 *     'thistle' : (216, 191, 216),
 *     'thistle1' : (255, 225, 255),             # <<<<<<<<<<<<<<
 *     'thistle2' : (238, 210, 238),
 *     'thistle3' : (205, 181, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_thistle1, __pyx_tuple__485) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":631
 *     'thistle' : (216, 191, 216),
 *     'thistle1' : (255, 225, 255),
 *     'thistle2' : (238, 210, 238),             # <<<<<<<<<<<<<<
 *     'thistle3' : (205, 181, 205),
 *     'thistle4' : (139, 123, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_thistle2, __pyx_tuple__486) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":632
 *     'thistle1' : (255, 225, 255),
 *     'thistle2' : (238, 210, 238),
 *     'thistle3' : (205, 181, 205),             # <<<<<<<<<<<<<<
 *     'thistle4' : (139, 123, 139),
 *     'tomato' : (255, 99, 71),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_thistle3, __pyx_tuple__487) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":633
 *     'thistle2' : (238, 210, 238),
 *     'thistle3' : (205, 181, 205),
 *     'thistle4' : (139, 123, 139),             # <<<<<<<<<<<<<<
 *     'tomato' : (255, 99, 71),
 *     'tomato1' : (255, 99, 71),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_thistle4, __pyx_tuple__488) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":634
 *     'thistle3' : (205, 181, 205),
 *     'thistle4' : (139, 123, 139),
 *     'tomato' : (255, 99, 71),             # <<<<<<<<<<<<<<
 *     'tomato1' : (255, 99, 71),
 *     'tomato2' : (238, 92, 66),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tomato, __pyx_tuple__489) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":635
 *     'thistle4' : (139, 123, 139),
 *     'tomato' : (255, 99, 71),
 *     'tomato1' : (255, 99, 71),             # <<<<<<<<<<<<<<
 *     'tomato2' : (238, 92, 66),
 *     'tomato3' : (205, 79, 57),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tomato1, __pyx_tuple__489) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":636
 *     'tomato' : (255, 99, 71),
 *     'tomato1' : (255, 99, 71),
 *     'tomato2' : (238, 92, 66),             # <<<<<<<<<<<<<<
 *     'tomato3' : (205, 79, 57),
 *     'tomato4' : (139, 54, 38),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tomato2, __pyx_tuple__490) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":637
 *     'tomato1' : (255, 99, 71),
 *     'tomato2' : (238, 92, 66),
 *     'tomato3' : (205, 79, 57),             # <<<<<<<<<<<<<<
 *     'tomato4' : (139, 54, 38),
 *     'turquoise' : (64, 224, 208),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tomato3, __pyx_tuple__491) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":638
 *     'tomato2' : (238, 92, 66),
 *     'tomato3' : (205, 79, 57),
 *     'tomato4' : (139, 54, 38),             # <<<<<<<<<<<<<<
 *     'turquoise' : (64, 224, 208),
 *     'turquoise1' : (0, 245, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_tomato4, __pyx_tuple__492) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":639
 *     'tomato3' : (205, 79, 57),
 *     'tomato4' : (139, 54, 38),
 *     'turquoise' : (64, 224, 208),             # <<<<<<<<<<<<<<
 *     'turquoise1' : (0, 245, 255),
 *     'turquoise2' : (0, 229, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_turquoise, __pyx_tuple__493) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":640
 *     'tomato4' : (139, 54, 38),
 *     'turquoise' : (64, 224, 208),
 *     'turquoise1' : (0, 245, 255),             # <<<<<<<<<<<<<<
 *     'turquoise2' : (0, 229, 238),
 *     'turquoise3' : (0, 197, 205),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_turquoise1, __pyx_tuple__494) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":641
 *     'turquoise' : (64, 224, 208),
 *     'turquoise1' : (0, 245, 255),
 *     'turquoise2' : (0, 229, 238),             # <<<<<<<<<<<<<<
 *     'turquoise3' : (0, 197, 205),
 *     'turquoise4' : (0, 134, 139),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_turquoise2, __pyx_tuple__495) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":642
 *     'turquoise1' : (0, 245, 255),
 *     'turquoise2' : (0, 229, 238),
 *     'turquoise3' : (0, 197, 205),             # <<<<<<<<<<<<<<
 *     'turquoise4' : (0, 134, 139),
 *     'violet' : (238, 130, 238),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_turquoise3, __pyx_tuple__496) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":643
 *     'turquoise2' : (0, 229, 238),
 *     'turquoise3' : (0, 197, 205),
 *     'turquoise4' : (0, 134, 139),             # <<<<<<<<<<<<<<
 *     'violet' : (238, 130, 238),
 *     'violetred' : (208, 32, 144),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_turquoise4, __pyx_tuple__497) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":644
 *     'turquoise3' : (0, 197, 205),
 *     'turquoise4' : (0, 134, 139),
 *     'violet' : (238, 130, 238),             # <<<<<<<<<<<<<<
 *     'violetred' : (208, 32, 144),
 *     'violetred1' : (255, 62, 150),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_violet, __pyx_tuple__498) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":645
 *     'turquoise4' : (0, 134, 139),
 *     'violet' : (238, 130, 238),
 *     'violetred' : (208, 32, 144),             # <<<<<<<<<<<<<<
 *     'violetred1' : (255, 62, 150),
 *     'violetred2' : (238, 58, 140),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_violetred, __pyx_tuple__499) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":646
 *     'violet' : (238, 130, 238),
 *     'violetred' : (208, 32, 144),
 *     'violetred1' : (255, 62, 150),             # <<<<<<<<<<<<<<
 *     'violetred2' : (238, 58, 140),
 *     'violetred3' : (205, 50, 120),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_violetred1, __pyx_tuple__500) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":647
 *     'violetred' : (208, 32, 144),
 *     'violetred1' : (255, 62, 150),
 *     'violetred2' : (238, 58, 140),             # <<<<<<<<<<<<<<
 *     'violetred3' : (205, 50, 120),
 *     'violetred4' : (139, 34, 82),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_violetred2, __pyx_tuple__501) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":648
 *     'violetred1' : (255, 62, 150),
 *     'violetred2' : (238, 58, 140),
 *     'violetred3' : (205, 50, 120),             # <<<<<<<<<<<<<<
 *     'violetred4' : (139, 34, 82),
 *     'wheat' : (245, 222, 179),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_violetred3, __pyx_tuple__502) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":649
 *     'violetred2' : (238, 58, 140),
 *     'violetred3' : (205, 50, 120),
 *     'violetred4' : (139, 34, 82),             # <<<<<<<<<<<<<<
 *     'wheat' : (245, 222, 179),
 *     'wheat1' : (255, 231, 186),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_violetred4, __pyx_tuple__503) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":650
 *     'violetred3' : (205, 50, 120),
 *     'violetred4' : (139, 34, 82),
 *     'wheat' : (245, 222, 179),             # <<<<<<<<<<<<<<
 *     'wheat1' : (255, 231, 186),
 *     'wheat2' : (238, 216, 174),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_wheat, __pyx_tuple__504) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":651
 *     'violetred4' : (139, 34, 82),
 *     'wheat' : (245, 222, 179),
 *     'wheat1' : (255, 231, 186),             # <<<<<<<<<<<<<<
 *     'wheat2' : (238, 216, 174),
 *     'wheat3' : (205, 186, 150),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_wheat1, __pyx_tuple__505) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":652
 *     'wheat' : (245, 222, 179),
 *     'wheat1' : (255, 231, 186),
 *     'wheat2' : (238, 216, 174),             # <<<<<<<<<<<<<<
 *     'wheat3' : (205, 186, 150),
 *     'wheat4' : (139, 126, 102),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_wheat2, __pyx_tuple__506) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":653
 *     'wheat1' : (255, 231, 186),
 *     'wheat2' : (238, 216, 174),
 *     'wheat3' : (205, 186, 150),             # <<<<<<<<<<<<<<
 *     'wheat4' : (139, 126, 102),
 *     'white' : (255, 255, 255),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_wheat3, __pyx_tuple__507) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":654
 *     'wheat2' : (238, 216, 174),
 *     'wheat3' : (205, 186, 150),
 *     'wheat4' : (139, 126, 102),             # <<<<<<<<<<<<<<
 *     'white' : (255, 255, 255),
 *     'whitesmoke' : (245, 245, 245),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_wheat4, __pyx_tuple__508) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":655
 *     'wheat3' : (205, 186, 150),
 *     'wheat4' : (139, 126, 102),
 *     'white' : (255, 255, 255),             # <<<<<<<<<<<<<<
 *     'whitesmoke' : (245, 245, 245),
 *     'yellow' : (255, 255, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_white, __pyx_tuple__148) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":656
 *     'wheat4' : (139, 126, 102),
 *     'white' : (255, 255, 255),
 *     'whitesmoke' : (245, 245, 245),             # <<<<<<<<<<<<<<
 *     'yellow' : (255, 255, 0),
 *     'yellow1' : (255, 255, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_whitesmoke, __pyx_tuple__241) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":657
 *     'white' : (255, 255, 255),
 *     'whitesmoke' : (245, 245, 245),
 *     'yellow' : (255, 255, 0),             # <<<<<<<<<<<<<<
 *     'yellow1' : (255, 255, 0),
 *     'yellow2' : (238, 238, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_yellow, __pyx_tuple__509) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":658
 *     'whitesmoke' : (245, 245, 245),
 *     'yellow' : (255, 255, 0),
 *     'yellow1' : (255, 255, 0),             # <<<<<<<<<<<<<<
 *     'yellow2' : (238, 238, 0),
 *     'yellow3' : (205, 205, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_yellow1, __pyx_tuple__509) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":659
 *     'yellow' : (255, 255, 0),
 *     'yellow1' : (255, 255, 0),
 *     'yellow2' : (238, 238, 0),             # <<<<<<<<<<<<<<
 *     'yellow3' : (205, 205, 0),
 *     'yellow4' : (139, 139, 0),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_yellow2, __pyx_tuple__510) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":660
 *     'yellow1' : (255, 255, 0),
 *     'yellow2' : (238, 238, 0),
 *     'yellow3' : (205, 205, 0),             # <<<<<<<<<<<<<<
 *     'yellow4' : (139, 139, 0),
 *     'yellowgreen' : (154, 205, 50),
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_yellow3, __pyx_tuple__511) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":661
 *     'yellow2' : (238, 238, 0),
 *     'yellow3' : (205, 205, 0),
 *     'yellow4' : (139, 139, 0),             # <<<<<<<<<<<<<<
 *     'yellowgreen' : (154, 205, 50),
 * }
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_yellow4, __pyx_tuple__512) < 0) __PYX_ERR(2, 5, __pyx_L1_error)

  /* "include/color_dict.pxi":662
 *     'yellow3' : (205, 205, 0),
 *     'yellow4' : (139, 139, 0),
 *     'yellowgreen' : (154, 205, 50),             # <<<<<<<<<<<<<<
 * }
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_u_yellowgreen, __pyx_tuple__368) < 0) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_XGOTREF(__pyx_v_11pygame_sdl2_5color_colors);
  __Pyx_DECREF_SET(__pyx_v_11pygame_sdl2_5color_colors, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pygame_sdl2/color.pyx":1
 * # Copyright 2014 Tom Rothamel <tom@rothamel.us>             # <<<<<<<<<<<<<<
 * # Copyright 2014 Patrick Dawson <pat@dw.is>
 * #
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pygame_sdl2.color", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pygame_sdl2.color");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
#include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractCObj(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        const long a = intval;
        long x;
        long b = PyInt_AS_LONG(op2);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op2))) {
        const long a = intval;
        long b, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG lla = intval;
        PY_LONG_LONG llb, llx;
#endif
        const digit* digits = ((PyLongObject*)op2)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op2);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            b = likely(size) ? digits[0] : 0;
            if (size == -1) b = -b;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        b = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        llb = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        b = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        llb = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        b = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        llb = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        b = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        llb = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        b = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        llb = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        b = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        llb = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op2)) {
        const long a = intval;
        double b = PyFloat_AS_DOUBLE(op2);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* None */
static CYTHON_INLINE double __Pyx_mod_double(double a, double b) {
    double r = fmod(a, b);
    r += ((r != 0) & ((r < 0) ^ (b < 0))) * b;
    return r;
}

/* PyFloatBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_RemainderObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double b = floatval;
    double a, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op1))) {
        a = PyFloat_AS_DOUBLE(op1);
        
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        a = (double) PyInt_AS_LONG(op1);
        
    } else
    #endif
    if (likely(PyLong_CheckExact(op1))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        switch (size) {
            case  0: a = 0.0; break;
            case -1: a = -(double) digits[0]; break;
            case  1: a = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    a = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            a = PyLong_AsDouble(op1);
            if (unlikely(a == -1.0 && PyErr_Occurred())) return NULL;
            
        }
    } else {
        return (inplace ? PyNumber_InPlaceRemainder : PyNumber_Remainder)(op1, op2);
    }
        
        PyFPE_START_PROTECT("remainder", return NULL)
        result = fmod(a, b);
        if (result)
            result += ((result < 0) ^ (b < 0)) * b;
        else
            result = copysign(0.0, b);
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* PyFloatBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double b = floatval;
    double a, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op1))) {
        a = PyFloat_AS_DOUBLE(op1);
        
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        a = (double) PyInt_AS_LONG(op1);
        
    } else
    #endif
    if (likely(PyLong_CheckExact(op1))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        switch (size) {
            case  0: a = 0.0; break;
            case -1: a = -(double) digits[0]; break;
            case  1: a = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    a = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            a = PyLong_AsDouble(op1);
            if (unlikely(a == -1.0 && PyErr_Occurred())) return NULL;
            
        }
    } else {
        return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
    }
        
        PyFPE_START_PROTECT("divide", return NULL)
        result = a / b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* None */
    static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* PyObject_GenericGetAttrNoDict */
    #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
    #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
    static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* Import */
    static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* CLineInTraceback */
    #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
    static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
    #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
    #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8_t(uint8_t value) {
    const uint8_t neg_one = (uint8_t) ((uint8_t) 0 - (uint8_t) 1), const_zero = (uint8_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint8_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint8_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint8_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint8_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint8_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE uint8_t __Pyx_PyInt_As_uint8_t(PyObject *x) {
    const uint8_t neg_one = (uint8_t) ((uint8_t) 0 - (uint8_t) 1), const_zero = (uint8_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint8_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint8_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint8_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint8_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint8_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 2 * PyLong_SHIFT) {
                            return (uint8_t) (((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 3 * PyLong_SHIFT) {
                            return (uint8_t) (((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 4 * PyLong_SHIFT) {
                            return (uint8_t) (((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint8_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint8_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint8_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint8_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint8_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint8_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8_t) ((((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8_t) ((((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8_t) ((((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint8_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint8_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint8_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint8_t) -1;
        }
    } else {
        uint8_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint8_t) -1;
        val = __Pyx_PyInt_As_uint8_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint8_t");
    return (uint8_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint8_t");
    return (uint8_t) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *x) {
    const uint32_t neg_one = (uint32_t) ((uint32_t) 0 - (uint32_t) 1), const_zero = (uint32_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT) {
                            return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT) {
                            return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT) {
                            return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint32_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint32_t) -1;
        }
    } else {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyInt_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
    #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
    static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionExport */
    static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* InitStrings */
    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
