/* Generated by Cython 0.29.20 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_20"
#define CYTHON_HEX_VERSION 0x001D14F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pygame_sdl2__locals
#define __PYX_HAVE_API__pygame_sdl2__locals
/* Early includes */
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <stddef.h>
#include "SDL.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "src/pygame_sdl2/locals.pyx",
  "include/event_list.pxi",
  "include/keycode_list.pxi",
  "include/glattr.pxi",
  "include/controller.pxi",
};

/*--- Type declarations ---*/

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* Globals.proto */
static PyObject* __Pyx_Globals(void);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_EventType(SDL_EventType value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_GLattr(SDL_GLattr value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_GLprofile(SDL_GLprofile value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_WindowFlags(SDL_WindowFlags value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_Keymod(SDL_Keymod value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_GameControllerButton(SDL_GameControllerButton value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_GameControllerAxis(SDL_GameControllerAxis value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_PowerState(SDL_PowerState value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libc.stdint' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'libc.stddef' */

/* Module declarations from 'sdl2' */

/* Module declarations from 'pygame_sdl2.locals' */
#define __Pyx_MODULE_NAME "pygame_sdl2.locals"
extern int __pyx_module_is_main_pygame_sdl2__locals;
int __pyx_module_is_main_pygame_sdl2__locals = 0;

/* Implementation of 'pygame_sdl2.locals' */
static const char __pyx_k_K_0[] = "K_0";
static const char __pyx_k_K_1[] = "K_1";
static const char __pyx_k_K_2[] = "K_2";
static const char __pyx_k_K_3[] = "K_3";
static const char __pyx_k_K_4[] = "K_4";
static const char __pyx_k_K_5[] = "K_5";
static const char __pyx_k_K_6[] = "K_6";
static const char __pyx_k_K_7[] = "K_7";
static const char __pyx_k_K_8[] = "K_8";
static const char __pyx_k_K_9[] = "K_9";
static const char __pyx_k_K_a[] = "K_a";
static const char __pyx_k_K_b[] = "K_b";
static const char __pyx_k_K_c[] = "K_c";
static const char __pyx_k_K_d[] = "K_d";
static const char __pyx_k_K_e[] = "K_e";
static const char __pyx_k_K_f[] = "K_f";
static const char __pyx_k_K_g[] = "K_g";
static const char __pyx_k_K_h[] = "K_h";
static const char __pyx_k_K_i[] = "K_i";
static const char __pyx_k_K_j[] = "K_j";
static const char __pyx_k_K_k[] = "K_k";
static const char __pyx_k_K_l[] = "K_l";
static const char __pyx_k_K_m[] = "K_m";
static const char __pyx_k_K_n[] = "K_n";
static const char __pyx_k_K_o[] = "K_o";
static const char __pyx_k_K_p[] = "K_p";
static const char __pyx_k_K_q[] = "K_q";
static const char __pyx_k_K_r[] = "K_r";
static const char __pyx_k_K_s[] = "K_s";
static const char __pyx_k_K_t[] = "K_t";
static const char __pyx_k_K_u[] = "K_u";
static const char __pyx_k_K_v[] = "K_v";
static const char __pyx_k_K_w[] = "K_w";
static const char __pyx_k_K_x[] = "K_x";
static const char __pyx_k_K_y[] = "K_y";
static const char __pyx_k_K_z[] = "K_z";
static const char __pyx_k_K_AT[] = "K_AT";
static const char __pyx_k_K_F1[] = "K_F1";
static const char __pyx_k_K_F2[] = "K_F2";
static const char __pyx_k_K_F3[] = "K_F3";
static const char __pyx_k_K_F4[] = "K_F4";
static const char __pyx_k_K_F5[] = "K_F5";
static const char __pyx_k_K_F6[] = "K_F6";
static const char __pyx_k_K_F7[] = "K_F7";
static const char __pyx_k_K_F8[] = "K_F8";
static const char __pyx_k_K_F9[] = "K_F9";
static const char __pyx_k_K_UP[] = "K_UP";
static const char __pyx_k_QUIT[] = "QUIT";
static const char __pyx_k_Rect[] = "Rect";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_Color[] = "Color";
static const char __pyx_k_KEYUP[] = "KEYUP";
static const char __pyx_k_K_CUT[] = "K_CUT";
static const char __pyx_k_K_END[] = "K_END";
static const char __pyx_k_K_F10[] = "K_F10";
static const char __pyx_k_K_F11[] = "K_F11";
static const char __pyx_k_K_F12[] = "K_F12";
static const char __pyx_k_K_F13[] = "K_F13";
static const char __pyx_k_K_F14[] = "K_F14";
static const char __pyx_k_K_F15[] = "K_F15";
static const char __pyx_k_K_F16[] = "K_F16";
static const char __pyx_k_K_F17[] = "K_F17";
static const char __pyx_k_K_F18[] = "K_F18";
static const char __pyx_k_K_F19[] = "K_F19";
static const char __pyx_k_K_F20[] = "K_F20";
static const char __pyx_k_K_F21[] = "K_F21";
static const char __pyx_k_K_F22[] = "K_F22";
static const char __pyx_k_K_F23[] = "K_F23";
static const char __pyx_k_K_F24[] = "K_F24";
static const char __pyx_k_K_KP0[] = "K_KP0";
static const char __pyx_k_K_KP1[] = "K_KP1";
static const char __pyx_k_K_KP2[] = "K_KP2";
static const char __pyx_k_K_KP3[] = "K_KP3";
static const char __pyx_k_K_KP4[] = "K_KP4";
static const char __pyx_k_K_KP5[] = "K_KP5";
static const char __pyx_k_K_KP6[] = "K_KP6";
static const char __pyx_k_K_KP7[] = "K_KP7";
static const char __pyx_k_K_KP8[] = "K_KP8";
static const char __pyx_k_K_KP9[] = "K_KP9";
static const char __pyx_k_K_OUT[] = "K_OUT";
static const char __pyx_k_K_TAB[] = "K_TAB";
static const char __pyx_k_K_WWW[] = "K_WWW";
static const char __pyx_k_K_APP1[] = "K_APP1";
static const char __pyx_k_K_APP2[] = "K_APP2";
static const char __pyx_k_K_COPY[] = "K_COPY";
static const char __pyx_k_K_DOWN[] = "K_DOWN";
static const char __pyx_k_K_EURO[] = "K_EURO";
static const char __pyx_k_K_FIND[] = "K_FIND";
static const char __pyx_k_K_HASH[] = "K_HASH";
static const char __pyx_k_K_HELP[] = "K_HELP";
static const char __pyx_k_K_HOME[] = "K_HOME";
static const char __pyx_k_K_KP_0[] = "K_KP_0";
static const char __pyx_k_K_KP_1[] = "K_KP_1";
static const char __pyx_k_K_KP_2[] = "K_KP_2";
static const char __pyx_k_K_KP_3[] = "K_KP_3";
static const char __pyx_k_K_KP_4[] = "K_KP_4";
static const char __pyx_k_K_KP_5[] = "K_KP_5";
static const char __pyx_k_K_KP_6[] = "K_KP_6";
static const char __pyx_k_K_KP_7[] = "K_KP_7";
static const char __pyx_k_K_KP_8[] = "K_KP_8";
static const char __pyx_k_K_KP_9[] = "K_KP_9";
static const char __pyx_k_K_KP_A[] = "K_KP_A";
static const char __pyx_k_K_KP_B[] = "K_KP_B";
static const char __pyx_k_K_KP_C[] = "K_KP_C";
static const char __pyx_k_K_KP_D[] = "K_KP_D";
static const char __pyx_k_K_KP_E[] = "K_KP_E";
static const char __pyx_k_K_KP_F[] = "K_KP_F";
static const char __pyx_k_K_LALT[] = "K_LALT";
static const char __pyx_k_K_LAST[] = "K_LAST";
static const char __pyx_k_K_LEFT[] = "K_LEFT";
static const char __pyx_k_K_LESS[] = "K_LESS";
static const char __pyx_k_K_LGUI[] = "K_LGUI";
static const char __pyx_k_K_MAIL[] = "K_MAIL";
static const char __pyx_k_K_MENU[] = "K_MENU";
static const char __pyx_k_K_MODE[] = "K_MODE";
static const char __pyx_k_K_MUTE[] = "K_MUTE";
static const char __pyx_k_K_OPER[] = "K_OPER";
static const char __pyx_k_K_PLUS[] = "K_PLUS";
static const char __pyx_k_K_RALT[] = "K_RALT";
static const char __pyx_k_K_RGUI[] = "K_RGUI";
static const char __pyx_k_K_STOP[] = "K_STOP";
static const char __pyx_k_K_UNDO[] = "K_UNDO";
static const char __pyx_k_OPENGL[] = "OPENGL";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_KEYDOWN[] = "KEYDOWN";
static const char __pyx_k_K_AGAIN[] = "K_AGAIN";
static const char __pyx_k_K_BREAK[] = "K_BREAK";
static const char __pyx_k_K_CARET[] = "K_CARET";
static const char __pyx_k_K_CLEAR[] = "K_CLEAR";
static const char __pyx_k_K_COLON[] = "K_COLON";
static const char __pyx_k_K_COMMA[] = "K_COMMA";
static const char __pyx_k_K_CRSEL[] = "K_CRSEL";
static const char __pyx_k_K_EJECT[] = "K_EJECT";
static const char __pyx_k_K_EXSEL[] = "K_EXSEL";
static const char __pyx_k_K_FIRST[] = "K_FIRST";
static const char __pyx_k_K_KP_00[] = "K_KP_00";
static const char __pyx_k_K_KP_AT[] = "K_KP_AT";
static const char __pyx_k_K_LCTRL[] = "K_LCTRL";
static const char __pyx_k_K_LMETA[] = "K_LMETA";
static const char __pyx_k_K_MINUS[] = "K_MINUS";
static const char __pyx_k_K_PASTE[] = "K_PASTE";
static const char __pyx_k_K_PAUSE[] = "K_PAUSE";
static const char __pyx_k_K_POWER[] = "K_POWER";
static const char __pyx_k_K_PRINT[] = "K_PRINT";
static const char __pyx_k_K_PRIOR[] = "K_PRIOR";
static const char __pyx_k_K_QUOTE[] = "K_QUOTE";
static const char __pyx_k_K_RCTRL[] = "K_RCTRL";
static const char __pyx_k_K_RIGHT[] = "K_RIGHT";
static const char __pyx_k_K_RMETA[] = "K_RMETA";
static const char __pyx_k_K_SLASH[] = "K_SLASH";
static const char __pyx_k_K_SLEEP[] = "K_SLEEP";
static const char __pyx_k_K_SPACE[] = "K_SPACE";
static const char __pyx_k_NOEVENT[] = "NOEVENT";
static const char __pyx_k_NOFRAME[] = "NOFRAME";
static const char __pyx_k_DROPFILE[] = "DROPFILE";
static const char __pyx_k_DROPTEXT[] = "DROPTEXT";
static const char __pyx_k_FINGERUP[] = "FINGERUP";
static const char __pyx_k_KMOD_ALT[] = "KMOD_ALT";
static const char __pyx_k_KMOD_GUI[] = "KMOD_GUI";
static const char __pyx_k_KMOD_NUM[] = "KMOD_NUM";
static const char __pyx_k_K_CANCEL[] = "K_CANCEL";
static const char __pyx_k_K_DELETE[] = "K_DELETE";
static const char __pyx_k_K_DOLLAR[] = "K_DOLLAR";
static const char __pyx_k_K_EQUALS[] = "K_EQUALS";
static const char __pyx_k_K_ESCAPE[] = "K_ESCAPE";
static const char __pyx_k_K_INSERT[] = "K_INSERT";
static const char __pyx_k_K_KP_000[] = "K_KP_000";
static const char __pyx_k_K_KP_TAB[] = "K_KP_TAB";
static const char __pyx_k_K_KP_XOR[] = "K_KP_XOR";
static const char __pyx_k_K_LSHIFT[] = "K_LSHIFT";
static const char __pyx_k_K_LSUPER[] = "K_LSUPER";
static const char __pyx_k_K_PAGEUP[] = "K_PAGEUP";
static const char __pyx_k_K_PERIOD[] = "K_PERIOD";
static const char __pyx_k_K_RETURN[] = "K_RETURN";
static const char __pyx_k_K_RSHIFT[] = "K_RSHIFT";
static const char __pyx_k_K_RSUPER[] = "K_RSUPER";
static const char __pyx_k_K_SELECT[] = "K_SELECT";
static const char __pyx_k_K_SYSREQ[] = "K_SYSREQ";
static const char __pyx_k_RLEACCEL[] = "RLEACCEL";
static const char __pyx_k_SRCALPHA[] = "SRCALPHA";
static const char __pyx_k_BLEND_ADD[] = "BLEND_ADD";
static const char __pyx_k_BLEND_MAX[] = "BLEND_MAX";
static const char __pyx_k_BLEND_MIN[] = "BLEND_MIN";
static const char __pyx_k_BLEND_SUB[] = "BLEND_SUB";
static const char __pyx_k_BYTEORDER[] = "BYTEORDER";
static const char __pyx_k_DOUBLEBUF[] = "DOUBLEBUF";
static const char __pyx_k_DROPBEGIN[] = "DROPBEGIN";
static const char __pyx_k_GL_STEREO[] = "GL_STEREO";
static const char __pyx_k_HWSURFACE[] = "HWSURFACE";
static const char __pyx_k_KMOD_CAPS[] = "KMOD_CAPS";
static const char __pyx_k_KMOD_CTRL[] = "KMOD_CTRL";
static const char __pyx_k_KMOD_LALT[] = "KMOD_LALT";
static const char __pyx_k_KMOD_LGUI[] = "KMOD_LGUI";
static const char __pyx_k_KMOD_META[] = "KMOD_META";
static const char __pyx_k_KMOD_MODE[] = "KMOD_MODE";
static const char __pyx_k_KMOD_NONE[] = "KMOD_NONE";
static const char __pyx_k_KMOD_RALT[] = "KMOD_RALT";
static const char __pyx_k_KMOD_RGUI[] = "KMOD_RGUI";
static const char __pyx_k_K_AC_BACK[] = "K_AC_BACK";
static const char __pyx_k_K_AC_HOME[] = "K_AC_HOME";
static const char __pyx_k_K_AC_STOP[] = "K_AC_STOP";
static const char __pyx_k_K_EXCLAIM[] = "K_EXCLAIM";
static const char __pyx_k_K_EXECUTE[] = "K_EXECUTE";
static const char __pyx_k_K_GREATER[] = "K_GREATER";
static const char __pyx_k_K_KP_HASH[] = "K_KP_HASH";
static const char __pyx_k_K_KP_LESS[] = "K_KP_LESS";
static const char __pyx_k_K_KP_PLUS[] = "K_KP_PLUS";
static const char __pyx_k_K_NUMLOCK[] = "K_NUMLOCK";
static const char __pyx_k_K_PERCENT[] = "K_PERCENT";
static const char __pyx_k_K_RETURN2[] = "K_RETURN2";
static const char __pyx_k_K_UNKNOWN[] = "K_UNKNOWN";
static const char __pyx_k_LASTEVENT[] = "LASTEVENT";
static const char __pyx_k_RESIZABLE[] = "RESIZABLE";
static const char __pyx_k_SWSURFACE[] = "SWSURFACE";
static const char __pyx_k_TEXTINPUT[] = "TEXTINPUT";
static const char __pyx_k_USEREVENT[] = "USEREVENT";
static const char __pyx_k_BIG_ENDIAN[] = "BIG_ENDIAN";
static const char __pyx_k_BLEND_MULT[] = "BLEND_MULT";
static const char __pyx_k_FINGERDOWN[] = "FINGERDOWN";
static const char __pyx_k_FULLSCREEN[] = "FULLSCREEN";
static const char __pyx_k_KMOD_LCTRL[] = "KMOD_LCTRL";
static const char __pyx_k_KMOD_LMETA[] = "KMOD_LMETA";
static const char __pyx_k_KMOD_RCTRL[] = "KMOD_RCTRL";
static const char __pyx_k_KMOD_RMETA[] = "KMOD_RMETA";
static const char __pyx_k_KMOD_SHIFT[] = "KMOD_SHIFT";
static const char __pyx_k_K_ALTERASE[] = "K_ALTERASE";
static const char __pyx_k_K_ASTERISK[] = "K_ASTERISK";
static const char __pyx_k_K_CAPSLOCK[] = "K_CAPSLOCK";
static const char __pyx_k_K_COMPUTER[] = "K_COMPUTER";
static const char __pyx_k_K_KP_CLEAR[] = "K_KP_CLEAR";
static const char __pyx_k_K_KP_COLON[] = "K_KP_COLON";
static const char __pyx_k_K_KP_COMMA[] = "K_KP_COMMA";
static const char __pyx_k_K_KP_ENTER[] = "K_KP_ENTER";
static const char __pyx_k_K_KP_MINUS[] = "K_KP_MINUS";
static const char __pyx_k_K_KP_OCTAL[] = "K_KP_OCTAL";
static const char __pyx_k_K_KP_POWER[] = "K_KP_POWER";
static const char __pyx_k_K_KP_SPACE[] = "K_KP_SPACE";
static const char __pyx_k_K_PAGEDOWN[] = "K_PAGEDOWN";
static const char __pyx_k_K_QUESTION[] = "K_QUESTION";
static const char __pyx_k_K_QUOTEDBL[] = "K_QUOTEDBL";
static const char __pyx_k_K_VOLUMEUP[] = "K_VOLUMEUP";
static const char __pyx_k_LIL_ENDIAN[] = "LIL_ENDIAN";
static const char __pyx_k_MOUSEWHEEL[] = "MOUSEWHEEL";
static const char __pyx_k_SCRAP_TEXT[] = "SCRAP_TEXT";
static const char __pyx_k_SYSWMEVENT[] = "SYSWMEVENT";
static const char __pyx_k_text_plain[] = "text/plain";
static const char __pyx_k_ACTIVEEVENT[] = "ACTIVEEVENT";
static const char __pyx_k_GL_RED_SIZE[] = "GL_RED_SIZE";
static const char __pyx_k_JOYBUTTONUP[] = "JOYBUTTONUP";
static const char __pyx_k_KMOD_LSHIFT[] = "KMOD_LSHIFT";
static const char __pyx_k_KMOD_RSHIFT[] = "KMOD_RSHIFT";
static const char __pyx_k_K_AC_SEARCH[] = "K_AC_SEARCH";
static const char __pyx_k_K_AMPERSAND[] = "K_AMPERSAND";
static const char __pyx_k_K_AUDIOMUTE[] = "K_AUDIOMUTE";
static const char __pyx_k_K_AUDIONEXT[] = "K_AUDIONEXT";
static const char __pyx_k_K_AUDIOPLAY[] = "K_AUDIOPLAY";
static const char __pyx_k_K_AUDIOPREV[] = "K_AUDIOPREV";
static const char __pyx_k_K_AUDIOSTOP[] = "K_AUDIOSTOP";
static const char __pyx_k_K_BACKQUOTE[] = "K_BACKQUOTE";
static const char __pyx_k_K_BACKSLASH[] = "K_BACKSLASH";
static const char __pyx_k_K_BACKSPACE[] = "K_BACKSPACE";
static const char __pyx_k_K_KP_BINARY[] = "K_KP_BINARY";
static const char __pyx_k_K_KP_DIVIDE[] = "K_KP_DIVIDE";
static const char __pyx_k_K_KP_EQUALS[] = "K_KP_EQUALS";
static const char __pyx_k_K_KP_EXCLAM[] = "K_KP_EXCLAM";
static const char __pyx_k_K_KP_MEMADD[] = "K_KP_MEMADD";
static const char __pyx_k_K_KP_PERIOD[] = "K_KP_PERIOD";
static const char __pyx_k_K_LEFTPAREN[] = "K_LEFTPAREN";
static const char __pyx_k_K_SCROLLOCK[] = "K_SCROLLOCK";
static const char __pyx_k_K_SEMICOLON[] = "K_SEMICOLON";
static const char __pyx_k_K_SEPARATOR[] = "K_SEPARATOR";
static const char __pyx_k_MOUSEMOTION[] = "MOUSEMOTION";
static const char __pyx_k_SRCCOLORKEY[] = "SRCCOLORKEY";
static const char __pyx_k_TEXTEDITING[] = "TEXTEDITING";
static const char __pyx_k_VIDEOEXPOSE[] = "VIDEOEXPOSE";
static const char __pyx_k_VIDEORESIZE[] = "VIDEORESIZE";
static const char __pyx_k_WINDOWEVENT[] = "WINDOWEVENT";
static const char __pyx_k_DISPLAYEVENT[] = "DISPLAYEVENT";
static const char __pyx_k_DOLLARRECORD[] = "DOLLARRECORD";
static const char __pyx_k_DROPCOMPLETE[] = "DROPCOMPLETE";
static const char __pyx_k_FINGERMOTION[] = "FINGERMOTION";
static const char __pyx_k_GL_BLUE_SIZE[] = "GL_BLUE_SIZE";
static const char __pyx_k_JOYHATMOTION[] = "JOYHATMOTION";
static const char __pyx_k_K_AC_FORWARD[] = "K_AC_FORWARD";
static const char __pyx_k_K_AC_REFRESH[] = "K_AC_REFRESH";
static const char __pyx_k_K_CALCULATOR[] = "K_CALCULATOR";
static const char __pyx_k_K_CLEARAGAIN[] = "K_CLEARAGAIN";
static const char __pyx_k_K_KBDILLUMUP[] = "K_KBDILLUMUP";
static const char __pyx_k_K_KP_DECIMAL[] = "K_KP_DECIMAL";
static const char __pyx_k_K_KP_GREATER[] = "K_KP_GREATER";
static const char __pyx_k_K_KP_PERCENT[] = "K_KP_PERCENT";
static const char __pyx_k_K_RIGHTPAREN[] = "K_RIGHTPAREN";
static const char __pyx_k_K_SCROLLLOCK[] = "K_SCROLLLOCK";
static const char __pyx_k_K_UNDERSCORE[] = "K_UNDERSCORE";
static const char __pyx_k_K_VOLUMEDOWN[] = "K_VOLUMEDOWN";
static const char __pyx_k_MULTIGESTURE[] = "MULTIGESTURE";
static const char __pyx_k_SENSORUPDATE[] = "SENSORUPDATE";
static const char __pyx_k_WINDOW_SHOWN[] = "WINDOW_SHOWN";
static const char __pyx_k_APP_LOWMEMORY[] = "APP_LOWMEMORY";
static const char __pyx_k_BLEND_RGB_ADD[] = "BLEND_RGB_ADD";
static const char __pyx_k_BLEND_RGB_MAX[] = "BLEND_RGB_MAX";
static const char __pyx_k_BLEND_RGB_MIN[] = "BLEND_RGB_MIN";
static const char __pyx_k_BLEND_RGB_SUB[] = "BLEND_RGB_SUB";
static const char __pyx_k_DOLLARGESTURE[] = "DOLLARGESTURE";
static const char __pyx_k_GL_ALPHA_SIZE[] = "GL_ALPHA_SIZE";
static const char __pyx_k_GL_DEPTH_SIZE[] = "GL_DEPTH_SIZE";
static const char __pyx_k_GL_GREEN_SIZE[] = "GL_GREEN_SIZE";
static const char __pyx_k_JOYAXISMOTION[] = "JOYAXISMOTION";
static const char __pyx_k_JOYBALLMOTION[] = "JOYBALLMOTION";
static const char __pyx_k_JOYBUTTONDOWN[] = "JOYBUTTONDOWN";
static const char __pyx_k_KEYMAPCHANGED[] = "KEYMAPCHANGED";
static const char __pyx_k_K_APPLICATION[] = "K_APPLICATION";
static const char __pyx_k_K_AUDIOREWIND[] = "K_AUDIOREWIND";
static const char __pyx_k_K_KP_MEMCLEAR[] = "K_KP_MEMCLEAR";
static const char __pyx_k_K_KP_MEMSTORE[] = "K_KP_MEMSTORE";
static const char __pyx_k_K_KP_MULTIPLY[] = "K_KP_MULTIPLY";
static const char __pyx_k_K_LEFTBRACKET[] = "K_LEFTBRACKET";
static const char __pyx_k_K_MEDIASELECT[] = "K_MEDIASELECT";
static const char __pyx_k_K_PRINTSCREEN[] = "K_PRINTSCREEN";
static const char __pyx_k_MOUSEBUTTONUP[] = "MOUSEBUTTONUP";
static const char __pyx_k_TOUCH_MOUSEID[] = "TOUCH_MOUSEID";
static const char __pyx_k_WINDOW_HIDDEN[] = "WINDOW_HIDDEN";
static const char __pyx_k_WINDOW_OPENGL[] = "WINDOW_OPENGL";
static const char __pyx_k_BLEND_RGBA_ADD[] = "BLEND_RGBA_ADD";
static const char __pyx_k_BLEND_RGBA_MAX[] = "BLEND_RGBA_MAX";
static const char __pyx_k_BLEND_RGBA_MIN[] = "BLEND_RGBA_MIN";
static const char __pyx_k_BLEND_RGBA_SUB[] = "BLEND_RGBA_SUB";
static const char __pyx_k_BLEND_RGB_MULT[] = "BLEND_RGB_MULT";
static const char __pyx_k_GL_BUFFER_SIZE[] = "GL_BUFFER_SIZE";
static const char __pyx_k_GL_CONTEXT_EGL[] = "GL_CONTEXT_EGL";
static const char __pyx_k_JOYDEVICEADDED[] = "JOYDEVICEADDED";
static const char __pyx_k_K_AC_BOOKMARKS[] = "K_AC_BOOKMARKS";
static const char __pyx_k_K_BRIGHTNESSUP[] = "K_BRIGHTNESSUP";
static const char __pyx_k_K_CURRENCYUNIT[] = "K_CURRENCYUNIT";
static const char __pyx_k_K_KBDILLUMDOWN[] = "K_KBDILLUMDOWN";
static const char __pyx_k_K_KP_AMPERSAND[] = "K_KP_AMPERSAND";
static const char __pyx_k_K_KP_BACKSPACE[] = "K_KP_BACKSPACE";
static const char __pyx_k_K_KP_LEFTBRACE[] = "K_KP_LEFTBRACE";
static const char __pyx_k_K_KP_LEFTPAREN[] = "K_KP_LEFTPAREN";
static const char __pyx_k_K_KP_MEMDIVIDE[] = "K_KP_MEMDIVIDE";
static const char __pyx_k_K_KP_MEMRECALL[] = "K_KP_MEMRECALL";
static const char __pyx_k_K_KP_PLUSMINUS[] = "K_KP_PLUSMINUS";
static const char __pyx_k_K_NUMLOCKCLEAR[] = "K_NUMLOCKCLEAR";
static const char __pyx_k_K_RIGHTBRACKET[] = "K_RIGHTBRACKET";
static const char __pyx_k_WINDOW_FOREIGN[] = "WINDOW_FOREIGN";
static const char __pyx_k_WINDOW_NOFRAME[] = "WINDOW_NOFRAME";
static const char __pyx_k_APP_TERMINATING[] = "APP_TERMINATING";
static const char __pyx_k_BLEND_RGBA_MULT[] = "BLEND_RGBA_MULT";
static const char __pyx_k_CLIPBOARDUPDATE[] = "CLIPBOARDUPDATE";
static const char __pyx_k_GL_DOUBLEBUFFER[] = "GL_DOUBLEBUFFER";
static const char __pyx_k_GL_STENCIL_SIZE[] = "GL_STENCIL_SIZE";
static const char __pyx_k_GL_SWAP_CONTROL[] = "GL_SWAP_CONTROL";
static const char __pyx_k_K_DISPLAYSWITCH[] = "K_DISPLAYSWITCH";
static const char __pyx_k_K_KP_CLEARENTRY[] = "K_KP_CLEARENTRY";
static const char __pyx_k_K_KP_RIGHTBRACE[] = "K_KP_RIGHTBRACE";
static const char __pyx_k_K_KP_RIGHTPAREN[] = "K_KP_RIGHTPAREN";
static const char __pyx_k_MOUSEBUTTONDOWN[] = "MOUSEBUTTONDOWN";
static const char __pyx_k_AUDIODEVICEADDED[] = "AUDIODEVICEADDED";
static const char __pyx_k_GL_CONTEXT_FLAGS[] = "GL_CONTEXT_FLAGS";
static const char __pyx_k_JOYDEVICEREMOVED[] = "JOYDEVICEREMOVED";
static const char __pyx_k_K_BRIGHTNESSDOWN[] = "K_BRIGHTNESSDOWN";
static const char __pyx_k_K_KBDILLUMTOGGLE[] = "K_KBDILLUMTOGGLE";
static const char __pyx_k_K_KP_EQUALSAS400[] = "K_KP_EQUALSAS400";
static const char __pyx_k_K_KP_HEXADECIMAL[] = "K_KP_HEXADECIMAL";
static const char __pyx_k_K_KP_MEMMULTIPLY[] = "K_KP_MEMMULTIPLY";
static const char __pyx_k_K_KP_MEMSUBTRACT[] = "K_KP_MEMSUBTRACT";
static const char __pyx_k_K_KP_VERTICALBAR[] = "K_KP_VERTICALBAR";
static const char __pyx_k_WINDOW_MAXIMIZED[] = "WINDOW_MAXIMIZED";
static const char __pyx_k_WINDOW_MINIMIZED[] = "WINDOW_MINIMIZED";
static const char __pyx_k_WINDOW_RESIZABLE[] = "WINDOW_RESIZABLE";
static const char __pyx_k_pygame_sdl2_rect[] = "pygame_sdl2.rect";
static const char __pyx_k_GL_ACCUM_RED_SIZE[] = "GL_ACCUM_RED_SIZE";
static const char __pyx_k_K_CURRENCYSUBUNIT[] = "K_CURRENCYSUBUNIT";
static const char __pyx_k_K_KP_DBLAMPERSAND[] = "K_KP_DBLAMPERSAND";
static const char __pyx_k_SDL_VERSION_TUPLE[] = "SDL_VERSION_TUPLE";
static const char __pyx_k_WINDOW_BORDERLESS[] = "WINDOW_BORDERLESS";
static const char __pyx_k_WINDOW_FULLSCREEN[] = "WINDOW_FULLSCREEN";
static const char __pyx_k_pygame_sdl2_color[] = "pygame_sdl2.color";
static const char __pyx_k_AUDIODEVICEREMOVED[] = "AUDIODEVICEREMOVED";
static const char __pyx_k_CONTROLLERBUTTONUP[] = "CONTROLLERBUTTONUP";
static const char __pyx_k_GL_ACCUM_BLUE_SIZE[] = "GL_ACCUM_BLUE_SIZE";
static const char __pyx_k_K_AUDIOFASTFORWARD[] = "K_AUDIOFASTFORWARD";
static const char __pyx_k_K_DECIMALSEPARATOR[] = "K_DECIMALSEPARATOR";
static const char __pyx_k_POWERSTATE_CHARGED[] = "POWERSTATE_CHARGED";
static const char __pyx_k_POWERSTATE_UNKNOWN[] = "POWERSTATE_UNKNOWN";
static const char __pyx_k_WINDOWPOS_CENTERED[] = "WINDOWPOS_CENTERED";
static const char __pyx_k_WINDOW_INPUT_FOCUS[] = "WINDOW_INPUT_FOCUS";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_BLEND_PREMULTIPLIED[] = "BLEND_PREMULTIPLIED";
static const char __pyx_k_CONTROLLER_AXIS_MAX[] = "CONTROLLER_AXIS_MAX";
static const char __pyx_k_CONTROLLER_BUTTON_A[] = "CONTROLLER_BUTTON_A";
static const char __pyx_k_CONTROLLER_BUTTON_B[] = "CONTROLLER_BUTTON_B";
static const char __pyx_k_CONTROLLER_BUTTON_X[] = "CONTROLLER_BUTTON_X";
static const char __pyx_k_CONTROLLER_BUTTON_Y[] = "CONTROLLER_BUTTON_Y";
static const char __pyx_k_GL_ACCUM_ALPHA_SIZE[] = "GL_ACCUM_ALPHA_SIZE";
static const char __pyx_k_GL_ACCUM_GREEN_SIZE[] = "GL_ACCUM_GREEN_SIZE";
static const char __pyx_k_GL_CONTEXT_NO_ERROR[] = "GL_CONTEXT_NO_ERROR";
static const char __pyx_k_GL_RETAINED_BACKING[] = "GL_RETAINED_BACKING";
static const char __pyx_k_K_KP_DBLVERTICALBAR[] = "K_KP_DBLVERTICALBAR";
static const char __pyx_k_POWERSTATE_CHARGING[] = "POWERSTATE_CHARGING";
static const char __pyx_k_RENDER_DEVICE_RESET[] = "RENDER_DEVICE_RESET";
static const char __pyx_k_WINDOWPOS_UNDEFINED[] = "WINDOWPOS_UNDEFINED";
static const char __pyx_k_CONTROLLERAXISMOTION[] = "CONTROLLERAXISMOTION";
static const char __pyx_k_CONTROLLERBUTTONDOWN[] = "CONTROLLERBUTTONDOWN";
static const char __pyx_k_K_THOUSANDSSEPARATOR[] = "K_THOUSANDSSEPARATOR";
static const char __pyx_k_RENDER_TARGETS_RESET[] = "RENDER_TARGETS_RESET";
static const char __pyx_k_WINDOW_ALLOW_HIGHDPI[] = "WINDOW_ALLOW_HIGHDPI";
static const char __pyx_k_WINDOW_INPUT_GRABBED[] = "WINDOW_INPUT_GRABBED";
static const char __pyx_k_CONTROLLERDEVICEADDED[] = "CONTROLLERDEVICEADDED";
static const char __pyx_k_CONTROLLER_AXIS_LEFTX[] = "CONTROLLER_AXIS_LEFTX";
static const char __pyx_k_CONTROLLER_AXIS_LEFTY[] = "CONTROLLER_AXIS_LEFTY";
static const char __pyx_k_CONTROLLER_BUTTON_MAX[] = "CONTROLLER_BUTTON_MAX";
static const char __pyx_k_GL_ACCELERATED_VISUAL[] = "GL_ACCELERATED_VISUAL";
static const char __pyx_k_GL_CONTEXT_PROFILE_ES[] = "GL_CONTEXT_PROFILE_ES";
static const char __pyx_k_GL_MULTISAMPLEBUFFERS[] = "GL_MULTISAMPLEBUFFERS";
static const char __pyx_k_GL_MULTISAMPLESAMPLES[] = "GL_MULTISAMPLESAMPLES";
static const char __pyx_k_POWERSTATE_NO_BATTERY[] = "POWERSTATE_NO_BATTERY";
static const char __pyx_k_POWERSTATE_ON_BATTERY[] = "POWERSTATE_ON_BATTERY";
static const char __pyx_k_APP_DIDENTERBACKGROUND[] = "APP_DIDENTERBACKGROUND";
static const char __pyx_k_APP_DIDENTERFOREGROUND[] = "APP_DIDENTERFOREGROUND";
static const char __pyx_k_CONTROLLER_AXIS_RIGHTX[] = "CONTROLLER_AXIS_RIGHTX";
static const char __pyx_k_CONTROLLER_AXIS_RIGHTY[] = "CONTROLLER_AXIS_RIGHTY";
static const char __pyx_k_CONTROLLER_BUTTON_BACK[] = "CONTROLLER_BUTTON_BACK";
static const char __pyx_k_APP_WILLENTERBACKGROUND[] = "APP_WILLENTERBACKGROUND";
static const char __pyx_k_APP_WILLENTERFOREGROUND[] = "APP_WILLENTERFOREGROUND";
static const char __pyx_k_CONTROLLERDEVICEREMOVED[] = "CONTROLLERDEVICEREMOVED";
static const char __pyx_k_CONTROLLER_AXIS_INVALID[] = "CONTROLLER_AXIS_INVALID";
static const char __pyx_k_CONTROLLER_BUTTON_GUIDE[] = "CONTROLLER_BUTTON_GUIDE";
static const char __pyx_k_CONTROLLER_BUTTON_START[] = "CONTROLLER_BUTTON_START";
static const char __pyx_k_GL_CONTEXT_PROFILE_CORE[] = "GL_CONTEXT_PROFILE_CORE";
static const char __pyx_k_GL_CONTEXT_PROFILE_MASK[] = "GL_CONTEXT_PROFILE_MASK";
static const char __pyx_k_CONTROLLERDEVICEREMAPPED[] = "CONTROLLERDEVICEREMAPPED";
static const char __pyx_k_GL_CONTEXT_MAJOR_VERSION[] = "GL_CONTEXT_MAJOR_VERSION";
static const char __pyx_k_GL_CONTEXT_MINOR_VERSION[] = "GL_CONTEXT_MINOR_VERSION";
static const char __pyx_k_CONTROLLER_BUTTON_DPAD_UP[] = "CONTROLLER_BUTTON_DPAD_UP";
static const char __pyx_k_CONTROLLER_BUTTON_INVALID[] = "CONTROLLER_BUTTON_INVALID";
static const char __pyx_k_WINDOW_FULLSCREEN_DESKTOP[] = "WINDOW_FULLSCREEN_DESKTOP";
static const char __pyx_k_CONTROLLER_AXIS_TRIGGERLEFT[] = "CONTROLLER_AXIS_TRIGGERLEFT";
static const char __pyx_k_CONTROLLER_BUTTON_DPAD_DOWN[] = "CONTROLLER_BUTTON_DPAD_DOWN";
static const char __pyx_k_CONTROLLER_BUTTON_DPAD_LEFT[] = "CONTROLLER_BUTTON_DPAD_LEFT";
static const char __pyx_k_CONTROLLER_BUTTON_LEFTSTICK[] = "CONTROLLER_BUTTON_LEFTSTICK";
static const char __pyx_k_GL_CONTEXT_RELEASE_BEHAVIOR[] = "GL_CONTEXT_RELEASE_BEHAVIOR";
static const char __pyx_k_GL_FRAMEBUFFER_SRGB_CAPABLE[] = "GL_FRAMEBUFFER_SRGB_CAPABLE";
static const char __pyx_k_CONTROLLER_AXIS_TRIGGERRIGHT[] = "CONTROLLER_AXIS_TRIGGERRIGHT";
static const char __pyx_k_CONTROLLER_BUTTON_DPAD_RIGHT[] = "CONTROLLER_BUTTON_DPAD_RIGHT";
static const char __pyx_k_CONTROLLER_BUTTON_RIGHTSTICK[] = "CONTROLLER_BUTTON_RIGHTSTICK";
static const char __pyx_k_GL_CONTEXT_RESET_NOTIFICATION[] = "GL_CONTEXT_RESET_NOTIFICATION";
static const char __pyx_k_GL_SHARE_WITH_CURRENT_CONTEXT[] = "GL_SHARE_WITH_CURRENT_CONTEXT";
static const char __pyx_k_CONTROLLER_BUTTON_LEFTSHOULDER[] = "CONTROLLER_BUTTON_LEFTSHOULDER";
static const char __pyx_k_CONTROLLER_BUTTON_RIGHTSHOULDER[] = "CONTROLLER_BUTTON_RIGHTSHOULDER";
static const char __pyx_k_GL_CONTEXT_PROFILE_COMPATIBILITY[] = "GL_CONTEXT_PROFILE_COMPATIBILITY";
static PyObject *__pyx_n_s_ACTIVEEVENT;
static PyObject *__pyx_n_s_APP_DIDENTERBACKGROUND;
static PyObject *__pyx_n_s_APP_DIDENTERFOREGROUND;
static PyObject *__pyx_n_s_APP_LOWMEMORY;
static PyObject *__pyx_n_s_APP_TERMINATING;
static PyObject *__pyx_n_s_APP_WILLENTERBACKGROUND;
static PyObject *__pyx_n_s_APP_WILLENTERFOREGROUND;
static PyObject *__pyx_n_s_AUDIODEVICEADDED;
static PyObject *__pyx_n_s_AUDIODEVICEREMOVED;
static PyObject *__pyx_n_s_BIG_ENDIAN;
static PyObject *__pyx_n_s_BLEND_ADD;
static PyObject *__pyx_n_s_BLEND_MAX;
static PyObject *__pyx_n_s_BLEND_MIN;
static PyObject *__pyx_n_s_BLEND_MULT;
static PyObject *__pyx_n_s_BLEND_PREMULTIPLIED;
static PyObject *__pyx_n_s_BLEND_RGBA_ADD;
static PyObject *__pyx_n_s_BLEND_RGBA_MAX;
static PyObject *__pyx_n_s_BLEND_RGBA_MIN;
static PyObject *__pyx_n_s_BLEND_RGBA_MULT;
static PyObject *__pyx_n_s_BLEND_RGBA_SUB;
static PyObject *__pyx_n_s_BLEND_RGB_ADD;
static PyObject *__pyx_n_s_BLEND_RGB_MAX;
static PyObject *__pyx_n_s_BLEND_RGB_MIN;
static PyObject *__pyx_n_s_BLEND_RGB_MULT;
static PyObject *__pyx_n_s_BLEND_RGB_SUB;
static PyObject *__pyx_n_s_BLEND_SUB;
static PyObject *__pyx_n_s_BYTEORDER;
static PyObject *__pyx_n_s_CLIPBOARDUPDATE;
static PyObject *__pyx_n_s_CONTROLLERAXISMOTION;
static PyObject *__pyx_n_s_CONTROLLERBUTTONDOWN;
static PyObject *__pyx_n_s_CONTROLLERBUTTONUP;
static PyObject *__pyx_n_s_CONTROLLERDEVICEADDED;
static PyObject *__pyx_n_s_CONTROLLERDEVICEREMAPPED;
static PyObject *__pyx_n_s_CONTROLLERDEVICEREMOVED;
static PyObject *__pyx_n_s_CONTROLLER_AXIS_INVALID;
static PyObject *__pyx_n_s_CONTROLLER_AXIS_LEFTX;
static PyObject *__pyx_n_s_CONTROLLER_AXIS_LEFTY;
static PyObject *__pyx_n_s_CONTROLLER_AXIS_MAX;
static PyObject *__pyx_n_s_CONTROLLER_AXIS_RIGHTX;
static PyObject *__pyx_n_s_CONTROLLER_AXIS_RIGHTY;
static PyObject *__pyx_n_s_CONTROLLER_AXIS_TRIGGERLEFT;
static PyObject *__pyx_n_s_CONTROLLER_AXIS_TRIGGERRIGHT;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_A;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_B;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_BACK;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_DPAD_DOWN;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_DPAD_LEFT;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_DPAD_RIGHT;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_DPAD_UP;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_GUIDE;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_INVALID;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_LEFTSHOULDER;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_LEFTSTICK;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_MAX;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_RIGHTSHOULDER;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_RIGHTSTICK;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_START;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_X;
static PyObject *__pyx_n_s_CONTROLLER_BUTTON_Y;
static PyObject *__pyx_n_s_Color;
static PyObject *__pyx_n_s_DISPLAYEVENT;
static PyObject *__pyx_n_s_DOLLARGESTURE;
static PyObject *__pyx_n_s_DOLLARRECORD;
static PyObject *__pyx_n_s_DOUBLEBUF;
static PyObject *__pyx_n_s_DROPBEGIN;
static PyObject *__pyx_n_s_DROPCOMPLETE;
static PyObject *__pyx_n_s_DROPFILE;
static PyObject *__pyx_n_s_DROPTEXT;
static PyObject *__pyx_n_s_FINGERDOWN;
static PyObject *__pyx_n_s_FINGERMOTION;
static PyObject *__pyx_n_s_FINGERUP;
static PyObject *__pyx_n_s_FULLSCREEN;
static PyObject *__pyx_n_s_GL_ACCELERATED_VISUAL;
static PyObject *__pyx_n_s_GL_ACCUM_ALPHA_SIZE;
static PyObject *__pyx_n_s_GL_ACCUM_BLUE_SIZE;
static PyObject *__pyx_n_s_GL_ACCUM_GREEN_SIZE;
static PyObject *__pyx_n_s_GL_ACCUM_RED_SIZE;
static PyObject *__pyx_n_s_GL_ALPHA_SIZE;
static PyObject *__pyx_n_s_GL_BLUE_SIZE;
static PyObject *__pyx_n_s_GL_BUFFER_SIZE;
static PyObject *__pyx_n_s_GL_CONTEXT_EGL;
static PyObject *__pyx_n_s_GL_CONTEXT_FLAGS;
static PyObject *__pyx_n_s_GL_CONTEXT_MAJOR_VERSION;
static PyObject *__pyx_n_s_GL_CONTEXT_MINOR_VERSION;
static PyObject *__pyx_n_s_GL_CONTEXT_NO_ERROR;
static PyObject *__pyx_n_s_GL_CONTEXT_PROFILE_COMPATIBILITY;
static PyObject *__pyx_n_s_GL_CONTEXT_PROFILE_CORE;
static PyObject *__pyx_n_s_GL_CONTEXT_PROFILE_ES;
static PyObject *__pyx_n_s_GL_CONTEXT_PROFILE_MASK;
static PyObject *__pyx_n_s_GL_CONTEXT_RELEASE_BEHAVIOR;
static PyObject *__pyx_n_s_GL_CONTEXT_RESET_NOTIFICATION;
static PyObject *__pyx_n_s_GL_DEPTH_SIZE;
static PyObject *__pyx_n_s_GL_DOUBLEBUFFER;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER_SRGB_CAPABLE;
static PyObject *__pyx_n_s_GL_GREEN_SIZE;
static PyObject *__pyx_n_s_GL_MULTISAMPLEBUFFERS;
static PyObject *__pyx_n_s_GL_MULTISAMPLESAMPLES;
static PyObject *__pyx_n_s_GL_RED_SIZE;
static PyObject *__pyx_n_s_GL_RETAINED_BACKING;
static PyObject *__pyx_n_s_GL_SHARE_WITH_CURRENT_CONTEXT;
static PyObject *__pyx_n_s_GL_STENCIL_SIZE;
static PyObject *__pyx_n_s_GL_STEREO;
static PyObject *__pyx_n_s_GL_SWAP_CONTROL;
static PyObject *__pyx_n_s_HWSURFACE;
static PyObject *__pyx_n_s_JOYAXISMOTION;
static PyObject *__pyx_n_s_JOYBALLMOTION;
static PyObject *__pyx_n_s_JOYBUTTONDOWN;
static PyObject *__pyx_n_s_JOYBUTTONUP;
static PyObject *__pyx_n_s_JOYDEVICEADDED;
static PyObject *__pyx_n_s_JOYDEVICEREMOVED;
static PyObject *__pyx_n_s_JOYHATMOTION;
static PyObject *__pyx_n_s_KEYDOWN;
static PyObject *__pyx_n_s_KEYMAPCHANGED;
static PyObject *__pyx_n_s_KEYUP;
static PyObject *__pyx_n_s_KMOD_ALT;
static PyObject *__pyx_n_s_KMOD_CAPS;
static PyObject *__pyx_n_s_KMOD_CTRL;
static PyObject *__pyx_n_s_KMOD_GUI;
static PyObject *__pyx_n_s_KMOD_LALT;
static PyObject *__pyx_n_s_KMOD_LCTRL;
static PyObject *__pyx_n_s_KMOD_LGUI;
static PyObject *__pyx_n_s_KMOD_LMETA;
static PyObject *__pyx_n_s_KMOD_LSHIFT;
static PyObject *__pyx_n_s_KMOD_META;
static PyObject *__pyx_n_s_KMOD_MODE;
static PyObject *__pyx_n_s_KMOD_NONE;
static PyObject *__pyx_n_s_KMOD_NUM;
static PyObject *__pyx_n_s_KMOD_RALT;
static PyObject *__pyx_n_s_KMOD_RCTRL;
static PyObject *__pyx_n_s_KMOD_RGUI;
static PyObject *__pyx_n_s_KMOD_RMETA;
static PyObject *__pyx_n_s_KMOD_RSHIFT;
static PyObject *__pyx_n_s_KMOD_SHIFT;
static PyObject *__pyx_n_s_K_0;
static PyObject *__pyx_n_s_K_1;
static PyObject *__pyx_n_s_K_2;
static PyObject *__pyx_n_s_K_3;
static PyObject *__pyx_n_s_K_4;
static PyObject *__pyx_n_s_K_5;
static PyObject *__pyx_n_s_K_6;
static PyObject *__pyx_n_s_K_7;
static PyObject *__pyx_n_s_K_8;
static PyObject *__pyx_n_s_K_9;
static PyObject *__pyx_n_s_K_AC_BACK;
static PyObject *__pyx_n_s_K_AC_BOOKMARKS;
static PyObject *__pyx_n_s_K_AC_FORWARD;
static PyObject *__pyx_n_s_K_AC_HOME;
static PyObject *__pyx_n_s_K_AC_REFRESH;
static PyObject *__pyx_n_s_K_AC_SEARCH;
static PyObject *__pyx_n_s_K_AC_STOP;
static PyObject *__pyx_n_s_K_AGAIN;
static PyObject *__pyx_n_s_K_ALTERASE;
static PyObject *__pyx_n_s_K_AMPERSAND;
static PyObject *__pyx_n_s_K_APP1;
static PyObject *__pyx_n_s_K_APP2;
static PyObject *__pyx_n_s_K_APPLICATION;
static PyObject *__pyx_n_s_K_ASTERISK;
static PyObject *__pyx_n_s_K_AT;
static PyObject *__pyx_n_s_K_AUDIOFASTFORWARD;
static PyObject *__pyx_n_s_K_AUDIOMUTE;
static PyObject *__pyx_n_s_K_AUDIONEXT;
static PyObject *__pyx_n_s_K_AUDIOPLAY;
static PyObject *__pyx_n_s_K_AUDIOPREV;
static PyObject *__pyx_n_s_K_AUDIOREWIND;
static PyObject *__pyx_n_s_K_AUDIOSTOP;
static PyObject *__pyx_n_s_K_BACKQUOTE;
static PyObject *__pyx_n_s_K_BACKSLASH;
static PyObject *__pyx_n_s_K_BACKSPACE;
static PyObject *__pyx_n_s_K_BREAK;
static PyObject *__pyx_n_s_K_BRIGHTNESSDOWN;
static PyObject *__pyx_n_s_K_BRIGHTNESSUP;
static PyObject *__pyx_n_s_K_CALCULATOR;
static PyObject *__pyx_n_s_K_CANCEL;
static PyObject *__pyx_n_s_K_CAPSLOCK;
static PyObject *__pyx_n_s_K_CARET;
static PyObject *__pyx_n_s_K_CLEAR;
static PyObject *__pyx_n_s_K_CLEARAGAIN;
static PyObject *__pyx_n_s_K_COLON;
static PyObject *__pyx_n_s_K_COMMA;
static PyObject *__pyx_n_s_K_COMPUTER;
static PyObject *__pyx_n_s_K_COPY;
static PyObject *__pyx_n_s_K_CRSEL;
static PyObject *__pyx_n_s_K_CURRENCYSUBUNIT;
static PyObject *__pyx_n_s_K_CURRENCYUNIT;
static PyObject *__pyx_n_s_K_CUT;
static PyObject *__pyx_n_s_K_DECIMALSEPARATOR;
static PyObject *__pyx_n_s_K_DELETE;
static PyObject *__pyx_n_s_K_DISPLAYSWITCH;
static PyObject *__pyx_n_s_K_DOLLAR;
static PyObject *__pyx_n_s_K_DOWN;
static PyObject *__pyx_n_s_K_EJECT;
static PyObject *__pyx_n_s_K_END;
static PyObject *__pyx_n_s_K_EQUALS;
static PyObject *__pyx_n_s_K_ESCAPE;
static PyObject *__pyx_n_s_K_EURO;
static PyObject *__pyx_n_s_K_EXCLAIM;
static PyObject *__pyx_n_s_K_EXECUTE;
static PyObject *__pyx_n_s_K_EXSEL;
static PyObject *__pyx_n_s_K_F1;
static PyObject *__pyx_n_s_K_F10;
static PyObject *__pyx_n_s_K_F11;
static PyObject *__pyx_n_s_K_F12;
static PyObject *__pyx_n_s_K_F13;
static PyObject *__pyx_n_s_K_F14;
static PyObject *__pyx_n_s_K_F15;
static PyObject *__pyx_n_s_K_F16;
static PyObject *__pyx_n_s_K_F17;
static PyObject *__pyx_n_s_K_F18;
static PyObject *__pyx_n_s_K_F19;
static PyObject *__pyx_n_s_K_F2;
static PyObject *__pyx_n_s_K_F20;
static PyObject *__pyx_n_s_K_F21;
static PyObject *__pyx_n_s_K_F22;
static PyObject *__pyx_n_s_K_F23;
static PyObject *__pyx_n_s_K_F24;
static PyObject *__pyx_n_s_K_F3;
static PyObject *__pyx_n_s_K_F4;
static PyObject *__pyx_n_s_K_F5;
static PyObject *__pyx_n_s_K_F6;
static PyObject *__pyx_n_s_K_F7;
static PyObject *__pyx_n_s_K_F8;
static PyObject *__pyx_n_s_K_F9;
static PyObject *__pyx_n_s_K_FIND;
static PyObject *__pyx_n_s_K_FIRST;
static PyObject *__pyx_n_s_K_GREATER;
static PyObject *__pyx_n_s_K_HASH;
static PyObject *__pyx_n_s_K_HELP;
static PyObject *__pyx_n_s_K_HOME;
static PyObject *__pyx_n_s_K_INSERT;
static PyObject *__pyx_n_s_K_KBDILLUMDOWN;
static PyObject *__pyx_n_s_K_KBDILLUMTOGGLE;
static PyObject *__pyx_n_s_K_KBDILLUMUP;
static PyObject *__pyx_n_s_K_KP0;
static PyObject *__pyx_n_s_K_KP1;
static PyObject *__pyx_n_s_K_KP2;
static PyObject *__pyx_n_s_K_KP3;
static PyObject *__pyx_n_s_K_KP4;
static PyObject *__pyx_n_s_K_KP5;
static PyObject *__pyx_n_s_K_KP6;
static PyObject *__pyx_n_s_K_KP7;
static PyObject *__pyx_n_s_K_KP8;
static PyObject *__pyx_n_s_K_KP9;
static PyObject *__pyx_n_s_K_KP_0;
static PyObject *__pyx_n_s_K_KP_00;
static PyObject *__pyx_n_s_K_KP_000;
static PyObject *__pyx_n_s_K_KP_1;
static PyObject *__pyx_n_s_K_KP_2;
static PyObject *__pyx_n_s_K_KP_3;
static PyObject *__pyx_n_s_K_KP_4;
static PyObject *__pyx_n_s_K_KP_5;
static PyObject *__pyx_n_s_K_KP_6;
static PyObject *__pyx_n_s_K_KP_7;
static PyObject *__pyx_n_s_K_KP_8;
static PyObject *__pyx_n_s_K_KP_9;
static PyObject *__pyx_n_s_K_KP_A;
static PyObject *__pyx_n_s_K_KP_AMPERSAND;
static PyObject *__pyx_n_s_K_KP_AT;
static PyObject *__pyx_n_s_K_KP_B;
static PyObject *__pyx_n_s_K_KP_BACKSPACE;
static PyObject *__pyx_n_s_K_KP_BINARY;
static PyObject *__pyx_n_s_K_KP_C;
static PyObject *__pyx_n_s_K_KP_CLEAR;
static PyObject *__pyx_n_s_K_KP_CLEARENTRY;
static PyObject *__pyx_n_s_K_KP_COLON;
static PyObject *__pyx_n_s_K_KP_COMMA;
static PyObject *__pyx_n_s_K_KP_D;
static PyObject *__pyx_n_s_K_KP_DBLAMPERSAND;
static PyObject *__pyx_n_s_K_KP_DBLVERTICALBAR;
static PyObject *__pyx_n_s_K_KP_DECIMAL;
static PyObject *__pyx_n_s_K_KP_DIVIDE;
static PyObject *__pyx_n_s_K_KP_E;
static PyObject *__pyx_n_s_K_KP_ENTER;
static PyObject *__pyx_n_s_K_KP_EQUALS;
static PyObject *__pyx_n_s_K_KP_EQUALSAS400;
static PyObject *__pyx_n_s_K_KP_EXCLAM;
static PyObject *__pyx_n_s_K_KP_F;
static PyObject *__pyx_n_s_K_KP_GREATER;
static PyObject *__pyx_n_s_K_KP_HASH;
static PyObject *__pyx_n_s_K_KP_HEXADECIMAL;
static PyObject *__pyx_n_s_K_KP_LEFTBRACE;
static PyObject *__pyx_n_s_K_KP_LEFTPAREN;
static PyObject *__pyx_n_s_K_KP_LESS;
static PyObject *__pyx_n_s_K_KP_MEMADD;
static PyObject *__pyx_n_s_K_KP_MEMCLEAR;
static PyObject *__pyx_n_s_K_KP_MEMDIVIDE;
static PyObject *__pyx_n_s_K_KP_MEMMULTIPLY;
static PyObject *__pyx_n_s_K_KP_MEMRECALL;
static PyObject *__pyx_n_s_K_KP_MEMSTORE;
static PyObject *__pyx_n_s_K_KP_MEMSUBTRACT;
static PyObject *__pyx_n_s_K_KP_MINUS;
static PyObject *__pyx_n_s_K_KP_MULTIPLY;
static PyObject *__pyx_n_s_K_KP_OCTAL;
static PyObject *__pyx_n_s_K_KP_PERCENT;
static PyObject *__pyx_n_s_K_KP_PERIOD;
static PyObject *__pyx_n_s_K_KP_PLUS;
static PyObject *__pyx_n_s_K_KP_PLUSMINUS;
static PyObject *__pyx_n_s_K_KP_POWER;
static PyObject *__pyx_n_s_K_KP_RIGHTBRACE;
static PyObject *__pyx_n_s_K_KP_RIGHTPAREN;
static PyObject *__pyx_n_s_K_KP_SPACE;
static PyObject *__pyx_n_s_K_KP_TAB;
static PyObject *__pyx_n_s_K_KP_VERTICALBAR;
static PyObject *__pyx_n_s_K_KP_XOR;
static PyObject *__pyx_n_s_K_LALT;
static PyObject *__pyx_n_s_K_LAST;
static PyObject *__pyx_n_s_K_LCTRL;
static PyObject *__pyx_n_s_K_LEFT;
static PyObject *__pyx_n_s_K_LEFTBRACKET;
static PyObject *__pyx_n_s_K_LEFTPAREN;
static PyObject *__pyx_n_s_K_LESS;
static PyObject *__pyx_n_s_K_LGUI;
static PyObject *__pyx_n_s_K_LMETA;
static PyObject *__pyx_n_s_K_LSHIFT;
static PyObject *__pyx_n_s_K_LSUPER;
static PyObject *__pyx_n_s_K_MAIL;
static PyObject *__pyx_n_s_K_MEDIASELECT;
static PyObject *__pyx_n_s_K_MENU;
static PyObject *__pyx_n_s_K_MINUS;
static PyObject *__pyx_n_s_K_MODE;
static PyObject *__pyx_n_s_K_MUTE;
static PyObject *__pyx_n_s_K_NUMLOCK;
static PyObject *__pyx_n_s_K_NUMLOCKCLEAR;
static PyObject *__pyx_n_s_K_OPER;
static PyObject *__pyx_n_s_K_OUT;
static PyObject *__pyx_n_s_K_PAGEDOWN;
static PyObject *__pyx_n_s_K_PAGEUP;
static PyObject *__pyx_n_s_K_PASTE;
static PyObject *__pyx_n_s_K_PAUSE;
static PyObject *__pyx_n_s_K_PERCENT;
static PyObject *__pyx_n_s_K_PERIOD;
static PyObject *__pyx_n_s_K_PLUS;
static PyObject *__pyx_n_s_K_POWER;
static PyObject *__pyx_n_s_K_PRINT;
static PyObject *__pyx_n_s_K_PRINTSCREEN;
static PyObject *__pyx_n_s_K_PRIOR;
static PyObject *__pyx_n_s_K_QUESTION;
static PyObject *__pyx_n_s_K_QUOTE;
static PyObject *__pyx_n_s_K_QUOTEDBL;
static PyObject *__pyx_n_s_K_RALT;
static PyObject *__pyx_n_s_K_RCTRL;
static PyObject *__pyx_n_s_K_RETURN;
static PyObject *__pyx_n_s_K_RETURN2;
static PyObject *__pyx_n_s_K_RGUI;
static PyObject *__pyx_n_s_K_RIGHT;
static PyObject *__pyx_n_s_K_RIGHTBRACKET;
static PyObject *__pyx_n_s_K_RIGHTPAREN;
static PyObject *__pyx_n_s_K_RMETA;
static PyObject *__pyx_n_s_K_RSHIFT;
static PyObject *__pyx_n_s_K_RSUPER;
static PyObject *__pyx_n_s_K_SCROLLLOCK;
static PyObject *__pyx_n_s_K_SCROLLOCK;
static PyObject *__pyx_n_s_K_SELECT;
static PyObject *__pyx_n_s_K_SEMICOLON;
static PyObject *__pyx_n_s_K_SEPARATOR;
static PyObject *__pyx_n_s_K_SLASH;
static PyObject *__pyx_n_s_K_SLEEP;
static PyObject *__pyx_n_s_K_SPACE;
static PyObject *__pyx_n_s_K_STOP;
static PyObject *__pyx_n_s_K_SYSREQ;
static PyObject *__pyx_n_s_K_TAB;
static PyObject *__pyx_n_s_K_THOUSANDSSEPARATOR;
static PyObject *__pyx_n_s_K_UNDERSCORE;
static PyObject *__pyx_n_s_K_UNDO;
static PyObject *__pyx_n_s_K_UNKNOWN;
static PyObject *__pyx_n_s_K_UP;
static PyObject *__pyx_n_s_K_VOLUMEDOWN;
static PyObject *__pyx_n_s_K_VOLUMEUP;
static PyObject *__pyx_n_s_K_WWW;
static PyObject *__pyx_n_s_K_a;
static PyObject *__pyx_n_s_K_b;
static PyObject *__pyx_n_s_K_c;
static PyObject *__pyx_n_s_K_d;
static PyObject *__pyx_n_s_K_e;
static PyObject *__pyx_n_s_K_f;
static PyObject *__pyx_n_s_K_g;
static PyObject *__pyx_n_s_K_h;
static PyObject *__pyx_n_s_K_i;
static PyObject *__pyx_n_s_K_j;
static PyObject *__pyx_n_s_K_k;
static PyObject *__pyx_n_s_K_l;
static PyObject *__pyx_n_s_K_m;
static PyObject *__pyx_n_s_K_n;
static PyObject *__pyx_n_s_K_o;
static PyObject *__pyx_n_s_K_p;
static PyObject *__pyx_n_s_K_q;
static PyObject *__pyx_n_s_K_r;
static PyObject *__pyx_n_s_K_s;
static PyObject *__pyx_n_s_K_t;
static PyObject *__pyx_n_s_K_u;
static PyObject *__pyx_n_s_K_v;
static PyObject *__pyx_n_s_K_w;
static PyObject *__pyx_n_s_K_x;
static PyObject *__pyx_n_s_K_y;
static PyObject *__pyx_n_s_K_z;
static PyObject *__pyx_n_s_LASTEVENT;
static PyObject *__pyx_n_s_LIL_ENDIAN;
static PyObject *__pyx_n_s_MOUSEBUTTONDOWN;
static PyObject *__pyx_n_s_MOUSEBUTTONUP;
static PyObject *__pyx_n_s_MOUSEMOTION;
static PyObject *__pyx_n_s_MOUSEWHEEL;
static PyObject *__pyx_n_s_MULTIGESTURE;
static PyObject *__pyx_n_s_NOEVENT;
static PyObject *__pyx_n_s_NOFRAME;
static PyObject *__pyx_n_s_OPENGL;
static PyObject *__pyx_n_s_POWERSTATE_CHARGED;
static PyObject *__pyx_n_s_POWERSTATE_CHARGING;
static PyObject *__pyx_n_s_POWERSTATE_NO_BATTERY;
static PyObject *__pyx_n_s_POWERSTATE_ON_BATTERY;
static PyObject *__pyx_n_s_POWERSTATE_UNKNOWN;
static PyObject *__pyx_n_s_QUIT;
static PyObject *__pyx_n_s_RENDER_DEVICE_RESET;
static PyObject *__pyx_n_s_RENDER_TARGETS_RESET;
static PyObject *__pyx_n_s_RESIZABLE;
static PyObject *__pyx_n_s_RLEACCEL;
static PyObject *__pyx_n_s_Rect;
static PyObject *__pyx_n_s_SCRAP_TEXT;
static PyObject *__pyx_n_s_SDL_VERSION_TUPLE;
static PyObject *__pyx_n_s_SENSORUPDATE;
static PyObject *__pyx_n_s_SRCALPHA;
static PyObject *__pyx_n_s_SRCCOLORKEY;
static PyObject *__pyx_n_s_SWSURFACE;
static PyObject *__pyx_n_s_SYSWMEVENT;
static PyObject *__pyx_n_s_TEXTEDITING;
static PyObject *__pyx_n_s_TEXTINPUT;
static PyObject *__pyx_n_s_TOUCH_MOUSEID;
static PyObject *__pyx_n_s_USEREVENT;
static PyObject *__pyx_n_s_VIDEOEXPOSE;
static PyObject *__pyx_n_s_VIDEORESIZE;
static PyObject *__pyx_n_s_WINDOWEVENT;
static PyObject *__pyx_n_s_WINDOWPOS_CENTERED;
static PyObject *__pyx_n_s_WINDOWPOS_UNDEFINED;
static PyObject *__pyx_n_s_WINDOW_ALLOW_HIGHDPI;
static PyObject *__pyx_n_s_WINDOW_BORDERLESS;
static PyObject *__pyx_n_s_WINDOW_FOREIGN;
static PyObject *__pyx_n_s_WINDOW_FULLSCREEN;
static PyObject *__pyx_n_s_WINDOW_FULLSCREEN_DESKTOP;
static PyObject *__pyx_n_s_WINDOW_HIDDEN;
static PyObject *__pyx_n_s_WINDOW_INPUT_FOCUS;
static PyObject *__pyx_n_s_WINDOW_INPUT_GRABBED;
static PyObject *__pyx_n_s_WINDOW_MAXIMIZED;
static PyObject *__pyx_n_s_WINDOW_MINIMIZED;
static PyObject *__pyx_n_s_WINDOW_NOFRAME;
static PyObject *__pyx_n_s_WINDOW_OPENGL;
static PyObject *__pyx_n_s_WINDOW_RESIZABLE;
static PyObject *__pyx_n_s_WINDOW_SHOWN;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_pygame_sdl2_color;
static PyObject *__pyx_n_s_pygame_sdl2_rect;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_kp_s_text_plain;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_5;
static PyObject *__pyx_int_6;
static PyObject *__pyx_int_7;
static PyObject *__pyx_int_8;
static PyObject *__pyx_int_9;
static PyObject *__pyx_int_16;
static PyObject *__pyx_int_17;
static PyObject *__pyx_int_1073741824;
static PyObject *__pyx_int_2147483648;
static PyObject *__pyx_int_neg_1;
/* Late includes */

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_locals(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_locals},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "locals",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_ACTIVEEVENT, __pyx_k_ACTIVEEVENT, sizeof(__pyx_k_ACTIVEEVENT), 0, 0, 1, 1},
  {&__pyx_n_s_APP_DIDENTERBACKGROUND, __pyx_k_APP_DIDENTERBACKGROUND, sizeof(__pyx_k_APP_DIDENTERBACKGROUND), 0, 0, 1, 1},
  {&__pyx_n_s_APP_DIDENTERFOREGROUND, __pyx_k_APP_DIDENTERFOREGROUND, sizeof(__pyx_k_APP_DIDENTERFOREGROUND), 0, 0, 1, 1},
  {&__pyx_n_s_APP_LOWMEMORY, __pyx_k_APP_LOWMEMORY, sizeof(__pyx_k_APP_LOWMEMORY), 0, 0, 1, 1},
  {&__pyx_n_s_APP_TERMINATING, __pyx_k_APP_TERMINATING, sizeof(__pyx_k_APP_TERMINATING), 0, 0, 1, 1},
  {&__pyx_n_s_APP_WILLENTERBACKGROUND, __pyx_k_APP_WILLENTERBACKGROUND, sizeof(__pyx_k_APP_WILLENTERBACKGROUND), 0, 0, 1, 1},
  {&__pyx_n_s_APP_WILLENTERFOREGROUND, __pyx_k_APP_WILLENTERFOREGROUND, sizeof(__pyx_k_APP_WILLENTERFOREGROUND), 0, 0, 1, 1},
  {&__pyx_n_s_AUDIODEVICEADDED, __pyx_k_AUDIODEVICEADDED, sizeof(__pyx_k_AUDIODEVICEADDED), 0, 0, 1, 1},
  {&__pyx_n_s_AUDIODEVICEREMOVED, __pyx_k_AUDIODEVICEREMOVED, sizeof(__pyx_k_AUDIODEVICEREMOVED), 0, 0, 1, 1},
  {&__pyx_n_s_BIG_ENDIAN, __pyx_k_BIG_ENDIAN, sizeof(__pyx_k_BIG_ENDIAN), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_ADD, __pyx_k_BLEND_ADD, sizeof(__pyx_k_BLEND_ADD), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_MAX, __pyx_k_BLEND_MAX, sizeof(__pyx_k_BLEND_MAX), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_MIN, __pyx_k_BLEND_MIN, sizeof(__pyx_k_BLEND_MIN), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_MULT, __pyx_k_BLEND_MULT, sizeof(__pyx_k_BLEND_MULT), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_PREMULTIPLIED, __pyx_k_BLEND_PREMULTIPLIED, sizeof(__pyx_k_BLEND_PREMULTIPLIED), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_RGBA_ADD, __pyx_k_BLEND_RGBA_ADD, sizeof(__pyx_k_BLEND_RGBA_ADD), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_RGBA_MAX, __pyx_k_BLEND_RGBA_MAX, sizeof(__pyx_k_BLEND_RGBA_MAX), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_RGBA_MIN, __pyx_k_BLEND_RGBA_MIN, sizeof(__pyx_k_BLEND_RGBA_MIN), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_RGBA_MULT, __pyx_k_BLEND_RGBA_MULT, sizeof(__pyx_k_BLEND_RGBA_MULT), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_RGBA_SUB, __pyx_k_BLEND_RGBA_SUB, sizeof(__pyx_k_BLEND_RGBA_SUB), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_RGB_ADD, __pyx_k_BLEND_RGB_ADD, sizeof(__pyx_k_BLEND_RGB_ADD), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_RGB_MAX, __pyx_k_BLEND_RGB_MAX, sizeof(__pyx_k_BLEND_RGB_MAX), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_RGB_MIN, __pyx_k_BLEND_RGB_MIN, sizeof(__pyx_k_BLEND_RGB_MIN), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_RGB_MULT, __pyx_k_BLEND_RGB_MULT, sizeof(__pyx_k_BLEND_RGB_MULT), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_RGB_SUB, __pyx_k_BLEND_RGB_SUB, sizeof(__pyx_k_BLEND_RGB_SUB), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_SUB, __pyx_k_BLEND_SUB, sizeof(__pyx_k_BLEND_SUB), 0, 0, 1, 1},
  {&__pyx_n_s_BYTEORDER, __pyx_k_BYTEORDER, sizeof(__pyx_k_BYTEORDER), 0, 0, 1, 1},
  {&__pyx_n_s_CLIPBOARDUPDATE, __pyx_k_CLIPBOARDUPDATE, sizeof(__pyx_k_CLIPBOARDUPDATE), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLERAXISMOTION, __pyx_k_CONTROLLERAXISMOTION, sizeof(__pyx_k_CONTROLLERAXISMOTION), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLERBUTTONDOWN, __pyx_k_CONTROLLERBUTTONDOWN, sizeof(__pyx_k_CONTROLLERBUTTONDOWN), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLERBUTTONUP, __pyx_k_CONTROLLERBUTTONUP, sizeof(__pyx_k_CONTROLLERBUTTONUP), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLERDEVICEADDED, __pyx_k_CONTROLLERDEVICEADDED, sizeof(__pyx_k_CONTROLLERDEVICEADDED), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLERDEVICEREMAPPED, __pyx_k_CONTROLLERDEVICEREMAPPED, sizeof(__pyx_k_CONTROLLERDEVICEREMAPPED), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLERDEVICEREMOVED, __pyx_k_CONTROLLERDEVICEREMOVED, sizeof(__pyx_k_CONTROLLERDEVICEREMOVED), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_AXIS_INVALID, __pyx_k_CONTROLLER_AXIS_INVALID, sizeof(__pyx_k_CONTROLLER_AXIS_INVALID), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_AXIS_LEFTX, __pyx_k_CONTROLLER_AXIS_LEFTX, sizeof(__pyx_k_CONTROLLER_AXIS_LEFTX), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_AXIS_LEFTY, __pyx_k_CONTROLLER_AXIS_LEFTY, sizeof(__pyx_k_CONTROLLER_AXIS_LEFTY), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_AXIS_MAX, __pyx_k_CONTROLLER_AXIS_MAX, sizeof(__pyx_k_CONTROLLER_AXIS_MAX), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_AXIS_RIGHTX, __pyx_k_CONTROLLER_AXIS_RIGHTX, sizeof(__pyx_k_CONTROLLER_AXIS_RIGHTX), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_AXIS_RIGHTY, __pyx_k_CONTROLLER_AXIS_RIGHTY, sizeof(__pyx_k_CONTROLLER_AXIS_RIGHTY), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_AXIS_TRIGGERLEFT, __pyx_k_CONTROLLER_AXIS_TRIGGERLEFT, sizeof(__pyx_k_CONTROLLER_AXIS_TRIGGERLEFT), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_AXIS_TRIGGERRIGHT, __pyx_k_CONTROLLER_AXIS_TRIGGERRIGHT, sizeof(__pyx_k_CONTROLLER_AXIS_TRIGGERRIGHT), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_A, __pyx_k_CONTROLLER_BUTTON_A, sizeof(__pyx_k_CONTROLLER_BUTTON_A), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_B, __pyx_k_CONTROLLER_BUTTON_B, sizeof(__pyx_k_CONTROLLER_BUTTON_B), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_BACK, __pyx_k_CONTROLLER_BUTTON_BACK, sizeof(__pyx_k_CONTROLLER_BUTTON_BACK), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_DPAD_DOWN, __pyx_k_CONTROLLER_BUTTON_DPAD_DOWN, sizeof(__pyx_k_CONTROLLER_BUTTON_DPAD_DOWN), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_DPAD_LEFT, __pyx_k_CONTROLLER_BUTTON_DPAD_LEFT, sizeof(__pyx_k_CONTROLLER_BUTTON_DPAD_LEFT), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_DPAD_RIGHT, __pyx_k_CONTROLLER_BUTTON_DPAD_RIGHT, sizeof(__pyx_k_CONTROLLER_BUTTON_DPAD_RIGHT), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_DPAD_UP, __pyx_k_CONTROLLER_BUTTON_DPAD_UP, sizeof(__pyx_k_CONTROLLER_BUTTON_DPAD_UP), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_GUIDE, __pyx_k_CONTROLLER_BUTTON_GUIDE, sizeof(__pyx_k_CONTROLLER_BUTTON_GUIDE), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_INVALID, __pyx_k_CONTROLLER_BUTTON_INVALID, sizeof(__pyx_k_CONTROLLER_BUTTON_INVALID), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_LEFTSHOULDER, __pyx_k_CONTROLLER_BUTTON_LEFTSHOULDER, sizeof(__pyx_k_CONTROLLER_BUTTON_LEFTSHOULDER), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_LEFTSTICK, __pyx_k_CONTROLLER_BUTTON_LEFTSTICK, sizeof(__pyx_k_CONTROLLER_BUTTON_LEFTSTICK), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_MAX, __pyx_k_CONTROLLER_BUTTON_MAX, sizeof(__pyx_k_CONTROLLER_BUTTON_MAX), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_RIGHTSHOULDER, __pyx_k_CONTROLLER_BUTTON_RIGHTSHOULDER, sizeof(__pyx_k_CONTROLLER_BUTTON_RIGHTSHOULDER), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_RIGHTSTICK, __pyx_k_CONTROLLER_BUTTON_RIGHTSTICK, sizeof(__pyx_k_CONTROLLER_BUTTON_RIGHTSTICK), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_START, __pyx_k_CONTROLLER_BUTTON_START, sizeof(__pyx_k_CONTROLLER_BUTTON_START), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_X, __pyx_k_CONTROLLER_BUTTON_X, sizeof(__pyx_k_CONTROLLER_BUTTON_X), 0, 0, 1, 1},
  {&__pyx_n_s_CONTROLLER_BUTTON_Y, __pyx_k_CONTROLLER_BUTTON_Y, sizeof(__pyx_k_CONTROLLER_BUTTON_Y), 0, 0, 1, 1},
  {&__pyx_n_s_Color, __pyx_k_Color, sizeof(__pyx_k_Color), 0, 0, 1, 1},
  {&__pyx_n_s_DISPLAYEVENT, __pyx_k_DISPLAYEVENT, sizeof(__pyx_k_DISPLAYEVENT), 0, 0, 1, 1},
  {&__pyx_n_s_DOLLARGESTURE, __pyx_k_DOLLARGESTURE, sizeof(__pyx_k_DOLLARGESTURE), 0, 0, 1, 1},
  {&__pyx_n_s_DOLLARRECORD, __pyx_k_DOLLARRECORD, sizeof(__pyx_k_DOLLARRECORD), 0, 0, 1, 1},
  {&__pyx_n_s_DOUBLEBUF, __pyx_k_DOUBLEBUF, sizeof(__pyx_k_DOUBLEBUF), 0, 0, 1, 1},
  {&__pyx_n_s_DROPBEGIN, __pyx_k_DROPBEGIN, sizeof(__pyx_k_DROPBEGIN), 0, 0, 1, 1},
  {&__pyx_n_s_DROPCOMPLETE, __pyx_k_DROPCOMPLETE, sizeof(__pyx_k_DROPCOMPLETE), 0, 0, 1, 1},
  {&__pyx_n_s_DROPFILE, __pyx_k_DROPFILE, sizeof(__pyx_k_DROPFILE), 0, 0, 1, 1},
  {&__pyx_n_s_DROPTEXT, __pyx_k_DROPTEXT, sizeof(__pyx_k_DROPTEXT), 0, 0, 1, 1},
  {&__pyx_n_s_FINGERDOWN, __pyx_k_FINGERDOWN, sizeof(__pyx_k_FINGERDOWN), 0, 0, 1, 1},
  {&__pyx_n_s_FINGERMOTION, __pyx_k_FINGERMOTION, sizeof(__pyx_k_FINGERMOTION), 0, 0, 1, 1},
  {&__pyx_n_s_FINGERUP, __pyx_k_FINGERUP, sizeof(__pyx_k_FINGERUP), 0, 0, 1, 1},
  {&__pyx_n_s_FULLSCREEN, __pyx_k_FULLSCREEN, sizeof(__pyx_k_FULLSCREEN), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ACCELERATED_VISUAL, __pyx_k_GL_ACCELERATED_VISUAL, sizeof(__pyx_k_GL_ACCELERATED_VISUAL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ACCUM_ALPHA_SIZE, __pyx_k_GL_ACCUM_ALPHA_SIZE, sizeof(__pyx_k_GL_ACCUM_ALPHA_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ACCUM_BLUE_SIZE, __pyx_k_GL_ACCUM_BLUE_SIZE, sizeof(__pyx_k_GL_ACCUM_BLUE_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ACCUM_GREEN_SIZE, __pyx_k_GL_ACCUM_GREEN_SIZE, sizeof(__pyx_k_GL_ACCUM_GREEN_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ACCUM_RED_SIZE, __pyx_k_GL_ACCUM_RED_SIZE, sizeof(__pyx_k_GL_ACCUM_RED_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ALPHA_SIZE, __pyx_k_GL_ALPHA_SIZE, sizeof(__pyx_k_GL_ALPHA_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLUE_SIZE, __pyx_k_GL_BLUE_SIZE, sizeof(__pyx_k_GL_BLUE_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BUFFER_SIZE, __pyx_k_GL_BUFFER_SIZE, sizeof(__pyx_k_GL_BUFFER_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_EGL, __pyx_k_GL_CONTEXT_EGL, sizeof(__pyx_k_GL_CONTEXT_EGL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_FLAGS, __pyx_k_GL_CONTEXT_FLAGS, sizeof(__pyx_k_GL_CONTEXT_FLAGS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_MAJOR_VERSION, __pyx_k_GL_CONTEXT_MAJOR_VERSION, sizeof(__pyx_k_GL_CONTEXT_MAJOR_VERSION), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_MINOR_VERSION, __pyx_k_GL_CONTEXT_MINOR_VERSION, sizeof(__pyx_k_GL_CONTEXT_MINOR_VERSION), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_NO_ERROR, __pyx_k_GL_CONTEXT_NO_ERROR, sizeof(__pyx_k_GL_CONTEXT_NO_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_PROFILE_COMPATIBILITY, __pyx_k_GL_CONTEXT_PROFILE_COMPATIBILITY, sizeof(__pyx_k_GL_CONTEXT_PROFILE_COMPATIBILITY), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_PROFILE_CORE, __pyx_k_GL_CONTEXT_PROFILE_CORE, sizeof(__pyx_k_GL_CONTEXT_PROFILE_CORE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_PROFILE_ES, __pyx_k_GL_CONTEXT_PROFILE_ES, sizeof(__pyx_k_GL_CONTEXT_PROFILE_ES), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_PROFILE_MASK, __pyx_k_GL_CONTEXT_PROFILE_MASK, sizeof(__pyx_k_GL_CONTEXT_PROFILE_MASK), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_RELEASE_BEHAVIOR, __pyx_k_GL_CONTEXT_RELEASE_BEHAVIOR, sizeof(__pyx_k_GL_CONTEXT_RELEASE_BEHAVIOR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONTEXT_RESET_NOTIFICATION, __pyx_k_GL_CONTEXT_RESET_NOTIFICATION, sizeof(__pyx_k_GL_CONTEXT_RESET_NOTIFICATION), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_SIZE, __pyx_k_GL_DEPTH_SIZE, sizeof(__pyx_k_GL_DEPTH_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DOUBLEBUFFER, __pyx_k_GL_DOUBLEBUFFER, sizeof(__pyx_k_GL_DOUBLEBUFFER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER_SRGB_CAPABLE, __pyx_k_GL_FRAMEBUFFER_SRGB_CAPABLE, sizeof(__pyx_k_GL_FRAMEBUFFER_SRGB_CAPABLE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_GREEN_SIZE, __pyx_k_GL_GREEN_SIZE, sizeof(__pyx_k_GL_GREEN_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MULTISAMPLEBUFFERS, __pyx_k_GL_MULTISAMPLEBUFFERS, sizeof(__pyx_k_GL_MULTISAMPLEBUFFERS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MULTISAMPLESAMPLES, __pyx_k_GL_MULTISAMPLESAMPLES, sizeof(__pyx_k_GL_MULTISAMPLESAMPLES), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RED_SIZE, __pyx_k_GL_RED_SIZE, sizeof(__pyx_k_GL_RED_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RETAINED_BACKING, __pyx_k_GL_RETAINED_BACKING, sizeof(__pyx_k_GL_RETAINED_BACKING), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SHARE_WITH_CURRENT_CONTEXT, __pyx_k_GL_SHARE_WITH_CURRENT_CONTEXT, sizeof(__pyx_k_GL_SHARE_WITH_CURRENT_CONTEXT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_SIZE, __pyx_k_GL_STENCIL_SIZE, sizeof(__pyx_k_GL_STENCIL_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STEREO, __pyx_k_GL_STEREO, sizeof(__pyx_k_GL_STEREO), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SWAP_CONTROL, __pyx_k_GL_SWAP_CONTROL, sizeof(__pyx_k_GL_SWAP_CONTROL), 0, 0, 1, 1},
  {&__pyx_n_s_HWSURFACE, __pyx_k_HWSURFACE, sizeof(__pyx_k_HWSURFACE), 0, 0, 1, 1},
  {&__pyx_n_s_JOYAXISMOTION, __pyx_k_JOYAXISMOTION, sizeof(__pyx_k_JOYAXISMOTION), 0, 0, 1, 1},
  {&__pyx_n_s_JOYBALLMOTION, __pyx_k_JOYBALLMOTION, sizeof(__pyx_k_JOYBALLMOTION), 0, 0, 1, 1},
  {&__pyx_n_s_JOYBUTTONDOWN, __pyx_k_JOYBUTTONDOWN, sizeof(__pyx_k_JOYBUTTONDOWN), 0, 0, 1, 1},
  {&__pyx_n_s_JOYBUTTONUP, __pyx_k_JOYBUTTONUP, sizeof(__pyx_k_JOYBUTTONUP), 0, 0, 1, 1},
  {&__pyx_n_s_JOYDEVICEADDED, __pyx_k_JOYDEVICEADDED, sizeof(__pyx_k_JOYDEVICEADDED), 0, 0, 1, 1},
  {&__pyx_n_s_JOYDEVICEREMOVED, __pyx_k_JOYDEVICEREMOVED, sizeof(__pyx_k_JOYDEVICEREMOVED), 0, 0, 1, 1},
  {&__pyx_n_s_JOYHATMOTION, __pyx_k_JOYHATMOTION, sizeof(__pyx_k_JOYHATMOTION), 0, 0, 1, 1},
  {&__pyx_n_s_KEYDOWN, __pyx_k_KEYDOWN, sizeof(__pyx_k_KEYDOWN), 0, 0, 1, 1},
  {&__pyx_n_s_KEYMAPCHANGED, __pyx_k_KEYMAPCHANGED, sizeof(__pyx_k_KEYMAPCHANGED), 0, 0, 1, 1},
  {&__pyx_n_s_KEYUP, __pyx_k_KEYUP, sizeof(__pyx_k_KEYUP), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_ALT, __pyx_k_KMOD_ALT, sizeof(__pyx_k_KMOD_ALT), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_CAPS, __pyx_k_KMOD_CAPS, sizeof(__pyx_k_KMOD_CAPS), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_CTRL, __pyx_k_KMOD_CTRL, sizeof(__pyx_k_KMOD_CTRL), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_GUI, __pyx_k_KMOD_GUI, sizeof(__pyx_k_KMOD_GUI), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_LALT, __pyx_k_KMOD_LALT, sizeof(__pyx_k_KMOD_LALT), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_LCTRL, __pyx_k_KMOD_LCTRL, sizeof(__pyx_k_KMOD_LCTRL), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_LGUI, __pyx_k_KMOD_LGUI, sizeof(__pyx_k_KMOD_LGUI), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_LMETA, __pyx_k_KMOD_LMETA, sizeof(__pyx_k_KMOD_LMETA), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_LSHIFT, __pyx_k_KMOD_LSHIFT, sizeof(__pyx_k_KMOD_LSHIFT), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_META, __pyx_k_KMOD_META, sizeof(__pyx_k_KMOD_META), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_MODE, __pyx_k_KMOD_MODE, sizeof(__pyx_k_KMOD_MODE), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_NONE, __pyx_k_KMOD_NONE, sizeof(__pyx_k_KMOD_NONE), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_NUM, __pyx_k_KMOD_NUM, sizeof(__pyx_k_KMOD_NUM), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_RALT, __pyx_k_KMOD_RALT, sizeof(__pyx_k_KMOD_RALT), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_RCTRL, __pyx_k_KMOD_RCTRL, sizeof(__pyx_k_KMOD_RCTRL), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_RGUI, __pyx_k_KMOD_RGUI, sizeof(__pyx_k_KMOD_RGUI), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_RMETA, __pyx_k_KMOD_RMETA, sizeof(__pyx_k_KMOD_RMETA), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_RSHIFT, __pyx_k_KMOD_RSHIFT, sizeof(__pyx_k_KMOD_RSHIFT), 0, 0, 1, 1},
  {&__pyx_n_s_KMOD_SHIFT, __pyx_k_KMOD_SHIFT, sizeof(__pyx_k_KMOD_SHIFT), 0, 0, 1, 1},
  {&__pyx_n_s_K_0, __pyx_k_K_0, sizeof(__pyx_k_K_0), 0, 0, 1, 1},
  {&__pyx_n_s_K_1, __pyx_k_K_1, sizeof(__pyx_k_K_1), 0, 0, 1, 1},
  {&__pyx_n_s_K_2, __pyx_k_K_2, sizeof(__pyx_k_K_2), 0, 0, 1, 1},
  {&__pyx_n_s_K_3, __pyx_k_K_3, sizeof(__pyx_k_K_3), 0, 0, 1, 1},
  {&__pyx_n_s_K_4, __pyx_k_K_4, sizeof(__pyx_k_K_4), 0, 0, 1, 1},
  {&__pyx_n_s_K_5, __pyx_k_K_5, sizeof(__pyx_k_K_5), 0, 0, 1, 1},
  {&__pyx_n_s_K_6, __pyx_k_K_6, sizeof(__pyx_k_K_6), 0, 0, 1, 1},
  {&__pyx_n_s_K_7, __pyx_k_K_7, sizeof(__pyx_k_K_7), 0, 0, 1, 1},
  {&__pyx_n_s_K_8, __pyx_k_K_8, sizeof(__pyx_k_K_8), 0, 0, 1, 1},
  {&__pyx_n_s_K_9, __pyx_k_K_9, sizeof(__pyx_k_K_9), 0, 0, 1, 1},
  {&__pyx_n_s_K_AC_BACK, __pyx_k_K_AC_BACK, sizeof(__pyx_k_K_AC_BACK), 0, 0, 1, 1},
  {&__pyx_n_s_K_AC_BOOKMARKS, __pyx_k_K_AC_BOOKMARKS, sizeof(__pyx_k_K_AC_BOOKMARKS), 0, 0, 1, 1},
  {&__pyx_n_s_K_AC_FORWARD, __pyx_k_K_AC_FORWARD, sizeof(__pyx_k_K_AC_FORWARD), 0, 0, 1, 1},
  {&__pyx_n_s_K_AC_HOME, __pyx_k_K_AC_HOME, sizeof(__pyx_k_K_AC_HOME), 0, 0, 1, 1},
  {&__pyx_n_s_K_AC_REFRESH, __pyx_k_K_AC_REFRESH, sizeof(__pyx_k_K_AC_REFRESH), 0, 0, 1, 1},
  {&__pyx_n_s_K_AC_SEARCH, __pyx_k_K_AC_SEARCH, sizeof(__pyx_k_K_AC_SEARCH), 0, 0, 1, 1},
  {&__pyx_n_s_K_AC_STOP, __pyx_k_K_AC_STOP, sizeof(__pyx_k_K_AC_STOP), 0, 0, 1, 1},
  {&__pyx_n_s_K_AGAIN, __pyx_k_K_AGAIN, sizeof(__pyx_k_K_AGAIN), 0, 0, 1, 1},
  {&__pyx_n_s_K_ALTERASE, __pyx_k_K_ALTERASE, sizeof(__pyx_k_K_ALTERASE), 0, 0, 1, 1},
  {&__pyx_n_s_K_AMPERSAND, __pyx_k_K_AMPERSAND, sizeof(__pyx_k_K_AMPERSAND), 0, 0, 1, 1},
  {&__pyx_n_s_K_APP1, __pyx_k_K_APP1, sizeof(__pyx_k_K_APP1), 0, 0, 1, 1},
  {&__pyx_n_s_K_APP2, __pyx_k_K_APP2, sizeof(__pyx_k_K_APP2), 0, 0, 1, 1},
  {&__pyx_n_s_K_APPLICATION, __pyx_k_K_APPLICATION, sizeof(__pyx_k_K_APPLICATION), 0, 0, 1, 1},
  {&__pyx_n_s_K_ASTERISK, __pyx_k_K_ASTERISK, sizeof(__pyx_k_K_ASTERISK), 0, 0, 1, 1},
  {&__pyx_n_s_K_AT, __pyx_k_K_AT, sizeof(__pyx_k_K_AT), 0, 0, 1, 1},
  {&__pyx_n_s_K_AUDIOFASTFORWARD, __pyx_k_K_AUDIOFASTFORWARD, sizeof(__pyx_k_K_AUDIOFASTFORWARD), 0, 0, 1, 1},
  {&__pyx_n_s_K_AUDIOMUTE, __pyx_k_K_AUDIOMUTE, sizeof(__pyx_k_K_AUDIOMUTE), 0, 0, 1, 1},
  {&__pyx_n_s_K_AUDIONEXT, __pyx_k_K_AUDIONEXT, sizeof(__pyx_k_K_AUDIONEXT), 0, 0, 1, 1},
  {&__pyx_n_s_K_AUDIOPLAY, __pyx_k_K_AUDIOPLAY, sizeof(__pyx_k_K_AUDIOPLAY), 0, 0, 1, 1},
  {&__pyx_n_s_K_AUDIOPREV, __pyx_k_K_AUDIOPREV, sizeof(__pyx_k_K_AUDIOPREV), 0, 0, 1, 1},
  {&__pyx_n_s_K_AUDIOREWIND, __pyx_k_K_AUDIOREWIND, sizeof(__pyx_k_K_AUDIOREWIND), 0, 0, 1, 1},
  {&__pyx_n_s_K_AUDIOSTOP, __pyx_k_K_AUDIOSTOP, sizeof(__pyx_k_K_AUDIOSTOP), 0, 0, 1, 1},
  {&__pyx_n_s_K_BACKQUOTE, __pyx_k_K_BACKQUOTE, sizeof(__pyx_k_K_BACKQUOTE), 0, 0, 1, 1},
  {&__pyx_n_s_K_BACKSLASH, __pyx_k_K_BACKSLASH, sizeof(__pyx_k_K_BACKSLASH), 0, 0, 1, 1},
  {&__pyx_n_s_K_BACKSPACE, __pyx_k_K_BACKSPACE, sizeof(__pyx_k_K_BACKSPACE), 0, 0, 1, 1},
  {&__pyx_n_s_K_BREAK, __pyx_k_K_BREAK, sizeof(__pyx_k_K_BREAK), 0, 0, 1, 1},
  {&__pyx_n_s_K_BRIGHTNESSDOWN, __pyx_k_K_BRIGHTNESSDOWN, sizeof(__pyx_k_K_BRIGHTNESSDOWN), 0, 0, 1, 1},
  {&__pyx_n_s_K_BRIGHTNESSUP, __pyx_k_K_BRIGHTNESSUP, sizeof(__pyx_k_K_BRIGHTNESSUP), 0, 0, 1, 1},
  {&__pyx_n_s_K_CALCULATOR, __pyx_k_K_CALCULATOR, sizeof(__pyx_k_K_CALCULATOR), 0, 0, 1, 1},
  {&__pyx_n_s_K_CANCEL, __pyx_k_K_CANCEL, sizeof(__pyx_k_K_CANCEL), 0, 0, 1, 1},
  {&__pyx_n_s_K_CAPSLOCK, __pyx_k_K_CAPSLOCK, sizeof(__pyx_k_K_CAPSLOCK), 0, 0, 1, 1},
  {&__pyx_n_s_K_CARET, __pyx_k_K_CARET, sizeof(__pyx_k_K_CARET), 0, 0, 1, 1},
  {&__pyx_n_s_K_CLEAR, __pyx_k_K_CLEAR, sizeof(__pyx_k_K_CLEAR), 0, 0, 1, 1},
  {&__pyx_n_s_K_CLEARAGAIN, __pyx_k_K_CLEARAGAIN, sizeof(__pyx_k_K_CLEARAGAIN), 0, 0, 1, 1},
  {&__pyx_n_s_K_COLON, __pyx_k_K_COLON, sizeof(__pyx_k_K_COLON), 0, 0, 1, 1},
  {&__pyx_n_s_K_COMMA, __pyx_k_K_COMMA, sizeof(__pyx_k_K_COMMA), 0, 0, 1, 1},
  {&__pyx_n_s_K_COMPUTER, __pyx_k_K_COMPUTER, sizeof(__pyx_k_K_COMPUTER), 0, 0, 1, 1},
  {&__pyx_n_s_K_COPY, __pyx_k_K_COPY, sizeof(__pyx_k_K_COPY), 0, 0, 1, 1},
  {&__pyx_n_s_K_CRSEL, __pyx_k_K_CRSEL, sizeof(__pyx_k_K_CRSEL), 0, 0, 1, 1},
  {&__pyx_n_s_K_CURRENCYSUBUNIT, __pyx_k_K_CURRENCYSUBUNIT, sizeof(__pyx_k_K_CURRENCYSUBUNIT), 0, 0, 1, 1},
  {&__pyx_n_s_K_CURRENCYUNIT, __pyx_k_K_CURRENCYUNIT, sizeof(__pyx_k_K_CURRENCYUNIT), 0, 0, 1, 1},
  {&__pyx_n_s_K_CUT, __pyx_k_K_CUT, sizeof(__pyx_k_K_CUT), 0, 0, 1, 1},
  {&__pyx_n_s_K_DECIMALSEPARATOR, __pyx_k_K_DECIMALSEPARATOR, sizeof(__pyx_k_K_DECIMALSEPARATOR), 0, 0, 1, 1},
  {&__pyx_n_s_K_DELETE, __pyx_k_K_DELETE, sizeof(__pyx_k_K_DELETE), 0, 0, 1, 1},
  {&__pyx_n_s_K_DISPLAYSWITCH, __pyx_k_K_DISPLAYSWITCH, sizeof(__pyx_k_K_DISPLAYSWITCH), 0, 0, 1, 1},
  {&__pyx_n_s_K_DOLLAR, __pyx_k_K_DOLLAR, sizeof(__pyx_k_K_DOLLAR), 0, 0, 1, 1},
  {&__pyx_n_s_K_DOWN, __pyx_k_K_DOWN, sizeof(__pyx_k_K_DOWN), 0, 0, 1, 1},
  {&__pyx_n_s_K_EJECT, __pyx_k_K_EJECT, sizeof(__pyx_k_K_EJECT), 0, 0, 1, 1},
  {&__pyx_n_s_K_END, __pyx_k_K_END, sizeof(__pyx_k_K_END), 0, 0, 1, 1},
  {&__pyx_n_s_K_EQUALS, __pyx_k_K_EQUALS, sizeof(__pyx_k_K_EQUALS), 0, 0, 1, 1},
  {&__pyx_n_s_K_ESCAPE, __pyx_k_K_ESCAPE, sizeof(__pyx_k_K_ESCAPE), 0, 0, 1, 1},
  {&__pyx_n_s_K_EURO, __pyx_k_K_EURO, sizeof(__pyx_k_K_EURO), 0, 0, 1, 1},
  {&__pyx_n_s_K_EXCLAIM, __pyx_k_K_EXCLAIM, sizeof(__pyx_k_K_EXCLAIM), 0, 0, 1, 1},
  {&__pyx_n_s_K_EXECUTE, __pyx_k_K_EXECUTE, sizeof(__pyx_k_K_EXECUTE), 0, 0, 1, 1},
  {&__pyx_n_s_K_EXSEL, __pyx_k_K_EXSEL, sizeof(__pyx_k_K_EXSEL), 0, 0, 1, 1},
  {&__pyx_n_s_K_F1, __pyx_k_K_F1, sizeof(__pyx_k_K_F1), 0, 0, 1, 1},
  {&__pyx_n_s_K_F10, __pyx_k_K_F10, sizeof(__pyx_k_K_F10), 0, 0, 1, 1},
  {&__pyx_n_s_K_F11, __pyx_k_K_F11, sizeof(__pyx_k_K_F11), 0, 0, 1, 1},
  {&__pyx_n_s_K_F12, __pyx_k_K_F12, sizeof(__pyx_k_K_F12), 0, 0, 1, 1},
  {&__pyx_n_s_K_F13, __pyx_k_K_F13, sizeof(__pyx_k_K_F13), 0, 0, 1, 1},
  {&__pyx_n_s_K_F14, __pyx_k_K_F14, sizeof(__pyx_k_K_F14), 0, 0, 1, 1},
  {&__pyx_n_s_K_F15, __pyx_k_K_F15, sizeof(__pyx_k_K_F15), 0, 0, 1, 1},
  {&__pyx_n_s_K_F16, __pyx_k_K_F16, sizeof(__pyx_k_K_F16), 0, 0, 1, 1},
  {&__pyx_n_s_K_F17, __pyx_k_K_F17, sizeof(__pyx_k_K_F17), 0, 0, 1, 1},
  {&__pyx_n_s_K_F18, __pyx_k_K_F18, sizeof(__pyx_k_K_F18), 0, 0, 1, 1},
  {&__pyx_n_s_K_F19, __pyx_k_K_F19, sizeof(__pyx_k_K_F19), 0, 0, 1, 1},
  {&__pyx_n_s_K_F2, __pyx_k_K_F2, sizeof(__pyx_k_K_F2), 0, 0, 1, 1},
  {&__pyx_n_s_K_F20, __pyx_k_K_F20, sizeof(__pyx_k_K_F20), 0, 0, 1, 1},
  {&__pyx_n_s_K_F21, __pyx_k_K_F21, sizeof(__pyx_k_K_F21), 0, 0, 1, 1},
  {&__pyx_n_s_K_F22, __pyx_k_K_F22, sizeof(__pyx_k_K_F22), 0, 0, 1, 1},
  {&__pyx_n_s_K_F23, __pyx_k_K_F23, sizeof(__pyx_k_K_F23), 0, 0, 1, 1},
  {&__pyx_n_s_K_F24, __pyx_k_K_F24, sizeof(__pyx_k_K_F24), 0, 0, 1, 1},
  {&__pyx_n_s_K_F3, __pyx_k_K_F3, sizeof(__pyx_k_K_F3), 0, 0, 1, 1},
  {&__pyx_n_s_K_F4, __pyx_k_K_F4, sizeof(__pyx_k_K_F4), 0, 0, 1, 1},
  {&__pyx_n_s_K_F5, __pyx_k_K_F5, sizeof(__pyx_k_K_F5), 0, 0, 1, 1},
  {&__pyx_n_s_K_F6, __pyx_k_K_F6, sizeof(__pyx_k_K_F6), 0, 0, 1, 1},
  {&__pyx_n_s_K_F7, __pyx_k_K_F7, sizeof(__pyx_k_K_F7), 0, 0, 1, 1},
  {&__pyx_n_s_K_F8, __pyx_k_K_F8, sizeof(__pyx_k_K_F8), 0, 0, 1, 1},
  {&__pyx_n_s_K_F9, __pyx_k_K_F9, sizeof(__pyx_k_K_F9), 0, 0, 1, 1},
  {&__pyx_n_s_K_FIND, __pyx_k_K_FIND, sizeof(__pyx_k_K_FIND), 0, 0, 1, 1},
  {&__pyx_n_s_K_FIRST, __pyx_k_K_FIRST, sizeof(__pyx_k_K_FIRST), 0, 0, 1, 1},
  {&__pyx_n_s_K_GREATER, __pyx_k_K_GREATER, sizeof(__pyx_k_K_GREATER), 0, 0, 1, 1},
  {&__pyx_n_s_K_HASH, __pyx_k_K_HASH, sizeof(__pyx_k_K_HASH), 0, 0, 1, 1},
  {&__pyx_n_s_K_HELP, __pyx_k_K_HELP, sizeof(__pyx_k_K_HELP), 0, 0, 1, 1},
  {&__pyx_n_s_K_HOME, __pyx_k_K_HOME, sizeof(__pyx_k_K_HOME), 0, 0, 1, 1},
  {&__pyx_n_s_K_INSERT, __pyx_k_K_INSERT, sizeof(__pyx_k_K_INSERT), 0, 0, 1, 1},
  {&__pyx_n_s_K_KBDILLUMDOWN, __pyx_k_K_KBDILLUMDOWN, sizeof(__pyx_k_K_KBDILLUMDOWN), 0, 0, 1, 1},
  {&__pyx_n_s_K_KBDILLUMTOGGLE, __pyx_k_K_KBDILLUMTOGGLE, sizeof(__pyx_k_K_KBDILLUMTOGGLE), 0, 0, 1, 1},
  {&__pyx_n_s_K_KBDILLUMUP, __pyx_k_K_KBDILLUMUP, sizeof(__pyx_k_K_KBDILLUMUP), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP0, __pyx_k_K_KP0, sizeof(__pyx_k_K_KP0), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP1, __pyx_k_K_KP1, sizeof(__pyx_k_K_KP1), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP2, __pyx_k_K_KP2, sizeof(__pyx_k_K_KP2), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP3, __pyx_k_K_KP3, sizeof(__pyx_k_K_KP3), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP4, __pyx_k_K_KP4, sizeof(__pyx_k_K_KP4), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP5, __pyx_k_K_KP5, sizeof(__pyx_k_K_KP5), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP6, __pyx_k_K_KP6, sizeof(__pyx_k_K_KP6), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP7, __pyx_k_K_KP7, sizeof(__pyx_k_K_KP7), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP8, __pyx_k_K_KP8, sizeof(__pyx_k_K_KP8), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP9, __pyx_k_K_KP9, sizeof(__pyx_k_K_KP9), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_0, __pyx_k_K_KP_0, sizeof(__pyx_k_K_KP_0), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_00, __pyx_k_K_KP_00, sizeof(__pyx_k_K_KP_00), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_000, __pyx_k_K_KP_000, sizeof(__pyx_k_K_KP_000), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_1, __pyx_k_K_KP_1, sizeof(__pyx_k_K_KP_1), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_2, __pyx_k_K_KP_2, sizeof(__pyx_k_K_KP_2), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_3, __pyx_k_K_KP_3, sizeof(__pyx_k_K_KP_3), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_4, __pyx_k_K_KP_4, sizeof(__pyx_k_K_KP_4), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_5, __pyx_k_K_KP_5, sizeof(__pyx_k_K_KP_5), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_6, __pyx_k_K_KP_6, sizeof(__pyx_k_K_KP_6), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_7, __pyx_k_K_KP_7, sizeof(__pyx_k_K_KP_7), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_8, __pyx_k_K_KP_8, sizeof(__pyx_k_K_KP_8), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_9, __pyx_k_K_KP_9, sizeof(__pyx_k_K_KP_9), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_A, __pyx_k_K_KP_A, sizeof(__pyx_k_K_KP_A), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_AMPERSAND, __pyx_k_K_KP_AMPERSAND, sizeof(__pyx_k_K_KP_AMPERSAND), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_AT, __pyx_k_K_KP_AT, sizeof(__pyx_k_K_KP_AT), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_B, __pyx_k_K_KP_B, sizeof(__pyx_k_K_KP_B), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_BACKSPACE, __pyx_k_K_KP_BACKSPACE, sizeof(__pyx_k_K_KP_BACKSPACE), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_BINARY, __pyx_k_K_KP_BINARY, sizeof(__pyx_k_K_KP_BINARY), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_C, __pyx_k_K_KP_C, sizeof(__pyx_k_K_KP_C), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_CLEAR, __pyx_k_K_KP_CLEAR, sizeof(__pyx_k_K_KP_CLEAR), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_CLEARENTRY, __pyx_k_K_KP_CLEARENTRY, sizeof(__pyx_k_K_KP_CLEARENTRY), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_COLON, __pyx_k_K_KP_COLON, sizeof(__pyx_k_K_KP_COLON), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_COMMA, __pyx_k_K_KP_COMMA, sizeof(__pyx_k_K_KP_COMMA), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_D, __pyx_k_K_KP_D, sizeof(__pyx_k_K_KP_D), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_DBLAMPERSAND, __pyx_k_K_KP_DBLAMPERSAND, sizeof(__pyx_k_K_KP_DBLAMPERSAND), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_DBLVERTICALBAR, __pyx_k_K_KP_DBLVERTICALBAR, sizeof(__pyx_k_K_KP_DBLVERTICALBAR), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_DECIMAL, __pyx_k_K_KP_DECIMAL, sizeof(__pyx_k_K_KP_DECIMAL), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_DIVIDE, __pyx_k_K_KP_DIVIDE, sizeof(__pyx_k_K_KP_DIVIDE), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_E, __pyx_k_K_KP_E, sizeof(__pyx_k_K_KP_E), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_ENTER, __pyx_k_K_KP_ENTER, sizeof(__pyx_k_K_KP_ENTER), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_EQUALS, __pyx_k_K_KP_EQUALS, sizeof(__pyx_k_K_KP_EQUALS), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_EQUALSAS400, __pyx_k_K_KP_EQUALSAS400, sizeof(__pyx_k_K_KP_EQUALSAS400), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_EXCLAM, __pyx_k_K_KP_EXCLAM, sizeof(__pyx_k_K_KP_EXCLAM), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_F, __pyx_k_K_KP_F, sizeof(__pyx_k_K_KP_F), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_GREATER, __pyx_k_K_KP_GREATER, sizeof(__pyx_k_K_KP_GREATER), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_HASH, __pyx_k_K_KP_HASH, sizeof(__pyx_k_K_KP_HASH), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_HEXADECIMAL, __pyx_k_K_KP_HEXADECIMAL, sizeof(__pyx_k_K_KP_HEXADECIMAL), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_LEFTBRACE, __pyx_k_K_KP_LEFTBRACE, sizeof(__pyx_k_K_KP_LEFTBRACE), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_LEFTPAREN, __pyx_k_K_KP_LEFTPAREN, sizeof(__pyx_k_K_KP_LEFTPAREN), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_LESS, __pyx_k_K_KP_LESS, sizeof(__pyx_k_K_KP_LESS), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_MEMADD, __pyx_k_K_KP_MEMADD, sizeof(__pyx_k_K_KP_MEMADD), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_MEMCLEAR, __pyx_k_K_KP_MEMCLEAR, sizeof(__pyx_k_K_KP_MEMCLEAR), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_MEMDIVIDE, __pyx_k_K_KP_MEMDIVIDE, sizeof(__pyx_k_K_KP_MEMDIVIDE), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_MEMMULTIPLY, __pyx_k_K_KP_MEMMULTIPLY, sizeof(__pyx_k_K_KP_MEMMULTIPLY), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_MEMRECALL, __pyx_k_K_KP_MEMRECALL, sizeof(__pyx_k_K_KP_MEMRECALL), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_MEMSTORE, __pyx_k_K_KP_MEMSTORE, sizeof(__pyx_k_K_KP_MEMSTORE), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_MEMSUBTRACT, __pyx_k_K_KP_MEMSUBTRACT, sizeof(__pyx_k_K_KP_MEMSUBTRACT), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_MINUS, __pyx_k_K_KP_MINUS, sizeof(__pyx_k_K_KP_MINUS), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_MULTIPLY, __pyx_k_K_KP_MULTIPLY, sizeof(__pyx_k_K_KP_MULTIPLY), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_OCTAL, __pyx_k_K_KP_OCTAL, sizeof(__pyx_k_K_KP_OCTAL), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_PERCENT, __pyx_k_K_KP_PERCENT, sizeof(__pyx_k_K_KP_PERCENT), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_PERIOD, __pyx_k_K_KP_PERIOD, sizeof(__pyx_k_K_KP_PERIOD), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_PLUS, __pyx_k_K_KP_PLUS, sizeof(__pyx_k_K_KP_PLUS), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_PLUSMINUS, __pyx_k_K_KP_PLUSMINUS, sizeof(__pyx_k_K_KP_PLUSMINUS), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_POWER, __pyx_k_K_KP_POWER, sizeof(__pyx_k_K_KP_POWER), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_RIGHTBRACE, __pyx_k_K_KP_RIGHTBRACE, sizeof(__pyx_k_K_KP_RIGHTBRACE), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_RIGHTPAREN, __pyx_k_K_KP_RIGHTPAREN, sizeof(__pyx_k_K_KP_RIGHTPAREN), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_SPACE, __pyx_k_K_KP_SPACE, sizeof(__pyx_k_K_KP_SPACE), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_TAB, __pyx_k_K_KP_TAB, sizeof(__pyx_k_K_KP_TAB), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_VERTICALBAR, __pyx_k_K_KP_VERTICALBAR, sizeof(__pyx_k_K_KP_VERTICALBAR), 0, 0, 1, 1},
  {&__pyx_n_s_K_KP_XOR, __pyx_k_K_KP_XOR, sizeof(__pyx_k_K_KP_XOR), 0, 0, 1, 1},
  {&__pyx_n_s_K_LALT, __pyx_k_K_LALT, sizeof(__pyx_k_K_LALT), 0, 0, 1, 1},
  {&__pyx_n_s_K_LAST, __pyx_k_K_LAST, sizeof(__pyx_k_K_LAST), 0, 0, 1, 1},
  {&__pyx_n_s_K_LCTRL, __pyx_k_K_LCTRL, sizeof(__pyx_k_K_LCTRL), 0, 0, 1, 1},
  {&__pyx_n_s_K_LEFT, __pyx_k_K_LEFT, sizeof(__pyx_k_K_LEFT), 0, 0, 1, 1},
  {&__pyx_n_s_K_LEFTBRACKET, __pyx_k_K_LEFTBRACKET, sizeof(__pyx_k_K_LEFTBRACKET), 0, 0, 1, 1},
  {&__pyx_n_s_K_LEFTPAREN, __pyx_k_K_LEFTPAREN, sizeof(__pyx_k_K_LEFTPAREN), 0, 0, 1, 1},
  {&__pyx_n_s_K_LESS, __pyx_k_K_LESS, sizeof(__pyx_k_K_LESS), 0, 0, 1, 1},
  {&__pyx_n_s_K_LGUI, __pyx_k_K_LGUI, sizeof(__pyx_k_K_LGUI), 0, 0, 1, 1},
  {&__pyx_n_s_K_LMETA, __pyx_k_K_LMETA, sizeof(__pyx_k_K_LMETA), 0, 0, 1, 1},
  {&__pyx_n_s_K_LSHIFT, __pyx_k_K_LSHIFT, sizeof(__pyx_k_K_LSHIFT), 0, 0, 1, 1},
  {&__pyx_n_s_K_LSUPER, __pyx_k_K_LSUPER, sizeof(__pyx_k_K_LSUPER), 0, 0, 1, 1},
  {&__pyx_n_s_K_MAIL, __pyx_k_K_MAIL, sizeof(__pyx_k_K_MAIL), 0, 0, 1, 1},
  {&__pyx_n_s_K_MEDIASELECT, __pyx_k_K_MEDIASELECT, sizeof(__pyx_k_K_MEDIASELECT), 0, 0, 1, 1},
  {&__pyx_n_s_K_MENU, __pyx_k_K_MENU, sizeof(__pyx_k_K_MENU), 0, 0, 1, 1},
  {&__pyx_n_s_K_MINUS, __pyx_k_K_MINUS, sizeof(__pyx_k_K_MINUS), 0, 0, 1, 1},
  {&__pyx_n_s_K_MODE, __pyx_k_K_MODE, sizeof(__pyx_k_K_MODE), 0, 0, 1, 1},
  {&__pyx_n_s_K_MUTE, __pyx_k_K_MUTE, sizeof(__pyx_k_K_MUTE), 0, 0, 1, 1},
  {&__pyx_n_s_K_NUMLOCK, __pyx_k_K_NUMLOCK, sizeof(__pyx_k_K_NUMLOCK), 0, 0, 1, 1},
  {&__pyx_n_s_K_NUMLOCKCLEAR, __pyx_k_K_NUMLOCKCLEAR, sizeof(__pyx_k_K_NUMLOCKCLEAR), 0, 0, 1, 1},
  {&__pyx_n_s_K_OPER, __pyx_k_K_OPER, sizeof(__pyx_k_K_OPER), 0, 0, 1, 1},
  {&__pyx_n_s_K_OUT, __pyx_k_K_OUT, sizeof(__pyx_k_K_OUT), 0, 0, 1, 1},
  {&__pyx_n_s_K_PAGEDOWN, __pyx_k_K_PAGEDOWN, sizeof(__pyx_k_K_PAGEDOWN), 0, 0, 1, 1},
  {&__pyx_n_s_K_PAGEUP, __pyx_k_K_PAGEUP, sizeof(__pyx_k_K_PAGEUP), 0, 0, 1, 1},
  {&__pyx_n_s_K_PASTE, __pyx_k_K_PASTE, sizeof(__pyx_k_K_PASTE), 0, 0, 1, 1},
  {&__pyx_n_s_K_PAUSE, __pyx_k_K_PAUSE, sizeof(__pyx_k_K_PAUSE), 0, 0, 1, 1},
  {&__pyx_n_s_K_PERCENT, __pyx_k_K_PERCENT, sizeof(__pyx_k_K_PERCENT), 0, 0, 1, 1},
  {&__pyx_n_s_K_PERIOD, __pyx_k_K_PERIOD, sizeof(__pyx_k_K_PERIOD), 0, 0, 1, 1},
  {&__pyx_n_s_K_PLUS, __pyx_k_K_PLUS, sizeof(__pyx_k_K_PLUS), 0, 0, 1, 1},
  {&__pyx_n_s_K_POWER, __pyx_k_K_POWER, sizeof(__pyx_k_K_POWER), 0, 0, 1, 1},
  {&__pyx_n_s_K_PRINT, __pyx_k_K_PRINT, sizeof(__pyx_k_K_PRINT), 0, 0, 1, 1},
  {&__pyx_n_s_K_PRINTSCREEN, __pyx_k_K_PRINTSCREEN, sizeof(__pyx_k_K_PRINTSCREEN), 0, 0, 1, 1},
  {&__pyx_n_s_K_PRIOR, __pyx_k_K_PRIOR, sizeof(__pyx_k_K_PRIOR), 0, 0, 1, 1},
  {&__pyx_n_s_K_QUESTION, __pyx_k_K_QUESTION, sizeof(__pyx_k_K_QUESTION), 0, 0, 1, 1},
  {&__pyx_n_s_K_QUOTE, __pyx_k_K_QUOTE, sizeof(__pyx_k_K_QUOTE), 0, 0, 1, 1},
  {&__pyx_n_s_K_QUOTEDBL, __pyx_k_K_QUOTEDBL, sizeof(__pyx_k_K_QUOTEDBL), 0, 0, 1, 1},
  {&__pyx_n_s_K_RALT, __pyx_k_K_RALT, sizeof(__pyx_k_K_RALT), 0, 0, 1, 1},
  {&__pyx_n_s_K_RCTRL, __pyx_k_K_RCTRL, sizeof(__pyx_k_K_RCTRL), 0, 0, 1, 1},
  {&__pyx_n_s_K_RETURN, __pyx_k_K_RETURN, sizeof(__pyx_k_K_RETURN), 0, 0, 1, 1},
  {&__pyx_n_s_K_RETURN2, __pyx_k_K_RETURN2, sizeof(__pyx_k_K_RETURN2), 0, 0, 1, 1},
  {&__pyx_n_s_K_RGUI, __pyx_k_K_RGUI, sizeof(__pyx_k_K_RGUI), 0, 0, 1, 1},
  {&__pyx_n_s_K_RIGHT, __pyx_k_K_RIGHT, sizeof(__pyx_k_K_RIGHT), 0, 0, 1, 1},
  {&__pyx_n_s_K_RIGHTBRACKET, __pyx_k_K_RIGHTBRACKET, sizeof(__pyx_k_K_RIGHTBRACKET), 0, 0, 1, 1},
  {&__pyx_n_s_K_RIGHTPAREN, __pyx_k_K_RIGHTPAREN, sizeof(__pyx_k_K_RIGHTPAREN), 0, 0, 1, 1},
  {&__pyx_n_s_K_RMETA, __pyx_k_K_RMETA, sizeof(__pyx_k_K_RMETA), 0, 0, 1, 1},
  {&__pyx_n_s_K_RSHIFT, __pyx_k_K_RSHIFT, sizeof(__pyx_k_K_RSHIFT), 0, 0, 1, 1},
  {&__pyx_n_s_K_RSUPER, __pyx_k_K_RSUPER, sizeof(__pyx_k_K_RSUPER), 0, 0, 1, 1},
  {&__pyx_n_s_K_SCROLLLOCK, __pyx_k_K_SCROLLLOCK, sizeof(__pyx_k_K_SCROLLLOCK), 0, 0, 1, 1},
  {&__pyx_n_s_K_SCROLLOCK, __pyx_k_K_SCROLLOCK, sizeof(__pyx_k_K_SCROLLOCK), 0, 0, 1, 1},
  {&__pyx_n_s_K_SELECT, __pyx_k_K_SELECT, sizeof(__pyx_k_K_SELECT), 0, 0, 1, 1},
  {&__pyx_n_s_K_SEMICOLON, __pyx_k_K_SEMICOLON, sizeof(__pyx_k_K_SEMICOLON), 0, 0, 1, 1},
  {&__pyx_n_s_K_SEPARATOR, __pyx_k_K_SEPARATOR, sizeof(__pyx_k_K_SEPARATOR), 0, 0, 1, 1},
  {&__pyx_n_s_K_SLASH, __pyx_k_K_SLASH, sizeof(__pyx_k_K_SLASH), 0, 0, 1, 1},
  {&__pyx_n_s_K_SLEEP, __pyx_k_K_SLEEP, sizeof(__pyx_k_K_SLEEP), 0, 0, 1, 1},
  {&__pyx_n_s_K_SPACE, __pyx_k_K_SPACE, sizeof(__pyx_k_K_SPACE), 0, 0, 1, 1},
  {&__pyx_n_s_K_STOP, __pyx_k_K_STOP, sizeof(__pyx_k_K_STOP), 0, 0, 1, 1},
  {&__pyx_n_s_K_SYSREQ, __pyx_k_K_SYSREQ, sizeof(__pyx_k_K_SYSREQ), 0, 0, 1, 1},
  {&__pyx_n_s_K_TAB, __pyx_k_K_TAB, sizeof(__pyx_k_K_TAB), 0, 0, 1, 1},
  {&__pyx_n_s_K_THOUSANDSSEPARATOR, __pyx_k_K_THOUSANDSSEPARATOR, sizeof(__pyx_k_K_THOUSANDSSEPARATOR), 0, 0, 1, 1},
  {&__pyx_n_s_K_UNDERSCORE, __pyx_k_K_UNDERSCORE, sizeof(__pyx_k_K_UNDERSCORE), 0, 0, 1, 1},
  {&__pyx_n_s_K_UNDO, __pyx_k_K_UNDO, sizeof(__pyx_k_K_UNDO), 0, 0, 1, 1},
  {&__pyx_n_s_K_UNKNOWN, __pyx_k_K_UNKNOWN, sizeof(__pyx_k_K_UNKNOWN), 0, 0, 1, 1},
  {&__pyx_n_s_K_UP, __pyx_k_K_UP, sizeof(__pyx_k_K_UP), 0, 0, 1, 1},
  {&__pyx_n_s_K_VOLUMEDOWN, __pyx_k_K_VOLUMEDOWN, sizeof(__pyx_k_K_VOLUMEDOWN), 0, 0, 1, 1},
  {&__pyx_n_s_K_VOLUMEUP, __pyx_k_K_VOLUMEUP, sizeof(__pyx_k_K_VOLUMEUP), 0, 0, 1, 1},
  {&__pyx_n_s_K_WWW, __pyx_k_K_WWW, sizeof(__pyx_k_K_WWW), 0, 0, 1, 1},
  {&__pyx_n_s_K_a, __pyx_k_K_a, sizeof(__pyx_k_K_a), 0, 0, 1, 1},
  {&__pyx_n_s_K_b, __pyx_k_K_b, sizeof(__pyx_k_K_b), 0, 0, 1, 1},
  {&__pyx_n_s_K_c, __pyx_k_K_c, sizeof(__pyx_k_K_c), 0, 0, 1, 1},
  {&__pyx_n_s_K_d, __pyx_k_K_d, sizeof(__pyx_k_K_d), 0, 0, 1, 1},
  {&__pyx_n_s_K_e, __pyx_k_K_e, sizeof(__pyx_k_K_e), 0, 0, 1, 1},
  {&__pyx_n_s_K_f, __pyx_k_K_f, sizeof(__pyx_k_K_f), 0, 0, 1, 1},
  {&__pyx_n_s_K_g, __pyx_k_K_g, sizeof(__pyx_k_K_g), 0, 0, 1, 1},
  {&__pyx_n_s_K_h, __pyx_k_K_h, sizeof(__pyx_k_K_h), 0, 0, 1, 1},
  {&__pyx_n_s_K_i, __pyx_k_K_i, sizeof(__pyx_k_K_i), 0, 0, 1, 1},
  {&__pyx_n_s_K_j, __pyx_k_K_j, sizeof(__pyx_k_K_j), 0, 0, 1, 1},
  {&__pyx_n_s_K_k, __pyx_k_K_k, sizeof(__pyx_k_K_k), 0, 0, 1, 1},
  {&__pyx_n_s_K_l, __pyx_k_K_l, sizeof(__pyx_k_K_l), 0, 0, 1, 1},
  {&__pyx_n_s_K_m, __pyx_k_K_m, sizeof(__pyx_k_K_m), 0, 0, 1, 1},
  {&__pyx_n_s_K_n, __pyx_k_K_n, sizeof(__pyx_k_K_n), 0, 0, 1, 1},
  {&__pyx_n_s_K_o, __pyx_k_K_o, sizeof(__pyx_k_K_o), 0, 0, 1, 1},
  {&__pyx_n_s_K_p, __pyx_k_K_p, sizeof(__pyx_k_K_p), 0, 0, 1, 1},
  {&__pyx_n_s_K_q, __pyx_k_K_q, sizeof(__pyx_k_K_q), 0, 0, 1, 1},
  {&__pyx_n_s_K_r, __pyx_k_K_r, sizeof(__pyx_k_K_r), 0, 0, 1, 1},
  {&__pyx_n_s_K_s, __pyx_k_K_s, sizeof(__pyx_k_K_s), 0, 0, 1, 1},
  {&__pyx_n_s_K_t, __pyx_k_K_t, sizeof(__pyx_k_K_t), 0, 0, 1, 1},
  {&__pyx_n_s_K_u, __pyx_k_K_u, sizeof(__pyx_k_K_u), 0, 0, 1, 1},
  {&__pyx_n_s_K_v, __pyx_k_K_v, sizeof(__pyx_k_K_v), 0, 0, 1, 1},
  {&__pyx_n_s_K_w, __pyx_k_K_w, sizeof(__pyx_k_K_w), 0, 0, 1, 1},
  {&__pyx_n_s_K_x, __pyx_k_K_x, sizeof(__pyx_k_K_x), 0, 0, 1, 1},
  {&__pyx_n_s_K_y, __pyx_k_K_y, sizeof(__pyx_k_K_y), 0, 0, 1, 1},
  {&__pyx_n_s_K_z, __pyx_k_K_z, sizeof(__pyx_k_K_z), 0, 0, 1, 1},
  {&__pyx_n_s_LASTEVENT, __pyx_k_LASTEVENT, sizeof(__pyx_k_LASTEVENT), 0, 0, 1, 1},
  {&__pyx_n_s_LIL_ENDIAN, __pyx_k_LIL_ENDIAN, sizeof(__pyx_k_LIL_ENDIAN), 0, 0, 1, 1},
  {&__pyx_n_s_MOUSEBUTTONDOWN, __pyx_k_MOUSEBUTTONDOWN, sizeof(__pyx_k_MOUSEBUTTONDOWN), 0, 0, 1, 1},
  {&__pyx_n_s_MOUSEBUTTONUP, __pyx_k_MOUSEBUTTONUP, sizeof(__pyx_k_MOUSEBUTTONUP), 0, 0, 1, 1},
  {&__pyx_n_s_MOUSEMOTION, __pyx_k_MOUSEMOTION, sizeof(__pyx_k_MOUSEMOTION), 0, 0, 1, 1},
  {&__pyx_n_s_MOUSEWHEEL, __pyx_k_MOUSEWHEEL, sizeof(__pyx_k_MOUSEWHEEL), 0, 0, 1, 1},
  {&__pyx_n_s_MULTIGESTURE, __pyx_k_MULTIGESTURE, sizeof(__pyx_k_MULTIGESTURE), 0, 0, 1, 1},
  {&__pyx_n_s_NOEVENT, __pyx_k_NOEVENT, sizeof(__pyx_k_NOEVENT), 0, 0, 1, 1},
  {&__pyx_n_s_NOFRAME, __pyx_k_NOFRAME, sizeof(__pyx_k_NOFRAME), 0, 0, 1, 1},
  {&__pyx_n_s_OPENGL, __pyx_k_OPENGL, sizeof(__pyx_k_OPENGL), 0, 0, 1, 1},
  {&__pyx_n_s_POWERSTATE_CHARGED, __pyx_k_POWERSTATE_CHARGED, sizeof(__pyx_k_POWERSTATE_CHARGED), 0, 0, 1, 1},
  {&__pyx_n_s_POWERSTATE_CHARGING, __pyx_k_POWERSTATE_CHARGING, sizeof(__pyx_k_POWERSTATE_CHARGING), 0, 0, 1, 1},
  {&__pyx_n_s_POWERSTATE_NO_BATTERY, __pyx_k_POWERSTATE_NO_BATTERY, sizeof(__pyx_k_POWERSTATE_NO_BATTERY), 0, 0, 1, 1},
  {&__pyx_n_s_POWERSTATE_ON_BATTERY, __pyx_k_POWERSTATE_ON_BATTERY, sizeof(__pyx_k_POWERSTATE_ON_BATTERY), 0, 0, 1, 1},
  {&__pyx_n_s_POWERSTATE_UNKNOWN, __pyx_k_POWERSTATE_UNKNOWN, sizeof(__pyx_k_POWERSTATE_UNKNOWN), 0, 0, 1, 1},
  {&__pyx_n_s_QUIT, __pyx_k_QUIT, sizeof(__pyx_k_QUIT), 0, 0, 1, 1},
  {&__pyx_n_s_RENDER_DEVICE_RESET, __pyx_k_RENDER_DEVICE_RESET, sizeof(__pyx_k_RENDER_DEVICE_RESET), 0, 0, 1, 1},
  {&__pyx_n_s_RENDER_TARGETS_RESET, __pyx_k_RENDER_TARGETS_RESET, sizeof(__pyx_k_RENDER_TARGETS_RESET), 0, 0, 1, 1},
  {&__pyx_n_s_RESIZABLE, __pyx_k_RESIZABLE, sizeof(__pyx_k_RESIZABLE), 0, 0, 1, 1},
  {&__pyx_n_s_RLEACCEL, __pyx_k_RLEACCEL, sizeof(__pyx_k_RLEACCEL), 0, 0, 1, 1},
  {&__pyx_n_s_Rect, __pyx_k_Rect, sizeof(__pyx_k_Rect), 0, 0, 1, 1},
  {&__pyx_n_s_SCRAP_TEXT, __pyx_k_SCRAP_TEXT, sizeof(__pyx_k_SCRAP_TEXT), 0, 0, 1, 1},
  {&__pyx_n_s_SDL_VERSION_TUPLE, __pyx_k_SDL_VERSION_TUPLE, sizeof(__pyx_k_SDL_VERSION_TUPLE), 0, 0, 1, 1},
  {&__pyx_n_s_SENSORUPDATE, __pyx_k_SENSORUPDATE, sizeof(__pyx_k_SENSORUPDATE), 0, 0, 1, 1},
  {&__pyx_n_s_SRCALPHA, __pyx_k_SRCALPHA, sizeof(__pyx_k_SRCALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_SRCCOLORKEY, __pyx_k_SRCCOLORKEY, sizeof(__pyx_k_SRCCOLORKEY), 0, 0, 1, 1},
  {&__pyx_n_s_SWSURFACE, __pyx_k_SWSURFACE, sizeof(__pyx_k_SWSURFACE), 0, 0, 1, 1},
  {&__pyx_n_s_SYSWMEVENT, __pyx_k_SYSWMEVENT, sizeof(__pyx_k_SYSWMEVENT), 0, 0, 1, 1},
  {&__pyx_n_s_TEXTEDITING, __pyx_k_TEXTEDITING, sizeof(__pyx_k_TEXTEDITING), 0, 0, 1, 1},
  {&__pyx_n_s_TEXTINPUT, __pyx_k_TEXTINPUT, sizeof(__pyx_k_TEXTINPUT), 0, 0, 1, 1},
  {&__pyx_n_s_TOUCH_MOUSEID, __pyx_k_TOUCH_MOUSEID, sizeof(__pyx_k_TOUCH_MOUSEID), 0, 0, 1, 1},
  {&__pyx_n_s_USEREVENT, __pyx_k_USEREVENT, sizeof(__pyx_k_USEREVENT), 0, 0, 1, 1},
  {&__pyx_n_s_VIDEOEXPOSE, __pyx_k_VIDEOEXPOSE, sizeof(__pyx_k_VIDEOEXPOSE), 0, 0, 1, 1},
  {&__pyx_n_s_VIDEORESIZE, __pyx_k_VIDEORESIZE, sizeof(__pyx_k_VIDEORESIZE), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOWEVENT, __pyx_k_WINDOWEVENT, sizeof(__pyx_k_WINDOWEVENT), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOWPOS_CENTERED, __pyx_k_WINDOWPOS_CENTERED, sizeof(__pyx_k_WINDOWPOS_CENTERED), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOWPOS_UNDEFINED, __pyx_k_WINDOWPOS_UNDEFINED, sizeof(__pyx_k_WINDOWPOS_UNDEFINED), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_ALLOW_HIGHDPI, __pyx_k_WINDOW_ALLOW_HIGHDPI, sizeof(__pyx_k_WINDOW_ALLOW_HIGHDPI), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_BORDERLESS, __pyx_k_WINDOW_BORDERLESS, sizeof(__pyx_k_WINDOW_BORDERLESS), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_FOREIGN, __pyx_k_WINDOW_FOREIGN, sizeof(__pyx_k_WINDOW_FOREIGN), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_FULLSCREEN, __pyx_k_WINDOW_FULLSCREEN, sizeof(__pyx_k_WINDOW_FULLSCREEN), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_FULLSCREEN_DESKTOP, __pyx_k_WINDOW_FULLSCREEN_DESKTOP, sizeof(__pyx_k_WINDOW_FULLSCREEN_DESKTOP), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_HIDDEN, __pyx_k_WINDOW_HIDDEN, sizeof(__pyx_k_WINDOW_HIDDEN), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_INPUT_FOCUS, __pyx_k_WINDOW_INPUT_FOCUS, sizeof(__pyx_k_WINDOW_INPUT_FOCUS), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_INPUT_GRABBED, __pyx_k_WINDOW_INPUT_GRABBED, sizeof(__pyx_k_WINDOW_INPUT_GRABBED), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_MAXIMIZED, __pyx_k_WINDOW_MAXIMIZED, sizeof(__pyx_k_WINDOW_MAXIMIZED), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_MINIMIZED, __pyx_k_WINDOW_MINIMIZED, sizeof(__pyx_k_WINDOW_MINIMIZED), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_NOFRAME, __pyx_k_WINDOW_NOFRAME, sizeof(__pyx_k_WINDOW_NOFRAME), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_OPENGL, __pyx_k_WINDOW_OPENGL, sizeof(__pyx_k_WINDOW_OPENGL), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_RESIZABLE, __pyx_k_WINDOW_RESIZABLE, sizeof(__pyx_k_WINDOW_RESIZABLE), 0, 0, 1, 1},
  {&__pyx_n_s_WINDOW_SHOWN, __pyx_k_WINDOW_SHOWN, sizeof(__pyx_k_WINDOW_SHOWN), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_pygame_sdl2_color, __pyx_k_pygame_sdl2_color, sizeof(__pyx_k_pygame_sdl2_color), 0, 0, 1, 1},
  {&__pyx_n_s_pygame_sdl2_rect, __pyx_k_pygame_sdl2_rect, sizeof(__pyx_k_pygame_sdl2_rect), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_kp_s_text_plain, __pyx_k_text_plain, sizeof(__pyx_k_text_plain), 0, 0, 1, 0},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  return 0;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_6 = PyInt_FromLong(6); if (unlikely(!__pyx_int_6)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_7 = PyInt_FromLong(7); if (unlikely(!__pyx_int_7)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_8 = PyInt_FromLong(8); if (unlikely(!__pyx_int_8)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_9 = PyInt_FromLong(9); if (unlikely(!__pyx_int_9)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16 = PyInt_FromLong(16); if (unlikely(!__pyx_int_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_17 = PyInt_FromLong(17); if (unlikely(!__pyx_int_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1073741824 = PyInt_FromLong(1073741824L); if (unlikely(!__pyx_int_1073741824)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2147483648 = PyInt_FromString((char *)"2147483648", 0, 0); if (unlikely(!__pyx_int_2147483648)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initlocals(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initlocals(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_locals(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_locals(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_locals(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'locals' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_locals(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("locals", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pygame_sdl2__locals) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pygame_sdl2.locals")) {
      if (unlikely(PyDict_SetItemString(modules, "pygame_sdl2.locals", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  (void)__Pyx_modinit_type_init_code();
  (void)__Pyx_modinit_type_import_code();
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "pygame_sdl2/locals.pyx":4
 * cimport sdl2
 * 
 * from pygame_sdl2.rect import Rect             # <<<<<<<<<<<<<<
 * from pygame_sdl2.color import Color
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_Rect);
  __Pyx_GIVEREF(__pyx_n_s_Rect);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_Rect);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_pygame_sdl2_rect, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Rect); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Rect, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":5
 * 
 * from pygame_sdl2.rect import Rect
 * from pygame_sdl2.color import Color             # <<<<<<<<<<<<<<
 * 
 * include "event_list.pxi"
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_Color);
  __Pyx_GIVEREF(__pyx_n_s_Color);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_Color);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_pygame_sdl2_color, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_Color); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Color, __pyx_t_2) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":4
 * # Do not edit!
 * 
 * NOEVENT = SDL_FIRSTEVENT             # <<<<<<<<<<<<<<
 * QUIT = SDL_QUIT
 * APP_TERMINATING = SDL_APP_TERMINATING
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_FIRSTEVENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_NOEVENT, __pyx_t_1) < 0) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":5
 * 
 * NOEVENT = SDL_FIRSTEVENT
 * QUIT = SDL_QUIT             # <<<<<<<<<<<<<<
 * APP_TERMINATING = SDL_APP_TERMINATING
 * APP_LOWMEMORY = SDL_APP_LOWMEMORY
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_QUIT); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_QUIT, __pyx_t_1) < 0) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":6
 * NOEVENT = SDL_FIRSTEVENT
 * QUIT = SDL_QUIT
 * APP_TERMINATING = SDL_APP_TERMINATING             # <<<<<<<<<<<<<<
 * APP_LOWMEMORY = SDL_APP_LOWMEMORY
 * APP_WILLENTERBACKGROUND = SDL_APP_WILLENTERBACKGROUND
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_APP_TERMINATING); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_APP_TERMINATING, __pyx_t_1) < 0) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":7
 * QUIT = SDL_QUIT
 * APP_TERMINATING = SDL_APP_TERMINATING
 * APP_LOWMEMORY = SDL_APP_LOWMEMORY             # <<<<<<<<<<<<<<
 * APP_WILLENTERBACKGROUND = SDL_APP_WILLENTERBACKGROUND
 * APP_DIDENTERBACKGROUND = SDL_APP_DIDENTERBACKGROUND
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_APP_LOWMEMORY); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_APP_LOWMEMORY, __pyx_t_1) < 0) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":8
 * APP_TERMINATING = SDL_APP_TERMINATING
 * APP_LOWMEMORY = SDL_APP_LOWMEMORY
 * APP_WILLENTERBACKGROUND = SDL_APP_WILLENTERBACKGROUND             # <<<<<<<<<<<<<<
 * APP_DIDENTERBACKGROUND = SDL_APP_DIDENTERBACKGROUND
 * APP_WILLENTERFOREGROUND = SDL_APP_WILLENTERFOREGROUND
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_APP_WILLENTERBACKGROUND); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_APP_WILLENTERBACKGROUND, __pyx_t_1) < 0) __PYX_ERR(1, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":9
 * APP_LOWMEMORY = SDL_APP_LOWMEMORY
 * APP_WILLENTERBACKGROUND = SDL_APP_WILLENTERBACKGROUND
 * APP_DIDENTERBACKGROUND = SDL_APP_DIDENTERBACKGROUND             # <<<<<<<<<<<<<<
 * APP_WILLENTERFOREGROUND = SDL_APP_WILLENTERFOREGROUND
 * APP_DIDENTERFOREGROUND = SDL_APP_DIDENTERFOREGROUND
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_APP_DIDENTERBACKGROUND); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_APP_DIDENTERBACKGROUND, __pyx_t_1) < 0) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":10
 * APP_WILLENTERBACKGROUND = SDL_APP_WILLENTERBACKGROUND
 * APP_DIDENTERBACKGROUND = SDL_APP_DIDENTERBACKGROUND
 * APP_WILLENTERFOREGROUND = SDL_APP_WILLENTERFOREGROUND             # <<<<<<<<<<<<<<
 * APP_DIDENTERFOREGROUND = SDL_APP_DIDENTERFOREGROUND
 * DISPLAYEVENT = SDL_DISPLAYEVENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_APP_WILLENTERFOREGROUND); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_APP_WILLENTERFOREGROUND, __pyx_t_1) < 0) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":11
 * APP_DIDENTERBACKGROUND = SDL_APP_DIDENTERBACKGROUND
 * APP_WILLENTERFOREGROUND = SDL_APP_WILLENTERFOREGROUND
 * APP_DIDENTERFOREGROUND = SDL_APP_DIDENTERFOREGROUND             # <<<<<<<<<<<<<<
 * DISPLAYEVENT = SDL_DISPLAYEVENT
 * WINDOWEVENT = SDL_WINDOWEVENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_APP_DIDENTERFOREGROUND); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_APP_DIDENTERFOREGROUND, __pyx_t_1) < 0) __PYX_ERR(1, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":12
 * APP_WILLENTERFOREGROUND = SDL_APP_WILLENTERFOREGROUND
 * APP_DIDENTERFOREGROUND = SDL_APP_DIDENTERFOREGROUND
 * DISPLAYEVENT = SDL_DISPLAYEVENT             # <<<<<<<<<<<<<<
 * WINDOWEVENT = SDL_WINDOWEVENT
 * SYSWMEVENT = SDL_SYSWMEVENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_DISPLAYEVENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DISPLAYEVENT, __pyx_t_1) < 0) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":13
 * APP_DIDENTERFOREGROUND = SDL_APP_DIDENTERFOREGROUND
 * DISPLAYEVENT = SDL_DISPLAYEVENT
 * WINDOWEVENT = SDL_WINDOWEVENT             # <<<<<<<<<<<<<<
 * SYSWMEVENT = SDL_SYSWMEVENT
 * KEYDOWN = SDL_KEYDOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_WINDOWEVENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOWEVENT, __pyx_t_1) < 0) __PYX_ERR(1, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":14
 * DISPLAYEVENT = SDL_DISPLAYEVENT
 * WINDOWEVENT = SDL_WINDOWEVENT
 * SYSWMEVENT = SDL_SYSWMEVENT             # <<<<<<<<<<<<<<
 * KEYDOWN = SDL_KEYDOWN
 * KEYUP = SDL_KEYUP
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_SYSWMEVENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SYSWMEVENT, __pyx_t_1) < 0) __PYX_ERR(1, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":15
 * WINDOWEVENT = SDL_WINDOWEVENT
 * SYSWMEVENT = SDL_SYSWMEVENT
 * KEYDOWN = SDL_KEYDOWN             # <<<<<<<<<<<<<<
 * KEYUP = SDL_KEYUP
 * TEXTEDITING = SDL_TEXTEDITING
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_KEYDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_KEYDOWN, __pyx_t_1) < 0) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":16
 * SYSWMEVENT = SDL_SYSWMEVENT
 * KEYDOWN = SDL_KEYDOWN
 * KEYUP = SDL_KEYUP             # <<<<<<<<<<<<<<
 * TEXTEDITING = SDL_TEXTEDITING
 * TEXTINPUT = SDL_TEXTINPUT
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_KEYUP); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_KEYUP, __pyx_t_1) < 0) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":17
 * KEYDOWN = SDL_KEYDOWN
 * KEYUP = SDL_KEYUP
 * TEXTEDITING = SDL_TEXTEDITING             # <<<<<<<<<<<<<<
 * TEXTINPUT = SDL_TEXTINPUT
 * KEYMAPCHANGED = SDL_KEYMAPCHANGED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_TEXTEDITING); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TEXTEDITING, __pyx_t_1) < 0) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":18
 * KEYUP = SDL_KEYUP
 * TEXTEDITING = SDL_TEXTEDITING
 * TEXTINPUT = SDL_TEXTINPUT             # <<<<<<<<<<<<<<
 * KEYMAPCHANGED = SDL_KEYMAPCHANGED
 * MOUSEMOTION = SDL_MOUSEMOTION
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_TEXTINPUT); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TEXTINPUT, __pyx_t_1) < 0) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":19
 * TEXTEDITING = SDL_TEXTEDITING
 * TEXTINPUT = SDL_TEXTINPUT
 * KEYMAPCHANGED = SDL_KEYMAPCHANGED             # <<<<<<<<<<<<<<
 * MOUSEMOTION = SDL_MOUSEMOTION
 * MOUSEBUTTONDOWN = SDL_MOUSEBUTTONDOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_KEYMAPCHANGED); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_KEYMAPCHANGED, __pyx_t_1) < 0) __PYX_ERR(1, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":20
 * TEXTINPUT = SDL_TEXTINPUT
 * KEYMAPCHANGED = SDL_KEYMAPCHANGED
 * MOUSEMOTION = SDL_MOUSEMOTION             # <<<<<<<<<<<<<<
 * MOUSEBUTTONDOWN = SDL_MOUSEBUTTONDOWN
 * MOUSEBUTTONUP = SDL_MOUSEBUTTONUP
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_MOUSEMOTION); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MOUSEMOTION, __pyx_t_1) < 0) __PYX_ERR(1, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":21
 * KEYMAPCHANGED = SDL_KEYMAPCHANGED
 * MOUSEMOTION = SDL_MOUSEMOTION
 * MOUSEBUTTONDOWN = SDL_MOUSEBUTTONDOWN             # <<<<<<<<<<<<<<
 * MOUSEBUTTONUP = SDL_MOUSEBUTTONUP
 * MOUSEWHEEL = SDL_MOUSEWHEEL
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_MOUSEBUTTONDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MOUSEBUTTONDOWN, __pyx_t_1) < 0) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":22
 * MOUSEMOTION = SDL_MOUSEMOTION
 * MOUSEBUTTONDOWN = SDL_MOUSEBUTTONDOWN
 * MOUSEBUTTONUP = SDL_MOUSEBUTTONUP             # <<<<<<<<<<<<<<
 * MOUSEWHEEL = SDL_MOUSEWHEEL
 * JOYAXISMOTION = SDL_JOYAXISMOTION
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_MOUSEBUTTONUP); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MOUSEBUTTONUP, __pyx_t_1) < 0) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":23
 * MOUSEBUTTONDOWN = SDL_MOUSEBUTTONDOWN
 * MOUSEBUTTONUP = SDL_MOUSEBUTTONUP
 * MOUSEWHEEL = SDL_MOUSEWHEEL             # <<<<<<<<<<<<<<
 * JOYAXISMOTION = SDL_JOYAXISMOTION
 * JOYBALLMOTION = SDL_JOYBALLMOTION
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_MOUSEWHEEL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MOUSEWHEEL, __pyx_t_1) < 0) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":24
 * MOUSEBUTTONUP = SDL_MOUSEBUTTONUP
 * MOUSEWHEEL = SDL_MOUSEWHEEL
 * JOYAXISMOTION = SDL_JOYAXISMOTION             # <<<<<<<<<<<<<<
 * JOYBALLMOTION = SDL_JOYBALLMOTION
 * JOYHATMOTION = SDL_JOYHATMOTION
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_JOYAXISMOTION); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_JOYAXISMOTION, __pyx_t_1) < 0) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":25
 * MOUSEWHEEL = SDL_MOUSEWHEEL
 * JOYAXISMOTION = SDL_JOYAXISMOTION
 * JOYBALLMOTION = SDL_JOYBALLMOTION             # <<<<<<<<<<<<<<
 * JOYHATMOTION = SDL_JOYHATMOTION
 * JOYBUTTONDOWN = SDL_JOYBUTTONDOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_JOYBALLMOTION); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_JOYBALLMOTION, __pyx_t_1) < 0) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":26
 * JOYAXISMOTION = SDL_JOYAXISMOTION
 * JOYBALLMOTION = SDL_JOYBALLMOTION
 * JOYHATMOTION = SDL_JOYHATMOTION             # <<<<<<<<<<<<<<
 * JOYBUTTONDOWN = SDL_JOYBUTTONDOWN
 * JOYBUTTONUP = SDL_JOYBUTTONUP
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_JOYHATMOTION); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_JOYHATMOTION, __pyx_t_1) < 0) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":27
 * JOYBALLMOTION = SDL_JOYBALLMOTION
 * JOYHATMOTION = SDL_JOYHATMOTION
 * JOYBUTTONDOWN = SDL_JOYBUTTONDOWN             # <<<<<<<<<<<<<<
 * JOYBUTTONUP = SDL_JOYBUTTONUP
 * JOYDEVICEADDED = SDL_JOYDEVICEADDED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_JOYBUTTONDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_JOYBUTTONDOWN, __pyx_t_1) < 0) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":28
 * JOYHATMOTION = SDL_JOYHATMOTION
 * JOYBUTTONDOWN = SDL_JOYBUTTONDOWN
 * JOYBUTTONUP = SDL_JOYBUTTONUP             # <<<<<<<<<<<<<<
 * JOYDEVICEADDED = SDL_JOYDEVICEADDED
 * JOYDEVICEREMOVED = SDL_JOYDEVICEREMOVED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_JOYBUTTONUP); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_JOYBUTTONUP, __pyx_t_1) < 0) __PYX_ERR(1, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":29
 * JOYBUTTONDOWN = SDL_JOYBUTTONDOWN
 * JOYBUTTONUP = SDL_JOYBUTTONUP
 * JOYDEVICEADDED = SDL_JOYDEVICEADDED             # <<<<<<<<<<<<<<
 * JOYDEVICEREMOVED = SDL_JOYDEVICEREMOVED
 * CONTROLLERAXISMOTION = SDL_CONTROLLERAXISMOTION
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_JOYDEVICEADDED); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_JOYDEVICEADDED, __pyx_t_1) < 0) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":30
 * JOYBUTTONUP = SDL_JOYBUTTONUP
 * JOYDEVICEADDED = SDL_JOYDEVICEADDED
 * JOYDEVICEREMOVED = SDL_JOYDEVICEREMOVED             # <<<<<<<<<<<<<<
 * CONTROLLERAXISMOTION = SDL_CONTROLLERAXISMOTION
 * CONTROLLERBUTTONDOWN = SDL_CONTROLLERBUTTONDOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_JOYDEVICEREMOVED); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_JOYDEVICEREMOVED, __pyx_t_1) < 0) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":31
 * JOYDEVICEADDED = SDL_JOYDEVICEADDED
 * JOYDEVICEREMOVED = SDL_JOYDEVICEREMOVED
 * CONTROLLERAXISMOTION = SDL_CONTROLLERAXISMOTION             # <<<<<<<<<<<<<<
 * CONTROLLERBUTTONDOWN = SDL_CONTROLLERBUTTONDOWN
 * CONTROLLERBUTTONUP = SDL_CONTROLLERBUTTONUP
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_CONTROLLERAXISMOTION); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLERAXISMOTION, __pyx_t_1) < 0) __PYX_ERR(1, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":32
 * JOYDEVICEREMOVED = SDL_JOYDEVICEREMOVED
 * CONTROLLERAXISMOTION = SDL_CONTROLLERAXISMOTION
 * CONTROLLERBUTTONDOWN = SDL_CONTROLLERBUTTONDOWN             # <<<<<<<<<<<<<<
 * CONTROLLERBUTTONUP = SDL_CONTROLLERBUTTONUP
 * CONTROLLERDEVICEADDED = SDL_CONTROLLERDEVICEADDED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_CONTROLLERBUTTONDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLERBUTTONDOWN, __pyx_t_1) < 0) __PYX_ERR(1, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":33
 * CONTROLLERAXISMOTION = SDL_CONTROLLERAXISMOTION
 * CONTROLLERBUTTONDOWN = SDL_CONTROLLERBUTTONDOWN
 * CONTROLLERBUTTONUP = SDL_CONTROLLERBUTTONUP             # <<<<<<<<<<<<<<
 * CONTROLLERDEVICEADDED = SDL_CONTROLLERDEVICEADDED
 * CONTROLLERDEVICEREMOVED = SDL_CONTROLLERDEVICEREMOVED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_CONTROLLERBUTTONUP); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLERBUTTONUP, __pyx_t_1) < 0) __PYX_ERR(1, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":34
 * CONTROLLERBUTTONDOWN = SDL_CONTROLLERBUTTONDOWN
 * CONTROLLERBUTTONUP = SDL_CONTROLLERBUTTONUP
 * CONTROLLERDEVICEADDED = SDL_CONTROLLERDEVICEADDED             # <<<<<<<<<<<<<<
 * CONTROLLERDEVICEREMOVED = SDL_CONTROLLERDEVICEREMOVED
 * CONTROLLERDEVICEREMAPPED = SDL_CONTROLLERDEVICEREMAPPED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_CONTROLLERDEVICEADDED); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLERDEVICEADDED, __pyx_t_1) < 0) __PYX_ERR(1, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":35
 * CONTROLLERBUTTONUP = SDL_CONTROLLERBUTTONUP
 * CONTROLLERDEVICEADDED = SDL_CONTROLLERDEVICEADDED
 * CONTROLLERDEVICEREMOVED = SDL_CONTROLLERDEVICEREMOVED             # <<<<<<<<<<<<<<
 * CONTROLLERDEVICEREMAPPED = SDL_CONTROLLERDEVICEREMAPPED
 * FINGERDOWN = SDL_FINGERDOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_CONTROLLERDEVICEREMOVED); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLERDEVICEREMOVED, __pyx_t_1) < 0) __PYX_ERR(1, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":36
 * CONTROLLERDEVICEADDED = SDL_CONTROLLERDEVICEADDED
 * CONTROLLERDEVICEREMOVED = SDL_CONTROLLERDEVICEREMOVED
 * CONTROLLERDEVICEREMAPPED = SDL_CONTROLLERDEVICEREMAPPED             # <<<<<<<<<<<<<<
 * FINGERDOWN = SDL_FINGERDOWN
 * FINGERUP = SDL_FINGERUP
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_CONTROLLERDEVICEREMAPPED); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLERDEVICEREMAPPED, __pyx_t_1) < 0) __PYX_ERR(1, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":37
 * CONTROLLERDEVICEREMOVED = SDL_CONTROLLERDEVICEREMOVED
 * CONTROLLERDEVICEREMAPPED = SDL_CONTROLLERDEVICEREMAPPED
 * FINGERDOWN = SDL_FINGERDOWN             # <<<<<<<<<<<<<<
 * FINGERUP = SDL_FINGERUP
 * FINGERMOTION = SDL_FINGERMOTION
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_FINGERDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FINGERDOWN, __pyx_t_1) < 0) __PYX_ERR(1, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":38
 * CONTROLLERDEVICEREMAPPED = SDL_CONTROLLERDEVICEREMAPPED
 * FINGERDOWN = SDL_FINGERDOWN
 * FINGERUP = SDL_FINGERUP             # <<<<<<<<<<<<<<
 * FINGERMOTION = SDL_FINGERMOTION
 * DOLLARGESTURE = SDL_DOLLARGESTURE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_FINGERUP); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FINGERUP, __pyx_t_1) < 0) __PYX_ERR(1, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":39
 * FINGERDOWN = SDL_FINGERDOWN
 * FINGERUP = SDL_FINGERUP
 * FINGERMOTION = SDL_FINGERMOTION             # <<<<<<<<<<<<<<
 * DOLLARGESTURE = SDL_DOLLARGESTURE
 * DOLLARRECORD = SDL_DOLLARRECORD
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_FINGERMOTION); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FINGERMOTION, __pyx_t_1) < 0) __PYX_ERR(1, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":40
 * FINGERUP = SDL_FINGERUP
 * FINGERMOTION = SDL_FINGERMOTION
 * DOLLARGESTURE = SDL_DOLLARGESTURE             # <<<<<<<<<<<<<<
 * DOLLARRECORD = SDL_DOLLARRECORD
 * MULTIGESTURE = SDL_MULTIGESTURE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_DOLLARGESTURE); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DOLLARGESTURE, __pyx_t_1) < 0) __PYX_ERR(1, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":41
 * FINGERMOTION = SDL_FINGERMOTION
 * DOLLARGESTURE = SDL_DOLLARGESTURE
 * DOLLARRECORD = SDL_DOLLARRECORD             # <<<<<<<<<<<<<<
 * MULTIGESTURE = SDL_MULTIGESTURE
 * CLIPBOARDUPDATE = SDL_CLIPBOARDUPDATE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_DOLLARRECORD); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DOLLARRECORD, __pyx_t_1) < 0) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":42
 * DOLLARGESTURE = SDL_DOLLARGESTURE
 * DOLLARRECORD = SDL_DOLLARRECORD
 * MULTIGESTURE = SDL_MULTIGESTURE             # <<<<<<<<<<<<<<
 * CLIPBOARDUPDATE = SDL_CLIPBOARDUPDATE
 * DROPFILE = SDL_DROPFILE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_MULTIGESTURE); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MULTIGESTURE, __pyx_t_1) < 0) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":43
 * DOLLARRECORD = SDL_DOLLARRECORD
 * MULTIGESTURE = SDL_MULTIGESTURE
 * CLIPBOARDUPDATE = SDL_CLIPBOARDUPDATE             # <<<<<<<<<<<<<<
 * DROPFILE = SDL_DROPFILE
 * DROPTEXT = SDL_DROPTEXT
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_CLIPBOARDUPDATE); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CLIPBOARDUPDATE, __pyx_t_1) < 0) __PYX_ERR(1, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":44
 * MULTIGESTURE = SDL_MULTIGESTURE
 * CLIPBOARDUPDATE = SDL_CLIPBOARDUPDATE
 * DROPFILE = SDL_DROPFILE             # <<<<<<<<<<<<<<
 * DROPTEXT = SDL_DROPTEXT
 * DROPBEGIN = SDL_DROPBEGIN
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_DROPFILE); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DROPFILE, __pyx_t_1) < 0) __PYX_ERR(1, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":45
 * CLIPBOARDUPDATE = SDL_CLIPBOARDUPDATE
 * DROPFILE = SDL_DROPFILE
 * DROPTEXT = SDL_DROPTEXT             # <<<<<<<<<<<<<<
 * DROPBEGIN = SDL_DROPBEGIN
 * DROPCOMPLETE = SDL_DROPCOMPLETE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_DROPTEXT); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DROPTEXT, __pyx_t_1) < 0) __PYX_ERR(1, 45, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":46
 * DROPFILE = SDL_DROPFILE
 * DROPTEXT = SDL_DROPTEXT
 * DROPBEGIN = SDL_DROPBEGIN             # <<<<<<<<<<<<<<
 * DROPCOMPLETE = SDL_DROPCOMPLETE
 * AUDIODEVICEADDED = SDL_AUDIODEVICEADDED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_DROPBEGIN); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DROPBEGIN, __pyx_t_1) < 0) __PYX_ERR(1, 46, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":47
 * DROPTEXT = SDL_DROPTEXT
 * DROPBEGIN = SDL_DROPBEGIN
 * DROPCOMPLETE = SDL_DROPCOMPLETE             # <<<<<<<<<<<<<<
 * AUDIODEVICEADDED = SDL_AUDIODEVICEADDED
 * AUDIODEVICEREMOVED = SDL_AUDIODEVICEREMOVED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_DROPCOMPLETE); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DROPCOMPLETE, __pyx_t_1) < 0) __PYX_ERR(1, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":48
 * DROPBEGIN = SDL_DROPBEGIN
 * DROPCOMPLETE = SDL_DROPCOMPLETE
 * AUDIODEVICEADDED = SDL_AUDIODEVICEADDED             # <<<<<<<<<<<<<<
 * AUDIODEVICEREMOVED = SDL_AUDIODEVICEREMOVED
 * SENSORUPDATE = SDL_SENSORUPDATE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_AUDIODEVICEADDED); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_AUDIODEVICEADDED, __pyx_t_1) < 0) __PYX_ERR(1, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":49
 * DROPCOMPLETE = SDL_DROPCOMPLETE
 * AUDIODEVICEADDED = SDL_AUDIODEVICEADDED
 * AUDIODEVICEREMOVED = SDL_AUDIODEVICEREMOVED             # <<<<<<<<<<<<<<
 * SENSORUPDATE = SDL_SENSORUPDATE
 * RENDER_TARGETS_RESET = SDL_RENDER_TARGETS_RESET
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_AUDIODEVICEREMOVED); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_AUDIODEVICEREMOVED, __pyx_t_1) < 0) __PYX_ERR(1, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":50
 * AUDIODEVICEADDED = SDL_AUDIODEVICEADDED
 * AUDIODEVICEREMOVED = SDL_AUDIODEVICEREMOVED
 * SENSORUPDATE = SDL_SENSORUPDATE             # <<<<<<<<<<<<<<
 * RENDER_TARGETS_RESET = SDL_RENDER_TARGETS_RESET
 * RENDER_DEVICE_RESET = SDL_RENDER_DEVICE_RESET
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_SENSORUPDATE); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SENSORUPDATE, __pyx_t_1) < 0) __PYX_ERR(1, 50, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":51
 * AUDIODEVICEREMOVED = SDL_AUDIODEVICEREMOVED
 * SENSORUPDATE = SDL_SENSORUPDATE
 * RENDER_TARGETS_RESET = SDL_RENDER_TARGETS_RESET             # <<<<<<<<<<<<<<
 * RENDER_DEVICE_RESET = SDL_RENDER_DEVICE_RESET
 * USEREVENT = SDL_USEREVENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_RENDER_TARGETS_RESET); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RENDER_TARGETS_RESET, __pyx_t_1) < 0) __PYX_ERR(1, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":52
 * SENSORUPDATE = SDL_SENSORUPDATE
 * RENDER_TARGETS_RESET = SDL_RENDER_TARGETS_RESET
 * RENDER_DEVICE_RESET = SDL_RENDER_DEVICE_RESET             # <<<<<<<<<<<<<<
 * USEREVENT = SDL_USEREVENT
 * LASTEVENT = SDL_LASTEVENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_RENDER_DEVICE_RESET); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RENDER_DEVICE_RESET, __pyx_t_1) < 0) __PYX_ERR(1, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":53
 * RENDER_TARGETS_RESET = SDL_RENDER_TARGETS_RESET
 * RENDER_DEVICE_RESET = SDL_RENDER_DEVICE_RESET
 * USEREVENT = SDL_USEREVENT             # <<<<<<<<<<<<<<
 * LASTEVENT = SDL_LASTEVENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_USEREVENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_USEREVENT, __pyx_t_1) < 0) __PYX_ERR(1, 53, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/event_list.pxi":54
 * RENDER_DEVICE_RESET = SDL_RENDER_DEVICE_RESET
 * USEREVENT = SDL_USEREVENT
 * LASTEVENT = SDL_LASTEVENT             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_EventType(SDL_LASTEVENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_LASTEVENT, __pyx_t_1) < 0) __PYX_ERR(1, 54, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":8
 * 
 * include "event_list.pxi"
 * ACTIVEEVENT = SDL_LASTEVENT - 1             # <<<<<<<<<<<<<<
 * VIDEORESIZE = SDL_LASTEVENT - 2
 * VIDEOEXPOSE = SDL_LASTEVENT - 3
 */
  __pyx_t_1 = __Pyx_PyInt_From_long((SDL_LASTEVENT - 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ACTIVEEVENT, __pyx_t_1) < 0) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":9
 * include "event_list.pxi"
 * ACTIVEEVENT = SDL_LASTEVENT - 1
 * VIDEORESIZE = SDL_LASTEVENT - 2             # <<<<<<<<<<<<<<
 * VIDEOEXPOSE = SDL_LASTEVENT - 3
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_long((SDL_LASTEVENT - 2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_VIDEORESIZE, __pyx_t_1) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":10
 * ACTIVEEVENT = SDL_LASTEVENT - 1
 * VIDEORESIZE = SDL_LASTEVENT - 2
 * VIDEOEXPOSE = SDL_LASTEVENT - 3             # <<<<<<<<<<<<<<
 * 
 * include "keycode_list.pxi"
 */
  __pyx_t_1 = __Pyx_PyInt_From_long((SDL_LASTEVENT - 3)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_VIDEOEXPOSE, __pyx_t_1) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":4
 * # Do not edit!
 * 
 * K_UNKNOWN = SDLK_UNKNOWN             # <<<<<<<<<<<<<<
 * K_RETURN = SDLK_RETURN
 * K_ESCAPE = SDLK_ESCAPE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_UNKNOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_UNKNOWN, __pyx_t_1) < 0) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":5
 * 
 * K_UNKNOWN = SDLK_UNKNOWN
 * K_RETURN = SDLK_RETURN             # <<<<<<<<<<<<<<
 * K_ESCAPE = SDLK_ESCAPE
 * K_BACKSPACE = SDLK_BACKSPACE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_RETURN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_RETURN, __pyx_t_1) < 0) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":6
 * K_UNKNOWN = SDLK_UNKNOWN
 * K_RETURN = SDLK_RETURN
 * K_ESCAPE = SDLK_ESCAPE             # <<<<<<<<<<<<<<
 * K_BACKSPACE = SDLK_BACKSPACE
 * K_TAB = SDLK_TAB
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_ESCAPE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_ESCAPE, __pyx_t_1) < 0) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":7
 * K_RETURN = SDLK_RETURN
 * K_ESCAPE = SDLK_ESCAPE
 * K_BACKSPACE = SDLK_BACKSPACE             # <<<<<<<<<<<<<<
 * K_TAB = SDLK_TAB
 * K_SPACE = SDLK_SPACE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_BACKSPACE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_BACKSPACE, __pyx_t_1) < 0) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":8
 * K_ESCAPE = SDLK_ESCAPE
 * K_BACKSPACE = SDLK_BACKSPACE
 * K_TAB = SDLK_TAB             # <<<<<<<<<<<<<<
 * K_SPACE = SDLK_SPACE
 * K_EXCLAIM = SDLK_EXCLAIM
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_TAB); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_TAB, __pyx_t_1) < 0) __PYX_ERR(2, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":9
 * K_BACKSPACE = SDLK_BACKSPACE
 * K_TAB = SDLK_TAB
 * K_SPACE = SDLK_SPACE             # <<<<<<<<<<<<<<
 * K_EXCLAIM = SDLK_EXCLAIM
 * K_QUOTEDBL = SDLK_QUOTEDBL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_SPACE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_SPACE, __pyx_t_1) < 0) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":10
 * K_TAB = SDLK_TAB
 * K_SPACE = SDLK_SPACE
 * K_EXCLAIM = SDLK_EXCLAIM             # <<<<<<<<<<<<<<
 * K_QUOTEDBL = SDLK_QUOTEDBL
 * K_HASH = SDLK_HASH
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_EXCLAIM); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_EXCLAIM, __pyx_t_1) < 0) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":11
 * K_SPACE = SDLK_SPACE
 * K_EXCLAIM = SDLK_EXCLAIM
 * K_QUOTEDBL = SDLK_QUOTEDBL             # <<<<<<<<<<<<<<
 * K_HASH = SDLK_HASH
 * K_PERCENT = SDLK_PERCENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_QUOTEDBL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_QUOTEDBL, __pyx_t_1) < 0) __PYX_ERR(2, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":12
 * K_EXCLAIM = SDLK_EXCLAIM
 * K_QUOTEDBL = SDLK_QUOTEDBL
 * K_HASH = SDLK_HASH             # <<<<<<<<<<<<<<
 * K_PERCENT = SDLK_PERCENT
 * K_DOLLAR = SDLK_DOLLAR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_HASH); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_HASH, __pyx_t_1) < 0) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":13
 * K_QUOTEDBL = SDLK_QUOTEDBL
 * K_HASH = SDLK_HASH
 * K_PERCENT = SDLK_PERCENT             # <<<<<<<<<<<<<<
 * K_DOLLAR = SDLK_DOLLAR
 * K_AMPERSAND = SDLK_AMPERSAND
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_PERCENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_PERCENT, __pyx_t_1) < 0) __PYX_ERR(2, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":14
 * K_HASH = SDLK_HASH
 * K_PERCENT = SDLK_PERCENT
 * K_DOLLAR = SDLK_DOLLAR             # <<<<<<<<<<<<<<
 * K_AMPERSAND = SDLK_AMPERSAND
 * K_QUOTE = SDLK_QUOTE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_DOLLAR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_DOLLAR, __pyx_t_1) < 0) __PYX_ERR(2, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":15
 * K_PERCENT = SDLK_PERCENT
 * K_DOLLAR = SDLK_DOLLAR
 * K_AMPERSAND = SDLK_AMPERSAND             # <<<<<<<<<<<<<<
 * K_QUOTE = SDLK_QUOTE
 * K_LEFTPAREN = SDLK_LEFTPAREN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AMPERSAND); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AMPERSAND, __pyx_t_1) < 0) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":16
 * K_DOLLAR = SDLK_DOLLAR
 * K_AMPERSAND = SDLK_AMPERSAND
 * K_QUOTE = SDLK_QUOTE             # <<<<<<<<<<<<<<
 * K_LEFTPAREN = SDLK_LEFTPAREN
 * K_RIGHTPAREN = SDLK_RIGHTPAREN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_QUOTE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_QUOTE, __pyx_t_1) < 0) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":17
 * K_AMPERSAND = SDLK_AMPERSAND
 * K_QUOTE = SDLK_QUOTE
 * K_LEFTPAREN = SDLK_LEFTPAREN             # <<<<<<<<<<<<<<
 * K_RIGHTPAREN = SDLK_RIGHTPAREN
 * K_ASTERISK = SDLK_ASTERISK
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_LEFTPAREN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_LEFTPAREN, __pyx_t_1) < 0) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":18
 * K_QUOTE = SDLK_QUOTE
 * K_LEFTPAREN = SDLK_LEFTPAREN
 * K_RIGHTPAREN = SDLK_RIGHTPAREN             # <<<<<<<<<<<<<<
 * K_ASTERISK = SDLK_ASTERISK
 * K_PLUS = SDLK_PLUS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_RIGHTPAREN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_RIGHTPAREN, __pyx_t_1) < 0) __PYX_ERR(2, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":19
 * K_LEFTPAREN = SDLK_LEFTPAREN
 * K_RIGHTPAREN = SDLK_RIGHTPAREN
 * K_ASTERISK = SDLK_ASTERISK             # <<<<<<<<<<<<<<
 * K_PLUS = SDLK_PLUS
 * K_COMMA = SDLK_COMMA
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_ASTERISK); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_ASTERISK, __pyx_t_1) < 0) __PYX_ERR(2, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":20
 * K_RIGHTPAREN = SDLK_RIGHTPAREN
 * K_ASTERISK = SDLK_ASTERISK
 * K_PLUS = SDLK_PLUS             # <<<<<<<<<<<<<<
 * K_COMMA = SDLK_COMMA
 * K_MINUS = SDLK_MINUS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_PLUS); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_PLUS, __pyx_t_1) < 0) __PYX_ERR(2, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":21
 * K_ASTERISK = SDLK_ASTERISK
 * K_PLUS = SDLK_PLUS
 * K_COMMA = SDLK_COMMA             # <<<<<<<<<<<<<<
 * K_MINUS = SDLK_MINUS
 * K_PERIOD = SDLK_PERIOD
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_COMMA); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_COMMA, __pyx_t_1) < 0) __PYX_ERR(2, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":22
 * K_PLUS = SDLK_PLUS
 * K_COMMA = SDLK_COMMA
 * K_MINUS = SDLK_MINUS             # <<<<<<<<<<<<<<
 * K_PERIOD = SDLK_PERIOD
 * K_SLASH = SDLK_SLASH
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_MINUS); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_MINUS, __pyx_t_1) < 0) __PYX_ERR(2, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":23
 * K_COMMA = SDLK_COMMA
 * K_MINUS = SDLK_MINUS
 * K_PERIOD = SDLK_PERIOD             # <<<<<<<<<<<<<<
 * K_SLASH = SDLK_SLASH
 * K_0 = SDLK_0
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_PERIOD); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_PERIOD, __pyx_t_1) < 0) __PYX_ERR(2, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":24
 * K_MINUS = SDLK_MINUS
 * K_PERIOD = SDLK_PERIOD
 * K_SLASH = SDLK_SLASH             # <<<<<<<<<<<<<<
 * K_0 = SDLK_0
 * K_1 = SDLK_1
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_SLASH); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_SLASH, __pyx_t_1) < 0) __PYX_ERR(2, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":25
 * K_PERIOD = SDLK_PERIOD
 * K_SLASH = SDLK_SLASH
 * K_0 = SDLK_0             # <<<<<<<<<<<<<<
 * K_1 = SDLK_1
 * K_2 = SDLK_2
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_0); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_0, __pyx_t_1) < 0) __PYX_ERR(2, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":26
 * K_SLASH = SDLK_SLASH
 * K_0 = SDLK_0
 * K_1 = SDLK_1             # <<<<<<<<<<<<<<
 * K_2 = SDLK_2
 * K_3 = SDLK_3
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_1, __pyx_t_1) < 0) __PYX_ERR(2, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":27
 * K_0 = SDLK_0
 * K_1 = SDLK_1
 * K_2 = SDLK_2             # <<<<<<<<<<<<<<
 * K_3 = SDLK_3
 * K_4 = SDLK_4
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_2, __pyx_t_1) < 0) __PYX_ERR(2, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":28
 * K_1 = SDLK_1
 * K_2 = SDLK_2
 * K_3 = SDLK_3             # <<<<<<<<<<<<<<
 * K_4 = SDLK_4
 * K_5 = SDLK_5
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_3, __pyx_t_1) < 0) __PYX_ERR(2, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":29
 * K_2 = SDLK_2
 * K_3 = SDLK_3
 * K_4 = SDLK_4             # <<<<<<<<<<<<<<
 * K_5 = SDLK_5
 * K_6 = SDLK_6
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_4, __pyx_t_1) < 0) __PYX_ERR(2, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":30
 * K_3 = SDLK_3
 * K_4 = SDLK_4
 * K_5 = SDLK_5             # <<<<<<<<<<<<<<
 * K_6 = SDLK_6
 * K_7 = SDLK_7
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_5, __pyx_t_1) < 0) __PYX_ERR(2, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":31
 * K_4 = SDLK_4
 * K_5 = SDLK_5
 * K_6 = SDLK_6             # <<<<<<<<<<<<<<
 * K_7 = SDLK_7
 * K_8 = SDLK_8
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_6, __pyx_t_1) < 0) __PYX_ERR(2, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":32
 * K_5 = SDLK_5
 * K_6 = SDLK_6
 * K_7 = SDLK_7             # <<<<<<<<<<<<<<
 * K_8 = SDLK_8
 * K_9 = SDLK_9
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_7, __pyx_t_1) < 0) __PYX_ERR(2, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":33
 * K_6 = SDLK_6
 * K_7 = SDLK_7
 * K_8 = SDLK_8             # <<<<<<<<<<<<<<
 * K_9 = SDLK_9
 * K_COLON = SDLK_COLON
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_8, __pyx_t_1) < 0) __PYX_ERR(2, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":34
 * K_7 = SDLK_7
 * K_8 = SDLK_8
 * K_9 = SDLK_9             # <<<<<<<<<<<<<<
 * K_COLON = SDLK_COLON
 * K_SEMICOLON = SDLK_SEMICOLON
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_9, __pyx_t_1) < 0) __PYX_ERR(2, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":35
 * K_8 = SDLK_8
 * K_9 = SDLK_9
 * K_COLON = SDLK_COLON             # <<<<<<<<<<<<<<
 * K_SEMICOLON = SDLK_SEMICOLON
 * K_LESS = SDLK_LESS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_COLON); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_COLON, __pyx_t_1) < 0) __PYX_ERR(2, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":36
 * K_9 = SDLK_9
 * K_COLON = SDLK_COLON
 * K_SEMICOLON = SDLK_SEMICOLON             # <<<<<<<<<<<<<<
 * K_LESS = SDLK_LESS
 * K_EQUALS = SDLK_EQUALS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_SEMICOLON); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_SEMICOLON, __pyx_t_1) < 0) __PYX_ERR(2, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":37
 * K_COLON = SDLK_COLON
 * K_SEMICOLON = SDLK_SEMICOLON
 * K_LESS = SDLK_LESS             # <<<<<<<<<<<<<<
 * K_EQUALS = SDLK_EQUALS
 * K_GREATER = SDLK_GREATER
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_LESS); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_LESS, __pyx_t_1) < 0) __PYX_ERR(2, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":38
 * K_SEMICOLON = SDLK_SEMICOLON
 * K_LESS = SDLK_LESS
 * K_EQUALS = SDLK_EQUALS             # <<<<<<<<<<<<<<
 * K_GREATER = SDLK_GREATER
 * K_QUESTION = SDLK_QUESTION
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_EQUALS); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_EQUALS, __pyx_t_1) < 0) __PYX_ERR(2, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":39
 * K_LESS = SDLK_LESS
 * K_EQUALS = SDLK_EQUALS
 * K_GREATER = SDLK_GREATER             # <<<<<<<<<<<<<<
 * K_QUESTION = SDLK_QUESTION
 * K_AT = SDLK_AT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_GREATER); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_GREATER, __pyx_t_1) < 0) __PYX_ERR(2, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":40
 * K_EQUALS = SDLK_EQUALS
 * K_GREATER = SDLK_GREATER
 * K_QUESTION = SDLK_QUESTION             # <<<<<<<<<<<<<<
 * K_AT = SDLK_AT
 * K_LEFTBRACKET = SDLK_LEFTBRACKET
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_QUESTION); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_QUESTION, __pyx_t_1) < 0) __PYX_ERR(2, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":41
 * K_GREATER = SDLK_GREATER
 * K_QUESTION = SDLK_QUESTION
 * K_AT = SDLK_AT             # <<<<<<<<<<<<<<
 * K_LEFTBRACKET = SDLK_LEFTBRACKET
 * K_BACKSLASH = SDLK_BACKSLASH
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AT, __pyx_t_1) < 0) __PYX_ERR(2, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":42
 * K_QUESTION = SDLK_QUESTION
 * K_AT = SDLK_AT
 * K_LEFTBRACKET = SDLK_LEFTBRACKET             # <<<<<<<<<<<<<<
 * K_BACKSLASH = SDLK_BACKSLASH
 * K_RIGHTBRACKET = SDLK_RIGHTBRACKET
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_LEFTBRACKET); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_LEFTBRACKET, __pyx_t_1) < 0) __PYX_ERR(2, 42, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":43
 * K_AT = SDLK_AT
 * K_LEFTBRACKET = SDLK_LEFTBRACKET
 * K_BACKSLASH = SDLK_BACKSLASH             # <<<<<<<<<<<<<<
 * K_RIGHTBRACKET = SDLK_RIGHTBRACKET
 * K_CARET = SDLK_CARET
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_BACKSLASH); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_BACKSLASH, __pyx_t_1) < 0) __PYX_ERR(2, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":44
 * K_LEFTBRACKET = SDLK_LEFTBRACKET
 * K_BACKSLASH = SDLK_BACKSLASH
 * K_RIGHTBRACKET = SDLK_RIGHTBRACKET             # <<<<<<<<<<<<<<
 * K_CARET = SDLK_CARET
 * K_UNDERSCORE = SDLK_UNDERSCORE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_RIGHTBRACKET); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_RIGHTBRACKET, __pyx_t_1) < 0) __PYX_ERR(2, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":45
 * K_BACKSLASH = SDLK_BACKSLASH
 * K_RIGHTBRACKET = SDLK_RIGHTBRACKET
 * K_CARET = SDLK_CARET             # <<<<<<<<<<<<<<
 * K_UNDERSCORE = SDLK_UNDERSCORE
 * K_BACKQUOTE = SDLK_BACKQUOTE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_CARET); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_CARET, __pyx_t_1) < 0) __PYX_ERR(2, 45, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":46
 * K_RIGHTBRACKET = SDLK_RIGHTBRACKET
 * K_CARET = SDLK_CARET
 * K_UNDERSCORE = SDLK_UNDERSCORE             # <<<<<<<<<<<<<<
 * K_BACKQUOTE = SDLK_BACKQUOTE
 * K_a = SDLK_a
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_UNDERSCORE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_UNDERSCORE, __pyx_t_1) < 0) __PYX_ERR(2, 46, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":47
 * K_CARET = SDLK_CARET
 * K_UNDERSCORE = SDLK_UNDERSCORE
 * K_BACKQUOTE = SDLK_BACKQUOTE             # <<<<<<<<<<<<<<
 * K_a = SDLK_a
 * K_b = SDLK_b
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_BACKQUOTE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_BACKQUOTE, __pyx_t_1) < 0) __PYX_ERR(2, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":48
 * K_UNDERSCORE = SDLK_UNDERSCORE
 * K_BACKQUOTE = SDLK_BACKQUOTE
 * K_a = SDLK_a             # <<<<<<<<<<<<<<
 * K_b = SDLK_b
 * K_c = SDLK_c
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_a); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_a, __pyx_t_1) < 0) __PYX_ERR(2, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":49
 * K_BACKQUOTE = SDLK_BACKQUOTE
 * K_a = SDLK_a
 * K_b = SDLK_b             # <<<<<<<<<<<<<<
 * K_c = SDLK_c
 * K_d = SDLK_d
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_b); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_b, __pyx_t_1) < 0) __PYX_ERR(2, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":50
 * K_a = SDLK_a
 * K_b = SDLK_b
 * K_c = SDLK_c             # <<<<<<<<<<<<<<
 * K_d = SDLK_d
 * K_e = SDLK_e
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_c); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_c, __pyx_t_1) < 0) __PYX_ERR(2, 50, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":51
 * K_b = SDLK_b
 * K_c = SDLK_c
 * K_d = SDLK_d             # <<<<<<<<<<<<<<
 * K_e = SDLK_e
 * K_f = SDLK_f
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_d); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_d, __pyx_t_1) < 0) __PYX_ERR(2, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":52
 * K_c = SDLK_c
 * K_d = SDLK_d
 * K_e = SDLK_e             # <<<<<<<<<<<<<<
 * K_f = SDLK_f
 * K_g = SDLK_g
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_e); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_e, __pyx_t_1) < 0) __PYX_ERR(2, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":53
 * K_d = SDLK_d
 * K_e = SDLK_e
 * K_f = SDLK_f             # <<<<<<<<<<<<<<
 * K_g = SDLK_g
 * K_h = SDLK_h
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_f); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_f, __pyx_t_1) < 0) __PYX_ERR(2, 53, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":54
 * K_e = SDLK_e
 * K_f = SDLK_f
 * K_g = SDLK_g             # <<<<<<<<<<<<<<
 * K_h = SDLK_h
 * K_i = SDLK_i
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_g); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_g, __pyx_t_1) < 0) __PYX_ERR(2, 54, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":55
 * K_f = SDLK_f
 * K_g = SDLK_g
 * K_h = SDLK_h             # <<<<<<<<<<<<<<
 * K_i = SDLK_i
 * K_j = SDLK_j
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_h); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_h, __pyx_t_1) < 0) __PYX_ERR(2, 55, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":56
 * K_g = SDLK_g
 * K_h = SDLK_h
 * K_i = SDLK_i             # <<<<<<<<<<<<<<
 * K_j = SDLK_j
 * K_k = SDLK_k
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_i, __pyx_t_1) < 0) __PYX_ERR(2, 56, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":57
 * K_h = SDLK_h
 * K_i = SDLK_i
 * K_j = SDLK_j             # <<<<<<<<<<<<<<
 * K_k = SDLK_k
 * K_l = SDLK_l
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_j); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_j, __pyx_t_1) < 0) __PYX_ERR(2, 57, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":58
 * K_i = SDLK_i
 * K_j = SDLK_j
 * K_k = SDLK_k             # <<<<<<<<<<<<<<
 * K_l = SDLK_l
 * K_m = SDLK_m
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_k); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_k, __pyx_t_1) < 0) __PYX_ERR(2, 58, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":59
 * K_j = SDLK_j
 * K_k = SDLK_k
 * K_l = SDLK_l             # <<<<<<<<<<<<<<
 * K_m = SDLK_m
 * K_n = SDLK_n
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_l); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_l, __pyx_t_1) < 0) __PYX_ERR(2, 59, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":60
 * K_k = SDLK_k
 * K_l = SDLK_l
 * K_m = SDLK_m             # <<<<<<<<<<<<<<
 * K_n = SDLK_n
 * K_o = SDLK_o
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_m); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_m, __pyx_t_1) < 0) __PYX_ERR(2, 60, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":61
 * K_l = SDLK_l
 * K_m = SDLK_m
 * K_n = SDLK_n             # <<<<<<<<<<<<<<
 * K_o = SDLK_o
 * K_p = SDLK_p
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_n, __pyx_t_1) < 0) __PYX_ERR(2, 61, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":62
 * K_m = SDLK_m
 * K_n = SDLK_n
 * K_o = SDLK_o             # <<<<<<<<<<<<<<
 * K_p = SDLK_p
 * K_q = SDLK_q
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_o); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_o, __pyx_t_1) < 0) __PYX_ERR(2, 62, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":63
 * K_n = SDLK_n
 * K_o = SDLK_o
 * K_p = SDLK_p             # <<<<<<<<<<<<<<
 * K_q = SDLK_q
 * K_r = SDLK_r
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_p); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_p, __pyx_t_1) < 0) __PYX_ERR(2, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":64
 * K_o = SDLK_o
 * K_p = SDLK_p
 * K_q = SDLK_q             # <<<<<<<<<<<<<<
 * K_r = SDLK_r
 * K_s = SDLK_s
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_q); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_q, __pyx_t_1) < 0) __PYX_ERR(2, 64, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":65
 * K_p = SDLK_p
 * K_q = SDLK_q
 * K_r = SDLK_r             # <<<<<<<<<<<<<<
 * K_s = SDLK_s
 * K_t = SDLK_t
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_r); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_r, __pyx_t_1) < 0) __PYX_ERR(2, 65, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":66
 * K_q = SDLK_q
 * K_r = SDLK_r
 * K_s = SDLK_s             # <<<<<<<<<<<<<<
 * K_t = SDLK_t
 * K_u = SDLK_u
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_s); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_s, __pyx_t_1) < 0) __PYX_ERR(2, 66, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":67
 * K_r = SDLK_r
 * K_s = SDLK_s
 * K_t = SDLK_t             # <<<<<<<<<<<<<<
 * K_u = SDLK_u
 * K_v = SDLK_v
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_t); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_t, __pyx_t_1) < 0) __PYX_ERR(2, 67, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":68
 * K_s = SDLK_s
 * K_t = SDLK_t
 * K_u = SDLK_u             # <<<<<<<<<<<<<<
 * K_v = SDLK_v
 * K_w = SDLK_w
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_u); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_u, __pyx_t_1) < 0) __PYX_ERR(2, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":69
 * K_t = SDLK_t
 * K_u = SDLK_u
 * K_v = SDLK_v             # <<<<<<<<<<<<<<
 * K_w = SDLK_w
 * K_x = SDLK_x
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_v, __pyx_t_1) < 0) __PYX_ERR(2, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":70
 * K_u = SDLK_u
 * K_v = SDLK_v
 * K_w = SDLK_w             # <<<<<<<<<<<<<<
 * K_x = SDLK_x
 * K_y = SDLK_y
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_w); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_w, __pyx_t_1) < 0) __PYX_ERR(2, 70, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":71
 * K_v = SDLK_v
 * K_w = SDLK_w
 * K_x = SDLK_x             # <<<<<<<<<<<<<<
 * K_y = SDLK_y
 * K_z = SDLK_z
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_x); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_x, __pyx_t_1) < 0) __PYX_ERR(2, 71, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":72
 * K_w = SDLK_w
 * K_x = SDLK_x
 * K_y = SDLK_y             # <<<<<<<<<<<<<<
 * K_z = SDLK_z
 * K_CAPSLOCK = SDLK_CAPSLOCK
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_y); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 72, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_y, __pyx_t_1) < 0) __PYX_ERR(2, 72, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":73
 * K_x = SDLK_x
 * K_y = SDLK_y
 * K_z = SDLK_z             # <<<<<<<<<<<<<<
 * K_CAPSLOCK = SDLK_CAPSLOCK
 * K_F1 = SDLK_F1
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_z); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_z, __pyx_t_1) < 0) __PYX_ERR(2, 73, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":74
 * K_y = SDLK_y
 * K_z = SDLK_z
 * K_CAPSLOCK = SDLK_CAPSLOCK             # <<<<<<<<<<<<<<
 * K_F1 = SDLK_F1
 * K_F2 = SDLK_F2
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_CAPSLOCK); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_CAPSLOCK, __pyx_t_1) < 0) __PYX_ERR(2, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":75
 * K_z = SDLK_z
 * K_CAPSLOCK = SDLK_CAPSLOCK
 * K_F1 = SDLK_F1             # <<<<<<<<<<<<<<
 * K_F2 = SDLK_F2
 * K_F3 = SDLK_F3
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F1, __pyx_t_1) < 0) __PYX_ERR(2, 75, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":76
 * K_CAPSLOCK = SDLK_CAPSLOCK
 * K_F1 = SDLK_F1
 * K_F2 = SDLK_F2             # <<<<<<<<<<<<<<
 * K_F3 = SDLK_F3
 * K_F4 = SDLK_F4
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F2, __pyx_t_1) < 0) __PYX_ERR(2, 76, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":77
 * K_F1 = SDLK_F1
 * K_F2 = SDLK_F2
 * K_F3 = SDLK_F3             # <<<<<<<<<<<<<<
 * K_F4 = SDLK_F4
 * K_F5 = SDLK_F5
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F3, __pyx_t_1) < 0) __PYX_ERR(2, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":78
 * K_F2 = SDLK_F2
 * K_F3 = SDLK_F3
 * K_F4 = SDLK_F4             # <<<<<<<<<<<<<<
 * K_F5 = SDLK_F5
 * K_F6 = SDLK_F6
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F4); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F4, __pyx_t_1) < 0) __PYX_ERR(2, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":79
 * K_F3 = SDLK_F3
 * K_F4 = SDLK_F4
 * K_F5 = SDLK_F5             # <<<<<<<<<<<<<<
 * K_F6 = SDLK_F6
 * K_F7 = SDLK_F7
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F5); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F5, __pyx_t_1) < 0) __PYX_ERR(2, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":80
 * K_F4 = SDLK_F4
 * K_F5 = SDLK_F5
 * K_F6 = SDLK_F6             # <<<<<<<<<<<<<<
 * K_F7 = SDLK_F7
 * K_F8 = SDLK_F8
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F6); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F6, __pyx_t_1) < 0) __PYX_ERR(2, 80, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":81
 * K_F5 = SDLK_F5
 * K_F6 = SDLK_F6
 * K_F7 = SDLK_F7             # <<<<<<<<<<<<<<
 * K_F8 = SDLK_F8
 * K_F9 = SDLK_F9
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F7); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F7, __pyx_t_1) < 0) __PYX_ERR(2, 81, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":82
 * K_F6 = SDLK_F6
 * K_F7 = SDLK_F7
 * K_F8 = SDLK_F8             # <<<<<<<<<<<<<<
 * K_F9 = SDLK_F9
 * K_F10 = SDLK_F10
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F8); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F8, __pyx_t_1) < 0) __PYX_ERR(2, 82, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":83
 * K_F7 = SDLK_F7
 * K_F8 = SDLK_F8
 * K_F9 = SDLK_F9             # <<<<<<<<<<<<<<
 * K_F10 = SDLK_F10
 * K_F11 = SDLK_F11
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F9); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F9, __pyx_t_1) < 0) __PYX_ERR(2, 83, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":84
 * K_F8 = SDLK_F8
 * K_F9 = SDLK_F9
 * K_F10 = SDLK_F10             # <<<<<<<<<<<<<<
 * K_F11 = SDLK_F11
 * K_F12 = SDLK_F12
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F10); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F10, __pyx_t_1) < 0) __PYX_ERR(2, 84, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":85
 * K_F9 = SDLK_F9
 * K_F10 = SDLK_F10
 * K_F11 = SDLK_F11             # <<<<<<<<<<<<<<
 * K_F12 = SDLK_F12
 * K_PRINTSCREEN = SDLK_PRINTSCREEN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F11); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F11, __pyx_t_1) < 0) __PYX_ERR(2, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":86
 * K_F10 = SDLK_F10
 * K_F11 = SDLK_F11
 * K_F12 = SDLK_F12             # <<<<<<<<<<<<<<
 * K_PRINTSCREEN = SDLK_PRINTSCREEN
 * K_SCROLLLOCK = SDLK_SCROLLLOCK
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F12); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F12, __pyx_t_1) < 0) __PYX_ERR(2, 86, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":87
 * K_F11 = SDLK_F11
 * K_F12 = SDLK_F12
 * K_PRINTSCREEN = SDLK_PRINTSCREEN             # <<<<<<<<<<<<<<
 * K_SCROLLLOCK = SDLK_SCROLLLOCK
 * K_PAUSE = SDLK_PAUSE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_PRINTSCREEN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_PRINTSCREEN, __pyx_t_1) < 0) __PYX_ERR(2, 87, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":88
 * K_F12 = SDLK_F12
 * K_PRINTSCREEN = SDLK_PRINTSCREEN
 * K_SCROLLLOCK = SDLK_SCROLLLOCK             # <<<<<<<<<<<<<<
 * K_PAUSE = SDLK_PAUSE
 * K_INSERT = SDLK_INSERT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_SCROLLLOCK); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_SCROLLLOCK, __pyx_t_1) < 0) __PYX_ERR(2, 88, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":89
 * K_PRINTSCREEN = SDLK_PRINTSCREEN
 * K_SCROLLLOCK = SDLK_SCROLLLOCK
 * K_PAUSE = SDLK_PAUSE             # <<<<<<<<<<<<<<
 * K_INSERT = SDLK_INSERT
 * K_HOME = SDLK_HOME
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_PAUSE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_PAUSE, __pyx_t_1) < 0) __PYX_ERR(2, 89, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":90
 * K_SCROLLLOCK = SDLK_SCROLLLOCK
 * K_PAUSE = SDLK_PAUSE
 * K_INSERT = SDLK_INSERT             # <<<<<<<<<<<<<<
 * K_HOME = SDLK_HOME
 * K_PAGEUP = SDLK_PAGEUP
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_INSERT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_INSERT, __pyx_t_1) < 0) __PYX_ERR(2, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":91
 * K_PAUSE = SDLK_PAUSE
 * K_INSERT = SDLK_INSERT
 * K_HOME = SDLK_HOME             # <<<<<<<<<<<<<<
 * K_PAGEUP = SDLK_PAGEUP
 * K_DELETE = SDLK_DELETE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_HOME); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_HOME, __pyx_t_1) < 0) __PYX_ERR(2, 91, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":92
 * K_INSERT = SDLK_INSERT
 * K_HOME = SDLK_HOME
 * K_PAGEUP = SDLK_PAGEUP             # <<<<<<<<<<<<<<
 * K_DELETE = SDLK_DELETE
 * K_END = SDLK_END
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_PAGEUP); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_PAGEUP, __pyx_t_1) < 0) __PYX_ERR(2, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":93
 * K_HOME = SDLK_HOME
 * K_PAGEUP = SDLK_PAGEUP
 * K_DELETE = SDLK_DELETE             # <<<<<<<<<<<<<<
 * K_END = SDLK_END
 * K_PAGEDOWN = SDLK_PAGEDOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_DELETE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_DELETE, __pyx_t_1) < 0) __PYX_ERR(2, 93, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":94
 * K_PAGEUP = SDLK_PAGEUP
 * K_DELETE = SDLK_DELETE
 * K_END = SDLK_END             # <<<<<<<<<<<<<<
 * K_PAGEDOWN = SDLK_PAGEDOWN
 * K_RIGHT = SDLK_RIGHT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_END); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_END, __pyx_t_1) < 0) __PYX_ERR(2, 94, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":95
 * K_DELETE = SDLK_DELETE
 * K_END = SDLK_END
 * K_PAGEDOWN = SDLK_PAGEDOWN             # <<<<<<<<<<<<<<
 * K_RIGHT = SDLK_RIGHT
 * K_LEFT = SDLK_LEFT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_PAGEDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_PAGEDOWN, __pyx_t_1) < 0) __PYX_ERR(2, 95, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":96
 * K_END = SDLK_END
 * K_PAGEDOWN = SDLK_PAGEDOWN
 * K_RIGHT = SDLK_RIGHT             # <<<<<<<<<<<<<<
 * K_LEFT = SDLK_LEFT
 * K_DOWN = SDLK_DOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_RIGHT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_RIGHT, __pyx_t_1) < 0) __PYX_ERR(2, 96, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":97
 * K_PAGEDOWN = SDLK_PAGEDOWN
 * K_RIGHT = SDLK_RIGHT
 * K_LEFT = SDLK_LEFT             # <<<<<<<<<<<<<<
 * K_DOWN = SDLK_DOWN
 * K_UP = SDLK_UP
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_LEFT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_LEFT, __pyx_t_1) < 0) __PYX_ERR(2, 97, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":98
 * K_RIGHT = SDLK_RIGHT
 * K_LEFT = SDLK_LEFT
 * K_DOWN = SDLK_DOWN             # <<<<<<<<<<<<<<
 * K_UP = SDLK_UP
 * K_NUMLOCKCLEAR = SDLK_NUMLOCKCLEAR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_DOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_DOWN, __pyx_t_1) < 0) __PYX_ERR(2, 98, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":99
 * K_LEFT = SDLK_LEFT
 * K_DOWN = SDLK_DOWN
 * K_UP = SDLK_UP             # <<<<<<<<<<<<<<
 * K_NUMLOCKCLEAR = SDLK_NUMLOCKCLEAR
 * K_KP_DIVIDE = SDLK_KP_DIVIDE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_UP); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_UP, __pyx_t_1) < 0) __PYX_ERR(2, 99, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":100
 * K_DOWN = SDLK_DOWN
 * K_UP = SDLK_UP
 * K_NUMLOCKCLEAR = SDLK_NUMLOCKCLEAR             # <<<<<<<<<<<<<<
 * K_KP_DIVIDE = SDLK_KP_DIVIDE
 * K_KP_MULTIPLY = SDLK_KP_MULTIPLY
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_NUMLOCKCLEAR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_NUMLOCKCLEAR, __pyx_t_1) < 0) __PYX_ERR(2, 100, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":101
 * K_UP = SDLK_UP
 * K_NUMLOCKCLEAR = SDLK_NUMLOCKCLEAR
 * K_KP_DIVIDE = SDLK_KP_DIVIDE             # <<<<<<<<<<<<<<
 * K_KP_MULTIPLY = SDLK_KP_MULTIPLY
 * K_KP_MINUS = SDLK_KP_MINUS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_DIVIDE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_DIVIDE, __pyx_t_1) < 0) __PYX_ERR(2, 101, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":102
 * K_NUMLOCKCLEAR = SDLK_NUMLOCKCLEAR
 * K_KP_DIVIDE = SDLK_KP_DIVIDE
 * K_KP_MULTIPLY = SDLK_KP_MULTIPLY             # <<<<<<<<<<<<<<
 * K_KP_MINUS = SDLK_KP_MINUS
 * K_KP_PLUS = SDLK_KP_PLUS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_MULTIPLY); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_MULTIPLY, __pyx_t_1) < 0) __PYX_ERR(2, 102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":103
 * K_KP_DIVIDE = SDLK_KP_DIVIDE
 * K_KP_MULTIPLY = SDLK_KP_MULTIPLY
 * K_KP_MINUS = SDLK_KP_MINUS             # <<<<<<<<<<<<<<
 * K_KP_PLUS = SDLK_KP_PLUS
 * K_KP_ENTER = SDLK_KP_ENTER
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_MINUS); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_MINUS, __pyx_t_1) < 0) __PYX_ERR(2, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":104
 * K_KP_MULTIPLY = SDLK_KP_MULTIPLY
 * K_KP_MINUS = SDLK_KP_MINUS
 * K_KP_PLUS = SDLK_KP_PLUS             # <<<<<<<<<<<<<<
 * K_KP_ENTER = SDLK_KP_ENTER
 * K_KP_1 = SDLK_KP_1
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_PLUS); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_PLUS, __pyx_t_1) < 0) __PYX_ERR(2, 104, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":105
 * K_KP_MINUS = SDLK_KP_MINUS
 * K_KP_PLUS = SDLK_KP_PLUS
 * K_KP_ENTER = SDLK_KP_ENTER             # <<<<<<<<<<<<<<
 * K_KP_1 = SDLK_KP_1
 * K_KP_2 = SDLK_KP_2
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_ENTER); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_ENTER, __pyx_t_1) < 0) __PYX_ERR(2, 105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":106
 * K_KP_PLUS = SDLK_KP_PLUS
 * K_KP_ENTER = SDLK_KP_ENTER
 * K_KP_1 = SDLK_KP_1             # <<<<<<<<<<<<<<
 * K_KP_2 = SDLK_KP_2
 * K_KP_3 = SDLK_KP_3
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_1, __pyx_t_1) < 0) __PYX_ERR(2, 106, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":107
 * K_KP_ENTER = SDLK_KP_ENTER
 * K_KP_1 = SDLK_KP_1
 * K_KP_2 = SDLK_KP_2             # <<<<<<<<<<<<<<
 * K_KP_3 = SDLK_KP_3
 * K_KP_4 = SDLK_KP_4
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_2, __pyx_t_1) < 0) __PYX_ERR(2, 107, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":108
 * K_KP_1 = SDLK_KP_1
 * K_KP_2 = SDLK_KP_2
 * K_KP_3 = SDLK_KP_3             # <<<<<<<<<<<<<<
 * K_KP_4 = SDLK_KP_4
 * K_KP_5 = SDLK_KP_5
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_3, __pyx_t_1) < 0) __PYX_ERR(2, 108, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":109
 * K_KP_2 = SDLK_KP_2
 * K_KP_3 = SDLK_KP_3
 * K_KP_4 = SDLK_KP_4             # <<<<<<<<<<<<<<
 * K_KP_5 = SDLK_KP_5
 * K_KP_6 = SDLK_KP_6
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_4, __pyx_t_1) < 0) __PYX_ERR(2, 109, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":110
 * K_KP_3 = SDLK_KP_3
 * K_KP_4 = SDLK_KP_4
 * K_KP_5 = SDLK_KP_5             # <<<<<<<<<<<<<<
 * K_KP_6 = SDLK_KP_6
 * K_KP_7 = SDLK_KP_7
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_5, __pyx_t_1) < 0) __PYX_ERR(2, 110, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":111
 * K_KP_4 = SDLK_KP_4
 * K_KP_5 = SDLK_KP_5
 * K_KP_6 = SDLK_KP_6             # <<<<<<<<<<<<<<
 * K_KP_7 = SDLK_KP_7
 * K_KP_8 = SDLK_KP_8
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_6, __pyx_t_1) < 0) __PYX_ERR(2, 111, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":112
 * K_KP_5 = SDLK_KP_5
 * K_KP_6 = SDLK_KP_6
 * K_KP_7 = SDLK_KP_7             # <<<<<<<<<<<<<<
 * K_KP_8 = SDLK_KP_8
 * K_KP_9 = SDLK_KP_9
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_7, __pyx_t_1) < 0) __PYX_ERR(2, 112, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":113
 * K_KP_6 = SDLK_KP_6
 * K_KP_7 = SDLK_KP_7
 * K_KP_8 = SDLK_KP_8             # <<<<<<<<<<<<<<
 * K_KP_9 = SDLK_KP_9
 * K_KP_0 = SDLK_KP_0
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_8, __pyx_t_1) < 0) __PYX_ERR(2, 113, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":114
 * K_KP_7 = SDLK_KP_7
 * K_KP_8 = SDLK_KP_8
 * K_KP_9 = SDLK_KP_9             # <<<<<<<<<<<<<<
 * K_KP_0 = SDLK_KP_0
 * K_KP_PERIOD = SDLK_KP_PERIOD
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_9, __pyx_t_1) < 0) __PYX_ERR(2, 114, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":115
 * K_KP_8 = SDLK_KP_8
 * K_KP_9 = SDLK_KP_9
 * K_KP_0 = SDLK_KP_0             # <<<<<<<<<<<<<<
 * K_KP_PERIOD = SDLK_KP_PERIOD
 * K_APPLICATION = SDLK_APPLICATION
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_0); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_0, __pyx_t_1) < 0) __PYX_ERR(2, 115, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":116
 * K_KP_9 = SDLK_KP_9
 * K_KP_0 = SDLK_KP_0
 * K_KP_PERIOD = SDLK_KP_PERIOD             # <<<<<<<<<<<<<<
 * K_APPLICATION = SDLK_APPLICATION
 * K_POWER = SDLK_POWER
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_PERIOD); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_PERIOD, __pyx_t_1) < 0) __PYX_ERR(2, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":117
 * K_KP_0 = SDLK_KP_0
 * K_KP_PERIOD = SDLK_KP_PERIOD
 * K_APPLICATION = SDLK_APPLICATION             # <<<<<<<<<<<<<<
 * K_POWER = SDLK_POWER
 * K_KP_EQUALS = SDLK_KP_EQUALS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_APPLICATION); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_APPLICATION, __pyx_t_1) < 0) __PYX_ERR(2, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":118
 * K_KP_PERIOD = SDLK_KP_PERIOD
 * K_APPLICATION = SDLK_APPLICATION
 * K_POWER = SDLK_POWER             # <<<<<<<<<<<<<<
 * K_KP_EQUALS = SDLK_KP_EQUALS
 * K_F13 = SDLK_F13
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_POWER); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_POWER, __pyx_t_1) < 0) __PYX_ERR(2, 118, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":119
 * K_APPLICATION = SDLK_APPLICATION
 * K_POWER = SDLK_POWER
 * K_KP_EQUALS = SDLK_KP_EQUALS             # <<<<<<<<<<<<<<
 * K_F13 = SDLK_F13
 * K_F14 = SDLK_F14
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_EQUALS); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_EQUALS, __pyx_t_1) < 0) __PYX_ERR(2, 119, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":120
 * K_POWER = SDLK_POWER
 * K_KP_EQUALS = SDLK_KP_EQUALS
 * K_F13 = SDLK_F13             # <<<<<<<<<<<<<<
 * K_F14 = SDLK_F14
 * K_F15 = SDLK_F15
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F13); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F13, __pyx_t_1) < 0) __PYX_ERR(2, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":121
 * K_KP_EQUALS = SDLK_KP_EQUALS
 * K_F13 = SDLK_F13
 * K_F14 = SDLK_F14             # <<<<<<<<<<<<<<
 * K_F15 = SDLK_F15
 * K_F16 = SDLK_F16
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F14); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F14, __pyx_t_1) < 0) __PYX_ERR(2, 121, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":122
 * K_F13 = SDLK_F13
 * K_F14 = SDLK_F14
 * K_F15 = SDLK_F15             # <<<<<<<<<<<<<<
 * K_F16 = SDLK_F16
 * K_F17 = SDLK_F17
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F15); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F15, __pyx_t_1) < 0) __PYX_ERR(2, 122, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":123
 * K_F14 = SDLK_F14
 * K_F15 = SDLK_F15
 * K_F16 = SDLK_F16             # <<<<<<<<<<<<<<
 * K_F17 = SDLK_F17
 * K_F18 = SDLK_F18
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F16); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F16, __pyx_t_1) < 0) __PYX_ERR(2, 123, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":124
 * K_F15 = SDLK_F15
 * K_F16 = SDLK_F16
 * K_F17 = SDLK_F17             # <<<<<<<<<<<<<<
 * K_F18 = SDLK_F18
 * K_F19 = SDLK_F19
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F17); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F17, __pyx_t_1) < 0) __PYX_ERR(2, 124, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":125
 * K_F16 = SDLK_F16
 * K_F17 = SDLK_F17
 * K_F18 = SDLK_F18             # <<<<<<<<<<<<<<
 * K_F19 = SDLK_F19
 * K_F20 = SDLK_F20
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F18); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F18, __pyx_t_1) < 0) __PYX_ERR(2, 125, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":126
 * K_F17 = SDLK_F17
 * K_F18 = SDLK_F18
 * K_F19 = SDLK_F19             # <<<<<<<<<<<<<<
 * K_F20 = SDLK_F20
 * K_F21 = SDLK_F21
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F19); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F19, __pyx_t_1) < 0) __PYX_ERR(2, 126, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":127
 * K_F18 = SDLK_F18
 * K_F19 = SDLK_F19
 * K_F20 = SDLK_F20             # <<<<<<<<<<<<<<
 * K_F21 = SDLK_F21
 * K_F22 = SDLK_F22
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F20); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F20, __pyx_t_1) < 0) __PYX_ERR(2, 127, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":128
 * K_F19 = SDLK_F19
 * K_F20 = SDLK_F20
 * K_F21 = SDLK_F21             # <<<<<<<<<<<<<<
 * K_F22 = SDLK_F22
 * K_F23 = SDLK_F23
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F21); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F21, __pyx_t_1) < 0) __PYX_ERR(2, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":129
 * K_F20 = SDLK_F20
 * K_F21 = SDLK_F21
 * K_F22 = SDLK_F22             # <<<<<<<<<<<<<<
 * K_F23 = SDLK_F23
 * K_F24 = SDLK_F24
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F22); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F22, __pyx_t_1) < 0) __PYX_ERR(2, 129, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":130
 * K_F21 = SDLK_F21
 * K_F22 = SDLK_F22
 * K_F23 = SDLK_F23             # <<<<<<<<<<<<<<
 * K_F24 = SDLK_F24
 * K_EXECUTE = SDLK_EXECUTE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F23); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F23, __pyx_t_1) < 0) __PYX_ERR(2, 130, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":131
 * K_F22 = SDLK_F22
 * K_F23 = SDLK_F23
 * K_F24 = SDLK_F24             # <<<<<<<<<<<<<<
 * K_EXECUTE = SDLK_EXECUTE
 * K_HELP = SDLK_HELP
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_F24); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_F24, __pyx_t_1) < 0) __PYX_ERR(2, 131, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":132
 * K_F23 = SDLK_F23
 * K_F24 = SDLK_F24
 * K_EXECUTE = SDLK_EXECUTE             # <<<<<<<<<<<<<<
 * K_HELP = SDLK_HELP
 * K_MENU = SDLK_MENU
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_EXECUTE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_EXECUTE, __pyx_t_1) < 0) __PYX_ERR(2, 132, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":133
 * K_F24 = SDLK_F24
 * K_EXECUTE = SDLK_EXECUTE
 * K_HELP = SDLK_HELP             # <<<<<<<<<<<<<<
 * K_MENU = SDLK_MENU
 * K_SELECT = SDLK_SELECT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_HELP); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_HELP, __pyx_t_1) < 0) __PYX_ERR(2, 133, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":134
 * K_EXECUTE = SDLK_EXECUTE
 * K_HELP = SDLK_HELP
 * K_MENU = SDLK_MENU             # <<<<<<<<<<<<<<
 * K_SELECT = SDLK_SELECT
 * K_STOP = SDLK_STOP
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_MENU); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_MENU, __pyx_t_1) < 0) __PYX_ERR(2, 134, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":135
 * K_HELP = SDLK_HELP
 * K_MENU = SDLK_MENU
 * K_SELECT = SDLK_SELECT             # <<<<<<<<<<<<<<
 * K_STOP = SDLK_STOP
 * K_AGAIN = SDLK_AGAIN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_SELECT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_SELECT, __pyx_t_1) < 0) __PYX_ERR(2, 135, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":136
 * K_MENU = SDLK_MENU
 * K_SELECT = SDLK_SELECT
 * K_STOP = SDLK_STOP             # <<<<<<<<<<<<<<
 * K_AGAIN = SDLK_AGAIN
 * K_UNDO = SDLK_UNDO
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_STOP); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_STOP, __pyx_t_1) < 0) __PYX_ERR(2, 136, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":137
 * K_SELECT = SDLK_SELECT
 * K_STOP = SDLK_STOP
 * K_AGAIN = SDLK_AGAIN             # <<<<<<<<<<<<<<
 * K_UNDO = SDLK_UNDO
 * K_CUT = SDLK_CUT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AGAIN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AGAIN, __pyx_t_1) < 0) __PYX_ERR(2, 137, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":138
 * K_STOP = SDLK_STOP
 * K_AGAIN = SDLK_AGAIN
 * K_UNDO = SDLK_UNDO             # <<<<<<<<<<<<<<
 * K_CUT = SDLK_CUT
 * K_COPY = SDLK_COPY
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_UNDO); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_UNDO, __pyx_t_1) < 0) __PYX_ERR(2, 138, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":139
 * K_AGAIN = SDLK_AGAIN
 * K_UNDO = SDLK_UNDO
 * K_CUT = SDLK_CUT             # <<<<<<<<<<<<<<
 * K_COPY = SDLK_COPY
 * K_PASTE = SDLK_PASTE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_CUT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_CUT, __pyx_t_1) < 0) __PYX_ERR(2, 139, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":140
 * K_UNDO = SDLK_UNDO
 * K_CUT = SDLK_CUT
 * K_COPY = SDLK_COPY             # <<<<<<<<<<<<<<
 * K_PASTE = SDLK_PASTE
 * K_FIND = SDLK_FIND
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_COPY); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_COPY, __pyx_t_1) < 0) __PYX_ERR(2, 140, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":141
 * K_CUT = SDLK_CUT
 * K_COPY = SDLK_COPY
 * K_PASTE = SDLK_PASTE             # <<<<<<<<<<<<<<
 * K_FIND = SDLK_FIND
 * K_MUTE = SDLK_MUTE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_PASTE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_PASTE, __pyx_t_1) < 0) __PYX_ERR(2, 141, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":142
 * K_COPY = SDLK_COPY
 * K_PASTE = SDLK_PASTE
 * K_FIND = SDLK_FIND             # <<<<<<<<<<<<<<
 * K_MUTE = SDLK_MUTE
 * K_VOLUMEUP = SDLK_VOLUMEUP
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_FIND); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_FIND, __pyx_t_1) < 0) __PYX_ERR(2, 142, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":143
 * K_PASTE = SDLK_PASTE
 * K_FIND = SDLK_FIND
 * K_MUTE = SDLK_MUTE             # <<<<<<<<<<<<<<
 * K_VOLUMEUP = SDLK_VOLUMEUP
 * K_VOLUMEDOWN = SDLK_VOLUMEDOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_MUTE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_MUTE, __pyx_t_1) < 0) __PYX_ERR(2, 143, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":144
 * K_FIND = SDLK_FIND
 * K_MUTE = SDLK_MUTE
 * K_VOLUMEUP = SDLK_VOLUMEUP             # <<<<<<<<<<<<<<
 * K_VOLUMEDOWN = SDLK_VOLUMEDOWN
 * K_KP_COMMA = SDLK_KP_COMMA
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_VOLUMEUP); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_VOLUMEUP, __pyx_t_1) < 0) __PYX_ERR(2, 144, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":145
 * K_MUTE = SDLK_MUTE
 * K_VOLUMEUP = SDLK_VOLUMEUP
 * K_VOLUMEDOWN = SDLK_VOLUMEDOWN             # <<<<<<<<<<<<<<
 * K_KP_COMMA = SDLK_KP_COMMA
 * K_KP_EQUALSAS400 = SDLK_KP_EQUALSAS400
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_VOLUMEDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_VOLUMEDOWN, __pyx_t_1) < 0) __PYX_ERR(2, 145, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":146
 * K_VOLUMEUP = SDLK_VOLUMEUP
 * K_VOLUMEDOWN = SDLK_VOLUMEDOWN
 * K_KP_COMMA = SDLK_KP_COMMA             # <<<<<<<<<<<<<<
 * K_KP_EQUALSAS400 = SDLK_KP_EQUALSAS400
 * K_ALTERASE = SDLK_ALTERASE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_COMMA); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_COMMA, __pyx_t_1) < 0) __PYX_ERR(2, 146, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":147
 * K_VOLUMEDOWN = SDLK_VOLUMEDOWN
 * K_KP_COMMA = SDLK_KP_COMMA
 * K_KP_EQUALSAS400 = SDLK_KP_EQUALSAS400             # <<<<<<<<<<<<<<
 * K_ALTERASE = SDLK_ALTERASE
 * K_SYSREQ = SDLK_SYSREQ
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_EQUALSAS400); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_EQUALSAS400, __pyx_t_1) < 0) __PYX_ERR(2, 147, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":148
 * K_KP_COMMA = SDLK_KP_COMMA
 * K_KP_EQUALSAS400 = SDLK_KP_EQUALSAS400
 * K_ALTERASE = SDLK_ALTERASE             # <<<<<<<<<<<<<<
 * K_SYSREQ = SDLK_SYSREQ
 * K_CANCEL = SDLK_CANCEL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_ALTERASE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_ALTERASE, __pyx_t_1) < 0) __PYX_ERR(2, 148, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":149
 * K_KP_EQUALSAS400 = SDLK_KP_EQUALSAS400
 * K_ALTERASE = SDLK_ALTERASE
 * K_SYSREQ = SDLK_SYSREQ             # <<<<<<<<<<<<<<
 * K_CANCEL = SDLK_CANCEL
 * K_CLEAR = SDLK_CLEAR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_SYSREQ); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_SYSREQ, __pyx_t_1) < 0) __PYX_ERR(2, 149, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":150
 * K_ALTERASE = SDLK_ALTERASE
 * K_SYSREQ = SDLK_SYSREQ
 * K_CANCEL = SDLK_CANCEL             # <<<<<<<<<<<<<<
 * K_CLEAR = SDLK_CLEAR
 * K_PRIOR = SDLK_PRIOR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_CANCEL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_CANCEL, __pyx_t_1) < 0) __PYX_ERR(2, 150, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":151
 * K_SYSREQ = SDLK_SYSREQ
 * K_CANCEL = SDLK_CANCEL
 * K_CLEAR = SDLK_CLEAR             # <<<<<<<<<<<<<<
 * K_PRIOR = SDLK_PRIOR
 * K_RETURN2 = SDLK_RETURN2
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_CLEAR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_CLEAR, __pyx_t_1) < 0) __PYX_ERR(2, 151, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":152
 * K_CANCEL = SDLK_CANCEL
 * K_CLEAR = SDLK_CLEAR
 * K_PRIOR = SDLK_PRIOR             # <<<<<<<<<<<<<<
 * K_RETURN2 = SDLK_RETURN2
 * K_SEPARATOR = SDLK_SEPARATOR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_PRIOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_PRIOR, __pyx_t_1) < 0) __PYX_ERR(2, 152, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":153
 * K_CLEAR = SDLK_CLEAR
 * K_PRIOR = SDLK_PRIOR
 * K_RETURN2 = SDLK_RETURN2             # <<<<<<<<<<<<<<
 * K_SEPARATOR = SDLK_SEPARATOR
 * K_OUT = SDLK_OUT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_RETURN2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_RETURN2, __pyx_t_1) < 0) __PYX_ERR(2, 153, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":154
 * K_PRIOR = SDLK_PRIOR
 * K_RETURN2 = SDLK_RETURN2
 * K_SEPARATOR = SDLK_SEPARATOR             # <<<<<<<<<<<<<<
 * K_OUT = SDLK_OUT
 * K_OPER = SDLK_OPER
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_SEPARATOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_SEPARATOR, __pyx_t_1) < 0) __PYX_ERR(2, 154, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":155
 * K_RETURN2 = SDLK_RETURN2
 * K_SEPARATOR = SDLK_SEPARATOR
 * K_OUT = SDLK_OUT             # <<<<<<<<<<<<<<
 * K_OPER = SDLK_OPER
 * K_CLEARAGAIN = SDLK_CLEARAGAIN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_OUT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_OUT, __pyx_t_1) < 0) __PYX_ERR(2, 155, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":156
 * K_SEPARATOR = SDLK_SEPARATOR
 * K_OUT = SDLK_OUT
 * K_OPER = SDLK_OPER             # <<<<<<<<<<<<<<
 * K_CLEARAGAIN = SDLK_CLEARAGAIN
 * K_CRSEL = SDLK_CRSEL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_OPER); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_OPER, __pyx_t_1) < 0) __PYX_ERR(2, 156, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":157
 * K_OUT = SDLK_OUT
 * K_OPER = SDLK_OPER
 * K_CLEARAGAIN = SDLK_CLEARAGAIN             # <<<<<<<<<<<<<<
 * K_CRSEL = SDLK_CRSEL
 * K_EXSEL = SDLK_EXSEL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_CLEARAGAIN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_CLEARAGAIN, __pyx_t_1) < 0) __PYX_ERR(2, 157, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":158
 * K_OPER = SDLK_OPER
 * K_CLEARAGAIN = SDLK_CLEARAGAIN
 * K_CRSEL = SDLK_CRSEL             # <<<<<<<<<<<<<<
 * K_EXSEL = SDLK_EXSEL
 * K_KP_00 = SDLK_KP_00
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_CRSEL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_CRSEL, __pyx_t_1) < 0) __PYX_ERR(2, 158, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":159
 * K_CLEARAGAIN = SDLK_CLEARAGAIN
 * K_CRSEL = SDLK_CRSEL
 * K_EXSEL = SDLK_EXSEL             # <<<<<<<<<<<<<<
 * K_KP_00 = SDLK_KP_00
 * K_KP_000 = SDLK_KP_000
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_EXSEL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_EXSEL, __pyx_t_1) < 0) __PYX_ERR(2, 159, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":160
 * K_CRSEL = SDLK_CRSEL
 * K_EXSEL = SDLK_EXSEL
 * K_KP_00 = SDLK_KP_00             # <<<<<<<<<<<<<<
 * K_KP_000 = SDLK_KP_000
 * K_THOUSANDSSEPARATOR = SDLK_THOUSANDSSEPARATOR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_00); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_00, __pyx_t_1) < 0) __PYX_ERR(2, 160, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":161
 * K_EXSEL = SDLK_EXSEL
 * K_KP_00 = SDLK_KP_00
 * K_KP_000 = SDLK_KP_000             # <<<<<<<<<<<<<<
 * K_THOUSANDSSEPARATOR = SDLK_THOUSANDSSEPARATOR
 * K_DECIMALSEPARATOR = SDLK_DECIMALSEPARATOR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_000); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_000, __pyx_t_1) < 0) __PYX_ERR(2, 161, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":162
 * K_KP_00 = SDLK_KP_00
 * K_KP_000 = SDLK_KP_000
 * K_THOUSANDSSEPARATOR = SDLK_THOUSANDSSEPARATOR             # <<<<<<<<<<<<<<
 * K_DECIMALSEPARATOR = SDLK_DECIMALSEPARATOR
 * K_CURRENCYUNIT = SDLK_CURRENCYUNIT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_THOUSANDSSEPARATOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_THOUSANDSSEPARATOR, __pyx_t_1) < 0) __PYX_ERR(2, 162, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":163
 * K_KP_000 = SDLK_KP_000
 * K_THOUSANDSSEPARATOR = SDLK_THOUSANDSSEPARATOR
 * K_DECIMALSEPARATOR = SDLK_DECIMALSEPARATOR             # <<<<<<<<<<<<<<
 * K_CURRENCYUNIT = SDLK_CURRENCYUNIT
 * K_CURRENCYSUBUNIT = SDLK_CURRENCYSUBUNIT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_DECIMALSEPARATOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_DECIMALSEPARATOR, __pyx_t_1) < 0) __PYX_ERR(2, 163, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":164
 * K_THOUSANDSSEPARATOR = SDLK_THOUSANDSSEPARATOR
 * K_DECIMALSEPARATOR = SDLK_DECIMALSEPARATOR
 * K_CURRENCYUNIT = SDLK_CURRENCYUNIT             # <<<<<<<<<<<<<<
 * K_CURRENCYSUBUNIT = SDLK_CURRENCYSUBUNIT
 * K_KP_LEFTPAREN = SDLK_KP_LEFTPAREN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_CURRENCYUNIT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_CURRENCYUNIT, __pyx_t_1) < 0) __PYX_ERR(2, 164, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":165
 * K_DECIMALSEPARATOR = SDLK_DECIMALSEPARATOR
 * K_CURRENCYUNIT = SDLK_CURRENCYUNIT
 * K_CURRENCYSUBUNIT = SDLK_CURRENCYSUBUNIT             # <<<<<<<<<<<<<<
 * K_KP_LEFTPAREN = SDLK_KP_LEFTPAREN
 * K_KP_RIGHTPAREN = SDLK_KP_RIGHTPAREN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_CURRENCYSUBUNIT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_CURRENCYSUBUNIT, __pyx_t_1) < 0) __PYX_ERR(2, 165, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":166
 * K_CURRENCYUNIT = SDLK_CURRENCYUNIT
 * K_CURRENCYSUBUNIT = SDLK_CURRENCYSUBUNIT
 * K_KP_LEFTPAREN = SDLK_KP_LEFTPAREN             # <<<<<<<<<<<<<<
 * K_KP_RIGHTPAREN = SDLK_KP_RIGHTPAREN
 * K_KP_LEFTBRACE = SDLK_KP_LEFTBRACE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_LEFTPAREN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_LEFTPAREN, __pyx_t_1) < 0) __PYX_ERR(2, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":167
 * K_CURRENCYSUBUNIT = SDLK_CURRENCYSUBUNIT
 * K_KP_LEFTPAREN = SDLK_KP_LEFTPAREN
 * K_KP_RIGHTPAREN = SDLK_KP_RIGHTPAREN             # <<<<<<<<<<<<<<
 * K_KP_LEFTBRACE = SDLK_KP_LEFTBRACE
 * K_KP_RIGHTBRACE = SDLK_KP_RIGHTBRACE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_RIGHTPAREN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_RIGHTPAREN, __pyx_t_1) < 0) __PYX_ERR(2, 167, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":168
 * K_KP_LEFTPAREN = SDLK_KP_LEFTPAREN
 * K_KP_RIGHTPAREN = SDLK_KP_RIGHTPAREN
 * K_KP_LEFTBRACE = SDLK_KP_LEFTBRACE             # <<<<<<<<<<<<<<
 * K_KP_RIGHTBRACE = SDLK_KP_RIGHTBRACE
 * K_KP_TAB = SDLK_KP_TAB
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_LEFTBRACE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_LEFTBRACE, __pyx_t_1) < 0) __PYX_ERR(2, 168, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":169
 * K_KP_RIGHTPAREN = SDLK_KP_RIGHTPAREN
 * K_KP_LEFTBRACE = SDLK_KP_LEFTBRACE
 * K_KP_RIGHTBRACE = SDLK_KP_RIGHTBRACE             # <<<<<<<<<<<<<<
 * K_KP_TAB = SDLK_KP_TAB
 * K_KP_BACKSPACE = SDLK_KP_BACKSPACE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_RIGHTBRACE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_RIGHTBRACE, __pyx_t_1) < 0) __PYX_ERR(2, 169, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":170
 * K_KP_LEFTBRACE = SDLK_KP_LEFTBRACE
 * K_KP_RIGHTBRACE = SDLK_KP_RIGHTBRACE
 * K_KP_TAB = SDLK_KP_TAB             # <<<<<<<<<<<<<<
 * K_KP_BACKSPACE = SDLK_KP_BACKSPACE
 * K_KP_A = SDLK_KP_A
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_TAB); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_TAB, __pyx_t_1) < 0) __PYX_ERR(2, 170, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":171
 * K_KP_RIGHTBRACE = SDLK_KP_RIGHTBRACE
 * K_KP_TAB = SDLK_KP_TAB
 * K_KP_BACKSPACE = SDLK_KP_BACKSPACE             # <<<<<<<<<<<<<<
 * K_KP_A = SDLK_KP_A
 * K_KP_B = SDLK_KP_B
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_BACKSPACE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_BACKSPACE, __pyx_t_1) < 0) __PYX_ERR(2, 171, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":172
 * K_KP_TAB = SDLK_KP_TAB
 * K_KP_BACKSPACE = SDLK_KP_BACKSPACE
 * K_KP_A = SDLK_KP_A             # <<<<<<<<<<<<<<
 * K_KP_B = SDLK_KP_B
 * K_KP_C = SDLK_KP_C
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_A); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_A, __pyx_t_1) < 0) __PYX_ERR(2, 172, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":173
 * K_KP_BACKSPACE = SDLK_KP_BACKSPACE
 * K_KP_A = SDLK_KP_A
 * K_KP_B = SDLK_KP_B             # <<<<<<<<<<<<<<
 * K_KP_C = SDLK_KP_C
 * K_KP_D = SDLK_KP_D
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_B); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_B, __pyx_t_1) < 0) __PYX_ERR(2, 173, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":174
 * K_KP_A = SDLK_KP_A
 * K_KP_B = SDLK_KP_B
 * K_KP_C = SDLK_KP_C             # <<<<<<<<<<<<<<
 * K_KP_D = SDLK_KP_D
 * K_KP_E = SDLK_KP_E
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_C); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_C, __pyx_t_1) < 0) __PYX_ERR(2, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":175
 * K_KP_B = SDLK_KP_B
 * K_KP_C = SDLK_KP_C
 * K_KP_D = SDLK_KP_D             # <<<<<<<<<<<<<<
 * K_KP_E = SDLK_KP_E
 * K_KP_F = SDLK_KP_F
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_D); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_D, __pyx_t_1) < 0) __PYX_ERR(2, 175, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":176
 * K_KP_C = SDLK_KP_C
 * K_KP_D = SDLK_KP_D
 * K_KP_E = SDLK_KP_E             # <<<<<<<<<<<<<<
 * K_KP_F = SDLK_KP_F
 * K_KP_XOR = SDLK_KP_XOR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_E); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_E, __pyx_t_1) < 0) __PYX_ERR(2, 176, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":177
 * K_KP_D = SDLK_KP_D
 * K_KP_E = SDLK_KP_E
 * K_KP_F = SDLK_KP_F             # <<<<<<<<<<<<<<
 * K_KP_XOR = SDLK_KP_XOR
 * K_KP_POWER = SDLK_KP_POWER
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_F); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_F, __pyx_t_1) < 0) __PYX_ERR(2, 177, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":178
 * K_KP_E = SDLK_KP_E
 * K_KP_F = SDLK_KP_F
 * K_KP_XOR = SDLK_KP_XOR             # <<<<<<<<<<<<<<
 * K_KP_POWER = SDLK_KP_POWER
 * K_KP_PERCENT = SDLK_KP_PERCENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_XOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_XOR, __pyx_t_1) < 0) __PYX_ERR(2, 178, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":179
 * K_KP_F = SDLK_KP_F
 * K_KP_XOR = SDLK_KP_XOR
 * K_KP_POWER = SDLK_KP_POWER             # <<<<<<<<<<<<<<
 * K_KP_PERCENT = SDLK_KP_PERCENT
 * K_KP_LESS = SDLK_KP_LESS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_POWER); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_POWER, __pyx_t_1) < 0) __PYX_ERR(2, 179, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":180
 * K_KP_XOR = SDLK_KP_XOR
 * K_KP_POWER = SDLK_KP_POWER
 * K_KP_PERCENT = SDLK_KP_PERCENT             # <<<<<<<<<<<<<<
 * K_KP_LESS = SDLK_KP_LESS
 * K_KP_GREATER = SDLK_KP_GREATER
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_PERCENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_PERCENT, __pyx_t_1) < 0) __PYX_ERR(2, 180, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":181
 * K_KP_POWER = SDLK_KP_POWER
 * K_KP_PERCENT = SDLK_KP_PERCENT
 * K_KP_LESS = SDLK_KP_LESS             # <<<<<<<<<<<<<<
 * K_KP_GREATER = SDLK_KP_GREATER
 * K_KP_AMPERSAND = SDLK_KP_AMPERSAND
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_LESS); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_LESS, __pyx_t_1) < 0) __PYX_ERR(2, 181, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":182
 * K_KP_PERCENT = SDLK_KP_PERCENT
 * K_KP_LESS = SDLK_KP_LESS
 * K_KP_GREATER = SDLK_KP_GREATER             # <<<<<<<<<<<<<<
 * K_KP_AMPERSAND = SDLK_KP_AMPERSAND
 * K_KP_DBLAMPERSAND = SDLK_KP_DBLAMPERSAND
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_GREATER); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_GREATER, __pyx_t_1) < 0) __PYX_ERR(2, 182, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":183
 * K_KP_LESS = SDLK_KP_LESS
 * K_KP_GREATER = SDLK_KP_GREATER
 * K_KP_AMPERSAND = SDLK_KP_AMPERSAND             # <<<<<<<<<<<<<<
 * K_KP_DBLAMPERSAND = SDLK_KP_DBLAMPERSAND
 * K_KP_VERTICALBAR = SDLK_KP_VERTICALBAR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_AMPERSAND); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_AMPERSAND, __pyx_t_1) < 0) __PYX_ERR(2, 183, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":184
 * K_KP_GREATER = SDLK_KP_GREATER
 * K_KP_AMPERSAND = SDLK_KP_AMPERSAND
 * K_KP_DBLAMPERSAND = SDLK_KP_DBLAMPERSAND             # <<<<<<<<<<<<<<
 * K_KP_VERTICALBAR = SDLK_KP_VERTICALBAR
 * K_KP_DBLVERTICALBAR = SDLK_KP_DBLVERTICALBAR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_DBLAMPERSAND); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_DBLAMPERSAND, __pyx_t_1) < 0) __PYX_ERR(2, 184, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":185
 * K_KP_AMPERSAND = SDLK_KP_AMPERSAND
 * K_KP_DBLAMPERSAND = SDLK_KP_DBLAMPERSAND
 * K_KP_VERTICALBAR = SDLK_KP_VERTICALBAR             # <<<<<<<<<<<<<<
 * K_KP_DBLVERTICALBAR = SDLK_KP_DBLVERTICALBAR
 * K_KP_COLON = SDLK_KP_COLON
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_VERTICALBAR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_VERTICALBAR, __pyx_t_1) < 0) __PYX_ERR(2, 185, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":186
 * K_KP_DBLAMPERSAND = SDLK_KP_DBLAMPERSAND
 * K_KP_VERTICALBAR = SDLK_KP_VERTICALBAR
 * K_KP_DBLVERTICALBAR = SDLK_KP_DBLVERTICALBAR             # <<<<<<<<<<<<<<
 * K_KP_COLON = SDLK_KP_COLON
 * K_KP_HASH = SDLK_KP_HASH
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_DBLVERTICALBAR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_DBLVERTICALBAR, __pyx_t_1) < 0) __PYX_ERR(2, 186, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":187
 * K_KP_VERTICALBAR = SDLK_KP_VERTICALBAR
 * K_KP_DBLVERTICALBAR = SDLK_KP_DBLVERTICALBAR
 * K_KP_COLON = SDLK_KP_COLON             # <<<<<<<<<<<<<<
 * K_KP_HASH = SDLK_KP_HASH
 * K_KP_SPACE = SDLK_KP_SPACE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_COLON); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_COLON, __pyx_t_1) < 0) __PYX_ERR(2, 187, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":188
 * K_KP_DBLVERTICALBAR = SDLK_KP_DBLVERTICALBAR
 * K_KP_COLON = SDLK_KP_COLON
 * K_KP_HASH = SDLK_KP_HASH             # <<<<<<<<<<<<<<
 * K_KP_SPACE = SDLK_KP_SPACE
 * K_KP_AT = SDLK_KP_AT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_HASH); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_HASH, __pyx_t_1) < 0) __PYX_ERR(2, 188, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":189
 * K_KP_COLON = SDLK_KP_COLON
 * K_KP_HASH = SDLK_KP_HASH
 * K_KP_SPACE = SDLK_KP_SPACE             # <<<<<<<<<<<<<<
 * K_KP_AT = SDLK_KP_AT
 * K_KP_EXCLAM = SDLK_KP_EXCLAM
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_SPACE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_SPACE, __pyx_t_1) < 0) __PYX_ERR(2, 189, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":190
 * K_KP_HASH = SDLK_KP_HASH
 * K_KP_SPACE = SDLK_KP_SPACE
 * K_KP_AT = SDLK_KP_AT             # <<<<<<<<<<<<<<
 * K_KP_EXCLAM = SDLK_KP_EXCLAM
 * K_KP_MEMSTORE = SDLK_KP_MEMSTORE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_AT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_AT, __pyx_t_1) < 0) __PYX_ERR(2, 190, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":191
 * K_KP_SPACE = SDLK_KP_SPACE
 * K_KP_AT = SDLK_KP_AT
 * K_KP_EXCLAM = SDLK_KP_EXCLAM             # <<<<<<<<<<<<<<
 * K_KP_MEMSTORE = SDLK_KP_MEMSTORE
 * K_KP_MEMRECALL = SDLK_KP_MEMRECALL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_EXCLAM); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_EXCLAM, __pyx_t_1) < 0) __PYX_ERR(2, 191, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":192
 * K_KP_AT = SDLK_KP_AT
 * K_KP_EXCLAM = SDLK_KP_EXCLAM
 * K_KP_MEMSTORE = SDLK_KP_MEMSTORE             # <<<<<<<<<<<<<<
 * K_KP_MEMRECALL = SDLK_KP_MEMRECALL
 * K_KP_MEMCLEAR = SDLK_KP_MEMCLEAR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_MEMSTORE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_MEMSTORE, __pyx_t_1) < 0) __PYX_ERR(2, 192, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":193
 * K_KP_EXCLAM = SDLK_KP_EXCLAM
 * K_KP_MEMSTORE = SDLK_KP_MEMSTORE
 * K_KP_MEMRECALL = SDLK_KP_MEMRECALL             # <<<<<<<<<<<<<<
 * K_KP_MEMCLEAR = SDLK_KP_MEMCLEAR
 * K_KP_MEMADD = SDLK_KP_MEMADD
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_MEMRECALL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_MEMRECALL, __pyx_t_1) < 0) __PYX_ERR(2, 193, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":194
 * K_KP_MEMSTORE = SDLK_KP_MEMSTORE
 * K_KP_MEMRECALL = SDLK_KP_MEMRECALL
 * K_KP_MEMCLEAR = SDLK_KP_MEMCLEAR             # <<<<<<<<<<<<<<
 * K_KP_MEMADD = SDLK_KP_MEMADD
 * K_KP_MEMSUBTRACT = SDLK_KP_MEMSUBTRACT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_MEMCLEAR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_MEMCLEAR, __pyx_t_1) < 0) __PYX_ERR(2, 194, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":195
 * K_KP_MEMRECALL = SDLK_KP_MEMRECALL
 * K_KP_MEMCLEAR = SDLK_KP_MEMCLEAR
 * K_KP_MEMADD = SDLK_KP_MEMADD             # <<<<<<<<<<<<<<
 * K_KP_MEMSUBTRACT = SDLK_KP_MEMSUBTRACT
 * K_KP_MEMMULTIPLY = SDLK_KP_MEMMULTIPLY
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_MEMADD); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_MEMADD, __pyx_t_1) < 0) __PYX_ERR(2, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":196
 * K_KP_MEMCLEAR = SDLK_KP_MEMCLEAR
 * K_KP_MEMADD = SDLK_KP_MEMADD
 * K_KP_MEMSUBTRACT = SDLK_KP_MEMSUBTRACT             # <<<<<<<<<<<<<<
 * K_KP_MEMMULTIPLY = SDLK_KP_MEMMULTIPLY
 * K_KP_MEMDIVIDE = SDLK_KP_MEMDIVIDE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_MEMSUBTRACT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_MEMSUBTRACT, __pyx_t_1) < 0) __PYX_ERR(2, 196, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":197
 * K_KP_MEMADD = SDLK_KP_MEMADD
 * K_KP_MEMSUBTRACT = SDLK_KP_MEMSUBTRACT
 * K_KP_MEMMULTIPLY = SDLK_KP_MEMMULTIPLY             # <<<<<<<<<<<<<<
 * K_KP_MEMDIVIDE = SDLK_KP_MEMDIVIDE
 * K_KP_PLUSMINUS = SDLK_KP_PLUSMINUS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_MEMMULTIPLY); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_MEMMULTIPLY, __pyx_t_1) < 0) __PYX_ERR(2, 197, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":198
 * K_KP_MEMSUBTRACT = SDLK_KP_MEMSUBTRACT
 * K_KP_MEMMULTIPLY = SDLK_KP_MEMMULTIPLY
 * K_KP_MEMDIVIDE = SDLK_KP_MEMDIVIDE             # <<<<<<<<<<<<<<
 * K_KP_PLUSMINUS = SDLK_KP_PLUSMINUS
 * K_KP_CLEAR = SDLK_KP_CLEAR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_MEMDIVIDE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_MEMDIVIDE, __pyx_t_1) < 0) __PYX_ERR(2, 198, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":199
 * K_KP_MEMMULTIPLY = SDLK_KP_MEMMULTIPLY
 * K_KP_MEMDIVIDE = SDLK_KP_MEMDIVIDE
 * K_KP_PLUSMINUS = SDLK_KP_PLUSMINUS             # <<<<<<<<<<<<<<
 * K_KP_CLEAR = SDLK_KP_CLEAR
 * K_KP_CLEARENTRY = SDLK_KP_CLEARENTRY
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_PLUSMINUS); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_PLUSMINUS, __pyx_t_1) < 0) __PYX_ERR(2, 199, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":200
 * K_KP_MEMDIVIDE = SDLK_KP_MEMDIVIDE
 * K_KP_PLUSMINUS = SDLK_KP_PLUSMINUS
 * K_KP_CLEAR = SDLK_KP_CLEAR             # <<<<<<<<<<<<<<
 * K_KP_CLEARENTRY = SDLK_KP_CLEARENTRY
 * K_KP_BINARY = SDLK_KP_BINARY
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_CLEAR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_CLEAR, __pyx_t_1) < 0) __PYX_ERR(2, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":201
 * K_KP_PLUSMINUS = SDLK_KP_PLUSMINUS
 * K_KP_CLEAR = SDLK_KP_CLEAR
 * K_KP_CLEARENTRY = SDLK_KP_CLEARENTRY             # <<<<<<<<<<<<<<
 * K_KP_BINARY = SDLK_KP_BINARY
 * K_KP_OCTAL = SDLK_KP_OCTAL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_CLEARENTRY); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_CLEARENTRY, __pyx_t_1) < 0) __PYX_ERR(2, 201, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":202
 * K_KP_CLEAR = SDLK_KP_CLEAR
 * K_KP_CLEARENTRY = SDLK_KP_CLEARENTRY
 * K_KP_BINARY = SDLK_KP_BINARY             # <<<<<<<<<<<<<<
 * K_KP_OCTAL = SDLK_KP_OCTAL
 * K_KP_DECIMAL = SDLK_KP_DECIMAL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_BINARY); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_BINARY, __pyx_t_1) < 0) __PYX_ERR(2, 202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":203
 * K_KP_CLEARENTRY = SDLK_KP_CLEARENTRY
 * K_KP_BINARY = SDLK_KP_BINARY
 * K_KP_OCTAL = SDLK_KP_OCTAL             # <<<<<<<<<<<<<<
 * K_KP_DECIMAL = SDLK_KP_DECIMAL
 * K_KP_HEXADECIMAL = SDLK_KP_HEXADECIMAL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_OCTAL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_OCTAL, __pyx_t_1) < 0) __PYX_ERR(2, 203, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":204
 * K_KP_BINARY = SDLK_KP_BINARY
 * K_KP_OCTAL = SDLK_KP_OCTAL
 * K_KP_DECIMAL = SDLK_KP_DECIMAL             # <<<<<<<<<<<<<<
 * K_KP_HEXADECIMAL = SDLK_KP_HEXADECIMAL
 * K_LCTRL = SDLK_LCTRL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_DECIMAL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_DECIMAL, __pyx_t_1) < 0) __PYX_ERR(2, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":205
 * K_KP_OCTAL = SDLK_KP_OCTAL
 * K_KP_DECIMAL = SDLK_KP_DECIMAL
 * K_KP_HEXADECIMAL = SDLK_KP_HEXADECIMAL             # <<<<<<<<<<<<<<
 * K_LCTRL = SDLK_LCTRL
 * K_LSHIFT = SDLK_LSHIFT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KP_HEXADECIMAL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KP_HEXADECIMAL, __pyx_t_1) < 0) __PYX_ERR(2, 205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":206
 * K_KP_DECIMAL = SDLK_KP_DECIMAL
 * K_KP_HEXADECIMAL = SDLK_KP_HEXADECIMAL
 * K_LCTRL = SDLK_LCTRL             # <<<<<<<<<<<<<<
 * K_LSHIFT = SDLK_LSHIFT
 * K_LALT = SDLK_LALT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_LCTRL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_LCTRL, __pyx_t_1) < 0) __PYX_ERR(2, 206, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":207
 * K_KP_HEXADECIMAL = SDLK_KP_HEXADECIMAL
 * K_LCTRL = SDLK_LCTRL
 * K_LSHIFT = SDLK_LSHIFT             # <<<<<<<<<<<<<<
 * K_LALT = SDLK_LALT
 * K_LGUI = SDLK_LGUI
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_LSHIFT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_LSHIFT, __pyx_t_1) < 0) __PYX_ERR(2, 207, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":208
 * K_LCTRL = SDLK_LCTRL
 * K_LSHIFT = SDLK_LSHIFT
 * K_LALT = SDLK_LALT             # <<<<<<<<<<<<<<
 * K_LGUI = SDLK_LGUI
 * K_RCTRL = SDLK_RCTRL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_LALT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_LALT, __pyx_t_1) < 0) __PYX_ERR(2, 208, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":209
 * K_LSHIFT = SDLK_LSHIFT
 * K_LALT = SDLK_LALT
 * K_LGUI = SDLK_LGUI             # <<<<<<<<<<<<<<
 * K_RCTRL = SDLK_RCTRL
 * K_RSHIFT = SDLK_RSHIFT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_LGUI); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_LGUI, __pyx_t_1) < 0) __PYX_ERR(2, 209, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":210
 * K_LALT = SDLK_LALT
 * K_LGUI = SDLK_LGUI
 * K_RCTRL = SDLK_RCTRL             # <<<<<<<<<<<<<<
 * K_RSHIFT = SDLK_RSHIFT
 * K_RALT = SDLK_RALT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_RCTRL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_RCTRL, __pyx_t_1) < 0) __PYX_ERR(2, 210, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":211
 * K_LGUI = SDLK_LGUI
 * K_RCTRL = SDLK_RCTRL
 * K_RSHIFT = SDLK_RSHIFT             # <<<<<<<<<<<<<<
 * K_RALT = SDLK_RALT
 * K_RGUI = SDLK_RGUI
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_RSHIFT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_RSHIFT, __pyx_t_1) < 0) __PYX_ERR(2, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":212
 * K_RCTRL = SDLK_RCTRL
 * K_RSHIFT = SDLK_RSHIFT
 * K_RALT = SDLK_RALT             # <<<<<<<<<<<<<<
 * K_RGUI = SDLK_RGUI
 * K_MODE = SDLK_MODE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_RALT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_RALT, __pyx_t_1) < 0) __PYX_ERR(2, 212, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":213
 * K_RSHIFT = SDLK_RSHIFT
 * K_RALT = SDLK_RALT
 * K_RGUI = SDLK_RGUI             # <<<<<<<<<<<<<<
 * K_MODE = SDLK_MODE
 * K_AUDIONEXT = SDLK_AUDIONEXT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_RGUI); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_RGUI, __pyx_t_1) < 0) __PYX_ERR(2, 213, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":214
 * K_RALT = SDLK_RALT
 * K_RGUI = SDLK_RGUI
 * K_MODE = SDLK_MODE             # <<<<<<<<<<<<<<
 * K_AUDIONEXT = SDLK_AUDIONEXT
 * K_AUDIOPREV = SDLK_AUDIOPREV
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_MODE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_MODE, __pyx_t_1) < 0) __PYX_ERR(2, 214, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":215
 * K_RGUI = SDLK_RGUI
 * K_MODE = SDLK_MODE
 * K_AUDIONEXT = SDLK_AUDIONEXT             # <<<<<<<<<<<<<<
 * K_AUDIOPREV = SDLK_AUDIOPREV
 * K_AUDIOSTOP = SDLK_AUDIOSTOP
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AUDIONEXT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AUDIONEXT, __pyx_t_1) < 0) __PYX_ERR(2, 215, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":216
 * K_MODE = SDLK_MODE
 * K_AUDIONEXT = SDLK_AUDIONEXT
 * K_AUDIOPREV = SDLK_AUDIOPREV             # <<<<<<<<<<<<<<
 * K_AUDIOSTOP = SDLK_AUDIOSTOP
 * K_AUDIOPLAY = SDLK_AUDIOPLAY
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AUDIOPREV); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AUDIOPREV, __pyx_t_1) < 0) __PYX_ERR(2, 216, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":217
 * K_AUDIONEXT = SDLK_AUDIONEXT
 * K_AUDIOPREV = SDLK_AUDIOPREV
 * K_AUDIOSTOP = SDLK_AUDIOSTOP             # <<<<<<<<<<<<<<
 * K_AUDIOPLAY = SDLK_AUDIOPLAY
 * K_AUDIOMUTE = SDLK_AUDIOMUTE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AUDIOSTOP); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AUDIOSTOP, __pyx_t_1) < 0) __PYX_ERR(2, 217, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":218
 * K_AUDIOPREV = SDLK_AUDIOPREV
 * K_AUDIOSTOP = SDLK_AUDIOSTOP
 * K_AUDIOPLAY = SDLK_AUDIOPLAY             # <<<<<<<<<<<<<<
 * K_AUDIOMUTE = SDLK_AUDIOMUTE
 * K_MEDIASELECT = SDLK_MEDIASELECT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AUDIOPLAY); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AUDIOPLAY, __pyx_t_1) < 0) __PYX_ERR(2, 218, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":219
 * K_AUDIOSTOP = SDLK_AUDIOSTOP
 * K_AUDIOPLAY = SDLK_AUDIOPLAY
 * K_AUDIOMUTE = SDLK_AUDIOMUTE             # <<<<<<<<<<<<<<
 * K_MEDIASELECT = SDLK_MEDIASELECT
 * K_WWW = SDLK_WWW
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AUDIOMUTE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AUDIOMUTE, __pyx_t_1) < 0) __PYX_ERR(2, 219, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":220
 * K_AUDIOPLAY = SDLK_AUDIOPLAY
 * K_AUDIOMUTE = SDLK_AUDIOMUTE
 * K_MEDIASELECT = SDLK_MEDIASELECT             # <<<<<<<<<<<<<<
 * K_WWW = SDLK_WWW
 * K_MAIL = SDLK_MAIL
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_MEDIASELECT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_MEDIASELECT, __pyx_t_1) < 0) __PYX_ERR(2, 220, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":221
 * K_AUDIOMUTE = SDLK_AUDIOMUTE
 * K_MEDIASELECT = SDLK_MEDIASELECT
 * K_WWW = SDLK_WWW             # <<<<<<<<<<<<<<
 * K_MAIL = SDLK_MAIL
 * K_CALCULATOR = SDLK_CALCULATOR
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_WWW); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_WWW, __pyx_t_1) < 0) __PYX_ERR(2, 221, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":222
 * K_MEDIASELECT = SDLK_MEDIASELECT
 * K_WWW = SDLK_WWW
 * K_MAIL = SDLK_MAIL             # <<<<<<<<<<<<<<
 * K_CALCULATOR = SDLK_CALCULATOR
 * K_COMPUTER = SDLK_COMPUTER
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_MAIL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_MAIL, __pyx_t_1) < 0) __PYX_ERR(2, 222, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":223
 * K_WWW = SDLK_WWW
 * K_MAIL = SDLK_MAIL
 * K_CALCULATOR = SDLK_CALCULATOR             # <<<<<<<<<<<<<<
 * K_COMPUTER = SDLK_COMPUTER
 * K_AC_SEARCH = SDLK_AC_SEARCH
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_CALCULATOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_CALCULATOR, __pyx_t_1) < 0) __PYX_ERR(2, 223, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":224
 * K_MAIL = SDLK_MAIL
 * K_CALCULATOR = SDLK_CALCULATOR
 * K_COMPUTER = SDLK_COMPUTER             # <<<<<<<<<<<<<<
 * K_AC_SEARCH = SDLK_AC_SEARCH
 * K_AC_HOME = SDLK_AC_HOME
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_COMPUTER); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_COMPUTER, __pyx_t_1) < 0) __PYX_ERR(2, 224, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":225
 * K_CALCULATOR = SDLK_CALCULATOR
 * K_COMPUTER = SDLK_COMPUTER
 * K_AC_SEARCH = SDLK_AC_SEARCH             # <<<<<<<<<<<<<<
 * K_AC_HOME = SDLK_AC_HOME
 * K_AC_BACK = SDLK_AC_BACK
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AC_SEARCH); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AC_SEARCH, __pyx_t_1) < 0) __PYX_ERR(2, 225, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":226
 * K_COMPUTER = SDLK_COMPUTER
 * K_AC_SEARCH = SDLK_AC_SEARCH
 * K_AC_HOME = SDLK_AC_HOME             # <<<<<<<<<<<<<<
 * K_AC_BACK = SDLK_AC_BACK
 * K_AC_FORWARD = SDLK_AC_FORWARD
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AC_HOME); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AC_HOME, __pyx_t_1) < 0) __PYX_ERR(2, 226, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":227
 * K_AC_SEARCH = SDLK_AC_SEARCH
 * K_AC_HOME = SDLK_AC_HOME
 * K_AC_BACK = SDLK_AC_BACK             # <<<<<<<<<<<<<<
 * K_AC_FORWARD = SDLK_AC_FORWARD
 * K_AC_STOP = SDLK_AC_STOP
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AC_BACK); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AC_BACK, __pyx_t_1) < 0) __PYX_ERR(2, 227, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":228
 * K_AC_HOME = SDLK_AC_HOME
 * K_AC_BACK = SDLK_AC_BACK
 * K_AC_FORWARD = SDLK_AC_FORWARD             # <<<<<<<<<<<<<<
 * K_AC_STOP = SDLK_AC_STOP
 * K_AC_REFRESH = SDLK_AC_REFRESH
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AC_FORWARD); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AC_FORWARD, __pyx_t_1) < 0) __PYX_ERR(2, 228, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":229
 * K_AC_BACK = SDLK_AC_BACK
 * K_AC_FORWARD = SDLK_AC_FORWARD
 * K_AC_STOP = SDLK_AC_STOP             # <<<<<<<<<<<<<<
 * K_AC_REFRESH = SDLK_AC_REFRESH
 * K_AC_BOOKMARKS = SDLK_AC_BOOKMARKS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AC_STOP); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AC_STOP, __pyx_t_1) < 0) __PYX_ERR(2, 229, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":230
 * K_AC_FORWARD = SDLK_AC_FORWARD
 * K_AC_STOP = SDLK_AC_STOP
 * K_AC_REFRESH = SDLK_AC_REFRESH             # <<<<<<<<<<<<<<
 * K_AC_BOOKMARKS = SDLK_AC_BOOKMARKS
 * K_BRIGHTNESSDOWN = SDLK_BRIGHTNESSDOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AC_REFRESH); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AC_REFRESH, __pyx_t_1) < 0) __PYX_ERR(2, 230, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":231
 * K_AC_STOP = SDLK_AC_STOP
 * K_AC_REFRESH = SDLK_AC_REFRESH
 * K_AC_BOOKMARKS = SDLK_AC_BOOKMARKS             # <<<<<<<<<<<<<<
 * K_BRIGHTNESSDOWN = SDLK_BRIGHTNESSDOWN
 * K_BRIGHTNESSUP = SDLK_BRIGHTNESSUP
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AC_BOOKMARKS); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AC_BOOKMARKS, __pyx_t_1) < 0) __PYX_ERR(2, 231, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":232
 * K_AC_REFRESH = SDLK_AC_REFRESH
 * K_AC_BOOKMARKS = SDLK_AC_BOOKMARKS
 * K_BRIGHTNESSDOWN = SDLK_BRIGHTNESSDOWN             # <<<<<<<<<<<<<<
 * K_BRIGHTNESSUP = SDLK_BRIGHTNESSUP
 * K_DISPLAYSWITCH = SDLK_DISPLAYSWITCH
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_BRIGHTNESSDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_BRIGHTNESSDOWN, __pyx_t_1) < 0) __PYX_ERR(2, 232, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":233
 * K_AC_BOOKMARKS = SDLK_AC_BOOKMARKS
 * K_BRIGHTNESSDOWN = SDLK_BRIGHTNESSDOWN
 * K_BRIGHTNESSUP = SDLK_BRIGHTNESSUP             # <<<<<<<<<<<<<<
 * K_DISPLAYSWITCH = SDLK_DISPLAYSWITCH
 * K_KBDILLUMTOGGLE = SDLK_KBDILLUMTOGGLE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_BRIGHTNESSUP); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_BRIGHTNESSUP, __pyx_t_1) < 0) __PYX_ERR(2, 233, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":234
 * K_BRIGHTNESSDOWN = SDLK_BRIGHTNESSDOWN
 * K_BRIGHTNESSUP = SDLK_BRIGHTNESSUP
 * K_DISPLAYSWITCH = SDLK_DISPLAYSWITCH             # <<<<<<<<<<<<<<
 * K_KBDILLUMTOGGLE = SDLK_KBDILLUMTOGGLE
 * K_KBDILLUMDOWN = SDLK_KBDILLUMDOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_DISPLAYSWITCH); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_DISPLAYSWITCH, __pyx_t_1) < 0) __PYX_ERR(2, 234, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":235
 * K_BRIGHTNESSUP = SDLK_BRIGHTNESSUP
 * K_DISPLAYSWITCH = SDLK_DISPLAYSWITCH
 * K_KBDILLUMTOGGLE = SDLK_KBDILLUMTOGGLE             # <<<<<<<<<<<<<<
 * K_KBDILLUMDOWN = SDLK_KBDILLUMDOWN
 * K_KBDILLUMUP = SDLK_KBDILLUMUP
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KBDILLUMTOGGLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KBDILLUMTOGGLE, __pyx_t_1) < 0) __PYX_ERR(2, 235, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":236
 * K_DISPLAYSWITCH = SDLK_DISPLAYSWITCH
 * K_KBDILLUMTOGGLE = SDLK_KBDILLUMTOGGLE
 * K_KBDILLUMDOWN = SDLK_KBDILLUMDOWN             # <<<<<<<<<<<<<<
 * K_KBDILLUMUP = SDLK_KBDILLUMUP
 * K_EJECT = SDLK_EJECT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KBDILLUMDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KBDILLUMDOWN, __pyx_t_1) < 0) __PYX_ERR(2, 236, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":237
 * K_KBDILLUMTOGGLE = SDLK_KBDILLUMTOGGLE
 * K_KBDILLUMDOWN = SDLK_KBDILLUMDOWN
 * K_KBDILLUMUP = SDLK_KBDILLUMUP             # <<<<<<<<<<<<<<
 * K_EJECT = SDLK_EJECT
 * K_SLEEP = SDLK_SLEEP
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_KBDILLUMUP); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_KBDILLUMUP, __pyx_t_1) < 0) __PYX_ERR(2, 237, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":238
 * K_KBDILLUMDOWN = SDLK_KBDILLUMDOWN
 * K_KBDILLUMUP = SDLK_KBDILLUMUP
 * K_EJECT = SDLK_EJECT             # <<<<<<<<<<<<<<
 * K_SLEEP = SDLK_SLEEP
 * K_APP1 = SDLK_APP1
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_EJECT); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_EJECT, __pyx_t_1) < 0) __PYX_ERR(2, 238, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":239
 * K_KBDILLUMUP = SDLK_KBDILLUMUP
 * K_EJECT = SDLK_EJECT
 * K_SLEEP = SDLK_SLEEP             # <<<<<<<<<<<<<<
 * K_APP1 = SDLK_APP1
 * K_APP2 = SDLK_APP2
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_SLEEP); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_SLEEP, __pyx_t_1) < 0) __PYX_ERR(2, 239, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":240
 * K_EJECT = SDLK_EJECT
 * K_SLEEP = SDLK_SLEEP
 * K_APP1 = SDLK_APP1             # <<<<<<<<<<<<<<
 * K_APP2 = SDLK_APP2
 * K_AUDIOREWIND = SDLK_AUDIOREWIND
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_APP1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_APP1, __pyx_t_1) < 0) __PYX_ERR(2, 240, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":241
 * K_SLEEP = SDLK_SLEEP
 * K_APP1 = SDLK_APP1
 * K_APP2 = SDLK_APP2             # <<<<<<<<<<<<<<
 * K_AUDIOREWIND = SDLK_AUDIOREWIND
 * K_AUDIOFASTFORWARD = SDLK_AUDIOFASTFORWARD
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_APP2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_APP2, __pyx_t_1) < 0) __PYX_ERR(2, 241, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":242
 * K_APP1 = SDLK_APP1
 * K_APP2 = SDLK_APP2
 * K_AUDIOREWIND = SDLK_AUDIOREWIND             # <<<<<<<<<<<<<<
 * K_AUDIOFASTFORWARD = SDLK_AUDIOFASTFORWARD
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AUDIOREWIND); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AUDIOREWIND, __pyx_t_1) < 0) __PYX_ERR(2, 242, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/keycode_list.pxi":243
 * K_APP2 = SDLK_APP2
 * K_AUDIOREWIND = SDLK_AUDIOREWIND
 * K_AUDIOFASTFORWARD = SDLK_AUDIOFASTFORWARD             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDLK_AUDIOFASTFORWARD); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_K_AUDIOFASTFORWARD, __pyx_t_1) < 0) __PYX_ERR(2, 243, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":1
 * GL_RED_SIZE = SDL_GL_RED_SIZE             # <<<<<<<<<<<<<<
 * GL_GREEN_SIZE = SDL_GL_GREEN_SIZE
 * GL_BLUE_SIZE = SDL_GL_BLUE_SIZE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_RED_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RED_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":2
 * GL_RED_SIZE = SDL_GL_RED_SIZE
 * GL_GREEN_SIZE = SDL_GL_GREEN_SIZE             # <<<<<<<<<<<<<<
 * GL_BLUE_SIZE = SDL_GL_BLUE_SIZE
 * GL_ALPHA_SIZE = SDL_GL_ALPHA_SIZE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_GREEN_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_GREEN_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":3
 * GL_RED_SIZE = SDL_GL_RED_SIZE
 * GL_GREEN_SIZE = SDL_GL_GREEN_SIZE
 * GL_BLUE_SIZE = SDL_GL_BLUE_SIZE             # <<<<<<<<<<<<<<
 * GL_ALPHA_SIZE = SDL_GL_ALPHA_SIZE
 * GL_BUFFER_SIZE = SDL_GL_BUFFER_SIZE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_BLUE_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLUE_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":4
 * GL_GREEN_SIZE = SDL_GL_GREEN_SIZE
 * GL_BLUE_SIZE = SDL_GL_BLUE_SIZE
 * GL_ALPHA_SIZE = SDL_GL_ALPHA_SIZE             # <<<<<<<<<<<<<<
 * GL_BUFFER_SIZE = SDL_GL_BUFFER_SIZE
 * GL_DOUBLEBUFFER = SDL_GL_DOUBLEBUFFER
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_ALPHA_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ALPHA_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":5
 * GL_BLUE_SIZE = SDL_GL_BLUE_SIZE
 * GL_ALPHA_SIZE = SDL_GL_ALPHA_SIZE
 * GL_BUFFER_SIZE = SDL_GL_BUFFER_SIZE             # <<<<<<<<<<<<<<
 * GL_DOUBLEBUFFER = SDL_GL_DOUBLEBUFFER
 * GL_DEPTH_SIZE = SDL_GL_DEPTH_SIZE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_BUFFER_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BUFFER_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":6
 * GL_ALPHA_SIZE = SDL_GL_ALPHA_SIZE
 * GL_BUFFER_SIZE = SDL_GL_BUFFER_SIZE
 * GL_DOUBLEBUFFER = SDL_GL_DOUBLEBUFFER             # <<<<<<<<<<<<<<
 * GL_DEPTH_SIZE = SDL_GL_DEPTH_SIZE
 * GL_STENCIL_SIZE = SDL_GL_STENCIL_SIZE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_DOUBLEBUFFER); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DOUBLEBUFFER, __pyx_t_1) < 0) __PYX_ERR(3, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":7
 * GL_BUFFER_SIZE = SDL_GL_BUFFER_SIZE
 * GL_DOUBLEBUFFER = SDL_GL_DOUBLEBUFFER
 * GL_DEPTH_SIZE = SDL_GL_DEPTH_SIZE             # <<<<<<<<<<<<<<
 * GL_STENCIL_SIZE = SDL_GL_STENCIL_SIZE
 * GL_ACCUM_RED_SIZE = SDL_GL_ACCUM_RED_SIZE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_DEPTH_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":8
 * GL_DOUBLEBUFFER = SDL_GL_DOUBLEBUFFER
 * GL_DEPTH_SIZE = SDL_GL_DEPTH_SIZE
 * GL_STENCIL_SIZE = SDL_GL_STENCIL_SIZE             # <<<<<<<<<<<<<<
 * GL_ACCUM_RED_SIZE = SDL_GL_ACCUM_RED_SIZE
 * GL_ACCUM_GREEN_SIZE = SDL_GL_ACCUM_GREEN_SIZE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_STENCIL_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":9
 * GL_DEPTH_SIZE = SDL_GL_DEPTH_SIZE
 * GL_STENCIL_SIZE = SDL_GL_STENCIL_SIZE
 * GL_ACCUM_RED_SIZE = SDL_GL_ACCUM_RED_SIZE             # <<<<<<<<<<<<<<
 * GL_ACCUM_GREEN_SIZE = SDL_GL_ACCUM_GREEN_SIZE
 * GL_ACCUM_BLUE_SIZE = SDL_GL_ACCUM_BLUE_SIZE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_ACCUM_RED_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ACCUM_RED_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":10
 * GL_STENCIL_SIZE = SDL_GL_STENCIL_SIZE
 * GL_ACCUM_RED_SIZE = SDL_GL_ACCUM_RED_SIZE
 * GL_ACCUM_GREEN_SIZE = SDL_GL_ACCUM_GREEN_SIZE             # <<<<<<<<<<<<<<
 * GL_ACCUM_BLUE_SIZE = SDL_GL_ACCUM_BLUE_SIZE
 * GL_ACCUM_ALPHA_SIZE = SDL_GL_ACCUM_ALPHA_SIZE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_ACCUM_GREEN_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ACCUM_GREEN_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":11
 * GL_ACCUM_RED_SIZE = SDL_GL_ACCUM_RED_SIZE
 * GL_ACCUM_GREEN_SIZE = SDL_GL_ACCUM_GREEN_SIZE
 * GL_ACCUM_BLUE_SIZE = SDL_GL_ACCUM_BLUE_SIZE             # <<<<<<<<<<<<<<
 * GL_ACCUM_ALPHA_SIZE = SDL_GL_ACCUM_ALPHA_SIZE
 * GL_STEREO = SDL_GL_STEREO
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_ACCUM_BLUE_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ACCUM_BLUE_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":12
 * GL_ACCUM_GREEN_SIZE = SDL_GL_ACCUM_GREEN_SIZE
 * GL_ACCUM_BLUE_SIZE = SDL_GL_ACCUM_BLUE_SIZE
 * GL_ACCUM_ALPHA_SIZE = SDL_GL_ACCUM_ALPHA_SIZE             # <<<<<<<<<<<<<<
 * GL_STEREO = SDL_GL_STEREO
 * GL_MULTISAMPLEBUFFERS = SDL_GL_MULTISAMPLEBUFFERS
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_ACCUM_ALPHA_SIZE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ACCUM_ALPHA_SIZE, __pyx_t_1) < 0) __PYX_ERR(3, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":13
 * GL_ACCUM_BLUE_SIZE = SDL_GL_ACCUM_BLUE_SIZE
 * GL_ACCUM_ALPHA_SIZE = SDL_GL_ACCUM_ALPHA_SIZE
 * GL_STEREO = SDL_GL_STEREO             # <<<<<<<<<<<<<<
 * GL_MULTISAMPLEBUFFERS = SDL_GL_MULTISAMPLEBUFFERS
 * GL_MULTISAMPLESAMPLES = SDL_GL_MULTISAMPLESAMPLES
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_STEREO); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STEREO, __pyx_t_1) < 0) __PYX_ERR(3, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":14
 * GL_ACCUM_ALPHA_SIZE = SDL_GL_ACCUM_ALPHA_SIZE
 * GL_STEREO = SDL_GL_STEREO
 * GL_MULTISAMPLEBUFFERS = SDL_GL_MULTISAMPLEBUFFERS             # <<<<<<<<<<<<<<
 * GL_MULTISAMPLESAMPLES = SDL_GL_MULTISAMPLESAMPLES
 * GL_ACCELERATED_VISUAL = SDL_GL_ACCELERATED_VISUAL
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_MULTISAMPLEBUFFERS); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MULTISAMPLEBUFFERS, __pyx_t_1) < 0) __PYX_ERR(3, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":15
 * GL_STEREO = SDL_GL_STEREO
 * GL_MULTISAMPLEBUFFERS = SDL_GL_MULTISAMPLEBUFFERS
 * GL_MULTISAMPLESAMPLES = SDL_GL_MULTISAMPLESAMPLES             # <<<<<<<<<<<<<<
 * GL_ACCELERATED_VISUAL = SDL_GL_ACCELERATED_VISUAL
 * GL_RETAINED_BACKING = SDL_GL_RETAINED_BACKING
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_MULTISAMPLESAMPLES); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MULTISAMPLESAMPLES, __pyx_t_1) < 0) __PYX_ERR(3, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":16
 * GL_MULTISAMPLEBUFFERS = SDL_GL_MULTISAMPLEBUFFERS
 * GL_MULTISAMPLESAMPLES = SDL_GL_MULTISAMPLESAMPLES
 * GL_ACCELERATED_VISUAL = SDL_GL_ACCELERATED_VISUAL             # <<<<<<<<<<<<<<
 * GL_RETAINED_BACKING = SDL_GL_RETAINED_BACKING
 * GL_CONTEXT_MAJOR_VERSION = SDL_GL_CONTEXT_MAJOR_VERSION
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_ACCELERATED_VISUAL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ACCELERATED_VISUAL, __pyx_t_1) < 0) __PYX_ERR(3, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":17
 * GL_MULTISAMPLESAMPLES = SDL_GL_MULTISAMPLESAMPLES
 * GL_ACCELERATED_VISUAL = SDL_GL_ACCELERATED_VISUAL
 * GL_RETAINED_BACKING = SDL_GL_RETAINED_BACKING             # <<<<<<<<<<<<<<
 * GL_CONTEXT_MAJOR_VERSION = SDL_GL_CONTEXT_MAJOR_VERSION
 * GL_CONTEXT_MINOR_VERSION = SDL_GL_CONTEXT_MINOR_VERSION
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_RETAINED_BACKING); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RETAINED_BACKING, __pyx_t_1) < 0) __PYX_ERR(3, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":18
 * GL_ACCELERATED_VISUAL = SDL_GL_ACCELERATED_VISUAL
 * GL_RETAINED_BACKING = SDL_GL_RETAINED_BACKING
 * GL_CONTEXT_MAJOR_VERSION = SDL_GL_CONTEXT_MAJOR_VERSION             # <<<<<<<<<<<<<<
 * GL_CONTEXT_MINOR_VERSION = SDL_GL_CONTEXT_MINOR_VERSION
 * GL_CONTEXT_EGL = SDL_GL_CONTEXT_EGL
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_CONTEXT_MAJOR_VERSION); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_MAJOR_VERSION, __pyx_t_1) < 0) __PYX_ERR(3, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":19
 * GL_RETAINED_BACKING = SDL_GL_RETAINED_BACKING
 * GL_CONTEXT_MAJOR_VERSION = SDL_GL_CONTEXT_MAJOR_VERSION
 * GL_CONTEXT_MINOR_VERSION = SDL_GL_CONTEXT_MINOR_VERSION             # <<<<<<<<<<<<<<
 * GL_CONTEXT_EGL = SDL_GL_CONTEXT_EGL
 * GL_CONTEXT_FLAGS = SDL_GL_CONTEXT_FLAGS
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_CONTEXT_MINOR_VERSION); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_MINOR_VERSION, __pyx_t_1) < 0) __PYX_ERR(3, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":20
 * GL_CONTEXT_MAJOR_VERSION = SDL_GL_CONTEXT_MAJOR_VERSION
 * GL_CONTEXT_MINOR_VERSION = SDL_GL_CONTEXT_MINOR_VERSION
 * GL_CONTEXT_EGL = SDL_GL_CONTEXT_EGL             # <<<<<<<<<<<<<<
 * GL_CONTEXT_FLAGS = SDL_GL_CONTEXT_FLAGS
 * GL_CONTEXT_PROFILE_MASK = SDL_GL_CONTEXT_PROFILE_MASK
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_CONTEXT_EGL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_EGL, __pyx_t_1) < 0) __PYX_ERR(3, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":21
 * GL_CONTEXT_MINOR_VERSION = SDL_GL_CONTEXT_MINOR_VERSION
 * GL_CONTEXT_EGL = SDL_GL_CONTEXT_EGL
 * GL_CONTEXT_FLAGS = SDL_GL_CONTEXT_FLAGS             # <<<<<<<<<<<<<<
 * GL_CONTEXT_PROFILE_MASK = SDL_GL_CONTEXT_PROFILE_MASK
 * GL_SHARE_WITH_CURRENT_CONTEXT = SDL_GL_SHARE_WITH_CURRENT_CONTEXT
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_CONTEXT_FLAGS); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_FLAGS, __pyx_t_1) < 0) __PYX_ERR(3, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":22
 * GL_CONTEXT_EGL = SDL_GL_CONTEXT_EGL
 * GL_CONTEXT_FLAGS = SDL_GL_CONTEXT_FLAGS
 * GL_CONTEXT_PROFILE_MASK = SDL_GL_CONTEXT_PROFILE_MASK             # <<<<<<<<<<<<<<
 * GL_SHARE_WITH_CURRENT_CONTEXT = SDL_GL_SHARE_WITH_CURRENT_CONTEXT
 * GL_FRAMEBUFFER_SRGB_CAPABLE = SDL_GL_FRAMEBUFFER_SRGB_CAPABLE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_CONTEXT_PROFILE_MASK); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_PROFILE_MASK, __pyx_t_1) < 0) __PYX_ERR(3, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":23
 * GL_CONTEXT_FLAGS = SDL_GL_CONTEXT_FLAGS
 * GL_CONTEXT_PROFILE_MASK = SDL_GL_CONTEXT_PROFILE_MASK
 * GL_SHARE_WITH_CURRENT_CONTEXT = SDL_GL_SHARE_WITH_CURRENT_CONTEXT             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER_SRGB_CAPABLE = SDL_GL_FRAMEBUFFER_SRGB_CAPABLE
 * GL_CONTEXT_RELEASE_BEHAVIOR = SDL_GL_CONTEXT_RELEASE_BEHAVIOR
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_SHARE_WITH_CURRENT_CONTEXT); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SHARE_WITH_CURRENT_CONTEXT, __pyx_t_1) < 0) __PYX_ERR(3, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":24
 * GL_CONTEXT_PROFILE_MASK = SDL_GL_CONTEXT_PROFILE_MASK
 * GL_SHARE_WITH_CURRENT_CONTEXT = SDL_GL_SHARE_WITH_CURRENT_CONTEXT
 * GL_FRAMEBUFFER_SRGB_CAPABLE = SDL_GL_FRAMEBUFFER_SRGB_CAPABLE             # <<<<<<<<<<<<<<
 * GL_CONTEXT_RELEASE_BEHAVIOR = SDL_GL_CONTEXT_RELEASE_BEHAVIOR
 * GL_CONTEXT_RESET_NOTIFICATION = SDL_GL_CONTEXT_RESET_NOTIFICATION
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_FRAMEBUFFER_SRGB_CAPABLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER_SRGB_CAPABLE, __pyx_t_1) < 0) __PYX_ERR(3, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":25
 * GL_SHARE_WITH_CURRENT_CONTEXT = SDL_GL_SHARE_WITH_CURRENT_CONTEXT
 * GL_FRAMEBUFFER_SRGB_CAPABLE = SDL_GL_FRAMEBUFFER_SRGB_CAPABLE
 * GL_CONTEXT_RELEASE_BEHAVIOR = SDL_GL_CONTEXT_RELEASE_BEHAVIOR             # <<<<<<<<<<<<<<
 * GL_CONTEXT_RESET_NOTIFICATION = SDL_GL_CONTEXT_RESET_NOTIFICATION
 * GL_CONTEXT_NO_ERROR = SDL_GL_CONTEXT_NO_ERROR
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_CONTEXT_RELEASE_BEHAVIOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_RELEASE_BEHAVIOR, __pyx_t_1) < 0) __PYX_ERR(3, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":26
 * GL_FRAMEBUFFER_SRGB_CAPABLE = SDL_GL_FRAMEBUFFER_SRGB_CAPABLE
 * GL_CONTEXT_RELEASE_BEHAVIOR = SDL_GL_CONTEXT_RELEASE_BEHAVIOR
 * GL_CONTEXT_RESET_NOTIFICATION = SDL_GL_CONTEXT_RESET_NOTIFICATION             # <<<<<<<<<<<<<<
 * GL_CONTEXT_NO_ERROR = SDL_GL_CONTEXT_NO_ERROR
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_CONTEXT_RESET_NOTIFICATION); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_RESET_NOTIFICATION, __pyx_t_1) < 0) __PYX_ERR(3, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "include/glattr.pxi":27
 * GL_CONTEXT_RELEASE_BEHAVIOR = SDL_GL_CONTEXT_RELEASE_BEHAVIOR
 * GL_CONTEXT_RESET_NOTIFICATION = SDL_GL_CONTEXT_RESET_NOTIFICATION
 * GL_CONTEXT_NO_ERROR = SDL_GL_CONTEXT_NO_ERROR             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLattr(SDL_GL_CONTEXT_NO_ERROR); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_NO_ERROR, __pyx_t_1) < 0) __PYX_ERR(3, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":15
 * 
 * include "glattr.pxi"
 * GL_SWAP_CONTROL = -1             # <<<<<<<<<<<<<<
 * 
 * GL_CONTEXT_PROFILE_CORE = SDL_GL_CONTEXT_PROFILE_CORE
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SWAP_CONTROL, __pyx_int_neg_1) < 0) __PYX_ERR(0, 15, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":17
 * GL_SWAP_CONTROL = -1
 * 
 * GL_CONTEXT_PROFILE_CORE = SDL_GL_CONTEXT_PROFILE_CORE             # <<<<<<<<<<<<<<
 * GL_CONTEXT_PROFILE_COMPATIBILITY = SDL_GL_CONTEXT_PROFILE_COMPATIBILITY
 * GL_CONTEXT_PROFILE_ES = SDL_GL_CONTEXT_PROFILE_ES
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLprofile(SDL_GL_CONTEXT_PROFILE_CORE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_PROFILE_CORE, __pyx_t_1) < 0) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":18
 * 
 * GL_CONTEXT_PROFILE_CORE = SDL_GL_CONTEXT_PROFILE_CORE
 * GL_CONTEXT_PROFILE_COMPATIBILITY = SDL_GL_CONTEXT_PROFILE_COMPATIBILITY             # <<<<<<<<<<<<<<
 * GL_CONTEXT_PROFILE_ES = SDL_GL_CONTEXT_PROFILE_ES
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLprofile(SDL_GL_CONTEXT_PROFILE_COMPATIBILITY); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_PROFILE_COMPATIBILITY, __pyx_t_1) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":19
 * GL_CONTEXT_PROFILE_CORE = SDL_GL_CONTEXT_PROFILE_CORE
 * GL_CONTEXT_PROFILE_COMPATIBILITY = SDL_GL_CONTEXT_PROFILE_COMPATIBILITY
 * GL_CONTEXT_PROFILE_ES = SDL_GL_CONTEXT_PROFILE_ES             # <<<<<<<<<<<<<<
 * 
 * RLEACCEL = SDL_RLEACCEL
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_GLprofile(SDL_GL_CONTEXT_PROFILE_ES); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONTEXT_PROFILE_ES, __pyx_t_1) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":21
 * GL_CONTEXT_PROFILE_ES = SDL_GL_CONTEXT_PROFILE_ES
 * 
 * RLEACCEL = SDL_RLEACCEL             # <<<<<<<<<<<<<<
 * SRCCOLORKEY = 0
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDL_RLEACCEL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RLEACCEL, __pyx_t_1) < 0) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":22
 * 
 * RLEACCEL = SDL_RLEACCEL
 * SRCCOLORKEY = 0             # <<<<<<<<<<<<<<
 * 
 * # Surface flags.
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SRCCOLORKEY, __pyx_int_0) < 0) __PYX_ERR(0, 22, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":25
 * 
 * # Surface flags.
 * SRCALPHA  = 0x80000000             # <<<<<<<<<<<<<<
 * HWSURFACE = 0x40000000
 * SWSURFACE = 0
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SRCALPHA, __pyx_int_2147483648) < 0) __PYX_ERR(0, 25, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":26
 * # Surface flags.
 * SRCALPHA  = 0x80000000
 * HWSURFACE = 0x40000000             # <<<<<<<<<<<<<<
 * SWSURFACE = 0
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_HWSURFACE, __pyx_int_1073741824) < 0) __PYX_ERR(0, 26, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":27
 * SRCALPHA  = 0x80000000
 * HWSURFACE = 0x40000000
 * SWSURFACE = 0             # <<<<<<<<<<<<<<
 * 
 * # Window flags.
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SWSURFACE, __pyx_int_0) < 0) __PYX_ERR(0, 27, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":30
 * 
 * # Window flags.
 * FULLSCREEN = SDL_WINDOW_FULLSCREEN             # <<<<<<<<<<<<<<
 * OPENGL = SDL_WINDOW_OPENGL
 * NOFRAME = SDL_WINDOW_BORDERLESS
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_FULLSCREEN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FULLSCREEN, __pyx_t_1) < 0) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":31
 * # Window flags.
 * FULLSCREEN = SDL_WINDOW_FULLSCREEN
 * OPENGL = SDL_WINDOW_OPENGL             # <<<<<<<<<<<<<<
 * NOFRAME = SDL_WINDOW_BORDERLESS
 * RESIZABLE = SDL_WINDOW_RESIZABLE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_OPENGL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_OPENGL, __pyx_t_1) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":32
 * FULLSCREEN = SDL_WINDOW_FULLSCREEN
 * OPENGL = SDL_WINDOW_OPENGL
 * NOFRAME = SDL_WINDOW_BORDERLESS             # <<<<<<<<<<<<<<
 * RESIZABLE = SDL_WINDOW_RESIZABLE
 * DOUBLEBUF = 0
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_BORDERLESS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_NOFRAME, __pyx_t_1) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":33
 * OPENGL = SDL_WINDOW_OPENGL
 * NOFRAME = SDL_WINDOW_BORDERLESS
 * RESIZABLE = SDL_WINDOW_RESIZABLE             # <<<<<<<<<<<<<<
 * DOUBLEBUF = 0
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_RESIZABLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RESIZABLE, __pyx_t_1) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":34
 * NOFRAME = SDL_WINDOW_BORDERLESS
 * RESIZABLE = SDL_WINDOW_RESIZABLE
 * DOUBLEBUF = 0             # <<<<<<<<<<<<<<
 * 
 * WINDOW_FULLSCREEN = SDL_WINDOW_FULLSCREEN
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DOUBLEBUF, __pyx_int_0) < 0) __PYX_ERR(0, 34, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":36
 * DOUBLEBUF = 0
 * 
 * WINDOW_FULLSCREEN = SDL_WINDOW_FULLSCREEN             # <<<<<<<<<<<<<<
 * WINDOW_SHOWN = SDL_WINDOW_SHOWN
 * WINDOW_HIDDEN = SDL_WINDOW_HIDDEN
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_FULLSCREEN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_FULLSCREEN, __pyx_t_1) < 0) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":37
 * 
 * WINDOW_FULLSCREEN = SDL_WINDOW_FULLSCREEN
 * WINDOW_SHOWN = SDL_WINDOW_SHOWN             # <<<<<<<<<<<<<<
 * WINDOW_HIDDEN = SDL_WINDOW_HIDDEN
 * WINDOW_OPENGL = SDL_WINDOW_OPENGL
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_SHOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_SHOWN, __pyx_t_1) < 0) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":38
 * WINDOW_FULLSCREEN = SDL_WINDOW_FULLSCREEN
 * WINDOW_SHOWN = SDL_WINDOW_SHOWN
 * WINDOW_HIDDEN = SDL_WINDOW_HIDDEN             # <<<<<<<<<<<<<<
 * WINDOW_OPENGL = SDL_WINDOW_OPENGL
 * WINDOW_NOFRAME = SDL_WINDOW_BORDERLESS
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_HIDDEN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_HIDDEN, __pyx_t_1) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":39
 * WINDOW_SHOWN = SDL_WINDOW_SHOWN
 * WINDOW_HIDDEN = SDL_WINDOW_HIDDEN
 * WINDOW_OPENGL = SDL_WINDOW_OPENGL             # <<<<<<<<<<<<<<
 * WINDOW_NOFRAME = SDL_WINDOW_BORDERLESS
 * WINDOW_BORDERLESS = SDL_WINDOW_BORDERLESS
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_OPENGL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_OPENGL, __pyx_t_1) < 0) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":40
 * WINDOW_HIDDEN = SDL_WINDOW_HIDDEN
 * WINDOW_OPENGL = SDL_WINDOW_OPENGL
 * WINDOW_NOFRAME = SDL_WINDOW_BORDERLESS             # <<<<<<<<<<<<<<
 * WINDOW_BORDERLESS = SDL_WINDOW_BORDERLESS
 * WINDOW_RESIZABLE = SDL_WINDOW_RESIZABLE
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_BORDERLESS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_NOFRAME, __pyx_t_1) < 0) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":41
 * WINDOW_OPENGL = SDL_WINDOW_OPENGL
 * WINDOW_NOFRAME = SDL_WINDOW_BORDERLESS
 * WINDOW_BORDERLESS = SDL_WINDOW_BORDERLESS             # <<<<<<<<<<<<<<
 * WINDOW_RESIZABLE = SDL_WINDOW_RESIZABLE
 * WINDOW_MINIMIZED = SDL_WINDOW_MINIMIZED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_BORDERLESS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_BORDERLESS, __pyx_t_1) < 0) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":42
 * WINDOW_NOFRAME = SDL_WINDOW_BORDERLESS
 * WINDOW_BORDERLESS = SDL_WINDOW_BORDERLESS
 * WINDOW_RESIZABLE = SDL_WINDOW_RESIZABLE             # <<<<<<<<<<<<<<
 * WINDOW_MINIMIZED = SDL_WINDOW_MINIMIZED
 * WINDOW_MAXIMIZED = SDL_WINDOW_MAXIMIZED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_RESIZABLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_RESIZABLE, __pyx_t_1) < 0) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":43
 * WINDOW_BORDERLESS = SDL_WINDOW_BORDERLESS
 * WINDOW_RESIZABLE = SDL_WINDOW_RESIZABLE
 * WINDOW_MINIMIZED = SDL_WINDOW_MINIMIZED             # <<<<<<<<<<<<<<
 * WINDOW_MAXIMIZED = SDL_WINDOW_MAXIMIZED
 * WINDOW_INPUT_GRABBED = SDL_WINDOW_INPUT_GRABBED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_MINIMIZED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_MINIMIZED, __pyx_t_1) < 0) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":44
 * WINDOW_RESIZABLE = SDL_WINDOW_RESIZABLE
 * WINDOW_MINIMIZED = SDL_WINDOW_MINIMIZED
 * WINDOW_MAXIMIZED = SDL_WINDOW_MAXIMIZED             # <<<<<<<<<<<<<<
 * WINDOW_INPUT_GRABBED = SDL_WINDOW_INPUT_GRABBED
 * WINDOW_INPUT_FOCUS = SDL_WINDOW_MOUSE_FOCUS
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_MAXIMIZED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_MAXIMIZED, __pyx_t_1) < 0) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":45
 * WINDOW_MINIMIZED = SDL_WINDOW_MINIMIZED
 * WINDOW_MAXIMIZED = SDL_WINDOW_MAXIMIZED
 * WINDOW_INPUT_GRABBED = SDL_WINDOW_INPUT_GRABBED             # <<<<<<<<<<<<<<
 * WINDOW_INPUT_FOCUS = SDL_WINDOW_MOUSE_FOCUS
 * WINDOW_FULLSCREEN_DESKTOP = SDL_WINDOW_FULLSCREEN_DESKTOP
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_INPUT_GRABBED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_INPUT_GRABBED, __pyx_t_1) < 0) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":46
 * WINDOW_MAXIMIZED = SDL_WINDOW_MAXIMIZED
 * WINDOW_INPUT_GRABBED = SDL_WINDOW_INPUT_GRABBED
 * WINDOW_INPUT_FOCUS = SDL_WINDOW_MOUSE_FOCUS             # <<<<<<<<<<<<<<
 * WINDOW_FULLSCREEN_DESKTOP = SDL_WINDOW_FULLSCREEN_DESKTOP
 * WINDOW_FOREIGN = SDL_WINDOW_FOREIGN
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_MOUSE_FOCUS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_INPUT_FOCUS, __pyx_t_1) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":47
 * WINDOW_INPUT_GRABBED = SDL_WINDOW_INPUT_GRABBED
 * WINDOW_INPUT_FOCUS = SDL_WINDOW_MOUSE_FOCUS
 * WINDOW_FULLSCREEN_DESKTOP = SDL_WINDOW_FULLSCREEN_DESKTOP             # <<<<<<<<<<<<<<
 * WINDOW_FOREIGN = SDL_WINDOW_FOREIGN
 * WINDOW_ALLOW_HIGHDPI = SDL_WINDOW_ALLOW_HIGHDPI
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_FULLSCREEN_DESKTOP); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_FULLSCREEN_DESKTOP, __pyx_t_1) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":48
 * WINDOW_INPUT_FOCUS = SDL_WINDOW_MOUSE_FOCUS
 * WINDOW_FULLSCREEN_DESKTOP = SDL_WINDOW_FULLSCREEN_DESKTOP
 * WINDOW_FOREIGN = SDL_WINDOW_FOREIGN             # <<<<<<<<<<<<<<
 * WINDOW_ALLOW_HIGHDPI = SDL_WINDOW_ALLOW_HIGHDPI
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_FOREIGN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_FOREIGN, __pyx_t_1) < 0) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":49
 * WINDOW_FULLSCREEN_DESKTOP = SDL_WINDOW_FULLSCREEN_DESKTOP
 * WINDOW_FOREIGN = SDL_WINDOW_FOREIGN
 * WINDOW_ALLOW_HIGHDPI = SDL_WINDOW_ALLOW_HIGHDPI             # <<<<<<<<<<<<<<
 * 
 * WINDOWPOS_CENTERED = SDL_WINDOWPOS_CENTERED
 */
  __pyx_t_1 = __Pyx_PyInt_From_SDL_WindowFlags(SDL_WINDOW_ALLOW_HIGHDPI); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOW_ALLOW_HIGHDPI, __pyx_t_1) < 0) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":51
 * WINDOW_ALLOW_HIGHDPI = SDL_WINDOW_ALLOW_HIGHDPI
 * 
 * WINDOWPOS_CENTERED = SDL_WINDOWPOS_CENTERED             # <<<<<<<<<<<<<<
 * WINDOWPOS_UNDEFINED = SDL_WINDOWPOS_UNDEFINED
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDL_WINDOWPOS_CENTERED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOWPOS_CENTERED, __pyx_t_1) < 0) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":52
 * 
 * WINDOWPOS_CENTERED = SDL_WINDOWPOS_CENTERED
 * WINDOWPOS_UNDEFINED = SDL_WINDOWPOS_UNDEFINED             # <<<<<<<<<<<<<<
 * 
 * # Blend modes
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDL_WINDOWPOS_UNDEFINED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WINDOWPOS_UNDEFINED, __pyx_t_1) < 0) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":55
 * 
 * # Blend modes
 * BLEND_RGB_ADD = BLEND_ADD = 0x1             # <<<<<<<<<<<<<<
 * BLEND_RGB_SUB = BLEND_SUB = 0x2
 * BLEND_RGB_MULT = BLEND_MULT = 0x3
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_RGB_ADD, __pyx_int_1) < 0) __PYX_ERR(0, 55, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_ADD, __pyx_int_1) < 0) __PYX_ERR(0, 55, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":56
 * # Blend modes
 * BLEND_RGB_ADD = BLEND_ADD = 0x1
 * BLEND_RGB_SUB = BLEND_SUB = 0x2             # <<<<<<<<<<<<<<
 * BLEND_RGB_MULT = BLEND_MULT = 0x3
 * BLEND_RGB_MIN = BLEND_MIN = 0x4
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_RGB_SUB, __pyx_int_2) < 0) __PYX_ERR(0, 56, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_SUB, __pyx_int_2) < 0) __PYX_ERR(0, 56, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":57
 * BLEND_RGB_ADD = BLEND_ADD = 0x1
 * BLEND_RGB_SUB = BLEND_SUB = 0x2
 * BLEND_RGB_MULT = BLEND_MULT = 0x3             # <<<<<<<<<<<<<<
 * BLEND_RGB_MIN = BLEND_MIN = 0x4
 * BLEND_RGB_MAX = BLEND_MAX = 0x5
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_RGB_MULT, __pyx_int_3) < 0) __PYX_ERR(0, 57, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_MULT, __pyx_int_3) < 0) __PYX_ERR(0, 57, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":58
 * BLEND_RGB_SUB = BLEND_SUB = 0x2
 * BLEND_RGB_MULT = BLEND_MULT = 0x3
 * BLEND_RGB_MIN = BLEND_MIN = 0x4             # <<<<<<<<<<<<<<
 * BLEND_RGB_MAX = BLEND_MAX = 0x5
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_RGB_MIN, __pyx_int_4) < 0) __PYX_ERR(0, 58, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_MIN, __pyx_int_4) < 0) __PYX_ERR(0, 58, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":59
 * BLEND_RGB_MULT = BLEND_MULT = 0x3
 * BLEND_RGB_MIN = BLEND_MIN = 0x4
 * BLEND_RGB_MAX = BLEND_MAX = 0x5             # <<<<<<<<<<<<<<
 * 
 * BLEND_RGBA_ADD = 0x6
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_RGB_MAX, __pyx_int_5) < 0) __PYX_ERR(0, 59, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_MAX, __pyx_int_5) < 0) __PYX_ERR(0, 59, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":61
 * BLEND_RGB_MAX = BLEND_MAX = 0x5
 * 
 * BLEND_RGBA_ADD = 0x6             # <<<<<<<<<<<<<<
 * BLEND_RGBA_SUB = 0x7
 * BLEND_RGBA_MULT = 0x8
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_RGBA_ADD, __pyx_int_6) < 0) __PYX_ERR(0, 61, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":62
 * 
 * BLEND_RGBA_ADD = 0x6
 * BLEND_RGBA_SUB = 0x7             # <<<<<<<<<<<<<<
 * BLEND_RGBA_MULT = 0x8
 * BLEND_RGBA_MIN = 0x9
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_RGBA_SUB, __pyx_int_7) < 0) __PYX_ERR(0, 62, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":63
 * BLEND_RGBA_ADD = 0x6
 * BLEND_RGBA_SUB = 0x7
 * BLEND_RGBA_MULT = 0x8             # <<<<<<<<<<<<<<
 * BLEND_RGBA_MIN = 0x9
 * BLEND_RGBA_MAX = 0x10
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_RGBA_MULT, __pyx_int_8) < 0) __PYX_ERR(0, 63, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":64
 * BLEND_RGBA_SUB = 0x7
 * BLEND_RGBA_MULT = 0x8
 * BLEND_RGBA_MIN = 0x9             # <<<<<<<<<<<<<<
 * BLEND_RGBA_MAX = 0x10
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_RGBA_MIN, __pyx_int_9) < 0) __PYX_ERR(0, 64, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":65
 * BLEND_RGBA_MULT = 0x8
 * BLEND_RGBA_MIN = 0x9
 * BLEND_RGBA_MAX = 0x10             # <<<<<<<<<<<<<<
 * 
 * BLEND_PREMULTIPLIED = 0x11
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_RGBA_MAX, __pyx_int_16) < 0) __PYX_ERR(0, 65, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":67
 * BLEND_RGBA_MAX = 0x10
 * 
 * BLEND_PREMULTIPLIED = 0x11             # <<<<<<<<<<<<<<
 * 
 * # Scrap types.
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_PREMULTIPLIED, __pyx_int_17) < 0) __PYX_ERR(0, 67, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":70
 * 
 * # Scrap types.
 * SCRAP_TEXT = "text/plain"             # <<<<<<<<<<<<<<
 * 
 * BYTEORDER = SDL_BYTEORDER
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SCRAP_TEXT, __pyx_kp_s_text_plain) < 0) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":72
 * SCRAP_TEXT = "text/plain"
 * 
 * BYTEORDER = SDL_BYTEORDER             # <<<<<<<<<<<<<<
 * LIL_ENDIAN = SDL_LIL_ENDIAN
 * BIG_ENDIAN = SDL_BIG_ENDIAN
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDL_BYTEORDER); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 72, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BYTEORDER, __pyx_t_1) < 0) __PYX_ERR(0, 72, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":73
 * 
 * BYTEORDER = SDL_BYTEORDER
 * LIL_ENDIAN = SDL_LIL_ENDIAN             # <<<<<<<<<<<<<<
 * BIG_ENDIAN = SDL_BIG_ENDIAN
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDL_LIL_ENDIAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_LIL_ENDIAN, __pyx_t_1) < 0) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":74
 * BYTEORDER = SDL_BYTEORDER
 * LIL_ENDIAN = SDL_LIL_ENDIAN
 * BIG_ENDIAN = SDL_BIG_ENDIAN             # <<<<<<<<<<<<<<
 * 
 * SDL_VERSION_TUPLE = (SDL_MAJOR_VERSION, SDL_MINOR_VERSION, SDL_PATCHLEVEL)
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDL_BIG_ENDIAN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BIG_ENDIAN, __pyx_t_1) < 0) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pygame_sdl2/locals.pyx":76
 * BIG_ENDIAN = SDL_BIG_ENDIAN
 * 
 * SDL_VERSION_TUPLE = (SDL_MAJOR_VERSION, SDL_MINOR_VERSION, SDL_PATCHLEVEL)             # <<<<<<<<<<<<<<
 * 
 * globals().update(dict(
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(SDL_MAJOR_VERSION); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(SDL_MINOR_VERSION); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(SDL_PATCHLEVEL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SDL_VERSION_TUPLE, __pyx_t_4) < 0) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pygame_sdl2/locals.pyx":78
 * SDL_VERSION_TUPLE = (SDL_MAJOR_VERSION, SDL_MINOR_VERSION, SDL_PATCHLEVEL)
 * 
 * globals().update(dict(             # <<<<<<<<<<<<<<
 *     KMOD_NONE = sdl2.KMOD_NONE,
 * 
 */
  __pyx_t_4 = __Pyx_Globals(); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_update); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pygame_sdl2/locals.pyx":79
 * 
 * globals().update(dict(
 *     KMOD_NONE = sdl2.KMOD_NONE,             # <<<<<<<<<<<<<<
 * 
 *     KMOD_LSHIFT = sdl2.KMOD_LSHIFT,
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(40); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_NONE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_NONE, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":81
 *     KMOD_NONE = sdl2.KMOD_NONE,
 * 
 *     KMOD_LSHIFT = sdl2.KMOD_LSHIFT,             # <<<<<<<<<<<<<<
 *     KMOD_RSHIFT = sdl2.KMOD_RSHIFT,
 *     KMOD_SHIFT = sdl2.KMOD_SHIFT,
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_LSHIFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_LSHIFT, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":82
 * 
 *     KMOD_LSHIFT = sdl2.KMOD_LSHIFT,
 *     KMOD_RSHIFT = sdl2.KMOD_RSHIFT,             # <<<<<<<<<<<<<<
 *     KMOD_SHIFT = sdl2.KMOD_SHIFT,
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_RSHIFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_RSHIFT, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":83
 *     KMOD_LSHIFT = sdl2.KMOD_LSHIFT,
 *     KMOD_RSHIFT = sdl2.KMOD_RSHIFT,
 *     KMOD_SHIFT = sdl2.KMOD_SHIFT,             # <<<<<<<<<<<<<<
 * 
 *     KMOD_LCTRL = sdl2.KMOD_LCTRL,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(KMOD_SHIFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_SHIFT, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":85
 *     KMOD_SHIFT = sdl2.KMOD_SHIFT,
 * 
 *     KMOD_LCTRL = sdl2.KMOD_LCTRL,             # <<<<<<<<<<<<<<
 *     KMOD_RCTRL = sdl2.KMOD_RCTRL,
 *     KMOD_CTRL = sdl2.KMOD_CTRL,
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_LCTRL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_LCTRL, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":86
 * 
 *     KMOD_LCTRL = sdl2.KMOD_LCTRL,
 *     KMOD_RCTRL = sdl2.KMOD_RCTRL,             # <<<<<<<<<<<<<<
 *     KMOD_CTRL = sdl2.KMOD_CTRL,
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_RCTRL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_RCTRL, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":87
 *     KMOD_LCTRL = sdl2.KMOD_LCTRL,
 *     KMOD_RCTRL = sdl2.KMOD_RCTRL,
 *     KMOD_CTRL = sdl2.KMOD_CTRL,             # <<<<<<<<<<<<<<
 * 
 *     KMOD_LALT = sdl2.KMOD_LALT,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(KMOD_CTRL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_CTRL, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":89
 *     KMOD_CTRL = sdl2.KMOD_CTRL,
 * 
 *     KMOD_LALT = sdl2.KMOD_LALT,             # <<<<<<<<<<<<<<
 *     KMOD_RALT = sdl2.KMOD_RALT,
 *     KMOD_ALT = sdl2.KMOD_ALT,
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_LALT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_LALT, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":90
 * 
 *     KMOD_LALT = sdl2.KMOD_LALT,
 *     KMOD_RALT = sdl2.KMOD_RALT,             # <<<<<<<<<<<<<<
 *     KMOD_ALT = sdl2.KMOD_ALT,
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_RALT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_RALT, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":91
 *     KMOD_LALT = sdl2.KMOD_LALT,
 *     KMOD_RALT = sdl2.KMOD_RALT,
 *     KMOD_ALT = sdl2.KMOD_ALT,             # <<<<<<<<<<<<<<
 * 
 *     KMOD_LGUI = sdl2.KMOD_LGUI,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(KMOD_ALT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_ALT, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":93
 *     KMOD_ALT = sdl2.KMOD_ALT,
 * 
 *     KMOD_LGUI = sdl2.KMOD_LGUI,             # <<<<<<<<<<<<<<
 *     KMOD_RGUI = sdl2.KMOD_RGUI,
 *     KMOD_GUI = sdl2.KMOD_GUI,
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_LGUI); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_LGUI, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":94
 * 
 *     KMOD_LGUI = sdl2.KMOD_LGUI,
 *     KMOD_RGUI = sdl2.KMOD_RGUI,             # <<<<<<<<<<<<<<
 *     KMOD_GUI = sdl2.KMOD_GUI,
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_RGUI); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_RGUI, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":95
 *     KMOD_LGUI = sdl2.KMOD_LGUI,
 *     KMOD_RGUI = sdl2.KMOD_RGUI,
 *     KMOD_GUI = sdl2.KMOD_GUI,             # <<<<<<<<<<<<<<
 * 
 *     KMOD_LMETA = sdl2.KMOD_LGUI,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(KMOD_GUI); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_GUI, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":97
 *     KMOD_GUI = sdl2.KMOD_GUI,
 * 
 *     KMOD_LMETA = sdl2.KMOD_LGUI,             # <<<<<<<<<<<<<<
 *     KMOD_RMETA = sdl2.KMOD_RGUI,
 *     KMOD_META = sdl2.KMOD_GUI,
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_LGUI); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_LMETA, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":98
 * 
 *     KMOD_LMETA = sdl2.KMOD_LGUI,
 *     KMOD_RMETA = sdl2.KMOD_RGUI,             # <<<<<<<<<<<<<<
 *     KMOD_META = sdl2.KMOD_GUI,
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_RGUI); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_RMETA, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":99
 *     KMOD_LMETA = sdl2.KMOD_LGUI,
 *     KMOD_RMETA = sdl2.KMOD_RGUI,
 *     KMOD_META = sdl2.KMOD_GUI,             # <<<<<<<<<<<<<<
 * 
 *     KMOD_NUM = sdl2.KMOD_NUM,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(KMOD_GUI); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_META, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":101
 *     KMOD_META = sdl2.KMOD_GUI,
 * 
 *     KMOD_NUM = sdl2.KMOD_NUM,             # <<<<<<<<<<<<<<
 *     KMOD_CAPS = sdl2.KMOD_CAPS,
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_NUM); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_NUM, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":102
 * 
 *     KMOD_NUM = sdl2.KMOD_NUM,
 *     KMOD_CAPS = sdl2.KMOD_CAPS,             # <<<<<<<<<<<<<<
 * 
 *     KMOD_MODE = sdl2.KMOD_MODE,
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_CAPS); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_CAPS, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":104
 *     KMOD_CAPS = sdl2.KMOD_CAPS,
 * 
 *     KMOD_MODE = sdl2.KMOD_MODE,             # <<<<<<<<<<<<<<
 * 
 *     K_FIRST = 0,
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_Keymod(KMOD_MODE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_KMOD_MODE, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_FIRST, __pyx_int_0) < 0) __PYX_ERR(0, 79, __pyx_L1_error)

  /* "pygame_sdl2/locals.pyx":107
 * 
 *     K_FIRST = 0,
 *     K_LAST = sdl2.SDLK_SLEEP,             # <<<<<<<<<<<<<<
 * 
 *     K_BREAK = sdl2.SDLK_PAUSE,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_SLEEP); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_LAST, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":109
 *     K_LAST = sdl2.SDLK_SLEEP,
 * 
 *     K_BREAK = sdl2.SDLK_PAUSE,             # <<<<<<<<<<<<<<
 *     K_EURO = sdl2.SDLK_CURRENCYUNIT,
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_PAUSE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_BREAK, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":110
 * 
 *     K_BREAK = sdl2.SDLK_PAUSE,
 *     K_EURO = sdl2.SDLK_CURRENCYUNIT,             # <<<<<<<<<<<<<<
 * 
 *     K_KP0 = sdl2.SDLK_KP_0,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_CURRENCYUNIT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_EURO, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":112
 *     K_EURO = sdl2.SDLK_CURRENCYUNIT,
 * 
 *     K_KP0 = sdl2.SDLK_KP_0,             # <<<<<<<<<<<<<<
 *     K_KP1 = sdl2.SDLK_KP_1,
 *     K_KP2 = sdl2.SDLK_KP_2,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_KP_0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_KP0, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":113
 * 
 *     K_KP0 = sdl2.SDLK_KP_0,
 *     K_KP1 = sdl2.SDLK_KP_1,             # <<<<<<<<<<<<<<
 *     K_KP2 = sdl2.SDLK_KP_2,
 *     K_KP3 = sdl2.SDLK_KP_3,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_KP_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_KP1, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":114
 *     K_KP0 = sdl2.SDLK_KP_0,
 *     K_KP1 = sdl2.SDLK_KP_1,
 *     K_KP2 = sdl2.SDLK_KP_2,             # <<<<<<<<<<<<<<
 *     K_KP3 = sdl2.SDLK_KP_3,
 *     K_KP4 = sdl2.SDLK_KP_4,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_KP_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_KP2, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":115
 *     K_KP1 = sdl2.SDLK_KP_1,
 *     K_KP2 = sdl2.SDLK_KP_2,
 *     K_KP3 = sdl2.SDLK_KP_3,             # <<<<<<<<<<<<<<
 *     K_KP4 = sdl2.SDLK_KP_4,
 *     K_KP5 = sdl2.SDLK_KP_5,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_KP_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_KP3, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":116
 *     K_KP2 = sdl2.SDLK_KP_2,
 *     K_KP3 = sdl2.SDLK_KP_3,
 *     K_KP4 = sdl2.SDLK_KP_4,             # <<<<<<<<<<<<<<
 *     K_KP5 = sdl2.SDLK_KP_5,
 *     K_KP6 = sdl2.SDLK_KP_6,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_KP_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_KP4, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":117
 *     K_KP3 = sdl2.SDLK_KP_3,
 *     K_KP4 = sdl2.SDLK_KP_4,
 *     K_KP5 = sdl2.SDLK_KP_5,             # <<<<<<<<<<<<<<
 *     K_KP6 = sdl2.SDLK_KP_6,
 *     K_KP7 = sdl2.SDLK_KP_7,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_KP_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_KP5, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":118
 *     K_KP4 = sdl2.SDLK_KP_4,
 *     K_KP5 = sdl2.SDLK_KP_5,
 *     K_KP6 = sdl2.SDLK_KP_6,             # <<<<<<<<<<<<<<
 *     K_KP7 = sdl2.SDLK_KP_7,
 *     K_KP8 = sdl2.SDLK_KP_8,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_KP_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_KP6, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":119
 *     K_KP5 = sdl2.SDLK_KP_5,
 *     K_KP6 = sdl2.SDLK_KP_6,
 *     K_KP7 = sdl2.SDLK_KP_7,             # <<<<<<<<<<<<<<
 *     K_KP8 = sdl2.SDLK_KP_8,
 *     K_KP9 = sdl2.SDLK_KP_9,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_KP_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_KP7, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":120
 *     K_KP6 = sdl2.SDLK_KP_6,
 *     K_KP7 = sdl2.SDLK_KP_7,
 *     K_KP8 = sdl2.SDLK_KP_8,             # <<<<<<<<<<<<<<
 *     K_KP9 = sdl2.SDLK_KP_9,
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_KP_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_KP8, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":121
 *     K_KP7 = sdl2.SDLK_KP_7,
 *     K_KP8 = sdl2.SDLK_KP_8,
 *     K_KP9 = sdl2.SDLK_KP_9,             # <<<<<<<<<<<<<<
 * 
 *     K_LMETA = sdl2.SDLK_LGUI,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_KP_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_KP9, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":123
 *     K_KP9 = sdl2.SDLK_KP_9,
 * 
 *     K_LMETA = sdl2.SDLK_LGUI,             # <<<<<<<<<<<<<<
 *     K_LSUPER = sdl2.SDLK_LGUI,
 *     K_RMETA = sdl2.SDLK_RGUI,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_LGUI); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_LMETA, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":124
 * 
 *     K_LMETA = sdl2.SDLK_LGUI,
 *     K_LSUPER = sdl2.SDLK_LGUI,             # <<<<<<<<<<<<<<
 *     K_RMETA = sdl2.SDLK_RGUI,
 *     K_RSUPER = sdl2.SDLK_RGUI,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_LGUI); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_LSUPER, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":125
 *     K_LMETA = sdl2.SDLK_LGUI,
 *     K_LSUPER = sdl2.SDLK_LGUI,
 *     K_RMETA = sdl2.SDLK_RGUI,             # <<<<<<<<<<<<<<
 *     K_RSUPER = sdl2.SDLK_RGUI,
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_RGUI); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_RMETA, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":126
 *     K_LSUPER = sdl2.SDLK_LGUI,
 *     K_RMETA = sdl2.SDLK_RGUI,
 *     K_RSUPER = sdl2.SDLK_RGUI,             # <<<<<<<<<<<<<<
 * 
 *     K_NUMLOCK = sdl2.SDLK_NUMLOCKCLEAR,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_RGUI); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_RSUPER, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":128
 *     K_RSUPER = sdl2.SDLK_RGUI,
 * 
 *     K_NUMLOCK = sdl2.SDLK_NUMLOCKCLEAR,             # <<<<<<<<<<<<<<
 *     K_SCROLLOCK = sdl2.SDLK_SCROLLLOCK,
 *     K_PRINT = sdl2.SDLK_PRINTSCREEN,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_NUMLOCKCLEAR); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_NUMLOCK, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":129
 * 
 *     K_NUMLOCK = sdl2.SDLK_NUMLOCKCLEAR,
 *     K_SCROLLOCK = sdl2.SDLK_SCROLLLOCK,             # <<<<<<<<<<<<<<
 *     K_PRINT = sdl2.SDLK_PRINTSCREEN,
 * ))
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_SCROLLLOCK); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_SCROLLOCK, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":130
 *     K_NUMLOCK = sdl2.SDLK_NUMLOCKCLEAR,
 *     K_SCROLLOCK = sdl2.SDLK_SCROLLLOCK,
 *     K_PRINT = sdl2.SDLK_PRINTSCREEN,             # <<<<<<<<<<<<<<
 * ))
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(SDLK_PRINTSCREEN); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_K_PRINT, __pyx_t_2) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":78
 * SDL_VERSION_TUPLE = (SDL_MAJOR_VERSION, SDL_MINOR_VERSION, SDL_PATCHLEVEL)
 * 
 * globals().update(dict(             # <<<<<<<<<<<<<<
 *     KMOD_NONE = sdl2.KMOD_NONE,
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":133
 * ))
 * 
 * TOUCH_MOUSEID = -1             # <<<<<<<<<<<<<<
 * 
 * include "controller.pxi"
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TOUCH_MOUSEID, __pyx_int_neg_1) < 0) __PYX_ERR(0, 133, __pyx_L1_error)

  /* "include/controller.pxi":1
 * CONTROLLER_BUTTON_INVALID = SDL_CONTROLLER_BUTTON_INVALID             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_A = SDL_CONTROLLER_BUTTON_A
 * CONTROLLER_BUTTON_B = SDL_CONTROLLER_BUTTON_B
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_INVALID); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_INVALID, __pyx_t_2) < 0) __PYX_ERR(4, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":2
 * CONTROLLER_BUTTON_INVALID = SDL_CONTROLLER_BUTTON_INVALID
 * CONTROLLER_BUTTON_A = SDL_CONTROLLER_BUTTON_A             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_B = SDL_CONTROLLER_BUTTON_B
 * CONTROLLER_BUTTON_X = SDL_CONTROLLER_BUTTON_X
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_A); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_A, __pyx_t_2) < 0) __PYX_ERR(4, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":3
 * CONTROLLER_BUTTON_INVALID = SDL_CONTROLLER_BUTTON_INVALID
 * CONTROLLER_BUTTON_A = SDL_CONTROLLER_BUTTON_A
 * CONTROLLER_BUTTON_B = SDL_CONTROLLER_BUTTON_B             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_X = SDL_CONTROLLER_BUTTON_X
 * CONTROLLER_BUTTON_Y = SDL_CONTROLLER_BUTTON_Y
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_B); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_B, __pyx_t_2) < 0) __PYX_ERR(4, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":4
 * CONTROLLER_BUTTON_A = SDL_CONTROLLER_BUTTON_A
 * CONTROLLER_BUTTON_B = SDL_CONTROLLER_BUTTON_B
 * CONTROLLER_BUTTON_X = SDL_CONTROLLER_BUTTON_X             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_Y = SDL_CONTROLLER_BUTTON_Y
 * CONTROLLER_BUTTON_BACK = SDL_CONTROLLER_BUTTON_BACK
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_X); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_X, __pyx_t_2) < 0) __PYX_ERR(4, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":5
 * CONTROLLER_BUTTON_B = SDL_CONTROLLER_BUTTON_B
 * CONTROLLER_BUTTON_X = SDL_CONTROLLER_BUTTON_X
 * CONTROLLER_BUTTON_Y = SDL_CONTROLLER_BUTTON_Y             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_BACK = SDL_CONTROLLER_BUTTON_BACK
 * CONTROLLER_BUTTON_GUIDE = SDL_CONTROLLER_BUTTON_GUIDE
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_Y); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_Y, __pyx_t_2) < 0) __PYX_ERR(4, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":6
 * CONTROLLER_BUTTON_X = SDL_CONTROLLER_BUTTON_X
 * CONTROLLER_BUTTON_Y = SDL_CONTROLLER_BUTTON_Y
 * CONTROLLER_BUTTON_BACK = SDL_CONTROLLER_BUTTON_BACK             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_GUIDE = SDL_CONTROLLER_BUTTON_GUIDE
 * CONTROLLER_BUTTON_START = SDL_CONTROLLER_BUTTON_START
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_BACK); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_BACK, __pyx_t_2) < 0) __PYX_ERR(4, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":7
 * CONTROLLER_BUTTON_Y = SDL_CONTROLLER_BUTTON_Y
 * CONTROLLER_BUTTON_BACK = SDL_CONTROLLER_BUTTON_BACK
 * CONTROLLER_BUTTON_GUIDE = SDL_CONTROLLER_BUTTON_GUIDE             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_START = SDL_CONTROLLER_BUTTON_START
 * CONTROLLER_BUTTON_LEFTSTICK = SDL_CONTROLLER_BUTTON_LEFTSTICK
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_GUIDE); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_GUIDE, __pyx_t_2) < 0) __PYX_ERR(4, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":8
 * CONTROLLER_BUTTON_BACK = SDL_CONTROLLER_BUTTON_BACK
 * CONTROLLER_BUTTON_GUIDE = SDL_CONTROLLER_BUTTON_GUIDE
 * CONTROLLER_BUTTON_START = SDL_CONTROLLER_BUTTON_START             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_LEFTSTICK = SDL_CONTROLLER_BUTTON_LEFTSTICK
 * CONTROLLER_BUTTON_RIGHTSTICK = SDL_CONTROLLER_BUTTON_RIGHTSTICK
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_START); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_START, __pyx_t_2) < 0) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":9
 * CONTROLLER_BUTTON_GUIDE = SDL_CONTROLLER_BUTTON_GUIDE
 * CONTROLLER_BUTTON_START = SDL_CONTROLLER_BUTTON_START
 * CONTROLLER_BUTTON_LEFTSTICK = SDL_CONTROLLER_BUTTON_LEFTSTICK             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_RIGHTSTICK = SDL_CONTROLLER_BUTTON_RIGHTSTICK
 * CONTROLLER_BUTTON_LEFTSHOULDER = SDL_CONTROLLER_BUTTON_LEFTSHOULDER
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_LEFTSTICK); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_LEFTSTICK, __pyx_t_2) < 0) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":10
 * CONTROLLER_BUTTON_START = SDL_CONTROLLER_BUTTON_START
 * CONTROLLER_BUTTON_LEFTSTICK = SDL_CONTROLLER_BUTTON_LEFTSTICK
 * CONTROLLER_BUTTON_RIGHTSTICK = SDL_CONTROLLER_BUTTON_RIGHTSTICK             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_LEFTSHOULDER = SDL_CONTROLLER_BUTTON_LEFTSHOULDER
 * CONTROLLER_BUTTON_RIGHTSHOULDER = SDL_CONTROLLER_BUTTON_RIGHTSHOULDER
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_RIGHTSTICK); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_RIGHTSTICK, __pyx_t_2) < 0) __PYX_ERR(4, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":11
 * CONTROLLER_BUTTON_LEFTSTICK = SDL_CONTROLLER_BUTTON_LEFTSTICK
 * CONTROLLER_BUTTON_RIGHTSTICK = SDL_CONTROLLER_BUTTON_RIGHTSTICK
 * CONTROLLER_BUTTON_LEFTSHOULDER = SDL_CONTROLLER_BUTTON_LEFTSHOULDER             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_RIGHTSHOULDER = SDL_CONTROLLER_BUTTON_RIGHTSHOULDER
 * CONTROLLER_BUTTON_DPAD_UP = SDL_CONTROLLER_BUTTON_DPAD_UP
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_LEFTSHOULDER); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_LEFTSHOULDER, __pyx_t_2) < 0) __PYX_ERR(4, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":12
 * CONTROLLER_BUTTON_RIGHTSTICK = SDL_CONTROLLER_BUTTON_RIGHTSTICK
 * CONTROLLER_BUTTON_LEFTSHOULDER = SDL_CONTROLLER_BUTTON_LEFTSHOULDER
 * CONTROLLER_BUTTON_RIGHTSHOULDER = SDL_CONTROLLER_BUTTON_RIGHTSHOULDER             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_DPAD_UP = SDL_CONTROLLER_BUTTON_DPAD_UP
 * CONTROLLER_BUTTON_DPAD_DOWN = SDL_CONTROLLER_BUTTON_DPAD_DOWN
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_RIGHTSHOULDER); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_RIGHTSHOULDER, __pyx_t_2) < 0) __PYX_ERR(4, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":13
 * CONTROLLER_BUTTON_LEFTSHOULDER = SDL_CONTROLLER_BUTTON_LEFTSHOULDER
 * CONTROLLER_BUTTON_RIGHTSHOULDER = SDL_CONTROLLER_BUTTON_RIGHTSHOULDER
 * CONTROLLER_BUTTON_DPAD_UP = SDL_CONTROLLER_BUTTON_DPAD_UP             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_DPAD_DOWN = SDL_CONTROLLER_BUTTON_DPAD_DOWN
 * CONTROLLER_BUTTON_DPAD_LEFT = SDL_CONTROLLER_BUTTON_DPAD_LEFT
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_DPAD_UP); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_DPAD_UP, __pyx_t_2) < 0) __PYX_ERR(4, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":14
 * CONTROLLER_BUTTON_RIGHTSHOULDER = SDL_CONTROLLER_BUTTON_RIGHTSHOULDER
 * CONTROLLER_BUTTON_DPAD_UP = SDL_CONTROLLER_BUTTON_DPAD_UP
 * CONTROLLER_BUTTON_DPAD_DOWN = SDL_CONTROLLER_BUTTON_DPAD_DOWN             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_DPAD_LEFT = SDL_CONTROLLER_BUTTON_DPAD_LEFT
 * CONTROLLER_BUTTON_DPAD_RIGHT = SDL_CONTROLLER_BUTTON_DPAD_RIGHT
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_DPAD_DOWN); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_DPAD_DOWN, __pyx_t_2) < 0) __PYX_ERR(4, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":15
 * CONTROLLER_BUTTON_DPAD_UP = SDL_CONTROLLER_BUTTON_DPAD_UP
 * CONTROLLER_BUTTON_DPAD_DOWN = SDL_CONTROLLER_BUTTON_DPAD_DOWN
 * CONTROLLER_BUTTON_DPAD_LEFT = SDL_CONTROLLER_BUTTON_DPAD_LEFT             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_DPAD_RIGHT = SDL_CONTROLLER_BUTTON_DPAD_RIGHT
 * CONTROLLER_BUTTON_MAX = SDL_CONTROLLER_BUTTON_MAX
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_DPAD_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_DPAD_LEFT, __pyx_t_2) < 0) __PYX_ERR(4, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":16
 * CONTROLLER_BUTTON_DPAD_DOWN = SDL_CONTROLLER_BUTTON_DPAD_DOWN
 * CONTROLLER_BUTTON_DPAD_LEFT = SDL_CONTROLLER_BUTTON_DPAD_LEFT
 * CONTROLLER_BUTTON_DPAD_RIGHT = SDL_CONTROLLER_BUTTON_DPAD_RIGHT             # <<<<<<<<<<<<<<
 * CONTROLLER_BUTTON_MAX = SDL_CONTROLLER_BUTTON_MAX
 * CONTROLLER_AXIS_INVALID = SDL_CONTROLLER_AXIS_INVALID
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_DPAD_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_DPAD_RIGHT, __pyx_t_2) < 0) __PYX_ERR(4, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":17
 * CONTROLLER_BUTTON_DPAD_LEFT = SDL_CONTROLLER_BUTTON_DPAD_LEFT
 * CONTROLLER_BUTTON_DPAD_RIGHT = SDL_CONTROLLER_BUTTON_DPAD_RIGHT
 * CONTROLLER_BUTTON_MAX = SDL_CONTROLLER_BUTTON_MAX             # <<<<<<<<<<<<<<
 * CONTROLLER_AXIS_INVALID = SDL_CONTROLLER_AXIS_INVALID
 * CONTROLLER_AXIS_LEFTX = SDL_CONTROLLER_AXIS_LEFTX
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerButton(SDL_CONTROLLER_BUTTON_MAX); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_BUTTON_MAX, __pyx_t_2) < 0) __PYX_ERR(4, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":18
 * CONTROLLER_BUTTON_DPAD_RIGHT = SDL_CONTROLLER_BUTTON_DPAD_RIGHT
 * CONTROLLER_BUTTON_MAX = SDL_CONTROLLER_BUTTON_MAX
 * CONTROLLER_AXIS_INVALID = SDL_CONTROLLER_AXIS_INVALID             # <<<<<<<<<<<<<<
 * CONTROLLER_AXIS_LEFTX = SDL_CONTROLLER_AXIS_LEFTX
 * CONTROLLER_AXIS_LEFTY = SDL_CONTROLLER_AXIS_LEFTY
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerAxis(SDL_CONTROLLER_AXIS_INVALID); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_AXIS_INVALID, __pyx_t_2) < 0) __PYX_ERR(4, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":19
 * CONTROLLER_BUTTON_MAX = SDL_CONTROLLER_BUTTON_MAX
 * CONTROLLER_AXIS_INVALID = SDL_CONTROLLER_AXIS_INVALID
 * CONTROLLER_AXIS_LEFTX = SDL_CONTROLLER_AXIS_LEFTX             # <<<<<<<<<<<<<<
 * CONTROLLER_AXIS_LEFTY = SDL_CONTROLLER_AXIS_LEFTY
 * CONTROLLER_AXIS_RIGHTX = SDL_CONTROLLER_AXIS_RIGHTX
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerAxis(SDL_CONTROLLER_AXIS_LEFTX); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_AXIS_LEFTX, __pyx_t_2) < 0) __PYX_ERR(4, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":20
 * CONTROLLER_AXIS_INVALID = SDL_CONTROLLER_AXIS_INVALID
 * CONTROLLER_AXIS_LEFTX = SDL_CONTROLLER_AXIS_LEFTX
 * CONTROLLER_AXIS_LEFTY = SDL_CONTROLLER_AXIS_LEFTY             # <<<<<<<<<<<<<<
 * CONTROLLER_AXIS_RIGHTX = SDL_CONTROLLER_AXIS_RIGHTX
 * CONTROLLER_AXIS_RIGHTY = SDL_CONTROLLER_AXIS_RIGHTY
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerAxis(SDL_CONTROLLER_AXIS_LEFTY); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_AXIS_LEFTY, __pyx_t_2) < 0) __PYX_ERR(4, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":21
 * CONTROLLER_AXIS_LEFTX = SDL_CONTROLLER_AXIS_LEFTX
 * CONTROLLER_AXIS_LEFTY = SDL_CONTROLLER_AXIS_LEFTY
 * CONTROLLER_AXIS_RIGHTX = SDL_CONTROLLER_AXIS_RIGHTX             # <<<<<<<<<<<<<<
 * CONTROLLER_AXIS_RIGHTY = SDL_CONTROLLER_AXIS_RIGHTY
 * CONTROLLER_AXIS_TRIGGERLEFT = SDL_CONTROLLER_AXIS_TRIGGERLEFT
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerAxis(SDL_CONTROLLER_AXIS_RIGHTX); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_AXIS_RIGHTX, __pyx_t_2) < 0) __PYX_ERR(4, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":22
 * CONTROLLER_AXIS_LEFTY = SDL_CONTROLLER_AXIS_LEFTY
 * CONTROLLER_AXIS_RIGHTX = SDL_CONTROLLER_AXIS_RIGHTX
 * CONTROLLER_AXIS_RIGHTY = SDL_CONTROLLER_AXIS_RIGHTY             # <<<<<<<<<<<<<<
 * CONTROLLER_AXIS_TRIGGERLEFT = SDL_CONTROLLER_AXIS_TRIGGERLEFT
 * CONTROLLER_AXIS_TRIGGERRIGHT = SDL_CONTROLLER_AXIS_TRIGGERRIGHT
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerAxis(SDL_CONTROLLER_AXIS_RIGHTY); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_AXIS_RIGHTY, __pyx_t_2) < 0) __PYX_ERR(4, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":23
 * CONTROLLER_AXIS_RIGHTX = SDL_CONTROLLER_AXIS_RIGHTX
 * CONTROLLER_AXIS_RIGHTY = SDL_CONTROLLER_AXIS_RIGHTY
 * CONTROLLER_AXIS_TRIGGERLEFT = SDL_CONTROLLER_AXIS_TRIGGERLEFT             # <<<<<<<<<<<<<<
 * CONTROLLER_AXIS_TRIGGERRIGHT = SDL_CONTROLLER_AXIS_TRIGGERRIGHT
 * CONTROLLER_AXIS_MAX = SDL_CONTROLLER_AXIS_MAX
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerAxis(SDL_CONTROLLER_AXIS_TRIGGERLEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_AXIS_TRIGGERLEFT, __pyx_t_2) < 0) __PYX_ERR(4, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":24
 * CONTROLLER_AXIS_RIGHTY = SDL_CONTROLLER_AXIS_RIGHTY
 * CONTROLLER_AXIS_TRIGGERLEFT = SDL_CONTROLLER_AXIS_TRIGGERLEFT
 * CONTROLLER_AXIS_TRIGGERRIGHT = SDL_CONTROLLER_AXIS_TRIGGERRIGHT             # <<<<<<<<<<<<<<
 * CONTROLLER_AXIS_MAX = SDL_CONTROLLER_AXIS_MAX
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerAxis(SDL_CONTROLLER_AXIS_TRIGGERRIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_AXIS_TRIGGERRIGHT, __pyx_t_2) < 0) __PYX_ERR(4, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/controller.pxi":25
 * CONTROLLER_AXIS_TRIGGERLEFT = SDL_CONTROLLER_AXIS_TRIGGERLEFT
 * CONTROLLER_AXIS_TRIGGERRIGHT = SDL_CONTROLLER_AXIS_TRIGGERRIGHT
 * CONTROLLER_AXIS_MAX = SDL_CONTROLLER_AXIS_MAX             # <<<<<<<<<<<<<<
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_GameControllerAxis(SDL_CONTROLLER_AXIS_MAX); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONTROLLER_AXIS_MAX, __pyx_t_2) < 0) __PYX_ERR(4, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":137
 * include "controller.pxi"
 * 
 * POWERSTATE_UNKNOWN = SDL_POWERSTATE_UNKNOWN             # <<<<<<<<<<<<<<
 * POWERSTATE_ON_BATTERY = SDL_POWERSTATE_ON_BATTERY
 * POWERSTATE_NO_BATTERY = SDL_POWERSTATE_NO_BATTERY
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_PowerState(SDL_POWERSTATE_UNKNOWN); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_POWERSTATE_UNKNOWN, __pyx_t_2) < 0) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":138
 * 
 * POWERSTATE_UNKNOWN = SDL_POWERSTATE_UNKNOWN
 * POWERSTATE_ON_BATTERY = SDL_POWERSTATE_ON_BATTERY             # <<<<<<<<<<<<<<
 * POWERSTATE_NO_BATTERY = SDL_POWERSTATE_NO_BATTERY
 * POWERSTATE_CHARGING = SDL_POWERSTATE_CHARGING
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_PowerState(SDL_POWERSTATE_ON_BATTERY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_POWERSTATE_ON_BATTERY, __pyx_t_2) < 0) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":139
 * POWERSTATE_UNKNOWN = SDL_POWERSTATE_UNKNOWN
 * POWERSTATE_ON_BATTERY = SDL_POWERSTATE_ON_BATTERY
 * POWERSTATE_NO_BATTERY = SDL_POWERSTATE_NO_BATTERY             # <<<<<<<<<<<<<<
 * POWERSTATE_CHARGING = SDL_POWERSTATE_CHARGING
 * POWERSTATE_CHARGED = SDL_POWERSTATE_CHARGED
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_PowerState(SDL_POWERSTATE_NO_BATTERY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_POWERSTATE_NO_BATTERY, __pyx_t_2) < 0) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":140
 * POWERSTATE_ON_BATTERY = SDL_POWERSTATE_ON_BATTERY
 * POWERSTATE_NO_BATTERY = SDL_POWERSTATE_NO_BATTERY
 * POWERSTATE_CHARGING = SDL_POWERSTATE_CHARGING             # <<<<<<<<<<<<<<
 * POWERSTATE_CHARGED = SDL_POWERSTATE_CHARGED
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_PowerState(SDL_POWERSTATE_CHARGING); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_POWERSTATE_CHARGING, __pyx_t_2) < 0) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":141
 * POWERSTATE_NO_BATTERY = SDL_POWERSTATE_NO_BATTERY
 * POWERSTATE_CHARGING = SDL_POWERSTATE_CHARGING
 * POWERSTATE_CHARGED = SDL_POWERSTATE_CHARGED             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_SDL_PowerState(SDL_POWERSTATE_CHARGED); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_POWERSTATE_CHARGED, __pyx_t_2) < 0) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pygame_sdl2/locals.pyx":1
 * from sdl2 cimport *             # <<<<<<<<<<<<<<
 * cimport sdl2
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pygame_sdl2.locals", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pygame_sdl2.locals");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* Globals */
static PyObject* __Pyx_Globals(void) {
    Py_ssize_t i;
    PyObject *names;
    PyObject *globals = __pyx_d;
    Py_INCREF(globals);
    names = PyObject_Dir(__pyx_m);
    if (!names)
        goto bad;
    for (i = PyList_GET_SIZE(names)-1; i >= 0; i--) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject* name = PySequence_ITEM(names, i);
        if (!name)
            goto bad;
#else
        PyObject* name = PyList_GET_ITEM(names, i);
#endif
        if (!PyDict_Contains(globals, name)) {
            PyObject* value = __Pyx_GetAttr(__pyx_m, name);
            if (!value) {
#if CYTHON_COMPILING_IN_PYPY
                Py_DECREF(name);
#endif
                goto bad;
            }
            if (PyDict_SetItem(globals, name, value) < 0) {
#if CYTHON_COMPILING_IN_PYPY
                Py_DECREF(name);
#endif
                Py_DECREF(value);
                goto bad;
            }
        }
#if CYTHON_COMPILING_IN_PYPY
        Py_DECREF(name);
#endif
    }
    Py_DECREF(names);
    return globals;
bad:
    Py_XDECREF(names);
    Py_XDECREF(globals);
    return NULL;
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_EventType(SDL_EventType value) {
    const SDL_EventType neg_one = (SDL_EventType) ((SDL_EventType) 0 - (SDL_EventType) 1), const_zero = (SDL_EventType) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(SDL_EventType) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(SDL_EventType) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_EventType) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(SDL_EventType) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_EventType) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(SDL_EventType),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_GLattr(SDL_GLattr value) {
    const SDL_GLattr neg_one = (SDL_GLattr) ((SDL_GLattr) 0 - (SDL_GLattr) 1), const_zero = (SDL_GLattr) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(SDL_GLattr) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(SDL_GLattr) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_GLattr) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(SDL_GLattr) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_GLattr) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(SDL_GLattr),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_GLprofile(SDL_GLprofile value) {
    const SDL_GLprofile neg_one = (SDL_GLprofile) ((SDL_GLprofile) 0 - (SDL_GLprofile) 1), const_zero = (SDL_GLprofile) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(SDL_GLprofile) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(SDL_GLprofile) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_GLprofile) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(SDL_GLprofile) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_GLprofile) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(SDL_GLprofile),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_WindowFlags(SDL_WindowFlags value) {
    const SDL_WindowFlags neg_one = (SDL_WindowFlags) ((SDL_WindowFlags) 0 - (SDL_WindowFlags) 1), const_zero = (SDL_WindowFlags) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(SDL_WindowFlags) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(SDL_WindowFlags) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_WindowFlags) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(SDL_WindowFlags) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_WindowFlags) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(SDL_WindowFlags),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_Keymod(SDL_Keymod value) {
    const SDL_Keymod neg_one = (SDL_Keymod) ((SDL_Keymod) 0 - (SDL_Keymod) 1), const_zero = (SDL_Keymod) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(SDL_Keymod) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(SDL_Keymod) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_Keymod) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(SDL_Keymod) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_Keymod) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(SDL_Keymod),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_GameControllerButton(SDL_GameControllerButton value) {
    const SDL_GameControllerButton neg_one = (SDL_GameControllerButton) ((SDL_GameControllerButton) 0 - (SDL_GameControllerButton) 1), const_zero = (SDL_GameControllerButton) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(SDL_GameControllerButton) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(SDL_GameControllerButton) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_GameControllerButton) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(SDL_GameControllerButton) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_GameControllerButton) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(SDL_GameControllerButton),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_GameControllerAxis(SDL_GameControllerAxis value) {
    const SDL_GameControllerAxis neg_one = (SDL_GameControllerAxis) ((SDL_GameControllerAxis) 0 - (SDL_GameControllerAxis) 1), const_zero = (SDL_GameControllerAxis) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(SDL_GameControllerAxis) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(SDL_GameControllerAxis) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_GameControllerAxis) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(SDL_GameControllerAxis) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_GameControllerAxis) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(SDL_GameControllerAxis),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_SDL_PowerState(SDL_PowerState value) {
    const SDL_PowerState neg_one = (SDL_PowerState) ((SDL_PowerState) 0 - (SDL_PowerState) 1), const_zero = (SDL_PowerState) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(SDL_PowerState) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(SDL_PowerState) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_PowerState) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(SDL_PowerState) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(SDL_PowerState) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(SDL_PowerState),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
